/**
 * SECTION:applet-calendar-item
 * @short_description: 日历项显示
 *
 * 当选项为月份时，点击显示该月份的天数；
 * 当选项为年时，显示该年的12个月份；
 * 当选项为年的区间时，显示该区间的年份。
 *
 */

#include "applet-calendar-item.h"

struct _AppletCalendarItemPrivate {
    CalendarItemOption option;
    int data;
};

G_DEFINE_TYPE_WITH_PRIVATE(AppletCalendarItem, applet_calendar_item, ST_TYPE_BUTTON)

static void applet_calendar_item_finalize(GObject * calendaritem) {
    g_return_if_fail(APPLET_IS_CALENDAR_ITEM(calendaritem));
    AppletCalendarItemPrivate* priv = APPLET_CALENDAR_ITEM(calendaritem)->priv;
    priv->option = OPT_INVALID;
    priv->data = 0;
    G_OBJECT_CLASS (applet_calendar_item_parent_class)->finalize(calendaritem);
}

static void applet_calendar_item_class_init(AppletCalendarItemClass *klass) {
    GObjectClass* objcls = G_OBJECT_CLASS(klass);

    objcls->finalize = applet_calendar_item_finalize;
}

static void applet_calendar_item_init(AppletCalendarItem *self) {
    AppletCalendarItemPrivate * priv = self->priv = applet_calendar_item_get_instance_private(self);
    priv->option = OPT_INVALID;
    priv->data = -1;
}

StWidget* applet_calendar_item_new_with_label_month(const gchar *text, int month) {
    StWidget* item = g_object_new(APPLET_TYPE_CALENDAR_ITEM, "label", text, NULL);
    AppletCalendarItemPrivate* priv = APPLET_CALENDAR_ITEM(item)->priv;
    priv->option = OPT_MONTH;
    priv->data = month;
    return item;
}

StWidget* applet_calendar_item_new_with_label_year(const gchar *text, int year) {
    StWidget* item = g_object_new(APPLET_TYPE_CALENDAR_ITEM, "label", text, NULL);
    AppletCalendarItemPrivate* priv = APPLET_CALENDAR_ITEM(item)->priv;
    priv->option = OPT_YEAR;
    priv->data = year;
    return item;
}

StWidget* applet_calendar_item_new_with_label_day_of_month(const gchar *text, int day_of_month) {
    StWidget* item = g_object_new(APPLET_TYPE_CALENDAR_ITEM, "label", text, NULL);
    AppletCalendarItemPrivate* priv = APPLET_CALENDAR_ITEM(item)->priv;
    priv->option = OPT_DAY;
    priv->data = day_of_month;
    return item;
}

/**
 * applet_calendar_item_get_option:
 * @item: AppletCalendarItem对象
 *
 * Return value:返回当前item的用途，值为枚举值（无效/年/月/日）的一项。
 */
CalendarItemOption applet_calendar_item_get_option(AppletCalendarItem* item) {
    AppletCalendarItemPrivate* priv = item->priv;
    return priv->option;
}

/**
 * applet_calendar_item_get_item_data:
 * @item: AppletCalendarItem对象
 *
 * Return value:当前的年份、月份或者是日期。
 */
int applet_calendar_item_get_item_data(AppletCalendarItem* item) {
    AppletCalendarItemPrivate* priv = item->priv;
    return priv->data;
}
