/**
 * SECTION:applet-calendar-main
 * @short_description: 日历applet总入口
 *
 * 提供给applet-manager调用。
 *
 * 实现日期与时间的显示，系统“日期和时间”的设置入口。
 *
 */
#include "applet-calendar-main.h"
#include "cdos-popup-menu-manager.h"
#include "cdos-popup-menu.h"

#include "cdos.h"
#include "applet-calendar.h"
// modify by linjj for remove clock begin
// #include "applet-calendar-clock.h"
// modify by linjj for remove clock end

#include "applet-calendar-private.h"

#define UPDATE_INTERVAL_FOR_SEC   (500)
#define UPDATE_INTERVAL_FOR_NOSEC (10000)

struct _AppletCalendarMainPrivate {
    StWidget* actor;
    CdosPopupMenu* popupMenu;
    GSettings* settings;
    gboolean  need_show_seconds;
    gboolean  is_use_24h;
    gboolean  need_show_date;
    gchar *format_class_name;

    StWidget* title;
    StWidget* datetimeSet;
    StWidget* actortimelabel;
    StWidget* actordatelabel;

    guint  datetimeupdateid;

    AppletCalendar* calendar;
    gint year;
    gint month;
    gint date;
    StWidget* timeLabel;
    StWidget* currentDateBtn;
    CdosPopupMenuItem *timeSettingBtn;
    CdosPopupSeparatorMenuItem *separatorItem;
};

static const gchar * calendar_back_ground[] = {
    "applet-calendar-bg-74",
    "applet-calendar-bg-94",
    "applet-calendar-bg-94",
    "applet-calendar-bg-94",
    "applet-calendar-bg-48",
    "applet-calendar-bg-68",
    "applet-calendar-bg-94",
    "applet-calendar-bg-94"
};


G_DEFINE_TYPE_WITH_PRIVATE(AppletCalendarMain, applet_calendar_main, CDOS_TYPE_APPLET)

static void load_settings(AppletCalendarMain *self);
static void apply_settings(AppletCalendarMain *self);
static void update_datetime(AppletCalendarMain *self);
static void update_hover_background(AppletCalendarMain *self,gchar *key);

static void gsetting_value_changed (GSettings *settings, gchar *key, gpointer user_data);

static gboolean callUpdateTime(gpointer user_data) {
    g_return_if_fail(APPLET_IS_CALENDAR_MAIN(user_data));
    AppletCalendarMain* self = APPLET_CALENDAR_MAIN(user_data);
    update_datetime(self);
    return TRUE;
}

static void onDateSettingClicked(ClutterActor *actor, ClutterEvent *event, gboolean keep, gpointer data) {
    cdos_util_spawn_command_line ("cdos-settings calendar");
}

static void onCurrentDateClicked(StWidget* button, int clickedButton, gpointer user_data) {
     g_return_if_fail(NULL != user_data && APPLET_IS_CALENDAR_MAIN(user_data));
     AppletCalendarMainPrivate* priv = APPLET_CALENDAR_MAIN(user_data)->priv;
     applet_calendar_update(priv->calendar);
}

static void applet_calendar_main_finalize(GObject * calendarapplet) {
    g_return_if_fail(APPLET_IS_CALENDAR_MAIN(calendarapplet));
    AppletCalendarMain* applet = APPLET_CALENDAR_MAIN(calendarapplet);
    AppletCalendarMainPrivate* priv = applet->priv;

    clutter_actor_destroy_all_children(CLUTTER_ACTOR(priv->popupMenu));

    g_object_unref(priv->title);
    g_object_unref(priv->datetimeSet);
    g_object_unref(priv->actortimelabel);
    g_object_unref(priv->actordatelabel);
    g_object_unref(priv->actor);
    g_object_unref(priv->separatorItem);
    g_object_unref(priv->timeSettingBtn);
    G_OBJECT_CLASS (applet_calendar_main_parent_class)->finalize(calendarapplet);
}

static void applet_calendar_main_class_init(AppletCalendarMainClass *klass) {
    GObjectClass* objcls = G_OBJECT_CLASS(klass);

    objcls->finalize = applet_calendar_main_finalize;
}

static void applet_calendar_main_init(AppletCalendarMain *self) {
    AppletCalendarMainPrivate* priv;
    cdos_applet_set_popup_menu_active(CDOS_APPLET(self),TRUE);
    priv = self->priv = applet_calendar_main_get_instance_private(self);

    priv->settings = g_settings_new(GSETTING_SCHEMA_NAME);
    load_settings(self);
    priv ->date = 0;
    priv ->month = 0;
    priv ->year = 0;
    g_signal_connect(G_OBJECT(priv->settings), "changed::"SHOW_DATE_KEY, G_CALLBACK(gsetting_value_changed), self);
    g_signal_connect(G_OBJECT(priv->settings), "changed::"SHOW_SEC_KEY, G_CALLBACK(gsetting_value_changed), self);
    g_signal_connect(G_OBJECT(priv->settings), "changed::"USE_24H_KEY, G_CALLBACK(gsetting_value_changed), self);

    priv->actor = CDOS_APPLET(self)->actor;
    st_widget_add_style_class_name(ST_WIDGET(priv->actor), "applet-calendar-bg");

    priv->actortimelabel = st_label_new("");
    st_widget_set_style_class_name(priv->actortimelabel, "applet-label");
    clutter_actor_set_x_align(CLUTTER_ACTOR(priv->actortimelabel), CLUTTER_ACTOR_ALIGN_CENTER);
    clutter_actor_set_y_align(CLUTTER_ACTOR(priv->actortimelabel), CLUTTER_ACTOR_ALIGN_CENTER);

    priv->actordatelabel = st_label_new("");
    st_widget_set_style_class_name(priv->actordatelabel, "applet-label");
    clutter_actor_set_x_align(CLUTTER_ACTOR(priv->actordatelabel), CLUTTER_ACTOR_ALIGN_CENTER);
    clutter_actor_set_y_align(CLUTTER_ACTOR(priv->actordatelabel), CLUTTER_ACTOR_ALIGN_CENTER);

    ClutterActor *clutter_text = st_label_get_clutter_text (priv->actortimelabel);
    clutter_text_set_ellipsize (clutter_text, PANGO_ELLIPSIZE_NONE);

    clutter_text = st_label_get_clutter_text (priv->actordatelabel);
    clutter_text_set_ellipsize (clutter_text, PANGO_ELLIPSIZE_NONE);

    priv->popupMenu = cdos_applet_get_popup_menu(CDOS_APPLET(self));

    StBoxLayout* calendarbox = cdos_popup_menu_base_get_box(CDOS_POPUP_MENU_BASE(priv->popupMenu));

    StWidget* mainLayout = st_box_layout_new();
    st_box_layout_set_vertical(ST_BOX_LAYOUT(mainLayout), TRUE);

    priv->timeLabel = st_label_new("");
    st_widget_set_style_class_name(ST_WIDGET(priv->timeLabel), "datemenu-date-label");

    priv->currentDateBtn = st_button_new_with_label("");
    st_widget_set_style_class_name(ST_WIDGET(priv->currentDateBtn), "calendar-time-current-date");
    g_signal_connect(G_OBJECT(priv->currentDateBtn), "clicked", G_CALLBACK(onCurrentDateClicked), self);

    StWidget* calendarLayout = st_box_layout_new();
    st_box_layout_set_vertical(ST_BOX_LAYOUT(calendarLayout), FALSE);
    priv->calendar = applet_calendar_new();
    st_box_layout_insert_actor(ST_BOX_LAYOUT(calendarLayout), CLUTTER_ACTOR(priv->calendar->actor), -1);

    //CdosPopupSeparatorMenuItem *separatorItem = cdos_popup_separator_menu_item_new ();

    // modify by linjj for remove clock begin
    // AppletCalendarClock* clock = applet_calendar_clock_new();
    // st_box_layout_insert_actor(ST_BOX_LAYOUT(calendarLayout), CLUTTER_ACTOR(clock->actor), -1);
    // modify by linjj for remove clock end

    //StWidget* timeSettingBtn = st_button_new_with_label(_("Date and Time Settings"));
    //st_widget_set_style_class_name(ST_WIDGET(timeSettingBtn), "calendar-time-setting-button");
    //clutter_actor_set_x_align(CLUTTER_ACTOR(timeSettingBtn), CLUTTER_ACTOR_ALIGN_CENTER);
    //st_widget_set_track_hover (timeSettingBtn, TRUE);
    //g_signal_connect(G_OBJECT(timeSettingBtn), "clicked", G_CALLBACK(onDateSettingClicked), self);

    st_box_layout_insert_actor(ST_BOX_LAYOUT(mainLayout), CLUTTER_ACTOR(priv->timeLabel), -1);
    st_box_layout_insert_actor(ST_BOX_LAYOUT(mainLayout), CLUTTER_ACTOR(priv->currentDateBtn), -1);
    st_box_layout_insert_actor(ST_BOX_LAYOUT(mainLayout), CLUTTER_ACTOR(calendarLayout), -1);
    //st_box_layout_insert_actor(ST_BOX_LAYOUT(mainLayout), CLUTTER_ACTOR(cdos_popup_base_menu_item_get_actor(separatorItem)), -1);
    //st_box_layout_insert_actor(ST_BOX_LAYOUT(mainLayout), CLUTTER_ACTOR(timeSettingBtn), -1);

    clutter_container_child_set(CLUTTER_CONTAINER(mainLayout), CLUTTER_ACTOR(priv->timeLabel), "expand", TRUE, "x_fill", FALSE, "y_fill", FALSE, "x_align", ST_ALIGN_START, "y_align", ST_ALIGN_MIDDLE, NULL);
    clutter_container_child_set(CLUTTER_CONTAINER(mainLayout), CLUTTER_ACTOR(priv->currentDateBtn), "expand", TRUE, "x_fill", FALSE, "y_fill", FALSE, "x_align", ST_ALIGN_START, "y_align", ST_ALIGN_MIDDLE, NULL);
    clutter_actor_set_margin_left(CLUTTER_ACTOR(priv->currentDateBtn),20);

    clutter_container_add_actor(CLUTTER_CONTAINER(calendarbox), CLUTTER_ACTOR(mainLayout));

    priv->separatorItem = cdos_popup_separator_menu_item_new ();
    cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE (priv->popupMenu), CDOS_POPUP_BASE_MENU_ITEM(priv->separatorItem), -1);

    priv->timeSettingBtn = cdos_popup_menu_item_new ();
    cdos_popup_menu_item_set_label_align(priv->timeSettingBtn,ST_ALIGN_MIDDLE,0);
    cdos_popup_menu_item_set_label(priv->timeSettingBtn,_("Date and Time Settings"));
    StLabel *label = cdos_popup_menu_item_get_label(priv->timeSettingBtn);
    st_widget_add_style_class_name(ST_WIDGET(label),"popup-menu-item-label-link");

    g_signal_connect(priv->timeSettingBtn, "activate", G_CALLBACK(onDateSettingClicked), self);
    cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE (priv->popupMenu), CDOS_POPUP_BASE_MENU_ITEM(priv->timeSettingBtn), -1);


//    static const ClutterColor blue_color = { 0x00, 0x00, 0xff, 0xff };
//    clutter_actor_set_background_color(CLUTTER_ACTOR(calendarbox), &blue_color);

    StWidget* datetimegroup = st_box_layout_new();
    st_box_layout_set_vertical(ST_BOX_LAYOUT(datetimegroup), TRUE);
    clutter_actor_set_x_align(CLUTTER_ACTOR(datetimegroup), CLUTTER_ACTOR_ALIGN_CENTER);
    clutter_actor_set_y_align(CLUTTER_ACTOR(datetimegroup), CLUTTER_ACTOR_ALIGN_CENTER);

    st_box_layout_insert_actor(ST_BOX_LAYOUT(datetimegroup), CLUTTER_ACTOR(priv->actortimelabel), -1);
    st_box_layout_insert_actor(ST_BOX_LAYOUT(datetimegroup), CLUTTER_ACTOR(priv->actordatelabel), -1);

    clutter_container_child_set(CLUTTER_CONTAINER(datetimegroup), CLUTTER_ACTOR(priv->actortimelabel), "expand", TRUE, "x_fill", FALSE, "y_fill", FALSE, "x_align", ST_ALIGN_MIDDLE, "y_align", ST_ALIGN_MIDDLE, NULL);
    clutter_container_child_set(CLUTTER_CONTAINER(datetimegroup), CLUTTER_ACTOR(priv->actordatelabel), "expand", TRUE, "x_fill", FALSE, "y_fill", FALSE, "x_align", ST_ALIGN_MIDDLE, "y_align", ST_ALIGN_MIDDLE, NULL);

    st_box_layout_insert_actor(ST_BOX_LAYOUT(priv->actor), CLUTTER_ACTOR(datetimegroup), -1);
    clutter_actor_set_reactive(CLUTTER_ACTOR(priv->actor), TRUE);

    update_hover_background(self,NULL);

    apply_settings(self);
}

/**
 * applet_calendar_main_new:
 * @orientation:（未使用）
 * @panel_height:任务栏的高度
 *
 * 构建日历applet对象，用于在任务栏中显示日期和时间。
 *
 * Returns: 日历applet对象
 */
CdosApplet* applet_calendar_main_new(StSide orientation,  gfloat panel_height) {
    return g_object_new(APPLET_TYPE_CALENDAR_MAIN, "orientation", orientation, "panel-height", panel_height, NULL);
}

/**
  * applets_calendar_main:
  * @{meta}:(未使用)
  * @orientation:(未使用)
  * @panel_height:任务栏高度
  *
  * 外部模块调用applet-calendar模块入口。
  *
  * Return value:日历applet对象
  */
CdosApplet * applets_calendar_main(MetaData *meta, StSide orientation,  gfloat panel_height) {
    return applet_calendar_main_new(orientation, panel_height);
}

static void load_settings(AppletCalendarMain *self) {
    g_return_if_fail(NULL != self && APPLET_IS_CALENDAR_MAIN(self));
    AppletCalendarMainPrivate* priv = self->priv;
    priv->need_show_date = g_settings_get_boolean(priv->settings, SHOW_DATE_KEY);
    priv->need_show_seconds = g_settings_get_boolean(priv->settings, SHOW_SEC_KEY);
    priv->is_use_24h = g_settings_get_boolean(priv->settings, USE_24H_KEY);
}

static void apply_settings(AppletCalendarMain *self) {
    g_return_if_fail(NULL != self && APPLET_IS_CALENDAR_MAIN(self));
    AppletCalendarMainPrivate* priv = self->priv;
    
    update_datetime(self);

    if (0 != priv->datetimeupdateid) {
        g_source_remove(priv->datetimeupdateid);
        priv->datetimeupdateid = 0;
    }
    priv->datetimeupdateid = g_timeout_add((priv->need_show_seconds ? UPDATE_INTERVAL_FOR_SEC : UPDATE_INTERVAL_FOR_NOSEC), callUpdateTime, self);
}

static void update_datetime(AppletCalendarMain *self) {
    g_return_if_fail(NULL != self && APPLET_IS_CALENDAR_MAIN(self));
    AppletCalendarMainPrivate* priv = self->priv;

    GDateTime* now = g_date_time_new_now_local();
    gint date = g_date_time_get_day_of_month (now);
    gint month = g_date_time_get_month(now);
    gint year = g_date_time_get_year(now);
    if (date != priv->date || month != priv->month || year != priv->year)
    {
	    applet_calendar_update(priv->calendar);
	    priv->date = date;
	    priv->month = month;
	    priv->year = year;
    }
    gchar* datetimeformat = g_strdup_printf("%s:%%M%s", 
                                            priv->is_use_24h ? "%H" : "%p %l",
                                            priv->need_show_seconds ? ":%S" : "");
    gchar* timestrtmp = g_date_time_format(now, datetimeformat);
    gchar* datestrtmp = g_date_time_format(now, "%Y/%m/%d");
    gchar* currentstrtmp = g_date_time_format(now, "%x  %A");

    st_label_set_text(ST_LABEL(priv->actortimelabel), timestrtmp);
    st_label_set_text(ST_LABEL(priv->timeLabel), timestrtmp);
    st_label_set_text(ST_LABEL(priv->actordatelabel), datestrtmp);
    st_button_set_label(ST_BUTTON(priv->currentDateBtn), currentstrtmp);

    if (TRUE == priv->need_show_date) {
        clutter_actor_show(CLUTTER_ACTOR(priv->actordatelabel));
    } else {
        clutter_actor_hide(CLUTTER_ACTOR(priv->actordatelabel));
    }
    cdos_applet_set_applet_tooltip(CDOS_APPLET(self),currentstrtmp);
    g_free(currentstrtmp);
    g_free(datestrtmp);
    g_free(timestrtmp);
    g_free(datetimeformat);
    g_date_time_unref(now);
}

static void gsetting_value_changed (GSettings *settings, gchar *key, gpointer user_data) {
    g_return_if_fail(NULL != user_data && APPLET_IS_CALENDAR_MAIN(user_data));

    load_settings(APPLET_CALENDAR_MAIN(user_data));
    update_hover_background(APPLET_CALENDAR_MAIN(user_data),key);
    apply_settings(APPLET_CALENDAR_MAIN(user_data));
}

static void update_hover_background(AppletCalendarMain *self,gchar *key){
    AppletCalendarMainPrivate* priv = self->priv;
    int index = (priv->is_use_24h << 2) | (priv->need_show_date << 1) | (priv->need_show_seconds << 0);
    if(priv->format_class_name != NULL){
        st_widget_remove_style_class_name(ST_WIDGET(priv->actor),priv->format_class_name);
    }
    st_widget_add_style_class_name(ST_WIDGET(priv->actor),calendar_back_ground[index]);
    priv->format_class_name = calendar_back_ground[index];
}
