#include <girepository.h>
#include <gio/gio.h>
#include "applets-group.h"

G_DEFINE_TYPE(AppletGroup, applet_group, CDOS_TYPE_ICONAPPLET);

#define APPLET_GROUP_GET_PRIVATE(obj) \
    (G_TYPE_INSTANCE_GET_PRIVATE((obj),APPLET_TYPE_GROUP, AppletGroupPrivate))

//set the field of single window mode for settings;
static const gchar *single_wm_class_settings = "Cellrendercombo.py";
#define APPLET_GROUP_COLUMN 3

struct _AppletGroupPrivate
{
    CdosPopupMenu         *menu;
    guint                 orientation;
    CdosPopupMenuItem     *setting_item;
    CdosPopupMenuItem     *separator;
    GList                 *applets;
    StTable               *table;
    StBoxLayout           *box;
    guint                 count;
    GSettings             *settings;
    gint                  row_size;
};


static void applet_group_add_separator(AppletGroup *self);
static AppletGroup* applet_group_new(MetaData *metadata , guint orientation , gfloat panel_height);
static gboolean applet_group_has_applet(AppletGroup *self,CdosApplet *applet);

static void applet_group_table_add_child(AppletGroup *self, ClutterActor *actor);

static void
applet_group_finalize(GObject *object)
{
    AppletGroupPrivate *priv = APPLET_GROUP(object) ->priv;
    G_OBJECT_CLASS(applet_group_parent_class)->finalize(object);
}
static void
applet_group_dispose(GObject *object)
{
    AppletGroupPrivate *priv = APPLET_GROUP(object) ->priv;
    g_clear_object(&priv->separator);
    g_clear_object(&priv->setting_item);
    g_clear_object(&priv->table);
    g_clear_object(&priv->box);

    G_OBJECT_CLASS(applet_group_parent_class)->dispose(object);
}

static gboolean
applet_group_setting_item_on_activate(ClutterActor *actor, gpointer event , gboolean keep,gpointer data){
    AppletGroup *self = APPLET_GROUP(data);
    guint pid = cdos_applet_get_single_window_pid(CDOS_APPLET(self),single_wm_class_settings);
    if(pid > 0){
        cdos_applet_reset_single_window_status(CDOS_APPLET(self),single_wm_class_settings);
    }else{
        cdos_util_spawn_command_line("cellrendercombo");
    }
    return TRUE;
}

static void
applet_group_add_separator(AppletGroup *self)
{
    AppletGroupPrivate *priv = self->priv;

    if(priv->count > 0){
        cdos_popup_separator_menu_item_show(priv->separator);
    }else{
        cdos_popup_separator_menu_item_hide(priv->separator);
    }
}

static gboolean
applet_group_has_applet(AppletGroup *self,CdosApplet *applet){
    AppletGroupPrivate *priv = self->priv;
    if(priv->applets){
        GList *applets = priv->applets;
        while (applets != NULL){
            CdosApplet *applet_tmp = applets->data;
            if(applet_tmp && applet_tmp == applet){
                return TRUE;
            }
            applets = applets->next;
        }
    }
    return FALSE;
}

static void
applet_group_table_add_child(AppletGroup *self, ClutterActor *actor){
    AppletGroupPrivate *priv = self->priv;
    ClutterContainer *container = CLUTTER_CONTAINER(priv->table);
    clutter_container_add(container,actor,NULL);
    StTableChild *meta = (StTableChild *) clutter_container_get_child_meta (container, actor);
    gint row_size,row,col,x;
    row_size = (int)ceil(sqrt (priv->count));

    if( row_size <= 3){
        x = priv->count - priv->row_size * priv->row_size;
        if(x < row_size){
            row = x - 1;
            col = row_size - 1;
        } else {
            col = priv->count % row_size - 1;
            row = row_size - 1;
        }
        if(col < 0){
            col = col + row_size;
            priv->row_size = row_size;
        }
    } else {
        row_size = APPLET_GROUP_COLUMN;
        col = (priv->count - 1) % row_size;
        row = (int)floor(priv->count / row_size);
    }
    g_object_set(meta,"row",row,"col",col,NULL);
}

void
applet_group_add_applet(AppletGroup *self,CdosApplet *applet)
{
    g_return_if_fail(APPLET_IS_GROUP(self));
    g_return_if_fail(CDOS_IS_APPLET(applet));

    AppletGroupPrivate *priv = self->priv;
    gboolean visible = TRUE;
    ClutterActor *actor = CLUTTER_ACTOR(cdos_applet_get_actor (applet));
    if(applet_group_has_applet(self,applet) == TRUE){
        return;
    }
    priv->applets = g_list_append(priv->applets,CDOS_APPLET(applet));
    priv->count ++;

    StBin *applet_icon_box = cdos_iconapplet_get_applet_icon_box(CDOS_ICONAPPLET(applet));
    st_widget_add_style_class_name(ST_WIDGET(applet_icon_box),"applet-group-item-child");

    applet_group_table_add_child(self,actor);

    cdos_popup_separator_menu_item_show(priv->separator);

}

void
applet_group_remove_all_applets(AppletGroup *self)
{
    g_return_if_fail(APPLET_IS_GROUP(self));
    AppletGroupPrivate *priv = self->priv;
    if(priv->applets){
        GList *applets = priv->applets;
        while (applets){
            CdosApplet *applet = applets->data;
            if(applet){
                ClutterActor *actor = CLUTTER_ACTOR(cdos_applet_get_actor (applet));
                StBin *applet_icon_box = cdos_iconapplet_get_applet_icon_box(CDOS_ICONAPPLET(applet));
                st_widget_remove_style_class_name(ST_WIDGET(applet_icon_box),"applet-group-item-child");
                clutter_container_remove(CLUTTER_CONTAINER(priv->table), actor,NULL);
            }
            applets = applets->next;
        }
        g_list_free(priv->applets);
        priv->applets = NULL;
    }
    priv->count = 0;
    priv->row_size = 0;
    cdos_popup_separator_menu_item_hide(priv->separator);
}

AppletGroup *
applet_group_main(MetaData *metadata , guint orientation , gfloat panel_height)
{
    static AppletGroup *group;
    if(group == NULL){
        group = applet_group_new(metadata,orientation,panel_height);
    }
    return group;
}

static void
applet_group_class_init(AppletGroupClass *kclass)
{
    GObjectClass *object_class = G_OBJECT_CLASS(kclass);
    object_class->finalize = applet_group_finalize;
    object_class->dispose = applet_group_dispose;

    g_type_class_add_private(kclass, sizeof(AppletGroupPrivate));
}

static void
applet_group_init(AppletGroup *self)
{
    AppletGroupPrivate *priv;
    priv = self->priv = APPLET_GROUP_GET_PRIVATE(self);
    cdos_applet_set_single_window_mode(CDOS_APPLET(self),single_wm_class_settings,TRUE);

    CdosIconApplet *iconapplet = (CdosIconApplet *)self;
    cdos_iconapplet_set_icon_name((CdosIconApplet *)iconapplet, "group");
    cdos_applet_set_applet_tooltip((CdosApplet *)iconapplet,_("running applets"));

    priv->menu = cdos_applet_get_popup_menu((CdosApplet *)self);
    st_widget_set_style(ST_WIDGET(priv->menu->actor),"min-width:90px;");

    priv->box = ST_BOX_LAYOUT(st_box_layout_new());
    st_box_layout_set_vertical(priv->box, TRUE);
    cdos_popup_menu_base_add_actor(priv->menu, priv->box);

    priv->table = ST_TABLE(st_table_new());
    g_object_set(priv->table,"homogeneous",FALSE,NULL);
    clutter_container_add_actor((ClutterContainer*) priv->box, CLUTTER_ACTOR(priv->table));

    priv->count = 0;
    priv->row_size = 0;
    priv->separator = cdos_popup_separator_menu_item_new();

    cdos_popup_menu_base_add_menu_item(CDOS_POPUP_MENU_BASE(priv->menu),priv->separator,-1);
    cdos_popup_separator_menu_item_hide(priv->separator);

    priv->setting_item = cdos_popup_menu_item_new();
    cdos_popup_menu_item_set_label(priv->setting_item,_("Settings"));
    cdos_popup_menu_item_set_label_align(priv->setting_item,ST_ALIGN_MIDDLE,0);
    g_signal_connect((CdosPopupBaseMenuItem *)priv->setting_item, "activate", G_CALLBACK(applet_group_setting_item_on_activate), self);
    cdos_popup_menu_base_add_menu_item(CDOS_POPUP_MENU_BASE(priv->menu),priv->setting_item,-1);
    StLabel *label = cdos_popup_menu_item_get_label(priv->setting_item);
    st_widget_add_style_class_name(ST_WIDGET(label),"popup-menu-item-label-link");
}

static AppletGroup *
applet_group_new(MetaData *metadata , guint orientation , gfloat panel_height)
{
    AppletGroup *group;
    AppletGroupPrivate *priv = NULL;
    group = g_object_new(APPLET_TYPE_GROUP, NULL);
    priv = group->priv;
    CdosIconApplet *iconapplet = (CdosIconApplet *)group;

    priv->orientation = orientation;
    cdos_iconapplet_reset_for_systray(iconapplet);
    return group;
}
