#include "applet-button.h"


G_DEFINE_TYPE(AppletButton, applet_button, CDOS_TYPE_POPUP_BASE_MENU_ITEM);

#define APPLET_BUTTON_GET_PRIVATE(obj) \
       (G_TYPE_INSTANCE_GET_PRIVATE ((obj), APPLET_TYPE_BUTTON, AppletButtonPrivate))


struct _AppletButtonPrivate
{
        CdosApp                 *app;
        CdosPopupMenu           *menu;
        AppletMenu              *applet_button;
        CdosDocInfo           *file;
};

static void 
applet_button_dispose(GObject *gobject);

static void 
applet_button_init(AppletButton *button);

static void 
applet_button_class_init(AppletButtonClass *klass);


static void 
applet_button_class_init(AppletButtonClass *klass)
{
        GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
        GParamSpec *pspec;

        gobject_class->dispose = applet_button_dispose;

        g_type_class_add_private(klass, sizeof(AppletButtonPrivate));
}

static void 
applet_button_init(AppletButton *button)
{
        button->priv = APPLET_BUTTON_GET_PRIVATE(button);
        button->priv->app = NULL;
        button->priv->menu = NULL;
        button->priv->applet_button = NULL;
        button->priv->file = NULL;
}

AppletButton * 
applet_button_new()
{
        GObject *gobject = g_object_new(APPLET_TYPE_BUTTON, NULL);

        return APPLET_BUTTON(gobject);
}


static void 
applet_button_dispose(GObject *gobject)
{
        AppletButtonPrivate *priv = APPLET_BUTTON(gobject)->priv;

        if (priv->app)
            g_object_unref(priv->app);

        g_clear_object (&priv->menu);

        g_clear_object (&priv->applet_button);

        if (priv->file)
            g_object_unref(priv->file);
}

AppletButton *
applet_button_set_params(AppletButton *button, 
                        CdosApp *app, 
                        CdosPopupMenu *menu, 
                        AppletMenu *applet_button,
                        CdosDocInfo *file)
{
        if (button == NULL)
            return NULL;

        AppletButtonPrivate *priv = button->priv;

        if (app) {
            if (priv->app)
                g_object_unref(priv->app);

            g_object_ref(app);
            priv->app = app;
        } 

        if (menu) {
            if (priv->menu)
                g_object_unref(priv->menu);

            g_object_ref(menu);
            priv->menu = menu;     
        }

        if (applet_button) {
            if (applet_button)
                g_object_unref(priv->applet_button);

            g_object_ref(applet_button);
            priv->applet_button = applet_button;
        }

        if (file) {
            if (priv->file)
                g_object_unref(priv->file);

            g_object_ref(file);
            priv->file = file;
        }

        return button;
}

AppletMenu *
applet_button_get_applet_menu(AppletButton *button)
{
        if (!button)
                return NULL;

        return button->priv->applet_button;
}

void 
applet_button_set_applet_menu(AppletButton *button, AppletMenu *applet_button)
{
        if (button == NULL || applet_button == NULL)
                return;

        if (button->priv->applet_button)
            g_object_unref(button->priv->applet_button);

        g_object_ref(applet_button);
        button->priv->applet_button = applet_button;
}


CdosPopupMenu *
applet_button_get_popup_menu(AppletButton *button)
{
        if (!button)
                return NULL;

        return button->priv->menu;
}

void 
applet_button_set_popup_menu(AppletButton *button, CdosPopupMenu *menu)
{
        if (button == NULL || menu == NULL)
                return;

        if (button->priv->menu) 
            g_object_unref(button->priv->menu);

        g_object_ref(menu);
        button->priv->menu = menu;
}

CdosApp *
applet_button_get_app(AppletButton *button)
{
        if (!button)
                return NULL;

        return button->priv->app;
}


void 
applet_button_set_app(AppletButton *button, CdosApp *app)
{
        if (button == NULL || app == NULL)
                return;

        if (button->priv->app)
            g_object_unref(button->priv->app);

        g_object_ref(app);
        button->priv->app = app;
}

CdosDocInfo *
applet_button_get_file(AppletButton *button)
{ 
    if (button == NULL)
        return NULL;

    return button->priv->file;
}

void 
applet_button_set_file(AppletButton *button, CdosDocInfo *file)
{
    if (button == NULL || file == NULL)
        return;

    if (button->priv->file)
        g_object_unref(button->priv->file);

    g_object_ref(file);
    button->priv->file = file;
}


