#include "applet-category-button.h"
#include "applet-menu.h"

G_DEFINE_TYPE (AppletCategoryButton, applet_category_button, CDOS_TYPE_POPUP_SUBMENU_MENU_ITEM);

/*"所有应用程序"类别按钮*/

static void submenu_open_state_changed (GObject *object, gpointer user_data);
//zhangdj start
static gboolean
on_button_press_event (ClutterActor *actor,
                       ClutterEvent *event,
                       gpointer user_data);
                       
static gboolean
on_button_release_event (ClutterActor *actor,
                         ClutterEvent *event,
                         gpointer user_data);
//zhangdj end

typedef struct
{
	AppletCategoryButton *category_button;
	gboolean open;
}MenuData;

static void
applet_category_button_finalize (GObject *object)
{
	AppletCategoryButton *category_button;
	
	category_button = APPLET_CATEGORY_BUTTON (object);
	
	g_free (category_button->icon_name);
	
	g_clear_object (&category_button->menu);
	
	G_OBJECT_CLASS (applet_category_button_parent_class)->finalize (object);
}

static void
applet_category_button_class_init (AppletCategoryButtonClass *klass)
{
	GObjectClass *object_class;
	
	object_class = G_OBJECT_CLASS (klass);
	
	object_class->finalize = applet_category_button_finalize;

    CdosPopupBaseMenuItemClass *base_class = CDOS_POPUP_BASE_MENU_ITEM_CLASS (klass);
    base_class->activate = applet_category_button_activate;
}

static void
menu_data_free (gpointer data)
{
	g_slice_free (MenuData, data);
}

static void
applet_category_button_init (AppletCategoryButton *self)
{
	StWidget *actor;
	MenuData *data;
	
	self->icon_name = g_strdup ("");
	self->label = NULL;
	self->icon = NULL;
	
	PopupBaseMenuItemInit  *params = g_slice_new (PopupBaseMenuItemInit);
	params = cdos_popup_base_menu_item_get_init_params (params);
	params->hover = FALSE;
	params->focusOnHover = TRUE;
	cdos_popup_base_menu_item_set_params (CDOS_POPUP_BASE_MENU_ITEM (self), params);
	
	actor = ST_WIDGET(cdos_popup_base_menu_item_get_actor (CDOS_POPUP_BASE_MENU_ITEM (self)));
	st_widget_add_style_class_name (actor, "menu-category-button");
	
	self->menu = applet_application_menu_new (actor);

	data = g_slice_new (MenuData);
	data->category_button = self;
	data->open = FALSE;

	//zhangdj start
	g_signal_connect (actor,
	                  "button-press-event",
	                  G_CALLBACK(on_button_press_event),
	                  NULL);
	                  
    g_signal_connect (actor,
	                  "button-release-event",
	                  G_CALLBACK(on_button_release_event),
	                  NULL);
    //zhangdj end
	
	g_signal_connect_data (self->menu, "open-state-changed",
	                       G_CALLBACK(submenu_open_state_changed),
	                       data,
	                       (GClosureNotify) menu_data_free, 0);
	//st_widget_set_style (ST_WIDGET(self->menu->actor), "padding:1px 0; background: #fff;");
}

AppletCategoryButton *
applet_category_button_new (GMenuTreeDirectory *category)
{
	AppletCategoryButton *category_button;
	const gchar *label;
	GIcon *icon;
	ActorChild params;
    const gchar *others = "applications-other";

	category_button = g_object_new (APPLET_TYPE_CATEGORY_BUTTON, NULL);
	
	if (category)
	{
		icon = gmenu_tree_directory_get_icon (category);
		g_autofree gchar *name = NULL;
		if (!icon || !(name = g_icon_to_string (icon)))
		{
            icon = NULL;
		}
		
		label = gmenu_tree_directory_get_name (category);
		//g_print ("\n%s:---->category_icon_name = %s\n",__func__, category_button->icon_name);   //////调试
	}
	else
	{
		label = _("All Applications");
	}
	
	category_button->label = ST_LABEL(st_label_new (label));
	st_widget_add_style_class_name (ST_WIDGET(category_button->label), "menu-application-button-label");
	
	if (category)
	{
		category_button->icon = ST_ICON(st_icon_new ());
        if (!icon)
            st_icon_set_icon_name (category_button->icon, others);
        else
            st_icon_set_gicon (category_button->icon, icon);
		st_icon_set_icon_size (category_button->icon, CATEGORY_ICON_SIZE);
		st_icon_set_icon_type (category_button->icon, ST_ICON_FULLCOLOR);
		
		cdos_popup_base_menu_item_get_actor_child (&params);
		cdos_popup_base_menu_item_add_actor (CDOS_POPUP_BASE_MENU_ITEM(category_button), CLUTTER_ACTOR(category_button->icon), &params);
	}
	
	cdos_popup_base_menu_item_add_actor (CDOS_POPUP_BASE_MENU_ITEM(category_button), CLUTTER_ACTOR(category_button->label), &params);
    
    //g_signal_connect(category_button, "activate", G_CALLBACK(applet_category_button_activate), NULL);

	return category_button;
}

void
applet_category_button_activate (AppletCategoryButton *category_button, gpointer event, gboolean keep, gpointer data)
{
	g_return_if_fail (APPLET_IS_CATEGORY_BUTTON (category_button));
	guint event_type = clutter_event_get_button((ClutterEvent *)event);
	if(event_type == 1){
		applet_application_menu_togglex (category_button->menu);
	}
}

void
applet_category_button_remove_items (AppletCategoryButton *category_button)
{
	GList *list;
	GList *l;
	gpointer item;
	StBoxLayout *box;
	ClutterActor *actor;
	CdosPopupMenu *menu;
	
	g_return_if_fail (APPLET_IS_CATEGORY_BUTTON (category_button));
	
	box = cdos_popup_menu_base_get_box (CDOS_POPUP_MENU_BASE(category_button->menu));
	
	list = cdos_popup_menu_base_get_menu_items (CDOS_POPUP_MENU_BASE(category_button->menu));
	
	for (l = list; l != NULL; l = l->next)
	{
		item = l->data;
		
		/*item是CdosPopupBaseMenuItem*/
		if (CDOS_IS_POPUP_BASE_MENU_ITEM(item))
		{
			actor = CLUTTER_ACTOR(cdos_popup_base_menu_item_get_actor (CDOS_POPUP_BASE_MENU_ITEM(item)));
			clutter_actor_remove_child (CLUTTER_ACTOR(box), actor);
		}
		
		/*item是CdosPopupSubMenuItem*/
		if (CDOS_IS_POPUP_SUBMENU_MENU_ITEM(item))
		{
			menu = cdos_popup_submenu_menu_item_get_menu (CDOS_POPUP_SUBMENU_MENU_ITEM(item));
			actor = CLUTTER_ACTOR(menu->actor);
			clutter_actor_remove_child (CLUTTER_ACTOR(box), actor);
		}
	}
}

void
applet_category_button_destroy (AppletCategoryButton *category_button)
{
	g_return_if_fail (APPLET_IS_CATEGORY_BUTTON (category_button));
	
	cdos_popup_menu_base_destroy (CDOS_POPUP_MENU_BASE(category_button->menu));
}

//回调函数
static void
submenu_open_state_changed (GObject *object, gpointer user_data)
{
	/*没有被调用*/
	MenuData *data;
	StWidget *actor;
	
	data = user_data;
	
	actor = ST_WIDGET(cdos_popup_base_menu_item_get_actor (CDOS_POPUP_BASE_MENU_ITEM (data->category_button)));
	
	if (data->open)
	{
		st_widget_add_style_pseudo_class (actor, "open");
	}
	else
	{
		st_widget_remove_style_pseudo_class (actor, "open");
	}
}

//zhangdj start
static gboolean
on_button_press_event (ClutterActor *actor,
                       ClutterEvent *event,
                       gpointer user_data)
{
    st_widget_add_style_class_name (ST_WIDGET(actor), "menu-category-button-selected");
    
    return TRUE;
}

static gboolean
on_button_release_event (ClutterActor *actor,
                       ClutterEvent *event,
                       gpointer user_data)
{
    st_widget_remove_style_class_name (ST_WIDGET(actor), "menu-category-button-selected");
    
    return TRUE;
}
//zhangdj end
