#include "applet-hover-icon.h"
#include "gtk/gtk.h"   //zhangdj

G_DEFINE_TYPE (AppletHoverIcon, applet_hover_icon, G_TYPE_OBJECT);

static void applet_hover_icon_on_user_changed (GObject *object, gpointer *user_data);

static gboolean on_button_press_callback (ClutterActor *menu,ClutterEvent *event,AppletHoverIcon *self);

struct _AppletHoverIconPrivate
{
	StBin 		*user_icon;
	ActUser 	*user;
	guint 		user_loaded_id;
	guint 		user_changed_id;
	StIcon 		*icon;
};

static void
applet_hover_icon_finalize (GObject *object)
{
	AppletHoverIcon *hover_icon;
	
	hover_icon = APPLET_HOVER_ICON (object);
	
	g_clear_object (&hover_icon->user_box);
	g_clear_object (&hover_icon->user_label);
	g_clear_object (&hover_icon->actor);
	g_clear_object (&hover_icon->image);
	
	G_OBJECT_CLASS (applet_hover_icon_parent_class)->finalize (object);
}

static void
applet_hover_icon_class_init (AppletHoverIconClass *klass)
{
	GObjectClass *object_class;
	
	object_class = G_OBJECT_CLASS (klass);
	
	object_class->finalize = applet_hover_icon_finalize;
	
	g_type_class_add_private (klass, sizeof(AppletHoverIconPrivate));
}

//zhangdj  start
const gchar *
get_default_user_icon_path (const gchar *icon_name, gint size)
{
    GtkIconTheme *icon_theme;
    GtkIconInfo *icon_info;
    const gchar *filename;
    
    icon_theme = gtk_icon_theme_get_default ();
    icon_info = gtk_icon_theme_lookup_icon (icon_theme, icon_name, size, 0);
    filename = gtk_icon_info_get_filename (icon_info);
    
    return filename;
}
//zhangdj  end

static void
applet_hover_icon_on_user_changed (GObject *object, gpointer *user_data)
{
	const gchar *user_name;
	const gchar *icon_file_name;
	GFile *icon_file;
	//GIcon *icon;
	//StTextureCache *texture_cache;
	AppletHoverIcon *self;   //added
	
	//g_return_if_fail (APPLET_IS_HOVER_ICON(self));
	
	self = APPLET_HOVER_ICON (user_data);
	
	self->priv = APPLET_HOVER_ICON_GET_PRIVATE (self);
	
	clutter_actor_show (CLUTTER_ACTOR(self->user_box));
	
	/*用户已登录*/
	//ActUser *user = self->priv->user;
	//g_print ("---->count=%d\n", count);   ////////调试
	//gboolean is_loaded = act_user_is_loaded (self->priv->user);
	//gboolean is_loaded = TRUE;
	//g_print ("---->is_loaded=%d\n", is_loaded);   ////////调试
	if (act_user_is_loaded (self->priv->user))
	{
		user_name = g_get_user_name ();
		if (user_name)
		{
			if(strlen(user_name) > 8){
				st_widget_set_style(ST_WIDGET(self->user_label),"max-width:4em;");
			}
			st_label_set_text (self->user_label, user_name);

		}
		else
		{
			st_label_set_text (self->user_label, "");
		}
		
		if (self->priv->user_icon)
		{
			//zhangdj start
			clutter_actor_remove_all_children (CLUTTER_ACTOR(self->priv->user_icon));
			g_object_unref(self->image);
			self->image = NULL;
			
			const gchar *home_dir;
			home_dir = g_get_home_dir ();
			icon_file_name =g_build_filename (home_dir, ".face", NULL);
			//zhangdj end
			
			//icon_file_name = act_user_get_icon_file (self->priv->user);   //用户账户图标所在路径
			icon_file = g_file_new_for_path (icon_file_name);
			
			/*
			if (g_file_query_exists (icon_file, NULL))
			{
				icon = g_file_icon_new (icon_file);
			}
			else
			{
				icon = g_themed_icon_new ("avatar-default");
			}
			
			texture_cache = st_texture_cache_get_default ();
			self->image = st_texture_cache_load_gicon (texture_cache, 
			                                     NULL,
			                                     icon,
			                                     HOVER_ICON_SIZE);   //获取图片
            */
            
            //zhangdj start
            if (!g_file_query_exists (icon_file, NULL))
			{
			    icon_file_name = get_default_user_icon_path ("avatar-default", 96);
			}
			
			self->image = clutter_texture_new_from_file (icon_file_name, NULL);
			clutter_actor_set_size (self->image, HOVER_ICON_SIZE, HOVER_ICON_SIZE);
			//zhangdj end
            
			                                     
			st_bin_set_child (self->priv->user_icon, self->image);
			g_object_ref(self->image);

			st_widget_add_style_class_name (ST_WIDGET(self->priv->user_icon), "light");
			
			clutter_actor_show (CLUTTER_ACTOR(self->priv->user_icon));
			clutter_actor_show (CLUTTER_ACTOR(self->user_label));
		}
	}
}

static void
applet_hover_icon_on_user_loaded (GObject *object,
                                  GParamSpec *pspec, 
                                  AppletHoverIcon *self)
{
    applet_hover_icon_on_user_changed (object, self);
}

static gboolean
on_button_press_callback (ClutterActor *menu,ClutterEvent *event,AppletHoverIcon *self)
{
	guint32 button = clutter_event_get_button(event);
	if(button == 1){
		cdos_util_spawn_command_line ("cdos-settings-users");
	}
	return FALSE;
}

void
applet_hover_icon_refresh (AppletHoverIcon *hover_icon, const gchar *icon_name)
{
	hover_icon->priv = APPLET_HOVER_ICON_GET_PRIVATE (hover_icon);


	if (icon_name)
	{	
		st_bin_set_child (hover_icon->priv->user_icon, CLUTTER_ACTOR(hover_icon->priv->icon));
		g_object_ref(hover_icon->priv->icon);
		st_widget_remove_style_class_name (ST_WIDGET(hover_icon->priv->user_icon), "light");
		
		st_icon_set_icon_name (hover_icon->priv->icon, icon_name);
		clutter_actor_show (CLUTTER_ACTOR(hover_icon->priv->icon));
		
		clutter_actor_hide (CLUTTER_ACTOR(hover_icon->user_label));
	}
	else
	{
		st_bin_set_child (hover_icon->priv->user_icon, hover_icon->image);
		g_object_ref(hover_icon->image);
		st_widget_add_style_class_name (ST_WIDGET(hover_icon->priv->user_icon), "light");
		
		clutter_actor_show (CLUTTER_ACTOR(hover_icon->user_box));
		clutter_actor_show (CLUTTER_ACTOR(hover_icon->user_label));
	}
}

static void
applet_hover_icon_init (AppletHoverIcon *self)
{
	StBoxLayout *user_box;
	StBoxLayout *box;
	ActUserManager *manager;
	const gchar *user_name;
	
	self->priv = APPLET_HOVER_ICON_GET_PRIVATE (self);

	user_box = g_object_new(ST_TYPE_BOX_LAYOUT, 
			"style-class", "user-box",
			"reactive", TRUE,
			"vertical", TRUE,
			NULL);
	
	/*用户图标*/
	self->priv->user_icon = ST_BIN(st_bin_new ());
	st_widget_add_style_class_name (ST_WIDGET(self->priv->user_icon), "user-icon");
	st_bin_set_fill (self->priv->user_icon, TRUE, FALSE);	
	st_bin_set_alignment (self->priv->user_icon, ST_ALIGN_END, ST_ALIGN_START);
	
	clutter_actor_hide (CLUTTER_ACTOR(self->priv->user_icon));
	
	st_box_layout_insert_actor (user_box, CLUTTER_ACTOR(self->priv->user_icon), -1);
	
	/*用户名*/
	self->user_label = ST_LABEL(st_label_new (""));
	st_widget_add_style_class_name (ST_WIDGET(self->user_label), "user-label");
	
	box = ST_BOX_LAYOUT(st_box_layout_new ());
	st_box_layout_set_vertical (box, TRUE);
	//st_widget_set_style (ST_WIDGET(box), "padding: 5px 0");
	st_box_layout_insert_actor (box, CLUTTER_ACTOR(self->user_label), -1);
	
	st_box_layout_insert_actor (user_box, CLUTTER_ACTOR(box), -1);
    ClutterChildMeta *child = clutter_container_get_child_meta ((ClutterContainer *)user_box, (ClutterActor *)box);
    g_object_set (child, "x-fill", TRUE, "y-fill", FALSE, "x-align", ST_ALIGN_END, "y-align", ST_ALIGN_END, NULL);
	
	manager = act_user_manager_get_default ();
	user_name = g_get_user_name ();   //获取用户名

	self->priv->user = act_user_manager_get_user (manager, user_name);   //获取用户登录信息

	self->priv->user_loaded_id = g_signal_connect (self->priv->user, 
	                                               "notify::is_loaded",
	                                               G_CALLBACK(applet_hover_icon_on_user_loaded),
	                                               self);

	self->priv->user_changed_id = g_signal_connect (self->priv->user, 
	                                               "changed",
	                                               G_CALLBACK(applet_hover_icon_on_user_changed),
	                                               self);
	
	
	self->actor = ST_BIN(st_bin_new ());
	st_widget_add_style_class_name (ST_WIDGET(self->actor), "menu-hover-icon");
	
	self->user_box = user_box;
	
	self->priv->icon = ST_ICON(st_icon_new ());
	st_icon_set_icon_size (self->priv->icon, HOVER_ICON_SIZE);
	st_icon_set_icon_type (self->priv->icon ,ST_ICON_FULLCOLOR);
	
	st_bin_set_child (self->actor, CLUTTER_ACTOR(self->user_box));
	clutter_actor_show (CLUTTER_ACTOR(self->actor));
	
	applet_hover_icon_on_user_changed (NULL, self);
}

AppletHoverIcon *
applet_hover_icon_new (AppletMenu *menu)    //参数应该是menu，menu类型未确定
{
	AppletHoverIcon *hover_icon = g_object_new (APPLET_TYPE_HOVER_ICON, NULL);

	g_signal_connect (CLUTTER_ACTOR(hover_icon->user_box),
	                  "button-press-event",
	                  G_CALLBACK(on_button_press_callback),
			  menu);
	return hover_icon;
}
