#include "applet-recent-button.h"
#include "cdos-util.h"
#include "cdos-debug.h"
#include "cdos-message-tray.h"
#include "cdos-monkey-plugin.h"

G_DEFINE_TYPE(AppletRecentButton, applet_recent_button, APPLET_TYPE_BUTTON);

#define APPLET_RECENT_BUTTON_GET_PRIVATE(obj)   \
    (G_TYPE_INSTANCE_GET_PRIVATE ((obj), APPLET_TYPE_RECENT_BUTTON, AppletRecentButtonPrivate))

struct _AppletRecentButtonPrivate 
{
    StLabel *label;
};

enum {
    PROP_0
};

enum
{
    LAST_SIGNAL
};


static void
applet_recent_button_finalize(GObject *object)
{
     AppletRecentButton *self = APPLET_RECENT_BUTTON(object);
     AppletRecentButtonPrivate *priv = self->priv;

     g_object_unref(priv->label);

     if (self->name)
     {
        g_free(self->name);
     }

     /*调用父类的finalize()方法*/
     G_OBJECT_CLASS(applet_recent_button_parent_class)->finalize(object);
}

static void
applet_recent_button_set_property(GObject       *gobject,
                                  guint          prop_id,
                                  const GValue  *value, 
                                  GParamSpec    *pspec)
{
    AppletRecentButton *obj = APPLET_RECENT_BUTTON(gobject);

    switch (prop_id)
    {
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID(gobject, prop_id, pspec);
            break;
    }
}

static void
applet_recent_button_get_property(GObject * gobject,
                guint prop_id, GValue * value, GParamSpec * pspec)
{
    AppletRecentButtonPrivate *priv = APPLET_RECENT_BUTTON(gobject)->priv;
    switch (prop_id)
    {
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID(gobject, prop_id, pspec);
            break;
    }
}

static void
applet_recent_button_class_init(AppletRecentButtonClass * klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
    GParamSpec *pspec;
    gobject_class->set_property = applet_recent_button_set_property;
    gobject_class->get_property = applet_recent_button_get_property;

    gobject_class->finalize = applet_recent_button_finalize;

    g_type_class_add_private(klass, sizeof (AppletRecentButtonPrivate));
}

static void
applet_recent_button_init(AppletRecentButton * self)
{
    AppletRecentButtonPrivate *priv;
    priv = self->priv = APPLET_RECENT_BUTTON_GET_PRIVATE(self);

    priv->label = NULL;
    self->name = NULL;
}

static void
on_button_release_event(AppletRecentButton *self )
{

}

static void 
applet_recent_button_load_menu(AppletRecentButton *button, AppletMenu *applet_button)
{

        CdosPopupMenu *menu;
        AppletApplicationContextmenuItem *menu_item;
        AppletMenu *applet;
        PopupBaseMenuItemInit  params;

        cdos_popup_base_menu_item_get_init_params (&params);
        params.hover = FALSE;
        params.focusOnHover = FALSE;
        cdos_popup_base_menu_item_set_params (CDOS_POPUP_BASE_MENU_ITEM(button), &params);


        // get global value of uiGroup
        MonkeyPlugin *plugin = monkey_plugin_get ();
        ClutterActor *uigroup = monkey_plugin_get_ui_group(plugin);

        g_signal_connect(button, "activate", G_CALLBACK(applet_recent_button_activate), NULL);

        applet = applet_button;
        applet_button_set_applet_menu(APPLET_BUTTON(button), applet);

        menu = applet_contextmenu_new(button);

        applet_button_set_popup_menu(button, menu);
        
        st_widget_set_style_class_name(menu->actor, "menu-context-menu");
        menu_item = applet_application_contextmenu_item_new(button, _("Remove from list"), REMOVE_RECENT_ITEM);
        cdos_popup_menu_base_add_menu_item(CDOS_POPUP_MENU_BASE(menu), CDOS_POPUP_BASE_MENU_ITEM(menu_item), -1);
        menu_item = applet_application_contextmenu_item_new(button, _("Clear list"), REMOVE_RECENT_ALL);
        cdos_popup_menu_base_add_menu_item(CDOS_POPUP_MENU_BASE(menu), CDOS_POPUP_BASE_MENU_ITEM(menu_item), -1);

        cdos_popup_menu_base_add_child_menu(applet->menu, menu);
        st_container_add_actor ((ClutterContainer *) uigroup, CLUTTER_ACTOR(menu->actor));

}

//zhangdj start
static gboolean
applet_recent_button_button_press_event (ClutterActor *actor,
                                         ClutterEvent *event,
                                         gpointer user_data)
{
    st_widget_add_style_pseudo_class (ST_WIDGET(actor), "click");
	
	return TRUE;
}

static gboolean
applet_recent_button_button_release_event (ClutterActor *actor,
                                         ClutterEvent *event,
                                         gpointer user_data)
{
    st_widget_remove_style_pseudo_class (ST_WIDGET(actor), "click");
	
	return TRUE;
}
//zhangdj end

static void
on_allocation (StWidget *widget,
               GParamSpec *pspec,
               AppletRecentButton *self)
{
    AppletRecentButtonPrivate *priv = self->priv;
    ClutterText *clutter_text = (ClutterText *)st_label_get_clutter_text (priv->label);
    PangoLayout *layout = clutter_text_get_layout (clutter_text);
    if (pango_layout_is_ellipsized (layout))
    {
        if (st_widget_get_has_tooltip (widget))
            return;
        st_widget_set_has_tooltip (widget, TRUE);
        st_widget_set_tooltip_text (widget, st_label_get_text (priv->label));
    }
    else
    {
        st_widget_set_has_tooltip (widget, FALSE);
    }
}

AppletRecentButton *
applet_recent_button_new(AppletMenu *applet, CdosDocInfo *file)
{

    AppletRecentButton *object = NULL;
    object = g_object_new(APPLET_TYPE_RECENT_BUTTON, NULL);

    AppletRecentButtonPrivate *priv;
    PopupBaseMenuItemInit params;
    CdosGenericContainer *actor;
    gchar *name;
    ClutterActor *icon;
    gint icon_size = MAX_FAV_ICON_SIZE;
    ActorChild child;

    priv = object->priv;
    cdos_popup_base_menu_item_get_init_params(&params);
    params.hover = FALSE;
    cdos_popup_base_menu_item_set_params(object, &params);

    applet_button_set_file(object, file);
    actor = cdos_popup_base_menu_item_get_actor(CDOS_POPUP_BASE_MENU_ITEM(object));
    st_widget_set_style_class_name(ST_WIDGET(actor), "menu-favorites-button");
    //zhangdj start
    g_signal_connect (actor,
                      "button-press-event",
                      G_CALLBACK(applet_recent_button_button_press_event),
                      NULL);
                      
    g_signal_connect (actor,
                      "button-release-event",
                      G_CALLBACK(applet_recent_button_button_release_event),
                      NULL);
    //zhangdj end

    name = cdos_doc_info_get_name(file);
    priv->label = g_object_new(ST_TYPE_LABEL, "text", name, "style-class", "menu-application-button-label", NULL);
    ClutterText *clutter_text = (ClutterText *)st_label_get_clutter_text (priv->label);
    clutter_text_set_ellipsize (clutter_text, PANGO_ELLIPSIZE_MIDDLE);
    st_widget_set_style(ST_WIDGET(priv->label), "max-width:14em;");

    icon = cdos_doc_info_create_icon(file, icon_size);
    cdos_popup_base_menu_item_get_actor_child(&child);
    cdos_popup_base_menu_item_add_actor(object, icon, &child);
    cdos_popup_base_menu_item_add_actor(object, priv->label, &child);

    g_signal_connect (actor, "notify::allocation", G_CALLBACK (on_allocation), object);
    applet_recent_button_load_menu(object, applet);
    g_object_unref(icon);

    return object;
}

void 
applet_recent_button_activate(AppletRecentButton *self,  gpointer event, gboolean keep, gpointer data)
{
	AppletRecentButtonPrivate *priv;
	guint event_type = clutter_event_get_button((ClutterEvent *)event);
	priv = self->priv;
    gfloat x, y;


    CdosPopupMenu *menu;
    AppletMenu *applet;
    CdosDocInfo *file;
    menu = applet_button_get_popup_menu(self);
    applet = applet_button_get_applet_menu(self);
    applet_menu_set_recent_button_popup_menu (applet, menu);   //zhangdj
    file = applet_button_get_file(self);

    const gchar *uri = cdos_doc_info_get_uri(file);
    GFile *g_file = g_file_new_for_uri(uri);
    if(g_file_query_exists(g_file,NULL) == FALSE ||
       g_ascii_strncasecmp (uri, "smb:", 4) == 0)
    {
        gchar *msg = g_strconcat("notify-send ","'",_("This file is not exist..."),"' '",g_file_get_path(g_file),"' ",NULL);
        cdos_util_spawn_command_line(msg);
        cdos_message_tray_want_send_to_applet(FALSE);
        StLabel *label = applet_recent_button_get_label(self);
        st_widget_set_style(ST_WIDGET(label),"color:#666;text-decoration:line-through;max-width:14em;");
//        g_object_unref(g_file);
        //return ;
    }
//    g_object_unref(g_file);

    if(event_type == 1){
        if(menu && CDOS_POPUP_MENU_BASE(menu)->is_open){
            cdos_popup_menu_close(menu, TRUE);
        }else{
             if(g_file_query_exists(g_file,NULL) == TRUE)
             {
                cdos_doc_info_launch(file);
                cdos_popup_menu_close(applet->menu, TRUE);
             }
        }
    }else if(event_type == 3){
        if(menu){ 
            clutter_event_get_coords(event, &x, &y);  
            APPLET_CONTEXTMENU(menu)->x = x;
            APPLET_CONTEXTMENU(menu)->y = y;
            cdos_popup_menu_base_toggle(menu);
        }
    }
    g_object_unref(g_file);
}

void  
applet_recent_button_toggle_menu(AppletRecentButton *self)
{
    if (!self)
        return;

	AppletMenu *applet = applet_button_get_applet_menu(self);

	cdos_popup_menu_base_toggle(CDOS_POPUP_MENU_BASE(applet->menu));
}

/**
 * applet_recent_button_get_label:
 * @self: A #AppletRecentButton
 *
 * 获取self的label成员
 *
 * Returns: 
 */
StLabel *
applet_recent_button_get_label(AppletRecentButton *self)
{
    g_return_val_if_fail(APPLET_IS_RECENT_BUTTON(self),NULL);
    AppletRecentButtonPrivate *priv = self->priv;

    return priv->label;
}



