#include "applet-search-result-button.h"
#include "applet-menu.h"

/*搜索结果按钮*/

G_DEFINE_TYPE (AppletSearchResultButton, applet_search_result_button, APPLET_TYPE_GENERIC_APPLICATION_BUTTON);

/*用于访问私有结构*/
#define APPLET_SEARCH_RESULT_BUTTON_GET_PRIVATE(obj) \
  (G_TYPE_INSTANCE_GET_PRIVATE ((obj), APPLET_TYPE_SEARCH_RESULT_BUTTON, AppletSearchResultButtonPrivate))

struct _AppletSearchResultButtonPrivate
{

};

static void
applet_search_result_button_finalize (GObject *object)
{
	AppletSearchResultButton *search_result_button;
	
	search_result_button = APPLET_SEARCH_RESULT_BUTTON (object);
	
	clutter_actor_destroy (CLUTTER_ACTOR(search_result_button->icon));
	clutter_actor_destroy (CLUTTER_ACTOR(search_result_button->label));
	
	G_OBJECT_CLASS (applet_search_result_button_parent_class)->finalize (object);
}

static void
applet_search_result_button_class_init (AppletSearchResultButtonClass *klass)
{
	GObjectClass *object_class;
	
	object_class = G_OBJECT_CLASS (klass);
	
	object_class->finalize = applet_search_result_button_finalize;
}

static void
applet_search_result_button_init (AppletSearchResultButton *self)
{
	self->icon = NULL;
	self->label = NULL;
}

static gboolean
applet_search_result_button_enter_callback (ClutterActor *actor, ClutterEvent *event, gpointer user_data)
{
	//g_print ("in %s\n", __func__);   //////调试
	AppletMenu *menu = user_data;
	
	clear_previous_selection (menu);
    applet_menu_set_previous_selected_actor (menu, (CdosGenericContainer *)actor);
	applet_menu_set_active_actor (menu, (CdosGenericContainer *)actor);
	
	//st_widget_add_style_class_name (ST_WIDGET(actor), "menu-searchs-button-selected");
	st_widget_add_style_pseudo_class (ST_WIDGET(actor), "hover");
	
	applet_menu_set_state (menu, HOVER);
	
	return TRUE;
}

static gboolean
applet_search_result_button_leave_callback (ClutterActor *actor, ClutterEvent *event, gpointer user_data)
{
	//st_widget_remove_style_class_name (ST_WIDGET(actor), "menu-searchs-button-selected");
	st_widget_remove_style_pseudo_class (ST_WIDGET(actor), "hover");
	
	return TRUE;
}

//zhangdj start
static gboolean
applet_search_result_button_press_callback (ClutterActor *actor, ClutterEvent *event, gpointer user_data)
{
    st_widget_add_style_pseudo_class (ST_WIDGET(actor), "click");
	
	return TRUE;
}

static gboolean
applet_search_result_button_release_callback (ClutterActor *actor, ClutterEvent *event, gpointer user_data)
{
    st_widget_remove_style_pseudo_class (ST_WIDGET(actor), "click");
	
	return TRUE;
}
//zhangdj end

AppletSearchResultButton *
applet_search_result_button_new (AppletMenu *apps_menu_button, CdosApp *app)
{
	AppletSearchResultButton *search_result_button;
	StWidget *actor;
	const gchar *name;
	ActorChild params;
	
	search_result_button = g_object_new (APPLET_TYPE_SEARCH_RESULT_BUTTON, NULL);
	
	applet_generic_application_button_constructor(search_result_button, apps_menu_button, app, TRUE);
	
	APPLET_GENERIC_APPLICATION_BUTTON (search_result_button)->applet_button = apps_menu_button;    //menu完成时再加入
	APPLET_GENERIC_APPLICATION_BUTTON (search_result_button)->app = app;
	APPLET_GENERIC_APPLICATION_BUTTON (search_result_button)->with_menu = TRUE;

	actor = ST_WIDGET(cdos_popup_base_menu_item_get_actor (CDOS_POPUP_BASE_MENU_ITEM (search_result_button)));
	st_widget_add_style_class_name (actor, "menu-searchs-button");
	
	if (app)
	{
		search_result_button->icon = cdos_app_create_icon_texture (app, APPLICATION_ICON_SIZE);
		
		name = cdos_app_get_name (app);
		search_result_button->label = ST_LABEL(st_label_new (name));
		st_widget_add_style_class_name (ST_WIDGET(search_result_button->label), "menu-application-button-label");
		st_widget_set_style(ST_WIDGET(search_result_button->label), "max-width:14em;");

		cdos_popup_base_menu_item_get_actor_child (&params);
		cdos_popup_base_menu_item_add_actor (CDOS_POPUP_BASE_MENU_ITEM(search_result_button), search_result_button->icon, &params);
		cdos_popup_base_menu_item_add_actor (CDOS_POPUP_BASE_MENU_ITEM(search_result_button), CLUTTER_ACTOR(search_result_button->label), &params);
	}
	
	g_signal_connect (actor, "enter-event", G_CALLBACK(applet_search_result_button_enter_callback), apps_menu_button);
	g_signal_connect (actor, "leave-event", G_CALLBACK(applet_search_result_button_leave_callback), apps_menu_button);
	//zhangdj start
	g_signal_connect (actor, "button-press-event", G_CALLBACK(applet_search_result_button_press_callback), NULL);
	g_signal_connect (actor, "button-release-event", G_CALLBACK(applet_search_result_button_release_callback), NULL);
	//zhangdj end
	
	return search_result_button;
}

const gchar *
applet_search_result_button_get_app_id (CdosApp *app)
{
	const char *id;
	
	id = cdos_app_get_id (app);
	
	return id;
}
