#include "applet-shortcut-item.h"
#include "cdos-util.h"
#include "st.h"

/*开始菜单右侧快速启动项*/

struct _AppletShortcutItemPrivate
{
        GObject         *parent;
        AppletHoverIcon *hoverIcon;
        gchar           *command;
        const gchar     *icon_name;
        StLabel         *label;
};

G_DEFINE_TYPE(AppletShortcutItem, applet_shortcut_item, CDOS_TYPE_POPUP_BASE_MENU_ITEM);

#define APPLET_SHORTCUT_ITEM_GET_PRIVATE(obj)    \
        (G_TYPE_INSTANCE_GET_PRIVATE((obj), APPLET_TYPE_SHORTCUT_ITEM, AppletShortcutItemPrivate))

/*
static void
applet_shortcut_item_on_button_release_event(CdosGenericContainer *actor, 
                                        ClutterEvent *event,
                                        AppletShortcutItem *item);
*/
//added
static gboolean
applet_shortcut_item_on_button_press_event (CdosGenericContainer *actor,
                                            ClutterEvent *event,
                                            gpointer user_data);
                                            
static gboolean
applet_shortcut_item_on_button_release_event (CdosGenericContainer *actor,
                                              ClutterEvent *event,
                                              gpointer user_data);

static void 
applet_shortcut_item_finalize(GObject *gobject);

static void 
applet_shortcut_item_class_init(AppletShortcutItemClass *klass);

static void
applet_shortcut_item_init(AppletShortcutItem *item);


static void
applet_shortcut_item_init(AppletShortcutItem *item)
{
        AppletShortcutItemPrivate *priv;
        priv = item->priv = APPLET_SHORTCUT_ITEM_GET_PRIVATE(item);
        priv->parent = NULL;
 //       priv->hoverIcon = NULL;
        priv->command = NULL;
        priv->icon_name = NULL;
        priv->label = NULL;
}

static void 
applet_shortcut_item_class_init(AppletShortcutItemClass *klass)
{
        GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
        GParamSpec *pspec;

        g_type_class_add_private(klass, sizeof(AppletShortcutItemPrivate));

        gobject_class->finalize = applet_shortcut_item_finalize;
}

AppletShortcutItem *
applet_shortcut_item_new(const gchar *label, 
                        const gchar *icon_name, 
                        gchar *command, 
                        GObject *parent,
                        AppletHoverIcon *hoverIcon)
{
        GObject *object;
        object = g_object_new(APPLET_TYPE_SHORTCUT_ITEM, NULL);
        applet_shortcut_item_set_params(object, label, icon_name, command, parent, hoverIcon);

        return object;
}

AppletShortcutItem *
applet_shortcut_item_set_params(AppletShortcutItem *item,
                                const gchar *label, 
                                const gchar *icon_name, 
                                const gchar *command, 
                                GObject *parent,
                                AppletHoverIcon *hoverIcon)
{
        AppletShortcutItemPrivate *priv;
        ActorChild actor_params;

        priv = item->priv;

        if (priv->parent) {
                g_object_unref(priv->parent);
        }

        priv->parent = parent;
        g_object_ref(priv->parent);

        priv->hoverIcon = hoverIcon;
        g_object_ref(priv->hoverIcon);

        if (priv->icon_name) {
                g_free(priv->icon_name);
        }

        priv->icon_name = g_strdup(icon_name);

        if (priv->command) {
                g_free(priv->command);
        }

        priv->command = g_strdup(command);

        PopupBaseMenuItemInit init;
        cdos_popup_base_menu_item_get_init_params(&init);

        cdos_popup_base_menu_item_set_params(CDOS_POPUP_BASE_MENU_ITEM(item), &init);

        CdosGenericContainer  *actor = cdos_popup_base_menu_item_get_actor(CDOS_POPUP_BASE_MENU_ITEM(item));

        st_widget_set_style_class_name(ST_WIDGET(actor), "menu-shortcut-item");

        cdos_popup_base_menu_item_get_actor_child(&actor_params);
        priv->label = g_object_new(ST_TYPE_LABEL, "text",  label, "style-class", "menu-shortcut-item-label", NULL);
        cdos_popup_base_menu_item_add_actor(item, priv->label, &actor_params);

/*       g_signal_connect(actor,
                       "button-release-event",
                       G_CALLBACK(applet_shortcut_item_on_button_release_event),
                       item);*/
                       
        //added
        g_signal_connect (actor,
                          "button-press-event",
                          G_CALLBACK(applet_shortcut_item_on_button_press_event),
                          NULL);
                          
        g_signal_connect (actor,
                          "button-release-event",
                          G_CALLBACK(applet_shortcut_item_on_button_release_event),
                          NULL);

        g_signal_connect(item,
                        "active-changed",
                        G_CALLBACK(applet_shortcut_item_set_active),
                        NULL);

        g_signal_connect(item, 
                        "activate",
						G_CALLBACK(applet_shortcut_item_activate),
						priv->parent);
}


static void 
applet_shortcut_item_finalize(GObject *gobject)
{
        AppletShortcutItemPrivate *priv = APPLET_SHORTCUT_ITEM(gobject)->priv;

        if (priv->parent) {
                g_object_unref(priv->parent);
                priv->parent = NULL;
        }
        if (priv->icon_name) {
                g_free(priv->icon_name);
                priv->icon_name = NULL;
        }

        if (priv->command) {
                g_free(priv->command);
                priv->command = NULL;
        }

        if (priv->label) {
                clutter_actor_destroy(priv->label);
                priv->label = NULL;
        }

        if (priv->hoverIcon) {
                g_object_unref(priv->hoverIcon);
               priv->hoverIcon = NULL;
       }
}

void
applet_shortcut_item_set_active(AppletShortcutItem *item, gboolean active)
{
        AppletShortcutItemPrivate *priv;
        g_return_if_fail(APPLET_IS_SHORTCUT_ITEM(item));
        priv = item->priv;
        CdosGenericContainer  *actor;
        actor = cdos_popup_base_menu_item_get_actor(CDOS_POPUP_BASE_MENU_ITEM(item));
        
        if (active) {
                st_widget_remove_style_class_name(ST_WIDGET(actor), "menu-shortcut-item");
                st_widget_add_style_class_name(ST_WIDGET(actor), "menu-shortcut-item-selected");
                applet_hover_icon_refresh(priv->hoverIcon, priv->icon_name);
        } else {
                st_widget_remove_style_class_name(ST_WIDGET(actor), "menu-shortcut-item-selected");
                st_widget_add_style_class_name(ST_WIDGET(actor), "menu-shortcut-item");
                applet_hover_icon_refresh(priv->hoverIcon, NULL);
        }
}

/*static void
applet_shortcut_item_on_button_release_event(CdosGenericContainer *actor,
                                       ClutterEvent *event,
                                       AppletShortcutItem *item)
{
	g_print("applet_shortcut_item_on_button_release_event\n");
       if(clutter_event_get_button(event) == 1)
               applet_shortcut_item_activate(item, event);
}*/

//added
static gboolean
applet_shortcut_item_on_button_press_event (CdosGenericContainer *actor,
                                            ClutterEvent *event,
                                            gpointer user_data)
{
    st_widget_add_style_pseudo_class (ST_WIDGET(actor), "click");
    
    return TRUE;
}

static gboolean
applet_shortcut_item_on_button_release_event (CdosGenericContainer *actor,
                                              ClutterEvent *event,
                                              gpointer user_data)
{
    st_widget_remove_style_pseudo_class (ST_WIDGET(actor), "click");
    
    return TRUE;
}

void
applet_shortcut_item_activate(AppletShortcutItem *item, gpointer event, gboolean keep, gpointer data)
{
    guint32 button = clutter_event_get_button(event);
                if (button == 1){
                        cdos_util_spawn_command_line(item->priv->command);
                        cdos_popup_menu_close(item->priv->parent, TRUE);
                }

}
