/**
 * SECTION: AppletNotification
 * @short_description: 通知应用
 * @include: applet-notification.h
 * 
 * AppletNotificationClass 类用来接收，展示，处理 #CdosMessageTray 发送来地消息
 */
#include "applet-notifications.h"
#include <stdio.h>
#include "string.h"
#include <sys/time.h>
#include <math.h>
#include "ui/cdos-monkey-plugin.h"
#include "ui/cdos-popup-base-menu-item.h"
#define APPLET_NOTIFICATIONS_MAX_COUNT 48

/* ============ BASE ============ */
/* 注册函数 */
G_DEFINE_TYPE(AppletNotifications, applet_notifications, CDOS_TYPE_TEXT_ICON_APPLET)

/* 继承关系
 *
 * CdosGenericContainer->StContainer->StWidget->ClutterActor->GInitiallyUnowned->GObject
 * AppletNotifications->CdosTextIconApplet->IconApplet->Applet
 * StBoxLayout->StContainer->StWidget->ClutterActor->GInitiallyUnknowned->GObject
 * CdosPopupMenuItem->CdosPopupBaseMenuItem->GObject
 */

/* 自定义数据结构 */
typedef struct _DataItemClicked DataItemClicked;
struct _DataItemClicked {
    AppletNotifications *self;
    gboolean destroyed;
};

/* ========== BASE END ========== */

/* ============ 函数声明 ============ */

static void applet_notifications_item_destroy_cb (GObject *obj, guint reason, gpointer user_data);

static void applet_notifications_item_clicked_cb (GObject *obj, gpointer user_data);

/* 私有方法声明 */
static void applet_notifications_display (AppletNotifications *self);

static void applet_notifications_update_timestamp (AppletNotifications *self);

/* CALL_BACK */
static void applet_notifications_notification_added_cb (GObject *obj, CdosNotification *notification, gpointer user_data);

static void applet_notifications_clear_all_cb (GObject *obj, gpointer event, gboolean keepMenu, gpointer user_data);

static void callback_scroll_start (GObject *obj, gpointer user_data);

static void callback_scroll_stop (GObject *obj, gpointer user_data);

/* 待替换函数 */


/* 独立函数 */

/* ============ 独立函数 ============ */

static gchar *applet_notifications_stringify (gint count)
{
    gchar * str = NULL;

    if (count == 0) {

        str = g_strdup (_("No notifications"));

    } else if (count == 1) {

        str = g_strdup_printf("%s ( %d )", _(" notification"), count);

    } else if (count > 1) {

        str = g_strdup_printf("%s ( %d )", _(" notifications"), count);

    }
    return str;
}

static gchar *applet_notifications_timeify (GDateTime* orig_time, const char *format)
{
    GDateTime* now_timestamp = g_date_time_new_now_local();
    GTimeSpan diff = g_date_time_difference(now_timestamp, orig_time) / 1000000;
    gchar * str = NULL, *str_diff = NULL;
    gchar *orig = g_date_time_format (orig_time, format);

    if (diff <= 15) {
        str_diff = g_strdup(_(" (Just now)"));
    } else if (diff > 15 && diff <= 59) {
        gchar s_diff[3];
        s_diff[0] = '0' + diff / 10;
        s_diff[1] = '0' + diff % 10;
        s_diff[2] = '\0';
        str_diff = g_strdup_printf(_(" (%s seconds ago)"), s_diff, NULL);
    } else if (diff > 59 && diff <= 119) {
        gchar s_diff[2];
        s_diff[0] = '0' + diff / 60;
        s_diff[1] = '\0';
        str_diff = g_strdup_printf(_(" (%s minute ago)"), s_diff, NULL);
    //} else if (diff > 119 && diff <= 3540){
    } else if (diff > 119 && diff <= 599){
        gchar s_diff[2];
        s_diff[0] = '0' + diff / 60;
        s_diff[1] = '\0';
        str_diff = g_strdup_printf(_(" (%s minutes ago)"), s_diff, NULL);
    } else if (diff > 599 && diff <= 3540){
        gchar s_diff[3];
        gint d = diff / 60;
        s_diff[0] = '0' + d / 10;
        s_diff[1] = '0' + d % 10;
        s_diff[2] = '\0';
        str_diff = g_strdup_printf(_(" (%s minutes ago)"), s_diff, NULL);
    }

    str = g_strconcat (orig, str_diff, NULL);
    g_free (orig);

    g_date_time_unref(now_timestamp);
    
    return str;
}

/* ======== 独立函数 END ============ */

/* ========== 回调函数 ============ */

/**
 * applet_notifications_on_panel_edit_mode_changed_cb:
 * @self: A #global_settings
 * @pspec: GSettings的panel-edit-mode属性
 * @self: 回调函数发送的数据
 *
 * 信号"changed::panel-edit-mode"的回调函数
 *
 */
static void applet_notifications_on_panel_edit_mode_changed_cb (GSettings *global_settings, GParamSpec *pspec, AppletNotifications *self)
{

}

static void
applet_notifications_notification_added_cb (GObject *obj, CdosNotification *notification, gpointer user_data)
{
    g_return_if_fail (notification != NULL);
    AppletNotifications *self = APPLET_NOTIFICATIONS (user_data);

    if (self->ignore_transient_notifications && notification->isTransient)
        return;

    g_object_ref (notification->actor);//引用计数加1，防止被释放

    clutter_actor_unparent (CLUTTER_ACTOR(notification->actor));

    gint existing_index = g_list_index (self->notifications, notification);
    if (existing_index != -1) {

        /*NULL if it is not found*/
        if (notification->destroyed) {
            /*可移除的*/
            self->notifications = g_list_remove (self->notifications, notification);
        } else {
            notification->inNotificationBin = TRUE;
            /*global.reparentActor(notification.actor, this._notificationbin);*/
            cdos_reparent_actor (CLUTTER_ACTOR (notification->actor), CLUTTER_ACTOR (self->p_notification_bin));
            cdos_notification_expand (notification, TRUE);//TODO:好像没实现, 不选择展示动画
            clutter_actor_show (CLUTTER_ACTOR(notification->timeLabel));
        }

        applet_notifications_update_list (self);
        return;
    }

    if (notification->destroyed) {
        return;
    }

    st_widget_add_style_class_name(ST_WIDGET(notification->table),"notification-box-list-item");
    notification->inNotificationBin = TRUE;

    //st_widget_set_style (notification->actor, "background-color: red;");
    if (self->notification_count == APPLET_NOTIFICATIONS_MAX_COUNT) {
        GList *firstNode = g_list_first(self->notifications);
        cdos_notification_destroy (CDOS_NOTIFICATION(firstNode->data), 0);
        self->notification_count = self->notification_count - 1;
    }

    self->notifications = g_list_append (self->notifications, notification);
    self->notification_count += 1;
    cdos_notification_expand (notification, TRUE);
    st_box_layout_insert_actor(ST_BOX_LAYOUT (self->p_notification_bin), CLUTTER_ACTOR(notification->actor), -1);

    if(self->notification_count > 0){
        cdos_popup_separator_menu_item_show(self->clear_separator);
    }else{
        cdos_popup_separator_menu_item_hide(self->clear_separator);
    }
    /*下一句是否需要*/
    //cdos_draggable_set_delegate(notification->actor, notification->actor_delegate);
    notification->actor_parent_container = ST_WIDGET (self->p_notification_bin);
    st_widget_add_style_class_name (ST_WIDGET(notification->actor), "notification-applet-padding");

    DataItemClicked *data_clicked, *data_destroy;
    data_clicked = g_new0(DataItemClicked, 1);
    data_destroy = g_new0(DataItemClicked, 1);
    data_clicked->self = self;
    data_clicked->destroyed = FALSE;
    data_destroy->self = self;
    data_destroy->destroyed = TRUE;

    //g_signal_connect(notification, "clicked", G_CALLBACK (applet_notifications_item_clicked_cb), data_clicked);
    g_signal_connect(notification, "destroy", G_CALLBACK (applet_notifications_item_destroy_cb), data_destroy);

    clutter_actor_show (CLUTTER_ACTOR (notification->timeLabel));
    gfloat height;
    height = clutter_actor_get_height ((ClutterActor *)notification->actor);
    clutter_actor_set_size ((ClutterActor *)notification->actor, -1, height);
    /*
       guint count = self->notification_count;
    gfloat width, height;

    clutter_actor_get_size (CLUTTER_ACTOR (self->p_main_container), &width, &height);
    
    if ( height > 300 || count >= 5) {
        clutter_actor_set_size (CLUTTER_ACTOR (self->p_main_container), -1, 300);
    } else {
        clutter_actor_set_size (CLUTTER_ACTOR (self->p_main_container), -1, -1);
    }

    if (count == 0) {
        clutter_actor_set_size (CLUTTER_ACTOR(self->p_main_container), 0, 0);
    }
    */
    applet_notifications_update_list (self);
    //CdosLayoutManager *layout_manager = cdos_layout_manager_get_default ();
    //cdos_layout_manager_update_regions (layout_manager);
}

static void applet_notifications_clear_all_cb (GObject *obj, gpointer event, gboolean keepMenu, gpointer user_data)
{
    AppletNotifications *self = APPLET_NOTIFICATIONS (user_data);

    guint count;

    count = g_list_length (self->notifications);
    CdosNotification *notification_tmp = NULL;
    GList *notifications = self->notifications;
    cdos_debug();
    if (count > 0) {
        while (notifications != NULL) {
            if (notifications->data){
                notification_tmp = CDOS_NOTIFICATION (notifications->data);
                notifications = notifications->next;
                clutter_container_remove_actor ((ClutterContainer *) self->p_notification_bin,
                                                CLUTTER_ACTOR(notification_tmp->actor));
                cdos_notification_destroy (notification_tmp, DISMISSED);
            }
        }
        g_list_free(self->notifications);
    }
    self->notifications = NULL;
    self->notification_count = 0;
    applet_notifications_update_list (self);
}

static void applet_notifications_item_destroy_cb (GObject *obj, guint reason, gpointer user_data)
{
    CdosNotification *notification = CDOS_NOTIFICATION (obj);
    DataItemClicked *data = (DataItemClicked *)user_data;
    AppletNotifications *self = APPLET_NOTIFICATIONS (data->self);

    gint i = g_list_index (self->notifications, notification);
    if (i != -1) {
        /*NULL if it is not found*/
        self->notifications = g_list_remove (self->notifications, notification);
        if (!data->destroyed) {
            cdos_notification_destroy (notification, DISMISSED);
        }
    }

    g_print ("\n Signal: Clicked \n");
    applet_notifications_update_list (self);
}


static void applet_notifications_item_clicked_cb (GObject *obj, gpointer user_data)
{
    CdosNotification *notification = CDOS_NOTIFICATION (obj);
    DataItemClicked *data = (DataItemClicked *)user_data;
    AppletNotifications *self = APPLET_NOTIFICATIONS (data->self);

    gint i = g_list_index (self->notifications, notification);

    if (i != -1) {
        /*NULL if it is not found*/
        self->notifications = g_list_remove (self->notifications, notification);
        if (!data->destroyed) {
            cdos_notification_destroy (notification, DISMISSED);
        }
    }

    g_print ("\n Signal: Destroy \n");
    applet_notifications_update_list (self);
}

static void callback_scroll_start (GObject *obj, gpointer user_data)
{
    AppletNotifications *self = APPLET_NOTIFICATIONS (user_data);
    CDOS_POPUP_MENU_BASE(self->menu)->pass_events = TRUE;
}

static void callback_scroll_stop (GObject *obj, gpointer user_data)
{
    AppletNotifications *self = APPLET_NOTIFICATIONS (user_data);
    CDOS_POPUP_MENU_BASE(self->menu)->pass_events = FALSE;
}

/* ========== 回调函数 END ========== */

/* ============ GObject ============ */

/* finalize */
static void cdos_notification_finalize (GObject *gobject)
{
    AppletNotifications *self = APPLET_NOTIFICATIONS(gobject);

    g_object_unref (self->settings);

    g_list_free (self->notifications);
    clutter_actor_destroy ((ClutterActor *)self->p_notification_bin);
    clutter_actor_destroy ((ClutterActor *)self->button_label_box);
    clutter_actor_destroy ((ClutterActor *)self->menu_label);
    clutter_actor_destroy ((ClutterActor *)self->clear_separator);
    clutter_actor_destroy ((ClutterActor *)self->scroll_view);
    clutter_actor_destroy (self->p_crit_icon);
    clutter_actor_destroy (self->p_alt_crit_icon);
    clutter_actor_destroy ((ClutterActor *)self->p_main_container);
    g_free (self->menu_text);
    g_object_unref (self->menu);

    /* 调用 finalize() 方法 */
    G_OBJECT_CLASS(applet_notifications_parent_class)->finalize (gobject);
}

/* _class_init */
static void applet_notifications_class_init (AppletNotificationsClass *klass)
{
    /*继承*/
    GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
    gobject_class->finalize = cdos_notification_finalize;

    CdosAppletClass *parent_class = CDOS_APPLET_CLASS (klass);
    //parent_class->on_applet_clicked = applet_notifications_on_applet_clicked_cb;
}

/* _init */
static void applet_notifications_init (AppletNotifications *self)
{

    /*_init中存放的都是与参数无关的初始化,剩余在_main里面*/
    self->menu_manager = cdos_applet_get_menu_manager((CdosApplet *)self);
    self->notification_count = 0;
    self->notifications = NULL;
    self->p_blinking = FALSE;
    self->p_blink_toggle = FALSE;

    CdosMessageTray *messageTray = cdos_monkey_plugin_get_message_tray (monkey_plugin_get());

    g_signal_connect(messageTray, "notify-applet-update",
                     G_CALLBACK (applet_notifications_notification_added_cb), self);

    g_signal_connect(cdos_global_get_settings (global), "changed::panel-edit-mode",
                     G_CALLBACK (applet_notifications_on_panel_edit_mode_changed_cb), self);
}

/**
 * applet_notifications_main:
 * @metadata: applet的元数据
 * @orientation: 方向
 * @panel_height: 面板高度
 * @instanceId： 字符串类型的id
 *
 * 新建一个 #CdosApplet 类的实例。
 *
 * Returns: 返回一个 #CdosApplet 的实例。
 */
CdosApplet *applet_notifications_main (MetaData *metadata, StSide orientation, gfloat panel_height,
                                       gchar *instanceId)
{
    
    AppletNotifications *self = g_object_new (APPLET_TYPE_NOTIFICATIONS, NULL);

    g_object_set (CDOS_APPLET (self), "panel-height", panel_height, "orientation", orientation, NULL);

    self->menu = cdos_applet_get_popup_menu((CdosApplet *)self);

    self->settings = cdos_applet_settings_new ((GObject *)self, (const gchar *)metadata->uuid, (const gchar *)instanceId);

//    cdos_provider_bind_property (CDOS_PROVIDER (self->settings),
//                                 IN,
//                                 "ignoreTransientNotifications",
//                                 "ignoreTransientNotifications",
//                                 NULL,
//                                 NULL);

    //gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (), metadata->path);


    //ClutterActor *ui_group = monkey_plugin_get_ui_group (monkey_plugin_get ());

    //st_container_add_actor ((ClutterContainer *) ui_group, CLUTTER_ACTOR(self->menu->actor));

    applet_notifications_display(self);

    //g_signal_connect (CDOS_APPLET (self)->actor, "button-press-event", G_CALLBACK (applet_notifications_on_applet_clicked_cb), self);

    cdos_iconapplet_reset_for_systray(CDOS_ICONAPPLET(self));

    return CDOS_APPLET (self);
}

/* 其他方法 */
static void applet_notifications_display (AppletNotifications *self)
{
    PopupBaseMenuItemInit params_for_new_item;;
    cdos_popup_base_menu_item_get_init_params (&params_for_new_item);

    cdos_iconapplet_set_icon_symbolic_name (CDOS_ICONAPPLET (self), "empty-notif");
    cdos_applet_set_applet_tooltip (CDOS_APPLET(self), _("Notifications"));

    self->p_main_container = st_box_layout_new ();
    clutter_actor_set_name( CLUTTER_ACTOR(self->p_main_container), "traycontainer");
    st_box_layout_set_vertical (ST_BOX_LAYOUT (self->p_main_container), TRUE);

    self->p_notification_bin = st_box_layout_new ();
    st_box_layout_set_vertical (ST_BOX_LAYOUT (self->p_notification_bin), TRUE);

    self->button_label_box = ST_BOX_LAYOUT (st_box_layout_new ());

    self->menu_text = applet_notifications_stringify (self->notification_count);
    //self->menu_label = cdos_popup_menu_item_new(self->menu_text, &params_for_new_item);
    self->menu_label = cdos_popup_menu_item_new();
    cdos_popup_menu_item_set_label(self->menu_label,self->menu_text);
    cdos_popup_menu_item_set_label_align(self->menu_label,ST_ALIGN_MIDDLE,0);

    g_object_set (CLUTTER_ACTOR (cdos_popup_base_menu_item_get_actor (CDOS_POPUP_BASE_MENU_ITEM (self->menu_label))),
                  "reactive", FALSE,
                  "can-focus", FALSE,
                  NULL);
    /*第一个参数类型,CdosGenericContainer *actor, 设置属性是在CdosPopupBaseMenuItem内完成的 */   
    st_widget_set_style_class_name (ST_WIDGET(cdos_popup_menu_item_get_label (self->menu_label)),
                                    "popup-subtitle-menu-item");

    self->clear_separator = cdos_popup_separator_menu_item_new ();
    if(self->notification_count > 0){
        cdos_popup_separator_menu_item_show(self->clear_separator);
    }else{
        cdos_popup_separator_menu_item_hide(self->clear_separator);
    }

    self->clear_action = cdos_popup_menu_item_new_with_params (_("Clear notifications"), &params_for_new_item);
    g_signal_connect(CDOS_POPUP_BASE_MENU_ITEM (self->clear_action), "activate", G_CALLBACK (applet_notifications_clear_all_cb), self);
    clutter_actor_hide (CLUTTER_ACTOR(cdos_popup_base_menu_item_get_actor (CDOS_POPUP_BASE_MENU_ITEM (self->clear_action))));

    if (self->p_orientation == ST_SIDE_BOTTOM) {

        /*StSide类型为 StSide, 定义在st-theme-node.h, this.menu的类型是CdosPopupMenu*/
        /*addActor是ClutterActor的方法, addMenuItem是CdosPopupMenuBase方法*/

        cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE(self->menu),
                                            CDOS_POPUP_BASE_MENU_ITEM(self->menu_label), -1);
        cdos_popup_menu_base_add_actor (CDOS_POPUP_MENU_BASE(self->menu), CLUTTER_ACTOR(self->p_main_container));
        cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE(self->menu),
                                            CDOS_POPUP_BASE_MENU_ITEM(self->clear_separator), -1);
        cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE(self->menu),
                                            CDOS_POPUP_BASE_MENU_ITEM(self->clear_action), -1);
        
    } else {

        cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE(self->menu),
                                            CDOS_POPUP_BASE_MENU_ITEM(self->menu_label), -1);
        cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE(self->menu),
                                            CDOS_POPUP_BASE_MENU_ITEM(self->clear_action), -1);
        cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE(self->menu),
                                            CDOS_POPUP_BASE_MENU_ITEM(self->clear_separator), -1);

        cdos_popup_menu_base_add_actor (CDOS_POPUP_MENU_BASE(self->menu), CLUTTER_ACTOR(self->p_main_container));
        
    }

    self->scroll_view = st_scroll_view_new ();
    g_object_set (self->scroll_view, "x-fill", TRUE, "y-fill", TRUE, "y-align", ST_ALIGN_START, NULL);
    g_object_set (self->scroll_view, "style-class", "vfade", NULL);

    st_box_layout_insert_actor(ST_BOX_LAYOUT(self->p_main_container), CLUTTER_ACTOR(self->scroll_view), -1);
    clutter_container_add_actor (CLUTTER_CONTAINER(self->scroll_view), CLUTTER_ACTOR(self->p_notification_bin));
    st_scroll_view_set_policy (ST_SCROLL_VIEW (self->scroll_view), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

    ClutterActor *vscroll = st_scroll_view_get_vscroll_bar (ST_SCROLL_VIEW (self->scroll_view));
    g_signal_connect(vscroll, "scroll-start", G_CALLBACK (callback_scroll_start), self);
    g_signal_connect(vscroll, "scroll-stop", G_CALLBACK (callback_scroll_stop), self);

    self->p_crit_icon = st_icon_new ();
    self->p_alt_crit_icon = st_icon_new ();

    /* TODO:
     * 对于这些属性:
     * reactive->ClutterActor
     * style_class->StWidget
     * track_over->StWidget
     * icon_name->StIcon
     * icon_type->StIcon
     */
    
    g_object_set (self->p_crit_icon,
                  "icon-name", "critical-notif",
                  "icon-type", ST_ICON_SYMBOLIC,
                  "track-hover", TRUE,
                  "style-class", "system-status-icon",
                  "reactive", TRUE,
                  NULL);
    

    g_object_set (self->p_alt_crit_icon,
                  "icon-name", "alt-critical-notif",
                  "icon-type", ST_ICON_SYMBOLIC,
                  "track-hover", TRUE,
                  "style-class", "system-status-icon",
                  "reactive", TRUE,
                  NULL);

    applet_notifications_update_list (self);

}

/**
 * applet_notifications_on_applet_added_to_panel:
 * @self: A #AppletNotifications
 *
 * 添加到panel
 *
 */
void applet_notifications_on_applet_added_to_panel (AppletNotifications *self)
{
    applet_notifications_on_orientation_changed (self, self->p_orientation);
}

/**
 * applet_notifications_on_orientation_changed:
 * @self: A #AppletNotifications
 * @orientation: 方向 
 *
 * 处理通知展示的方向。
 *
 */
void applet_notifications_on_orientation_changed (AppletNotifications *self, StSide orientation)
{
    
    self->p_orientation = orientation;

    if (self->menu) {
        cdos_popup_base_menu_item_destroy (CDOS_POPUP_BASE_MENU_ITEM (self->menu));
    }

    self->menu = cdos_applet_get_popup_menu (CDOS_APPLET(self));
    cdos_popup_menu_manager_add_menu (self->menu_manager, CDOS_POPUP_MENU_BASE (self->menu), -1);
    applet_notifications_display (self);
}

/**
 * applet_notifications_update_list:
 * @self: A #AppletNotifications
 *
 * 更新applet里notifications的排序，包括特定情况下的图标，紧急程度高的上浮等等。
 *
 */
void applet_notifications_update_list (AppletNotifications *self)
{
    
    /*本部分包含try...catch, 只写try部分*/
    self->notification_count = g_list_length (self->notifications);
    

    gint max_urgency = -1;
    if (self->notification_count > 0) {
        clutter_actor_show (CLUTTER_ACTOR(cdos_popup_base_menu_item_get_actor (CDOS_POPUP_BASE_MENU_ITEM (self->clear_action))));
        cdos_popup_separator_menu_item_show(self->clear_separator);
        clutter_actor_show ((ClutterActor *)self->scroll_view);
        cdos_text_icon_applet_set_applet_label (CDOS_TEXT_ICON_APPLET (self), g_strdup_printf ("%d", self->notification_count));
        gint cur_urgency;
        GList *iter = NULL;
        for (iter = self->notifications; iter; iter = iter->next)
        {
            CdosNotification *notification_tmp = iter->data;
            cur_urgency = notification_tmp->urgency;

            if (cur_urgency > max_urgency) {
                max_urgency = cur_urgency;
            }
        }
        
        //测试critical_message时图标状态：max_urgency = 3;
        switch (max_urgency) {
            case LOW:
                self->p_blinking = FALSE;
                cdos_iconapplet_set_icon_symbolic_name (CDOS_ICONAPPLET (self), "low-notif");
                break;

            case NORMAL:
                self->p_blinking = FALSE;
                cdos_iconapplet_set_icon_symbolic_name (CDOS_ICONAPPLET (self), "normal-notif");
                break;

            case CRITICAL:
                if (!self->p_blinking) {
                    self->p_blinking = TRUE;
                    g_timeout_add (1000, (GSourceFunc) applet_notifications_critical_blink, self);
                    //applet_notifications_critical_blink (self);
                }
                break;

            default:
                g_warning ("\nHave not correct max_urgency %s:%d\n", __FUNCTION__, __LINE__);
                break;
        }
    } else {
        self->p_blinking = FALSE;
        
        cdos_text_icon_applet_set_applet_label (CDOS_TEXT_ICON_APPLET(self), "");
        cdos_iconapplet_set_icon_symbolic_name (CDOS_ICONAPPLET (self), "empty-notif");

        /*CdosPopupMenuItem->CdosPopupBaseMenuItem, 获得CdosGenericContainer->ClutterActor*/
        clutter_actor_hide (CLUTTER_ACTOR(cdos_popup_base_menu_item_get_actor (CDOS_POPUP_BASE_MENU_ITEM (self->clear_action))));
        cdos_popup_separator_menu_item_hide(self->clear_separator);
    }
    gchar* strtmp = applet_notifications_stringify (self->notification_count);
    cdos_popup_menu_item_set_label (self->menu_label, strtmp);
    clutter_actor_queue_relayout (CLUTTER_ACTOR (self->p_notification_bin));
    g_free(strtmp);

    gfloat width =0, height = 0;
    clutter_actor_get_size (CLUTTER_ACTOR(self->p_notification_bin), &width, &height);
    guint count = self->notification_count;

    if (height > 300 || count >= 5) {
        clutter_actor_set_size (CLUTTER_ACTOR (self->p_main_container), -1, 300);
    } else {
        clutter_actor_set_size (CLUTTER_ACTOR (self->p_main_container), -1, -1);
    }

    if (count == 0) {
        clutter_actor_set_size (CLUTTER_ACTOR(self->p_main_container), 0, 0);
    }
}

/**
 * applet_notifications_critical_blink:
 * @self: A #AppletNotifications
 *
 * 当出现紧急程度为urgent的通知时，为了产生闪烁过程而加入的g_timeout_add
 *
 * Returns: 当为FALSE的跳出循环
 */
gboolean applet_notifications_critical_blink (AppletNotifications *self)
{
    
    if (!self->p_blinking) {
        return FALSE;
    }

    StBin *critical_icon = cdos_iconapplet_get_applet_icon_box (CDOS_ICONAPPLET (self));

    if (self->p_blink_toggle) {
        /*_applet_icon_box是IconApplet的成员变量*/
        //FIXME
        st_bin_set_child (critical_icon, CLUTTER_ACTOR (self->p_crit_icon));
        g_object_ref(self->p_crit_icon);
    } else {
        st_bin_set_child (critical_icon, CLUTTER_ACTOR (self->p_alt_crit_icon));
        g_object_ref(self->p_alt_crit_icon);
    }
    self->p_blink_toggle = !(self->p_blink_toggle);
    //g_timeout_add (1000, (GSourceFunc) applet_notifications_critical_blink, self);

    
    return TRUE;
}

/**
 * applet_notifications_on_applet_clicked_cb:
 * @actor: A #ClutterActor
 * @event: 事件类型
 * @user_data: 回调函数传入参数
 * 
 * 这个函数是对父类on_applet_clicked方法地重写，监听applet的“button-press-event” 信号回调；弹出，收起applet
 */
void applet_notifications_on_applet_clicked_cb (ClutterActor* actor, ClutterEvent* event, gpointer user_data)
{
    if(clutter_event_get_button(event) != 1) {
        return ;
    }
    
    AppletNotifications *self = APPLET_NOTIFICATIONS (user_data);
    gfloat width, height;
    guint count;

    applet_notifications_update_timestamp (self);
    cdos_popup_menu_base_toggle (CDOS_POPUP_MENU_BASE(self->menu));

    clutter_actor_get_size (CLUTTER_ACTOR(self->p_main_container), &width, &height);
    count = self->notification_count;
    if (height > 300 || count >= 5) {
        clutter_actor_set_size (CLUTTER_ACTOR (self->p_main_container), -1, 300);
    } else {
        clutter_actor_set_size (CLUTTER_ACTOR (self->p_main_container), -1, -1);
    }

    if (count == 0) {
        clutter_actor_set_size (CLUTTER_ACTOR(self->p_main_container), 0, 0);
        //hide clear_separator
        clutter_actor_hide (CLUTTER_ACTOR(cdos_popup_base_menu_item_get_actor (CDOS_POPUP_BASE_MENU_ITEM (self->clear_separator))));
        //set menu_label in middle
    }
}

static void applet_notifications_update_timestamp (AppletNotifications *self)
{
    
    const gchar *dateFormat = "%p %l:%M";
    //GList *actors;
    //actors = clutter_actor_get_children (CLUTTER_ACTOR(self->p_notification_bin));
    
    if (self->notifications) {
        GList *iter = NULL;
        for (iter = self->notifications; iter; iter = iter->next)
        {
            //ClutterActor *actor = g_list_nth_data (actors, i);
            //FIXME: 此处总是返回NULL，已经在notification的post_init中初始化delegate，不知为何
            /*
            CdosNotification *notification = cdos_draggable_get_delegate (actor);
            if (!notification) {
                continue;
            }
            */
            CdosNotification *notification = iter->data;
            GDateTime* orig_time = notification->timestamp;
            ClutterActor *clutter_text;
            clutter_text = st_label_get_clutter_text (notification->timeLabel);
            gchar *date = applet_notifications_timeify (orig_time, dateFormat);
            clutter_text_set_markup ((ClutterText *)clutter_text, date);
            g_free (date);
        }
    }
}

