#include "cdos-start.h"
#include "cdos-global.h"
#include "cdos-util.h"
#include "applet-drive-menu-item.h"

/**
 * SECTION: applet-drive-menu-item.c
 * @Title: CdosDriveMenuItem
 * @short_description: description
 * 
 * CdosDriveMenuItem类是可移动磁盘里的一个项,当我们为某个可移动磁盘创建item时，可以使用这个类
 */

struct _CdosDriveMenuItemPrivate
{
    ClutterActor *label;
    GMount *mount;
};


G_DEFINE_TYPE(CdosDriveMenuItem, cdos_drive_menu_item, \
        CDOS_TYPE_POPUP_BASE_MENU_ITEM);

#define CDOS_DRIVE_MENU_ITEM_GET_PRIVATE(obj)    \
        (G_TYPE_INSTANCE_GET_PRIVATE((obj), CDOS_TYPE_DRIVE_MENU_ITEM, CdosDriveMenuItemPrivate))

/*
static void
cdos_drive_menu_item_eject(StButton *button,  gint click, CdosDriveMenuItem *item);
*/

static void
cdos_drive_menu_item_eject(CdosDriveMenuItem *self);

static CdosDriveMenuItem *
cdos_drive_menu_item_set_params(CdosDriveMenuItem   *item, 
                                GMount *mount);

static void
cdos_drive_menu_item_init(CdosDriveMenuItem *item);

static void 
cdos_drive_menu_item_class_init(CdosDriveMenuItemClass *klass);

static void 
cdos_drive_menu_item_class_init(CdosDriveMenuItemClass *klass)
{
        //GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
    g_type_class_add_private(klass, sizeof(CdosDriveMenuItemPrivate));
}

static void
cdos_drive_menu_item_init(CdosDriveMenuItem *item)
{
        CdosDriveMenuItemPrivate *priv;
        priv = item->priv = CDOS_DRIVE_MENU_ITEM_GET_PRIVATE(item);
        priv->label = NULL;
}

/**
 * cdos_drive_menu_item_new:
 * @place: param1_description 
 * 
 * 新建一个 #CdosDriveMenuItem 类的实例。
 *
 * Returns: 返回一个 #CdosDriveMenuItem 的实例。
 */
CdosDriveMenuItem *
cdos_drive_menu_item_new(GMount *mount)
{
        CdosDriveMenuItem *item;
        item = CDOS_DRIVE_MENU_ITEM(g_object_new(CDOS_TYPE_DRIVE_MENU_ITEM, NULL));
        item = cdos_drive_menu_item_set_params(item, mount);
        return item;
}

static void
on_button_clicked (StButton *button, 
                   gint click, 
                   CdosDriveMenuItem *self)
{
    cdos_drive_menu_item_eject (self);
}

static void
on_activate (CdosDriveMenuItem *self, 
             gpointer event, 
             gboolean keep, 
             gpointer data)
{
    guint event_type = clutter_event_get_button((ClutterEvent *)event);

    if(event_type == 1)
    {
        cdos_drive_menu_item_eject (self);
    }
}

static CdosDriveMenuItem *
cdos_drive_menu_item_set_params(CdosDriveMenuItem   *item, 
                                GMount  *mount)
{
        CdosDriveMenuItemPrivate *priv;
        priv = item->priv;

        PopupBaseMenuItemInit init;
        cdos_popup_base_menu_item_get_init_params(&init);
        cdos_popup_base_menu_item_set_params((CdosPopupBaseMenuItem *)item, &init);

        priv->mount = mount;
        char *name = g_mount_get_name (mount);
        name = g_strconcat (_("Eject"), " ", name, NULL);
        priv->label = (ClutterActor *)st_label_new(name);
        st_widget_add_style_class_name (ST_WIDGET (priv->label), "drive-menu-item-label");
        g_free (name);

        ActorChild child;
        cdos_popup_base_menu_item_get_actor_child(&child);

        StWidget *ejectIcon = g_object_new(ST_TYPE_ICON, 
                                        "icon-name", "media-eject",
                                        "icon-type", ST_ICON_SYMBOLIC,
                                        "style-class", "drive-menu-item-eject-icon",
                                        NULL);

        StWidget *ejectButton = st_button_new();
        
        st_bin_set_child ((StBin *)ejectButton, (ClutterActor *)ejectIcon);

        /*
        g_signal_connect(ejectButton, "clicked",
                        G_CALLBACK(cdos_drive_menu_item_eject), item);
        */
        
        //可移动磁盘优化：点击菜单项直接弹出或卸载，取消点击卷标名称打开功能，仍保留打开功能代码
        g_signal_connect (ejectButton, "clicked",
                          G_CALLBACK(on_button_clicked), item);

        child.align = ST_ALIGN_START;

        cdos_popup_base_menu_item_add_actor(CDOS_POPUP_BASE_MENU_ITEM(item), (ClutterActor *)ejectButton, &child);
        cdos_popup_base_menu_item_add_actor(CDOS_POPUP_BASE_MENU_ITEM(item), priv->label, &child);

        //g_signal_connect(item, "activate", G_CALLBACK(cdos_drive_menu_item_activate), item);
        
        g_signal_connect (item, "activate", G_CALLBACK(on_activate), NULL);

        return item;
}

static void
unmount_finish(GMount *mount,
               GAsyncResult *res,
               CdosDriveMenuItem *self)
{
    GError *error = NULL;
    gboolean status = g_mount_unmount_with_operation_finish(mount, res, &error);
    if (!status)
    {
        if (error)
        {
            char *name = g_mount_get_name (mount);

            // char *msg = g_strconcat ("unmount error: ", name, NULL);
            gchar *msg = g_strdup_printf (_("unmount error: %s"), name);
            gchar *err_msg = strstr (error->message, "exited with non-zero exit status 32: umount: ");
            if (err_msg) {
                glong err_msg_len = g_utf8_strlen(err_msg, -1);
                // length 45 is 'exited with non-zero exit status 32: umount: '
                error->message = g_strdup_printf (_("try unmount %s"), g_utf8_substring(err_msg, 45, err_msg_len));
            } else {
                error->message = g_strdup_printf (_("try unmount %s failed."), name);
            }

            g_free (name);
            cdos_notify_error (_(msg), error->message);
            g_free (msg);
            g_error_free (error);
        }
    }
}

static void
eject_finish(GMount *mount,
             GAsyncResult *res,
             CdosDriveMenuItem *self)
{
    GError *error = NULL;
    gboolean status = g_mount_eject_with_operation_finish(mount, res, &error);
    if (!status)
    {
        if (error)
        {
            char *name = g_mount_get_name (mount);

            // char *msg = g_strconcat ("unmount error: ", name, NULL);
            gchar *msg = g_strdup_printf (_("unmount error: %s"), name);
            gchar *err_msg = strstr (error->message, "exited with non-zero exit status 32: umount: ");
            if (err_msg) {
                glong err_msg_len = g_utf8_strlen(err_msg, -1);
                // length 45 is 'exited with non-zero exit status 32: umount: '
                error->message = g_strdup_printf (_("try unmount %s"), g_utf8_substring(err_msg, 45, err_msg_len));
            } else {
                error->message = g_strdup_printf (_("try unmount %s failed."), name);
            }

            g_free (name);
            cdos_notify_error (_(msg), error->message);
            g_free (msg);
            g_error_free (error);
        }
    }
}

/*
static void
cdos_drive_menu_item_eject(StButton *button, gint click, CdosDriveMenuItem *self)
{
    CdosDriveMenuItemPrivate *priv = self->priv;
    if (g_mount_can_eject(priv->mount)) 
        g_mount_eject_with_operation (priv->mount, 
                                      G_MOUNT_UNMOUNT_NONE,
                                      NULL, 
                                      NULL,
                                      (GAsyncReadyCallback)eject_finish, self);
    else 
        g_mount_unmount_with_operation (priv->mount,
                                        G_MOUNT_UNMOUNT_NONE,
                                        NULL, 
                                        NULL,
                                        (GAsyncReadyCallback )unmount_finish, self);
}
*/

static void
cdos_drive_menu_item_eject (CdosDriveMenuItem *self)
{
    CdosDriveMenuItemPrivate *priv = self->priv;
    if (g_mount_can_eject(priv->mount)) 
        g_mount_eject_with_operation (priv->mount, 
                                      G_MOUNT_UNMOUNT_NONE,
                                      NULL, 
                                      NULL,
                                      (GAsyncReadyCallback)eject_finish, self);
    else 
        g_mount_unmount_with_operation (priv->mount,
                                        G_MOUNT_UNMOUNT_NONE,
                                        NULL, 
                                        NULL,
                                        (GAsyncReadyCallback )unmount_finish, self);
}

/**
 * cdos_drive_menu_item_activate: 
 * @item: 绑定信号的对象
 * @event: clutterEvent
 * @keep:  gboolean
 * @data: 真对回调函数所传入的参数
 *
 * 当捕获到"activate"信号时,触发该函数
 */
void
cdos_drive_menu_item_activate(CdosDriveMenuItem *item, gpointer event, gboolean keep, gpointer data)
{

        CdosDriveMenuItemPrivate *priv;
        priv = item->priv;

        guint event_type = clutter_event_get_button((ClutterEvent *)event);

        if(event_type == 1){
            CdosGlobal *global = cdos_global_get ();
            GAppLaunchContext *context = cdos_global_create_app_launch_context (global);
            GFile *root = g_mount_get_root (priv->mount);
            gchar *uri = g_file_get_uri (root);
            g_object_unref (root);
            GError *error = NULL;
            gboolean ret = g_app_info_launch_default_for_uri (uri, context, &error);
            g_free (uri);
            if (!ret)
            {
                if (error)
                    g_warning ("%s", error->message);
            }
        }
}

GMount *
cdos_drive_menu_item_get_mount (CdosDriveMenuItem *self)
{
    return self->priv->mount;
}
