#include "applet-removable-drives.h"
#include "cdos-popup-menu-manager.h"
#include "cdos-popup-menu-base.h"
#include "applet-drive-menu-item.h"
#include "cdos-popup-menu-section.h"
#include "cdos-app-system.h"
#include "cdos-popup-separator-menu-item.h"

struct _AppletRemovableDrivesPrivate
{
    CdosPopupMenu   *menu;
    StBoxLayout     *contentSection;
    GVolumeMonitor  *monitor;
    gulong           add_id;
    gulong           remove_id;
    GList           *items;
};

G_DEFINE_TYPE(AppletRemovableDrives, applet_removable_drives, CDOS_TYPE_ICONAPPLET);

#define APPLET_REMOVABLE_DRIVES_GET_PRIVATE(obj)    \
        (G_TYPE_INSTANCE_GET_PRIVATE((obj), \
        APPLET_TYPE_REMOVABLE_DRIVES, AppletRemovableDrivesPrivate))

/*
static void
applet_removable_drives_add_item(AppletRemovableDrives *applet);
*/

static void
applet_removable_drives_update_visibility (AppletRemovableDrives *self)
{
    if (self->priv->items)
    {
        clutter_actor_show ((ClutterActor *)self->priv->contentSection);
        clutter_actor_show ((ClutterActor *)CDOS_APPLET(self)->actor);
    }
    else
    {
        cdos_popup_menu_close ((CdosPopupMenuBase *)self->priv->menu, 1);
        clutter_actor_hide ((ClutterActor *)CDOS_APPLET(self)->actor);
    }
}

static gboolean
check_item_exists (AppletRemovableDrives *self, GMount *mount)
{
    AppletRemovableDrivesPrivate *priv = self->priv;
    GMount *tmp_mount = NULL;
    CdosDriveMenuItem *item = NULL;
    GList *l = priv->items;
    while (l)
    {
        item = l->data;
        tmp_mount = cdos_drive_menu_item_get_mount (item);
        if (g_strcmp0 (g_mount_get_name (tmp_mount), g_mount_get_name (mount)) == 0)
        {
            return TRUE;
        }
        l = l->next;
    }
    return FALSE;
}

static void
add_mount (GVolumeMonitor *monitor,
           GMount   *mount,
           AppletRemovableDrives *self)
{
    if (check_item_exists (self, mount)) {
        g_debug ("\nAppletRemovableDrives: Menu item exists, just return.\n");
        return;
    }

    CdosDriveMenuItem *item = cdos_drive_menu_item_new (mount);
    AppletRemovableDrivesPrivate *priv = self->priv;
    priv->items = g_list_prepend (priv->items, item);
    st_box_layout_insert_actor (priv->contentSection,
        (ClutterActor *)cdos_popup_base_menu_item_get_actor ((CdosPopupBaseMenuItem *)item),
        0);
    applet_removable_drives_update_visibility (self);
}

static void
remove_mount (GVolumeMonitor *monitor,
              GMount   *mount,
              AppletRemovableDrives *self)
{
    AppletRemovableDrivesPrivate *priv = self->priv;
    for (GList *iter = priv->items; iter; iter = iter->next)
    {
        CdosDriveMenuItem *item = iter->data;
        if (cdos_drive_menu_item_get_mount (item) == mount)
        {
            priv->items = g_list_remove (priv->items, item);
            clutter_actor_destroy ((ClutterActor *)cdos_popup_base_menu_item_get_actor((CdosPopupBaseMenuItem *)item));
            //g_object_unref (item);
            applet_removable_drives_update_visibility (self);
            return;
        }
    }
}

//可移动磁盘优化：去除打开文件管理器项，保留代码
/*
static void
applet_removable_drives_fun(ClutterEvent *event)
{
    CdosAppSystem *appSystem = cdos_app_system_get_default ();
    CdosApp *app = cdos_app_system_lookup_app(appSystem , "nemo.desktop");
    cdos_app_activate_full(app, -1, clutter_event_get_time(event));
}

static void
applet_removable_drives_add_item(AppletRemovableDrives *applet)
{
        AppletRemovableDrivesPrivate *priv = applet->priv;

        CdosPopupSeparatorMenuItem  *sep = cdos_popup_separator_menu_item_new();

        clutter_container_add_actor((ClutterContainer *)priv->contentSection,
                (ClutterActor *)cdos_popup_base_menu_item_get_actor((CdosPopupBaseMenuItem *)sep));

        CdosPopupMenuItem *item = cdos_popup_menu_base_add_action((CdosPopupMenuBase *)priv->menu,
                _("Open File Manager"),
                G_CALLBACK(applet_removable_drives_fun), NULL);
        cdos_popup_menu_item_set_label_align(item,ST_ALIGN_MIDDLE,0);
        StLabel *label = cdos_popup_menu_item_get_label(item);
        st_widget_add_style_class_name(ST_WIDGET(label),"popup-menu-item-label-link");
}
*/

static void
applet_removable_drives_init(AppletRemovableDrives *applet)
{
    AppletRemovableDrivesPrivate *priv;
    priv = applet->priv = APPLET_REMOVABLE_DRIVES_GET_PRIVATE(applet);

    priv->items = NULL;

    cdos_iconapplet_set_icon_symbolic_name((CdosIconApplet *)applet, "drive-harddisk");
    cdos_applet_set_applet_tooltip((CdosApplet *)applet, _("Removable drives"));

    priv->menu = cdos_applet_get_popup_menu(CDOS_APPLET(applet));

    //ClutterActor *uigroup = monkey_plugin_get_ui_group (monkey_plugin_get ());
    priv->contentSection = cdos_popup_menu_base_get_box((CdosPopupMenuBase *)priv->menu);

    priv->monitor = g_volume_monitor_get ();
    priv->add_id = g_signal_connect (priv->monitor, "mount-added", G_CALLBACK(add_mount), applet);
    priv->remove_id = g_signal_connect (priv->monitor, "mount-removed", G_CALLBACK(remove_mount), applet);
    GList *mounts = g_volume_monitor_get_mounts (priv->monitor);
    for (GList *iter = mounts; iter; iter = iter->next)
    {
        GMount *mount = iter->data;
        if (!g_mount_can_eject (mount) && !g_mount_can_unmount (mount))
            continue;
        if (g_mount_is_shadowed (mount))
            continue;
        GVolume *volume = g_mount_get_volume (mount);
        gboolean skip = 0;
        if (volume)
        {
            gchar *identifier = g_volume_get_identifier (volume, "class");
            if (identifier != NULL)
            {
                if (!g_strcmp0 (identifier, "network"))
                    skip = 1;
                g_free (identifier);
            }
        }
        if (!skip)
        {
            add_mount (priv->monitor, mount, applet);
        }
        g_object_unref (volume);
    }
    g_list_free (mounts);

    /*
    applet_removable_drives_add_item (applet);
    applet_removable_drives_update_visibility (applet);
    */
    applet_removable_drives_update_visibility (applet);
}

static void
applet_removable_drives_finalize(GObject *gobject)
{
        AppletRemovableDrivesPrivate *priv = APPLET_REMOVABLE_DRIVES(gobject)->priv;

        if (priv->menu) {
            g_object_unref(priv->menu);
        }

        if (priv->contentSection) {
            g_object_unref(priv->contentSection);
        }
        g_signal_handler_disconnect (priv->monitor, priv->add_id);
        g_signal_handler_disconnect (priv->monitor, priv->remove_id);

        g_object_unref (priv->monitor);

        G_OBJECT_CLASS(applet_removable_drives_parent_class)->finalize(gobject);
}

static void
applet_removable_drives_class_init(AppletRemovableDrivesClass *klass)
{
        GObjectClass *gobject = G_OBJECT_CLASS(klass);
        gobject->finalize = applet_removable_drives_finalize;
        g_type_class_add_private(klass, sizeof(AppletRemovableDrivesPrivate));
}

static AppletRemovableDrives *
applet_removable_drives_new(guint orientation, gfloat panel_height)
{
        AppletRemovableDrives *applet;
        applet = g_object_new(APPLET_TYPE_REMOVABLE_DRIVES,
                              "orientation", orientation,
                              "panel-height", panel_height,
                              NULL);
        cdos_iconapplet_reset_for_systray(CDOS_ICONAPPLET(applet));
        return applet;
}

CdosApplet *
applet_removable_drives_main(MetaData *meta, guint orientation, guint panel_height)
{
        AppletRemovableDrives *applet;
        applet = applet_removable_drives_new(orientation, panel_height);
        return CDOS_APPLET (applet);
}
