#include <gio/gio.h>

#include "applets-scale.h"

G_DEFINE_TYPE(CdosScale, cdos_scale, CDOS_TYPE_ICONAPPLET);

#define CDOS_SCALE_GET_PRIVATE(obj) \
        (G_TYPE_INSTANCE_GET_PRIVATE((obj), CDOS_TYPE_SCALE, CdosScalePrivate))

struct _CdosScalePrivate
{
     GSettings     *settings;
     //StBin           *applet_icon_box;
     StBoxLayout     *actor;
     gboolean        hover_activates;
};

static gboolean
on_applet_clicked(ClutterActor *actor,ClutterEvent *event,CdosScale *self);


static void
cdos_scale_finalize(GObject *object)
{
     CdosScalePrivate *priv = CDOS_SCALE(object)->priv;
    
/*调用父类的finalize()方法*/
     G_OBJECT_CLASS(cdos_scale_parent_class)->finalize(object);
}

static void
cdos_set_hover_changed(CdosScale *self,gboolean active)
{
  CdosScalePrivate *priv;
  priv = self->priv = CDOS_SCALE_GET_PRIVATE(self);

  gboolean  activeChanged;
  activeChanged = (active !=priv->hover_activates);
  if(activeChanged){
      priv->hover_activates = active;
      if(priv->hover_activates){
         cdos_iconapplet_set_icon_name((CdosIconApplet *)self,"scale-hover");
         //clutter_actor_grab_key_focus ((ClutterActor *)priv->actor);
      }else{
          cdos_iconapplet_set_icon_name((CdosIconApplet *)self,"scale");
      }
    }
}

static void 
cdos_on_key_focus_in(ClutterActor *actor,CdosScale *self)
{
   CdosScalePrivate *priv;
   priv = self->priv = CDOS_SCALE_GET_PRIVATE(self);

   cdos_set_hover_changed(self,TRUE);
}

static void
cdos_on_key_focus_out(ClutterActor *actor,CdosScale *self)
{
  CdosScalePrivate *priv;
  priv = self->priv = CDOS_SCALE_GET_PRIVATE(self);
 
  cdos_set_hover_changed(self,FALSE);
}

static void
cdos_on_hover_changed(ClutterActor *actor,gpointer pspec, CdosScale *self)
{
   CdosScalePrivate *priv;
   priv = self->priv = CDOS_SCALE_GET_PRIVATE(self);

   cdos_set_hover_changed(self,st_widget_get_hover ((StWidget *)actor));
}

static void
cdos_do_action(CdosScale *self)
{
    CdosOverview *overview = cdos_overview_get ();
    if(!overview->animation_in_progress)
        cdos_overview_toggle(overview);
}

static gboolean
on_applet_clicked(ClutterActor *actor,ClutterEvent *event,CdosScale *self)
{
    gint32 button = clutter_event_get_button(event);
    if(button == 1){
        CdosOverview *overview = cdos_overview_get ();
        cdos_overview_toggle (overview);
    }
    return TRUE;
}

void 
on_entered(CdosScale *self,ClutterEvent *event)
{
  CdosScalePrivate *priv;
  priv = self->priv = CDOS_SCALE_GET_PRIVATE(self);
  CdosGlobal  *global;
  global = cdos_global_get();
  priv->settings = cdos_global_get_settings(global);
  if(!priv->hover_activates || g_settings_get_boolean(priv->settings,"panel-edit-mode"));
      return ;
  cdos_do_action(self);
}


static void
cdos_scale_function()
{ }

CdosScale *
cdos_scale_main(MetaData *meta,guint orientation, gfloat panel_height, guint instance_id)
{
    CdosScale *myapplet;
    myapplet =g_object_new(CDOS_TYPE_SCALE,
                           "metadata",meta,
                           "orientation",orientation,
                           "panel_height",panel_height,
                           "instance_id",instance_id,
                           NULL);
    return myapplet;
}

static void 
cdos_scale_class_init( CdosScaleClass *kclass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (kclass);
   //lutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (kclass);
    CdosAppletClass *parent_class = CDOS_APPLET_CLASS (kclass);
    object_class->finalize = cdos_scale_finalize;
  
    //parent_class->on_applet_clicked = on_applet_clicked;
    g_type_class_add_private (kclass, sizeof (CdosScalePrivate));

}

static void
cdos_scale_init(CdosScale *self)
{
    CdosScalePrivate *priv;
    priv = self->priv = CDOS_SCALE_GET_PRIVATE(self);
    cdos_applet_set_popup_menu_active(CDOS_APPLET(self),FALSE);
}


CdosScale *
cdos_scale_new(MetaData *metadata,guint orientation, gfloat panel_height, gchar *instance_id)
{
    CdosScale *scale;
    CdosIconApplet *iconapplet;
    CdosScalePrivate *priv = NULL;
    scale = g_object_new(CDOS_TYPE_SCALE, NULL);  //创建我们自定义的类对象
    priv = CDOS_SCALE_GET_PRIVATE(scale);
    //fixme path= g_file_get_path(metadata);
    gtk_icon_theme_append_search_path (gtk_icon_theme_get_default(),metadata->path);

    iconapplet = (CdosIconApplet *)scale;
    priv->actor = iconapplet->actor;
    //st_widget_remove_style_class_name ((StWidget *)priv->actor,"applet-box");
   // priv->applet_icon_box = g_object_new (ST_TYPE_BIN, NULL);
   // iconapplet->applet_icon_box = cdos_iconapplet_get_applet_icon_box(iconapplet);
    //clutter_actor_set_margin_left((ClutterActor *)iconapplet->applet_icon_box, 8.0);
   
    cdos_iconapplet_set_icon_name((CdosIconApplet *)iconapplet, "scale");
    cdos_applet_set_applet_tooltip((CdosApplet *)iconapplet, _("Scale"));
    priv->hover_activates =FALSE;
    //找杨龙龙要appletsettings接口
    priv->settings = cdos_applet_settings_new((GObject *)scale,metadata->uuid,instance_id);
   
    cdos_provider_bind_property((CdosProvider *)priv->settings,IN,"activate-on-hover","hover_activates",cdos_scale_function,NULL);    
    
    g_signal_connect (CDOS_APPLET(scale)->actor, "button-press-event",G_CALLBACK(on_applet_clicked),scale);

    st_widget_add_style_class_name(ST_WIDGET(CDOS_APPLET(scale)->actor),"applet-box-scale");

    g_signal_connect(priv->actor, "key-focus-in", G_CALLBACK(cdos_on_key_focus_in), scale);
    g_signal_connect(priv->actor, "key-focus-out", G_CALLBACK(cdos_on_key_focus_out), scale);
    st_widget_set_track_hover ((StWidget *)priv->actor, 1);
    g_signal_connect((StWidget *)priv->actor, "notify::hover", G_CALLBACK(cdos_on_hover_changed), scale);

    return scale;
}   








