//
// Created by cdos on 16-5-13.
//
/**
 * SECTION: AppletPopupSliderVMenuItem
 * @short_description: 声音应用的滑块部分
 * @include: applet-popup-slider-v-menu-item.h
 *
 * AppletPopupSliderVMenuItemClass 类用来构建声音applet的滑块，
 * 包括响应点击，拖动，滚轮事件。
 */
#include "applet-popup-slider-v-menu-item.h"
#include <math.h>
#include <clutter/clutter.h>
#include "ui/cdos-monkey-plugin.h"
#include "string.h"
#include "applet-text-image-menu-item.h"

/* ========== BASE ========== */
/* 注册函数 */
G_DEFINE_TYPE(AppletPopupSliderVMenuItem, applet_popup_slider_v_menu_item, CDOS_TYPE_POPUP_BASE_MENU_ITEM);

/* 全局变量 */
static gfloat PX = 0;
static gboolean is_scroll_end = FALSE;

/* CALL_BACK */
static void callback_update_slider_bar (GObject *obj, gfloat value, gpointer user_data);

static gboolean callback_start_dragging (GObject *obj, ClutterEvent *event, gpointer user_data);

static void callback_on_scroll_event (GObject *obj, GdkEvent *event, gpointer user_data);

static gboolean callback_end_dragging (GObject *obj, ClutterEvent *event, gpointer user_data);

static gboolean callback_motion_event (GObject *obj, ClutterEvent *event, gpointer user_data);

static gboolean
callback_on_key_press_event (ClutterActor *actor, ClutterEvent *event, gpointer user_data);

static void applet_popup_slider_v_menu_item_move_handle (AppletPopupSliderVMenuItem *self, float abs_x, float abs_y);

static void callback_update_slider_bar (GObject *obj, gfloat value, gpointer user_data)
{
    AppletPopupSliderVMenuItem *self = APPLET_POPUP_SLIDER_V_MENU_ITEM (user_data);
    gfloat width, height, bar_width, bar_height;
    clutter_actor_get_size (CLUTTER_ACTOR(self->p_container), &width, &height);
    clutter_actor_get_size (CLUTTER_ACTOR(self->p_slider), &bar_width, &bar_height);
    gfloat pv = self->p_value;
    if(PX == 0){
        gfloat c_min_witdth,c_natural_width,s_min_witdth,s_natural_width;
        clutter_actor_get_preferred_width(CLUTTER_ACTOR (self->p_container),-1,&c_min_witdth,&c_natural_width);
        clutter_actor_get_preferred_width(CLUTTER_ACTOR (self->p_slider),-1,&s_min_witdth,&s_natural_width);
        PX = c_natural_width - s_natural_width;
    }

    gint top = (gint) ((height - self->p_slider_height) / 2);

    gint tmp_height = top + (gint) (self->p_slider_height * (1 - pv));

    gfloat px = (width/2) - (bar_width/2);
    gfloat py = tmp_height - bar_height / 2;

    clutter_actor_set_position (CLUTTER_ACTOR(self->p_slider), PX / 2, py);
    clutter_actor_set_size (CLUTTER_ACTOR(self->value_flag), width, tmp_height);
}

static gboolean callback_start_dragging (GObject *obj, ClutterEvent *event, gpointer user_data)
{
    AppletPopupSliderVMenuItem *self = APPLET_POPUP_SLIDER_V_MENU_ITEM(user_data);
    if (self->p_dragging) {
        // don't allow two drags at the same time
        return;
    }
    self->p_dragging = TRUE;
    // FIXME:
    // we should only grab the specific device that originated
    // the event, but for some weird reason events are still delivered
    // outside the slider if using clutter_grab_pointer_for_device

    clutter_grab_pointer (CLUTTER_ACTOR(self->p_slider));

    if(self->p_release_id == 0){

    }
    //FIXME:参数可能要调整
    self->p_release_id = g_signal_connect(self->p_slider,
                                          "button-release-event",
                                          G_CALLBACK (callback_end_dragging),
                                          self);
    if(self->p_montion_id == 0){

    }
    self->p_montion_id = g_signal_connect(self->p_slider,
                                           "motion-event",
                                           G_CALLBACK (callback_motion_event),
                                           self);
    gfloat abs_x, abs_y;
    //TODO:获取坐标系
    clutter_event_get_coords (event, &abs_x, &abs_y);

    applet_popup_slider_v_menu_item_move_handle (self, abs_x, abs_y);
    return FALSE;

}

static gboolean callback_end_dragging (GObject *obj, ClutterEvent *evnet, gpointer user_data)
{
    AppletPopupSliderVMenuItem *self = APPLET_POPUP_SLIDER_V_MENU_ITEM (user_data);
    if (self->p_dragging) {
        g_signal_handler_disconnect (self->p_slider, self->p_release_id);
        g_signal_handler_disconnect (self->p_slider, self->p_montion_id);
        clutter_ungrab_pointer ();
        self->p_dragging = FALSE;

        // g_signal_emit_by_name (self, "drag-end");
    }
    cdos_sound_manager_play ("volume");
    return FALSE;
}

static void callback_on_scroll_event (GObject *obj, GdkEvent *event, gpointer user_data)
{
    AppletPopupSliderVMenuItem *self = APPLET_POPUP_SLIDER_V_MENU_ITEM(user_data);

    ClutterScrollDirection direction = clutter_event_get_scroll_direction ((const ClutterEvent *) event);

    if (direction == CLUTTER_SCROLL_DOWN) {
        self->p_value = (gfloat)(MAX(0, self->p_value - SLIDER_SCROLL_STEP));
    } else if (direction == CLUTTER_SCROLL_UP) {
        self->p_value = (gfloat)(MIN(1, self->p_value + SLIDER_SCROLL_STEP));
    }

    //this._slider.queue_repaint();原代码已被注销
    //FIXME
    g_signal_emit_by_name (self, "value-changed", self->p_value);
    // g_signal_emit_by_name (self, "drag-end");

    // 造成重复播放原因是 clutter 的 "scroll-event" 会调用两次回调，设置标志位，单次播放
    if (!is_scroll_end) {
        cdos_sound_manager_play ("volume");
    }

    is_scroll_end = !is_scroll_end;
}

static gboolean callback_motion_event (GObject *obj, ClutterEvent *event, gpointer user_data)
{
    AppletPopupSliderVMenuItem *self = APPLET_POPUP_SLIDER_V_MENU_ITEM(user_data);

    gfloat abs_x, abs_y;
    clutter_event_get_coords ((const ClutterEvent *) event, &abs_x, &abs_y);
    applet_popup_slider_v_menu_item_move_handle (self, abs_x, abs_y);
    return TRUE;
}

static gboolean
callback_on_key_press_event (ClutterActor *actor, ClutterEvent *event, gpointer user_data)
{
    AppletPopupSliderVMenuItem *self = APPLET_POPUP_SLIDER_V_MENU_ITEM (user_data);
    gdouble delta;
    guint key = clutter_event_get_key_symbol ((const ClutterEvent *) event);

    if (key == CLUTTER_KEY_Up || key == CLUTTER_KEY_Down) {
        if (key == CLUTTER_KEY_Up) {
            delta = 0.1;
        } else {
            delta = -0.1;
        }

        self->p_value = (gfloat)(MAX(0, MIN (self->p_value + delta, 1)));
        g_signal_emit_by_name (self, "value-changed", self->p_value);
        // g_signal_emit_by_name (self, "drag-end");
        cdos_sound_manager_play ("volume");
        return TRUE;
    }

    return FALSE;
}

/* finalize */
static void applet_popup_slider_v_menu_item_finalize (GObject *obj)
{
    AppletPopupSliderVMenuItem *self = APPLET_POPUP_SLIDER_V_MENU_ITEM (obj);

    G_OBJECT_CLASS (applet_popup_slider_v_menu_item_parent_class)->finalize (obj);
}

static void applet_popup_slider_v_menu_item_dispose (GObject *obj)
{
    AppletPopupSliderVMenuItem *self = APPLET_POPUP_SLIDER_V_MENU_ITEM (obj);
    if(self->p_release_id){
        g_signal_handler_disconnect (self->p_slider, self->p_release_id);
    }
    if(self->p_montion_id){
        g_signal_handler_disconnect (self->p_slider, self->p_montion_id);
    }
    g_clear_object(&self->p_box);
    g_clear_object(&self->p_slider);

    G_OBJECT_CLASS (applet_popup_slider_v_menu_item_parent_class)->dispose (obj);
}

/* _class_init */
static void applet_popup_slider_v_menu_item_class_init (AppletPopupSliderVMenuItemClass *klass)
{

    /*继承*/
    GObjectClass *gobject_class = G_OBJECT_CLASS(klass);

    gobject_class->finalize = applet_popup_slider_v_menu_item_finalize;
    gobject_class->dispose = applet_popup_slider_v_menu_item_dispose;

}

/* _init */
static void applet_popup_slider_v_menu_item_init (AppletPopupSliderVMenuItem *self)
{
    PopupBaseMenuItemInit *params = g_new0 (PopupBaseMenuItemInit, 1);
    params = cdos_popup_base_menu_item_get_init_params (params);
    params->activate = FALSE;
    cdos_popup_base_menu_item_set_params (CDOS_POPUP_BASE_MENU_ITEM(self), params);

    self->p_container = st_bin_new ();
    self->p_box = st_box_layout_new();
    self->value_flag = st_bin_new();
    self->p_slider = st_bin_new ();

    st_widget_add_style_class_name (self->p_box, "sound-slider-box");

    cdos_popup_base_menu_item_add_actor (CDOS_POPUP_BASE_MENU_ITEM(self),
                                         CLUTTER_ACTOR(self->p_box),
                                         NULL);
    cdos_popup_base_menu_item_set_actor_align(CDOS_POPUP_BASE_MENU_ITEM(self),CLUTTER_ACTOR(self->p_box),ST_ALIGN_MIDDLE);


    st_widget_add_style_class_name (self->p_container, "sound-slider-container");
    clutter_container_add_actor(CLUTTER_CONTAINER(self->p_box),CLUTTER_ACTOR(self->p_container));
    st_bin_set_alignment (ST_BIN (self->p_container), ST_ALIGN_START, ST_ALIGN_START);

    self->p_slider_height = 190;
    self->p_slider_cursor_size = 5;

    g_object_set (CLUTTER_ACTOR (self->p_box), "x-align", CLUTTER_ACTOR_ALIGN_CENTER, NULL);

    clutter_actor_set_reactive ((ClutterActor *)self->p_slider, TRUE);
    st_widget_set_track_hover (self->p_slider, TRUE);
    st_widget_set_style_class_name (self->p_slider, "sound-slider-btn");
    g_object_set (ST_BIN (self->p_slider), "x-align", ST_ALIGN_START, NULL);
    clutter_container_add_actor (CLUTTER_CONTAINER(self->p_box), CLUTTER_ACTOR(self->p_slider));

    st_bin_set_child (ST_BIN(self->p_container), CLUTTER_ACTOR(self->value_flag));
    st_widget_set_style_class_name (self->value_flag, "sound-slider-flag");

    self->p_montion_id = 0;
    self->p_release_id = 0;
    self->p_dragging = FALSE;

    CdosGenericContainer *actor = cdos_popup_base_menu_item_get_actor (CDOS_POPUP_BASE_MENU_ITEM (self));
    g_signal_connect(CDOS_POPUP_BASE_MENU_ITEM (self), "value-changed", G_CALLBACK (callback_update_slider_bar), self);
    g_signal_connect(actor, "button-press-event", G_CALLBACK (callback_start_dragging), self);
    g_signal_connect(actor, "scroll-event", G_CALLBACK (callback_on_scroll_event), self);
    //zhangdj
    g_signal_connect(actor, "key-press-event", G_CALLBACK (callback_on_key_press_event), self);

    st_widget_add_style_class_name(actor,"sound-slider");
}

/**
 * applet_popup_slider_v_menu_item_new:
 * @float: 音量
 *
 * 新建一个 #AppletPopupSliderVMenuItem 类的实例。
 *
 * Returns: 返回一个 #AppletPopupSliderVMenuItem 的实例。
 */
AppletPopupSliderVMenuItem *applet_popup_slider_v_menu_item_new (gfloat value)
{
    AppletPopupSliderVMenuItem *self = g_object_new (APPLET_TYPE_POPUP_SLIDER_V_MENU_ITEM, NULL);
    //applet_popup_slider_v_menu_item_set_value(self,value);
    return self;
}

/**
 * applet_popup_slider_v_menu_item_set_value:
 * @self: A #AppletPopupSliderVMenuItem
 * @value: 音量
 *
 * 设置声音滑块音量
 *
 */
void applet_popup_slider_v_menu_item_set_value (AppletPopupSliderVMenuItem *self, gfloat value)
{


    /* FIXME:
     * self->p_value = round (MAX(MIN (value, 1), 0) * 100) / 100;
     * 这句出问题了,无法理解
     */
    if (value > 1) {
        self->p_value = 1;
    } else if(value < 0) {
        self->p_value = 0;
    } else {
        self->p_value = value;
    }
    callback_update_slider_bar (NULL, 0, self);

}


static void applet_popup_slider_v_menu_item_move_handle (AppletPopupSliderVMenuItem *self, float abs_X, float abs_Y)
{

    gfloat rel_x, rel_y, slider_x, slider_y;
    gfloat width, height;
    gfloat pw_size, pwmin, pwmax;
    gfloat ph_rel_top, ph_rel_bottom, phmin, phmax, new_value;
    gfloat tmp;//暂存变量

    CdosGenericContainer *actor = cdos_popup_base_menu_item_get_actor(CDOS_POPUP_BASE_MENU_ITEM(self));

    clutter_actor_get_transformed_position (CLUTTER_ACTOR(actor), &slider_x, &slider_y);
    clutter_actor_get_size (CLUTTER_ACTOR(actor), &width, &height);

    rel_x = abs_X - slider_x;
    tmp = clutter_actor_get_width (CLUTTER_ACTOR(actor));

    pw_size = tmp / 2 + slider_x;
    pwmin = pw_size - 30;
    pwmax = pw_size + 30;
    if (abs_X < pwmin || abs_X > pwmax) {
        return;
    }
    ph_rel_top = slider_y + (gint) ((height - self->p_slider_height) / 2);
    ph_rel_bottom = ph_rel_top + self->p_slider_height;

    phmin = ph_rel_top - 5;
    phmax = ph_rel_bottom + 5;
    rel_y = abs_Y - ph_rel_top;

    if (abs_Y < phmin || abs_Y > phmax) {
        return;
    }
    if (rel_x < self->p_slider_cursor_size) {
        new_value = 0;
    } else if (rel_x > self->p_slider_height - self->p_slider_cursor_size) {
        new_value = 1;
    } else {
        new_value = 1 - (rel_y - self->p_slider_cursor_size) / (self->p_slider_height - 2 * (self->p_slider_cursor_size));
    }

    if (new_value > 1) {
        self->p_value = 1;
    } else if (new_value < 0) {
        self->p_value = 0;
    } else {
        self->p_value = new_value;
    }
    g_signal_emit_by_name (self, "value-changed", self->p_value);
}

/**
 * applet_popup_slider_v_menu_item_get_value:
 * @self: A #AppletPopupSliderVMenuItem
 *
 * 获取声音滑块音量
 *
 */
gfloat applet_popup_slider_v_menu_item_get_value (AppletPopupSliderVMenuItem *self)
{
    return self->p_value;
}

/* ======== GObject END ======== */
