/**
 * SECTION: AppletSound
 * @short_description: 声音应用
 * @include: applet-sound.h
 *
 * AppletSoundClass 类用来调节，开启或者禁用声音的输入输出
 */
#include <glib.h>
#include <glib/gstdio.h>
#include <pulse/volume.h>
#include "applet-sound.h"
#include <math.h>
#include "ui/cdos-util.h"
#include "ui/cdos-systray.h"
#include "ui/cdos-sound-manager.h"
#include "ui/cdos-monkey-plugin.h"

/* ========== BASE ========== */
/* 继承关系 */
/*CdosGenericContainer->StContainer->StWidget->ClutterActor->GInitiallyUnowned->GObject*/
/*AppletNotifications->CdosTextIconApplet->IconApplet->Applet*/

/* 注册函数 */
G_DEFINE_TYPE (AppletSound, applet_sound, CDOS_TYPE_TEXT_ICON_APPLET);

#define ANALOG_STEREO_NOT_MUTED (g_strconcat ("", g_get_home_dir (), "/.cdos/configs/analog_stereo_not_muted", NULL))

/* 全局变量 */
static PopupBaseMenuItemInit *params_for_new_item;
static guint counter = 1;
const gchar *CDOS_DESKTOP_FIRST_RUN = "org.cdos";

/* 自定义数据结构 */
typedef struct _paramsInChanged paramsInChanged;
typedef struct _paramsInActivate paramsInActivate;

struct _paramsInChanged {
    AppletSound *self;
    gchar *property;
};

struct _paramsInActivate {
    GvcMixerStream *sink;
    AppletSound *self;
};

/* 信号 */
enum {
    PLAYERS_LOADED,
    LAST_SIGNAL
};

static guint applet_sound_signals[LAST_SIGNAL] = {0,};

/* ======== BASE END ======== */

/* ========== 函数声明 ========== */

/* 私有方法声明 */
static void applet_sound_show_fixed_elements (AppletSound *self);

static gchar *applet_sound_volume_to_icon (AppletSound *self, gfloat volume);

/* CALL_BACK */
static void applet_sound_read_output_cb (GObject *obj, guint state, gpointer user_data);

static void applet_sound_read_input_cb (GObject *obj, guint state, gpointer user_data);

static void applet_sound_maybe_show_input_cb (GObject *obj, guint state, gpointer user_data);

static void applet_sound_on_scroll_event_cb (GObject *obj, ClutterEvent *event, gpointer user_data);

static void applet_sound_toggle_out_mute_cb (GObject *obj, gboolean state, gpointer user_data);

static void applet_sound_toggle_in_mute_cb (GObject *obj, gboolean state, gpointer user_data);

static void applet_sound_on_control_state_changed_cb (GObject *obj, guint state, gpointer user_data);

static void applet_sound_notify_volume_change_cb (GObject *obj, gpointer user_data);

static void applet_sound_slider_changed_cb (GObject *obj, gfloat value, gpointer user_data);

static void applet_sound_muted_changed_cb (GObject *obj, GParamSpec *param_spec, paramsInChanged *params);

static void applet_sound_volume_changed_cb (GObject *obj, GParamSpec *param_spec, paramsInChanged *params);

static void handle_systray_icon_cb (GObject *obj, gpointer user_data);

static gboolean applet_sound_on_click_sound_settings_cb (GObject *obj, ClutterEvent *event, gpointer user_data);

/* ======== 函数声明 END ======== */

/* ========== Player 方法 ========== */

/**
 * applet_sound_on_settings_changed_cb:
 * @object: 绑定信号的对象
 * @user_data: 真对回调函数所传入的参数
 *
 * 当捕获到"showtrack"信号时,触发该函数
 */
void applet_sound_on_settings_changed_cb (GObject *obj, gpointer user_data)
{
    //Player
    AppletSound *self = APPLET_SOUND(user_data);
    //gpointer player;

    if (!self->show_track) {
        //applet_sound_set_applet_text(self, player); 操作player,可能不需要
        //FIXME
    }

    if (!self->show_album) {
        //applet_sound_set_applet_icon(self, player); 操作player,可能不需要
        //FIXME
    }

    if (self->show_track || self->show_album) {
        //for (owner in this._players) {
        //    this._addPlayer(owner);
        //}
    }

}

/* ======== Player END ======== */

/* ========== 回调函数 ========== */
static void handle_systray_icon_cb (GObject *obj, gpointer user_data)
{
    AppletSound *self = APPLET_SOUND (user_data);

    if (self->hide_systray) {
        applet_sound_register_systray_icons (self);
    } else {
        applet_sound_unregister_systray_icons (self);
    }
}

static void applet_sound_maybe_show_input_cb (GObject *object, guint state, gpointer user_data)
{
    AppletSound *self = (AppletSound *)user_data;

    // only show input widgets if any application is recording audio
    self->p_show_input = FALSE;
    GSList *recording_apps = gvc_mixer_control_get_source_outputs (self->p_control);
    guint length, i;
    length = g_slist_length (recording_apps);
    if (self->p_input && length != 0) {
        for (i = 0; i < length; i++) {
            GvcMixerStream *output_stream = g_slist_nth_data(recording_apps, i);
            const gchar *id_str = gvc_mixer_stream_get_application_id (output_stream);
            // but skip gnome-volume-control and pavucontrol
            // (that appear as recording because they show the input level)
            gboolean tmp = (g_strcmp0 (id_str, "org.gnome.VolumeControl") != 0) && (g_strcmp0 (id_str, "org.PulseAudio.pavucontrol") != 0);
            if (!id_str || tmp) {
                self->p_show_input = TRUE;
                break;
            }
        }
    }

    CdosGenericContainer *title_actor =
      cdos_popup_base_menu_item_get_actor(CDOS_POPUP_BASE_MENU_ITEM(self->p_input_title));
    CdosGenericContainer *slider_actor =
      cdos_popup_base_menu_item_get_actor(CDOS_POPUP_BASE_MENU_ITEM(self->p_input_slider));
    CdosPopupSeparatorMenuItem* item2 = cdos_popup_separator_menu_item_new();

    if (self->p_show_input) {
        clutter_actor_show(CLUTTER_ACTOR(cdos_popup_base_menu_item_get_actor(CDOS_POPUP_BASE_MENU_ITEM(item2))));
        clutter_actor_show(CLUTTER_ACTOR(title_actor));
        clutter_actor_show(CLUTTER_ACTOR(slider_actor));
        clutter_actor_show(
          CLUTTER_ACTOR(cdos_popup_base_menu_item_get_actor(CDOS_POPUP_BASE_MENU_ITEM(self->mute_in_switch_1))));
    } else {
        clutter_actor_hide(CLUTTER_ACTOR(cdos_popup_base_menu_item_get_actor(CDOS_POPUP_BASE_MENU_ITEM(item2))));
        clutter_actor_hide(CLUTTER_ACTOR(title_actor));
        clutter_actor_hide(CLUTTER_ACTOR(slider_actor));
        clutter_actor_hide(
          CLUTTER_ACTOR(cdos_popup_base_menu_item_get_actor(CDOS_POPUP_BASE_MENU_ITEM(self->mute_in_switch_1))));
    }

}

static void applet_sound_toggle_out_mute_cb (GObject *obj, gboolean state, gpointer user_data)
{
    AppletSound *self = APPLET_SOUND(user_data);

    gboolean is_muted = gvc_mixer_stream_get_is_muted (self->p_output);

    if (is_muted) {
        gvc_mixer_stream_change_is_muted (self->p_output, FALSE);
        // cdos_popup_switch_menu_item_set_toggle_status (self->mute_out_switch, FALSE);
        cdos_popup_switch_menu_item_set_toggle_status (self->mute_out_switch_1, FALSE);


    } else {
        gvc_mixer_stream_change_is_muted (self->p_output, TRUE);
        // cdos_popup_switch_menu_item_set_toggle_status (self->mute_out_switch, TRUE);
        cdos_popup_switch_menu_item_set_toggle_status (self->mute_out_switch_1, TRUE);

    }
}

static void applet_sound_toggle_in_mute_cb (GObject *obj, gboolean state, gpointer user_data)
{
    AppletSound *self = APPLET_SOUND(user_data);
    gboolean is_muted = gvc_mixer_stream_get_is_muted (self->p_input);

    if (is_muted) {
        gvc_mixer_stream_change_is_muted (self->p_input, FALSE);
        // cdos_popup_switch_menu_item_set_toggle_status (self->mute_in_switch, FALSE);
        cdos_popup_switch_menu_item_set_toggle_status (self->mute_in_switch_1, FALSE);
    } else {
        gvc_mixer_stream_change_is_muted (self->p_input, TRUE);
        // cdos_popup_switch_menu_item_set_toggle_status (self->mute_in_switch, TRUE);
        cdos_popup_switch_menu_item_set_toggle_status (self->mute_in_switch_1, TRUE);
    }
}

static void applet_sound_on_scroll_event_cb (GObject *obj, ClutterEvent *event, gpointer user_data)
{

    AppletSound *self = APPLET_SOUND(user_data);
    guint cal_volume_value;
    ClutterScrollDirection direction = clutter_event_get_scroll_direction (event);

    guint current_volume = (guint) gvc_mixer_stream_get_volume (self->p_output);

    if (direction == CLUTTER_SCROLL_DOWN) {
        gboolean prev_muted = gvc_mixer_stream_get_is_muted (self->p_output);
        cal_volume_value = (guint) MAX(0, (guint)(current_volume - self->p_volume_max * VOLUME_ADJUSTMENT_STEP));

        gvc_mixer_stream_set_volume (self->p_output, cal_volume_value);
        if (gvc_mixer_stream_get_volume (self->p_output) < 1) {
            gvc_mixer_stream_set_volume (self->p_output, 0);

            if (!prev_muted) {
                gvc_mixer_stream_change_is_muted (self->p_output, TRUE);
            }
        }

        gvc_mixer_stream_push_volume (self->p_output);
    } else if (direction == CLUTTER_SCROLL_UP) {
        cal_volume_value =  MIN((guint)self->p_volume_max,
                                (guint)(current_volume + self->p_volume_max * VOLUME_ADJUSTMENT_STEP));
        gvc_mixer_stream_set_volume (self->p_output, cal_volume_value);
        gvc_mixer_stream_change_is_muted (self->p_output, FALSE);
        gvc_mixer_stream_push_volume (self->p_output);

    }

    applet_sound_notify_volume_change_cb (NULL, NULL);
}

static void applet_sound_notify_volume_change_cb (GObject *obj, gpointer user_data)
{
    cdos_sound_manager_play ("volume");
}

static void applet_sound_muted_changed_cb (GObject *obj, GParamSpec *param_spec, paramsInChanged *params)
{
    AppletSound *self = params->self;
    gchar *property = params->property;
    gboolean is_analog_stereo = FALSE;

    gboolean muted;
    if (g_utf8_collate ("_output", property) == 0) {
        //_output,下面两句公有
        gfloat tmp = (gvc_mixer_stream_get_volume (self->p_output)) / (guint) (self->p_volume_max);
        muted = gvc_mixer_stream_get_is_muted (self->p_output);
        gchar *volume_0 = NULL;
        if (!g_strcmp0 (gvc_mixer_stream_get_description (self->p_output), _("Built-in Auido Analog Stereo"))) {
            is_analog_stereo = TRUE;
        }

        if (muted) {
            applet_popup_slider_v_menu_item_set_value (self->p_output_slider, 0);
            applet_sound_set_icon_name (self, "audio-volume-muted");
            applet_text_image_menu_item_set_icon (self->p_output_title, "audio-volume-muted");

            volume_0 = g_strconcat (_("Volume"), ":   0%", NULL);

            cdos_applet_set_applet_tooltip (CDOS_APPLET (self), volume_0);
            applet_text_image_menu_item_set_text (self->p_output_title, volume_0);
            cdos_popup_switch_menu_item_set_toggle_status (self->mute_out_switch_1, TRUE);
            g_free (volume_0);
            if (is_analog_stereo && g_file_test (ANALOG_STEREO_NOT_MUTED, G_FILE_TEST_EXISTS)) {
                g_remove (ANALOG_STEREO_NOT_MUTED);
            }
        } else {
            tmp = gvc_mixer_stream_get_volume (self->p_output) / self->p_volume_max;
            applet_popup_slider_v_menu_item_set_value (self->p_output_slider, tmp);
            gchar *text, *icon_str;
            icon_str = applet_sound_volume_to_icon (self, gvc_mixer_stream_get_volume (self->p_output));
            applet_sound_set_icon_name (self, (const gchar *)icon_str);
            applet_text_image_menu_item_set_icon (self->p_output_title, (const gchar *)icon_str);
            gint percent_tooltip = (gint)ceil (gvc_mixer_stream_get_volume (self->p_output) * 100 / self->p_volume_max);

            text = g_strdup_printf (": %3d%%", percent_tooltip);
            volume_0 = g_strconcat (_("Volume"), text, NULL);

            cdos_applet_set_applet_tooltip (CDOS_APPLET (self), volume_0);
            applet_text_image_menu_item_set_text (self->p_output_title, volume_0);

            cdos_popup_switch_menu_item_set_toggle_status (self->mute_out_switch_1, FALSE);

            g_free (text);
            g_free (icon_str);
            g_free (volume_0);
            if (is_analog_stereo && !g_file_test (ANALOG_STEREO_NOT_MUTED, G_FILE_TEST_EXISTS)) {
                int res = g_creat (ANALOG_STEREO_NOT_MUTED, 0644);
                if (res < 0) {
                    g_warning ("\nFun:%s, Create analog stereo mute\n", __FUNCTION__);
                    return;
                }
            }
        }
    } else if (g_utf8_collate ("_input", property) == 0) {
        //_input,下面两句公有
        gfloat tmp = (gvc_mixer_stream_get_volume (self->p_input)) / (guint) (self->p_volume_max);
        muted = gvc_mixer_stream_get_is_muted (self->p_input);
        gchar *volume_0 = NULL;
        applet_popup_slider_v_menu_item_set_value (self->p_input_slider, (muted ? 0 : tmp));


        if (muted) {
            applet_popup_slider_v_menu_item_set_value (self->p_input_slider, 0);
            // applet_sound_set_icon_name (self, "audio-volume-muted");
            applet_text_image_menu_item_set_icon (self->p_input_title, "audio-volume-muted");

            volume_0 = g_strconcat (_("Microphone"), ":   0%", NULL);
            // cdos_applet_set_applet_tooltip (CDOS_APPLET (self), volume_0);
            applet_text_image_menu_item_set_text (self->p_input_title, volume_0);
            cdos_popup_switch_menu_item_set_toggle_status (self->mute_in_switch_1, TRUE);
            g_free (volume_0);
        } else {
            tmp = gvc_mixer_stream_get_volume (self->p_input) / self->p_volume_max;
            applet_popup_slider_v_menu_item_set_value (self->p_input_slider, tmp);
            gchar *text, *icon_str;
            icon_str = applet_sound_volume_to_icon (self, gvc_mixer_stream_get_volume (self->p_input));
            // applet_sound_set_icon_name (self, (const gchar *)icon_str);
            applet_text_image_menu_item_set_icon (self->p_input_title, (const gchar *)icon_str);
            gint percent_tooltip = (gint)ceil (gvc_mixer_stream_get_volume (self->p_input) * 100 / self->p_volume_max);
            text = g_strdup_printf (": %3d%%", percent_tooltip);
            volume_0 = g_strconcat (_("Microphone"), text, NULL);

            // cdos_applet_set_applet_tooltip (CDOS_APPLET (self), volume_0);
            applet_text_image_menu_item_set_text (self->p_input_title, volume_0);
            cdos_popup_switch_menu_item_set_toggle_status (self->mute_in_switch_1, FALSE);

            g_free (text);
            g_free (icon_str);
            g_free (volume_0);
        }
    } else {
        g_warning ("property is not suit");
    }
}

static void applet_sound_volume_changed_cb (GObject *obj, GParamSpec *param_spec, paramsInChanged *params)
{
    gchar *property = params->property;
    AppletSound *self = params->self;
    gfloat tmp;
    if (g_utf8_collate (property, "_output") == 0) {
        if (!self->p_output) {
            return;
        }
        if (!gvc_mixer_stream_get_is_muted (self->p_output)) {
            tmp = (gfloat)(gvc_mixer_stream_get_volume (self->p_output) / self->p_volume_max);
            applet_popup_slider_v_menu_item_set_value (self->p_output_slider, tmp);

            gchar * str = NULL;
            str = applet_sound_volume_to_icon (self, gvc_mixer_stream_get_volume (self->p_output));

            applet_text_image_menu_item_set_icon (self->p_output_title, str);
            applet_sound_set_icon_name (self, str);
            gint percent_tooltip = (gint)ceil (gvc_mixer_stream_get_volume (self->p_output) * 100 / self->p_volume_max);
            gchar *text = NULL, *volume_value = NULL;
            text = g_strdup_printf (": %3d%%", percent_tooltip);//输出%格式是%%
            volume_value = g_strconcat (_("Volume"), text, NULL);
            cdos_applet_set_applet_tooltip (CDOS_APPLET (self), volume_value);

            applet_text_image_menu_item_set_text (self->p_output_title, (const gchar *)volume_value);

            g_free (text);
            g_free (volume_value);
            g_free (str);
        }
    } else if (g_utf8_collate (property, "_input") == 0) {
        if (!self->p_input) {
            return ;
        }

        if (!gvc_mixer_stream_get_is_muted (self->p_input)) {
            tmp = (gfloat)(gvc_mixer_stream_get_volume (self->p_input) / self->p_volume_max);
            applet_popup_slider_v_menu_item_set_value (self->p_input_slider, tmp);

            gchar * str = NULL;
            str = applet_sound_volume_to_icon (self, gvc_mixer_stream_get_volume (self->p_input));

            applet_text_image_menu_item_set_icon (self->p_input_title, str);
            // applet_sound_set_icon_name (self, str);

            gint percent_tooltip = (gint)ceil (gvc_mixer_stream_get_volume (self->p_input) * 100 / self->p_volume_max);

            gchar *text = NULL, *volume_value = NULL;
            text = g_strdup_printf (": %3d%%", percent_tooltip);//输出%格式是%%
            volume_value = g_strconcat (_("Microphone"), text, NULL);
            // cdos_applet_set_applet_tooltip (CDOS_APPLET (self), volume_value);

            applet_text_image_menu_item_set_text (self->p_input_title, (const gchar *)volume_value);

            g_free (text);
            g_free (volume_value);
            g_free (str);
        }

    } else {
        g_warning ("Property not correct!\n");
        return ;
    }
}

static void applet_sound_slider_changed_cb (GObject *obj, gfloat value, gpointer user_data)
{

    paramsInChanged *params = (paramsInChanged *)user_data;
    gchar *property;
    property = params->property;
    AppletSound *self = APPLET_SOUND (params->self);
    //CdosPopupBaseMenuItem *slider = CDOS_POPUP_BASE_MENU_ITEM (obj);
    if (g_utf8_collate (property, "_input") != 0 && g_utf8_collate (property, "_output") != 0) {
        g_warning ("Volume slider changed for %s, but %s does not exist", property, property);
        return;
    }

    gfloat volume = value * self->p_volume_max;
    gboolean prev_muted;
    if (g_utf8_collate (property, "_input") == 0) {
        prev_muted = gvc_mixer_stream_get_is_muted (self->p_input);
        if (volume < 1) {
            gvc_mixer_stream_set_volume (self->p_input, 0);
            if (!prev_muted) {
                gvc_mixer_stream_change_is_muted (self->p_input, TRUE);
            }
        } else {
            gvc_mixer_stream_set_volume (self->p_input, (guint) volume);
            if (prev_muted) {
                gvc_mixer_stream_change_is_muted (self->p_input, FALSE);
                cdos_popup_switch_menu_item_set_toggle_status (self->mute_in_switch_1, FALSE);
            }
        }

        gvc_mixer_stream_push_volume (self->p_input);

    } else {

        prev_muted = gvc_mixer_stream_get_is_muted (self->p_output);

        if (volume < 1) {
            gvc_mixer_stream_set_volume (self->p_output, 0);
            if (!prev_muted) {
                gvc_mixer_stream_change_is_muted (self->p_output, TRUE);
            }
        } else {
            gvc_mixer_stream_set_volume (self->p_output, (guint) volume);
            if (prev_muted) {
                gvc_mixer_stream_change_is_muted (self->p_output, FALSE);
                cdos_popup_switch_menu_item_set_toggle_status (self->mute_out_switch_1, FALSE);
            }
        }

        gvc_mixer_stream_push_volume (self->p_output);

    }
}

static void applet_sound_on_control_state_changed_cb (GObject *obj, guint state, gpointer user_data)
{
    //GvcMixerControl *p_control = GVC_MIXER_CONTROL (obj);
    AppletSound *self = APPLET_SOUND (user_data);
    if (gvc_mixer_control_get_state (self->p_control) == GVC_STATE_READY) {
        applet_sound_read_output_cb (NULL, 0, self);
        applet_sound_read_input_cb (NULL, 0, self);
        clutter_actor_show (CLUTTER_ACTOR(CDOS_APPLET (self)->actor));
    } else {
        clutter_actor_hide (CLUTTER_ACTOR(CDOS_APPLET (self)->actor));
    }
}

static gboolean timeout_fixed_cb (gpointer user_data)
{
    AppletSound *self = APPLET_SOUND (user_data);
    gboolean is_analog_stereo = FALSE;
    gvc_mixer_stream_change_is_muted (self->p_output, FALSE);

    counter = counter - 1;

    if (counter != 0) {
        return TRUE;
    }
    return FALSE;
}

static void applet_sound_read_output_cb (GObject *obj, guint state, gpointer user_data)
{
    //GvcMixerControl *p_control = GVC_MIXER_CONTROL (obj);

    AppletSound *self = APPLET_SOUND (user_data);
    if (self->p_output_volume_id != 0) {
        g_signal_handler_disconnect (self->p_output, self->p_output_volume_id);
        g_signal_handler_disconnect (self->p_output, self->p_output_muted_id);
        self->p_output_volume_id = 0;
        self->p_output_muted_id = 0;
    }

    gboolean is_analog_stereo = FALSE;
    static guint once = 1;

    self->p_output = gvc_mixer_control_get_default_sink (self->p_control);
    if (self->p_output) {
        if (!g_strcmp0 (gvc_mixer_stream_get_description (self->p_output), _("Built-in Auido Analog Stereo"))) {
            is_analog_stereo = TRUE;
        }

        if (is_analog_stereo && g_file_test (ANALOG_STEREO_NOT_MUTED, G_FILE_TEST_EXISTS) && once) {
            // 模拟立体声，上一次状态是非静音
            if (gvc_mixer_stream_get_is_muted (self->p_output)) {
                gvc_mixer_stream_change_is_muted (self->p_output, FALSE);
                g_timeout_add_seconds (5, timeout_fixed_cb, self);
            }
            // 只修正一次
            once = once - 1;
        }

        paramsInChanged *params_output = g_new0(paramsInChanged, 1);
        params_output->self = self;
        params_output->property = "_output";
        self->p_output_muted_id = g_signal_connect (self->p_output, "notify::is-muted",
                                                    G_CALLBACK (applet_sound_muted_changed_cb), params_output);
        self->p_output_volume_id = g_signal_connect (self->p_output, "notify::volume",
                                                     G_CALLBACK (applet_sound_volume_changed_cb), params_output);
        applet_sound_muted_changed_cb (NULL, NULL, params_output);
        applet_sound_volume_changed_cb (NULL, NULL, params_output);
    } else {
        applet_popup_slider_v_menu_item_set_value (self->p_output_slider, 0);
        applet_sound_set_icon_name (self, "audio-volume-muted-symbolic");
    }
}

void applet_sound_read_input_cb (GObject *obj, guint state, gpointer user_data)
{
    AppletSound *self = APPLET_SOUND (user_data);
    if (self->p_input_volume_id != 0) {
        g_signal_handler_disconnect (self->p_input, self->p_input_volume_id);
        g_signal_handler_disconnect (self->p_input, self->p_input_muted_id);
        self->p_input_volume_id = 0;
        self->p_input_muted_id = 0;
    }

    self->p_input = gvc_mixer_control_get_default_source (self->p_control);
    if (self->p_input) {
        paramsInChanged *params = g_new0(paramsInChanged, 1);
        params->self = self;
        params->property = "_input";

        self->p_input_muted_id =
        g_signal_connect (self->p_input, "notify::is-muted", G_CALLBACK (applet_sound_muted_changed_cb), params);
        self->p_input_volume_id =
        g_signal_connect (self->p_input, "notify::volume", G_CALLBACK (applet_sound_volume_changed_cb), params);

        applet_sound_muted_changed_cb (NULL, NULL, params);
        applet_sound_volume_changed_cb (NULL, NULL, params);

    } else {
        CdosGenericContainer
        *title_actor = cdos_popup_base_menu_item_get_actor (CDOS_POPUP_BASE_MENU_ITEM (self->p_input_title));
        CdosGenericContainer
        *slider_actor = cdos_popup_base_menu_item_get_actor (CDOS_POPUP_BASE_MENU_ITEM (self->p_input_slider));
        clutter_actor_hide (CLUTTER_ACTOR (title_actor));
        clutter_actor_hide (CLUTTER_ACTOR (slider_actor));
    }
}

static gboolean applet_sound_on_click_sound_settings_cb (GObject *obj, ClutterEvent *event, gpointer user_data)
{
    //TODO:sound settings
    gint ret = cdos_util_spawn_command_line ("cdos-settings sound");
    if ( ret < 0){
        cdos_debug ();
        return FALSE;
    }

    return TRUE;
}

/* ======== 回调函数 END ======== */

/* ========== 替换函数 ========== */

/* ======== 替换函数 END ======== */

/* ========== GObject ========== */

/* finalize */
static void applet_sound_finalize (GObject *obj)
{
    AppletSound *self = APPLET_SOUND (obj);

    /*public*/
    g_free(self->metadata);
    g_free(self->settings);
    g_free(self->mute_out_switch);
    g_free(self->mute_out_switch_1);
    g_free(self->mute_in_switch);
    g_free(self->mute_in_switch_1);
    g_free(self->sound_settings);

    /*private*/
    g_free(self->p_control);   /*gvc_mixer_control_new                 (const char *name);*/
    g_free(self->p_output);    /*1409:gvc_mixer_control_get_default_source   (GvcMixerControl *control);*/
    g_free(self->p_input);
    g_free(self->p_icon_name);
    g_free(self->p_icon_path);
    g_free(self->p_output_title);
    g_free(self->p_input_title);
    g_free(self->p_output_slider);
    g_free(self->p_input_slider);
    g_free(self->p_select_device_item);
    g_free(self->applet_context_menu_p);

    /* 调用父类的 finalize () 方法 */
    G_OBJECT_CLASS (applet_sound_parent_class)->finalize (obj);
};

/* _class_init */
static void applet_sound_class_init (AppletSoundClass *klass)
{
    /*继承*/
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    gobject_class->finalize = applet_sound_finalize;

    /*创建信号*/
    applet_sound_signals[PLAYERS_LOADED] =
            g_signal_new ("players-loaded",
                          G_TYPE_FROM_CLASS (klass),
                          G_SIGNAL_RUN_LAST,
                          G_STRUCT_OFFSET (AppletSoundClass, players_loaded),
                          NULL,
                          NULL,
                          g_cclosure_marshal_VOID__VOID,
                          G_TYPE_NONE, 0,
                          NULL
            );

}

/* _init */
static void applet_sound_init (AppletSound *self)
{
    // Set default volume when first run
    self->first_run = g_settings_new (CDOS_DESKTOP_FIRST_RUN);
    gboolean is_initialized = g_settings_get_boolean (self->first_run, "has-sink-volume");
    if (!is_initialized) {
        GError *error = NULL;
        const gchar *set_sink_command = "pactl set-sink-volume @DEFAULT_SINK@ 43801"; // 67%
        gboolean res = g_spawn_command_line_async (set_sink_command, &error);
        if (!res) {
            g_warning ("\nCdosDesktop: set sink volume failed when first run!\n");
        }
        g_settings_set_boolean (self->first_run, "has-sink-volume", TRUE);
    }

    if (self->hide_systray) {
        applet_sound_register_systray_icons (self);
    }

    cdos_iconapplet_set_icon_symbolic_name (CDOS_ICONAPPLET (self), "audio-x-generic");

    self->p_control = gvc_mixer_control_new ("Cinnamon Volume Control");
    g_signal_connect (self->p_control, "state-changed", G_CALLBACK (applet_sound_on_control_state_changed_cb), self);
    g_signal_connect (self->p_control, "card-added", G_CALLBACK (applet_sound_on_control_state_changed_cb), self);
    g_signal_connect (self->p_control, "card-removed", G_CALLBACK (applet_sound_on_control_state_changed_cb), self);
    g_signal_connect (self->p_control, "default-sink-changed", G_CALLBACK (applet_sound_read_output_cb), self);
    g_signal_connect (self->p_control, "default-source-changed", G_CALLBACK (applet_sound_read_input_cb), self);
    g_signal_connect (self->p_control, "stream-added", G_CALLBACK (applet_sound_maybe_show_input_cb), self);
    g_signal_connect (self->p_control, "stream-removed", G_CALLBACK (applet_sound_maybe_show_input_cb), self);

    /* TODO:
     * previously was 1.5*this._control.get_vol_max_norm();,
     * but we'd need a little mark on the slider to make it obvious to the user we're going over 100%..
     */
   #if 1
    self->p_volume_max = (gfloat)(1 * gvc_mixer_control_get_vol_max_norm (self->p_control));
   #endif
   #if 0
    self->p_volume_max = (gfloat)(1 * gvc_mixer_control_get_vol_max_amplified (self->p_control));
   #endif
    self->p_output = NULL;
    self->p_output_volume_id = 0;
    self->p_output_muted_id = 0;

    self->p_input = NULL;
    self->p_input_volume_id = 0;
    self->p_input_muted_id = 0;

    self->p_icon_name = NULL;
    self->p_icon_path = NULL;
    self->p_volume_control_shown = FALSE;
};

/**
 * applet_sound_main:
 * @metadata: applet元数据
 * @orientation: 方向
 * @panel_height: 面板高度
 * @instanceId: 字符串类型的id
 *
 * 新建一个 #CdosApplet 类的实例。
 *
 * Returns: 返回一个 #CdosApplet 的实例。
 */
CdosApplet *applet_sound_main (MetaData *metadata, StSide orientation, guint panel_height, gchar *instanceId)
{
    /*JS文件中有try...cache,以下是try的部分*/
    AppletSound *self = g_object_new (APPLET_TYPE_SOUND, NULL);

    cdos_applet_set_orientation (CDOS_APPLET (self), orientation);
    cdos_applet_set_panel_height (CDOS_APPLET (self), panel_height);

    self->metadata = metadata;
    self->settings = cdos_applet_settings_new (G_OBJECT (self), (const gchar *)metadata->uuid, (const gchar *)instanceId);
    self->menu = cdos_applet_get_popup_menu((CdosApplet *)self);

    applet_sound_show_fixed_elements (self);

    gboolean result = gvc_mixer_control_open (self->p_control);
    if (result) {
        g_warning ("\n gvc_mixer_control_open error! \n");
    }

    g_signal_connect (cdos_popup_base_menu_item_get_actor (CDOS_POPUP_BASE_MENU_ITEM (self->sound_settings)),
                      "button-press-event", G_CALLBACK (applet_sound_on_click_sound_settings_cb), NULL);
    cdos_iconapplet_reset_for_systray(CDOS_ICONAPPLET(self));
    return CDOS_APPLET (self);
}

/**
 * applet_sound_on_applet_removed_from_panel:
 * @self: A #AppletSound
 *
 * 从面板移除applet
 *
 */
void applet_sound_on_applet_removed_from_panel (AppletSound *self)
{
    if (self->hide_systray) {
        applet_sound_unregister_systray_icons (self);
    }

    if (self->p_icon_timeout_id) {
        g_source_remove (self->p_icon_timeout_id);
    }
}

/**
 * applet_sound_set_icon_name:
 * @self: A #AppletSound
 * @icon: 图标名称
 *
 * 设置图标名称
 *
 */
void applet_sound_set_icon_name (AppletSound *self, const gchar *icon)
{
    self->p_icon_name = (gchar *) icon;
    cdos_iconapplet_set_icon_symbolic_name (CDOS_ICONAPPLET (self), icon);
}

/*
 * applet_sound_set_applet_icon:
 * @self: A #AppletSound
 * @player: player对象
 * @path: 路径
 *
 * 设置applet里player的名称，未完成
 *
 */
void applet_sound_set_applet_icon (AppletSound *self, GObject *player, gchar *path)
{
    //FIXME
    //Player相关,暂时不写!
}

/*
 * applet_sound_set_applet_text:
 * @self: A #AppletSound
 * @player: player对象
 *
 * 设置applet里player的图标，未完成
 *
 */
void applet_sound_set_applet_text (AppletSound *self, GObject *player)
{
    //FIXME
}

/*
 * applet_sound_set_applet_text_icon:
 * @self: A #AppletSound
 * @player: player对象
 * @path: 路径
 *
 * 设置applet里player的文本，未完成
 */
void applet_sound_set_applet_text_icon (AppletSound *self, GObject *player, gchar *path)
{
    applet_sound_set_applet_icon (self, player, path);
    applet_sound_set_applet_text (self, player);
}

static void applet_sound_show_fixed_elements (AppletSound *self)
{

    if (self->p_volume_control_shown) {
        return;
    }

    self->p_volume_control_shown = TRUE;
    /* FIXME:
     * 是否需要?
     */
    self->p_output_title = applet_text_image_menu_item_new (_("Volume"),
                                                            "audio-volume-high",
                                                            NULL,
                                                            "right",
                                                            "sound-volume-menu-item");

    self->p_output_slider = applet_popup_slider_v_menu_item_new (0);

    paramsInChanged *params_out = g_new0(paramsInChanged, 1);
    params_out->self = self;
    params_out->property = "_output";

    g_signal_connect(self->p_output_slider, "value-changed",
                     G_CALLBACK (applet_sound_slider_changed_cb), params_out);
    g_signal_connect(self->p_output_slider, "drag-end",
                     G_CALLBACK (applet_sound_notify_volume_change_cb), self);

    cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE(self->menu),
                                        CDOS_POPUP_BASE_MENU_ITEM(self->p_output_title),
                                        -1);
    cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE(self->menu),
                                        CDOS_POPUP_BASE_MENU_ITEM(self->p_output_slider),
                                        -1);

    params_for_new_item = g_new0 (PopupBaseMenuItemInit, 1);
    params_for_new_item = cdos_popup_base_menu_item_get_init_params (params_for_new_item);
    self->mute_out_switch_1 = cdos_popup_switch_menu_item_new_with_params (_("Mute output"), TRUE, params_for_new_item);

    cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE(self->menu),
                                        CDOS_POPUP_BASE_MENU_ITEM(self->mute_out_switch_1),
                                        -1);

    g_signal_connect(self->mute_out_switch_1, "toggled", G_CALLBACK (applet_sound_toggle_out_mute_cb), self);
    CdosPopupSeparatorMenuItem *item = cdos_popup_separator_menu_item_new ();
    cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE(self->menu),
                                        CDOS_POPUP_BASE_MENU_ITEM(item),
                                        -1);

    params_for_new_item->reactive = FALSE;
    self->p_input_title = applet_text_image_menu_item_new (_("Microphone"),
                                                        "audio-volume-high",
                                                        NULL,
                                                        "right",
                                                        "sound-volume-menu-item");
    self->p_input_slider = applet_popup_slider_v_menu_item_new (0);

    paramsInChanged *params_in = g_new0(paramsInChanged, 1);
    params_in->property = "_input";
    params_in->self = self;

    g_signal_connect (CDOS_POPUP_BASE_MENU_ITEM (self->p_input_slider),
                      "value-changed", G_CALLBACK (applet_sound_slider_changed_cb), params_in);
    g_signal_connect (CDOS_POPUP_BASE_MENU_ITEM (self->p_input_slider),
                      "drag-end", G_CALLBACK (applet_sound_notify_volume_change_cb), NULL);

    cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE (self->menu),
                                        CDOS_POPUP_BASE_MENU_ITEM (self->p_input_title),
                                        -1);

    cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE (self->menu),
                                        CDOS_POPUP_BASE_MENU_ITEM (self->p_input_slider),
                                        -1);
    //重置params
    g_free (params_for_new_item);
    params_for_new_item = g_new0 (PopupBaseMenuItemInit, 1);
    params_for_new_item = cdos_popup_base_menu_item_get_init_params (params_for_new_item);
    self->mute_in_switch_1 = cdos_popup_switch_menu_item_new_with_params (_("Mute input"), FALSE, params_for_new_item);

    //FIXME
    cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE (self->menu),
                                        CDOS_POPUP_BASE_MENU_ITEM (self->mute_in_switch_1),
                                        -1);

    g_signal_connect (self->mute_in_switch_1, "toggled", G_CALLBACK (applet_sound_toggle_in_mute_cb), self);

    /* FIXME:
     * 替换:cdos_popup_menu_base_add_settings_action (CDOS_POPUP_MENU_BASE (self->menu), "Sound Settings", "sound");
     * 暂时不写:self->p_select_device_item = cdos_popup_submenu_menu_item_new (_("Output device..."), TRUE);
     */

    CdosPopupSeparatorMenuItem *item2 = cdos_popup_separator_menu_item_new ();
    cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE(self->menu), CDOS_POPUP_BASE_MENU_ITEM (item2), -1);
    if (self->p_show_input) {
        clutter_actor_show (
                CLUTTER_ACTOR (cdos_popup_base_menu_item_get_actor (CDOS_POPUP_BASE_MENU_ITEM (item2))));
        clutter_actor_show (
                CLUTTER_ACTOR (cdos_popup_base_menu_item_get_actor (CDOS_POPUP_BASE_MENU_ITEM (self->p_input_title))));
        clutter_actor_show (
                CLUTTER_ACTOR (cdos_popup_base_menu_item_get_actor (CDOS_POPUP_BASE_MENU_ITEM (self->p_input_slider))));
        clutter_actor_show (
                CLUTTER_ACTOR (cdos_popup_base_menu_item_get_actor (CDOS_POPUP_BASE_MENU_ITEM (self->mute_in_switch_1))));
    } else {
        clutter_actor_hide (
                CLUTTER_ACTOR (cdos_popup_base_menu_item_get_actor (CDOS_POPUP_BASE_MENU_ITEM (item2))));
        clutter_actor_hide (
                CLUTTER_ACTOR (cdos_popup_base_menu_item_get_actor (CDOS_POPUP_BASE_MENU_ITEM (self->p_input_title))));
        clutter_actor_hide (
                CLUTTER_ACTOR (cdos_popup_base_menu_item_get_actor (CDOS_POPUP_BASE_MENU_ITEM (self->p_input_slider))));
        clutter_actor_hide (
                CLUTTER_ACTOR (cdos_popup_base_menu_item_get_actor (CDOS_POPUP_BASE_MENU_ITEM (self->mute_in_switch_1))));
    }

    self->sound_settings = cdos_popup_menu_item_new();
    //self->sound_settings = cdos_popup_menu_item_new_with_params (_("Sound Settings"), params_for_new_item);
    cdos_popup_menu_item_set_label(self->sound_settings,_("Sound Settings"));
    cdos_popup_menu_item_set_label_align(self->sound_settings,ST_ALIGN_MIDDLE,0);
    cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE(self->menu), CDOS_POPUP_BASE_MENU_ITEM(self->sound_settings), -1);
    StLabel *label = cdos_popup_menu_item_get_label(self->sound_settings);
    st_widget_add_style_class_name(ST_WIDGET(label),"popup-menu-item-label-link");
    //CdosGenericContainer *sound_settings_actor = cdos_popup_base_menu_item_get_actor(self->sound_settings);
    //st_widget_add_style_class_name(ST_WIDGET(sound_settings_actor),"sound-menu-item-settings");

    paramsInChanged *params_input, *params_output;
    params_input = g_new0(paramsInChanged, 1);
    params_input->self = self;
    params_input->property = "_input";

    params_output = g_new0(paramsInChanged, 1);
    params_output->self = self;
    params_output->property = "_output";

    applet_sound_volume_changed_cb (NULL, NULL, params_output);
    applet_sound_volume_changed_cb (NULL, NULL, params_input);
    g_free (params_for_new_item);

}

static gchar *applet_sound_volume_to_icon (AppletSound *self, gfloat volume)
{
    if (volume <= 0) {
        return g_strdup ("audio-volume-muted");
    } else {
        gfloat n = (gfloat)ceil (3 * volume / self->p_volume_max) + 1;

        if (n < 2) {
            return g_strdup ("audio-volume-low");
        }

        if (n >= 3) {
            return g_strdup ("audio-volume-high");
        }

        return g_strdup ("audio-volume-medium");
    }
}

/**
 * applet_sound_register_systray_icons:
 * @self: A #AppletSound
 *
 * 把self注册到CdosSystray中
 *
 */
void applet_sound_register_systray_icons (AppletSound *self)
{
    guint i;
    gchar *support_seek[] = {"cdos-player"};
    for(i = 0; i < 1; i++) {
        MonkeyPlugin *plugin = monkey_plugin_get ();
        CdosSystrayManager *systray_manager = cdos_monkey_plugin_get_systray_manager (plugin);
        cdos_systray_manager_register_role (systray_manager, support_seek[i], self->metadata->uuid);
    }
}

/**
 * applet_sound_unregister_systray_icons:
 * @self: A #AppletSound
 *
 * 把self从CdosSystray中移除
 *
 */
void applet_sound_unregister_systray_icons (AppletSound *self)
{
    MonkeyPlugin *plugin = monkey_plugin_get ();
    CdosSystrayManager *systray_manager = cdos_monkey_plugin_get_systray_manager (plugin);
    cdos_systray_manager_unregister_id(systray_manager, self->metadata->uuid);
}

/* ======== GObject END ======== */
