/**
 * SECTION: AppletTextImageMenuItem
 * @short_description: 声音应用的文字与图标部分
 * @include: applet-text-image-menu-item.h
 * 
 * AppletTextImageMenuItemClass 类用来构建声音应用的图标和文字部分
 */
#include "applet-text-image-menu-item.h"

G_DEFINE_TYPE(AppletTextImageMenuItem, applet_text_image_menu_item, CDOS_TYPE_POPUP_BASE_MENU_ITEM);

ClutterActor *
applet_text_image_menu_item_get_icon_image (AppletTextImageMenuItem *self, const gchar *icon_name);


static void applet_text_image_menu_item_finalize (GObject *obj)
{

    AppletTextImageMenuItem *self = APPLET_TEXT_IMAGE_MENU_ITEM(obj);

    G_OBJECT_CLASS(applet_text_image_menu_item_parent_class)->finalize(obj);
}

/* _class_init */
static void applet_text_image_menu_item_class_init(AppletTextImageMenuItemClass *klass)
{

    GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
    gobject_class->finalize = applet_text_image_menu_item_finalize;

}

/* _init */
static void applet_text_image_menu_item_init (AppletTextImageMenuItem *self)
{
    /*NULL*/
}

/**
 * applet_text_image_menu_item_new:
 * @text: 音量百分比
 * @icon: 喇叭图标
 * @image: 背景图片
 * @align: 文本和图标的排序 
 * @style: 样式
 *
 * 新建一个 #AppletTextImageMenuItem 类的实例。
 *
 * Returns: 返回一个 #AppletTextImageMenuItem 的实例。
 */
AppletTextImageMenuItem *
applet_text_image_menu_item_new (const gchar *text, const gchar *icon,
                                 const gchar *image, const gchar *align, const gchar *style)
{
    AppletTextImageMenuItem *self = g_object_new(APPLET_TYPE_TEXT_IMAGE_MENU_ITEM, NULL);

    //设置this.actor
    StWidget *sw = st_box_layout_new();
    cdos_popup_base_menu_item_set_actor (CDOS_POPUP_BASE_MENU_ITEM (self),(CdosGenericContainer *)sw);

    //使用this.actor
    CdosGenericContainer *actor = cdos_popup_base_menu_item_get_actor(CDOS_POPUP_BASE_MENU_ITEM (self));
    st_widget_add_style_class_name(ST_WIDGET(actor),"sound-text-image-menu-item");
    /* FIXME:
     * 自带样式有点问题
     * 暂时使用自定义样式
     * st_widget_add_style_class_name (ST_WIDGET (actor), style);
     */
    st_widget_add_style_class_name (ST_WIDGET (actor), style);
    st_widget_add_style_pseudo_class (ST_WIDGET (actor), "active");

    if (icon) {
        self->st_icon = ST_ICON(st_icon_new());
        st_icon_set_icon_name(ST_ICON(self->st_icon), icon);
    }

    if (image) {
        self->st_icon = ST_ICON (st_bin_new());
        st_bin_set_child(ST_BIN(self->st_icon), applet_text_image_menu_item_get_icon_image(self, image));
    }

    self->st_text = st_label_new(text);
    st_widget_set_style_class_name(self->st_text,"sound-volume-menu-item-label");

    //FIXME
    //'clutter_container_add_actor' is deprecated, Use 'clutter_actor_add_child' instead
    ActorChild actor_child;
    cdos_popup_base_menu_item_get_actor_child(&actor_child);

    //设置左右,可以向CDOS_POPUP_BASE_MENU_ITEM (self)添加actor,也可以直接添加container_actor
    if (g_utf8_collate(align, "left") == 0) {
        actor_child.span = 0;
        //this.actor.add_actor(this.icon, { span: 0 });
        //this.actor.add_actor(this.text, { span: -1 });
        //cdos_popup_base_menu_item_add_actor (CDOS_POPUP_BASE_MENU_ITEM (self), CLUTTER_ACTOR(self->icon), actor_child_1);
        //cdos_popup_base_menu_item_add_actor (CDOS_POPUP_BASE_MENU_ITEM (self), CLUTTER_ACTOR(self->text), actor_child_2);
        clutter_container_add_actor(CLUTTER_CONTAINER(actor), CLUTTER_ACTOR(self->st_icon));
        clutter_container_add_actor(CLUTTER_CONTAINER(actor), CLUTTER_ACTOR(self->st_text));
    } else {
        //this.actor.add_actor(this.text, { span: 0 });
        //this.actor.add_actor(this.icon, { span: -1 });
        actor_child.span = -1;
        //cdos_popup_base_menu_item_add_actor (CDOS_POPUP_BASE_MENU_ITEM (self), CLUTTER_ACTOR(self->icon), actor_child_2);
        //cdos_popup_base_menu_item_add_actor (CDOS_POPUP_BASE_MENU_ITEM (self), CLUTTER_ACTOR(self->text), actor_child_1);
        clutter_container_add_actor(CLUTTER_CONTAINER(actor), CLUTTER_ACTOR(self->st_text));
        
        clutter_container_add_actor(CLUTTER_CONTAINER(actor), CLUTTER_ACTOR(self->st_icon));
    }

    //st_widget_set_style (self->st_text, "width: 70px;");
    return self;
}

/**
 * applet_text_image_menu_item_set_text:
 * @self: A #AppletTextImageMenuItem
 * @text: 文本
 *
 * 设置self的文本
 *
 */
void applet_text_image_menu_item_set_text (AppletTextImageMenuItem *self, const gchar *text)
{
    st_label_set_text(ST_LABEL(self->st_text), text);
}

/**
 * applet_text_image_menu_item_set_icon:
 * @self: A #AppletTextImageMenuItem
 * @icon: 图标 
 *
 * 设置self的图标
 *
 */
void applet_text_image_menu_item_set_icon (AppletTextImageMenuItem *self, const gchar *icon)
{
    st_icon_set_icon_name(ST_ICON(self->st_icon), icon);
}

/**
 * applet_text_image_menu_item_set_image:
 * @self: A #AppletTextImageMenuItem
 * @image: 图片
 *
 * 设置self的背景图片
 *
 */
void applet_text_image_menu_item_set_image (AppletTextImageMenuItem *self, const gchar *image)
{
    st_bin_set_child(ST_BIN(self->st_icon), applet_text_image_menu_item_get_icon_image(self, image));
}

ClutterActor * applet_text_image_menu_item_get_icon_image (AppletTextImageMenuItem *self, const gchar *icon_name)
{
    ClutterActor *ca = NULL;
    gchar *icon_file;
    GFile *file_;

    icon_file = g_strconcat(THEME_ICON_PATH, icon_name, ".svg", NULL);
    file_ = g_file_new_for_path((const gchar *)icon_file);;
    char * icon_uri = g_file_get_uri(file_);

    ca = st_texture_cache_load_uri_async(st_texture_cache_get_default(),
                                         (const gchar *)icon_uri, 16, 16);
    return ca;
}

/* ======== GObject END ======== */
