#include "applets-systray.h"
#include "cdos-start.h"
#include "cdos-tooltip.h"
#include "cdos-debug.h"


#include "cdos-icon-dispatcher.h"
#include "cdos-tray-icon.h"

GType applets_systray_get_type(void);

#define APPLETS_TYPE_SYSTRAY		(applets_systray_get_type ())
#define APPLETS_SYSTRAY(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), APPLETS_TYPE_SYSTRAY, AppletsSystray))
#define APPLETS_IS_SYSTRAY(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), APPLETS_TYPE_SYSTRAY))
#define APPLETS_SYSTRAY_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST ((klass), APPLETS_TYPE_SYSTRAY, AppletsSystrayClass))
#define APPLETS_IS_SYSTRAY_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), APPLETS_TYPE_SYSTRAY))
#define APPLETS_SYSTRAY_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj), APPLETS_TYPE_SYSTRAY, AppletsSystrayClass))


#define ICON_SCALE_FACTOR  0.8 // for custom panel heights, 20 (default icon size) / 25 (default panel height)

typedef struct _AppletsSystray		AppletsSystray;
typedef struct _AppletsSystrayClass	AppletsSystrayClass;

struct _AppletsSystray
{
  CdosApplet	parent_instance;
  ClutterActor *manager_container;
  gulong signals_added;
  gulong signals_removed;
  gulong signals_redisplay;
  gulong signals_registered;

  GHashTable *role_position_table;
};

struct _AppletsSystrayClass
{
  CdosAppletClass	parent_class;
};

G_DEFINE_TYPE (AppletsSystray, applets_systray, CDOS_TYPE_APPLET);


static void
applets_systray_insert_status_item (AppletsSystray *systray, 
				ClutterActor *actor, 
				gfloat position)
{
  int role_position;
  void *temp = NULL;
  int i = clutter_actor_get_n_children (systray->manager_container) - 1;
  ClutterActor *iter = clutter_actor_get_last_child (systray->manager_container);
  for (;i >= 0 && iter != NULL; i--, iter = clutter_actor_get_previous_sibling (iter))
  {
    if (g_hash_table_contains (systray->role_position_table, iter)) {
      temp = g_hash_table_lookup (systray->role_position_table, iter);
      role_position =  (int)(intptr_t)temp;
      if (position > role_position) {
        clutter_actor_insert_child_at_index (systray->manager_container, actor, i + 1);
        break;
      }
    }
  }

  if (i == -1) {
      /* If we didn't find a position, we must be first */
      clutter_actor_insert_child_at_index (systray->manager_container, actor, 0);
  }
  g_hash_table_insert (systray->role_position_table, actor, (void *)(intptr_t)position);
}

static void
applets_systray_on_tray_icon_removed (CdosIconDispatcher *manager,
                                CdosTrayIcon *icon,
                                gpointer user_data)
{
  AppletsSystray *systray = APPLETS_SYSTRAY (user_data);
  ClutterActor *item = clutter_actor_get_parent (CLUTTER_ACTOR (icon));
  clutter_actor_remove_child (systray->manager_container, item);
  clutter_actor_destroy (CLUTTER_ACTOR (icon));
}

static int
applets_systray_glist_indexof (GSList *list, const gchar *str)
{
  if (list && str) {
    int i;
    GSList *iter = list;
    for (i = 0; iter; i++, iter = iter->next) {
      if (g_strcmp0 (iter->data, str) == 0) {
        return i;
      }
    }
  }
  return -1;
}

#if 0
static gboolean
on_icon_clicked (StButton *self, ClutterEvent *event, gpointer data){
    cdos_debug ();
    CdosTrayIcon *icon = (CdosTrayIcon *)data;
    cdos_tray_icon_click (icon, clutter_get_current_event());
    return FALSE;
}

static double
applets_systray_get_icon_size(AppletsSystray *systray) {

  StThemeNode *themeNode;
  double icon_size = 0;
  StWidget *st_icon = st_icon_new();
  clutter_actor_hide(st_icon);
  clutter_actor_add_child(systray->manager_container, st_icon);
  st_widget_set_style_class_name (st_icon, "applet-systray-icon");
  themeNode = st_widget_get_theme_node(st_icon);
  icon_size = st_theme_node_get_length(themeNode, "icon-size");
  clutter_actor_remove_child(systray->manager_container, st_icon);
  g_object_unref(st_icon);
  return icon_size;
}

static gboolean on_button_release(ClutterActor *actor, ClutterEvent *event, CdosTrayIcon *icon){
  cdos_tray_icon_click(icon,event);
  return FALSE;
}
#endif

static gboolean
applets_systray_on_tray_icon_added (CdosIconDispatcher *manager,
                                CdosTrayIcon *icon,
                                gchar *role,
				gpointer user_data)
{
  AppletsSystray *systray = APPLETS_SYSTRAY (user_data);
  CdosSystrayManager *systray_manager = \
                cdos_monkey_plugin_get_systray_manager (monkey_plugin_get ());
  GList *hidden_icons = cdos_systray_manager_get_roles (systray_manager, NULL);

  if (g_strcmp0 (role, "network") == 0) {
    return TRUE;
  }

  GSList *buggy_icons = NULL;
  buggy_icons = g_slist_append (buggy_icons, "pidgin");
  buggy_icons = g_slist_append (buggy_icons, "thunderbird");
  
  /*g_print ("Adding systray: %s(%fpx%fpx)\n", role, 
			clutter_actor_get_width (CLUTTER_ACTOR (icon)), 
			clutter_actor_get_height (CLUTTER_ACTOR (icon))); //global.log (...);*/

  ClutterActor *parent = clutter_actor_get_parent (CLUTTER_ACTOR (icon));
  if (parent)
    {
      g_object_ref (icon);
      clutter_actor_remove_child (parent, CLUTTER_ACTOR (icon));
      g_object_force_floating (G_OBJECT (icon));
    }
  
  StWidget *item = st_bin_new ();
  st_widget_set_style_class_name (item, "applet-box-systray");
  clutter_actor_set_reactive ((ClutterActor *)item, TRUE);
  st_widget_set_track_hover (item, TRUE);

  st_bin_set_child ((StBin *)item, (ClutterActor *)icon);
  //st_widget_set_can_focus ((StWidget *)item, TRUE);

  //g_signal_connect(CLUTTER_ACTOR(item),"button-release-event", G_CALLBACK (on_button_release),icon);

  st_bin_set_fill ((StBin *)item, TRUE, TRUE);
  clutter_actor_set_y_expand (CLUTTER_ACTOR(item),TRUE);
  st_bin_set_alignment (ST_BIN(item), ST_ALIGN_MIDDLE, ST_ALIGN_MIDDLE);
  clutter_actor_set_x_align (CLUTTER_ACTOR(icon), CLUTTER_ACTOR_ALIGN_CENTER);
  clutter_actor_set_y_align (CLUTTER_ACTOR(icon), CLUTTER_ACTOR_ALIGN_CENTER);

  /*  
  // calculate the scale factor 
  gfloat min_width_p, min_height_p, natural_width_p, natural_height_p;
  gdouble icon_size, scale_x = 1.0, scale_y = 1.0;
  clutter_actor_get_preferred_size(CLUTTER_ACTOR(icon),
                                   &min_width_p,
                                   &min_height_p,
                                   &natural_width_p,
                                   &natural_height_p);
  icon_size = applets_systray_get_icon_size(systray);
  if(natural_width_p != 0 && natural_height_p != 0){
    scale_x = icon_size/natural_width_p;
    scale_y = icon_size/natural_height_p;
  }
  clutter_actor_set_pivot_point(CLUTTER_ACTOR (icon), 0.5, 0.5);
  clutter_actor_set_scale(CLUTTER_ACTOR (icon), scale_x,scale_y);
	*/
  GSettings *settings = cdos_global_get_settings (cdos_global_get ());
  if (g_settings_get_boolean (settings, "panel-scale-text-icons")) {
    gfloat panel_height = cdos_applet_get_panel_height (CDOS_APPLET (systray));
    gfloat disp_size = panel_height * ICON_SCALE_FACTOR;
    if (clutter_actor_get_width (CLUTTER_ACTOR (icon)) == 1 || 
	clutter_actor_get_height (CLUTTER_ACTOR (icon)) == 1 || 
	applets_systray_glist_indexof (buggy_icons, role) != -1) {
      clutter_actor_set_height (CLUTTER_ACTOR (icon), disp_size);
    } else {
      clutter_actor_set_size (CLUTTER_ACTOR (icon), disp_size, disp_size);
    }
  }
  g_slist_free (buggy_icons);

  /* dropbox, for some reason, refuses to provide a correct size icon in our new situation.
   * Tried even with stalonetray, same results - all systray icons I tested work fine but dropbox.  I'm
   * assuming for now it's their problem.  For us, just scale it up.
   */
  if (role && g_strcmp0 ("dropbox", role) == 0) {
    guint ui_scale = cdos_global_get_ui_scale (cdos_global_get ());
    /*
    clutter_actor_set_scale_full (CLUTTER_ACTOR (icon), 
				(gdouble)ui_scale, 
				(gdouble)ui_scale, 
				clutter_actor_get_width (CLUTTER_ACTOR (icon))/2.0, 
				clutter_actor_get_width (CLUTTER_ACTOR (icon))/2.0); 
    */
    clutter_actor_set_pivot_point ((ClutterActor *)icon, 0.5f, 0.5f);
    clutter_actor_set_scale ((ClutterActor *)icon, (gdouble)ui_scale, (gdouble)ui_scale);
  }

  if (g_strcmp0 (role, "unregistered") == 0 || g_strcmp0 (role, "Unregister-indicator.py") == 0)
  {
    st_widget_set_style (item, "width: 37px;");
  }

  applets_systray_insert_status_item (systray, (ClutterActor *)item, -1);
  return FALSE;
}

static void
applets_systray_on_before_redisplay (CdosIconDispatcher *manager, 
				gpointer user_data)
{
  AppletsSystray *systray = APPLETS_SYSTRAY (user_data);
  GList *children = clutter_actor_get_children (systray->manager_container);
  GList *iter;
  for (iter = children; iter; iter = iter->next)
  {
      clutter_actor_destroy (iter->data);
  }
  g_list_free (children);
}

static void
applets_systray_changed_callback (CdosSystrayManager *systray_manager,
				gpointer user_data)
{
  CdosIconDispatcher *status_icon_dispatcher = CDOS_ICON_DISPATCHER(user_data);
  cdos_icon_dispatcher_redisplay (status_icon_dispatcher);
}

static gboolean
applets_systray_on_applet_clicked (CdosApplet *systray)
{
    return TRUE;
}

static gboolean
applets_systray_on_applet_added_to_panel (CdosApplet *applet, gboolean loaded)
{
  AppletsSystray *systray = APPLETS_SYSTRAY (applet);
  CdosIconDispatcher *status_icon_dispatcher = \
                cdos_monkey_plugin_get_status_icon_dispatcher (monkey_plugin_get ());
  if (systray->signals_added == 0)
  {
    systray->signals_added = g_signal_connect (status_icon_dispatcher, 
				  "status-icon-added", 
				  G_CALLBACK(applets_systray_on_tray_icon_added), 
				  systray);
  }
  
  if (systray->signals_removed == 0)
  {
    systray->signals_removed = g_signal_connect (status_icon_dispatcher, 
				  "status-icon-removed", 
				  G_CALLBACK(applets_systray_on_tray_icon_removed), 
				  systray);
  }
  
  if (systray->signals_redisplay == 0)
  {
    systray->signals_redisplay = g_signal_connect (status_icon_dispatcher,
				  "before-redisplay", 
				  G_CALLBACK (applets_systray_on_before_redisplay), 
				  systray);
  }
  
  if (systray->signals_registered == 0)
  {
    systray->signals_registered = g_signal_connect (
				  cdos_monkey_plugin_get_systray_manager (monkey_plugin_get ()), 
				  "changed", 
				  G_CALLBACK (applets_systray_changed_callback), 
				  status_icon_dispatcher);
  }
  return 1;
}

static gboolean
applets_systray_on_applet_removed_from_panel (CdosApplet *tray)
{
  AppletsSystray *systray = APPLETS_SYSTRAY (tray);
  CdosIconDispatcher *status_icon_dispatcher = \
		cdos_monkey_plugin_get_status_icon_dispatcher (monkey_plugin_get ());
  
  if (systray->signals_added > 0)
  {
    g_signal_handler_disconnect (status_icon_dispatcher, systray->signals_added);
    systray->signals_added = 0;
  }
  
  if (systray->signals_removed > 0)
  {
    g_signal_handler_disconnect (status_icon_dispatcher, systray->signals_removed);
    systray->signals_removed = 0;
  }
  
  if (systray->signals_redisplay > 0)
  {
    g_signal_handler_disconnect (status_icon_dispatcher, systray->signals_redisplay);
    systray->signals_redisplay = 0;
  }

  CdosSystrayManager *systray_manager = \
		cdos_monkey_plugin_get_systray_manager (monkey_plugin_get ());
  if (systray->signals_registered > 0)
  {
    g_signal_handler_disconnect (systray_manager, systray->signals_registered);
    systray->signals_registered = 0;
  }
  return 1;
}

static gboolean
applets_systray_on_panel_height_changed ()
{
  cdos_debug ();
  CdosIconDispatcher *status_icon_dispatcher = \
                cdos_monkey_plugin_get_status_icon_dispatcher (monkey_plugin_get ());
  cdos_icon_dispatcher_redisplay (status_icon_dispatcher);
  return 0;
}

static void
applets_systray_finalize (GObject *object) 
{

}

static void
applets_systray_init (AppletsSystray *self)
{
  st_widget_remove_style_class_name ((StWidget *)CDOS_APPLET(self)->actor, "applet-box");
  //st_widget_set_style ((StWidget *)CDOS_APPLET(self)->actor, "spacing: 5px");

  self->role_position_table = g_hash_table_new (NULL, NULL);

  self->signals_added = 0;
  self->signals_removed = 0;
  self->signals_redisplay = 0;
  self->signals_registered = 0;

  guint ui_scale = cdos_global_get_ui_scale (cdos_global_get ());
  ClutterLayoutManager *manager = clutter_box_layout_new ();
  clutter_box_layout_set_spacing ((ClutterBoxLayout *)manager, 2 * ui_scale);
  clutter_box_layout_set_homogeneous ((ClutterBoxLayout *)manager, TRUE);
  clutter_box_layout_set_orientation ((ClutterBoxLayout *)manager, CLUTTER_ORIENTATION_HORIZONTAL);
  
  ClutterActor *manager_container = clutter_actor_new ();
  clutter_actor_set_layout_manager (manager_container, manager);
  self->manager_container = manager_container;

  st_container_add_actor ((ClutterContainer *)CDOS_APPLET(self)->actor, manager_container);
  clutter_actor_show (manager_container);

  cdos_applet_set_popup_menu_active(CDOS_APPLET(self),FALSE);
}

static void
applets_systray_class_init (AppletsSystrayClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  CdosAppletClass *parent_class = CDOS_APPLET_CLASS (klass);

  object_class->finalize = applets_systray_finalize;

  //parent_class->on_applet_clicked = applets_systray_on_applet_clicked;
  parent_class->on_applet_added_to_panel = applets_systray_on_applet_added_to_panel;
  parent_class->on_applet_removed_from_panel = applets_systray_on_applet_removed_from_panel;
  parent_class->on_panel_height_changed = applets_systray_on_panel_height_changed;
}

static CdosApplet *
applets_systray_new (StSide orientation, gfloat panel_height)
{
  AppletsSystray *my_applet = g_object_new (APPLETS_TYPE_SYSTRAY,
				"orientation", orientation,
				"panel-height", panel_height,
				"init-func", TRUE,
				NULL);  
  return (CdosApplet *)my_applet;
}

CdosApplet *
applets_systray_main (MetaData *meta, 
                        StSide orientation,
                        gfloat panel_height)
{
  CdosApplet *my_applet = applets_systray_new (orientation, panel_height);
  
  return my_applet;
}
