//
// Created by zhangtao on 16-4-16.
// source:applets/window-list*/applet.js
// 原类名：MyApplet

/**
 * SECTION: applet-window-list
 * @short_description: 任务栏中的应用窗口列表管理器。
 * @title: AppletWindowList
 * @include: applet-window-list.h
 *
 * #AppletWindowList是为桌面任务栏提供应用窗口列表的相关管理，为用户提供应用的相关打开，关闭，应用切换，窗口预览等操作，同时提供部分应用的快捷启动操作。
 *
 * 应用窗口列表管理器托管在任务栏中部区域，默认靠左侧对齐，功能分为两个部分：1.快捷启动应用列表；2.应用窗口列表。
 *
 * 应用窗口列表管理器在当前工作区的任务栏中唯一存在，生命周期同当前工作区。
 *
 *
 */
#include <stdlib.h>
#include "applet-window-list.h"
#include "cdos-global.h"
#include "applet-app-list.h"
#include "cdos-monkey-plugin.h"
#include "applet-pinned-favs.h"
#include "applet-ordered-hash.h"

typedef struct _AppletWindowListSignalIds AppletWindowListSignalIds;

struct _AppletWindowListSignalIds {
    gulong global_settings_changed_panel_edit_mode_id;
    gulong wm_switch_workspace_id;
    gulong screen_notify_n_workspaces_id;
    gulong overview_showing_id;
    gulong overview_hiding_id;
};
/*-----------------信号枚举-------------------*/
enum {
    LAST_SIGNAL
};
/*-----------------实例私有结构体-------------------*/
struct _AppletWindowListPrivate{
    StSide orientation;
    gboolean drag_in_progress;
    CdosAppletSettings *settings;
    AppletPinnedFavs *pinned_apps_contr;
    AppletOrderedHash *meta_workspaces;
    AppletWindowListSignalIds *signal_ids;
    StWidget *actor;
    gchar *uuid;
    StWidget *box;
};

enum {
    PRO_0,
    PRO_GROUP_APPS,
    PRO_SHOW_PINNED,
    PRO_ARRANGE_PINNED,
    PRO_ENABLE_PEEK,
    PRO_ON_CLICK_THUMBS,
    PRO_PEEK_OPACITY,
    PRO_THUMB_TIMEOUT,
    PRO_THUMB_SIZE,
    PRO_VERTICAL_THUMBS,
    PRO_STACK_THUMBS,
    PRO_SHOW_THUMBS,
    PRO_NUM_DISPLAY,
    PRO_TITLE_DISPLAY,
    PRO_ICON_PADDING,
    PRO_PINNED_APPS
};

/*-------global variables-------------*/
static gint AppletWindowList_private_offset = 0;
static gint applet_window_list_signals[LAST_SIGNAL] = { 0 };
//static CdosGlobal *global = NULL;
static gpointer applet_window_list_parent_class = NULL;

/*-------prototypes---------*/
static void applet_window_list_init(AppletWindowList *self);
static void applet_window_list_class_init(AppletWindowListClass *klass);

static void applet_window_list_dispose(GObject *object);
static void applet_window_list_finalize(GObject *object);
static void applet_window_list_set_property(
        GObject *object,
        guint prop_id,
        const GValue *value,
        GParamSpec  *pspec
);
static void applet_window_list_get_property(
        GObject *object,
        guint prop_id,
        GValue *value,
        GParamSpec  *pspec
);


/*-------------private functions of class-------------------*/
static  AppletWindowList*  applet_window_list_new(MetaData *metadata,StSide orientation,gint panel_height,gchar *instance_id);

static void applet_window_list_on_switch_workspace(CdosBaseWM *cdos_base_wm,
                                                   gint previous_workspace_index,
                                                   gint current_workspace_index,
                                                   MetaMotionDirection direction,
                                                   AppletWindowList *self);

static void applet_window_list_on_workspace_created_or_destroyed(GObject *meta_screen,GParamSpec *pspec,AppletWindowList *self);

static void applet_window_list_on_overview_show(CdosOverview *overview,AppletWindowList *self);

static void applet_window_list_on_overview_hide(CdosOverview *overview,AppletWindowList *self);

static void applet_window_list_on_panel_mode_changed(GSettings *global_settings,gchar *key,AppletWindowList *self);

static void applet_window_list_meta_workspaces_foreach_func(MetaWorkspace *meta_workspace,AppletAppList *app_list,gint index,GList *user_data);

static void applet_window_list_to_delete_func(MetaWorkspace *meta_workspace,AppletOrderedHash *meta_workspaces);


/*-------functions-------------*/

/**
 * 注册类；
 *
 *
 * */
GType
applet_window_list_get_type(void)
{
    static GType applet_window_list_type = 0;
    if(G_UNLIKELY(applet_window_list_type==0))
    {
        static const GTypeInfo applet_window_list_info = {
                sizeof(AppletWindowListClass),
                NULL,NULL,
                (GClassInitFunc)applet_window_list_class_init,
                NULL,NULL,
                sizeof(AppletWindowList),
                0,
                (GInstanceInitFunc)applet_window_list_init
        };
        //类注册
        applet_window_list_type = g_type_register_static(CDOS_TYPE_APPLET,"AppletWindowList",&applet_window_list_info,0);
        //g_type_add_class_private(applet_window_list_type, sizeof(AppletWindowListClassPrivate));
        AppletWindowList_private_offset =
                g_type_add_instance_private(applet_window_list_type, sizeof(AppletWindowListPrivate));

    }
    return applet_window_list_type;
}
/**
 *  获取实例的私有属性结构
 * */
static inline gpointer
applet_window_list_get_instance_private(AppletWindowList *self){
    return (G_STRUCT_MEMBER_P (self, AppletWindowList_private_offset));
}


/**
 * 实例初始化；
 *
 * */
static void
applet_window_list_init(AppletWindowList *self)
{
    AppletWindowListPrivate *priv;
    self->priv=applet_window_list_get_instance_private(self);
    priv=self->priv;
    /*--------------实例初始化（构造实例）-------------*/
    if(global == NULL){
        global = cdos_global_get();
    }

    priv->signal_ids = g_new0(AppletWindowListSignalIds,1);
    
    //applet_window_list_exec_install_language(self);
    //gettext.bindtextdomaim;

    priv->drag_in_progress = FALSE;
    priv->box = st_bin_new();
    StWidget *actor = cdos_applet_get_actor(CDOS_APPLET(self));
    priv->actor = ST_WIDGET(actor);

    clutter_container_add_actor(CLUTTER_CONTAINER(actor),CLUTTER_ACTOR(priv->box));

    GSettings     *global_settings = cdos_global_get_settings(global);
    gboolean panel_edit_mode ;
    panel_edit_mode = g_settings_get_boolean(global_settings,"panel-edit-mode");
    clutter_actor_set_reactive(CLUTTER_ACTOR(actor),panel_edit_mode);

    priv->signal_ids->global_settings_changed_panel_edit_mode_id = \
    g_signal_connect(global_settings,"changed::panel-edit-mode",G_CALLBACK(applet_window_list_on_panel_mode_changed),self);


    priv->pinned_apps_contr = applet_pinned_favs_new(self);
    priv->meta_workspaces = applet_ordered_hash_new();

    CdosBaseWM *global_wm;

    gulong wm_switch_workspace_id,screen_notify_n_workspaces_id,overview_showing_id,overview_hiding_id;
    global_wm = cdos_global_get_window_manager(global);
    //g_object_get(global,"window-manager",&global_wm,NULL);
    wm_switch_workspace_id = g_signal_connect(global_wm,"switch-workspace",G_CALLBACK(applet_window_list_on_switch_workspace),self);
    priv->signal_ids->wm_switch_workspace_id = wm_switch_workspace_id;

    MetaScreen *global_screen = cdos_global_get_screen(global);
    screen_notify_n_workspaces_id = g_signal_connect(G_OBJECT (global_screen),"notify::n-workspaces",G_CALLBACK(applet_window_list_on_workspace_created_or_destroyed),self);
    priv->signal_ids->screen_notify_n_workspaces_id = screen_notify_n_workspaces_id;

    CdosOverview *main_overview = cdos_overview_get ();
    overview_showing_id = g_signal_connect(main_overview,"showing",G_CALLBACK(applet_window_list_on_overview_show),self);
    overview_hiding_id = g_signal_connect(main_overview,"hiding",G_CALLBACK(applet_window_list_on_overview_hide),self);
    priv->signal_ids->overview_showing_id = overview_showing_id;
    priv->signal_ids->overview_hiding_id = overview_hiding_id;
}

/**
 *
 * 类结构初始化；
 *
 * */
static void
applet_window_list_class_init(AppletWindowListClass *klass)
{
    //转换为gobject；
    GObjectClass *gobject_class=G_OBJECT_CLASS(klass);
    g_type_class_adjust_private_offset(klass,&AppletWindowList_private_offset);
    applet_window_list_parent_class = g_type_class_peek_parent(klass);

    /*-------------重写父类的虚方法---------------*/
    //gobject_class->constructed=applet_window_list_constructed;
    gobject_class->dispose=applet_window_list_dispose;
    gobject_class->finalize=applet_window_list_finalize;
    gobject_class->set_property=applet_window_list_set_property;
    gobject_class->get_property=applet_window_list_get_property;
}

/*------------类方法定义-----------*/
static void applet_window_list_dispose(GObject *object){
    AppletAppList *self = APPLET_WINDOW_LIST(object);
    AppletWindowListPrivate *priv = self->priv;
    applet_ordered_hash_destroy(priv->meta_workspaces,TRUE);
    if(priv->actor != NULL){
        clutter_container_remove_actor(CLUTTER_CONTAINER(priv->actor),CLUTTER_ACTOR(priv->box));
    }
    //g_clear_object(&priv->box);
    g_clear_object(&priv->settings);
    g_clear_object(&priv->pinned_apps_contr);
    g_clear_object(&priv->meta_workspaces);

    G_OBJECT_CLASS(applet_window_list_parent_class)->dispose(object);
}

static void applet_window_list_finalize(GObject *object){
    AppletAppList *self = APPLET_WINDOW_LIST(object);
    AppletWindowListPrivate *priv = self->priv;
    g_free(priv->signal_ids);
    G_OBJECT_CLASS(applet_window_list_parent_class)->finalize(object);
}

static void applet_window_list_set_property(
        GObject *object,
        guint prop_id,
        const GValue *value,
        GParamSpec  *pspec
){
    AppletWindowList *self = APPLET_WINDOW_LIST(object);
    AppletWindowListPrivate *priv = self->priv;
    switch (prop_id)
    {
        case PRO_GROUP_APPS:
            //cdos_provider_set_value(CDOS_PROVIDER(priv->settings),"group-apps");
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
            break;
    }
}

static void applet_window_list_get_property(
        GObject *object,
        guint prop_id,
        GValue *value,
        GParamSpec  *pspec
){
    AppletWindowList *self = APPLET_WINDOW_LIST(object);
}

static void applet_window_list_on_switch_workspace(CdosBaseWM *cdos_base_wm,
                                                   gint previous_workspace_index,
                                                   gint current_workspace_index,
                                                   MetaMotionDirection direction,
                                                   AppletWindowList *self){
    AppletWindowListPrivate *priv = self->priv;
    MetaScreen *global_screen = cdos_global_get_screen(global);
    MetaWorkspace *meta_workspace = meta_screen_get_workspace_by_index (global_screen,current_workspace_index);
    MetaWorkspace *prev_meta_workspace = meta_screen_get_workspace_by_index (global_screen,previous_workspace_index);
    AppletAppList *app_list;

    if(previous_workspace_index != current_workspace_index){
        AppletAppList *previous_app_list = applet_ordered_hash_get(priv->meta_workspaces,prev_meta_workspace);
        applet_app_list_remove_pagination(previous_app_list);
        StWidget *previous_app_list_actor = applet_app_list_get_actor(previous_app_list);
        g_object_ref(CLUTTER_ACTOR(previous_app_list_actor));
        clutter_container_remove_actor(CLUTTER_CONTAINER(priv->box),CLUTTER_ACTOR(previous_app_list_actor));
    }
    if(!applet_ordered_hash_contains(priv->meta_workspaces,meta_workspace)){
        app_list = applet_app_list_new(self,meta_workspace,priv->orientation);
        applet_ordered_hash_set(priv->meta_workspaces,meta_workspace,app_list);
    }else{
        app_list = applet_ordered_hash_get(priv->meta_workspaces,meta_workspace);
    }

    StWidget *app_list_actor = applet_app_list_get_actor(app_list);
    clutter_container_add_actor(CLUTTER_CONTAINER(priv->box),CLUTTER_ACTOR(app_list_actor));

    applet_app_list_add_pagination(app_list);
    applet_app_list_refresh_apps(app_list);

}

static void applet_window_list_on_workspace_created_or_destroyed(GObject *meta_screen,GParamSpec *pspec,AppletWindowList *self){

    AppletWindowListPrivate *priv = self->priv;
    MetaScreen *global_screen = cdos_global_get_screen(global);
    int n_workspaces =  meta_screen_get_n_workspaces (global_screen);

    GList *to_delete = NULL;
    GList *window_list_workspaces = NULL;
    window_list_workspaces = applet_ordered_hash_keys(priv->meta_workspaces);

    if(window_list_workspaces == NULL){
        return ;
    }
    while (window_list_workspaces != NULL){
        MetaWorkspace *window_list_workspace = window_list_workspaces->data;
        gint i = 0;
        gint flag = 0;
        for(i;i<n_workspaces;i++){
            MetaWorkspace *meta_workspace = meta_screen_get_workspace_by_index (global_screen,i);
            if(window_list_workspace == meta_workspace){
                flag = 1;
                break ;
            }
        }
        if(flag == 0){
            to_delete = g_list_append(to_delete,window_list_workspace);
        }
        window_list_workspaces = window_list_workspaces->next;
    };
    g_list_foreach(to_delete,applet_window_list_to_delete_func,priv->meta_workspaces);
    g_list_free(to_delete);
    g_list_free(window_list_workspaces);
}

static void applet_window_list_on_overview_show(CdosOverview *overview,AppletWindowList *self){
    AppletWindowListPrivate *priv = self->priv;
    clutter_actor_hide(CLUTTER_ACTOR(priv->actor));
}

static void applet_window_list_on_overview_hide(CdosOverview *overview,AppletWindowList *self){
    AppletWindowListPrivate *priv = self->priv;
    clutter_actor_show(CLUTTER_ACTOR(priv->actor));
}

static void applet_window_list_on_panel_mode_changed(GSettings *global_settings,gchar *key,AppletWindowList *self){
    AppletWindowListPrivate *priv = self->priv;
    gboolean panel_edit_mode = g_settings_get_boolean(global_settings,"panel-edit-mode");
    clutter_actor_set_reactive(CLUTTER_ACTOR(priv->actor),panel_edit_mode);
};



static void applet_window_list_meta_workspaces_foreach_func(MetaWorkspace *meta_workspace,AppletAppList *app_list,gint index,GList *user_data){

    GList *workspaces = g_list_first(user_data)->data;
    GList *to_delete = g_list_last(user_data)->data;

    ///guint index = g_list_index(user_data,meta_workspace);
    if(index == -1){
        applet_app_list_destroy(app_list);
        to_delete = g_list_append(to_delete,meta_workspace);
    }
}

static void applet_window_list_to_delete_func(MetaWorkspace *meta_workspace,AppletOrderedHash *meta_workspaces){
    AppletAppList *app_list = applet_ordered_hash_remove(meta_workspaces,meta_workspace);
    applet_app_list_destroy(app_list);
}


/**
 * 创建一个新的实例对象；
 *
 * */
static AppletWindowList* applet_window_list_new(MetaData *metadata,StSide orientation,gint panel_height,gchar *instance_id)
{
    AppletWindowList *self;
    self = g_object_new(APPLET_TYPE_WINDOW_LIST,
                        "orientation", (int)orientation,
                        "panel-height", (float)panel_height,NULL);

    //cdos_applet_constructor(CDOS_APPLET(self),orientation,panel_height,instance_id);
    AppletWindowListPrivate *priv;
    priv = self->priv;
    priv->uuid = metadata->uuid;
    priv->orientation = orientation;

    priv->settings = cdos_applet_settings_new(G_OBJECT(self),metadata->uuid,instance_id);

    MetaScreen *global_screen = cdos_global_get_screen(global);
    gint active_workspace_index = meta_screen_get_active_workspace_index (global_screen);
    CdosBaseWM *global_wm;
    //g_object_get(global,"window-manager",&global_wm,NULL);
    global_wm = cdos_global_get_window_manager(global);
    applet_window_list_on_switch_workspace(global_wm,0,active_workspace_index,META_MOTION_RIGHT,self);

/*
    cdos_provider_bind_property(
            CDOS_PROVIDER(priv->settings),
            CDOS_SETTINGS_BINDING_DIRECTION_BIDIRECTIONAL,
            "group-apps",
            "groupApps",
            NULL,NULL
    );
    cdos_provider_bind_property(
            CDOS_PROVIDER(priv->settings),
            CDOS_SETTINGS_BINDING_DIRECTION_BIDIRECTIONAL,
            "show-pinned",
            "showPinned",
            NULL,NULL
    );
    cdos_provider_bind_property(
            CDOS_PROVIDER(priv->settings),
            CDOS_SETTINGS_BINDING_DIRECTION_BIDIRECTIONAL,
            "arrange-pinnedApps",
            "arrangePinned",
            NULL,NULL
    );
    cdos_provider_bind_property(
            CDOS_PROVIDER(priv->settings),
            CDOS_SETTINGS_BINDING_DIRECTION_BIDIRECTIONAL,
            "enable-hover-peek",
            "enablePeek",
            NULL,NULL
    );
    cdos_provider_bind_property(
            CDOS_PROVIDER(priv->settings),
            CDOS_SETTINGS_BINDING_DIRECTION_BIDIRECTIONAL,
            "vertical-thumbnails",
            "verticalThumbs",
            NULL,NULL
    );
    cdos_provider_bind_property(
            CDOS_PROVIDER(priv->settings),
            CDOS_SETTINGS_BINDING_DIRECTION_BIDIRECTIONAL,
            "stack-thumbnails",
            "stackTumbs",
            NULL,NULL
    );
    cdos_provider_bind_property(
            CDOS_PROVIDER(priv->settings),
            CDOS_SETTINGS_BINDING_DIRECTION_BIDIRECTIONAL,
            "show-thumbnails",
            "showThumbs",
            NULL,NULL
    );
    cdos_provider_bind_property(
            CDOS_PROVIDER(priv->settings),
            CDOS_SETTINGS_BINDING_DIRECTION_BIDIRECTIONAL,
            "pinned-apps",
            "pinnedApps",
            NULL,NULL
    );
    cdos_provider_bind_property(
            CDOS_PROVIDER(priv->settings),
            CDOS_SETTINGS_BINDING_DIRECTION_IN,
            "onclick-thumbnails",
            "onclickThumbs",
            NULL,NULL
    );
    cdos_provider_bind_property(
            CDOS_PROVIDER(priv->settings),
            CDOS_SETTINGS_BINDING_DIRECTION_IN,
            "hover-peek-opacity",
            "peekOpacity",
            NULL,NULL
    );
    cdos_provider_bind_property(
            CDOS_PROVIDER(priv->settings),
            CDOS_SETTINGS_BINDING_DIRECTION_IN,
            "thumbnail-timeout",
            "thumbTimeout",
            NULL,NULL
    );
    cdos_provider_bind_property(
            CDOS_PROVIDER(priv->settings),
            CDOS_SETTINGS_BINDING_DIRECTION_IN,
            "thumbnail-size",
            "thumbSize",
            NULL,NULL
    );
    cdos_provider_bind_property(
            CDOS_PROVIDER(priv->settings),
            CDOS_SETTINGS_BINDING_DIRECTION_BIDIRECTIONAL,
            "number-display",
            "numDisplay",
            NULL,NULL
    );
    cdos_provider_bind_property(
            CDOS_PROVIDER(priv->settings),
            CDOS_SETTINGS_BINDING_DIRECTION_BIDIRECTIONAL,
            "title-display",
            "titleDisplay",
            NULL,NULL
    );
    cdos_provider_bind_property(
            CDOS_PROVIDER(priv->settings),
            CDOS_SETTINGS_BINDING_DIRECTION_BIDIRECTIONAL,
            "icon-padding",
            "iconPadding",
            NULL,NULL
    );
*/

    if(orientation == ST_SIDE_TOP){
        st_widget_set_style(priv->actor,"margin-top:0;padding-top:0;background:none;");
    }else{
        st_widget_set_style(priv->actor,"margin-bottom:0;padding-bottom:0;background:none;");
    }
    return self;
}
/*-------------public functions--------------*/

AppletPinnedFavs* applet_window_list_spinned_app_contr(AppletWindowList *self){
    g_return_if_fail(APPLET_IS_WINDOW_LIST(self));
    AppletWindowListPrivate *priv = self->priv;
    return priv->pinned_apps_contr;
}
/**
 * applet_window_list_destroy:
 * @self:当前实例对象
 *
 * 销毁应用窗口列表管理器。
 */
void applet_window_list_destroy(AppletWindowList *self){
    g_return_if_fail(APPLET_IS_WINDOW_LIST(self));
    AppletWindowListPrivate *priv = self->priv;

    CdosBaseWM *global_wm;
    g_object_get(global,"window-manager",&global_wm,NULL);
    g_signal_handler_disconnect(global_wm,priv->signal_ids->wm_switch_workspace_id);
    priv->signal_ids->wm_switch_workspace_id = 0;

    MetaScreen *global_screen = cdos_global_get_screen(global);
    g_signal_handler_disconnect(global_screen,priv->signal_ids->screen_notify_n_workspaces_id);
    priv->signal_ids ->screen_notify_n_workspaces_id = 0;

    CdosOverview *main_overview = cdos_overview_get();
    g_signal_handler_disconnect(main_overview,priv->signal_ids->overview_hiding_id);
    g_signal_handler_disconnect(main_overview,priv->signal_ids->overview_showing_id);
    priv->signal_ids->overview_hiding_id = 0;
    priv->signal_ids->overview_showing_id = 0;

    clutter_container_remove_actor(CLUTTER_CONTAINER(priv->actor),CLUTTER_ACTOR(priv->box));

    ClutterActor *applet_actor = CLUTTER_ACTOR(priv->actor);
    clutter_actor_destroy(applet_actor);
    priv->actor = NULL;
    priv->box = NULL;
    applet_ordered_hash_destroy(priv->meta_workspaces,TRUE);
}
gfloat applet_window_list_get_panel_height(AppletWindowList *self){

    gfloat panel_height = cdos_applet_get_panel_height(CDOS_APPLET(self));
    return panel_height;
}

/**
 * applets_window_list_main:
 * @metadata: window-list基本信息，由配置文件读取
 * @orientation: 对齐方式，StSide类型成员
 * @panel_height: 面板高度
 * @instance_id: 当前实例在面板中的位置序号
 *
 * 创建一个应用窗口列表管理器。
 *
 */
CdosApplet* applets_window_list_main(MetaData *metadata,StSide orientation,gfloat panel_height,gchar *instance_id){

    //gint applet_id = atoi(instance_id);
    /* for test */
    if(metadata == NULL){
        g_error("Error: metadata is NULL!");
        //metadata = g_new0(MetaData,1);
        //metadata->description = "Window List with App Grouping and Window Thumbnails!";
        //metadata->uuid = "window-list@cdos.org";
        //metadata->name = "Window List";
        //metadata->version = "2.8.5";
        //metadata->max_instances = 1;
        //metadata->last_edited = 1406393058;
    }
    if(metadata->description == NULL){
        metadata->description = "Window List with App Grouping and Window Thumbnails!";
    }
    if(metadata->name == NULL){
        metadata->name = "Window List";
    }
    if(metadata->uuid == NULL){
        metadata->uuid = "window-list@cdos.org";
    }
    if((int)panel_height == 0){
        panel_height = 50.0;
    }
    AppletWindowList *applet = applet_window_list_new(metadata,orientation,(int)panel_height,instance_id);

    return applet;
}

/**
 * applet_window_list_get_settings:
 * @self: 当前实例对象
 *
 * 获取当前实例对象的配置实例对象，通过该实例对象可以获取，修改配置项，并监听配置文件的变化。
 *
 * Returns: #CdosAppletSettings类型的实例对象。
 */
CdosAppletSettings* applet_window_list_get_settings(AppletWindowList *self){
    g_return_val_if_fail(APPLET_IS_WINDOW_LIST(self),NULL);
    AppletWindowListPrivate *priv = self->priv;
    return priv->settings;
}

gboolean applet_window_list_get_drag_in_progress(AppletWindowList *self){
    g_return_val_if_fail(APPLET_IS_WINDOW_LIST(self),NULL);
    AppletWindowListPrivate *priv = self->priv;
    return  priv->drag_in_progress;
}

void applet_window_list_set_drag_in_progress(AppletWindowList *self,gboolean drag_in_progress){
    g_return_if_fail(APPLET_IS_WINDOW_LIST(self));
    AppletWindowListPrivate *priv = self->priv;
    priv->drag_in_progress = drag_in_progress;
}

AppletPinnedFavs* applet_window_list_get_pinned_apps_contr(AppletWindowList *self){
    g_return_val_if_fail(APPLET_IS_WINDOW_LIST(self),NULL);
    AppletWindowListPrivate *priv = self->priv;
    return priv->pinned_apps_contr;
}
StSide applet_window_list_get_orientation(AppletWindowList *self){
    g_return_val_if_fail(APPLET_IS_WINDOW_LIST(self),ST_SIDE_BOTTOM);
    AppletWindowListPrivate *priv = self->priv;
    return priv->orientation;
}
/**
 * applet_window_list_get_actor:
 * @self:当前实例对象
 *
 * 获取应用窗口列表管理器的最外层容器元素。
 *
 * Returns: StWidget类型的容器元素，该元素来自#CdosApplet的actor。
 */
StWidget* applet_window_list_get_actor(AppletWindowList *self){
    g_return_val_if_fail(APPLET_IS_WINDOW_LIST(self),NULL);
    AppletWindowListPrivate *priv = self->priv;
    return priv->actor;
}
