/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */

#include "config.h"

#include "cdos-embedded-window-private.h"
#include "cdos-global.h"

#include <gdk/gdkx.h>
#include <meta/display.h>
#include <meta/window.h>

enum {
   PROP_0,

   PROP_WINDOW
};

typedef struct _CdosGtkEmbedPrivate CdosGtkEmbedPrivate;

struct _CdosGtkEmbedPrivate
{
  CdosEmbeddedWindow *window;

  ClutterActor *window_actor;
  guint window_actor_destroyed_handler;

  guint window_created_handler;
};

G_DEFINE_TYPE_WITH_PRIVATE (CdosGtkEmbed, cdos_gtk_embed, CLUTTER_X11_TYPE_TEXTURE_PIXMAP);

static void cdos_gtk_embed_set_window (CdosGtkEmbed       *embed,
                                        CdosEmbeddedWindow *window);

static void
cdos_gtk_embed_on_window_destroy (GtkWidget     *object,
                                   CdosGtkEmbed *embed)
{
  cdos_gtk_embed_set_window (embed, NULL);
}

static void
cdos_gtk_embed_on_window_realize (GtkWidget     *widget,
                                   CdosGtkEmbed *embed)
{
  /* Here automatic=FALSE means to use CompositeRedirectManual.
   * That is, the X server shouldn't draw the window onto the
   * screen.
   */
  clutter_x11_texture_pixmap_set_window (CLUTTER_X11_TEXTURE_PIXMAP (embed),
                                         gdk_x11_window_get_xid (gtk_widget_get_window (widget)),
                                         FALSE);
}

static void
cdos_gtk_embed_set_window (CdosGtkEmbed       *embed,
                           CdosEmbeddedWindow *window)
{
  CdosGtkEmbedPrivate *priv = embed->priv;
  MetaDisplay *display = cdos_global_get_display (cdos_global_get ());

  if (priv->window)
    {
      _cdos_embedded_window_set_actor (priv->window, NULL);

      g_object_unref (priv->window);

      clutter_x11_texture_pixmap_set_window (CLUTTER_X11_TEXTURE_PIXMAP (embed),
                                             None,
                                             FALSE);

      g_signal_handlers_disconnect_by_func (embed->priv->window,
                                            (gpointer)cdos_gtk_embed_on_window_destroy,
                                            embed);
      g_signal_handlers_disconnect_by_func (embed->priv->window,
                                            (gpointer)cdos_gtk_embed_on_window_realize,
                                            embed);
    }

  priv->window = window;

  if (priv->window)
    {
      g_object_ref (priv->window);

      _cdos_embedded_window_set_actor (priv->window, embed);

      g_signal_connect (priv->window, "destroy",
                        G_CALLBACK (cdos_gtk_embed_on_window_destroy), embed);

      g_signal_connect (priv->window, "realize",
                        G_CALLBACK (cdos_gtk_embed_on_window_realize), embed);
      if (gtk_widget_get_realized (GTK_WIDGET (priv->window)))
          cdos_gtk_embed_on_window_realize (GTK_WIDGET (priv->window), embed);
    }

  clutter_actor_queue_relayout (CLUTTER_ACTOR (embed));
}

static void
cdos_gtk_embed_set_property (GObject         *object,
                              guint            prop_id,
                              const GValue    *value,
                              GParamSpec      *pspec)
{
  CdosGtkEmbed *embed = CDOS_GTK_EMBED (object);

  switch (prop_id)
    {
    case PROP_WINDOW:
      cdos_gtk_embed_set_window (embed, (CdosEmbeddedWindow *)g_value_get_object (value));
      break;

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

static void
cdos_gtk_embed_get_property (GObject         *object,
                              guint            prop_id,
                              GValue          *value,
                              GParamSpec      *pspec)
{
  CdosGtkEmbed *embed = CDOS_GTK_EMBED (object);
  CdosGtkEmbedPrivate *priv = embed->priv;

  switch (prop_id)
    {
    case PROP_WINDOW:
      g_value_set_object (value, priv->window);
      break;

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

static void
cdos_gtk_embed_get_preferred_width (ClutterActor *actor,
                                     float         for_height,
                                     float        *min_width_p,
                                     float        *natural_width_p)
{
  CdosGtkEmbed *embed = CDOS_GTK_EMBED (actor);
  CdosGtkEmbedPrivate *priv = embed->priv;

  if (priv->window
      && gtk_widget_get_visible (GTK_WIDGET (priv->window)))
    {
      GtkRequisition min_req, natural_req;
      gtk_widget_get_preferred_size (GTK_WIDGET (priv->window), &min_req, &natural_req);

      *min_width_p = min_req.width;
      *natural_width_p = natural_req.width;
    }
  else
    *min_width_p = *natural_width_p = 0;
}

static void
cdos_gtk_embed_get_preferred_height (ClutterActor *actor,
                                      float         for_width,
                                      float        *min_height_p,
                                      float        *natural_height_p)
{
  CdosGtkEmbed *embed = CDOS_GTK_EMBED (actor);
  CdosGtkEmbedPrivate *priv = embed->priv;

  if (priv->window
      && gtk_widget_get_visible (GTK_WIDGET (priv->window)))
    {
      GtkRequisition min_req, natural_req;
      gtk_widget_get_preferred_size (GTK_WIDGET (priv->window), &min_req, &natural_req);

      *min_height_p = min_req.height;
      *natural_height_p = natural_req.height;
    }
  else
    *min_height_p = *natural_height_p = 0;
}

static void
cdos_gtk_embed_allocate (ClutterActor          *actor,
                          const ClutterActorBox *box,
                          ClutterAllocationFlags flags)
{
  CdosGtkEmbed *embed = CDOS_GTK_EMBED (actor);
  CdosGtkEmbedPrivate *priv = embed->priv;
  float wx = 0.0, wy = 0.0, x, y, ax, ay;

  CLUTTER_ACTOR_CLASS (cdos_gtk_embed_parent_class)->
    allocate (actor, box, flags);

  /* Find the actor's new coordinates in terms of the stage (which is
   * priv->window's parent window.
   */
  while (actor)
    {
      clutter_actor_get_position (actor, &x, &y);
      clutter_actor_get_anchor_point (actor, &ax, &ay);

      wx += x - ax;
      wy += y - ay;

      actor = clutter_actor_get_parent (actor);
    }

  _cdos_embedded_window_allocate (priv->window,
                                   (int)(0.5 + wx), (int)(0.5 + wy),
                                   box->x2 - box->x1,
                                   box->y2 - box->y1);
}

static void
cdos_gtk_embed_realize (ClutterActor *actor)
{
  CdosGtkEmbed *embed = CDOS_GTK_EMBED (actor);
  CdosGtkEmbedPrivate *priv = embed->priv;

  _cdos_embedded_window_realize (priv->window);

  CLUTTER_ACTOR_CLASS (cdos_gtk_embed_parent_class)->realize (actor);
}

static void
cdos_gtk_embed_unrealize (ClutterActor *actor)
{
  CdosGtkEmbed *embed = CDOS_GTK_EMBED (actor);
  CdosGtkEmbedPrivate *priv = embed->priv;

  _cdos_embedded_window_unrealize (priv->window);

  CLUTTER_ACTOR_CLASS (cdos_gtk_embed_parent_class)->unrealize (actor);
}

static void
cdos_gtk_embed_dispose (GObject *object)
{
  CdosGtkEmbed *embed = CDOS_GTK_EMBED (object);

  G_OBJECT_CLASS (cdos_gtk_embed_parent_class)->dispose (object);

  cdos_gtk_embed_set_window (embed, NULL);
}

static void
cdos_gtk_embed_class_init (CdosGtkEmbedClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);

  object_class->get_property = cdos_gtk_embed_get_property;
  object_class->set_property = cdos_gtk_embed_set_property;
  object_class->dispose      = cdos_gtk_embed_dispose;

  actor_class->get_preferred_width = cdos_gtk_embed_get_preferred_width;
  actor_class->get_preferred_height = cdos_gtk_embed_get_preferred_height;
  actor_class->allocate = cdos_gtk_embed_allocate;
  actor_class->realize = cdos_gtk_embed_realize;
  actor_class->unrealize = cdos_gtk_embed_unrealize;

  g_object_class_install_property (object_class,
                                   PROP_WINDOW,
                                   g_param_spec_object ("window",
                                                        "Window",
                                                        "CdosEmbeddedWindow to embed",
                                                        CDOS_TYPE_EMBEDDED_WINDOW,
                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
}

static void
cdos_gtk_embed_init (CdosGtkEmbed *embed)
{
  embed->priv = G_TYPE_INSTANCE_GET_PRIVATE (embed, CDOS_TYPE_GTK_EMBED,
                                             CdosGtkEmbedPrivate);

  /* automatic here means whether ClutterX11TexturePixmap should
   * process damage update and refresh the pixmap itself.
   */
  clutter_x11_texture_pixmap_set_automatic (CLUTTER_X11_TEXTURE_PIXMAP (embed), TRUE);
}

/*
 * Public API
 */
ClutterActor *
cdos_gtk_embed_new (CdosEmbeddedWindow *window)
{
  g_return_val_if_fail (CDOS_IS_EMBEDDED_WINDOW (window), NULL);

  return g_object_new (CDOS_TYPE_GTK_EMBED,
                       "window", window,
                       NULL);
}
