/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
#ifndef __CDOS_PERF_LOG_H__
#define __CDOS_PERF_LOG_H__

#include <glib-object.h>
#include <gio/gio.h>

G_BEGIN_DECLS

typedef struct _CdosPerfLog CdosPerfLog;
typedef struct _CdosPerfLogClass CdosPerfLogClass;

#define CDOS_TYPE_PERF_LOG              (cdos_perf_log_get_type ())
#define CDOS_PERF_LOG(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), CDOS_TYPE_PERF_LOG, CdosPerfLog))
#define CDOS_PERF_LOG_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), CDOS_TYPE_PERF_LOG, CdosPerfLogClass))
#define CDOS_IS_PERF_LOG(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), CDOS_TYPE_PERF_LOG))
#define CDOS_IS_PERF_LOG_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), CDOS_TYPE_PERF_LOG))
#define CDOS_PERF_LOG_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), CDOS_TYPE_PERF_LOG, CdosPerfLogClass))

GType cdos_perf_log_get_type (void) G_GNUC_CONST;

CdosPerfLog *cdos_perf_log_get_default (void);

void cdos_perf_log_set_enabled (CdosPerfLog *perf_log,
				 gboolean      enabled);

void cdos_perf_log_define_event (CdosPerfLog *perf_log,
				  const char   *name,
				  const char   *description,
				  const char   *signature);
void cdos_perf_log_event        (CdosPerfLog *perf_log,
				  const char   *name);
void cdos_perf_log_event_i      (CdosPerfLog *perf_log,
				  const char   *name,
				  gint32        arg);
void cdos_perf_log_event_x      (CdosPerfLog *perf_log,
				  const char   *name,
				  gint64        arg);
void cdos_perf_log_event_s      (CdosPerfLog *perf_log,
				  const char   *name,
				  const char   *arg);

void cdos_perf_log_define_statistic (CdosPerfLog *perf_log,
                                      const char   *name,
                                      const char   *description,
                                      const char   *signature);

void cdos_perf_log_update_statistic_i (CdosPerfLog *perf_log,
                                        const char   *name,
                                        int           value);
void cdos_perf_log_update_statistic_x (CdosPerfLog *perf_log,
                                        const char   *name,
                                        gint64        value);

typedef void (*CdosPerfStatisticsCallback) (CdosPerfLog *perf_log,
                                             gpointer      data);

void cdos_perf_log_add_statistics_callback (CdosPerfLog               *perf_log,
                                             CdosPerfStatisticsCallback callback,
                                             gpointer                    user_data,
                                             GDestroyNotify              notify);

void cdos_perf_log_collect_statistics (CdosPerfLog *perf_log);

typedef void (*CdosPerfReplayFunction) (gint64      time,
					 const char *name,
					 const char *signature,
					 GValue     *arg,
                                         gpointer    user_data);

void cdos_perf_log_replay (CdosPerfLog            *perf_log,
			    CdosPerfReplayFunction  replay_function,
                            gpointer                 user_data);

gboolean cdos_perf_log_dump_events (CdosPerfLog   *perf_log,
                                     GOutputStream  *out,
                                     GError        **error);
gboolean cdos_perf_log_dump_log    (CdosPerfLog   *perf_log,
                                     GOutputStream  *out,
                                     GError        **error);

G_END_DECLS

#endif /* __CDOS_PERF_LOG_H__ */
