/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
#ifndef __CDOS_RECORDER_H__
#define __CDOS_RECORDER_H__

#include <clutter/clutter.h>

G_BEGIN_DECLS

/**
 * SECTION:CdosRecorder
 * short_description: Record from a #ClutterStage
 *
 * The #CdosRecorder object is used to make recordings ("screencasts")
 * of a #ClutterStage. Recording is done via #GStreamer. The default is
 * to encode as a Theora movie and write it to a file in the current
 * directory named after the date, but the encoding and output can
 * be configured.
 */
typedef struct _CdosRecorder      CdosRecorder;
typedef struct _CdosRecorderClass CdosRecorderClass;

#define CDOS_TYPE_RECORDER              (cdos_recorder_get_type ())
#define CDOS_RECORDER(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), CDOS_TYPE_RECORDER, CdosRecorder))
#define CDOS_RECORDER_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), CDOS_TYPE_RECORDER, CdosRecorderClass))
#define CDOS_IS_RECORDER(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), CDOS_TYPE_RECORDER))
#define CDOS_IS_RECORDER_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), CDOS_TYPE_RECORDER))
#define CDOS_RECORDER_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), CDOS_TYPE_RECORDER, CdosRecorderClass))

GType              cdos_recorder_get_type     (void) G_GNUC_CONST;

CdosRecorder     *cdos_recorder_new (ClutterStage  *stage);

void               cdos_recorder_set_framerate (CdosRecorder *recorder,
                                                 int framerate);
void               cdos_recorder_set_filename (CdosRecorder *recorder,
						const char    *filename);
void               cdos_recorder_set_pipeline (CdosRecorder *recorder,
						const char    *pipeline);
gboolean           cdos_recorder_record       (CdosRecorder *recorder);
void               cdos_recorder_close        (CdosRecorder *recorder);
void               cdos_recorder_pause        (CdosRecorder *recorder);
gboolean           cdos_recorder_is_recording (CdosRecorder *recorder);

G_END_DECLS

#endif /* __CDOS_RECORDER_H__ */
