/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
#ifndef __CDOS_SCREEN_GRABBER_H__
#define __CDOS_SCREEN_GRABBER_H__

#include <glib-object.h>

G_BEGIN_DECLS

/**
 * SECTION:cdos-screen-grabber
 * @short_description: Grab pixel data from the screen
 *
 * The #CdosScreenGrabber object is used to download previous drawn
 * content to the screen. It internally uses pixel-buffer objects if
 * available, otherwise falls back to cogl_read_pixels().
 *
 * If you are repeatedly grabbing images of the same size from the
 * screen, it makes sense to create one #CdosScreenGrabber and keep
 * it around. Otherwise, it's fine to simply create one as needed and
 * then get rid of it.
 */

typedef struct _CdosScreenGrabber      CdosScreenGrabber;
typedef struct _CdosScreenGrabberClass CdosScreenGrabberClass;

#define CDOS_TYPE_SCREEN_GRABBER              (cdos_screen_grabber_get_type ())
#define CDOS_SCREEN_GRABBER(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), CDOS_TYPE_SCREEN_GRABBER, CdosScreenGrabber))
#define CDOS_SCREEN_GRABBER_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), CDOS_TYPE_SCREEN_GRABBER, CdosScreenGrabberClass))
#define CDOS_IS_SCREEN_GRABBER(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), CDOS_TYPE_SCREEN_GRABBER))
#define CDOS_IS_SCREEN_GRABBER_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), CDOS_TYPE_SCREEN_GRABBER))
#define CDOS_SCREEN_GRABBER_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), CDOS_TYPE_SCREEN_GRABBER, CdosScreenGrabberClass))

GType cdos_screen_grabber_get_type (void) G_GNUC_CONST;

CdosScreenGrabber *cdos_screen_grabber_new  (void);
guchar *            cdos_screen_grabber_grab (CdosScreenGrabber *grabber,
                                               int                 x,
                                               int                 y,
                                               int                 width,
                                               int                 height);

G_END_DECLS

#endif /* __CDOS_SCREEN_GRABBER_H__ */
