/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */

#include "config.h"

#ifdef HAVE_MALLINFO
#include <malloc.h>
#endif
#include <stdlib.h>
#include <string.h>
#include <X11/Xlib.h>
#include <clutter/clutter.h>
#include <clutter/x11/clutter-x11.h>
#include <dbus/dbus-glib.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>
#include <gtk/gtk.h>
#include <glib/gi18n-lib.h>
//#include <girepository.h>
#include <meta/main.h>
#include <meta/meta-plugin.h>
#include <meta/prefs.h>

#include <png.h>
//#include <jpeglib.h>
//#include <jerror.h>

#include "cdos-a11y.h"
#include "cdos-debug.h"
#include "cdos-global.h"
#include "cdos-global-private.h"
#include "cdos-perf-log.h"
#include "cdos_desktop_dbus.h"
#include "st.h"
#include <signal.h>

extern GType cdos_plugin_get_type (void);
extern int debug_mode;
extern int debug_call;

#define CDOS_DBUS_SERVICE "org.Cdos"
#define MAGNIFIER_DBUS_SERVICE "org.cdos.Magnifier"

#define WM_NAME "Cdos"
#define GNOME_WM_KEYBINDINGS "Gnome"
#ifndef u_char
#define u_char unsigned char
#endif		

int get_byte_order (void) {
	union {
		char c[sizeof(short)];
		short s;
	} order;

	order.s = 1;
	if ((1 == order.c[0])) {
		return LSBFirst;
	} else {
		return MSBFirst;
	}
}


/*void jpeg_error_exit (j_common_ptr cinfo) {
	cinfo->err->output_message (cinfo);
	exit (EXIT_FAILURE);
}*/

/*This returns an array for a 24 bit image.*/
/*
u_char *decode_jpeg (char *filename, int *widthPtr, int *heightPtr) {
	register JSAMPARRAY lineBuf;
	struct jpeg_decompress_struct cinfo;
	struct jpeg_error_mgr err_mgr;
	int bytesPerPix;
	FILE *inFile;
	u_char *retBuf;
	inFile = fopen (filename, "rb");
	if (NULL == inFile) { 
		perror (NULL);
		return NULL;
	}

	cinfo.err = jpeg_std_error (&err_mgr);
	err_mgr.error_exit = jpeg_error_exit;	

	jpeg_create_decompress (&cinfo);
	jpeg_stdio_src (&cinfo, inFile);
	jpeg_read_header (&cinfo, 1);
	cinfo.do_fancy_upsampling = 0;
	cinfo.do_block_smoothing = 0;
	jpeg_start_decompress (&cinfo);

	*widthPtr = cinfo.output_width;
	*heightPtr = cinfo.output_height;
	bytesPerPix = cinfo.output_components;

	lineBuf = cinfo.mem->alloc_sarray ((j_common_ptr) &cinfo, JPOOL_IMAGE, (*widthPtr * bytesPerPix), 1);
	retBuf = malloc (3 * (*widthPtr * *heightPtr));
	if (NULL == retBuf) {
		perror (NULL);
		return NULL;
	}

	if (3 == bytesPerPix) {
		int lineOffset = (*widthPtr * 3);
		int x;
		int y;
		for (y = 0; y < cinfo.output_height; ++y) {
			jpeg_read_scanlines (&cinfo, lineBuf, 1);

			for (x = 0; x < lineOffset; ++x) {
				retBuf[(lineOffset * y) + x] = lineBuf[0][x];
				++x;
				retBuf[(lineOffset * y) + x] = lineBuf[0][x];
				++x;
				retBuf[(lineOffset * y) + x] = lineBuf[0][x];
			}
		}
	} else if (1 == bytesPerPix) { 
		unsigned int col;
		int lineOffset = (*widthPtr * 3);
		int lineBufIndex;
		int x ;
		int y;
		for (y = 0; y < cinfo.output_height; ++y) {
			jpeg_read_scanlines (&cinfo, lineBuf, 1);

			lineBufIndex = 0;
			for (x = 0; x < lineOffset; ++x) {
				col = lineBuf[0][lineBufIndex];
				retBuf[(lineOffset * y) + x] = col;
				++x;
				retBuf[(lineOffset * y) + x] = col;
				++x;
				retBuf[(lineOffset * y) + x] = col;
				++lineBufIndex;
			}			
		}
	} else {
		fprintf (stderr, "Error: the number of color channels is %d.  This program only handles 1 or 3\n", bytesPerPix);
		return NULL;
	}
	jpeg_finish_decompress (&cinfo);
	jpeg_destroy_decompress (&cinfo);
	fclose (inFile);
	return retBuf;
}
*/

unsigned char * decode_png(char* path, int *width, int *height)
{
	unsigned char *content;

	int channel, color_type;
	int tmp, size;

	int i, j, pos;

	FILE *file;
	file = fopen (path, "rb");
	if (NULL == file) { 
		g_print("%s path can not be open!", path);
		return NULL;
	}

	png_structp png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, 0, 0, 0);
	png_infop info_ptr = png_create_info_struct(png_ptr);
	setjmp(png_jmpbuf(png_ptr));
	png_init_io(png_ptr, file);
	png_read_png(png_ptr, info_ptr, PNG_TRANSFORM_EXPAND, 0);

	*width = png_get_image_width(png_ptr, info_ptr);
	*height = png_get_image_height(png_ptr, info_ptr);

	
	color_type = png_get_color_type(png_ptr, info_ptr);
	channel = png_get_channels(png_ptr, info_ptr);
	png_bytep* row_pointers = png_get_rows(png_ptr, info_ptr);

	size = *width * *height * 3;
	content = (unsigned char*)malloc(size);

	if (channel == 4 || color_type == PNG_COLOR_TYPE_RGB_ALPHA) {
		tmp = 0;
		for (i = 0; i < *height; i++) {
			for (j = 0; j < *width * 4; j+=4) {
				content[tmp++] = row_pointers[i][j];
				content[tmp++] = row_pointers[i][j+1];
				content[tmp++] = row_pointers[i][j+2];
			}
		}
	} else if (channel == 3 || color_type == PNG_COLOR_TYPE_RGB) {
		tmp = *width * 3;
		for (i = 0; i < *height; i++) {
			for (j = 0; j < tmp; j+=3) {
				content[i*tmp +j+0] = row_pointers[i][j];
				content[i*tmp +j+1] = row_pointers[i][j+1];
				content[i*tmp +j+2] = row_pointers[i][j+2];
			}
		}
	}
	
   	png_destroy_read_struct(&png_ptr, &info_ptr, 0);
    	fclose(file);
	return content;
}

XImage *create_image_from_buffer (Display *dis, int screen, u_char *buf, int width, int height) {
	int depth;
	XImage *img = NULL;
	Visual *vis;
	double rRatio;
	double gRatio;
	double bRatio;
	int outIndex = 0;	
	int i;
	int numBufBytes = (3 * (width * height));

	depth = DefaultDepth (dis, screen);
	vis = DefaultVisual (dis, screen);

	rRatio = vis->red_mask / 255.0;
	gRatio = vis->green_mask / 255.0;
	bRatio = vis->blue_mask / 255.0;

	if (depth >= 24) {
		size_t numNewBufBytes = (4 * (width * height));
		u_int32_t *newBuf = malloc (numNewBufBytes);

		for (i = 0; i < numBufBytes; ++i) {
			unsigned int r, g, b;
			r = (buf[i] * rRatio);
			++i;
			g = (buf[i] * gRatio);
			++i;
			b = (buf[i] * bRatio);

			r &= vis->red_mask;
			g &= vis->green_mask;
			b &= vis->blue_mask;

			newBuf[outIndex] = r | g | b;
			++outIndex;
		}		

		img = XCreateImage (dis, 
				CopyFromParent, depth, 
				ZPixmap, 0, 
				(char *) newBuf,
				width, height,
				32, 0
				);

	} else if (depth >= 15) {
		size_t numNewBufBytes = (2 * (width * height));
		u_int16_t *newBuf = malloc (numNewBufBytes);

		for (i = 0; i < numBufBytes; ++i) {
			unsigned int r, g, b;

			r = (buf[i] * rRatio);
			++i;
			g = (buf[i] * gRatio);
			++i;
			b = (buf[i] * bRatio);

			r &= vis->red_mask;
			g &= vis->green_mask;
			b &= vis->blue_mask;

			newBuf[outIndex] = r | g | b;
			++outIndex;
		}		

		img = XCreateImage (dis,
				CopyFromParent, depth,
				ZPixmap, 0,
				(char *) newBuf,
				width, height,
				16, 0
				);
	} else {
		fprintf (stderr, "This program does not support displays with a depth less than 15.");
		return NULL;				
	}

	XInitImage (img);
	if ((LSBFirst == get_byte_order ())) {
		img->byte_order = LSBFirst;
	} else {
		img->byte_order = MSBFirst;
	}

	img->bitmap_bit_order = MSBFirst;

	return img;
}		

static XImage* cdos_startup_background (void)
{
	int imageWidth;
	int imageHeight;
	XImage *img;		
	int screen;
	Display *dis;
	u_char *buf;

	Screen *default_screen;
	Window root_window;
	GC image_gc;
	time_t timep;
	time(&timep);

	unsigned int src_x = 0,
		     src_y = 0,
		     dest_x = 0,
		     dest_y = 0;

	Window image_window;


	dis = XOpenDisplay (0);
	screen = DefaultScreen(dis);

	default_screen = DefaultScreenOfDisplay(dis);

	root_window = XRootWindow(dis, 0);

	/* decode jpeg */
	//buf = decode_jpeg ("/usr/share/backgrounds/linuxmint-qiana/simon__syon_8684670817.jpg", &imageWidth, &imageHeight);

	buf = decode_png("/usr/share/themes/NFS-Classic/cdos/wallpaper.png",
			&imageWidth, &imageHeight);

	if (!buf) {
		g_print("unable to decode JPEG\n");
		return NULL;
	}

	img = create_image_from_buffer (dis, screen, buf, imageWidth, imageHeight);

	if (!img) {
		g_print("unable to create image\n");
		return NULL;
	}

	/*
	 * create_image_from_buffer creates a new buffer after translation, so we can free.
	 */
	free (buf);

	image_gc = XCreateGC (dis, root_window, 0, NULL);

	XMapRaised (dis, root_window);

	XSelectInput(dis, root_window, ExposureMask | KeyPressMask);

	/* determine image size by width */
	if (imageWidth > WidthOfScreen(default_screen)) {
	/* background image out of bound */
		src_x = (imageWidth - WidthOfScreen(default_screen))>>1;
		src_y = (imageHeight - HeightOfScreen(default_screen))>>1;
	} else {
		dest_x = (WidthOfScreen(default_screen)-imageWidth)>>1;
		dest_y = (HeightOfScreen(default_screen)-imageHeight)>>1;
	}

	XPutImage (dis, root_window, image_gc, img, src_x, src_y, dest_x, dest_y, 
			WidthOfScreen(default_screen), HeightOfScreen(default_screen)) ;
	XFlush (dis);
	return img;
}

/*static void
on_handle_exit_none (Xunli *skeleton,
			GDBusMethodInvocation *invocation,
			guint test){
	GVariant *value;

	g_print ("test screensaver to desktop\n");
	
	xunli_complete_exit_none (skeleton, invocation);
}


static void
on_name_acquired (GDBusConnection *connection,
			const gchar *name,
			gpointer data){
	Xunli  *skeleton;

	skeleton = xunli_skeleton_new ();

	g_signal_connect (skeleton,
			   "handle-exit-none",
			   G_CALLBACK (on_handle_exit_none),
			   NULL);

	g_dbus_interface_skeleton_export (G_DBUS_INTERFACE_SKELETON (skeleton),
					   connection,
					   "/org/Cdos/Xunli",
					   NULL);

} */

static void
cdos_dbus_init (gboolean replace)
{
  GError *error = NULL;
  DBusGConnection *session;
  DBusGProxy *bus;
  guint32 request_name_flags;
  guint32 request_name_result;

  /** TODO:
   * In the future we should use GDBus for this.  However, in
   * order to do that, we need to port all of the JavaScript
   * code.  Otherwise, the name will be claimed on the wrong
   * connection.
   */
  session = dbus_g_bus_get (DBUS_BUS_SESSION, NULL);

  bus = dbus_g_proxy_new_for_name (session,
                                   DBUS_SERVICE_DBUS,
                                   DBUS_PATH_DBUS,
                                   DBUS_INTERFACE_DBUS);

  request_name_flags = DBUS_NAME_FLAG_DO_NOT_QUEUE | DBUS_NAME_FLAG_ALLOW_REPLACEMENT;
  if (replace)
    request_name_flags |= DBUS_NAME_FLAG_REPLACE_EXISTING;
  if (!dbus_g_proxy_call (bus, "RequestName", &error,
                          G_TYPE_STRING, CDOS_DBUS_SERVICE,
                          G_TYPE_UINT, request_name_flags,
                          G_TYPE_INVALID,
                          G_TYPE_UINT, &request_name_result,
                          G_TYPE_INVALID))
    {
      g_printerr ("failed to acquire org.Cdos: %s\n", error->message);
      exit (1);
    }
  if (!(request_name_result == DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER
        || request_name_result == DBUS_REQUEST_NAME_REPLY_ALREADY_OWNER))
    {
      g_printerr ("%s already exists on bus and --replace not specified\n",
                  CDOS_DBUS_SERVICE);
      exit (1);
    }

  // 这个 dbus 提供给 cdos-screensaver 使用，在某些情况下，锁屏无法 grab keyboard 和 mouse
  // 某些情况如下：
  // Alt-F2， popupmenu 等
  // 也就是说，锁屏的时候，需要主动发消息给 cdos desktop，让它退出这几种模式

/*    g_bus_own_name (G_BUS_TYPE_SESSION,
		    "org.Cdos.Xunli",
		    G_BUS_NAME_OWNER_FLAGS_NONE,
		    NULL,
		    on_name_acquired,
		    NULL,
		    NULL,
		    NULL); */

  // 以下均无用
  /* Also grab org.cdos.Panel to replace any existing panel process */
  if (!dbus_g_proxy_call (bus, "RequestName", &error, G_TYPE_STRING,
                          "org.cdos.Panel", G_TYPE_UINT,
                          DBUS_NAME_FLAG_REPLACE_EXISTING | request_name_flags,
                          G_TYPE_INVALID, G_TYPE_UINT,
                          &request_name_result, G_TYPE_INVALID))
    {
      g_print ("failed to acquire org.cdos.Panel: %s\n", error->message);
      exit (1);
    }

  /* ...and the org.cdos.Magnifier service.
   */
  if (!dbus_g_proxy_call (bus, "RequestName", &error,
                          G_TYPE_STRING, MAGNIFIER_DBUS_SERVICE,
                          G_TYPE_UINT, DBUS_NAME_FLAG_REPLACE_EXISTING | request_name_flags,
                          G_TYPE_INVALID,
                          G_TYPE_UINT, &request_name_result,
                          G_TYPE_INVALID))
    {
      g_print ("failed to acquire %s: %s\n", MAGNIFIER_DBUS_SERVICE, error->message);
      /* Failing to acquire the magnifer service is not fatal.  Log the error,
       * but keep going. */
    }

  /* ...and the org.freedesktop.Notifications service; we always
   * specify REPLACE_EXISTING to ensure we kill off
   * notification-daemon if it was running.
   */
  if (!dbus_g_proxy_call (bus, "RequestName", &error,
                          G_TYPE_STRING, "org.freedesktop.Notifications",
                          G_TYPE_UINT, DBUS_NAME_FLAG_REPLACE_EXISTING | request_name_flags,
                          G_TYPE_INVALID,
                          G_TYPE_UINT, &request_name_result,
                          G_TYPE_INVALID))
    {
      g_print ("failed to acquire org.freedesktop.Notifications: %s\n", error->message);
    }

  /* ...and the on-screen keyboard service */
  if (!dbus_g_proxy_call (bus, "RequestName", &error,
                          G_TYPE_STRING, "org.cdos.Caribou.Keyboard",
                          G_TYPE_UINT, DBUS_NAME_FLAG_REPLACE_EXISTING,
                          G_TYPE_INVALID,
                          G_TYPE_UINT, &request_name_result,
                          G_TYPE_INVALID))
    {
      g_print ("failed to acquire org.cdos.Caribou.Keyboard: %s\n", error->message);
    }

  g_object_unref (bus);
}

static void
malloc_statistics_callback (CdosPerfLog *perf_log,
                            gpointer      data)
{
#ifdef HAVE_MALLINFO
  struct mallinfo info = mallinfo ();

  cdos_perf_log_update_statistic_i (perf_log,
                                     "malloc.arenaSize",
                                     info.arena);
  cdos_perf_log_update_statistic_i (perf_log,
                                     "malloc.mmapSize",
                                     info.hblkhd);
  cdos_perf_log_update_statistic_i (perf_log,
                                     "malloc.usedSize",
                                     info.uordblks);
#endif
}

static void
cdos_perf_log_init (void)
{
  CdosPerfLog *perf_log = cdos_perf_log_get_default ();

  /* For probably historical reasons, mallinfo() defines the returned values,
   * even those in bytes as int, not size_t. We're determined not to use
   * more than 2G of malloc'ed memory, so are OK with that.
   */
  cdos_perf_log_define_statistic (perf_log,
                                   "malloc.arenaSize",
                                   "Amount of memory allocated by malloc() with brk(), in bytes",
                                   "i");
  cdos_perf_log_define_statistic (perf_log,
                                   "malloc.mmapSize",
                                   "Amount of memory allocated by malloc() with mmap(), in bytes",
                                   "i");
  cdos_perf_log_define_statistic (perf_log,
                                   "malloc.usedSize",
                                   "Amount of malloc'ed memory currently in use",
                                   "i");

  cdos_perf_log_add_statistics_callback (perf_log,
                                          malloc_statistics_callback,
                                          NULL, NULL);
}

void 
sigroutine(int dunno)
{
	meta_quit(META_EXIT_SUCCESS);
}

static void
muted_log_handler (const char     *log_domain,
                   GLogLevelFlags  log_level,
                   const char     *message,
                   gpointer        data)
{
  /* Intentionally empty to discard message */
}

static gboolean
print_version (const gchar    *option_name,
               const gchar    *value,
               gpointer        data,
               GError        **error)
{
  g_print ("Cinnamon %s\n", VERSION);
  exit (0);
}

static gboolean
set_debug_mode (const gchar    *option_name,
                const gchar    *value,
                gpointer        data,
                GError        **error)
{
    debug_mode = 1;
    return 1;
}

GOptionEntry gnome_cdos_options[] = {
  {
    "version", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
    print_version,
    N_("Print version"),
    NULL
  },
  {
    "debug-backtrace", 'b', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
    set_debug_mode,
    N_("print verbose debug message"),
    NULL
  },
  {
    "debug-function-call", 'f', 0, G_OPTION_ARG_INT,
    &debug_call,
    N_("Print function call stack when crashed."),
    NULL
  },
  { NULL }
};

static void
center_pointer_on_screen ()
{
  Display *dpy;
  Window root_window;
  Screen *screen;

  dpy = XOpenDisplay(0);
  root_window = XRootWindow(dpy, 0);
  XSelectInput(dpy, root_window, KeyReleaseMask);
  screen = DefaultScreenOfDisplay(dpy);
  XWarpPointer(dpy, None, root_window, 0, 0, 0, 0, WidthOfScreen(screen)/2, HeightOfScreen(screen)/2);
  XFlush(dpy);
  cdos_debug ();
}

static void 
backtrace_symbols (int signo)
{
    debug_mode = 1;
    cdos_backtrace ();
    debug_mode = 0;
    exit (0);
}

/* Copy from gmessage.c */
static const gchar *
log_level_to_color (GLogLevelFlags log_level,
                    gboolean       use_color)
{
  /* we may not call _any_ GLib functions here */

  if (!use_color)
    return "";

  if (log_level & G_LOG_LEVEL_ERROR)
    return "\033[1;31m";
  else if (log_level & G_LOG_LEVEL_CRITICAL)
    return "\033[1;35m";
  else if (log_level & G_LOG_LEVEL_WARNING)
    return "\033[1;33m";
  else if (log_level & G_LOG_LEVEL_MESSAGE)
    return "\033[1;32m";
  else if (log_level & G_LOG_LEVEL_INFO)
    return "\033[1;32m";
  else if (log_level & G_LOG_LEVEL_DEBUG)
    return "\033[1;32m";

  /* No color for custom log levels. */
  return "";
}

static gchar *
reset_color (gboolean use_color)
{
    if (!use_color)
        return "";
    else
        return "\033[0m";
}

static void
log_handler (const gchar *domain,
             GLogLevelFlags log_level,
             const gchar *msg,
             gpointer user_data)
{
    if (!debug_mode)
    {
        g_log_default_handler (domain, log_level, msg, NULL);
        return;
    }
    /* log flags */
    /*
       G_LOG_FLAG_RECURSION          = 1 << 0,
       G_LOG_FLAG_FATAL              = 1 << 1,

       G_LOG_LEVEL_ERROR             = 1 << 2,
       G_LOG_LEVEL_CRITICAL          = 1 << 3,
       G_LOG_LEVEL_WARNING           = 1 << 4,
       G_LOG_LEVEL_MESSAGE           = 1 << 5,
       G_LOG_LEVEL_INFO              = 1 << 6,
       G_LOG_LEVEL_DEBUG             = 1 << 7,
    */
    const gchar *msg_level[] = {
        "ERROR",
        "CRITICAL",
        "WARNING",
        "MESSAGE",
        "INFO",
        "DEBUG"
    };
    const char *level = msg_level[2];
    GLogLevelFlags l_level = log_level;
    if (l_level >= G_LOG_LEVEL_ERROR)
    {
        guint i = 0;
        l_level >>= 2;
        while (l_level >>= 1)
            i++;
        level = msg_level [i];
    }

    fprintf (stderr, "\n%s-%s%s%s **: %s\n", 
            domain, log_level_to_color(log_level,1),level, reset_color(1),  msg);

    //debug_mode = log_level & (G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING);

    if (log_level & (G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_ERROR))
        cdos_backtrace ();
    //debug_mode = 0;
}

static void
set_log_handler (void)
{
    const gchar *domains[] = {
        "GLib", "GLib-GObject", "Gtk", "Gdk",
        "GThread", "Pango",
        "St", "Clutter",
        "libnm-glib",
        "Desktop"
    };
    guint i;
    for (i = 0; i < G_N_ELEMENTS(domains); i++)
    {
        g_log_set_handler (domains[i], G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL |
            G_LOG_FLAG_RECURSION, log_handler, NULL);
    }
}

int
main (int argc, char **argv)
{
  clock_t start, end;
  start = clock();

  display_current_time ("main cdos");
  //XImage *img;		
  GOptionContext *ctx;
  GError *error = NULL;
  int ecode;
//  g_setenv ("CLUTTER_DISABLE_XINPUT", "1", TRUE);
  /*
   * 从 glib-2.36 开始，不再需要该函数
   * g_type_init ();
   */

  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
  textdomain (GETTEXT_PACKAGE);

  g_setenv ("GDK_SCALE", "1", TRUE);

  ctx = meta_get_option_context ();
  g_option_context_add_main_entries (ctx, gnome_cdos_options, GETTEXT_PACKAGE);
  if (!g_option_context_parse (ctx, &argc, &argv, &error))
    {
      g_printerr ("%s: %s\n", argv[0], error->message);
      exit (1);
    }

  g_option_context_free (ctx);

  if (debug_call >= 1)
    signal (SIGSEGV, backtrace_symbols);

    set_log_handler ();
  meta_plugin_manager_set_plugin_type (cdos_plugin_get_type ());
  meta_set_wm_name (WM_NAME);
  //cdos_debug ();
  meta_set_gnome_wm_keybindings (GNOME_WM_KEYBINDINGS);

  /* Prevent meta_init() from causing gtk to load gail and at-bridge */
  g_setenv ("NO_AT_BRIDGE", "1", TRUE);
  meta_init ();
  g_unsetenv ("NO_AT_BRIDGE");

  display_current_time ("main cdos after init");
  //cdos_debug ();
  /* FIXME: Add gjs API to set this stuff and don't depend on the
   * environment.  These propagate to child processes.
   */

  g_setenv ("CDOS_VERSION", VERSION, TRUE);


  display_current_time ("main cdos before pointer");
  center_pointer_on_screen();

  display_current_time ("main cdos before background");
//  img = cdos_startup_background();

  cdos_dbus_init (meta_get_replace_current_wm ());
  cdos_a11y_init ();
  /*
  cdos_perf_log_init ();

  g_irepository_prepend_search_path (CDOS_PKGLIBDIR);
#if HAVE_BLUETOOTH
  g_irepository_prepend_search_path (BLUETOOTH_DIR);
#endif
  */

  /* Disable debug spew from various libraries */
  g_log_set_handler ("Gvc", G_LOG_LEVEL_DEBUG,
                     muted_log_handler, NULL);
  g_log_set_handler ("AccountsService", G_LOG_LEVEL_DEBUG,
                     muted_log_handler, NULL);
  g_log_set_handler ("Bluetooth", G_LOG_LEVEL_DEBUG | G_LOG_LEVEL_MESSAGE,
                     muted_log_handler, NULL);

  /* Initialize the global object */
  _cdos_global_init (NULL);

  g_unsetenv ("GDK_SCALE");

  // add a siganl to exit the desktop 
  signal(SIGQUIT, sigroutine);

  display_current_time ("main cdos before meta run");
  ecode = meta_run ();

  g_print("closed cdos-desktop!\n");
//  XDestroyImage(img);

  if (g_getenv ("CDOS_ENABLE_CLEANUP"))
    {
      g_printerr ("Doing final cleanup...\n");
      g_object_unref (cdos_global_get ());
    }

  return ecode;
}
