#include "cdos-applet-manager.h"
#include "cdos-panel.h"
#include "cdos-extension.h"
#include "cdos-monkey-plugin.h"
#include "cdos-key-value.h"
#include "cdos-utils-private.h"
#include "applets/group/applets-group.h"

#include "cdos-global.h"
#include "st.h"

#include <stdio.h>
#include <stdlib.h>

guint menu_flag = 0;
guint calendar_flag = 0;
guint network_flag = 0;
guint removable_flag = 0;
guint sound_flag = 0;
guint systray_flag = 0;
GThreadPool *thread_pool = NULL;
//G_LOCK_DEFINE_STATIC  (thread_pool_lock);

typedef struct _AppletDefinition
{
  CdosPanel *panel;
  StSide orientation;
  StBoxLayout *location;
  gboolean center;
  gint order;
  gchar *uuid;
  gchar *applet_id;
};

typedef struct 
{
  /* the raw from gsettings */
  gchar **raw;
  /* _UuidMap: the each element */
  GHashTable *uuid_map;
  /* AppletDefinition: the each lelment */
  GHashTable *id_map;

} EnabledAppletDefinitions;

/* Maps uuid -> metadata object */
static GHashTable *applet_meta = NULL;
/* Maps uuid -> importer object (applet directory tree) */
static GHashTable *applets = NULL;
/* Maps applet_id -> applet objects */
static GSList *applet_obj = NULL;

static gchar **grouped_applets = NULL;
static gboolean group_loaded = FALSE;
static CdosApplet *applets_group = NULL;
static GHashTable *add_to_group = NULL;

static gboolean applets_loaded = FALSE;

static EnabledAppletDefinitions *enabled_applet_definitions;
static CdosIconDispatcher *status_icon_dispatcher;

static void
cdos_applet_manager_add_applet_to_panels (CdosExtension *extension, 
					AppletDefinition *definition);

static AppletDefinition *
cdos_applet_manager_get_applet_definition (const gchar *definition) 
{
  CdosPanel *panel;
  StSide orientation;
  gint order;
  gboolean center;
  StBoxLayout *location;

  MonkeyPlugin *plugin = monkey_plugin_get ();
  /**
   * format used in gsettings is 'panel:location:order:uuid:applet_id' where:
   * - panel is something like 'panel1'
   * - location is either 'left', 'center' or 'right'
   * - order is an integer representing the order of the applet within the panel/location (i.e. 1st, 2nd etc...)
   * - applet_id is a unique id assigned to the applet instance when added.
   */
  AppletDefinition *ret = g_new0 (AppletDefinition, 1);
  gchar **elements = g_strsplit (definition, ":", -1);

  if (g_strv_length (elements) > 4) 
  {
    if (elements[0] && (g_strcmp0 (elements[0], "panel2") == 0) && cdos_monkey_plugin_get_panel2 (plugin)) {
      panel = cdos_monkey_plugin_get_panel2 (plugin);
    } else {
      panel = cdos_monkey_plugin_get_panel (plugin);
    }

    if (cdos_panel_get_bottom_position (panel)) {
      orientation = ST_SIDE_BOTTOM;
    } else {
      orientation = ST_SIDE_TOP;
    }
    //g_print ("[TEST] %s, elements 2: %s\n", __func__, elements[2]);
    /*int temp = atoi (elements [2]);
    if (temp != 0) {
      order = temp;
    } else {
      order = 0;
    }*/
    order = atoi (elements[2]);

    location = cdos_panel_get_leftbox (panel);

    if (elements[1] && g_strcmp0 ("center", elements[1]) == 0) {
      center = TRUE;
    } else {
      center = FALSE;
    }

    if (center) {
      location = cdos_panel_get_centerbox (panel);
    } else if (elements[1] && g_strcmp0 (elements[1], "right") == 0) {
      location = cdos_panel_get_rightbox (panel);
    }

    ret->panel = panel;
    ret->orientation = orientation;
    ret->location = location;
    ret->center = center;
    ret->order = order;    
    ret->uuid = g_strdup (elements[3]);
    ret->applet_id = g_strdup (elements[4]);
    g_strfreev(elements);
    return ret;
  }

  g_print ("[LOG_ERROR]: Bad applet definition: %s\n", definition);
  return NULL;
}

static gboolean
cdos_applet_manager_check_for_upgrade (char **new_enabled_applets)
{
  int i;
  /* upgrade if old version */
  GSettings *settings = cdos_global_get_settings (cdos_global_get ());
  gint next_applet_id = g_settings_get_int (settings, "next-applet-id");

  for (i = 0; new_enabled_applets[i] != NULL; i ++)
  {
    g_print ("%s, new_enabled_applets[%d]: %s\n", __func__, i, new_enabled_applets[i]);
    gchar **elements = g_strsplit (new_enabled_applets[i], ":", -1);
    if (g_strv_length (elements) == 4) 
    {
      new_enabled_applets[i] = g_strconcat (new_enabled_applets[i], ":", g_strdup_printf ("%d", next_applet_id), NULL);
      next_applet_id ++;
    }
    g_strfreev(elements);
  }

  if (next_applet_id != g_settings_get_int (settings, "next-applet-id")) {
    g_settings_set_int (settings, "next-applet-id", next_applet_id);
    g_settings_set_strv (settings, "enabled-applets", (const gchar *const *)new_enabled_applets);
    return TRUE;
  }
  return FALSE;
}

/**
 * return struct of EnabledAppletDefinition
 */
static EnabledAppletDefinitions *
cdos_applet_manager_get_enabled_applet_definitions (void)
{
  int i, j;
  EnabledAppletDefinitions *result = g_new0 (EnabledAppletDefinitions, 1);

  GSettings *settings = cdos_global_get_settings (cdos_global_get ());
  result->raw = g_settings_get_strv (settings, "enabled-applets");
  result->uuid_map = g_hash_table_new (g_str_hash, g_str_equal);
  result->id_map = g_hash_table_new (g_str_hash, g_str_equal);

  /* Upgrade settings if required */
  cdos_applet_manager_check_for_upgrade (result->raw);

  /* Parse all definitions */
  for (i = 0; result->raw[i] != NULL; i ++) 
  {
    AppletDefinition *applet_definition  = cdos_applet_manager_get_applet_definition (result->raw[i]);
    gchar *uuid, *applet_id;
    if (applet_definition) {
      GList *definitions = NULL;
      uuid = applet_definition->uuid;
      definitions = g_hash_table_lookup (result->uuid_map, applet_definition->uuid);
      if (definitions != NULL) {
        definitions = g_list_append (definitions, applet_definition);
      } else {
        definitions = g_list_append (definitions, applet_definition);
      }
      g_hash_table_insert (result->uuid_map, applet_definition->uuid, definitions);
  
      g_hash_table_insert (result->id_map, applet_definition->applet_id, applet_definition);
    }
  }
  return result;
}

static void
cdos_applet_manager_remove_applet_config_file (gchar *uuid, gchar *instance_id)
{
  const char *home_dir = g_get_home_dir ();
  gchar *config_path = g_strconcat (home_dir, "/", 
				".cdos", "/",
				"configs", "/",
				uuid, "/",
				instance_id, ".json",
				NULL);
  GFile *file = g_file_new_for_path (config_path);
  if (g_file_query_exists (file, NULL)) {
    if (!g_file_delete (file, NULL, NULL)) {
      g_print ("Problem removing applet config file during cleanup.  UUID is %s and filename is %s\n", uuid, config_path);
    }
  }
  g_free(config_path);
  g_object_unref(file);
}

static void
cdos_applet_manager_remove_applet_from_panels (AppletDefinition *applet_definition) 
{
  CdosApplet *applet = cdos_kv_sa_find_by_key (applet_obj, applet_definition->applet_id);

  if (applet) {
    cdos_applet_on_applet_removed_from_panel (applet);

    StBoxLayout *panel_location = cdos_applet_get_panel_location (applet);
    if (panel_location != NULL) {
      StBoxLayout *actor = cdos_applet_get_actor (applet);
      clutter_container_remove_actor ((ClutterContainer *)panel_location, (ClutterActor *)actor);
      panel_location = NULL;
    }

    CdosExtension *extension = cdos_applet_get_extension (applet);
    GHashTable *definitions = cdos_extension_get_loaded_definitions (extension);
    g_hash_table_remove (definitions, applet_definition->applet_id);
    
    cdos_kv_sa_remove_by_key (applet_obj, applet_definition->applet_id);

    cdos_applet_manager_remove_applet_config_file (applet_definition->uuid, applet_definition->applet_id);
  }
}

/**
 * cdos_applet_manager_get_object_for_uuid:
 * @applet_id: gchar * uuid
 *
 * get CdosApplet object for uuid
 *
 * Returns: CdosApplet Object
 */
CdosApplet *
cdos_applet_manager_get_object_for_uuid (gchar *uuid) 
{
  if (uuid != NULL) {
    int i;
    CdosSApplet *sa = NULL;
    gchar *applet_uuid = NULL;
    guint len = g_slist_length (applet_obj);
    for (i = 0; i < len; i ++) {
      sa = g_slist_nth_data (applet_obj, i);
      applet_uuid = cdos_applet_get_uuid (sa->value);
      if (applet_uuid != NULL) {
        if (g_strcmp0 (uuid, applet_uuid) == 0) {
          return sa->value;
        }
      }
    }
  }

  return NULL;
}

/**
 * cdos_applet_manager_get_object_for_instance:
 * @applet_id: gchar * applet_id
 *
 * get CdosApplet object for instance
 *
 * Returns: CdosApplet Object
 */
CdosApplet *
cdos_applet_manager_get_object_for_instance (gchar *applet_id)
{
  if (applet_id != NULL) {
    return cdos_kv_sa_find_by_key (applet_obj, applet_id);
  }
  return NULL;
}

/* Deprecated, kept for compatibility reasons */
static GFile *
cdos_applet_manager_find_applet (gchar *uuid)
{
  return cdos_extension_find_extension_directory (uuid, 
			cdos_extension_get_extension_type (TYPE_APPLET));
}

static void 
cdos_applet_manager_update_app_panel_heights_cb (gpointer key, gpointer value, gpointer user_data)
{
  CdosApplet *applet = NULL;
  gfloat new_height;
  gboolean *force_recalc = user_data;
  applet = cdos_kv_sa_find_by_key (applet_obj, key);
  if (applet) {
    AppletDefinition *applet_definition = value;
    new_height = clutter_actor_get_height ( (ClutterActor *)applet_definition->panel);
    if ((cdos_applet_get_panel_height (applet) != new_height) || *force_recalc) {
      cdos_applet_set_panel_height (applet, new_height);
    }
  }
}

/**
 * cdos_applet_manager_update_applet_panel_heights:
 * @force_recalc: gboolean force_recalc
 *
 * update applet panel heights
 *
 */
void
cdos_applet_manager_update_applet_panel_heights (gboolean force_recalc) 
{
  if (!enabled_applet_definitions)
    return ;

  GHashTable *table = enabled_applet_definitions->id_map;
  g_hash_table_foreach (table, cdos_applet_manager_update_app_panel_heights_cb, &force_recalc);
}

static StBoxLayout *
cdos_applet_manager_get_panel_zone (CdosPanel *panel, gint key)
{
  if (panel) {
    switch (key) {
      case 0:
        return cdos_panel_get_leftbox (panel);
      case 1:
        return cdos_panel_get_centerbox (panel);
      case 2:
        return cdos_panel_get_rightbox (panel);
      default:
        break;
    }
  }
  return NULL;
}

/**
 * cdos_applet_manager_save_applets_positions:
 *
 * Save applets positions
 *
 */
void 
cdos_applet_manager_save_applets_positions (void)
{
  guint i, j, k;
  GSList *panels = NULL;
  gchar *zones_strings[] = {"left", "center", "right"};
  GSList *all_applets = NULL;
  gchar **strv_applets = {NULL, };
  MonkeyPlugin *plugin = monkey_plugin_get ();
  GSettings *settings = cdos_global_get_settings (cdos_global_get ());

  panels = g_slist_append (panels, cdos_monkey_plugin_get_panel (plugin));
  // panels = g_slist_append (panels, cdos_monkey_plugin_get_panel2 (plugin));
  guint pl = g_slist_length (panels);
  for (i = 0; i < pl; i ++) {
    CdosPanel *panel = g_slist_nth_data (panels, i);
    if (!panel) 
      continue;
    for (j = 0; zones_strings[j] != NULL; j++) {
      StBoxLayout *zone = cdos_applet_manager_get_panel_zone (panel, j);
      if (zone) {
        GList *children = st_container_get_children_list ((StContainer *)zone);
        for (k = 0; k < g_list_length (children); k ++) {
          ClutterActor *child = g_list_nth_data (children, k);
          CdosApplet *app = cdos_applet_get_actor_applet (child); // children[k]._applet
          if (app) {
            all_applets = g_slist_append (all_applets, app); // allApplets.push(children[k]._applet);
          }
        }
      }
    }
  }

  for (i = 0; i < pl; i ++) {
    CdosPanel *panel = g_slist_nth_data (panels, i);
    if (!panel)
      continue;
    gchar *panel_string = NULL;
    if (panel == cdos_monkey_plugin_get_panel (plugin)) {
      panel_string = "panel1";
    } else {
      panel_string = "panel2";
    }
    for (j = 0; zones_strings[j] != NULL; j++) {
      StBoxLayout *zone = cdos_applet_manager_get_panel_zone (panel, j);
      for (k = 0; k < g_slist_length (all_applets); k ++) {
        CdosApplet *applet = g_slist_nth_data (all_applets, k);
        StBoxLayout *applet_zone;
        StBoxLayout *new_panel_location = cdos_applet_get_new_panel_location (applet);
        if (new_panel_location != NULL) {
          applet_zone = new_panel_location;
        } else {
          applet_zone = cdos_applet_get_panel_location (applet);
        }
        gint applet_order;
        gint new_order = cdos_applet_get_new_order (applet);
        if (new_order != -1) {
          applet_order = new_order;
        } else {
          applet_order = cdos_applet_get_order (applet);
        }
        if (applet_zone == zone) {
          gchar *str = g_strconcat (panel_string, ":", zones_strings[j], ":", 
				g_strdup_printf ("%d", applet_order), ":", 
				cdos_applet_get_uuid (applet), ":",
				cdos_applet_get_instance_id (applet), NULL);
          strv_applets = cdos_strv_elem_add (strv_applets, str);
        }
      }
    }
  }

  for (i = 0; i < g_slist_length (all_applets); i ++) {
    cdos_applet_set_new_panel_location (g_slist_nth_data (all_applets, i), NULL);
    cdos_applet_set_new_order (g_slist_nth_data (all_applets, i), -1); // -1 instead of null that from js
  }

  g_settings_set_strv (settings, "enabled-applets", (const gchar * const *)strv_applets);
  g_slist_free(all_applets);
  g_slist_free(panels);
  //g_strfreev(zones_strings);
  g_strfreev(strv_applets);
}


/**
 * cdos_applet_manager_remove_applet_from_panel:
 * @obj: CdosApplet Object
 * @event: gchar * event
 * @keep_menu: gchar * keep_menu
 * @uuid: gchar * uuid
 * @applet_id: gchar * applet_id
 *
 * Remove applet from panel
 *
 */
void
cdos_applet_manager_remove_applet_from_panel (CdosApplet *obj, gchar *event,
				gchar *keep_menu, gchar *uuid, gchar *applet_id) 
{
  int i ;
  gchar **enabled_applets = enabled_applet_definitions->raw;
  for (i = 0; enabled_applets[i] != NULL; i ++) {
    AppletDefinition *applet_definition = cdos_applet_manager_get_applet_definition (enabled_applets[i]);
    if (applet_definition) {
      gboolean flag = FALSE;
      if (uuid && applet_id && applet_definition->uuid && applet_definition->applet_id && 
		(g_strcmp0 (applet_definition->uuid, uuid) == 0) &&
		(g_strcmp0 (applet_definition->applet_id, applet_id) == 0)) {
        gchar **new_enabled_applets = g_strdupv (enabled_applets);
        GSettings *settings = cdos_global_get_settings (cdos_global_get ());
        new_enabled_applets = cdos_strv_elem_del_by_nth (new_enabled_applets, i);
        g_settings_set_strv (settings, "enabled-applets", (const gchar * const *)new_enabled_applets);
        g_strfreev(new_enabled_applets);
        flag = TRUE;
      }
      g_free(applet_definition->uuid);
      g_free(applet_definition->applet_id);
      g_free(applet_definition);
      if(flag)
        break;
    }
  }

}

static CdosApplet *
cdos_applet_manager_create_applet (CdosExtension *extension, 
			AppletDefinition *applet_definition)
{
  gchar *applet_id = applet_definition->applet_id;
  StSide orientation = applet_definition->orientation;
  CdosGenericContainer *actor = (CdosGenericContainer *)applet_definition->panel;
  gfloat panel_height = clutter_actor_get_height ((ClutterActor *)actor);

  CdosApplet *applet = cdos_kv_sa_find_by_key (applet_obj, applet_id);
  if (applet != NULL) {
    if (cdos_applet_get_panel_height (applet) != panel_height) {
      cdos_applet_set_panel_height (applet, panel_height);
    }
    cdos_applet_set_orientation (applet, orientation);

    return applet;
  }

  // applet = extension.module.main(extension.meta, orientation, panel_height, applet_id);
  applet = cdos_extension_module_main (extension, orientation, panel_height, applet_id);

  if (NULL == applet) {
    g_print ("Failed to evaluate \'main\' function on applet: %s/%s\n", applet_definition->uuid, applet_definition->applet_id);
    return NULL;
  }

  cdos_kv_sa_remove_by_key (applet_obj, applet_id);
  CdosSApplet *sa = g_new0 (CdosSApplet, 1);
  sa->key = applet_id;
  sa->value = applet;
  applet_obj = g_slist_append (applet_obj, sa);
  cdos_applet_set_uuid (applet, cdos_extension_get_uuid (extension));
  cdos_applet_set_meta (applet, cdos_extension_get_meta (extension));
  cdos_applet_set_instance_id (applet, applet_id);

  return applet;
}

/**
 * cdos_applet_manager_get_role_provider:
 * @role: gcahr * role
 *
 * Returns: CdosApplet Object
 */
CdosApplet *
cdos_applet_manager_get_role_provider (gchar *role) 
{
  CdosExtensionType *type = cdos_extension_get_extension_type (TYPE_APPLET); // Extension.Type.APPLET
  CdosExtension *extension = cdos_kv_se_find_by_key (type->roles, role); // Extension.Type.APPLET.roles[role]
  g_object_unref(type);
  if (extension) {
    return cdos_extension_get_role_provider (extension);
  }
  return NULL;
}

/**
 * cdos_applet_manager_get_provider_exists:
 * @role: gcahr * role
 *
 * get provider exists
 *
 * Returns: gboolean
 */
gboolean 
cdos_applet_manager_get_provider_exists (gchar *role) 
{
  if (cdos_applet_manager_get_role_provider (role) != NULL)
    return TRUE;
  return FALSE;
}

static gboolean
cdos_applet_manager_in_grouped_applets (gchar *uuid)
{
  guint i;
  for (i = 0; i < g_strv_length (grouped_applets); i ++) {
    if (uuid && g_strcmp0 (grouped_applets[i], uuid) == 0) {
      return TRUE;
    }
  }
  return FALSE;
}

static void
cdos_applet_manager_load_group_applets (gpointer key, gpointer value, gpointer user_data)
{
  cdos_applet_manager_add_applet_to_panels (key, value);
}

static void
cdos_applet_manager_add_applet_to_panels (CdosExtension *extension, AppletDefinition *definition)
{
//  if (cdos_extension_lock_role (extension, NULL))
//     return ;


  /* create the applet */
  gchar *uuid = cdos_extension_get_uuid (extension);
  if (g_strcmp0 (uuid, "systray@cdos.org") == 0){
	systray_flag ++;
	g_print ("(%s %d) systray times %d\n",  __func__, __LINE__, systray_flag);
    if (systray_flag > 1) {
        return;
    }
  }
  CdosApplet *applet = cdos_applet_manager_create_applet (extension, definition);
  if (applet == NULL)
    return ;

  /* for grouped-applets */
  if (cdos_applet_manager_in_grouped_applets (definition->uuid)) {
    if (group_loaded) {
      if (applets_group == NULL) {
        group_loaded = FALSE;
      } else {
        // if (!cdos_extension_lock_role (extension, applet)) {
          // return ;
        cdos_applet_set_order (applet, definition->order); // 此order并不是针对grouped-applets的，并不是必要
        cdos_applet_set_extension (applet, extension);
        StBoxLayout *org_location = cdos_applet_get_panel_location (applet);
        if (org_location != NULL) {
          st_container_remove_actor (CLUTTER_CONTAINER(org_location), (ClutterActor *)cdos_applet_get_actor (applet));
          cdos_applet_set_panel_location (applet, NULL);
        }
        // CDOS_APPLET_CLASS(G_OBJECT_GET_CLASS(applets_group))->add_other_applet (applets_group, cdos_applet_get_actor (applet));
        applet_group_add_applet ((AppletGroup *)applets_group, CDOS_APPLET(applet));
//        cdos_applet_set_panel_location (applet, definition->location);
        GHashTable *loaded_definitions = cdos_extension_get_loaded_definitions (extension);
        if (!loaded_definitions) {
          loaded_definitions = g_hash_table_new (g_str_hash, g_str_equal);
          cdos_extension_set_loaded_definitions (extension, loaded_definitions);
        }
        g_hash_table_insert (loaded_definitions, definition->applet_id, definition);
        CDOS_APPLET_CLASS(G_OBJECT_GET_CLASS(applet))->on_applet_added_to_panel(applet, applets_loaded);
        return ;
      }
    } 
    if (!group_loaded) {
      if (add_to_group == NULL) {
        add_to_group = g_hash_table_new (g_direct_hash, g_direct_equal);
      }
      g_hash_table_insert (add_to_group, extension, definition);
      return ;
    }
  } // code for grouped-applets 与下文代码有重复，后续可进行优化整合，目前保证算法逻辑正常，以供测试


  /* Now actually lock the applets role and set the provider */
  // if (!cdos_extension_lock_role (extension, applet)) {
  //   return ;
  // }
 
  cdos_applet_set_order (applet, definition->order);
  cdos_applet_set_extension (applet, extension);

  /* Remove it from its previous panel location (if it had one) */
  StBoxLayout *panel_location = cdos_applet_get_panel_location (applet);
  if (panel_location != NULL) {
    st_container_remove_actor ((ClutterContainer *)panel_location, (ClutterActor *)cdos_applet_get_actor (applet));
    cdos_applet_set_panel_location (applet, NULL);
  }
  /* Add it to its new panel location */
  StBoxLayout *location = definition->location;
  GList *children = st_container_get_children_list (ST_CONTAINER(location));
  GSList *applets_to_move = NULL;
  gint i ;
  // g_print ("%s, uuid: %s, location name: %s\n", __func__, cdos_applet_get_uuid (applet), clutter_actor_get_name (CLUTTER_ACTOR (location)));
  for (i = 0; i < g_list_length (children); i ++) {
    ClutterActor *child = g_list_nth_data (children, i);
    // CdosApplet *app = NULL; // child._applet
    CdosApplet *app = cdos_applet_get_actor_applet (child); // child._applet
    gchar *uuid = cdos_applet_get_uuid (app);
    // if ((typeof child._applet !== "undefined") && (child._applet instanceof Applet.Applet))
    if (app && CDOS_IS_APPLET (app)) {
      if (definition->order < cdos_applet_get_order (app)) {
        if (g_strcmp0 (uuid, "systray@cdos.org") !=0 ){ 
            applets_to_move = g_slist_append (applets_to_move, child); // appletsToMove.push(child);
        } else {
            g_print ("do not remove systray\n");
        }
      }
    }
  }

  for (i = 0; i < g_slist_length (applets_to_move); i ++) {
    st_container_remove_actor ((ClutterContainer *)location, g_slist_nth_data (applets_to_move, i));
  }
  
  StBoxLayout *applet_actor = cdos_applet_get_actor (applet); // applet.actor
  if (definition->center) {
    // appletDefinition.location.add(applet.actor, {x_align: St.Align.CENTER_SPECIAL});
    st_box_layout_insert_actor (location, (ClutterActor *)applet_actor, ST_ALIGN_CENTER_SPECIAL);
  } else {
    st_container_add_actor (CLUTTER_CONTAINER (location), CLUTTER_ACTOR (applet_actor));
  }

  cdos_applet_set_panel_location (applet, location);

  for (i = 0; i < g_slist_length (applets_to_move); i ++) {
    st_container_add_actor (CLUTTER_CONTAINER (location), CLUTTER_ACTOR (g_slist_nth_data (applets_to_move, i)));
  }

  GHashTable *loaded_definitions = cdos_extension_get_loaded_definitions (extension);
  if (!loaded_definitions) {
    loaded_definitions = g_hash_table_new (g_str_hash, g_str_equal);
    cdos_extension_set_loaded_definitions (extension, loaded_definitions);
  }
  g_hash_table_insert (loaded_definitions, definition->applet_id, definition);
  
  CDOS_APPLET_CLASS(G_OBJECT_GET_CLASS(applet))->on_applet_added_to_panel(applet, applets_loaded);
  g_slist_free(applets_to_move);
  if (g_strcmp0 (definition->uuid, "grouped-applets@cdos.org") == 0) {
    applets_group = (CdosApplet *)applet;
    group_loaded = TRUE;
// 这里为什么要遍历再添加？ group applet 的添加走的是 group 自己的 add other 接口，不需要走这里
//    g_hash_table_foreach (add_to_group, cdos_applet_manager_load_group_applets, NULL);
  }
}

static gboolean
cdos_applet_manager_applet_definitions_equal (AppletDefinition *a, AppletDefinition *b) 
{
  if ((a->panel == b->panel) && 
	(a->orientation == b->orientation) &&
	(a->location == b->location) &&
	(a->order == b->order)) {
    return TRUE;
  }
  return FALSE;
}

/**
 * cdos_applet_manager_finish_extension_load:
 * @extension: CdosExtension Object
 *
 * Returns: gboolean
 */
gboolean 
cdos_applet_manager_finish_extension_load (CdosExtension *extension)
{
  guint i;
  GList *definitions = NULL;

  g_print  (cdos_extension_get_uuid (extension));
  display_current_time (""); 
  gchar *uuid = cdos_extension_get_uuid (extension);
  /* add all applet instances for this extension */
  definitions = g_hash_table_lookup (enabled_applet_definitions->uuid_map, cdos_extension_get_uuid (extension));
  if (definitions) {
    for (i = 0; i < g_list_length (definitions); i++) {
/*	if (menu_flag != 0 && g_strcmp0 (uuid, "menu@cdos.org") == 0) {
		return FALSE;	
  	} else if (calendar_flag != 0 && g_strcmp0 (uuid, "calendar@cdos.org") == 0){
		return FALSE;	
  	} else if (network_flag != 0 && g_strcmp0 (uuid, "network@cdos.org") == 0){
		return FALSE;	
  	} else if (sound_flag != 0 && g_strcmp0 (uuid, "sound@cdos.org") == 0){
		return FALSE;	
  	} */


        cdos_applet_manager_add_applet_to_panels (extension, g_list_nth_data (definitions, i));
    }
  }
 
  display_current_time (cdos_extension_get_uuid (extension));
  return TRUE;
}

static void
cdos_applet_manager_remove_all_applets_hash_cb (gpointer key, 
						gpointer value, 
						gpointer user_data)
{
  cdos_applet_manager_remove_applet_from_panels (value);
}

/**
 * cdos_applet_manager_prepare_extension_unload:
 * @extension: CdosExtension Object
 *
 * Remove all applet instances for this extension
 *
 */
void 
cdos_applet_manager_prepare_extension_unload (CdosExtension *extension)
{
  /* Remove all applet instances for this extension */
  GHashTable *loaded_definitions = cdos_extension_get_loaded_definitions (extension);
  g_hash_table_foreach (loaded_definitions, 
			cdos_applet_manager_remove_all_applets_hash_cb, 
			NULL);
}

static void 
cdos_applet_manager_remove_all_cb (gpointer key, gpointer value, gpointer user_data)
{
  EnabledAppletDefinitions *new_enabled_applet_definitions = user_data;
  if (!g_hash_table_lookup (new_enabled_applet_definitions->id_map, key)) {
    cdos_applet_manager_remove_applet_from_panels (value);
  }
}

static void
cdos_appplet_manager_unload_all_cb (gpointer key, gpointer value, gpointer user_data)
{
  EnabledAppletDefinitions *new_enabled_applet_definitions = user_data;
  if (!g_hash_table_lookup (new_enabled_applet_definitions->uuid_map, key)) {
    cdos_extension_unload_extension (key);
  }
}

static void
cdos_applet_manager_add_or_move_cb (gpointer key, gpointer value, gpointer user_data)
{
  AppletDefinition *new_def = value;
  AppletDefinition *old_def = g_hash_table_lookup (enabled_applet_definitions->id_map, key);
  if (!old_def || !cdos_applet_manager_applet_definitions_equal (new_def, old_def)) {
    CdosExtension *extension = cdos_extension_get_object_by_uuid (new_def->uuid);
    if (extension) {
      cdos_applet_manager_add_applet_to_panels (extension, new_def);
    }
  }
}

static void
cdos_applet_manager_make_sure_cb (gpointer key, gpointer value, gpointer user_data)
{
  cdos_extension_load_extension (key, user_data);
}

static void
cdos_applet_manager_on_enabled_applets_changed (void)
{
  EnabledAppletDefinitions *new_enabled_applet_definitions = 
		cdos_applet_manager_get_enabled_applet_definitions ();

  /* Remove all applet instances that do not exist in the definition anymore */
  g_hash_table_foreach (enabled_applet_definitions->id_map, 
			cdos_applet_manager_remove_all_cb, 
			new_enabled_applet_definitions);

  /* Unload all applet extensions that do not exist in the definiiton anymore */
  g_hash_table_foreach (enabled_applet_definitions->uuid_map, 
			cdos_appplet_manager_unload_all_cb, 
			new_enabled_applet_definitions);

  /* Add or move applet instances of already loaded applet extensions */
  g_hash_table_foreach (new_enabled_applet_definitions->id_map, 
			cdos_applet_manager_add_or_move_cb, 
			NULL); 

  enabled_applet_definitions = new_enabled_applet_definitions;

  /* Make sure all applet extensions are loaded.
   * Once loaded, the applets will add themselves via finishExtenisonLoad.
   * here via this (cdos_applet_manager_finish_extension_load).
   */
  CdosExtensionType *type_applet = cdos_extension_get_extension_type (TYPE_APPLET);
  g_hash_table_foreach (enabled_applet_definitions->uuid_map, 
			cdos_applet_manager_make_sure_cb, 
			type_applet);

  //CdosIconDispatcher * status_icon_dispatcher = cdos_monkey_plugin_get_status_icon_dispatcher (monkey_plugin_get ());
  cdos_icon_dispatcher_redisplay (status_icon_dispatcher);
  g_object_unref(type_applet);
}

static gchar **
cdos_applet_manager_get_grouped_applets ()
{
  GSettings *settings = cdos_global_get_settings (cdos_global_get ());
  return g_settings_get_strv (settings, "grouped-applets");
}

static void
cdos_applet_manager_on_grouped_applets_changed (GSettings *settings, gchar *key,gpointer user_data)
{
  grouped_applets = cdos_applet_manager_get_grouped_applets ();

  gchar **enabled_applets_tmp = g_settings_get_strv (settings, "enabled-applets");
  /* 暂且通过cdos_applet_manager_on_enabled_applets_changed 来做grouped-applets的显示更新（实际上是更新panel） */
//  cdos_applet_manager_on_enabled_applets_changed ();
  applet_group_remove_all_applets((AppletGroup *)applets_group);
  int i = 0;

  for (i = 0; enabled_applets_tmp[i] != NULL; i ++)
  {
    AppletDefinition *applet_definition  = cdos_applet_manager_get_applet_definition (enabled_applets_tmp[i]);
    if (applet_definition) {
        CdosExtension *extension = cdos_extension_get_object_by_uuid (applet_definition->uuid);
        if (extension) {
            cdos_applet_manager_add_applet_to_panels(extension, applet_definition);
        }

        g_free(applet_definition->uuid);
        g_free(applet_definition->applet_id);
        g_free(applet_definition);

    }
  }
  g_strfreev(enabled_applets_tmp);

}

static gboolean
cdos_applet_manager_finish_menu_load (CdosIconDispatcher *dispatcher, ClutterActor *applet, gchar *role,
											gpointer userdata){
	CdosExtension *extension = (CdosExtension *) userdata;
	gchar *uuid;

	if (extension){
		uuid = cdos_extension_get_uuid (extension);
	}else {
		return FALSE;
	}

	// 防止重复添加
	g_print ("(%s %d) role name is :%s\n", __func__, __LINE__, role);
	if (g_strcmp0 (role, "fcitx") == 0){
  		if (sound_flag == 0 && g_strcmp0 (uuid, "sound@cdos.org") == 0){
			cdos_applet_manager_finish_extension_load (extension);
  		}
	}else if (g_strcmp0 (role, "network") == 0){
		if (network_flag == 0 && g_strcmp0 (uuid, "network@cdos.org") == 0){
			cdos_applet_manager_finish_extension_load (extension);
		}
	}else {
		return FALSE;
	}
	return TRUE;
}

typedef struct _threadData{
	gpointer *key;
	gpointer *type;
} ThreadData;

static void load_applet_in_thread (gpointer data, gpointer user_data){
	// 如果没理解错误的话，这个 data 是 pool_push 传递的
	// 这个 user_data 是 pool_new 传入的
	ThreadData *applet_info = (ThreadData *)data;

//  	G_LOCK (thread_pool_lock);
	CdosExtension *extension = cdos_extension_load_extension (applet_info->key, applet_info->type);
	// 返回 extension 对象
	//g_thread_exit (extension);
//  	G_UNLOCK (thread_pool_lock);
}

static void
cdos_applet_manager_load_all_extensions_cb (gpointer key, gpointer value, gpointer user_data)
{
// 如果遍历所有的 applet，每个 applet 都单开一个线程来配额哦会怎样？
// 速度快了，但是右边的顺序错乱了
// 用个线程池试试

  CdosExtensionType *type = CDOS_EXTENSION_TYPE (user_data);
  ThreadData *for_thread = g_malloc0 (sizeof (ThreadData));
  for_thread->key = key;
  for_thread->type = type;

  gchar *uuid = (gchar *) key;

  g_thread_pool_push (thread_pool, (gpointer) for_thread, NULL);
//  GThread *applet_thread = g_thread_new ("load_applet", load_applet_in_thread, for_thread);
//  CdosExtension *extension = cdos_extension_load_extension (key, type);

/*  if ( g_strcmp0 (uuid, "network@cdos.org") == 0 
			 || g_strcmp0 (uuid, "sound@cdos.org") == 0){
	// 等待线程结束获取返回值
	CdosExtension *extension = g_thread_join (applet_thread);
  	  g_signal_connect (status_icon_dispatcher,  "status-icon-added", G_CALLBACK (cdos_applet_manager_finish_menu_load), extension);
	
  } */ 
}

/**
 * cdos_applet_manager_init:
 *
 * init for applet manager
 *
 */
void cdos_applet_manager_init (void)
{
  GSettings *settings = cdos_global_get_settings (cdos_global_get ());

  applet_meta = cdos_extension_get_extension_meta (); // appletMeta = Extension.meta;
  applets = cdos_extension_get_import_objects ();
  grouped_applets = cdos_applet_manager_get_grouped_applets ();
  status_icon_dispatcher =
                cdos_monkey_plugin_get_status_icon_dispatcher (monkey_plugin_get ());


  applets_loaded = FALSE;

  /* Load all applet extensions, the applets themselves will be added in finishExtension load */
  enabled_applet_definitions = cdos_applet_manager_get_enabled_applet_definitions ();
  CdosExtensionType *type_applet = cdos_extension_get_extension_type (TYPE_APPLET);
  display_current_time ("===== cdos applet manager init before load all extension");

// 应该在遍历之前新建线程池，在遍历中 pool_push, 恩…… 有道理
  thread_pool = g_thread_pool_new ((GFunc) load_applet_in_thread, NULL, 20, TRUE, NULL);

  g_hash_table_foreach (enabled_applet_definitions->uuid_map, 
			cdos_applet_manager_load_all_extensions_cb,
			type_applet);

  g_thread_pool_free (thread_pool, 0, 1);
  display_current_time ("===== cdos applet manager init after load all extension");

  applets_loaded = TRUE;

  g_signal_connect (settings, "changed::enabled-applets", G_CALLBACK (cdos_applet_manager_on_enabled_applets_changed), NULL);
  g_signal_connect (settings, "changed::grouped-applets", G_CALLBACK (cdos_applet_manager_on_grouped_applets_changed), NULL);
  g_object_unref(type_applet);
}
