#include "cdos-applet-settings.h"
#include "cdos-extension.h"
#include "cdos-extension-type.h"

/**
 * SECTION: cdos-applet-settings
 * @short_description: 为小应用提供额外的配置项操作，该类配置由配置文件中读取和写入。
 * @title: CdosAppletSettings
 * @include: cdos-applet-settings.h
 *
 * #CdosAppletSettings是CdosProvider的具体实现，为小应用扩展了有关配置文件方面的操作和管理。
 */

#define CDOS_APPLET_SETTINGS_GET_PRIVATE(obj)    \
        (G_TYPE_INSTANCE_GET_PRIVATE((obj), CDOS_TYPE_APPLET_SETTINGS, CdosAppletSettingsPrivate))

G_DEFINE_TYPE(CdosAppletSettings, cdos_applet_settings, CDOS_TYPE_PROVIDER);

static void
cdos_applet_settings_class_init(CdosAppletSettingsClass *klass)
{
        /*nothing*/
}

static void
cdos_applet_settings_init(CdosAppletSettings *settings)
{
        /*nothing*/
}
/**
 * cdos_applet_settings_new:
 * @xlet: 当前小应用实例对象
 * @uuid: 当前小应用uuid
 * @instance_id: 当前小应用的实例id
 *
 * 为小应用的创建配置文件对象，提供有关配置项的操作和管理。
 *
 * Returns: 返回一个#CdosAppletSettings实例对象。
 */
CdosAppletSettings *
cdos_applet_settings_new(GObject *xlet, const gchar *uuid, const gchar *instanceId)
{
        GObject *obj;       
        obj = g_object_new(CDOS_TYPE_APPLET_SETTINGS, NULL);
        CdosExtensionType *type = cdos_extension_get_extension_type(TYPE_APPLET);
        cdos_provider_set_params(obj, xlet, uuid, instanceId, type, "Applet");
        g_print("debug:%s\n",__func__);
        return obj;
}
/**
 * cdos_applet_settings_get_is_multi_instance_xlet：
 * @settings: 当前实例对象
 * @uuid: 小应用uuid
 *
 * 判断某个小应用是否存在多个实例。
 *
 * Returns: TRUE-存在多个，FALSE-一个或没有；
 */
gboolean 
cdos_applet_settings_get_is_multi_instance_xlet(CdosAppletSettings *settings,
                                                                                const gchar *uuid)
{
        return cdos_extension_get_max_instances(uuid) > 1;
}



