
/**
 * SECTION: cdos-applet
 * @short_description: 小应用开发的抽象基类
 * @title: CdosApplet
 * @include: cdos-applet.h
 *
 * #CdosApplet为小应用开发提供抽象基类,该基类对存储了小应用的基本信息以及提供了对停泊元素(如#CdosPanel)方面的相关操作。
 *
 * #CdosApplet只能被继承后使用，不推荐实例化后使用。
 *
 *
 */
#include <gio/gio.h>
//#include <girepository.h>
#include "config.h"

#include "cdos-applet.h"
#include "cdos.h"
#include "cdos-start.h"
#include "cdos-popup.h"
#include "cdos-panel-item-tooltip.h"
#include <string.h>
#include <math.h>
#include "cdos-app.h"
#include "cdos-window-tracker.h"
#include <meta/window.h>
#include <meta/workspace.h>

G_DEFINE_TYPE (CdosApplet, cdos_applet, G_TYPE_OBJECT);

#define CDOS_APPLET_GET_PRIVATE(obj) \
        (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CDOS_TYPE_APPLET, CdosAppletPrivate))

enum {
    APPLET_CLICKED, //点击 applet 的信号，子类集成并实现
    LAST_SIGNAL
};

static guint applet_signals [LAST_SIGNAL] = {0,};
struct _CdosAppletPrivate
{
	StBoxLayout   *panelLocation;
	StBoxLayout   *newPanelLocation;
	guint	       _order;
	/* instance_id 为整形字符串，可通过atoi转成int */
	gchar 	       *instance_id;
	gboolean       _scaleMode;
	gboolean       _panelEditModeChangedId;
	GSettings     *settings;
    gchar *_applet_tooltip_text;
    CdosPanelItemTooltip *_applet_tooltip;
	CdosApplet    *_applet;
	CdosApplet    *_delegate; // for dnd
        MetaData *meta;
	gchar	      *_uuid;
	gfloat	panel_height;
	guint panel_edit_mode_changedid;
	
	gboolean       _dragging;
    CdosDraggable *_draggable;

	gulong actor_button_press_id;
	gulong actor_button_release_id;

	CdosExtension *extension;
	guint new_order;
	guint orientation;

	StBoxLayout *actor;

	GList *single_window_mode_list;
	MetaWorkspace       *meta_workspace;
	gulong              window_added_id;
	gulong              window_removed_id;

	CdosPopupMenu     *context_menu;
	CdosPopupMenu     *popup_menu;
	CdosPopupMenuManager  *menu_manager;
	gboolean popup_menu_active;
	gboolean context_menu_active;
	gulong *popup_menu_open_state_changed_id;
	gulong *context_menu_open_state_changed_id;
};

typedef struct {
	guint               pid;
	gboolean 			*single_mode;
	MetaWindow          *meta_window;
	CdosApp				*app;
	const gchar 		*wm_class;

} SingleWindowMode;

enum {
  PROP_ACTOR = 1,
  PROP_PANELLOCATION,
  PROP_NEW_PANELLOCATION,
  PROP_ORIENTATION,
  PROP_PANEL_HEIGHT, 
  PROP_SCALEMODE,
  PROP_INSTANCE_ID,
  PROP_INIT_FUNC,
  NUM_PROPERTIES
};

static GParamSpec *pspecs[NUM_PROPERTIES] = { NULL, };

/* 为clutter actor 与applet 作数据绑定，actor 将用作查找applet的key */
typedef struct
{
  ClutterActor *actor;
  CdosApplet *applet;
}ActorApplet;
/* 用于保存actor 与 applet 对 */
static GSList *actor_applet = NULL;

static void cdos_applet_init_func (CdosApplet *applet);

static gboolean cdos_applet_actor_on_button_press(ClutterActor *actor,ClutterEvent *event,CdosApplet *applet);
static gboolean cdos_applet_actor_on_button_release(ClutterActor *actor,ClutterEvent *event,CdosApplet *applet);


static SingleWindowMode *
cdos_applet_lookup_single_mode_info_by_wm_class(CdosApplet *self,const gchar *wm_class);
static SingleWindowMode *
cdos_applet_lookup_single_mode_info_by_window(CdosApplet *self,MetaWindow *meta_window);

static void
cdos_applet_on_switch_workspace(CdosBaseWM *cdos_base_wm,
								   gint previous_workspace_index,
								   gint current_workspace_index,
								   MetaMotionDirection direction,
								   CdosApplet *self);
static void
cdos_applet_on_window_removed(MetaWorkspace *meta_workspace,MetaWindow *meta_window,CdosApplet *self);
static void
cdos_applet_on_window_added(MetaWorkspace *meta_workspace,MetaWindow *meta_window,CdosApplet *self);

static void cdos_applet_set_actor (CdosApplet    *self, StBoxLayout *actor);

static void cdos_applet_set_popup_menu(CdosApplet *self, guint orientation);

static void cdos_applet_popup_menu_on_open_state_changed(CdosPopupMenuBase *popup_menu, gboolean open,CdosApplet *self);

CdosApplet *
cdos_applet_get_actor_applet (ClutterActor *actor)
{
  if (actor) {
    guint i;
    ActorApplet *aa;
    for (i = 0; i < g_slist_length (actor_applet); i ++) {
      aa = g_slist_nth_data (actor_applet, i);
      if (aa->actor == actor) {
        return aa->applet;
      }
    }
  }
  return NULL;
}

static void
cdos_applet_set_actor_applet (CdosApplet *applet)
{
  g_return_if_fail (CDOS_IS_APPLET (applet));
  CdosAppletPrivate *priv = CDOS_APPLET (applet)->priv;
  if (applet->actor) {
    ActorApplet *aa = g_new0 (ActorApplet, 1);
    aa->actor = applet->actor;
    aa->applet = applet;
    actor_applet = g_slist_append (actor_applet, aa);
  }  
}
static void
cdos_applet_dispose(GObject *gobject){

	CdosAppletPrivate *priv = CDOS_APPLET (gobject)->priv;
	if(priv->single_window_mode_list){
		GList *single_list = priv->single_window_mode_list;
		while (single_list){
			SingleWindowMode *single_item = single_list->data;
			if(single_item){
				g_free(single_item);
			}
			single_list = single_list->next;
		}
		g_list_free(priv->single_window_mode_list);
	}
	g_object_unref (priv->panelLocation);
	g_object_unref (priv->newPanelLocation);
	g_object_unref (priv->settings);
	g_object_unref (priv->_applet_tooltip);
	g_object_unref (priv->_draggable);
	g_object_unref (priv->popup_menu);
	g_object_unref (priv->context_menu);
	g_object_unref (priv->menu_manager);
	G_OBJECT_CLASS (cdos_applet_parent_class)->dispose (gobject);

}

static void
cdos_applet_finalize(GObject *gobject){
	CdosAppletPrivate *priv = CDOS_APPLET (gobject)->priv;

  	G_OBJECT_CLASS (cdos_applet_parent_class)->finalize (gobject);

}

static void
cdos_applet_set_property (GObject *gobject,
			guint prop_id,
			const GValue *value,
			GParamSpec   *pspec)
{
  CdosApplet *obj = CDOS_APPLET (gobject);
  switch (prop_id){
    case PROP_ACTOR:
      cdos_applet_set_actor (obj, g_value_get_object (value));		
      break;
    case PROP_PANELLOCATION:
      cdos_applet_set_panel_location (obj, g_value_get_object (value));
      break;
    case PROP_NEW_PANELLOCATION:
      // cdos_applet_set_newlocation (obj, g_value_get_object (value));
      break;
    case PROP_ORIENTATION:
      //g_print ("%s, set orientation\n", __func__);
      cdos_applet_set_orientation (obj, g_value_get_int(value));
      break;
    case PROP_PANEL_HEIGHT:
      //g_print ("%s, set panel-height\n", __func__);
      cdos_applet_set_panel_height (obj, g_value_get_float (value));
      break;
    case PROP_SCALEMODE:
      cdos_applet_set_scalemode (obj, g_value_get_boolean (value));
      break;
    case PROP_INSTANCE_ID:
      //g_print ("%s, set instance-id\n", __func__);
      cdos_applet_set_instance_id (obj, g_value_get_string (value));
      break;
    case PROP_INIT_FUNC:
      //g_print ("%s, run init-func: %d\n", __func__, g_value_get_boolean (value));
      if (g_value_get_boolean (value)) {
        cdos_applet_init_func (obj);
      }
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
      break;
  }
}

static void
cdos_applet_get_property(GObject    *gobject,
                        guint       prop_id,
                        GValue     *value,
                        GParamSpec *pspec)
{

	CdosAppletPrivate *priv = NULL;

	priv = CDOS_APPLET (gobject)->priv;

	switch (prop_id){
	    case PROP_ACTOR:
		g_value_set_object (value, CDOS_APPLET (gobject)->actor);
		break;
	    case PROP_PANELLOCATION:
		g_value_set_object (value, priv->panelLocation);
		break;
	    case PROP_NEW_PANELLOCATION:
		g_value_set_object (value, priv->newPanelLocation);
		break;
	    case PROP_ORIENTATION:
		g_value_set_int (value, priv->_order);
		break;
	    case PROP_PANEL_HEIGHT:
		g_value_set_float (value, priv->panel_height);
		break;
	    case PROP_SCALEMODE:
                g_value_set_boolean (value, priv->_scaleMode);
		break;
    case PROP_INSTANCE_ID:
      g_value_set_string (value, priv->instance_id);
      break;

	    default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
     		break;
	}
}

static void
cdos_applet_popup_menu_on_open_state_changed(CdosPopupMenuBase *popup_menu, gboolean open,CdosApplet *self){
	CdosAppletPrivate *priv = self->priv;
	st_widget_remove_style_pseudo_class(ST_WIDGET(self->actor),"active");
	if(priv->popup_menu_active){
		if(open){
			st_widget_add_style_pseudo_class(ST_WIDGET(self->actor),"active");
			cdos_tooltip_hide(CDOS_TOOLTIP(priv->_applet_tooltip));
		}
	}
}

static void
cdos_applet_set_popup_menu(CdosApplet *self, guint orientation){
	CdosPopupMenu *popup_menu;
	gchar style[20]= {0,};

	ClutterActor *uiGroup = NULL;
	uiGroup = monkey_plugin_get_ui_group (monkey_plugin_get ());

	CdosAppletPrivate *priv = CDOS_APPLET_GET_PRIVATE (self);
	popup_menu = cdos_popup_menu_new_with_params (self->actor, 0, orientation);
	if ( uiGroup && popup_menu){
		st_container_add_actor ((ClutterContainer *)uiGroup, CLUTTER_ACTOR(popup_menu->actor));
	}
	clutter_actor_hide(popup_menu->actor);
	// setMaxHeight:
	MetaRectangle *monitor = NULL;
	CdosLayoutManager *layout = cdos_monkey_plugin_get_layout_manager (monkey_plugin_get ());
	CdosPanel *panel = cdos_monkey_plugin_get_panel (monkey_plugin_get ());
	monitor = cdos_layout_manager_get_primary_monitor (layout);
	gfloat panelheight = clutter_actor_get_height (panel);

	// monitor =  Main.primaryMonitor; // xunli FIXME
	// let maxHeight = Math.round(monitor.height - Main.panel.height - this.actor.get_theme_node().get_length('-boxpointer-gap'));
	StThemeNode *theme_node = st_widget_get_theme_node (ST_WIDGET (popup_menu->actor));
	gdouble gap_length = st_theme_node_get_length ( theme_node, "-boxpointer-gap");

	int maxHeight = round(monitor->height - panelheight - gap_length);

	memset (style, 0, 20);
	g_snprintf (style, 20, "max-height: %d px", maxHeight);
	//	printf ("%s %d : %f %s\n", __func__, __LINE__, panelheight, style);
	st_widget_set_style (popup_menu->actor, style); //FIXME check actor, maybe not private's
	priv->popup_menu = popup_menu;
	cdos_popup_menu_manager_add_menu (priv->menu_manager, CDOS_POPUP_MENU_BASE (priv->popup_menu), -1);
	priv->popup_menu_open_state_changed_id = \
			g_signal_connect(CDOS_POPUP_MENU_BASE(popup_menu),"open-state-changed",G_CALLBACK(cdos_applet_popup_menu_on_open_state_changed),self);

}

static void
on_applet_reactivity (gpointer user_data,gpointer user_data1,CdosApplet *self) {
	gboolean value;

	CdosAppletPrivate *priv = CDOS_APPLET_GET_PRIVATE (self);
	value = g_settings_get_boolean (priv->settings, "panel-edit-mode");	
	//g_object_set ( priv->_draggable, "inhibit", !value);
    //priv->_draggable->inhibit = value;
}

static void
on_drag_begin (CdosDraggable *drag, guint32 timestamp, CdosApplet *self){
	CdosAppletPrivate *priv = CDOS_APPLET_GET_PRIVATE (self);

	priv->_dragging = TRUE;
	//clutter_actor_hide (priv->_applet_tooltip);
	//g_object_set ( priv->_applet_tooltip, "preventShow", TRUE);// FIXME
	
}

static void
on_drag_end ( CdosDraggable *drag, guint32 timestamp, guint snap, CdosApplet *self){
	CdosAppletPrivate *priv = CDOS_APPLET_GET_PRIVATE (self);

    priv->_dragging = FALSE;
    //g_object_set ( priv->_applet_tooltip, "preventShow", FALSE);// FIXME
}

static void
on_drag_cancelled ( CdosDraggable *drag, guint32 timestamp, CdosApplet *self){
	CdosAppletPrivate *priv = CDOS_APPLET_GET_PRIVATE (self);

    priv->_dragging = FALSE;
    //g_object_set ( priv->_applet_tooltip, "preventShow", FALSE);// FIXME
}

static ClutterActor *
cdos_applet_get_dragactor( CdosApplet *self){
	return clutter_clone_new ((ClutterActor *)self->actor);
}

static ClutterActor *
cdos_applet_get_dragactor_source (CdosApplet *self){
	return CLUTTER_ACTOR (self->actor);
}

gboolean
cdos_applet_on_applet_clicked (CdosApplet *self)
{
  /* Impletmented by applets */
  // CdosAppletPrivate *priv = CDOS_APPLET_GET_PRIVATE (self);
  // cdos_popup_menu_base_toggle(CDOS_POPUP_MENU_BASE(self->applet_popup_menu));
  return TRUE;
}

static gboolean
on_applet_button_press_cb (ClutterActor *actor, ClutterEvent *event, gpointer data){
	gboolean inhibit;
    CdosApplet *self = (CdosApplet *)data;
	CdosAppletPrivate *priv = CDOS_APPLET_GET_PRIVATE (self);
    CdosPopupMenuBase *menu = CDOS_POPUP_MENU_BASE (cdos_applet_get_popup_menu(self));
	//g_object_get (priv->_draggable, "inhibit", &inhibit);
    
	if (clutter_event_get_button(event) == 1){
		// FIXME wuj
//        if (!priv->_draggable->inhibit)
//            return FALSE;
          if (menu && menu->is_open){
            cdos_popup_menu_base_toggle (menu);
          }
          CDOS_APPLET_CLASS(G_OBJECT_GET_CLASS(self))->on_applet_clicked (self);
	}

	if (clutter_event_get_button(event) == 3){
        if (menu){
          	cdos_popup_menu_base_toggle(CDOS_POPUP_MENU_BASE(menu));
	      	cdos_layout_manager_update_regions (cdos_monkey_plugin_get_layout_manager (monkey_plugin_get ()));
        }else{
			return FALSE;
		}
	}
	return TRUE;
}

/**
 * cdos_applet_set_applet_tooltip:
 * @self: 当前小应用对象实例
 * @text: 需要显示的tooltip内容
 *
 * 设置当前小应用需要显示的tooltip内容。
 */
void
cdos_applet_set_applet_tooltip (CdosApplet *self, gchar *text) {
	CdosAppletPrivate *priv = CDOS_APPLET_GET_PRIVATE (self);
    
	priv->_applet_tooltip_text = text;
	// get _applet_tooltip's _tooltip first

	// FIXME StLabel *label = cdos_tooltip_get_label (priv->_applet_tooltip);
	st_label_set_text ((StLabel *)priv->_applet_tooltip, text);
}

static gboolean
on_timeout_flash_spot_again (gpointer data){
	CdosApplet *object = CDOS_APPLET (data);

	//FIXME let flashspot = new Flashspot.Flashspot({ x : x, y : y, width: w, height: h});
        //FIXME flashspot.fire();

	g_source_remove (object->timeout_id);
	return FALSE;
}

/**
 * This function is called by appletManager when the applet is added to the panel. 
 * This is meant to be overridden in individual applets.
 */
gboolean
cdos_applet_on_applet_added_to_panel (CdosApplet *applet, gboolean user_enabled){
	gfloat x,y,w,h;

	CdosAppletPrivate *priv = CDOS_APPLET_GET_PRIVATE (applet);
	if (user_enabled) {
		clutter_actor_get_transformed_position (applet->actor, &x, &y);
		clutter_actor_get_transformed_size (applet->actor, &w, &h);
		h = h > priv->panel_height ? h :priv->panel_height;

		// FIXME flashspot
		//let flashspot = new Flashspot.Flashspot({ x : x, y : y, width: w, height: h});
            	//flashspot.fire();
		applet->timeout_id = g_timeout_add (300, on_timeout_flash_spot_again, applet);
	}
  // return FALSE; //TODO： 临时添加返回值，不确定值
}

gboolean
cdos_applet_on_applet_removed_from_panel (CdosApplet *applet)
{
  // CdosAppletPrivate *priv = CDOS_APPLET_GET_PRIVATE (applet);

  // g_signal_handler_disconnect (priv->settings, priv->panel_edit_mode_changedid);
  return FALSE;
}

gboolean
cdos_applet_on_panel_height_changed (void)
{

}

// used by appletManager
void
on_applet_removed_from_appmanager (CdosApplet *self){
	CdosAppletPrivate *priv = CDOS_APPLET_GET_PRIVATE (self);

	if(priv->panel_edit_mode_changedid > 0){
		g_signal_handler_disconnect (priv->settings, priv->panel_edit_mode_changedid);
		priv->panel_edit_mode_changedid = 0;
	}
	cdos_applet_on_applet_removed_from_panel (self);
}

static gboolean
on_orientation_changed (CdosApplet *self, StSide orientation){
	return FALSE;
}

void 
set_orientation (CdosApplet *self, StSide orientation){
	on_orientation_changed (self, orientation);
	//finalize_context_menu (self);
}
/**
 * cdos_applet_set_orientation:
 * @applet: 当前小应用对象实例
 * @orientation: 对其方式，#StSide枚举成员
 *
 * 设置当前小应用及其tooltip的上下对其方式；
 */
void
cdos_applet_set_orientation (CdosApplet *applet, guint orientation)
{
  g_return_if_fail (CDOS_IS_APPLET(applet));
  //set_orientation (applet, orientation);
  CdosAppletPrivate *priv = CDOS_APPLET_GET_PRIVATE (applet);

  priv->orientation = orientation;
  CdosPanelItemTooltip *tooltip = priv->_applet_tooltip;
  tooltip->orientation = orientation;
}
/**
 * cdos_applet_get_panel_height:
 * @applet: 当前小应用实例对象
 *
 * 获取面板高度
 *
 * Returns:返回高度尺寸
 */
gfloat 
cdos_applet_get_panel_height (CdosApplet *applet)
{
  CdosAppletPrivate *priv = CDOS_APPLET_GET_PRIVATE (applet);
  return priv->panel_height;
}
/**
 * cdos_applet_set_panel_height:
 * @applet: 当前小应用实例对象
 * @panel_height: 面板高度
 *
 * 设置面板高度值，注意：此函数并不是要修改面板的实际高度，而是根据面板的实际高度，将其值保存在当前结构体内。
 */
void 
cdos_applet_set_panel_height (CdosApplet *applet, gfloat panel_height)
{
  CdosAppletPrivate *priv = CDOS_APPLET_GET_PRIVATE (applet);
  priv->panel_height = panel_height;
}
/**
 * cdos_applet_get_actor：
 * @applet：当前小应用实例对象
 *
 * 获取当前小应用实例的主容器元素；
 *
 * Returns:返回一个#StBoxLayout的实例对象。
 *
 */
StBoxLayout *
cdos_applet_get_actor (CdosApplet *applet)
{
  return applet->actor;
}

/**
 * cdos_applet_set_actor:
 * @self: 当前小应用实例对象
 * @actor: #StBoxLayout实例对象
 *
 * 设置当前小应用的主容器元素，该函数不推荐在外部使用。
 *
 */
static void cdos_applet_set_actor (CdosApplet *self, StBoxLayout *actor){
	//CdosAppletPrivate *priv = CDOS_APPLET_GET_PRIVATE (self);
	self->actor = actor;
}
/**
 * cdos_applet_get_panel_location:
 * @applet: 当前小应用实例对象
 *
 * 获取当前小应用所在的面板区域的元素对象；
 *
 * Returns:返回一个#StBoxLayout实例对象。
 */
StBoxLayout *
cdos_applet_get_panel_location (CdosApplet *applet)
{
  CdosAppletPrivate *priv = applet->priv;
  return priv->panelLocation;  
}
/**
 * cdos_applet_set_panel_location:
 * @self: 当前小应用实例对象
 * @panellocation: 面板区域元素
 *
 * 设置当前小应用所在的面板区域元素，该函数不推荐在内部使用。
 */
void cdos_applet_set_panel_location (CdosApplet *self, StBoxLayout *panellocation){
	CdosAppletPrivate *priv = CDOS_APPLET_GET_PRIVATE (self);
    if (NULL != panellocation) {
        priv->panelLocation = g_object_ref (panellocation);
    } else {
        priv->panelLocation = NULL;
    }
}
/**
 * cdos_applet_get_new_panel_location:
 * @applet: 当前小应用实例对象
 *
 * 获取当前小应用的所在面板中的新区域元素，该成员主要用于小应用的移动。
 *
 * Returns: 返回一个#StBoxLayout的实例对象。
 */
StBoxLayout *
cdos_applet_get_new_panel_location (CdosApplet *applet)
{
  g_return_val_if_fail (CDOS_IS_APPLET (applet), NULL);
  return applet->priv->newPanelLocation;
}
/**
 * cdos_applet_set_new_panel_location:
 * @applet: 当前小应用实例对象
 * @new_panel_location: 面板区域元素
 *
 * 给当前小应用设置一个新的面板区域，当移动当前小应用的时候。
 */
void cdos_applet_set_new_panel_location (CdosApplet *applet, 
				StBoxLayout *new_panel_location)
{
  g_return_if_fail (CDOS_IS_APPLET (applet));
  CdosAppletPrivate *priv = CDOS_APPLET_GET_PRIVATE (applet);
  priv->newPanelLocation = g_object_ref (new_panel_location);
}
/**
 * cdos_applet_get_order:
 * @applet: 当前小应用实例对象
 *
 * 获取当前小应用在面板中的位置序列号。
 *
 * Returns: 返回当前的位置序列号。
 */
gint
cdos_applet_get_order (CdosApplet *applet)
{
  g_return_val_if_fail (CDOS_IS_APPLET(applet), -1);
  return applet->priv->_order;
}
/**
 * cdos_applet_set_order:
 * @self: 当前小应用实例对象
 * @order: 新的序列号；
 *
 * 设置当前小应用所在面板中新的位置序列号，该函数没有更改对应的GSettings中对应的序列号，使用时，需要在外部对GSettings进行更新。
 */
void cdos_applet_set_order (CdosApplet *self, guint order){
	CdosAppletPrivate *priv = CDOS_APPLET_GET_PRIVATE (self);

	priv->_order = order;
}
/**
 * cdos_applet_set_extension:
 * @applet: 当前小应用实例对象
 * @extension: 扩展实例对象
 *
 * 为当前小应用绑定扩展对象。
 */
void cdos_applet_set_extension (CdosApplet *applet, 
			CdosExtension *extension) 
{
  g_return_if_fail (CDOS_IS_APPLET (applet));
  CdosAppletPrivate *priv = applet->priv;
  priv->extension = extension;
}
/**
 * cdos_applet_get_extension
 * @applet: 当前小应用实例对象
 *
 * 获取当前小应用绑定的扩展对象。
 *
 * Returns: 返回一个#CdosExtension对象。
 */
CdosExtension *
cdos_applet_get_extension (CdosApplet *applet) 
{
  g_return_val_if_fail (CDOS_IS_APPLET (applet), NULL);
  return applet->priv->extension;
}
/**
 * cdos_applet_set_scalemode:
 * @self: 当前小应用实例对象
 * @scalemode: 变换模式
 *
 * 设置当前小应用是否为变换模式，变换模式下，小应用的显示尺寸将跟随面板的变化而变换。
 */
void  cdos_applet_set_scalemode (CdosApplet *self, gboolean scalemode){
	CdosAppletPrivate *priv = CDOS_APPLET_GET_PRIVATE (self);

	priv->_scaleMode = scalemode;
}
/**
 * cdos_applet_get_scalemode
 * @self: 当前小应用实例对象
 *
 * 获取当前小应用是否为变换摸索，变换模式下，小应用的显示尺寸将跟随面板的变化而变换。
 * Returns:TRUE-变换模式，FALSE-非变换模式。
 */
gboolean cdos_applet_get_scalemode (CdosApplet *self){
	CdosAppletPrivate *priv = CDOS_APPLET_GET_PRIVATE (self);

	return priv->_scaleMode;
}

static GObject *
cdos_applet_constructor (GType                  type,
                        guint                  n_construct_properties,
                        GObjectConstructParam *construct_properties){

    CdosApplet *object;
    object = (*G_OBJECT_CLASS (cdos_applet_parent_class)->constructor) (type, n_construct_properties, construct_properties);
    return object;
}

static void
cdos_applet_class_init (CdosAppletClass *klass)
{
  //g_print ("%s\n", __func__);	
  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
	
  gobject_class->finalize = cdos_applet_finalize;
  gobject_class->dispose = cdos_applet_dispose;
  gobject_class->set_property = cdos_applet_set_property;
  gobject_class->get_property = cdos_applet_get_property;
  gobject_class->constructor = cdos_applet_constructor;

  klass->on_applet_clicked = cdos_applet_on_applet_clicked;
  klass->on_applet_button_press_cb = on_applet_button_press_cb;
  klass->on_applet_added_to_panel = cdos_applet_on_applet_added_to_panel;
  klass->on_applet_removed_from_panel = cdos_applet_on_applet_removed_from_panel;
  klass->on_panel_height_changed = cdos_applet_on_panel_height_changed;

	/**
	 * CdosApplet:actor:
	 *
	 * 主容器元素
	 */
	pspecs[PROP_ACTOR] = g_param_spec_object ("actor",
                                "Layout Box",
                                "Applet's applet-box, hold icon bin",
                                ST_TYPE_BOX_LAYOUT,
                                G_PARAM_READWRITE);
	/**
	 * CdosApplet:panel-location:
	 *
	 * 主容器元素所在的面板区域元素
	 */
  	pspecs[PROP_PANELLOCATION] = g_param_spec_object("panel-location",
                                "panel box",
                                "Which panel box this applet locate",
                                ST_TYPE_BOX_LAYOUT,
                                G_PARAM_READWRITE);
	/**
	 * CdosApplet:new-panel-location:
	 *
	 * 主容器元素所在的新的面板区域元素
	 */
 	pspecs[PROP_NEW_PANELLOCATION] = g_param_spec_object("new-panel-location",
                                "panel box",
                                "Remove this applet to new location",
                                ST_TYPE_BOX_LAYOUT,
                                G_PARAM_READWRITE);

	/**
	 * CdosApplet:orientation:
	 *
	 * 小应用对其方式
	 */
	pspecs[PROP_ORIENTATION] = g_param_spec_int ("orientation",
				"orientation",
				"orientaion of this applet",
				0,
				20,
				2,
				G_PARAM_READWRITE);
	/**
	 * CdosApplet:scalemode:
	 *
	 * 是否是变换模式
	 */
		pspecs[PROP_SCALEMODE] = g_param_spec_boolean ("scalemode",
				"scale actor",
				"actor can scale or not",
				FALSE,
				G_PARAM_READWRITE);
	/**
	 * CdosApplet:panel-height:
	 *
	 * 当前面板高度
	 */
	pspecs[PROP_PANEL_HEIGHT] = g_param_spec_float ("panel-height",
				"panel height",
				"the height of panel",
				0,
				10000,
				25,
				G_PARAM_READWRITE);
	/**
	 * CdosApplet:instance-id:
	 *
	 * 小应用实例id
	 */
    pspecs[PROP_INSTANCE_ID] = g_param_spec_string ("instance-id",
				"instance id",
				"the instance id of the applet",
				NULL,
				G_PARAM_READWRITE);
	/**
	 * CdosApplet:init-func:
	 *
	 * 是否运行初始化过程
	 */
    pspecs[PROP_INIT_FUNC] = g_param_spec_boolean ("init-func",
                                "init func",
                                "the init func for applet init",
                                TRUE,
                                G_PARAM_READWRITE);

  	g_object_class_install_properties (gobject_class, NUM_PROPERTIES, pspecs);//新建书香并安装

  	g_type_class_add_private (klass, sizeof (CdosAppletPrivate));
	/**
	 * CdosApplet::applet_clicked:
	 * @applet:当前小应用实例对象
	 *
	 * 当点击当前小应用时，发射此信号
	 */
    applet_signals [APPLET_CLICKED] =
                    g_signal_new ("applet_clicked",
                    CDOS_TYPE_APPLET,
                    G_SIGNAL_RUN_FIRST,
                    G_STRUCT_OFFSET (CdosAppletClass, on_applet_clicked),
                    NULL, NULL,
                    g_cclosure_marshal_VOID__OBJECT,
                    G_TYPE_NONE, 1,
                    CDOS_TYPE_APPLET);
}

static void 
cdos_applet_init_func (CdosApplet *applet) {
  g_return_if_fail (CDOS_IS_APPLET (applet));
  //g_print ("%s\n", __func__);
  /* init 内相关代码在某些属性设置完后，当作参数设置其他成员或者进行其他操作 */
}

static void 
cdos_applet_init (CdosApplet *self)
{
  //g_print ("%s\n", __func__);
	CdosAppletPrivate *priv = NULL;
	CdosGlobal *global;
	gboolean scaleM;

	priv = self->priv = CDOS_APPLET_GET_PRIVATE (self);

    priv->_order = 0;
    priv->new_order = -1; //this._newOrder = null; but the type of new_order is gint
	StBoxLayout *actor = st_box_layout_new();
	StBoxLayout *panellocation = NULL;
	StBoxLayout *newpanellocation = NULL;

    priv->panelLocation = NULL;
    priv->newPanelLocation = NULL;
	MonkeyPlugin *plugin = monkey_plugin_get ();
	CdosPanel *panel = cdos_monkey_plugin_get_panel (plugin);
    priv->panel_height = cdos_panel_get_height(panel);
    priv->_uuid = NULL;
  // priv->_dragging = FALSE;
  // priv->_scaleMode = FALSE;
    priv->_applet_tooltip_text = "";
    priv->_applet_tooltip = cdos_panel_item_tooltip_new ((ClutterActor *)actor, 
          "", ST_SIDE_BOTTOM);

	st_widget_set_style_class_name (ST_WIDGET(actor), "applet-box");
	clutter_actor_set_reactive ((ClutterActor*) actor, TRUE);
	st_widget_set_track_hover (ST_WIDGET(actor), TRUE);
	cdos_applet_set_actor (self, actor);

    cdos_applet_set_actor_applet (self); // this.actor._applet = this;

	global = cdos_global_get();
	priv->settings = cdos_global_get_settings (global);

	priv->_scaleMode = FALSE;
	scaleM = g_settings_get_boolean (priv->settings, "panel-scale-text-icons");
	scaleM = priv->_scaleMode && g_settings_get_boolean (priv->settings, "panel-resizable");
	cdos_applet_set_scalemode (self, scaleM);
	
	priv->_dragging = FALSE;

	// g_signal_connect (priv->actor, "button-press-event",
	// 			G_CALLBACK (on_button_press_cb), self);

	priv->panel_edit_mode_changedid = g_signal_connect ( priv->settings, "changed::panel-edit-mode", G_CALLBACK(on_applet_reactivity), self);//FIXME

    //priv->_draggable = cdos_make_draggable((ClutterActor *)self->actor, NULL, 0);
    //cdos_draggable_set_delegate (self->actor, self);
    //cdos_draggable_set_func (self->actor, cdos_applet_get_dragactor, DND_GET_DRAG_ACTOR);
    //cdos_draggable_set_func (self->actor, cdos_applet_get_dragactor_source, DND_GET_DRAG_ACTOR_SRC);
	//g_signal_connect (priv->_draggable, "drag-begin", G_CALLBACK (on_drag_begin), self);
	//g_signal_connect (priv->_draggable, "drag-cancelled", G_CALLBACK (on_drag_cancelled), self);
	//g_signal_connect (priv->_draggable, "drag-end", G_CALLBACK (on_drag_end), self);
	cdos_applet_actor_set_click_effect(self,TRUE);

	//for single window mode;
	priv->window_added_id = 0L;
	priv->window_removed_id = 0L;
	MetaScreen *global_screen = cdos_global_get_screen(global);
	MetaWorkspace *meta_workspace = meta_screen_get_workspace_by_index (global_screen,0);

	CdosBaseWM *global_wm = cdos_global_get_window_manager(global);
	g_signal_connect(global_wm,"switch-workspace",G_CALLBACK(cdos_applet_on_switch_workspace),self);
	gint active_workspace_index = meta_screen_get_active_workspace_index (global_screen);
	cdos_applet_on_switch_workspace (global_wm,0,active_workspace_index,META_MOTION_RIGHT,self);

	priv->menu_manager = cdos_popup_menu_manager_new ((ClutterActor *)self->actor);

}

/**
 * cdos_applet_get_uuid:
 * @applet: 当前小应用实例对象
 *
 * 获取当前小应用的uuid。
 *
 * Returns: 当前小应用的uuid。
 */
gchar *
cdos_applet_get_uuid (CdosApplet *applet)
{
  g_return_val_if_fail (CDOS_IS_APPLET (applet), NULL);
  return applet->priv->_uuid;
}
/**
 * cdos_applet_set_uuid:
 * @applet: 当前小应用实例对象
 * @uuid: 设置当前小应用的uuid
 *
 * 设置当前小应用的uuid,该方法在构造过程中使用，没有更新GSettings内相应的值，不推荐在外部使用。
 */
void cdos_applet_set_uuid (CdosApplet *applet, gchar *uuid)
{
  g_return_if_fail (CDOS_IS_APPLET (applet));
  applet->priv->_uuid = uuid;
}
/**
 * cdos_applet_get_instance_id:
 * @applet: 当前小应用实例对象
 *
 * 获取当前小应用的实例id;
 *
 * Returns: 返回实例id;
 */
gchar *cdos_applet_get_instance_id (CdosApplet *applet)
{
  g_return_val_if_fail (CDOS_IS_APPLET (applet), NULL);
  return applet->priv->instance_id;
}
/**
 * cdos_applet_set_instance_id:
 * @applet: 当前小应用实例对象
 * @instance_id: 实例id
 *
 * 设置当前小应用的实例id，该方法没有更新GSettings内相应的值，不推荐在外部使用;
 *
 * Returns: 返回实例id;
 */
void cdos_applet_set_instance_id (CdosApplet *applet,
                                gchar *instance_id)
{
  g_return_if_fail (CDOS_IS_APPLET (applet));
  applet->priv->instance_id = g_strdup (instance_id);
}
/**
 * cdos_applet_get_new_order:
 * @applet: 当前小应用实例对象
 *
 * 获取当前小应用新的序列号，通常在当前小应用移动之后使用。
 *
 * Returns: 返回新的序列号。
 */
gint
cdos_applet_get_new_order (CdosApplet *applet)
{
  g_return_val_if_fail (CDOS_IS_APPLET(applet), -1);
  return applet->priv->new_order;
}
/**
 * cdos_applet_set_meta:
 * @applet: 当前小应用实例对象
 * @mata: 新的Meta结构体
 *
 * 设置当前小应用的Meta数据，不推荐在外部使用。
 */
void 
cdos_applet_set_meta (CdosApplet *applet, MetaData *meta)
{
  g_return_if_fail (CDOS_IS_APPLET (applet));
  applet->priv->meta = meta;
}
/**
 * cdos_applet_set_new_order:
 * @applet: 当前小应用实例对象
 *
 * 设置当前小应用新的序列号，通常在当前小应用移动过程中使用。
 */
void cdos_applet_set_new_order (CdosApplet *applet,
                                gint new_order)
{
  g_return_if_fail (CDOS_IS_APPLET (applet));
  applet->priv->new_order = new_order;
}

void  cdos_applet_actor_set_click_effect(CdosApplet *applet,gboolean has_effect){
	g_return_if_fail (CDOS_IS_APPLET (applet));
	CdosAppletPrivate *priv = applet->priv;
	StBoxLayout *actor = applet->actor;
	if(has_effect){
		if(priv->actor_button_press_id == 0){
			priv->actor_button_press_id = g_signal_connect(CLUTTER_ACTOR(actor),"button-press-event", G_CALLBACK (cdos_applet_actor_on_button_press), applet);
		}
		if(priv->actor_button_release_id == 0){
			priv->actor_button_release_id = g_signal_connect(CLUTTER_ACTOR(actor),"button-release-event", G_CALLBACK (cdos_applet_actor_on_button_release), applet);
		}
	} else {
		g_signal_handler_disconnect(CLUTTER_ACTOR(actor),priv->actor_button_press_id);
		g_signal_handler_disconnect(CLUTTER_ACTOR(actor),priv->actor_button_release_id);
		priv->actor_button_press_id = 0L;
		priv->actor_button_release_id = 0L;
	}
}

static gboolean cdos_applet_actor_on_button_press(ClutterActor *actor,ClutterEvent *event,CdosApplet *applet){
	CdosAppletPrivate *priv = applet->priv;
	if(clutter_event_get_button(event) == 1){
		if(priv->popup_menu && priv->popup_menu_active){
			cdos_popup_menu_base_toggle (CDOS_POPUP_MENU_BASE(priv->popup_menu));
			return FALSE;   //zhangdj
		} else {
			if(priv->popup_menu)
				cdos_popup_menu_close(CDOS_POPUP_MENU_BASE(priv->popup_menu),FALSE);
			return FALSE;
		}
	}else if(clutter_event_get_button(event) == 3){

		return FALSE;
	}
	return TRUE;
}
static gboolean cdos_applet_actor_on_button_release(ClutterActor *actor,ClutterEvent *event,CdosApplet *applet){

	return FALSE;
}

static SingleWindowMode *
cdos_applet_lookup_single_mode_info_by_window(CdosApplet *self,MetaWindow *meta_window){
	CdosAppletPrivate *priv = self->priv;
	if(meta_window == NULL){
		return NULL;
	}
	GList *single_list = priv->single_window_mode_list;
	while (single_list != NULL){
		SingleWindowMode *item = single_list->data;
		if(item->meta_window == meta_window){
			return item;
		}
		single_list = single_list->next;
	}
	return NULL;
}

static void cdos_applet_on_window_removed(MetaWorkspace *meta_workspace,MetaWindow *meta_window,CdosApplet *self){
	CdosAppletPrivate *priv = self->priv;
	MetaWindowType type = meta_window_get_window_type(meta_window);
	if(type == META_WINDOW_NORMAL){
		SingleWindowMode *item = cdos_applet_lookup_single_mode_info_by_window(self,meta_window);
		if(item){
			item->meta_window = NULL;
			item->pid = 0;
		}
	}

}
static void cdos_applet_on_window_added(MetaWorkspace *meta_workspace,MetaWindow *meta_window,CdosApplet *self){
	CdosAppletPrivate *priv = self->priv;
	MetaWindowType type = meta_window_get_window_type(meta_window);
	if(type != META_WINDOW_NORMAL){
		return ;
	}
	CdosWindowTracker *tracker = cdos_window_tracker_get_default();
	CdosApp *app = cdos_window_tracker_get_window_app (tracker, meta_window);
	
	const gchar *wm_class = meta_window_get_wm_class(meta_window);
	if(wm_class){
		SingleWindowMode *single_item = cdos_applet_lookup_single_mode_info_by_wm_class(self,wm_class);
		if(single_item && g_strcmp0(wm_class,single_item->wm_class) == 0){
			guint pid = meta_window_get_pid(meta_window);
			single_item->pid = pid;
			single_item->meta_window = meta_window;
			single_item->app = app;
		}
	}
}

static void cdos_applet_on_switch_workspace(CdosBaseWM *cdos_base_wm,
										   gint previous_workspace_index,
										   gint current_workspace_index,
										   MetaMotionDirection direction,
										   CdosApplet *self){
	CdosAppletPrivate *priv = self->priv;

	if(priv->window_added_id > 0){
		g_signal_handler_disconnect(priv->meta_workspace,priv->window_added_id);
		priv->window_added_id = 0L;
	}
	if(priv->window_removed_id > 0){
		g_signal_handler_disconnect(priv->meta_workspace,priv->window_removed_id);
		priv->window_removed_id = 0L;
	}

	MetaScreen *global_screen = cdos_global_get_screen(global);
	MetaWorkspace *meta_workspace = meta_screen_get_workspace_by_index (global_screen,current_workspace_index);

	priv->meta_workspace = meta_workspace;
	priv->window_removed_id = g_signal_connect(meta_workspace,"window-removed",G_CALLBACK(cdos_applet_on_window_removed),self);
	priv->window_added_id = g_signal_connect(meta_workspace,"window-added",G_CALLBACK(cdos_applet_on_window_added),self);

}

static SingleWindowMode *
cdos_applet_lookup_single_mode_info_by_wm_class(CdosApplet *self,const gchar *wm_class){
    /*
     * VALGRIND shows :
     * invalid read of size 1
     * This means we should check NUll string before comparing
     * */
    if (wm_class == NULL){
        return NULL;
    }
	CdosAppletPrivate *priv = self->priv;
	GList *single_list = priv->single_window_mode_list;
	while (single_list != NULL){
		SingleWindowMode *item = single_list->data;
        if (item->wm_class != NULL){
    		if(g_strcmp0(item->wm_class,wm_class) == 0){
	    		return item;
		    }
        }
		single_list = single_list->next;
	}
	return NULL;
}


gboolean
cdos_applet_is_single_window_mode(CdosApplet *self,const gchar *wm_class){
	g_return_if_fail (CDOS_IS_APPLET (self));
	CdosAppletPrivate *priv = self->priv;
	if(wm_class == NULL){
		return FALSE;
	}
	SingleWindowMode *info = cdos_applet_lookup_single_mode_info_by_wm_class(self,wm_class);
	return info->single_mode;
}


void cdos_applet_set_single_window_mode(CdosApplet *self,
										const gchar *wm_class,
										gboolean is_single){
	g_return_if_fail (CDOS_IS_APPLET (self));
	if(wm_class == NULL){
		return ;
	}
	CdosAppletPrivate *priv = self->priv;
	SingleWindowMode *info = cdos_applet_lookup_single_mode_info_by_wm_class(self,wm_class);
	if(info && is_single == FALSE){
		priv->single_window_mode_list = g_list_remove(priv->single_window_mode_list,info);
		g_free(info);
	} else if(info == NULL && is_single == TRUE) {
		SingleWindowMode *new_single = g_new0(SingleWindowMode,1);
		new_single->wm_class = wm_class;
		new_single->single_mode = is_single;
		new_single->pid = 0;

		priv->single_window_mode_list = g_list_append(priv->single_window_mode_list,new_single);
	}
}

void
cdos_applet_reset_single_window_status(CdosApplet *self, const gchar *wm_class){
	g_return_if_fail (CDOS_IS_APPLET (self));
	CdosAppletPrivate *priv = self->priv;
	if(wm_class == NULL){
		return ;
	}
	SingleWindowMode *info = cdos_applet_lookup_single_mode_info_by_wm_class(self,wm_class);
	if(info){
		MetaWindow *meta_window = info->meta_window;
		if(meta_window){
			MetaWorkspace *meta_workspace = meta_window_get_workspace(meta_window);
			if(meta_workspace != priv->meta_workspace){
				meta_window_change_workspace(meta_window,priv->meta_workspace);
			}
			guint32 current_time = cdos_global_get_current_time(global);
			gboolean minimized;
			g_object_get(meta_window,"miniminzed",&minimized,NULL);
			meta_window_activate(meta_window,current_time);
			if(!meta_window_has_focus(meta_window)){
				if(minimized){
					meta_window_unminimize(meta_window);
				}
				meta_window_raise(meta_window);
			}else{
				if(minimized){
					meta_window_unminimize(meta_window);
				} else {
					meta_window_minimize(meta_window);
				}
			}
		}
	}
}

guint
cdos_applet_get_single_window_pid(CdosApplet *self,const gchar *wm_class){
	g_return_if_fail (CDOS_IS_APPLET (self));
	CdosAppletPrivate *priv = self->priv;
	if(wm_class == NULL){
		return ;
	}
	SingleWindowMode *info = cdos_applet_lookup_single_mode_info_by_wm_class(self,wm_class);
	if(info){
		return info->pid;
	}
	return 0;
}

void cdos_applet_set_popup_menu_active(CdosApplet *self,gboolean active){
	g_return_if_fail (CDOS_IS_APPLET (self));
	CdosAppletPrivate *priv = self->priv;
	if(active){
		if(priv->popup_menu == NULL){
			cdos_applet_set_popup_menu(self,ST_SIDE_BOTTOM);
		}
		if(priv->popup_menu_open_state_changed_id == 0){
			priv->popup_menu_open_state_changed_id = \
			g_signal_connect(CDOS_POPUP_MENU_BASE(priv->popup_menu),"open-state-changed",G_CALLBACK(cdos_applet_popup_menu_on_open_state_changed),self);
		}
	}else{
		cdos_popup_menu_close(CDOS_POPUP_MENU_BASE(priv->popup_menu),FALSE);
		if(priv->popup_menu_open_state_changed_id > 0){
			g_signal_handler_disconnect(CDOS_POPUP_MENU_BASE(priv->popup_menu),priv->popup_menu_open_state_changed_id);
			priv->popup_menu_open_state_changed_id = 0L;
		}
	}
	priv->popup_menu_active = active;
}

void cdos_applet_set_context_menu_active(CdosApplet *self,gboolean active){
	g_return_if_fail (CDOS_IS_APPLET (self));


}

CdosPopupMenu *
cdos_applet_get_popup_menu(CdosApplet *self){
	g_return_val_if_fail (CDOS_IS_APPLET (self),NULL);
	CdosAppletPrivate *priv = self->priv;
	return priv->popup_menu;
}

CdosPopupMenu *
cdos_applet_get_context_menu(CdosApplet *self){
	g_return_val_if_fail (CDOS_IS_APPLET (self),NULL);
	CdosAppletPrivate *priv = self->priv;
	return priv->context_menu;
}

gboolean
cdos_applet_get_popup_menu_active(CdosApplet *self){
	g_return_val_if_fail (CDOS_IS_APPLET (self),FALSE);
	CdosAppletPrivate *priv = self->priv;
	return priv->popup_menu_active;
}

gboolean
cdos_applet_get_context_menu_active(CdosApplet *self){
	g_return_val_if_fail (CDOS_IS_APPLET (self),FALSE);
	CdosAppletPrivate *priv = self->priv;
	return priv->popup_menu_active;
}

CdosPopupMenuManager *
cdos_applet_get_menu_manager(CdosApplet *self){
	g_return_val_if_fail (CDOS_IS_APPLET (self),NULL);
	CdosAppletPrivate *priv = self->priv;
	return priv->menu_manager;
}
