#ifndef __CDOS_APPLET_H__
#define __CDOS_APPLET_H__

#include <clutter/clutter.h>
#include "cdos-global.h"
#include "cdos-popup-menu.h"
#include "cdos-popup-menu-manager.h"

#include "st.h"
#include "cdos-extension.h"

GType cdos_applet_get_type(void);

#define CDOS_TYPE_APPLET                (cdos_applet_get_type ())
#define CDOS_APPLET(obj)                (G_TYPE_CHECK_INSTANCE_CAST ((obj), CDOS_TYPE_APPLET, CdosApplet))
#define CDOS_IS_APPLET(obj)             (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CDOS_TYPE_APPLET))
#define CDOS_APPLET_CLASS(klass)        (G_TYPE_CHECK_CLASS_CAST ((klass), CDOS_TYPE_APPLET, CdosAppletClass))
#define CDOS_IS_APPLET_CLASS(klass)     (G_TYPE_CHECK_CLASS_TYPE ((klass), CDOS_TYPE_APPLET))
#define CDOS_APPLET_GET_CLASS(obj)      (G_TYPE_INSTANCE_GET_CLASS ((obj), CDOS_TYPE_APPLET, CdosAppletClass))

typedef struct _CdosApplet              CdosApplet;
typedef struct _CdosAppletPrivate       CdosAppletPrivate;
typedef struct _CdosAppletClass         CdosAppletClass;

struct _CdosApplet
{
  GObject 	parent_instance;
  StBoxLayout *actor;
  CdosAppletPrivate *priv;
  guint		timeout_id;
};

struct _CdosAppletClass
{
  GObjectClass parent_class;
  gboolean (*on_applet_clicked) (CdosApplet *self);
  gboolean (*on_applet_button_press_cb) (CdosApplet *self, ClutterEvent *event, gpointer data);
  gboolean (*on_applet_added_to_panel) (CdosApplet *applet, gboolean applets_loaded);
  gboolean (*on_applet_removed_from_panel) (CdosApplet *applet);
  gboolean (*on_panel_height_changed) (void);

};

CdosApplet *
cdos_applet_get_actor_applet (ClutterActor *actor);

StBoxLayout *
cdos_applet_get_actor (CdosApplet *applet);

StBoxLayout *
cdos_applet_get_panel_location (CdosApplet *applet);
void
cdos_applet_set_panel_location (CdosApplet *self, StBoxLayout *panellocation);

StBoxLayout *
cdos_applet_get_new_panel_location (CdosApplet *applet);
void
cdos_applet_set_new_panel_location (CdosApplet *applet, StBoxLayout *new_panel_location);

gint
cdos_applet_get_order (CdosApplet *applet);
void
cdos_applet_set_order (CdosApplet *self, guint order);

gint
cdos_applet_get_new_order (CdosApplet *applet);
void
cdos_applet_set_new_order (CdosApplet *applet, gint new_order);

void
cdos_applet_set_extension (CdosApplet *applet, CdosExtension *extension);
CdosExtension *
cdos_applet_get_extension (CdosApplet *applet);

gchar *
cdos_applet_get_uuid (CdosApplet *applet);
void
cdos_applet_set_uuid (CdosApplet *applet, gchar *uuid);

void 
cdos_applet_set_meta (CdosApplet *applet, MetaData *meta);

void
cdos_applet_set_applet_tooltip (CdosApplet *self, gchar *text);

gchar *
cdos_applet_get_instance_id (CdosApplet *applet);
void
cdos_applet_set_instance_id (CdosApplet *applet, gchar *instance_id);

void
cdos_applet_set_orientation (CdosApplet *applet, guint orientation);

gfloat
cdos_applet_get_panel_height (CdosApplet *applet);

void
cdos_applet_set_panel_height (CdosApplet *applet, gfloat panel_height);

gboolean
cdos_applet_on_applet_added_to_panel (CdosApplet *self, gboolean applets_loaded);
gboolean
cdos_applet_on_applet_removed_from_panel (CdosApplet *self);

gboolean
cdos_applet_get_scalemode (CdosApplet *self);
void
cdos_applet_set_scalemode (CdosApplet *self, gboolean scalemode);

void
cdos_applet_actor_set_click_effect(CdosApplet *applet,gboolean has_effect);

void
cdos_applet_set_single_window_mode(CdosApplet *self,const gchar *wm_class,gboolean is_single);
gboolean
cdos_applet_is_single_window_mode(CdosApplet *self,const gchar *wm_class);

void
cdos_applet_reset_single_window_status(CdosApplet *self, const gchar *wm_class);

guint
cdos_applet_get_single_window_pid(CdosApplet *self,const gchar *wm_class);

void
cdos_applet_set_popup_menu_active(CdosApplet *self,gboolean active);
void
cdos_applet_set_context_menu_active(CdosApplet *self,gboolean active);

gboolean
cdos_applet_get_popup_menu_active(CdosApplet *self);
gboolean
cdos_applet_get_context_menu_active(CdosApplet *self);

CdosPopupMenu *
cdos_applet_get_popup_menu(CdosApplet *self);
CdosPopupMenu *
cdos_applet_get_context_menu(CdosApplet *self);

CdosPopupMenuManager *
cdos_applet_get_menu_manager(CdosApplet *self);

#endif
