#include "cdos.h"
#include "cdos-classic-switcher.h"
#include "cdos-appswitcher-private.h"

#include <string.h>

#include "cdos-global.h"
#include "cdos-debug.h"

/* Classic Switcher */
#define POPUP_APPICON_SIZE 96
#define POPUP_SCROLL_TIME 0.10
#define POPOP_DELAY_TIMEOUT 150
#define POPOP_FADE_OUT_TIME 0.1

#define APP_ICON_HOVER_TIMEOUT 200

#define THUMBNAIL_DEFAULT_SIZE 256
#define THUMBNAIL_POPUP_TIME 180
#define THUMBNAIL_FADE_TIME 0.1

#define PREVIEW_DELAY_TIMEOUT 180
#define PREVIEW_SWTICHER_FADEOUT_TIME 0.5

typedef struct _ClassicSwitcher ClassicSwitcher;

struct _ClassicSwitcher {
    AppSwitcher parent;

    gulong      thumbnail_timeout_id;
    gulong      disable_preview_timeout_id;

    guint       thumbnail_visible: 1;    
    guint       icons_enabled: 1;
    guint       preview_enabled: 1;
    guint       thumbnail_enabled: 1;
    guint       show_thumbnails: 1;
    guint       show_arrows: 1;
};

static void
classic_switcher_get_preferred_width (ClutterActor *actor,
                                      gfloat for_width,
                                      CdosGenericContainerAllocation *alloc,
                                      gpointer data);

static void
classic_switcher_get_preferred_height (ClutterActor *actor,
                                       gfloat for_width,
                                       CdosGenericContainerAllocation *alloc,
                                       gpointer data);

static void
classic_switcher_allocate (ClutterActor *actor,
                           ClutterActorBox *box,
                           guint flags,
                           ClassicSwitcher *switcher);

static void classic_switcher_update_list (AppSwitcher *parent, int index);

ClassicSwitcher *
cdos_classic_switcher_new (MetaKeyBinding *binding)
{
    ClassicSwitcher *switcher = g_new0 (ClassicSwitcher, 1);
    AppSwitcher *parent = (AppSwitcher *)switcher;

    appswitcher_init (parent, binding);

    parent->actor = g_object_new (CDOS_TYPE_GENERIC_CONTAINER, NULL);
    clutter_actor_set_name (parent->actor, "altTabPopup");
    clutter_actor_set_reactive (parent->actor, 1);
    parent->actor->flags |= CLUTTER_ACTOR_VISIBLE;

    //ClutterActor *ui_group = UI_GROUP;
    st_container_add_actor ((ClutterContainer *)ui_group, parent->actor);

    if (!appswitcher_setup_modal (parent))
    {
        cdos_debug ();
        return NULL;
    }
    GSettings *settings = cdos_global_get_settings (global);
    char *style_settings = g_settings_get_string (settings, "alttab-switcher-style");
    if (strstr (style_settings, "icons") != NULL)
        switcher->icons_enabled = 1;
    if (strstr (style_settings, "preview") != NULL)
        switcher->preview_enabled = 1;
    if (strstr (style_settings, "thumbnails") != NULL)
        switcher->thumbnail_enabled = 1;

    if ((switcher->icons_enabled | switcher->preview_enabled | switcher->thumbnail_enabled) == 0)
        switcher->icons_enabled = 1;

    switcher->show_thumbnails = switcher->thumbnail_enabled & ~switcher->icons_enabled;
    switcher->show_arrows = switcher->thumbnail_enabled & switcher->icons_enabled;

    classic_switcher_update_list (parent, 0);

    g_signal_connect (parent->actor, "get-preferred-width",
            G_CALLBACK (classic_switcher_get_preferred_width), NULL);
    g_signal_connect (parent->actor, "get-preferred-height",
            G_CALLBACK (classic_switcher_get_preferred_height), NULL);
    g_signal_connect (parent->actor, "allocate",
            G_CALLBACK (classic_switcher_allocate), switcher);

    // Need to force an allocation so we can figure out whether we
    // need to scroll when slecting
    clutter_actor_set_opacity (parent->actor, 0);
    clutter_actor_show (parent->actor);
    ClutterActorBox box;
    clutter_actor_get_allocation_box (parent->actor, &box);
}

static void
classic_switcher_get_preferred_width (ClutterActor *actor,
                                      gfloat for_width,
                                      CdosGenericContainerAllocation *alloc,
                                      gpointer data)
{
    int w, h;
    cdos_global_get_screen_size (global, &w, &h);
    alloc->min_size = w;
    alloc->natural_size = w;
}

static void
classic_switcher_get_preferred_height (ClutterActor *actor,
                                       gfloat for_width,
                                       CdosGenericContainerAllocation *alloc,
                                       gpointer data)
{
    int w, h;
    cdos_global_get_screen_size (global, &w, &h);
    alloc->min_size = h;
    alloc->natural_size = h;
}

static void
classic_switcher_allocate (ClutterActor *actor,
                           ClutterActorBox *box,
                           guint flags,
                           ClassicSwitcher *switcher)
{
    AppSwitcher *parent = (AppSwitcher *)switcher;
    MetaRectangle *monitor = parent->active_monitor;
    StThemeNode *node = st_widget_get_theme_node ((StWidget *)parent->actor);
    gfloat left_padding = st_theme_node_get_padding (node, ST_SIDE_LEFT);
    gfloat right_padding = st_theme_node_get_padding (node, ST_SIDE_RIGHT);
    gfloat bottom_padding = st_theme_node_get_padding (node, ST_SIDE_BOTTOM);
    gfloat v_padding = st_theme_node_get_vertical_padding (node);
    gfloat h_padding = left_padding + right_padding; //st_theme_node_get_horizontal_padding (node);

    // Allocate the appSwitcher
    // We select a size based on an icon size that does not overflow the screen


    //ClutterActorBox *child_box = clutter_actor_box_new ();

}
