#include "cdos.h"
#include "cdos-confirm-dialog.h"
#include "cdos-debug.h"

/* Confirm dialog 例子
 */

struct _CdosConfirmDialog {
    /* 继承自 ModalDialog */
    ModalDialog *modal;
    GList *buttons;
    StWidget *label;
};

static void
confirm_dialog_close (gpointer     button,
                      gint         button_event,
                      ModalDialog *modal)
{
    cdos_debug();
    cdos_modal_dialog_close (modal, 0);
    //cdos_modal_dialog_destroy (modal);
}

CdosConfirmDialog *
cdos_confirm_dialog_new (const char *msg, ButtonCallback callback)
{
    CdosConfirmDialog *confirm_dialog = g_new0 (CdosConfirmDialog, 1);
    confirm_dialog->modal = g_new0(ModalDialog,1);
    cdos_modal_dialog_init (confirm_dialog->modal, 0, 0);

    StWidget *label = st_label_new (msg);
    g_assert (confirm_dialog->modal->content_layout != NULL);
    st_container_add_actor((ClutterContainer *)confirm_dialog->modal->content_layout,
            (ClutterActor *)label);
    confirm_dialog->label = label;
    GList *buttons = NULL;
    ButtonInfo *info = g_new0 (ButtonInfo, 1);
    info->label = _("Yes");
    if (callback)
        info->callback = callback;
    else
        info->callback = (ButtonCallback)confirm_dialog_close;
    info->userdata = confirm_dialog->modal;
    info->focused = 1;

    buttons = g_list_append (buttons, info);

    info = g_new0 (ButtonInfo, 1);
    info->label = _("No");
    info->callback = NULL;
    info->focused = 1;
    buttons = g_list_append (buttons, info);

    cdos_modal_dialog_set_buttons (confirm_dialog->modal, buttons);
    confirm_dialog->buttons = buttons;
    //g_free (info);

    return confirm_dialog;
    //cdos_modal_dialog_open (modal, 0);
}

void
cdos_confirm_dialog_show (CdosConfirmDialog *confirm)
{
    cdos_modal_dialog_open ((ModalDialog *)confirm->modal, 0);
}

void
cdos_confirm_dialog_destroy (CdosConfirmDialog *dialog){

    cdos_modal_dialog_close (dialog->modal, 0);
    GList *buttons = dialog->buttons;
    while (buttons != NULL){
        ButtonInfo *info = buttons->data;
        g_free(info->label);
        g_free(info);
        buttons = buttons->next;
    }
    g_list_free(buttons);
    g_clear_object(&dialog->label);
    cdos_modal_dialog_destroy(dialog->modal);
    g_free(dialog);
}