/**
 * SECTION: cdos-console-kit-manager
 * @short_description: 管理ConsolKit
 * @include: cdos-console-kit-manager.h
 * 
 * CdosConsoleKitManagerClass 类用来管理 ConsoleKit
 */
#include "cdos-console-kit-manager.h"


G_DEFINE_TYPE ( CdosConsoleKitManager, cdos_console_kit_manager, G_TYPE_OBJECT );



static void onActiveChanged ( GObject *gobject, gboolean isActive, gpointer user_data ) {

	CdosConsoleKitManager *self = CDOS_CONSOLE_KIT_MANAGER (user_data);

	//save isActive from singal
	self->sessionActive = isActive;
}


static void get_is_active_cb ( GObject *source_object, GAsyncResult *res, gpointer user_data ) {

	CdosConsoleKitManager *self = CDOS_CONSOLE_KIT_MANAGER (user_data);

        GVariant *result;
        GError *error = NULL;
	guint isActive;

        result = g_dbus_proxy_call_finish (G_DBUS_PROXY (source_object),
                                           res,
                                           &error);
        if (result == NULL) {
                g_warning ("couldn't get isActive : %s", error->message);
                g_error_free (error);
		return;
        } 
	
	g_variant_get (result, "(b)", &isActive);
        g_variant_unref (result);

	//save the query result of dbus isActive method.
	self->sessionActive = isActive;
}


static void onCurrentSession_cb ( GObject *source_object, GAsyncResult *res, gpointer user_data ) {

        CdosConsoleKitManager *self = CDOS_CONSOLE_KIT_MANAGER (user_data);
        GError *error = NULL;
        GVariant *result = NULL;
        gchar *object_path = NULL;

	//get the object path
        result = g_dbus_connection_call_finish ( G_DBUS_CONNECTION (source_object), res, &error );

        if (result == NULL) {
                g_warning ("consolekit GetCurrentSession failed: %s", error->message);
                g_error_free (error);
		return;
        }
        else
        {
                g_variant_get (result, "(o)", &object_path);
                
        }

        g_variant_unref (result);

	//connect to session
        self->ckSession = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
                                                        0,
                                                        NULL,
                                                        "org.freedesktop.ConsoleKit",
                                                        object_path,
                                                        "org.freedesktop.ConsoleKit.Session",
                                                        NULL, &error);

        if (self->proxy == NULL) {
                g_warning ("Couldn't org.freedesktop.ConsoleKit.Session proxy: %s", error->message);
                g_error_free (error);
                return;
        }

	//listen to ActiveChanged signal of dbus proxy
	g_signal_connect ( self->ckSession, "ActiveChanged", G_CALLBACK (onActiveChanged) , self );

	// 调用dbus的IsActive方法
        g_dbus_proxy_call (self->ckSession,
                           "IsActive",
                           NULL,
                           G_DBUS_CALL_FLAGS_NONE,
                           -1, NULL,
                           get_is_active_cb, NULL);	

}


static void onManagerAppeared_cb ( GDBusConnection *connection, const gchar *name, const gchar *name_owner, gpointer user_data ) {

        //调用dbus的GetCurrentSession方法      
        g_dbus_connection_call ( connection,
                                "org.freedesktop.ConsoleKit",
                                "/org/freedesktop/ConsoleKit/Manager",
                                "org.freedesktop.ConsoleKit.Manager",
                                "GetCurrentSession",
                                NULL, NULL,
                                G_DBUS_CALL_FLAGS_NONE, -1,
                                NULL, onCurrentSession_cb, NULL);




}


static void onManagerVanished_cb ( GDBusConnection *connection, const gchar *name, gpointer user_data) {

        CdosConsoleKitManager *self = CDOS_CONSOLE_KIT_MANAGER (user_data);

        self->sessionActive = TRUE;
}

static void cdos_console_kit_manager_dispose ( GObject *gobject ) {

        CdosConsoleKitManager *self = CDOS_CONSOLE_KIT_MANAGER(gobject);
        g_bus_unwatch_name(self->watchID);
        if(self->proxy){
                g_object_unref(self->proxy);
        }
        if(self->ckSession){
                g_object_unref(self->ckSession);
        }
        /* 调用父类的 finalize() 方法 */
        G_OBJECT_CLASS (cdos_console_kit_manager_parent_class)->dispose(gobject);
}

static void cdos_console_kit_manager_finalize ( GObject *gobject ) {



	/* 调用父类的 finalize() 方法 */
    	G_OBJECT_CLASS (cdos_console_kit_manager_parent_class)->finalize(gobject);

}



static void 
cdos_console_kit_manager_class_init ( CdosConsoleKitManagerClass *klass ) {

        GObjectClass *gobject_class = G_OBJECT_CLASS(klass);

	gobject_class->finalize = cdos_console_kit_manager_finalize;
	gobject_class->dispose = cdos_console_kit_manager_dispose;

}


static void
cdos_console_kit_manager_init ( CdosConsoleKitManager *self ) {

        GError *error = NULL;
        self->sessionActive = TRUE;
        self->proxy = NULL;
        self->watchID = 0;
	

        self->proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
                                                        0,
                                                        NULL,
                                                        "org.freedesktop.ConsoleKit",
                                                        "/org/freedesktop/ConsoleKit/Manager",
                                                        "org.freedesktop.ConsoleKit.Manager",
                                                        NULL, &error);

        if (self->proxy == NULL) {
                g_warning ("Couldn't org.freedesktop.ConsoleKit proxy: %s", error->message);
                g_error_free (error);
                return;
        }

        self->watchID = g_bus_watch_name ( G_BUS_TYPE_SYSTEM,
                                                "org.freedesktop.ConsoleKit",
                                                G_BUS_NAME_WATCHER_FLAGS_NONE,
                                                onManagerAppeared_cb,
                                                onManagerVanished_cb,
                                                self,
                                                NULL);

}

/**
 * cdos_console_kit_manager_new:
 * 
 * 新建一个 #CdosConsoleKitManager 类的实例。
 *
 * Returns: 返回一个 #CdosConsoleKitManager 的实例。
 */
CdosConsoleKitManager *cdos_console_kit_manager_new () {
 
        CdosConsoleKitManager *instance = g_object_new ( CDOS_TYPE_CONSOLE_KIT_MANAGER, NULL );

        return instance;
}






