
#include "cdos-expo-workspace-thumbnail.h"
#include "cdos-expo-window-clone.h"
#include "cdos-expo-thumbnails-box.h"

#include "cdos-monkey-plugin.h"
#include "cdos-layout-manager.h"
#include "cdos-window-manager.h"
#include "cdos-tooltip.h"
#include "cdos-base-util.h"

#include <math.h>

/**
 * SECTION:cdos-expo-workspace-thumbnail
 * @Title: CdosExpoWorkspaceThumbnail
 * @Short_description: 单个工作区的缩略图
 * 
 * 桌面默认有四个工作区，每个工作区缩略图除了工作区本身，还有个 #GtkEntry 显示这个缩略图的名称
 * 在当前工作区中打开、关闭窗口等事件，都需要这个对象来监听信号并更新窗口列表内容
 * 由于获取的窗口列表是所有已经打开的窗口，还需要过滤这些窗口是不是普通要显示的窗口，是不是当前工作区的窗口等。
 */
#define INACTIVE_OPACITY 120

#define ICON_OFFSET -5

#define REARRANGE_TIME_ON  100
#define REARRANGE_TIME_OFF 300
#define ICON_SIZE 128

//外部定义: main中尚未实现的函数

static void cdos_remove_workspace (MetaWorkspace *metaWorkspace) {}
static gboolean cdos_has_default_workspace_name (gint ws_index) { return TRUE; }
static gboolean cdos_is_window_actor_displayed_on_workspace (MetaWindowActor *win, gint ws_index) {return TRUE;}
static gboolean isMyWindow (CdosExpoWorkspaceThumbnail *self, MetaWindowActor *win);
static gboolean isExpoWindow (CdosExpoWorkspaceThumbnail *self, MetaWindowActor *win);

enum  {

	STICKTY_DETECTED,
	DRAG_BEGIN,
	DRAG_END,
    	LAST_SIGNAL
};


typedef struct {

	gpointer p1;
	gpointer p2;

} ParamForCallback; 


static gint cdos_expo_workspace_thumbnail_signals [LAST_SIGNAL] =  { 0 };


G_DEFINE_TYPE  ( CdosExpoWorkspaceThumbnail, cdos_expo_workspace_thumbnail, G_TYPE_OBJECT );


//宣告用
static void doAddWindow (CdosExpoWorkspaceThumbnail *self, MetaWindow *metaWin);
static void doRemoveWindow (CdosExpoWorkspaceThumbnail *self, MetaWindow *metaWin);
static void resetCloneHover (CdosExpoWorkspaceThumbnail *self);
static void restack (CdosExpoWorkspaceThumbnail *self, gboolean force);




static void activate (CdosExpoWorkspaceThumbnail *self, MetaWindowActor *clone, guint64 time) {

	MetaWindow *clone_metawindow;
	MetaScreen *global_screen = cdos_global_get_screen (cdos_global_get());

        if (self->state > THUMBNAIL_STATE_NORMAL)
                return;

        if (clone)
	        clone_metawindow = meta_window_actor_get_meta_window (clone);

        if (clone != NULL && clone_metawindow != NULL) {
                //调用main中的activeWindow
                cdos_activate_window (clone_metawindow, time, meta_workspace_index (self->metaWorkspace));
        }

        if (self->metaWorkspace != meta_screen_get_active_workspace (global_screen))
                meta_workspace_activate (self->metaWorkspace, time);

        cdos_expo_hide (cdos_monkey_plugin_get_expo (), FALSE); //gpointer *options 类型未知
}

static void removeAction (gpointer user_data) {

        CdosExpoWorkspaceThumbnail *self = CDOS_EXPO_WORKSPACE_THUMBNAIL (user_data);

        self->doomed = TRUE;

        //调用main中的移除工作区的函数
        cdos_remove_workspace (self->metaWorkspace);
}

static void onWorkspaceChanged (GObject *gobject, ClutterEvent *event, gpointer user_data) {

	CdosExpoWindowClone *clone = CDOS_EXPO_WINDOW_CLONE (gobject);
	CdosExpoWorkspaceThumbnail *self = CDOS_EXPO_WORKSPACE_THUMBNAIL (user_data);
	MetaWindow *metaWindow = clone->metaWindow;

	doRemoveWindow (self, metaWindow);

    if (meta_window_is_on_all_workspaces (metaWindow)) {
        // Muffin appears not to broadcast when a window turns sticky
        g_signal_emit_by_name (self->box, "sticky-detected", metaWindow );		
    }
}

static void onMiddleButtonRelease (GObject *gobject, guint32 time, gpointer user_data) {

        CdosExpoWindowClone *clone = CDOS_EXPO_WINDOW_CLONE (gobject);
        MetaWindow *metaWindow = clone->metaWindow;

	cdos_debug("middle?-----------\n");
	meta_window_delete (metaWindow, time);
}

static gboolean
onIdleBT_cb (ParamForCallback *param)
{	
	CdosExpoWorkspaceThumbnail *self = CDOS_EXPO_WORKSPACE_THUMBNAIL (param->p1);
	CdosExpoWindowClone *clone = CDOS_EXPO_WINDOW_CLONE (param->p2);
	g_free (param);

	gint x, y;
	ClutterModifierType mask;
	ClutterStage *stage;
	ClutterActor *target;

	self->buttonTimeoutId = 0;

	if (!self->windows) 
		return FALSE; /* being destroyed */

	cdos_global_get_pointer (cdos_global_get (), &x, &y, &mask);

    stage = CLUTTER_STAGE (clutter_actor_get_stage (self->contents)); 
	target = clutter_stage_get_actor_at_pos (stage, CLUTTER_PICK_REACTIVE, x, y);

    if (target != (ClutterActor *)clone) {
        resetCloneHover (self);
        return FALSE;
    }

    /*FIXME leit: tooltip 还有bug，暂时不加入 */
    /*
    if (self->tooltip)
        clutter_actor_destroy ((ClutterActor *)self->tooltip);

	self->tooltip = cdos_tooltip_new ((ClutterActor *)clone, g_strdup (meta_window_get_title (clone->metaWindow)));
    */

    return FALSE;
}

static void
onCloneHover (CdosExpoWindowClone *clone, gboolean hovering, gpointer user_data)
{
    /* FIXME leit: 暂不加入 tooltip */
    return;

	CdosExpoWorkspaceThumbnail *self = CDOS_EXPO_WORKSPACE_THUMBNAIL (user_data);

    if (!self->overviewMode) {
        resetCloneHover (self);
        return;
    }

    if (hovering && (clone != self->lastHoveredClone))
    {
        if (self->buttonTimeoutId) {
            g_source_remove (self->buttonTimeoutId);
        }
        //传递参数用
        ParamForCallback *param;
        param = g_new0 (ParamForCallback, 1);
        param->p1 = self; 
        param->p2 = clone;

        self->buttonTimeoutId = g_idle_add ((GSourceFunc) onIdleBT_cb, param);

        self->lastHoveredClone = clone;
    }
}

static void onDemandingAttention (GObject *gobject, gpointer user_data) {

	CdosExpoWorkspaceThumbnail *self = CDOS_EXPO_WORKSPACE_THUMBNAIL (user_data);

	cdos_expo_workspace_thumbnail_overview_mode_on (self);
}

static void onSelected (GObject *gobject, guint time, gpointer user_data) {

	CdosExpoWorkspaceThumbnail *self = CDOS_EXPO_WORKSPACE_THUMBNAIL (user_data);

	activate (self, NULL, time);
}

static void  onRemoveWorkspace (GObject *gobject, gpointer user_data) {

	CdosExpoWorkspaceThumbnail *self = CDOS_EXPO_WORKSPACE_THUMBNAIL (user_data);

	cdos_expo_workspace_thumbnail_remove (self);
}

static void onDragBegin (GObject *gobject, gpointer user_data) {

	CdosExpoWorkspaceThumbnail *self = CDOS_EXPO_WORKSPACE_THUMBNAIL (user_data);
        
	g_signal_emit_by_name (self, "drag-begin");

        resetCloneHover (self);
}

static void onDragEnd (GObject *gobject, gpointer user_data) {

	CdosExpoWorkspaceThumbnail *self = CDOS_EXPO_WORKSPACE_THUMBNAIL (user_data);
	CdosExpoWindowClone *clone = CDOS_EXPO_WINDOW_CLONE (gobject);

	g_signal_emit_by_name (self, "drag-end");

	if (clone->dragCancelled) {
                // stacking order may have been disturbed
                restack (self, TRUE);
        }
	cdos_expo_workspace_thumbnail_overview_mode_on (self);
}

/**
 * addWindowClone:
 * @self: #CdosExpoWorkspaceThumbnail 对象
 * @win: 普通窗口对象
 *
 * 添加窗口克隆链表：在 #CdosExpoWindowClone 对象的 cdos_expo_window_clone_refresh_clone 函数中，大概介绍了这个过程。
 * 传递进来的普通窗口，根据需要，可以为它的临时窗口也一起创建克隆体，保存在一个链表中，一一遍历保存到 #CdosExpoWindowClone 的 clone 成员中，
 * 它的这个 clone 不直接对外使用，而是继续添加到 actor 中，也就是我们这个函数中的 clone.
 * 每个窗口对应一个 #CdosExpoWindowClone 对象，工作区中所有窗口的克隆体就保存在 self->windows 链表中
 * 需要注意的是：每新加一个窗口，就将它放置在最顶层。
 */
static CdosExpoWindowClone *
addWindowClone (CdosExpoWorkspaceThumbnail *self, MetaWindowActor *win) {

        CdosExpoWindowClone *clone = cdos_expo_window_clone_new (win);
	//gint length;

	g_signal_connect (clone, "workspace-changed", G_CALLBACK (onWorkspaceChanged), self);
//	g_signal_connect (clone, "middle-button-release", G_CALLBACK (onMiddleButtonRelease), self);
	g_signal_connect (clone, "hovering", G_CALLBACK (onCloneHover), self);
	g_signal_connect (clone, "demanding-attention", G_CALLBACK (onDemandingAttention), self);
	g_signal_connect (clone, "selected", G_CALLBACK (onSelected), self);
	g_signal_connect (clone, "remove-workspace", G_CALLBACK (onRemoveWorkspace), self);
	g_signal_connect (clone, "drag-begin", G_CALLBACK (onDragBegin), self);
	g_signal_connect (clone, "drag-end", G_CALLBACK (onDragEnd), self);

	clutter_container_add_actor ((ClutterContainer *)self->contents, (ClutterActor *)clone);
	/*
	length = g_list_length (self->windows);

        if (length == 0)
		cdos_expo_window_clone_set_stack_above (clone, self->background);
        else
	{
		CdosExpoWindowClone *tmpClone = g_list_nth_data (self->windows, length - 1);
		cdos_expo_window_clone_set_stack_above (clone, (ClutterActor *)tmpClone);
	}
    */
    clutter_container_raise_child ((ClutterContainer *)self->contents, (ClutterActor *)clone, NULL);
	self->windows = g_list_append (self->windows, clone);

        return clone;
}

static gboolean onButtonPressEvent (GObject *gobject, ClutterEvent *event, gpointer user_data) {

	CdosExpoWorkspaceThumbnail *self = CDOS_EXPO_WORKSPACE_THUMBNAIL (user_data);
	ClutterActor *actor = CLUTTER_ACTOR (gobject);

        self->lastButtonPressTimeStamp = clutter_event_get_time (event);
        self->lastButtonPressActor = actor;

        return TRUE;
}

static gboolean onButtonReleaseEvent (GObject *gobject, ClutterEvent *event, gpointer user_data) {

	CdosExpoWorkspaceThumbnail *self = CDOS_EXPO_WORKSPACE_THUMBNAIL (user_data);
	ClutterActor *actor = CLUTTER_ACTOR (gobject);
	guint timeElapsed;
	ClutterModifierType evstate;

        if (self->lastButtonPressActor != actor) 
        	return TRUE;
        
        timeElapsed = clutter_event_get_time (event) - self->lastButtonPressTimeStamp;
        // A long time elapsed is probably due to a failed dnd attempt,
        // or some other mishap, so we'll ignore those.
        if (timeElapsed > 500)
        	return TRUE;
               
        evstate = cdos_base_util_get_event_state (event);

        if ((evstate & CLUTTER_BUTTON1_MASK) || (evstate & CLUTTER_BUTTON3_MASK)) {

        	activate (self, NULL, clutter_event_get_time (event));
                return TRUE;
	} 
	else if (evstate & CLUTTER_BUTTON2_MASK) 
	{
        	cdos_expo_workspace_thumbnail_remove (self);
                return TRUE;
        }

        return FALSE;
}

static void onScrollEvent (GObject *gobject, ClutterEvent *event, gpointer user_data) {

	//CdosExpoWorkspaceThumbnail *self = CDOS_EXPO_WORKSPACE_THUMBNAIL (user_data);
        ClutterScrollDirection direction = clutter_event_get_scroll_direction (event);
        switch (direction) {
            case CLUTTER_SCROLL_UP:
                cdos_wm_action_move_workspace_left ();
                break;
            case CLUTTER_SCROLL_DOWN:
                cdos_wm_action_move_workspace_right ();
                break;
            default:
                break;
        }
}

static gboolean onTitleKeyPressEvent (ClutterText *entry, ClutterEvent *event, gpointer user_data) {

	CdosExpoWorkspaceThumbnail *self = CDOS_EXPO_WORKSPACE_THUMBNAIL (user_data);

    self->undoTitleEdit = FALSE;

    guint symbol = clutter_event_get_key_symbol (event);

    if (symbol == CLUTTER_Return || symbol == CLUTTER_Escape) {

        // 	if (symbol == CLUTTER_Escape) 
        //     	self->undoTitleEdit = TRUE;

        clutter_stage_set_key_focus (cdos_global_get_stage (cdos_global_get ()), CLUTTER_ACTOR (self->actor));

        return TRUE;
    }

    return FALSE;
}

static gboolean onTitleKeyFocusInEvent (ClutterText *entry, gpointer user_data) {

    cdos_debug ();
	CdosExpoWorkspaceThumbnail *self = CDOS_EXPO_WORKSPACE_THUMBNAIL (user_data);
	cdos_debug ("%d\n", meta_workspace_index (self->metaWorkspace));
	//调用main中的getWorkspaceName获取工作区名称
	self->origTitle = cdos_get_workspace_name (meta_workspace_index (self->metaWorkspace));
	return FALSE;
}

static gboolean onTitleKeyFocusOutEvent (ClutterText *entry, gpointer user_data) {

	CdosExpoWorkspaceThumbnail *self = CDOS_EXPO_WORKSPACE_THUMBNAIL (user_data);
	//ClutterStage *global_stage = cdos_global_get_stage (cdos_global_get ());

    cdos_debug ("%s %d\n", __func__, __LINE__);
//	if (self->doomed) {
                // user probably deleted workspace while editing
//		clutter_stage_set_key_focus (global_stage, CLUTTER_ACTOR (self->box->actor));
 //               return;
//	}

        if (!self->undoTitleEdit) {
                gchar *newName = g_strdup (st_entry_get_text (self->title));
		//然后调用trim()方法去掉不必要的字符,尚未加

                g_print ("%s %d: old %s new %s\n", __func__, __LINE__, self->origTitle, newName);
                if (g_strcmp0 (newName, self->origTitle) != 0) 
                    	cdos_set_workspace_name (meta_workspace_index (self->metaWorkspace), newName);
                
        }
        gchar *workspace_new =  cdos_get_workspace_name (meta_workspace_index (self->metaWorkspace)); 
        cdos_debug ("index is %d, new name is %s \n", meta_workspace_index (self->metaWorkspace), workspace_new);
        st_entry_set_text (self->title, workspace_new); 
	return FALSE;
}

static GList*
lookupIndex (CdosExpoWorkspaceThumbnail *self, MetaWindow *metaWindow)
{
    GList *idx = NULL;
	for (idx = self->windows; idx; idx = idx->next)
    {
        CdosExpoWindowClone *clone = idx->data;

        if (clone->metaWindow == metaWindow)
            return idx;
    }
    return NULL;
}

static void setOverviewMode (CdosExpoWorkspaceThumbnail *self, gboolean turnOn) {

        if (turnOn) {
		    cdos_expo_workspace_thumbnail_overview_mode_on (self);
        }
        else {
		    cdos_expo_workspace_thumbnail_overview_mode_off (self, FALSE, FALSE);
        }
}

static gboolean onIdle_cb (gpointer user_data) {

	ParamForCallback *param = (ParamForCallback *) user_data;
	CdosExpoWorkspaceThumbnail *self = CDOS_EXPO_WORKSPACE_THUMBNAIL (param->p1);
	MetaWindow *metaWin = META_WINDOW (param->p2);

	/*will be null if we're closing down*/
	if (self->windows  && meta_window_get_compositor_private (metaWin))
                    doAddWindow (self, metaWin);
        
	//参数使用完了
	g_free (param);

        return FALSE;
}

static void doAddWindow (CdosExpoWorkspaceThumbnail *self, MetaWindow *metaWin) {

    MetaWindowActor *win = META_WINDOW_ACTOR (meta_window_get_compositor_private (metaWin));
	ParamForCallback *param;

    if (!win) {
        //纯粹為传参数用
        param = g_new0 (ParamForCallback, 1);
        param->p1 = self;
        param->p2 = metaWin;

        // Newly-created windows are added to a workspace before
        // the compositor finds out about them...
        g_idle_add ((GSourceFunc) onIdle_cb, param);
        return;
    }

    // We might have the window in our list already if it was on all workspaces and
    // now was moved to this workspace
    if (lookupIndex (self, metaWin) != NULL)
        return;

    if (!isMyWindow(self, win) || !isExpoWindow(self, win))
        return;

    addWindowClone (self, win);

    cdos_expo_workspace_thumbnail_overview_mode_on (self);
}

static gboolean isMyWindow (CdosExpoWorkspaceThumbnail *self, MetaWindowActor *win) {

    	// Tests if @win belongs to this workspace and monitor
        return window_actor_displayed_on_workspace (win, meta_workspace_index (self->metaWorkspace));
}

static gboolean isExpoWindow (CdosExpoWorkspaceThumbnail *self, MetaWindowActor *win) {

    // Tests if @win should be shown in the Expo
    MetaWindow *metaWindow = meta_window_actor_get_meta_window (win);
	MetaWindowType type;

    if (meta_window_is_override_redirect (metaWindow))
        return FALSE;

    type = meta_window_get_window_type (metaWindow);

    //如果是desktop或dock就不应该显示在expo中
    return type != META_WINDOW_DESKTOP && type != META_WINDOW_DOCK;
}

static void doRemoveWindow (CdosExpoWorkspaceThumbnail *self, MetaWindow *metaWin) {

	MetaWindowActor *win = META_WINDOW_ACTOR (meta_window_get_compositor_private (metaWin));
	CdosExpoWindowClone *clone;

    //找出actor在windows中的位置
    GList *index = lookupIndex (self, metaWin);

    if (index == NULL)
        return;

    // Check if window still should be here
    if (win && isMyWindow (self, win) && isExpoWindow (self, win))
        return;

    //找出index
	clone = index->data;

    //this.windows.splice(index, 1); 把找出的clone从List中移除
	self->windows = g_list_remove (self->windows, clone);
    cdos_expo_window_clone_destroy (clone);

    if (self->overviewMode)
        cdos_expo_workspace_thumbnail_overview_mode_on (self);
}

static  gboolean isOverviewWindow (MetaWindow *metaWindow) {
    	// Tests if @win should be shown in overview mode
	return cdos_is_interesting (metaWindow);
}

static gint minimizedDiff (CdosExpoWindowClone *a, CdosExpoWindowClone *b) {

    gboolean minimize_a = FALSE;
    gboolean minimize_b = FALSE;

    g_object_get (a->metaWindow, "minimized", &minimize_a, NULL);
    g_object_get (b->metaWindow, "minimized", &minimize_b, NULL);

	return minimize_b - minimize_a;
}

static gint noOverviewDiff (CdosExpoWindowClone *a, CdosExpoWindowClone *b) {

	gint noOverviewA = !isOverviewWindow (a->metaWindow) ? -1 : 0;
	gint noOverviewB = !isOverviewWindow (b->metaWindow) ? -1 : 0;
	return noOverviewA - noOverviewB;
}

static gint transientRelation (CdosExpoWindowClone *a, CdosExpoWindowClone *b) {

	gint overviewDifference = noOverviewDiff (a,b);

	if (overviewDifference) {
        gint transientA = (meta_window_get_transient_for (a->metaWindow) == b->metaWindow) ? 1 : 0;
        gint transientB = !transientA && (meta_window_get_transient_for (b->metaWindow) == a->metaWindow) ? 1 : 0;
        if (transientA - transientB)
            return transientA - transientB;
        else
            return overviewDifference;
    }
    return 0;
}

static gint window_sort_func (CdosExpoWindowClone *a, CdosExpoWindowClone *b, gpointer user_data)
{
	guint32 index_a = meta_window_get_stable_sequence (a->metaWindow);
	guint32 index_b = meta_window_get_stable_sequence (b->metaWindow);
    /*
	gint diff_a_b = g_hash_table_lookup (self->stackIndices, GUINT_TO_POINTER (index_a)) 
				- g_hash_table_lookup (self->stackIndices, GUINT_TO_POINTER (index_b));
    */

    gint diff_a_b = index_a - index_b;
    gint ret;
	if ((ret = transientRelation (a,b)))
        return ret;
    else if ((ret = minimizedDiff (a, b)))
       return ret;
    else
        return diff_a_b;
}

static void
syncStacking (CdosExpoWorkspaceThumbnail *self, GHashTable *stackIndices)
{
	//将window做一个排序
	//self->windows = g_list_sort_with_data (self->windows, (GCompareDataFunc) window_sort_func, self); 
    GList *iter;
    gboolean minimize = FALSE;
    for (iter = self->windows; iter; iter = iter->next)
    {
        CdosExpoWindowClone *clone = iter->data;

        MetaWindow *metaWindow = clone->metaWindow;
        
        g_object_get (metaWindow, "minimized", &minimize, NULL);
        if (minimize){
            continue;
        }
        if (iter == self->windows) {
            //cdos_expo_window_clone_set_stack_above (clone, self->background);
            clutter_container_raise_child ((ClutterContainer *)self->contents, (ClutterActor *)clone, self->background);
        } 
        else 
        {
            CdosExpoWindowClone *previousClone = iter->prev->data;
            //cdos_expo_window_clone_set_stack_above (clone, (ClutterActor *)previousClone);
            clutter_container_raise_child ((ClutterContainer *)self->contents, (ClutterActor *)clone, (ClutterActor *)previousClone);
        }
    }
}

static void refreshTitle (CdosExpoWorkspaceThumbnail *self) {

        if (!self->doomed) { 
		// better safe than sorry
		//st_entry_set_text (self->title, cdos_get_workspace_name (meta_workspace_index (self->metaWorkspace)));
        }
}

static void resetCloneHover (CdosExpoWorkspaceThumbnail *self) {

    self->lastHoveredClone = NULL;

    /*
    if (self->tooltip) {
        clutter_actor_destroy ((ClutterActor *)self->tooltip);
        self->tooltip = NULL;
    }
    */
}

static void restack (CdosExpoWorkspaceThumbnail *self, gboolean force) {

	gint i;

    if (self->state > THUMBNAIL_STATE_NORMAL)
        return;

    if ( self->isActive || !self->stackIndices || force) {

        //使用isMyWindow函数来提取出
        
        GList *stack = cdos_global_get_window_actors (cdos_global_get());  //filter(self->isMyWindow, self);

        if (self->stackIndices)
            g_hash_table_destroy (self->stackIndices);
        self->stackIndices = g_hash_table_new (NULL, NULL);

        gboolean minimize = FALSE;
        for (i = 0; i < g_list_length (stack); i++) {
            // Use the stable sequence for an integer to use as a hash key
            //this.stackIndices[stack[i].get_meta_window().get_stable_sequence()] = i;
            MetaWindowActor *win = g_list_nth_data (stack, i);
            MetaWindow *metaWindow = meta_window_actor_get_meta_window (win);
            //self->stackIndices[i] = meta_window_get_stable_sequence (metaWindow);
            g_object_get (metaWindow, "minimized", &minimize, NULL);

            if (!minimize) {
                g_hash_table_insert (	self->stackIndices,  
                    GINT_TO_POINTER (meta_window_get_stable_sequence (metaWindow)), 
                    GINT_TO_POINTER (i));	
            }
        }
        
    }

    syncStacking (self, self->stackIndices);
}

static void disconnectOtherSignals (CdosExpoWorkspaceThumbnail *self) {

	MetaScreen *global_screen = cdos_global_get_screen (cdos_global_get());
    if(self->restackedNotifyId > 0){
        g_signal_handler_disconnect (global_screen, self->restackedNotifyId);
        self->restackedNotifyId = 0L;
    }
    if(self->setOverviewModeId > 0){
        g_signal_handler_disconnect (self->box, self->setOverviewModeId);
        self->setOverviewModeId = 0L;
    }
	if(self->stickyAddedId > 0){
        g_signal_handler_disconnect (self->box, self->stickyAddedId);
        self->stickyAddedId = 0L;
    }
    if(self->windowAddedId > 0){
        g_signal_handler_disconnect (self->metaWorkspace, self->windowAddedId);
        self->windowAddedId = 0L;
    }
    if(self->windowRemovedId > 0){
        g_signal_handler_disconnect (self->metaWorkspace, self->windowRemovedId);
        self->windowRemovedId = 0L;
    }
    if(self->windowEnteredMonitorId > 0){
        g_signal_handler_disconnect (global_screen, self->windowEnteredMonitorId);
        self->windowEnteredMonitorId = 0L;
    }
    if(self->windowLeftMonitorId > 0){
        g_signal_handler_disconnect (global_screen, self->windowLeftMonitorId);
        self->windowLeftMonitorId = 0L;
    }
}

/**
 * cdos_expo_workspace_thumbnail_destroy:
 * @self: #CdosExpoWorkspaceThumbnail 对象
 *
 * 销毁工作区缩略图的组件，在隐藏工作区缩略图时调用这个接口来销毁。
 */
void
cdos_expo_workspace_thumbnail_destroy (CdosExpoWorkspaceThumbnail *self){
    cdos_debug ();
    if(self->title){
        clutter_actor_destroy ((ClutterActor *)self->title);
        self->title = NULL;
    }
    if(self->backgroundShade){
        clutter_actor_remove_child (self->background, (ClutterActor *)self->backgroundShade);
    }
    if(self->desktopBackground){
        clutter_actor_remove_child (self->background, (ClutterActor *)self->desktopBackground);
    }
    if(self->background){
        clutter_container_remove_actor ((ClutterContainer *)self->contents, self->background);
    }
    if(self->contents){
        st_container_remove_actor ((ClutterContainer *)self->actor, self->contents);
    }
    if(self->shader){
        st_container_remove_actor ((ClutterContainer *)self->actor, (ClutterActor *)self->shader);
    }
    if(self->actor){
        clutter_actor_destroy ((ClutterActor *)self->actor);
        self->actor = NULL;
    }
    if(self->frame){
        clutter_actor_destroy ((ClutterActor *)self->frame);
        self->frame = NULL;
    }
}

static void
onDestroy (GObject *gobject, gpointer user_data)
{
	CdosExpoWorkspaceThumbnail *self = CDOS_EXPO_WORKSPACE_THUMBNAIL (user_data);
    disconnectOtherSignals (self);

    resetCloneHover (self);

    GList *iter;
    for (iter = self->windows; iter; iter = iter->next) {
		//clutter_actor_destroy (iter->data);
        g_clear_object(&iter->data);
    }

	g_list_free (self->windows);
    self->windows = NULL;
}

static void windowAdded (GObject *gobject, MetaWindow *metaWin, gpointer user_data) {

	CdosExpoWorkspaceThumbnail *self = CDOS_EXPO_WORKSPACE_THUMBNAIL (user_data);

    doAddWindow (self, metaWin);
    restack (self, TRUE);
}

static void windowRemoved (GObject *gobject, MetaWindow *metaWin, gpointer user_data) {

   	CdosExpoWorkspaceThumbnail *self = CDOS_EXPO_WORKSPACE_THUMBNAIL (user_data);

    doRemoveWindow (self, metaWin);
}

static void 
windowEnteredMonitor (GObject *gobject, gint monitorIndex, MetaWindow *metaWin, gpointer user_data) {

	CdosExpoWorkspaceThumbnail *self = CDOS_EXPO_WORKSPACE_THUMBNAIL (user_data);
        // important if workspaces-only-on-primary is in effect
    doAddWindow (self, metaWin);
}

static void 
windowLeftMonitor (GObject *gobject, gint monitorIndex, MetaWindow *metaWin, gpointer user_data) {

	CdosExpoWorkspaceThumbnail *self = CDOS_EXPO_WORKSPACE_THUMBNAIL (user_data);
    // important if workspaces-only-on-primary is in effect
    doRemoveWindow (self, metaWin);
}

static void onSetOverviewMode (GObject *gobject, gboolean turnOn, gpointer user_data) {
	
	CdosExpoWorkspaceThumbnail *self = CDOS_EXPO_WORKSPACE_THUMBNAIL (user_data);

	setOverviewMode (self, turnOn);
    self->hovering = FALSE;
}

static void onStickyDetected (GObject *gobject, MetaWindow *metaWindow, gpointer user_data) {

	CdosExpoWorkspaceThumbnail *self = CDOS_EXPO_WORKSPACE_THUMBNAIL (user_data);

	doAddWindow (self, metaWindow);
}

static void onRestack (GObject *gobject, gpointer user_data) {

	CdosExpoWorkspaceThumbnail *self = CDOS_EXPO_WORKSPACE_THUMBNAIL (user_data);

        restack (self, TRUE);
        cdos_expo_workspace_thumbnail_refresh (self);
}


//固定的类方法
static void cdos_expo_workspace_thumbnail_dispose (GObject *object) {
    CdosExpoWorkspaceThumbnail *self = CDOS_EXPO_WORKSPACE_THUMBNAIL (object);
    cdos_expo_workspace_thumbnail_destroy(self);

    disconnectOtherSignals (self);
    if(self->windows){
        GList *windows = self->windows;
        while (windows != NULL){
            if(windows->data){
                g_clear_object(&windows->data);
            }
            windows = windows->next;
        }
        g_list_free(self->windows);
    }
    if(self->stackIndices){
        g_hash_table_unref(self->stackIndices);
    }

    g_clear_object(&self->box);
    g_clear_object(&self->tooltip);
    g_clear_object(&self->pointerTracker);

    G_OBJECT_CLASS (cdos_expo_workspace_thumbnail_parent_class)->dispose (object);
}
static void cdos_expo_workspace_thumbnail_finalize (GObject *object) {
    CdosExpoWorkspaceThumbnail *self = CDOS_EXPO_WORKSPACE_THUMBNAIL (object);


    G_OBJECT_CLASS (cdos_expo_workspace_thumbnail_parent_class)->finalize (object);
}

static void
cdos_expo_workspace_thumbnail_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec  *pspec) {

}

static void
cdos_expo_workspace_thumbnail_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec  *pspec) {

}

static void
cdos_expo_workspace_thumbnail_class_init (CdosExpoWorkspaceThumbnailClass *klass) {
 
    	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

    	gobject_class->finalize = cdos_expo_workspace_thumbnail_finalize;
    	gobject_class->dispose = cdos_expo_workspace_thumbnail_dispose;
    	gobject_class->set_property = cdos_expo_workspace_thumbnail_set_property;
    	gobject_class->get_property = cdos_expo_workspace_thumbnail_get_property;


     /*   cdos_expo_workspace_thumbnail_signals [STICKTY_DETECTED] =
                g_signal_new ("sticky-detected",
                        G_TYPE_FROM_CLASS (klass),
                        G_SIGNAL_RUN_LAST,
                        0,
                        NULL, NULL,
                        g_cclosure_marshal_VOID__OBJECT,
                        G_TYPE_NONE, 1 ,
                        META_TYPE_WINDOW );*/

        cdos_expo_workspace_thumbnail_signals [DRAG_BEGIN] =
                g_signal_new ("drag-begin",
                        G_TYPE_FROM_CLASS (klass),
                        G_SIGNAL_RUN_LAST,
                        0,
                        NULL, NULL,
                        g_cclosure_marshal_VOID__VOID,
                        G_TYPE_NONE, 0 );

        cdos_expo_workspace_thumbnail_signals [DRAG_END] =
                g_signal_new ("drag-end",
                        G_TYPE_FROM_CLASS (klass),
                        G_SIGNAL_RUN_LAST,
                        0,
                        NULL, NULL,
                        g_cclosure_marshal_VOID__VOID,
                        G_TYPE_NONE, 0 );
}

static void
cdos_expo_workspace_thumbnail_init (CdosExpoWorkspaceThumbnail *self) {
    self->overviewTimeoutId = 0;
    self->hovering = FALSE;
}

static void 
cdos_expo_workspace_thumbnail_post_init (CdosExpoWorkspaceThumbnail *self, MetaWorkspace *metaWorkspace, CdosExpoThumbnailsBox *box) {

	//CdosGlobal *global = cdos_global_get ();
	MetaScreen *global_screen = cdos_global_get_screen (cdos_global_get());

    self->box = box;
    self->metaWorkspace = metaWorkspace;

    self->frame = ST_GROUP (st_group_new ());
    clutter_actor_set_clip_to_allocation ( CLUTTER_ACTOR (self->frame), TRUE );
    st_widget_add_style_class_name (ST_WIDGET (self->frame), "expo-workspace-thumbnail-frame");

    self->actor = st_group_new ();
    clutter_actor_set_reactive ((ClutterActor *)self->actor, TRUE);
    clutter_actor_set_clip_to_allocation ((ClutterActor *)self->actor, TRUE );
    st_widget_add_style_class_name (self->actor, "workspace-thumbnail");

    //this.actor._delegate = this;

    gint w, h;
    cdos_global_get_screen_size (global, &w, &h);
    clutter_actor_set_size ((ClutterActor *)self->actor, (gfloat)w, (gfloat)h);

    self->contents = clutter_group_new ();
    st_container_add_actor ((ClutterContainer *)self->actor, self->contents);

    g_signal_connect (self->actor, "destroy", G_CALLBACK (onDestroy), self);

    self->lastButtonPressTimeStamp = 0;
    self->lastButtonPressActor = NULL;

	g_signal_connect (self->actor, "button-press-event", G_CALLBACK (onButtonPressEvent), self);

	g_signal_connect (self->actor, "button-release-event", G_CALLBACK (onButtonReleaseEvent), self);
	
	g_signal_connect (self->actor, "scroll-event", G_CALLBACK (onScrollEvent), self);

	self->title = ST_ENTRY (st_entry_new (""));
	st_widget_set_style_class_name (ST_WIDGET (self->title), "expo-workspaces-name-entry");
	st_widget_set_track_hover (ST_WIDGET (self->title), TRUE);
	st_widget_set_can_focus (ST_WIDGET (self->title), TRUE);

    self->title_spacing = 0;

    self->titleText = st_entry_get_clutter_text (self->title);
    clutter_text_set_max_length ((ClutterText *)self->titleText, 32);

	g_signal_connect (self->titleText, "key-press-event", G_CALLBACK (onTitleKeyPressEvent), self);
	g_signal_connect (self->titleText, "key-focus-in", G_CALLBACK (onTitleKeyFocusInEvent), self);
	g_signal_connect (self->titleText, "key-focus-out", G_CALLBACK (onTitleKeyFocusOutEvent), self);

	st_entry_set_text (self->title, cdos_get_workspace_name (meta_workspace_index (self->metaWorkspace)));

    self->background = clutter_actor_new ();
    clutter_container_add_actor ((ClutterContainer *)self->contents, self->background);

    self->desktopBackground = meta_background_actor_new (global_screen);

    clutter_actor_add_child (self->background, self->desktopBackground);

    self->backgroundShade = st_bin_new ();
    st_widget_set_style_class_name (self->backgroundShade, "workspace-overview-background-shade");	

    clutter_actor_add_child (self->background, (ClutterActor *)self->backgroundShade);

    clutter_actor_set_size ((ClutterActor *)self->backgroundShade, (gfloat)w, (gfloat)h);


    self->shader = st_bin_new ();
    st_widget_set_style (self->shader, "background-color: black;");

    st_container_add_actor ((ClutterContainer *)self->actor, (ClutterActor *)self->shader);

	clutter_actor_set_size ((ClutterActor *)self->shader, (gfloat)w, (gfloat)h);
        
	clutter_actor_set_opacity ((ClutterActor *)self->shader, INACTIVE_OPACITY);

    if (metaWorkspace == meta_screen_get_active_workspace (global_screen))
        clutter_actor_set_opacity ((ClutterActor *)self->shader, 0);

	//filter方法用指定的函数测试所有元素，并创见一个包含所有通过测试的元素的新数组
        //GList *windows = cdos_global_get_window_actors().filter(self->isMyWindow, self);
        
    GList *windows = cdos_global_get_window_actors (cdos_global_get());  //filter(self->isMyWindow, self);
    GList *window_filters = NULL;

    // 先过滤非本工作区的
    while (windows != NULL) {
        GList *next = windows->next;
        MetaWindowActor *win = windows->data;
        if ( isMyWindow (self, win)){
            window_filters = g_list_append (window_filters, win);
        }
        windows = next;
    }
    // 再过滤非普通窗口的
    GList *yes_windows = NULL;
    while (window_filters != NULL){
        GList *next = window_filters->next;
        MetaWindowActor *win = window_filters->data;
        if (isExpoWindow (self, win)){
            yes_windows = g_list_append (yes_windows, win); 
        }
        window_filters = next;
    }
    // Create clones for windows that should be visible in the Expo
    self->count = 0;
    self->windows = NULL;

// 在这里添加对最小化窗口的过滤试试，初始化的时候…… 貌似不太对的样子
    GList *iter;
    gboolean minimize = FALSE;
    for (iter = yes_windows; iter; iter = iter->next) {

        MetaWindowActor *index_window = iter->data;

        MetaWindow *metaWindow = meta_window_actor_get_meta_window (index_window);
        g_object_get (metaWindow, "minimized", &minimize, NULL);

        if (!minimize)
            addWindowClone (self, index_window);
    }

    g_list_free(window_filters);
    g_list_free(yes_windows);

    self->windowAddedId = g_signal_connect (self->metaWorkspace, "window-added", G_CALLBACK (windowAdded), self);
    self->windowRemovedId = g_signal_connect (self->metaWorkspace, "window-removed", G_CALLBACK (windowRemoved), self);

    self->windowEnteredMonitorId = g_signal_connect (global_screen, "window-entered-monitor", G_CALLBACK (windowEnteredMonitor), self);
    self->windowLeftMonitorId = g_signal_connect (global_screen, "window-left-monitor", G_CALLBACK (windowLeftMonitor), self);

    self->setOverviewModeId = g_signal_connect (box, "set-overview-mode", G_CALLBACK (onSetOverviewMode), self);

    self->stickyAddedId = g_signal_connect (box, "sticky-detected", G_CALLBACK (onStickyDetected), self);

    self->restackedNotifyId = g_signal_connect (global_screen, "restacked", G_CALLBACK (onRestack), self);

    self->isActive = FALSE;
    self->state = THUMBNAIL_STATE_NORMAL;

    restack (self, TRUE);

    self->slidePosition = 0; // Fully slid in

    setOverviewMode (self, FALSE);
}

/**
 * cdos_expo_workspace_thumbnail_new:
 * @metaWorkspace: #MetaWorkspace 工作区对象
 * @box: #CdosExpoThumbnailsBox 对象
 *
 * 新建 #CdosExpoWorkspaceThumbnail 对象，代表一个工作区的缩略图
 */
CdosExpoWorkspaceThumbnail* cdos_expo_workspace_thumbnail_new (MetaWorkspace *metaWorkspace, CdosExpoThumbnailsBox *box) {

    	CdosExpoWorkspaceThumbnail *instance;
    	instance = g_object_new (CDOS_TYPE_EXPO_WORKSPACE_THUMBNAIL, NULL);
        g_object_ref(box);
	cdos_expo_workspace_thumbnail_post_init (instance, metaWorkspace, box);

    	return instance;
}

static void 
overviewModeOnCb (ClutterTimeline *tml, ClutterActor *actor)
{
    clutter_actor_remove_transition (actor, "overview-mode-on");
    clutter_actor_hide (actor);
}

static void
overviewModeOnCb2 (ClutterTimeline *tml, CdosExpoWindowClone *window)
{
    clutter_actor_remove_transition ((ClutterActor *)window, "overview-mode-on");
    clutter_actor_show ((ClutterActor *)window);
    clutter_actor_show (window->icon);
}
/**
 * cdos_expo_workspace_thumbnail_overview_mode_on:
 * @self: #CdosExpoWorkspaceThumbnail 对象
 *
 * 开启overview 模式（未做），当鼠标进入工作区缩略图中时，显示这个工作区中所有窗口的缩略图，效果同 CdosOverview
 *
 */
void
cdos_expo_workspace_thumbnail_overview_mode_on (CdosExpoWorkspaceThumbnail *self)
{
    g_return_if_fail(CDOS_IS_EXPO_WORKSPACE_THUMBNAIL(self));
	gfloat box_scale = cdos_expo_thumbnails_box_get_scale (self->box);

    if (!box_scale) 
		return;

    self->overviewMode = TRUE;
    resetCloneHover (self);

    GList *windows = NULL;

    //遍历list, 如果是overviewWindow就放在新的list内
    GList *iter;
    for (iter = self->windows; iter; iter = iter->next)
    {
        CdosExpoWindowClone *window = iter->data;		

        if (isOverviewWindow (window->metaWindow)) {
            windows = g_list_append (windows, window);
        }
        else
        {
            cdos_transition_to ((ClutterActor *)window, CLUTTER_EASE_OUT_QUAD,
                                REARRANGE_TIME_ON, 0,
                                (TransCallback)overviewModeOnCb, window,
                                "overview-mode-on",
                                "scale-x", TRANS_PROP_DOUBLE, 0.f,
                                "scale-y", TRANS_PROP_DOUBLE, 0.f,
                                NULL);
        }
    }

    // top-to-bottom order
    windows = g_list_reverse (windows);

    CdosLayoutManager *layout_manager = cdos_monkey_plugin_get_layout_manager (monkey_plugin_get ());
    GPtrArray *monitors = cdos_layout_manager_get_monitors (layout_manager);

    //遍历layoutManager中monitor
    gint monitorIndex;
    for (monitorIndex = 0; monitors && monitorIndex < monitors->len; monitorIndex++) {

        MetaRectangle *monitor = monitors->pdata[monitorIndex];

        GList *monitorWindows = NULL;
        GList *it; //遍历self->windows, 把所有符合同一个monitor条件的window放到monitorWindows内
        for (it = windows; it; it = it->next)
        {
            CdosExpoWindowClone *window = it->data;
            MetaWindow *metaWindow = window->metaWindow;

            if (monitorIndex == meta_window_get_monitor (metaWindow))
                monitorWindows = g_list_append (monitorWindows, window);
        }                 

        gfloat spacing = 14.0;
        gint nWindows = g_list_length (monitorWindows);
        //[nCols, nRows]  = [Math.ceil(Math.sqrt(nWindows)), Math.round(Math.sqrt(nWindows))]
        gint nCols = ceil (sqrt (nWindows));
        gint nRows = round (sqrt (nWindows));
        gfloat maxWindowWidth = (monitor->width - (spacing * (nCols+1))) / nCols;
        gfloat maxWindowHeight = (monitor->height - (spacing * (nRows+1))) / nRows;

        gint col = 1;
        gint row = 1;

        gint lastRowCols = nWindows - ((nRows - 1) * nCols);
        gint lastRowOffset = (monitor->width - (maxWindowWidth * lastRowCols) - (spacing * (lastRowCols+1))) / 2;
        gint offset = 0;

        for (it = monitorWindows; it; it = it->next)
        {
            CdosExpoWindowClone *window = it->data;

            if (window->inDrag)
                return;

            cdos_expo_window_clone_refresh_clone (window, TRUE);
            cdos_expo_window_clone_show_urgency_state (window, FALSE, 0);

            if (row == nRows)
                offset = lastRowOffset;

            //let [wWidth, wHeight] = [window.realWindow.width, window.realWindow.height];
            gfloat wWidth = clutter_actor_get_width (CLUTTER_ACTOR (window->realWindow));
            gfloat wHeight = clutter_actor_get_height (CLUTTER_ACTOR (window->realWindow));

            //两个取最小再与1比较
            gdouble scale = fmin (1, fmin (maxWindowWidth / wWidth, maxWindowHeight / wHeight));

            gfloat x = monitor->x + offset + (spacing * col) + (maxWindowWidth * (col - 1)) + ((maxWindowWidth - (wWidth * scale)) / 2);
            gfloat y = monitor->y + (spacing * row) + (maxWindowHeight * (row - 1)) + ((maxWindowHeight - (wHeight * scale)) / 2);

            clutter_actor_set_child_above_sibling ((ClutterActor *)window, window->icon, NULL);

            // all icons should be the same size!
            gdouble iconScale = (0.25 / box_scale / scale);

            clutter_actor_set_scale (window->icon, iconScale, iconScale);

            //let [iconX, iconY] = [ICON_OFFSET / this.box.scale/scale, ICON_OFFSET / this.box.scale/scale];
            gfloat iconX = ICON_OFFSET / box_scale / scale;
            gfloat iconY = ICON_OFFSET / box_scale /scale;
            clutter_actor_set_position (window->icon, iconX, iconY);

            cdos_transition_to ((ClutterActor *)window,
                                CLUTTER_EASE_OUT_QUAD,
                                REARRANGE_TIME_ON,
                                0,
                                (TransCallback)overviewModeOnCb2, window,
                                "overview-mode-on",
                                "x", TRANS_PROP_FLOAT, x,
                                "y", TRANS_PROP_FLOAT, y,
                                "scale-x", TRANS_PROP_DOUBLE, scale,
                                "scale-y", TRANS_PROP_DOUBLE, scale,
                                "opacity", TRANS_PROP_UCHAR, 255,
                                NULL);
            col++;

            if (col > nCols){
                row ++;
                col = 1;
            }
		}
        g_list_free(monitorWindows);
	}
    g_list_free(windows);

}

/**
 * cdos_expo_workspace_thumbnail_overview_mode_off:
 * @self: #CdosExpoWorkspaceThumbnail 对象
 *
 * 关闭overview 模式（未做），当鼠标进入工作区缩略图中时，显示这个工作区中所有窗口的缩略图
 *
 */
void
cdos_expo_workspace_thumbnail_overview_mode_off (CdosExpoWorkspaceThumbnail *self,
                                                 gboolean force,
                                                 gboolean override)
{
    g_return_if_fail(CDOS_IS_EXPO_WORKSPACE_THUMBNAIL(self));
    if (!cdos_expo_thumbnails_box_get_scale (self->box))
        return;

    resetCloneHover (self);

    if (self->overviewMode == FALSE && !force)
        return;

    gint forceOverviewMode = cdos_expo_thumbnails_box_get_force_overview_mode ();//在thumbnails-box全局变量中
    if (forceOverviewMode && !override)
        return;

    self->overviewMode = FALSE;

    //guint iconSpacing = ICON_SIZE/4;

    gint rearrangeTime = force ? REARRANGE_TIME_OFF/2 : REARRANGE_TIME_OFF;

    CdosLayoutManager *layout_manager = cdos_monkey_plugin_get_layout_manager (monkey_plugin_get ());
    GPtrArray *monitors = cdos_layout_manager_get_monitors (layout_manager);

    gint monitorIndex; //遍历monitors
    for ( monitorIndex = 0; monitors && monitorIndex < monitors->len; monitorIndex++)
    {
        GList *iter;
        for (iter = self->windows; iter; iter = iter->next) {

            CdosExpoWindowClone *window = iter->data;
            MetaWindow *metaWindow = window->metaWindow;

            if (monitorIndex == meta_window_get_monitor (metaWindow)) {

                if (window->inDrag)
                    return;

                cdos_expo_window_clone_refresh_clone (window, FALSE);
                cdos_expo_window_clone_show_urgency_state (window, FALSE, 0);

                clutter_actor_hide (window->icon);
                clutter_actor_show ((ClutterActor *)window);

                gboolean a = meta_window_showing_on_its_workspace (metaWindow);
                guint8 opacity = a ? 255 : 127;
                cdos_transition_to ((ClutterActor *)window,
                                    CLUTTER_EASE_OUT_QUAD,
                                    rearrangeTime,
                                    0,
                                    NULL, NULL,
                                    "overview-mode-off",
                                    "x", TRANS_PROP_FLOAT, window->origX,
                                    "y", TRANS_PROP_FLOAT, window->origY,
                                    "scale-x", TRANS_PROP_DOUBLE, 1.0,
                                    "scale-y", TRANS_PROP_DOUBLE, 1.0,
                                    "opacity", TRANS_PROP_UCHAR, opacity,
                                    NULL);
            }
		}
    }
}

/**
 * cdos_expo_workspace_thumbnail_show_keyboard_selected_state:
 * @self: #CdosExpoWorkspaceThumbnail 对象
 * @selected: 布尔值，是否显示为选中状态
 *
 * 高亮显示，并设置为 overview 模式
 */
void
cdos_expo_workspace_thumbnail_show_keyboard_selected_state (CdosExpoWorkspaceThumbnail *self, gboolean selected)
{
    g_return_if_fail(CDOS_IS_EXPO_WORKSPACE_THUMBNAIL(self));
    self->isSelected = selected;

    gchar *name = selected ? "selected" : "";

    clutter_actor_set_name (CLUTTER_ACTOR (self->title), name);		

    if (selected) {
        cdos_expo_workspace_thumbnail_highlight (self);
        cdos_expo_workspace_thumbnail_overview_mode_on (self);
    }
    else 
    {
        self->hovering = false;
        cdos_expo_workspace_thumbnail_overview_mode_off (self, FALSE, FALSE);
        cdos_expo_workspace_thumbnail_shade (self, TRUE);
    }

}

/**
 * cdos_expo_workspace_thumbnail_refresh:
 * @self: #CdosExpoWorkspaceThumbnail 对象
 *
 * 刷新工作区的显示状态（目前并没有什么实际作用），因为没有做工作区 hover 状态的窗口缩略图显示模式。
 *
 */
void cdos_expo_workspace_thumbnail_refresh (CdosExpoWorkspaceThumbnail *self) {
    g_return_if_fail(CDOS_IS_EXPO_WORKSPACE_THUMBNAIL(self));
	refreshTitle (self);
    resetCloneHover (self);
    setOverviewMode (self, self->overviewMode);
}

/**
 * cdos_expo_workspace_thumbnail_remove:
 * @self: #CdosExpoWorkspaceThumbnail 对象
 *
 * 移除工作区缩略图 （没有做，工作区默认四个，不提供增加删除功能）
 */
void cdos_expo_workspace_thumbnail_remove (CdosExpoWorkspaceThumbnail *self) {

        g_return_if_fail(CDOS_IS_EXPO_WORKSPACE_THUMBNAIL(self));
        MetaScreen *global_screen = cdos_global_get_screen (cdos_global_get());

        // this workspace is already being removed
        if (self->doomed)
                return;

        //如果只有一个工作区就不能再移除
        if (meta_screen_get_n_workspaces (global_screen) <= 1)
                return;

        if (!cdos_has_default_workspace_name (meta_workspace_index (self->metaWorkspace))) {

                //CdosConfirmDialog *confirm;

                cdos_expo_workspace_thumbnail_overview_mode_on (self);

                //进行动画效果
                cdos_expo_workspace_thumbnail_highlight (self);

                //let prompt = _("Are you sure you want to remove workspace \"%s\"?\n\n").format(
                cdos_get_workspace_name (meta_workspace_index (self->metaWorkspace));

                //建立一个确认要不要删除workspace的对话框
                //let confirm = new ConfirmationDialog(prompt, removeAction, true);
                //confirm.open();

                //gboolean cdos_modal_dialog_open (ModalDialog *modal, guint32 timestamp);
        }
        else
                removeAction (self);
}

/**
 * cdos_expo_workspace_thumbnail_activate_workspace:
 * @self: #CdosExpoWorkspaceThumbnail 对象
 *
 * 激活这个缩略图对应的工作区，进入这个工作区。
 */
void cdos_expo_workspace_thumbnail_activate_workspace (CdosExpoWorkspaceThumbnail *self) {

    g_return_if_fail(CDOS_IS_EXPO_WORKSPACE_THUMBNAIL(self));
    MetaScreen *global_screen = cdos_global_get_screen (cdos_global_get());
    CdosExpo *main_expo = cdos_monkey_plugin_get_expo ();

    //如果  
    if (self->metaWorkspace != meta_screen_get_active_workspace (global_screen) ) {

        guint timestamp = cdos_global_get_current_time (cdos_global_get());

        meta_workspace_activate (self->metaWorkspace, timestamp);
    }

    cdos_expo_hide (main_expo, FALSE); //options
}

/**
 * cdos_expo_workspace_thumbnail_set_active:
 * @self: #CdosExpoWorkspaceThumbnail 对象
 * @isActive: 是否激活
 *
 * 设置这个工作区缩略图是否为 active 状态，比如切换工作区后，对应的 active 工作区就会修改为新的工作区。
 */
void cdos_expo_workspace_thumbnail_set_active (CdosExpoWorkspaceThumbnail *self, gboolean isActive)
{
    g_return_if_fail(CDOS_IS_EXPO_WORKSPACE_THUMBNAIL(self));
    self->isActive = isActive;
    //this.frame.name = isActive ? 'active' : '';
	if (isActive)
		clutter_actor_set_name (CLUTTER_ACTOR (self->frame), "active");
    else
		clutter_actor_set_name (CLUTTER_ACTOR (self->frame), "");
}
static void
on_shade_transitions_completed (ClutterTimeline *tml, ClutterActor *actor)
{
    //CdosExpoWorkspaceThumbnail *actor_shader = (CdosExpoWorkspaceThumbnail *)data;
    clutter_actor_remove_transition (actor, "shade-workspace"); 
}

/**
 * cdos_expo_workspace_thumbnail_shade:
 * @self: #CdosExpoWorkspaceThumbnail 对象
 * @force: 是否强制设置
 *
 * 上下左右键切换工作区时，离开的工作区阴影效果，不再高亮
 */
void cdos_expo_workspace_thumbnail_shade (CdosExpoWorkspaceThumbnail *self,gboolean force)
{
    g_return_if_fail(CDOS_IS_EXPO_WORKSPACE_THUMBNAIL(self));
    if (!self->isSelected || force){
        clutter_actor_remove_all_transitions ((ClutterActor *)self->shader);
        clutter_actor_save_easing_state ((ClutterActor *)self->shader);
        ClutterTransition *trans = clutter_property_transition_new ("opacity");

        clutter_transition_set_from (trans, G_TYPE_UCHAR, 0);
        clutter_transition_set_to (trans, G_TYPE_UCHAR, 120);

        clutter_actor_set_easing_mode ((ClutterActor *)self->shader, CLUTTER_EASE_OUT_QUAD);
        clutter_timeline_set_duration (CLUTTER_TIMELINE (trans), 300);
        clutter_timeline_set_repeat_count (CLUTTER_TIMELINE (trans), 0);

        clutter_actor_add_transition ((ClutterActor *)self->shader, "shade-workspace", trans);

        g_signal_connect (trans, "completed", G_CALLBACK (on_shade_transitions_completed),self->shader);
        clutter_actor_restore_easing_state ((ClutterActor *)self->shader); 
    }
}

static void
on_highlight_transitions_completed (ClutterTimeline *tml, ClutterActor *actor){
    //CdosExpoWorkspaceThumbnail *actor_shader = (CdosExpoWorkspaceThumbnail *)data;
    clutter_actor_remove_transition (actor, "highlight-workspace"); 
}

/**
 * cdos_expo_workspace_thumbnail_highlight:
 * @self: #CdosExpoWorkspaceThumbnail 对象
 *
 * 当进入某个工作区缩略图时，高亮显示
 */
void cdos_expo_workspace_thumbnail_highlight (CdosExpoWorkspaceThumbnail *self)
{
    g_return_if_fail(CDOS_IS_EXPO_WORKSPACE_THUMBNAIL(self));
    clutter_actor_remove_all_transitions ((ClutterActor *)self->shader);
    clutter_actor_save_easing_state ((ClutterActor *)self->shader);
    ClutterTransition *trans = clutter_property_transition_new ("opacity");

    clutter_transition_set_from (trans, G_TYPE_UCHAR, 120);
    clutter_transition_set_to (trans, G_TYPE_UCHAR, 0);

    clutter_actor_set_easing_mode ((ClutterActor *)self->shader, CLUTTER_EASE_OUT_QUAD);
    clutter_timeline_set_duration (CLUTTER_TIMELINE (trans), 300);
    clutter_timeline_set_repeat_count (CLUTTER_TIMELINE (trans), 0);

    clutter_actor_add_transition ((ClutterActor *)self->shader, "highlight-workspace", trans);

    g_signal_connect (trans, "completed", G_CALLBACK (on_highlight_transitions_completed), self->shader);
    clutter_actor_restore_easing_state ((ClutterActor *)self->shader); 
}


