
#ifndef __CDOS_EXPO_WORKSPACE_THUMBNAIL_H__
#define __CDOS_EXPO_WORKSPACE_THUMBNAIL_H__

#include <glib-object.h>
#include <meta/workspace.h>
#include <meta/window.h>
#include <meta/meta-window-actor.h>
#include <meta/meta-background-actor.h>
#include <meta/boxes.h>
#include <meta/screen.h>
#include <clutter/clutter.h>

#include "st.h"
#include "cdos-expo-thumbnail-private.h"
#include "cdos-tooltip.h"
#include "cdos-window-overlay.h"


GType	cdos_expo_workspace_thumbnail_get_type(void);


#define CDOS_TYPE_EXPO_WORKSPACE_THUMBNAIL (cdos_expo_workspace_thumbnail_get_type())
#define CDOS_EXPO_WORKSPACE_THUMBNAIL(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj),CDOS_TYPE_EXPO_WORKSPACE_THUMBNAIL,CdosExpoWorkspaceThumbnail))
#define CDOS_IS_EXPO_WORKSPACE_THUMBNAIL(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj),CDOS_TYPE_EXPO_WORKSPACE_THUMBNAIL))
#define CDOS_EXPO_WORKSPACE_THUMBNAIL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass),CDOS_TYPE_EXPO_WORKSPACE_THUMBNAIL,CdosExpoWorkspaceThumbnailClass))
#define CDOS_IS_EXPO_WORKSPACE_THUMBNAIL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((obj),CDOS_TYPE_EXPO_WORKSPACE_THUMBNAIL))
#define CDOS_EXPO_WORKSPACE_THUMBNAIL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS((obj),CDOS_TYPE_EXPO_WORKSPACE_THUMBNAIL,CdosExpoWorkspaceThumbnailClass))

typedef struct _CdosExpoWorkspaceThumbnail	 CdosExpoWorkspaceThumbnail;
typedef struct _CdosExpoWorkspaceThumbnailClass		CdosExpoWorkspaceThumbnailClass;
typedef struct _CdosExpoWorkspaceThumbnailPrivate	 CdosExpoWorkspaceThumbnailPrivate;


//外部定义: main中尚未实现的函数
//void cdos_set_workspace_name (gint ws_index, gchar *name);
//gchar *cdos_get_workspace_name (gint ws_index);
//void cdos_remove_workspace (MetaWorkspace *metaWorkspace);
//gboolean cdos_has_default_workspace_name (gint ws_index);
//gboolean cdos_is_window_actor_displayed_on_workspace (MetaWindowActor *win, gint ws_index);


struct _CdosExpoWorkspaceThumbnail {

    GObject parent;
    CdosExpoWorkspaceThumbnailPrivate *priv;

    CdosExpoThumbnailsBox *box;
    MetaWorkspace *metaWorkspace;
    /*ClutterActor *frame;*/  StGroup *frame;
    /*ClutterActor *actor;*/  StWidget *actor; /* StGroup */

    guint lastButtonPressTimeStamp;
    ClutterActor *lastButtonPressActor;

    gboolean doomed;
    gboolean isSelected;

	StEntry *title;
	guint title_spacing;
	ClutterActor *titleText;
	gchar *origTitle;
	gboolean undoTitleEdit;

    ClutterActor *background;	
    ClutterActor *contents;
	ClutterActor *desktopBackground; /* MetaBackgroundActor */
	StWidget *backgroundShade; /* StBin */
	StWidget *shader; /* StBin */

    gint count;
    GList *windows;

	guint windowAddedId;
	guint windowRemovedId;
	guint windowEnteredMonitorId;
	guint windowLeftMonitorId;
	guint setOverviewModeId;
	guint stickyAddedId;
	guint restackedNotifyId;

	gboolean isActive;
	gint state;
    	gboolean hovering;

	gint slidePosition;	//unknow
	gint overviewMode;	//unknow
	GHashTable *stackIndices;

    	CdosTooltip *tooltip;
	CdosExpoWindowClone *lastHoveredClone;
	guint buttonTimeoutId;
	//在expo-thumbnails-box中使用
	CdosPointerTracker *pointerTracker;
	guint overviewTimeoutId;
};


struct _CdosExpoWorkspaceThumbnailClass {

    	GObjectClass parent_class;
};


// thumbmail state 定义 

enum  {
    THUMBNAIL_STATE_NEW,                //0
    THUMBNAIL_STATE_ANIMATING_IN,       //1
    THUMBNAIL_STATE_NORMAL,             //2
    THUMBNAIL_STATE_REMOVING,           //3
    THUMBNAIL_STATE_ANIMATING_OUT,      //4
    THUMBNAIL_STATE_ANIMATED_OUT,       //5
    THUMBNAIL_STATE_COLLAPSING,         //6
    THUMBNAIL_STATE_DESTROYED           //7
};


//公有函数

CdosExpoWorkspaceThumbnail*  cdos_expo_workspace_thumbnail_new (MetaWorkspace *metaWorkspace, CdosExpoThumbnailsBox *box);

//overviewModeOn
void cdos_expo_workspace_thumbnail_overview_mode_on (CdosExpoWorkspaceThumbnail *self);

//overviewModeOff
void cdos_expo_workspace_thumbnail_overview_mode_off (CdosExpoWorkspaceThumbnail *self, gboolean force, gboolean isOverride);

//showKeyboardSelectedState
void cdos_expo_workspace_thumbnail_show_keyboard_selected_state (CdosExpoWorkspaceThumbnail *self, gboolean selected);

//refresh
void cdos_expo_workspace_thumbnail_refresh (CdosExpoWorkspaceThumbnail *self);

//remove
void cdos_expo_workspace_thumbnail_remove (CdosExpoWorkspaceThumbnail *self); 

//activeWorkspace
void cdos_expo_workspace_thumbnail_activate_workspace (CdosExpoWorkspaceThumbnail *self);

//setActive
void cdos_expo_workspace_thumbnail_set_active (CdosExpoWorkspaceThumbnail *self, gboolean isActive);

//shade
void cdos_expo_workspace_thumbnail_shade (CdosExpoWorkspaceThumbnail *self,gboolean force);

//highlight
void cdos_expo_workspace_thumbnail_highlight (CdosExpoWorkspaceThumbnail *self);


void cdos_expo_workspace_thumbnail_destroy (CdosExpoWorkspaceThumbnail *self);

#endif //__CDOS_EXPO_WORKSPACE_THUMBNAIL_H__

