// source:expo.js
#include "config.h"
#include "cdos-expo.h"

#include <meta/boxes.h>
#include "cdos-monkey-plugin.h"
#include "cdos-expo-thumbnails-box.h"
#include "cdos-global.h"
#include "cdos-layout-manager.h"
#include "cdos-start.h"
#include "cdos-base-util.h"
/**
 * SECTION:cdos-expo
 * @Title: CdosExpo
 * @Short_description: 工作区
 * 
 * CdosExpo 负责工作区的显示与隐藏，用户调用组合键 Ctrl-Alt-Up，打开所有工作区的缩略图
 * Escape 键和回车退出，点击工作区进入对应的工作区
 */
#define ANIMATION_TIME 300

enum {
    EXPO_DESTROY,
    EXPO_HIDING,
    EXPO_HIDDEN,
    EXPO_SHOWING,
    EXPO_SHOWN,
    LAST_SIGNAL
};

typedef struct _TransferData
{
    ClutterActor *cover;
    CdosExpo *expo;
    MetaRectangle *monitor;
    ClutterActor *clone;
} TransferData;

typedef struct _TransferDataForshow
{
    ClutterActor *clone;
    CdosExpo *expo;
} TransferDataForshow;

static gint cdos_expo_signals[LAST_SIGNAL] = { 0 };

static void cdos_expo_hide_done (CdosExpo *self);
static void cdos_expo_show_done (CdosExpo *self);

struct _CdosExpoPrivate {

    gboolean animationInProgress;
    gboolean visible;

    ClutterActor *background;
    gdouble spacing;
    ClutterActor *group; //StGroup *group;
    gboolean shown;
    gboolean modal;
    gboolean hideInProgress;
    StWidget *gradient; /* StButton */
    ClutterActor *cover_pane;
    StWidget *add_workspace_button; /* StButton */
    StWidget *window_close_area; /* StButton */
    gboolean workspace_operation_pending;
    CdosExpoThumbnailsBox *expo;
    gulong allocateId;
    GList *clones;
    //gint  (*add_workspace_button_handle_drag_over)  ();
};


G_DEFINE_TYPE  ( CdosExpo, cdos_expo, G_TYPE_OBJECT );

#define CDOS_EXPO_GET_PRIVATE( obj ) \
  (G_TYPE_INSTANCE_GET_PRIVATE( (obj), CDOS_TYPE_EXPO, CdosExpoPrivate ) )



static void relayout (CdosExpo *self) {

	CdosLayoutManager *layout_manager = cdos_monkey_plugin_get_layout_manager (monkey_plugin_get ());

    	CdosExpoPrivate *priv = CDOS_EXPO_GET_PRIVATE (self);

    	if (!priv->expo){
    		return ;
    	}

    	cdos_expo_hide (self, FALSE);

    	MetaRectangle *primary = cdos_layout_manager_get_primary_monitor (layout_manager);

    	//gboolean rtl = st_widget_get_default_direction () == ST_TEXT_DIRECTION_RTL ? TRUE : FALSE;

    	//gint contentY = 0;
    	gint contentHeight = primary->height;

    	clutter_actor_set_position (priv->group, primary->x, primary->y);
    	clutter_actor_set_size (priv->group, primary->width, primary->height);

    	clutter_actor_set_position (CLUTTER_ACTOR (priv->gradient), 0, 0);
    	clutter_actor_set_size (CLUTTER_ACTOR (priv->gradient), primary->width, primary->height);

    	clutter_actor_set_position (priv->cover_pane, 0, 0);
    	clutter_actor_set_size (priv->cover_pane, primary->width, contentHeight);

    	//gint viewWidth = primary->width - priv->spacing;
    	//gint viewHeight = contentHeight - 2 * priv->spacing;
    	//gint viewY = contentY + priv->spacing;
    	//gint viewX = rtl ? 0 : priv->spacing;

    	StThemeNode *node = st_widget_get_theme_node ((StWidget *)priv->add_workspace_button);
    	gdouble buttonWidth = st_theme_node_get_length (node, "width");
    	gdouble buttonHeight = st_theme_node_get_length (node, "height");

    	node = st_widget_get_theme_node  ((StWidget *)priv->window_close_area);
    	gdouble window_close_area_width = st_theme_node_get_length  (node, "width");
    	gdouble window_close_area_height = st_theme_node_get_length  (node, "height");

    	clutter_actor_set_position ((ClutterActor *)priv->expo, 0, 0);
    	clutter_actor_set_size ((ClutterActor *)priv->expo, primary->width - buttonWidth, primary->height);

    	gint buttonY =  (primary->height - buttonHeight)/2.0 ;

    	clutter_actor_set_position (CLUTTER_ACTOR (priv->add_workspace_button), (primary->width - buttonWidth), buttonY);
    	clutter_actor_set_size (CLUTTER_ACTOR (priv->add_workspace_button), buttonWidth, buttonHeight);

    	if (!st_theme_node_get_background_image (node)){
        	st_widget_set_style (ST_WIDGET (priv->add_workspace_button),
                    "background-image:url ('/usr/share/cdos-de/theme/add-workspace.png');");
    	}

    	clutter_actor_set_position (CLUTTER_ACTOR (priv->window_close_area), 
							(primary->width - window_close_area_width) / 2.0,
							primary->height);

    	clutter_actor_set_size (CLUTTER_ACTOR (priv->window_close_area), window_close_area_width, window_close_area_height);

	st_container_raise ((ClutterContainer *)priv->group, (ClutterActor *)priv->window_close_area, NULL);
}

static void onRelayout (CdosLayoutManager *cdosLayoutManager, CdosExpo *self) {

    	relayout (self);
}

static void onStyleChanged (GObject *gobject, CdosExpo *self) {

	//StGroup *group = ST_GROUP (gobject);
	ClutterActor *group = CLUTTER_ACTOR (gobject);
    CdosExpoPrivate *priv = CDOS_EXPO_GET_PRIVATE (self);

    	StThemeNode *node  = st_widget_get_theme_node  (ST_WIDGET (group));

    	gdouble spacing = st_theme_node_get_length  (node, "spacing");

    	if (spacing != priv->spacing) {

        	priv->spacing = spacing;
        	relayout (self);
    	}
}

static gboolean onEvent (ClutterActor *actor, ClutterEvent *event, CdosExpo *self) {

	return TRUE;
}

static void onClicked (StButton *button, gint clicked_button, CdosExpo *self) {
    //notice：处理好入口接口后，取消注释，原函数是私有接口，需要转为公有接口；
    //cdos_start_add_workspace ();
}



static gboolean onKeyPressEvent (ClutterActor *actor, ClutterEvent *event, CdosExpo *self){

    CdosExpoPrivate *priv = CDOS_EXPO_GET_PRIVATE (self);

    	guint ctrl_alt_mask = CLUTTER_CONTROL_MASK | CLUTTER_MOD1_MASK;

    	if (priv->shown) {
            cdos_debug ();
        	if (cdos_expo_thumbnails_box_handle_key_press_event (priv->expo, actor, event)){
            		return TRUE;
        	}

        	guint symbol = clutter_event_get_key_symbol (event);

        	if (symbol == CLUTTER_Insert || symbol == CLUTTER_plus){
            		priv->workspace_operation_pending = TRUE;
        	}

        	ClutterModifierType modifiers = cdos_base_util_get_event_state (event);

        	if ( (symbol == CLUTTER_Delete &&  (modifiers & ctrl_alt_mask) != ctrl_alt_mask) || 
							(symbol == CLUTTER_W &&  (modifiers & CLUTTER_CONTROL_MASK))){

            		priv->workspace_operation_pending = TRUE;
        	}

        	if (symbol == CLUTTER_Escape){
            		if (!priv->workspace_operation_pending){
                		cdos_expo_hide (self,FALSE);
            		}
            		priv->workspace_operation_pending = FALSE;
            		return TRUE;
        	}	
    	}
    	return FALSE;
}

static gboolean onKeyReleaseEvent (ClutterActor *actor, ClutterEvent *event, CdosExpo *self) {

    CdosExpoPrivate *priv = CDOS_EXPO_GET_PRIVATE (self);

   	guint ctrl_alt_mask = CLUTTER_CONTROL_MASK | CLUTTER_MOD1_MASK;

    	if (priv->shown) {

        	guint symbol = clutter_event_get_key_symbol (event);

        	if (symbol == CLUTTER_Insert || symbol == CLUTTER_plus) {
            		if (priv->workspace_operation_pending){

                		priv->workspace_operation_pending = FALSE;
                		cdos_start_add_workspace ();
            		}
            		return TRUE;
        	}

        	ClutterModifierType modifiers = cdos_base_util_get_event_state (event);

        	if ( (symbol == CLUTTER_Delete &&  (modifiers & ctrl_alt_mask) != ctrl_alt_mask) || 
							(symbol == CLUTTER_W &&  (modifiers & CLUTTER_CONTROL_MASK)))
		{
            		if (priv->workspace_operation_pending) {
                		priv->workspace_operation_pending = FALSE;
                		cdos_expo_thumbnails_box_remove_selected_workspace (priv->expo);
            		}
            		return TRUE;
        	}
    	}
    	return FALSE;
}


//固定函数

static void cdos_expo_dispose (GObject *object){

    	CdosExpo *self = CDOS_EXPO (object);
    CdosExpoPrivate *priv = CDOS_EXPO_GET_PRIVATE (self);
	MetaScreen *global_screen = cdos_global_get_screen (cdos_global_get());

    	cdos_expo_hide (self,FALSE);
    	ClutterActor *global_overlay_group = meta_get_stage_for_screen_for_screen  (global_screen);

        if(priv->gradient){
            g_clear_object(&priv->gradient);
        }
        if(priv->cover_pane){
            g_clear_object(&priv->cover_pane);
        }
        if(priv->add_workspace_button){
            g_clear_object(&priv->add_workspace_button);
        }
        if(priv->window_close_area){
            g_clear_object(&priv->window_close_area);
        }
        if(priv->expo){
            g_clear_object(&priv->expo);
        }
    	if (priv->background){
        	clutter_container_remove_actor (global_overlay_group,priv->background);
            clutter_actor_destroy ((ClutterActor *)priv->background);
    	}
    	if (priv->group){
        	clutter_container_remove_actor ((ClutterContainer *)global_overlay_group,CLUTTER_ACTOR (priv->group));
    	}
		GList *list = priv->clones;
		GList *it = NULL;
		for (it = list;it;it = it->next){
			clutter_actor_remove_child(global_overlay_group,it->data);
			//g_clear_object (&it->data);
			it->data = NULL;
		}
		g_list_free (list);
		priv->clones = NULL;

    	G_OBJECT_CLASS (cdos_expo_parent_class)->dispose (object);
}

static void cdos_expo_finalize (GObject *object){
    CdosExpo *self = CDOS_EXPO (object);
    CdosExpoPrivate *priv = CDOS_EXPO_GET_PRIVATE (self);

    G_OBJECT_CLASS (cdos_expo_parent_class)->finalize (object);

}

static void
cdos_expo_class_init (CdosExpoClass *klass)
{
    	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

	g_type_class_add_private (klass, sizeof(CdosExpoPrivate));

    	gobject_class->dispose = cdos_expo_dispose;
    	gobject_class->finalize = cdos_expo_finalize;

/*
    	klass->destroy = NULL;
    	klass->hidden = NULL;
    	klass->hiding = NULL;
    	klass->showing = NULL;
    	klass->shown = NULL;
*/

    	cdos_expo_signals[EXPO_DESTROY]   =
            g_signal_new (
                    "expo",
                    G_TYPE_FROM_CLASS (gobject_class),
                    G_SIGNAL_RUN_LAST,
                    0,
                    NULL,NULL,
                    g_cclosure_marshal_VOID__OBJECT,
                    G_TYPE_NONE,
                    0
            );
    	cdos_expo_signals[EXPO_HIDDEN]   =
            g_signal_new (
                    "expo-hidden",
                    G_TYPE_FROM_CLASS (gobject_class),
                    G_SIGNAL_RUN_LAST,
                    0,
                    NULL,NULL,
                    g_cclosure_marshal_VOID__OBJECT,
                    G_TYPE_NONE,
                    0
            );
    	cdos_expo_signals[EXPO_HIDING]   =
            g_signal_new (
                    "expo-hiding",
                    G_TYPE_FROM_CLASS (gobject_class),
                    G_SIGNAL_RUN_LAST,
                    0,
                    NULL,NULL,
                    g_cclosure_marshal_VOID__OBJECT,
                    G_TYPE_NONE,
                    0
            );
    	cdos_expo_signals[EXPO_SHOWING]   =
            g_signal_new (
                    "expo-showing",
                    G_TYPE_FROM_CLASS (gobject_class),
                    G_SIGNAL_RUN_LAST,
                    0,
                    NULL,NULL,
                    g_cclosure_marshal_VOID__OBJECT,
                    G_TYPE_NONE,
                    0
            );
    	cdos_expo_signals[EXPO_SHOWN]   =
            g_signal_new (
                    "expo-shown",
                    G_TYPE_FROM_CLASS (gobject_class),
                    G_SIGNAL_RUN_LAST,
                    0,
                    NULL,NULL,
                    g_cclosure_marshal_VOID__OBJECT,
                    G_TYPE_NONE,
                    0
            );

}

static void
cdos_expo_init (CdosExpo *self)
{
    CdosExpoPrivate *priv = self->priv = CDOS_EXPO_GET_PRIVATE (self);

	CdosLayoutManager *layout_manager = cdos_monkey_plugin_get_layout_manager (monkey_plugin_get ());

    g_signal_connect (layout_manager, "monitors-changed", G_CALLBACK (onRelayout), self);
}



static void cdos_expo_show_close_area (CdosExpoThumbnailsBox *expo, CdosExpo *self) {

    CdosExpoPrivate *priv = CDOS_EXPO_GET_PRIVATE (self);
	CdosLayoutManager *layout_manager = cdos_monkey_plugin_get_layout_manager (monkey_plugin_get ());
    MetaRectangle *primary = cdos_layout_manager_get_primary_monitor (layout_manager);

    clutter_actor_show (CLUTTER_ACTOR (priv->window_close_area));
    gfloat y = (gfloat)primary->height - clutter_actor_get_height ((ClutterActor *)priv->window_close_area);
    cdos_transition_to ((ClutterActor *)priv->window_close_area,
                        CLUTTER_EASE_OUT_QUAD,
                        ANIMATION_TIME,
                        0,
                        NULL, NULL,
                        "show-close-area",
                        "y", TRANS_PROP_FLOAT, y,
                        NULL);
}

static void cdos_expo_hide_close_area (CdosExpoThumbnailsBox *expo, CdosExpo *self) {

    	//CdosExpoPrivate *priv = self->priv;
	    //CdosLayoutManager *layout_manager = cdos_monkey_plugin_get_layout_manager (monkey_plugin_get ());
    	//MetaRectangle *primary = cdos_layout_manager_get_primary_monitor (layout_manager);

    	//notice:需要添加tweener动画
}

static void
on_clone_show_transitions_completed(ClutterTimeline *tml, gpointer data){
        MetaScreen *screen = cdos_global_get_screen (cdos_global_get ());
    	ClutterActor *global_overlay_group = meta_get_stage_for_screen_for_screen  (screen);
        TransferDataForshow *transfer = (TransferDataForshow *)data;
        //CdosExpoPrivate *priv = CDOS_EXPO_GET_PRIVATE (transfer->expo);
        clutter_actor_destroy (transfer->clone);
        cdos_expo_show_done (transfer->expo);
        g_free (data);

}


//used in cdos_expo_expo_allocated_callback
static void ea_monitors_foreach (MetaRectangle *monitor, ClutterActor *clone,  CdosExpo *self)
{
    CdosExpoPrivate *priv = CDOS_EXPO_GET_PRIVATE (self);
    //CdosLayoutManager *layout_manager = cdos_monkey_plugin_get_layout_manager (monkey_plugin_get ());

    CdosExpoWorkspaceThumbnail *active_workspace = cdos_expo_thumbnails_box_get_last_active_workspace (priv->expo);
    ClutterActor *active_workspace_actor = CLUTTER_ACTOR (active_workspace->actor);

    ClutterActorBox allocation;
    clutter_actor_get_allocation_box (CLUTTER_ACTOR (active_workspace_actor), &allocation);

    gfloat x = monitor->x + allocation.x1;
    gfloat y = monitor->y + allocation.y1;
    gdouble scale_x , scale_y;
    clutter_actor_get_scale (active_workspace_actor, &scale_x, &scale_y);

    TransferDataForshow *data = g_malloc0 (sizeof (TransferDataForshow));
    data->clone = clone;
    data->expo = self;

    cdos_transition_to (clone,
                        CLUTTER_EASE_OUT_QUAD,
                        ANIMATION_TIME,
                        0,
                        (TransCallback)on_clone_show_transitions_completed, data,
                        "show-all-workspace",
                        "x", TRANS_PROP_FLOAT, x,
                        "y", TRANS_PROP_FLOAT, y,
                        "scale-x", TRANS_PROP_DOUBLE, scale_x,
                        "scale-y", TRANS_PROP_DOUBLE, scale_y,
                        NULL);

}

static void onAllocated (CdosExpoThumbnailsBox *expo, CdosExpo *self)
{
    CdosExpoPrivate *priv = CDOS_EXPO_GET_PRIVATE (self);

    g_signal_handler_disconnect (priv->expo,priv->allocateId);

    CdosLayoutManager *layout_manager = cdos_layout_manager_get_default ();
    GPtrArray *monitors = cdos_layout_manager_get_monitors (layout_manager);

    //g_ptr_array_foreach ();
    CdosExpoWorkspaceThumbnail *active_workspace = cdos_expo_thumbnails_box_get_last_active_workspace (priv->expo);
    ClutterActor *active_workspace_actor = CLUTTER_ACTOR (active_workspace->actor);
    MetaScreen *screen = cdos_global_get_screen (global);
    ClutterActor *global_overlay_group = meta_get_stage_for_screen_for_screen  (screen);

    guint i;
    for (i = 0; i< monitors->len; i++){
        ClutterActor *clone = clutter_clone_new (active_workspace_actor);
        //clutter_actor_add_child (global_overlay_group, clone);
        clutter_container_add_actor ((ClutterContainer *)global_overlay_group, clone);
        MetaRectangle *monitor = monitors->pdata[i];
        clutter_actor_set_clip (clone, monitor->x, monitor->y, monitor->width, monitor->height);
        ea_monitors_foreach (monitor, clone, self);
    }
}

static void animateVisible (CdosExpo *self)
{
    CdosExpoPrivate *priv = self->priv;

    if (priv->visible || priv->animationInProgress)
        return ;

    priv->visible = TRUE;
    priv->animationInProgress = TRUE;

    MetaScreen *screen = cdos_global_get_screen (global);
    meta_disable_unredirect_for_screen (screen);

    ClutterActor *window_group;
    window_group = cdos_global_get_window_group (global);
    clutter_actor_hide (window_group);

    clutter_actor_show (priv->group);
    clutter_actor_show (priv->background);
    clutter_actor_show (CLUTTER_ACTOR (priv->add_workspace_button));
    cdos_expo_thumbnails_box_show (priv->expo);

    //g_signal_connect (priv->expo,"drag-begin",G_CALLBACK (cdos_expo_show_close_area),self);
    //g_signal_connect (priv->expo,"drag-end",G_CALLBACK (cdos_expo_hide_close_area),self);

    /*
    CdosLayoutManager *layout_manager = cdos_monkey_plugin_get_layout_manager (monkey_plugin_get ());
    GPtrArray *monitors = cdos_layout_manager_get_monitors (layout_manager);
    */

    priv->allocateId = g_signal_connect (priv->expo, "allocated", G_CALLBACK (onAllocated), self);

    clutter_actor_show ((ClutterActor *)priv->gradient);
    cdos_start_disable_panels ();

    g_object_set ((GObject *)priv->background, "dim-factor", 1.0, NULL);
    cdos_transition_to (priv->background,
                        CLUTTER_EASE_OUT_QUAD,
                        ANIMATION_TIME,
                        0,
                        NULL, NULL,
                        "dim-background",
                        "dim-factor", TRANS_PROP_FLOAT, 0.4,
                        NULL);
    
    st_container_raise ((ClutterContainer *)priv->group, priv->cover_pane, NULL);
    clutter_actor_show (priv->cover_pane);

    g_signal_emit (self, cdos_expo_signals[EXPO_SHOWING], 0, NULL);
}

static void syncInputMode (CdosExpo *self)
{
    CdosExpoPrivate *priv = self->priv;

    if (priv->animationInProgress)
        return ;

    if (priv->shown) {
        if (!priv->modal) {
            if (cdos_push_modal (priv->group, 0)) {
                priv->modal = TRUE;
            }
            else
            {
                cdos_expo_hide (self, FALSE);
            }
        }
    }
    else
    {
        gint global_stage_input_mode = cdos_global_get_stage_input_mode (global);

        if (priv->modal) {
            cdos_pop_modal (priv->group, 0);
            priv->modal = FALSE;
        }
        /*else if  (global_stage_input_mode == CDOS_STAGE_INPUT_MODE_FULLSCREEN) {

            cdos_global_set_stage_input_mode (global, CDOS_STAGE_INPUT_MODE_NORMAL);
        }*/
    }
}

static void
on_clone_transitions_completed (ClutterTimeline *tml, gpointer data){
    //MetaScreen *screen = cdos_global_get_screen (global);
    //ClutterActor *global_overlay_group = meta_get_stage_for_screen_for_screen  (screen);
    TransferData *transfer = (TransferData *)data;
    CdosExpoPrivate *priv = CDOS_EXPO_GET_PRIVATE (transfer->expo);
    clutter_actor_destroy (transfer->cover);
    CdosLayoutManager *layout = cdos_layout_manager_get_default ();
    GPtrArray *monitors = cdos_layout_manager_get_monitors (layout);
    MetaRectangle *monitor = transfer->monitor;
    if (monitor == monitors->pdata[monitors->len - 1])
    {
        clutter_actor_hide (priv->group);
        cdos_expo_hide_done (transfer->expo);
    }
    clutter_actor_destroy (transfer->clone);
    g_free (data);
}

static void anv_monitors_foreach (MetaRectangle *monitor, CdosExpo *self) {

    CdosExpoPrivate *priv = CDOS_EXPO_GET_PRIVATE (self);
   	MetaScreen *global_screen = cdos_global_get_screen (cdos_global_get());	
	
    CdosExpoWorkspaceThumbnail *active_workspace = cdos_expo_thumbnails_box_get_last_active_workspace (priv->expo);
    //StWidget *active_workspace_actor = cdos_expo_workspace_thumbnail_get_actor (active_workspace);
    ClutterActor *active_workspace_actor = CLUTTER_ACTOR (active_workspace->actor);

    ClutterActor *cover = clutter_group_new ();
    //ClutterActor *cover = clutter_actor_new ();

    ClutterActor *global_overlay_group = meta_get_stage_for_screen_for_screen  (global_screen);
    clutter_container_add_actor ((ClutterContainer *)global_overlay_group, cover);
    clutter_actor_set_position (cover, 0, 0);
    clutter_actor_set_clip (cover, monitor->x, monitor->y, monitor->width, monitor->height);

    ClutterActor *clone = clutter_clone_new (active_workspace_actor);
    //clutter_actor_add_child (cover,clone);
    clutter_container_add_actor ((ClutterContainer *)cover, clone);

    ClutterActorBox allocation;
    clutter_actor_get_allocation_box (active_workspace_actor, &allocation);

    CdosLayoutManager *layout_manager = cdos_layout_manager_get_default ();
    MetaRectangle *primary_monitor = cdos_layout_manager_get_primary_monitor (layout_manager);

    gfloat clone_x = primary_monitor->x + allocation.x1;
    gfloat clone_y = primary_monitor->y + allocation.y1;

    clutter_actor_set_position (clone, clone_x, clone_y);
    clutter_actor_set_clip (clone, monitor->x, monitor->y, monitor->width, monitor->height);

    gdouble active_workspace_actor_scale_x, active_workspace_actor_scale_y;

    clutter_actor_get_scale  (active_workspace_actor, &active_workspace_actor_scale_x, &active_workspace_actor_scale_y);
    clutter_actor_set_scale (clone, active_workspace_actor_scale_x, active_workspace_actor_scale_y);

    TransferData *data = g_malloc0 (sizeof (TransferData));
    data->cover = cover;
    data->expo  = self;
    data->monitor = monitor;
    data->clone = clone;

    cdos_transition_to (clone,
                        CLUTTER_EASE_OUT_QUAD,
                        ANIMATION_TIME,
                        0,
                        (TransCallback)on_clone_transitions_completed, data,
                        "",
                        "x", TRANS_PROP_FLOAT, 0.0,
                        "y", TRANS_PROP_FLOAT, 0.0,
                        "scale-x", TRANS_PROP_DOUBLE, 1.0,
                        "scale-y", TRANS_PROP_DOUBLE, 1.0,
                        NULL);
}

static void animateNotVisible (CdosExpo *self, gboolean options) {

    CdosExpoPrivate *priv = CDOS_EXPO_GET_PRIVATE (self);

    if (!priv->visible || priv->animationInProgress)
        return ;
    
    priv->animationInProgress = TRUE;
    priv->hideInProgress = TRUE;

    CdosExpoWorkspaceThumbnail *active_workspace;

    active_workspace = cdos_expo_thumbnails_box_get_last_active_workspace (priv->expo);

    if (!options)  {
        cdos_start_enable_panels ();
        cdos_expo_workspace_thumbnail_overview_mode_off (active_workspace, TRUE, TRUE);
    }

    CdosLayoutManager *layout_manager = cdos_monkey_plugin_get_layout_manager (monkey_plugin_get ());
    GPtrArray *monitors = cdos_layout_manager_get_monitors (layout_manager);

    g_ptr_array_foreach (monitors, (GFunc) anv_monitors_foreach, self);

    g_signal_emit (self, cdos_expo_signals[EXPO_HIDING], 0, NULL);
}

static void cdos_expo_show_done (CdosExpo *self) {

        CdosExpoPrivate *priv = self->priv;

        priv->animationInProgress = FALSE;
        clutter_actor_hide (priv->cover_pane);

        cdos_debug ();
        g_signal_emit (self, cdos_expo_signals[EXPO_SHOWN], 0, NULL);

        if (!priv->shown)
                animateNotVisible (self, FALSE);

        syncInputMode (self);
        cdos_global_sync_pointer (global);
}

static void cdos_expo_hide_done (CdosExpo *self) {

    CdosExpoPrivate *priv = CDOS_EXPO_GET_PRIVATE (self);
        //MetaScreen *global_screen = cdos_global_get_screen (global);
        MetaScreen *screen = cdos_global_get_screen (global);
        meta_enable_unredirect_for_screen (screen);

        ClutterActor *window_group;
	window_group = cdos_global_get_window_group (global);
        clutter_actor_show (window_group);

        cdos_expo_thumbnails_box_hide (priv->expo);
        //g_object_unref (priv->expo);
        clutter_actor_destroy ((ClutterActor *)priv->expo);
        priv->expo = NULL;

        clutter_actor_hide ((ClutterActor *)priv->add_workspace_button);
        clutter_actor_hide ((ClutterActor *)priv->window_close_area);
        clutter_actor_hide (priv->background);
        clutter_actor_hide ((ClutterActor *)priv->gradient);

        priv->visible = FALSE;
        priv->animationInProgress = FALSE;
        priv->hideInProgress = FALSE;

        clutter_actor_hide (priv->cover_pane);

        g_signal_emit (self, cdos_expo_signals[EXPO_HIDDEN], 0, NULL);

        if (priv->shown)
            animateVisible (self);


        syncInputMode (self);
        ClutterActor *overlay_group = meta_get_stage_for_screen_for_screen (screen);

        clutter_container_remove_actor ((ClutterContainer *)overlay_group, priv->group);
        //clutter_actor_destroy (priv->group);
        priv->group = NULL;

        clutter_container_remove_actor ((ClutterContainer *)overlay_group, priv->background);
        //clutter_actor_destroy (priv->background);

        CdosLayoutManager *layout_manager = cdos_layout_manager_get_default ();
        cdos_layout_manager_update_regions (layout_manager);
}

/**
 * cdos_expo_new:
 *
 * 新建对象 #CdosExpo.
 *
 * Return value:  #CdosExpo 对象.
 */
CdosExpo* cdos_expo_new (void) {

    	CdosExpo *self;
    	self = g_object_new (CDOS_TYPE_EXPO, NULL);
    	return self;
}

static void cdos_expo_before_show (CdosExpo *self) {

    CdosExpoPrivate *priv = CDOS_EXPO_GET_PRIVATE (self);

	MetaScreen *global_screen = cdos_global_get_screen (cdos_global_get());
    	//priv->background = meta_get_stage_for_screen_for_screen  (global_screen);

	// The main BackgroundActor is inside global.window_group which is
        // hidden when displaying the overview, so we create a new
        // one. Instances of this class share a single CoglTexture behind the
        // scenes which allows us to show the background with different
        // rendering options without duplicating the texture data.
    	
    	MetaRectangle *monitor = g_new0(MetaRectangle,1);
	priv->background = meta_background_actor_new  (global_screen,monitor);

    	clutter_actor_hide (priv->background);

        /* overlay_group 是 ClutterGroup */
    	ClutterActor *global_overlay_group = meta_get_stage_for_screen_for_screen  (global_screen);

        clutter_container_add_actor ((ClutterContainer *)global_overlay_group, priv->background);

    	priv->spacing = 0;

    	priv->group = (ClutterActor *)st_group_new ();
    	clutter_actor_set_name (priv->group, "expo");
    	clutter_actor_set_reactive (priv->group, TRUE);

    	g_signal_connect (priv->group, "style-changed", G_CALLBACK (onStyleChanged), self);

    	priv->visible = FALSE;	// animating to overview, in overview, animating out
    	priv->shown = FALSE;	// show() and not hide()
    	priv->modal = FALSE;	// have a modal grab
    	priv->animationInProgress = FALSE;
    	priv->hideInProgress = FALSE;

        // During transitions, we raise this to the top to avoid having the overview
        // area be reactive; it causes too many issues such as double clicks on
        // Dash elements, or mouseover handlers in the workspaces.

	//gradient
    	priv->gradient = st_button_new ();
    	clutter_actor_set_reactive ((ClutterActor *)priv->gradient, FALSE);
    	st_widget_set_style_class_name (priv->gradient, "expo-background");
    	st_container_add_actor ((ClutterContainer *)priv->group, (ClutterActor *)priv->gradient);

	//coverPane
    	priv->cover_pane = clutter_rectangle_new ();
    	//priv->cover_pane = clutter_actor_new ();
    	clutter_actor_set_opacity (priv->cover_pane, 0);
    	clutter_actor_set_reactive (priv->cover_pane, TRUE);
    	st_container_add_actor ((ClutterContainer *)priv->group, priv->cover_pane);
    	g_signal_connect (priv->cover_pane, "event", G_CALLBACK (onEvent), self);

	//workspaceButton
    	priv->add_workspace_button = st_button_new ();
    	st_widget_set_style_class_name (priv->add_workspace_button, "workspace-add-button");
    	st_container_add_actor ((ClutterContainer *)priv->group, CLUTTER_ACTOR (priv->add_workspace_button));
    	g_signal_connect (priv->add_workspace_button, "clicked", G_CALLBACK (onClicked), self);

    	//notice:此处需要添加delegate和handleDragOver的相关过程;

	/*
        this._addWorkspaceButton.handleDragOver = function(source, actor, x, y, time) {
                return source.metaWindow ? DND.DragMotionResult.MOVE_DROP : DND.DragMotionResult.CONTINUE;
            };
        this._addWorkspaceButton.acceptDrop = function(source, actor, x, y, time) {
            if (source.metaWindow) {
                let draggable = source._draggable;
                actor.get_parent().remove_actor(actor);
                draggable._dragOrigParent.add_actor(actor);
                actor.opacity = draggable._dragOrigOpacity;
                Main.moveWindowToNewWorkspace(source.metaWindow);
            }
            return true;
        };
        this._addWorkspaceButton._delegate = this._addWorkspaceButton;
	*/

	//WindowCloseArea
    priv->window_close_area = st_button_new ();
	st_widget_set_style_class_name (priv->window_close_area, "window-close-area");

	/*
        this._windowCloseArea.handleDragOver = function(source, actor, x, y, time) {
                return source.metaWindow ? DND.DragMotionResult.MOVE_DROP : DND.DragMotionResult.CONTINUE;
            };
        this._windowCloseArea.acceptDrop = function(source, actor, x, y, time) {
            if (source.metaWindow) {
                let draggable = source._draggable;
                actor.get_parent().remove_actor(actor);
                draggable._dragOrigParent.add_actor(actor);
                actor.opacity = draggable._dragOrigOpacity;
                source.metaWindow.delete(global.get_current_time());
            }
            return true;
        };

	this._windowCloseArea._delegate = this._windowCloseArea;
	*/

    st_container_add_actor ((ClutterContainer *)priv->group, (ClutterActor *)priv->window_close_area);

    clutter_actor_hide (priv->group);

    clutter_container_add_actor ((ClutterContainer *)global_overlay_group, priv->group);

    clutter_actor_hide ((ClutterActor *)priv->gradient);
    clutter_actor_hide (priv->cover_pane);
    clutter_actor_hide ((ClutterActor *)priv->add_workspace_button);
    clutter_actor_hide ((ClutterActor *)priv->window_close_area);

    g_signal_connect (priv->group,"key-press-event",G_CALLBACK (onKeyPressEvent),self);

    g_signal_connect (priv->group,"key-release-event",G_CALLBACK (onKeyReleaseEvent),self);

    priv->expo = cdos_expo_thumbnails_box_new ();

    //CdosGenericContainer *expo_actor = cdos_expo_thumbnails_box_get_actor (priv->expo);

    st_container_add_actor ((ClutterContainer *)priv->group, (ClutterActor *)priv->expo);

    relayout (self);
}

/**
 * cdos_expo_hide:
 * @sef: #CdosExpo 对象
 * options: (可选)
 * 
 * 隐藏工作区缩略图的动画效果
 *
 * Returns: none
 */
void  cdos_expo_hide (CdosExpo *self, gboolean options) {

        g_return_if_fail(CDOS_IS_EXPO(self));
    	//CdosExpoPrivate *priv = CDOS_EXPO_GET_PRIVATE (self);
    	CdosExpoPrivate *priv = self->priv;

    	if(priv->shown == TRUE){
            animateNotVisible (self, options);
            priv->shown = FALSE;
            syncInputMode (self);
        }

}

/**
 * cdos_expo_show:
 * @sef: #CdosExpo 对象
 * 
 * 显示工作区缩略图的动画效果，并获取鼠标键盘的输入焦点
 *
 * Returns: none
 */
void  cdos_expo_show (CdosExpo *self) {
    g_return_if_fail(CDOS_IS_EXPO(self));
    CdosExpoPrivate *priv = CDOS_EXPO_GET_PRIVATE (self);

    	if (priv->shown)
        	return ;
    
    	cdos_expo_before_show (self);

	// Do this manually instead of using _syncInputMode, to handle failure
    	if (!cdos_push_modal (priv->group, 0))
        	return ;
    
    	priv->modal = TRUE;
    	priv->shown = TRUE;

    	animateVisible (self);
}

/**
 * cdos_expo_toggle:
 * @sef: #CdosExpo 对象
 * 
 * 切换当前工作区缩略图的显示模式，显示/隐藏
 *
 * Returns: none
 */
void  cdos_expo_toggle (CdosExpo *self) {
    g_return_if_fail(CDOS_IS_EXPO(self));
    CdosExpoPrivate *priv = CDOS_EXPO_GET_PRIVATE (self);

    	if (priv->shown) 
        	cdos_expo_hide (self, FALSE);
	    else
        	cdos_expo_show (self);    
}

/**
 * cdos_expo_get_animation_in_progress:
 * @sef: #CdosExpo 对象
 * 
 * 获取当前是否处于显示工作区缩略图的状态
 *
 * Returns: 返回当前工作区缩略图是否显示
 */
gboolean cdos_expo_get_animation_in_progress (CdosExpo *self) {

    	g_return_val_if_fail (CDOS_IS_EXPO (self), FALSE);

        CdosExpoPrivate *priv = CDOS_EXPO_GET_PRIVATE (self);

    	return priv->animationInProgress;
}

/**
 * cdos_expo_set_animation_in_progress:
 * @sef: #CdosExpo 对象
 * @animation: 布尔值，是否可见
 * 
 * 设置当前是否处于显示工作区缩略图的状态
 *
 * Returns: none
 */
void cdos_expo_set_animation_in_progress (CdosExpo *self, gboolean animation) {

    	g_return_if_fail (CDOS_IS_EXPO (self));

        CdosExpoPrivate *priv = CDOS_EXPO_GET_PRIVATE (self);

    	priv->animationInProgress = animation;
}

/**
 * cdos_expo_get_visible:
 * @sef: #CdosExpo 对象
 * 
 * 获取工作区缩略图是否可见
 *
 * Returns: 返回当前工作区缩略图是否可见
 */
gboolean cdos_expo_get_visible (CdosExpo *self) {

    	g_return_val_if_fail (CDOS_IS_EXPO (self),FALSE);

    	CdosExpoPrivate *priv = CDOS_EXPO_GET_PRIVATE (self);

    	return priv->visible;
}

/**
 * cdos_expo_set_visible:
 * @sef: #CdosExpo 对象
 * @visible: 布尔值，是否可见
 * 
 * 设置工作区缩略图是否可见
 *
 * Returns: none
 */
void cdos_expo_set_visible (CdosExpo *self, gboolean visible) {

    	g_return_if_fail (CDOS_IS_EXPO (self));

    	CdosExpoPrivate *priv = CDOS_EXPO_GET_PRIVATE (self);

    	priv->visible = visible;
}
