#include "cdos-extension-type.h"

G_DEFINE_TYPE (CdosExtensionType, cdos_extension_type, G_TYPE_OBJECT);

enum {
  PROP_TYPE_NAME = 1,
  PROP_TYPE_FOLDER,
  NUM_TYPE_PROP
};

static GParamSpec *properties[NUM_TYPE_PROP] = {NULL, };

static void
cdos_extension_type_set_property (GObject *object,
                        gint prop_id,
                        GValue *value,
                        GParamSpec *pspec)
{
  CdosExtensionType *extension_type;
  extension_type = CDOS_EXTENSION_TYPE (object);

  switch (prop_id) {
    case PROP_TYPE_NAME:
      extension_type->name = g_strdup (g_value_get_string (value));
      break;
    case PROP_TYPE_FOLDER:
      extension_type->folder = g_strdup (g_value_get_string (value));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
  }
}

static void
cdos_extension_type_get_property (GObject    *gobject,
                                   guint       prop_id,
                                   GValue     *value,
                                   GParamSpec *pspec)
{
  g_print ("%s\n", __func__);
  CdosExtensionType *self = CDOS_EXTENSION_TYPE (gobject);

  switch (prop_id)
  {
    case PROP_TYPE_NAME:
      g_value_set_string (value, self->name);
      break;

    case PROP_TYPE_FOLDER:
      g_value_set_string (value, self->folder);

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
      break;
  }
}
static void
cdos_extension_type_dispose (GObject *object)
{


  G_OBJECT_CLASS (cdos_extension_type_parent_class)->dispose (object);
}
static void
cdos_extension_type_finalize (GObject *object)
{


  G_OBJECT_CLASS (cdos_extension_type_parent_class)->finalize (object);
}

static void
cdos_extension_type_init (CdosExtensionType *self)
{
  
}

static void
cdos_extension_type_class_init (CdosExtensionTypeClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);

  object_class->set_property = cdos_extension_type_set_property;
  object_class->get_property = cdos_extension_type_get_property;
  object_class->finalize = cdos_extension_type_finalize;
  object_class->dispose = cdos_extension_type_dispose;

  properties[PROP_TYPE_NAME] = g_param_spec_string ("type_name",
                                "extension type name",
                                "a property of extension type",
                                NULL,
                                G_PARAM_WRITABLE);

  properties[PROP_TYPE_FOLDER] = g_param_spec_string ("type_folder",
                                "extension type folder",
                                "a porperty of extension type",
				NULL,
                                G_PARAM_WRITABLE);

  g_object_class_install_properties (object_class, NUM_TYPE_PROP, properties);


  // TODO: signal
}

/**
 * cdos_extension_type_new:
 * @type: extension type
 *
 * 新建一个 #CdosExtensionType 类的实例。
 *
 * Returns: 返回一个 #CdosExtensionType 的实例。
 */
CdosExtensionType *
cdos_extension_type_new (gint type)
{
  CdosExtensionType *extension_type;
  GPtrArray *strv = g_ptr_array_new ();
  GPtrArray *strv2 = g_ptr_array_new ();
  GPtrArray *strv3 = g_ptr_array_new ();

  switch (type) {
    case TYPE_EXTENSION:
      extension_type = g_object_new (CDOS_TYPE_EXTENSION_TYPE,
			"type_name", "Extension",
			"type_folder", "extensions",
			NULL); 
      g_ptr_array_add (strv, "init");
      g_ptr_array_add (strv, "disable");
      g_ptr_array_add (strv, "enable");
      g_ptr_array_add (strv, NULL);
      extension_type->required_functions = (gchar **) g_ptr_array_free (strv, FALSE);
      g_ptr_array_add (strv2, "uuid");
      g_ptr_array_add (strv2, "name");
      g_ptr_array_add (strv2, "description");
      g_ptr_array_add (strv2, "cdos-version");
      g_ptr_array_add (strv2, NULL);
      extension_type->required_properties = (gchar **) g_ptr_array_free (strv2, FALSE);
      g_ptr_array_add (strv3, "url");
      g_ptr_array_add (strv3, NULL);
      extension_type->nice_to_have_properties = (gchar **) g_ptr_array_free (strv3, FALSE);
      break;
    case TYPE_APPLET:
      extension_type = g_object_new (CDOS_TYPE_EXTENSION_TYPE,
                        "type_name", "Applet",
                        "type_folder", "applets",
                        NULL);
      g_ptr_array_add (strv, "main");
      g_ptr_array_add (strv, NULL);
      extension_type->required_functions = (gchar **) g_ptr_array_free (strv, FALSE);
      g_ptr_array_add (strv2, "uuid");
      g_ptr_array_add (strv2, "name");
      g_ptr_array_add (strv2, "description");
      g_ptr_array_add (strv2, NULL);
      extension_type->required_properties = (gchar **) g_ptr_array_free (strv2, FALSE);
      g_ptr_array_add (strv3, NULL);
      extension_type->nice_to_have_properties = (gchar **) g_ptr_array_free (strv3, FALSE);
      break;
    case TYPE_DESKLET:
      extension_type = g_object_new (CDOS_TYPE_EXTENSION_TYPE,
                        "type_name", "Desklet",
                        "type_folder", "desklets",
                        NULL);
      g_ptr_array_add (strv, "main");
      g_ptr_array_add (strv, NULL);
      extension_type->required_functions = (gchar **) g_ptr_array_free (strv, FALSE);
      g_ptr_array_add (strv2, "uuid");
      g_ptr_array_add (strv2, "name");
      g_ptr_array_add (strv2, "description");
      g_ptr_array_add (strv2, NULL);
      extension_type->required_properties = (gchar **) g_ptr_array_free (strv2, FALSE);
      g_ptr_array_add (strv3, NULL);
      extension_type->nice_to_have_properties = (gchar **) g_ptr_array_free (strv3, FALSE);
      break;
    default:
      return NULL;
  }

  /* create user directies if they don't exist */
  // GPtrArray *pa = g_ptr_array_new ();
  gchar *user_data_dir = g_build_filename (g_get_user_data_dir (), "cdos", NULL);
  // g_ptr_array_add (pa, user_data_dir);
  // g_ptr_array_add (pa, extension_type->folder);
  // g_ptr_array_add (pa, NULL);
  // const char *path = g_build_filenamev ((gchar **) g_ptr_array_free (pa, FALSE));
  char *path = g_build_filename (user_data_dir, extension_type->folder, NULL);
  extension_type->user_dir = g_file_new_for_path (path);
  g_free (path);
  g_free (user_data_dir);
  if (!g_file_query_exists (extension_type->user_dir, NULL)) {
    if (!g_file_make_directory_with_parents (extension_type->user_dir, NULL, NULL)) {
      return NULL;
    }
  }

  return CDOS_EXTENSION_TYPE (extension_type);
}

/**
 * cdos_extension_type_callbacks_finish_extension_load:
 * @type: A #CdosExtensionType
 * @data: a pointer of data
 *
 * callbacks finish extension load
 *
 * Returns: return FALSE
 */
gboolean
cdos_extension_type_callbacks_finish_extension_load (CdosExtensionType *type, 
						gpointer data)
{
  return FALSE;
}

/**
 * cdos_extension_type_callbacks_prepare_extension_unload:
 * @type: A #CdosExtensionType
 * @data: a pointer of data
 *
 * callbacks prepare extension unload
 *
 * Returns: return FALSE
 */
gboolean
cdos_extension_type_callbacks_prepare_extension_unload (CdosExtensionType *type,
						gpointer data)
{
  return FALSE;
}
