#include <girepository.h>
#include <clutter/clutter.h>
#include <gio/gio.h>
#include "cdos-icon-applet.h"
#include "st.h"
#include "cdos-start.h"

/**
 * SECTION:cdos-icon-applet
 * @Title: CdosIconApplet
 * @Short_description: 加载和显示图标
 *
 * CdosIconApplet类是作为其他类的基类,功能为加载和显示图标
 */
G_DEFINE_TYPE(CdosIconApplet, cdos_iconapplet, CDOS_TYPE_APPLET);

#define CDOS_ICONAPPLET_GET_PRIVATE(obj) \
        (G_TYPE_INSTANCE_GET_PRIVATE((obj), CDOS_TYPE_ICONAPPLET, CdosIconAppletPrivate))

struct _CdosIconAppletPrivate
{
    gboolean      scaleMode;
    GSettings     *settings;
    ClutterActor *applet_icon;
    const gchar     *icon_name;
    StIconType      icon_type;
    guint           panelHeight;
};
static void
cdos_iconapplet_dispose(GObject *object)
{
    CdosIconApplet *self = CDOS_ICONAPPLET(object);
    CdosIconAppletPrivate *priv = self->priv;

    g_clear_object(&priv->applet_icon);
    g_clear_object(&self->applet_icon_box);

    /*调用父类的finalize()方法*/
    G_OBJECT_CLASS(cdos_iconapplet_parent_class)->dispose(object);
}

static void
cdos_iconapplet_finalize(GObject *object)
{
    CdosIconAppletPrivate *priv = CDOS_ICONAPPLET(object)->priv;

    /*调用父类的finalize()方法*/
    G_OBJECT_CLASS(cdos_iconapplet_parent_class)->finalize(object);
}

/**
 * cdos_iconapplet_set_icon_name:
 * @self: #CdosIconApplet,需要图标的applet
 * @icon_name: 图标的名称
 *
 * 设置applet图标的名称以及类型
 *
 * Returns: void
 */
void
cdos_iconapplet_set_icon_name(CdosIconApplet *self,const gchar *icon_name){
    g_return_if_fail(CDOS_IS_ICONAPPLET(self));
    CdosIconAppletPrivate *priv ;
    priv = self->priv ;
    if(priv->applet_icon){
        st_icon_set_icon_name(ST_ICON(priv->applet_icon),icon_name);
    }else {
        priv->applet_icon = g_object_new(ST_TYPE_ICON,
                                         "icon_name", icon_name,
                                         "icon_type", ST_ICON_FULLCOLOR,
                                         "reactive", TRUE,
                                         "track_hover", TRUE,
                                         "style_class", "applet-icon",
                                         NULL);
        st_bin_set_child(self->applet_icon_box,priv->applet_icon);
        priv->icon_type = ST_ICON_FULLCOLOR;
    }
    priv->icon_name = icon_name;
}


/**
 * cdos_iconapplet_set_applet_icon_gion:
 * @self: #CdosIconApplet,需要图标的applet
 * @icon: 图标
 *
 * 设置applet图标的名称以及类型
 *
 * Returns: void
 */
void
cdos_iconapplet_set_applet_icon_gion (CdosIconApplet *self, GIcon *gicon){
    g_return_if_fail(CDOS_IS_ICONAPPLET(self));
    CdosIconAppletPrivate *priv ;
    priv = self->priv ;
    if(priv->applet_icon){
        st_icon_set_gicon ((StIcon *)priv->applet_icon, gicon);
    }
}


/**
 * cdos_iconapplet_set_icon_symbolic_name:
 * @self: #CdosIconApplet,需要图标的applet
 * @icon_name: 图标的名称
 *
 * 设置applet图标的系统名称以及类型
 *
 * Returns: void
 */
void
cdos_iconapplet_set_icon_symbolic_name(CdosIconApplet     *self,
                                       const gchar      *icon_name)
{
    g_return_if_fail(CDOS_IS_ICONAPPLET(self));
    CdosIconAppletPrivate *priv ;
    priv = self->priv;
    if(priv->applet_icon){
        st_icon_set_icon_name(ST_ICON(priv->applet_icon),icon_name);
    }else {
        priv->applet_icon = g_object_new(ST_TYPE_ICON,
                                         "icon_name", icon_name,
                                         "icon_type", ST_ICON_SYMBOLIC,
                                         "reactive", TRUE,
                                         "track_hover", TRUE,
                                         "style_class", "applet-icon",
                                         NULL);
        st_bin_set_child(self->applet_icon_box,priv->applet_icon);
        priv->icon_type = ST_ICON_SYMBOLIC;
    }
    priv->icon_name = icon_name;
}


/**
 * cdos_iconapplet_set_icon_path:
 * @self: #CdosIconApplet,需要图标的applet
 * @icon_path: 图标的路径
 *
 * 设置applet图标的路径以及类型
 *
 * Returns: void
 */
void
cdos_iconapplet_set_icon_path( CdosIconApplet   *self,
                               const char     *icon_path)
{
    g_return_if_fail(CDOS_IS_ICONAPPLET(self));
    CdosIconAppletPrivate *priv ;
    priv = self->priv;
    //CdosGlobal  *global;
    //guint ui_scale;
    global = cdos_global_get();
    // global->ui_scale = 1;
    priv->scaleMode = cdos_applet_get_scalemode((CdosApplet *)self);
    if(icon_path){
        GFile *file;
        GIcon *gicon;
        guint height;
        file = g_file_new_for_path(icon_path);
        //StIconType type = ST_ICON_FULLCOLOR;
        gicon = g_file_icon_new(file);
        if(priv->applet_icon){
            st_icon_set_gicon(ST_ICON(priv->applet_icon),gicon);
        }else{
            priv->applet_icon = g_object_new ( ST_TYPE_ICON,
                                               "gicon",gicon,
                                               "icon_type",ST_ICON_FULLCOLOR,
                                               "reactive",TRUE,
                                               "track_hover",TRUE,
                                               "style_class","applet-icon",
                                               NULL);

        };
        st_bin_set_child(self->applet_icon_box,priv->applet_icon);
        g_object_unref(gicon);
        g_object_unref(file);
    }
    priv->icon_type = -1;
    priv->icon_name = icon_path;
}

static void
on_panel_height_changed(CdosIconApplet *self)
{
    g_return_if_fail(CDOS_IS_ICONAPPLET(self));
    CdosIconAppletPrivate *priv;
    priv = self->priv;
    CdosGlobal *global;
    //GSettings *settings;
    gboolean scaleM;
    // StBin       *applet_icon_box;
    //StIconType  icon_type;
    global = cdos_global_get();
    priv->settings = cdos_global_get_settings (global);

    priv->scaleMode = FALSE;
    scaleM = g_settings_get_boolean (priv->settings, "panel-scale-text-icons");
    scaleM = priv->scaleMode && g_settings_get_boolean (priv->settings, "panel-resizable");
    cdos_applet_set_scalemode ((CdosApplet *)self, scaleM);

    if(st_bin_get_child(self->applet_icon_box)){
        clutter_actor_destroy(st_bin_get_child(self->applet_icon_box));
    }
    printf("%s %d: %d \n", __func__, __LINE__, priv->icon_type);
    switch(priv->icon_type)  {
        case ST_ICON_FULLCOLOR:
            cdos_iconapplet_set_icon_name(self,priv->icon_name);
            break;
        case ST_ICON_SYMBOLIC:
            cdos_iconapplet_set_icon_symbolic_name(self,priv->icon_name);
            break;
        default:
            break;
    }

}


/**
 * cdos_iconapplet_set_icon_symbolic_path:
 * @self: #CdosIconApplet,需要图标的applet
 * @icon_path: 图标的路径
 *
 * 设置applet图标的系统路径以及类型
 *
 * Returns: void
 */
void
cdos_iconapplet_set_icon_symbolic_path(CdosIconApplet    *self,
                                       const char      *icon_path)
{
    g_return_if_fail(CDOS_IS_ICONAPPLET(self));
    CdosIconAppletPrivate *priv ;
    priv = self->priv;

    //CdosGlobal  *global;
    //guint           ui_scale;
    global = cdos_global_get();
    priv->scaleMode = cdos_applet_get_scalemode((CdosApplet *)self);//fixme

    if(icon_path){
        GFile *file;
        GIcon *gicon;
        file = g_file_new_for_path(icon_path);
        gicon = g_file_icon_new(file);
        if(priv->applet_icon){
            st_icon_set_gicon(ST_ICON(priv->applet_icon),gicon);
        }else{
            StIconType type = ST_ICON_FULLCOLOR;
            priv->applet_icon = g_object_new ( ST_TYPE_ICON,
                                               "gicon",gicon,
                                               "icon_type",type,
                                               "reactive",TRUE,
                                               "track_hover",TRUE,
                                               "style_class","applet-icon",
                                               NULL);
            st_bin_set_child(self->applet_icon_box,priv->applet_icon) ;
        }
        g_object_unref(gicon);
        g_object_unref(file);
    }
    priv->icon_type = -1;
    priv->icon_name = icon_path;
    on_panel_height_changed(self);
}

static void
cdos_iconapplet_class_init( CdosIconAppletClass *kclass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (kclass);
    object_class->finalize = cdos_iconapplet_finalize;
    object_class->dispose = cdos_iconapplet_dispose;

    g_type_class_add_private (kclass, sizeof (CdosIconAppletPrivate));

}

static void
cdos_iconapplet_init(CdosIconApplet *self)
{
    CdosIconAppletPrivate *priv;
    priv = self->priv = CDOS_ICONAPPLET_GET_PRIVATE(self);
    self->applet_icon_box = g_object_new (ST_TYPE_BIN,NULL);
    self->actor = CDOS_APPLET(self)->actor;
    st_container_add_actor((ClutterContainer*)self->actor, CLUTTER_ACTOR (self->applet_icon_box));
    priv->applet_icon = NULL;
    StBoxLayoutChild *button_meta =
            (StBoxLayoutChild *)clutter_container_get_child_meta (
                    CLUTTER_CONTAINER (self->actor), (ClutterActor *)self->applet_icon_box);
    button_meta->y_fill  = FALSE;
    button_meta->y_align = ST_ALIGN_MIDDLE;
    clutter_actor_queue_relayout ((ClutterActor *)self->actor);
    priv->panelHeight = cdos_applet_get_panel_height(CDOS_APPLET(self));
    cdos_applet_set_popup_menu_active(CDOS_APPLET(self),TRUE);
}

/**
 * cdos_iconapplet_get_applet_icon_box:
 * @self: #CdosIconApplet,需要图标的applet
 *
 * 获取类CdosIconApplet的变量applet_icon_box
 * Returns: applet_icon_box
 */
StBin *
cdos_iconapplet_get_applet_icon_box (CdosIconApplet *self){
    g_return_val_if_fail(CDOS_IS_ICONAPPLET(self),NULL);
    //CdosIconAppletPrivate *priv;

    //priv = CDOS_ICONAPPLET_GET_PRIVATE(self);
    return self->applet_icon_box;
}


/**
 * cdos_iconapplet_new:
 * @guint: panel的高度
 *
 * 根据panel的高度创建新的iconapplet
 * Returns: （CdosIconApplet）iconapplet
 */
CdosIconApplet *
cdos_iconapplet_new(guint panel_height)
{
    CdosIconApplet *iconapplet;
    iconapplet = g_object_new(CDOS_TYPE_ICONAPPLET,NULL);

    return iconapplet;
}

void cdos_iconapplet_reset_for_systray(CdosIconApplet *self){
    g_return_if_fail(CDOS_IS_ICONAPPLET(self));
    CdosIconAppletPrivate *priv = self->priv;
    st_widget_set_track_hover(ST_WIDGET(self->actor),FALSE);
    st_widget_set_track_hover(ST_WIDGET(self->applet_icon_box),TRUE);
    clutter_actor_set_reactive(CLUTTER_ACTOR(self->applet_icon_box),TRUE);

    st_widget_add_style_class_name(ST_WIDGET(self->actor),"applet-systray");
    st_widget_add_style_class_name(ST_WIDGET(self->applet_icon_box),"applet-systray-box");
    if(priv->applet_icon){
        st_widget_add_style_class_name(ST_WIDGET(priv->applet_icon),"applet-systray-icon");
    }
}
ClutterActor *
cdos_iconapplet_get_applet_icon(CdosIconApplet *self){
    g_return_val_if_fail(CDOS_IS_ICONAPPLET(self),NULL);
    CdosIconAppletPrivate *priv = self->priv;
    return priv->applet_icon;
}
