//
// Created by zhangtao on 16-7-04.
// source:keyboard.js:Key

#include "cdos-key.h"
#include "cdos-box-pointer.h"
#include "cdos-monkey-plugin.h"

typedef struct _CdosKeySignalIds CdosKeySignalIds;
typedef struct _MediaKeyType MediaKeyType;
typedef struct _CdosKeyBound CdosKeyBound;


/*-----------------信号枚举-------------------*/
enum {
    LAST_SIGNAL
};
/*-----------------实例私有结构体-------------------*/
struct _CdosKeyPrivate{
    gboolean grabbed;
    CaribouKeyModel *key;
    StWidget *actor;
    CaribouKeyModel** extended_keys;
    guint extended_keys_length;
    StWidget *extended_keyboard;
    CdosBoxPointer *box_pointer;
    CdosKeySignalIds *signal_ids;
    GHashTable *pretty_keys;

};

/*-----------------信号id-------------------*/
struct _CdosKeySignalIds{
    gulong key_pressed_id;
    gulong key_released_id;
    gulong key_notify_show_subkeys_id;
    gulong stage_captured_id;

};

/*-------global variables-------------*/
static gint CdosKey_private_offset = 0;
static gint cdos_key_signals[LAST_SIGNAL] = { 0 };
static gpointer cdos_key_parent_class = NULL;

//for actor in function cdos_key_constructor;
static GHashTable *actor_extended_keys = NULL;
//for key in function cdos_key_gt_extended_keys;
static GHashTable *key_extended_keys = NULL;
//for button in function cdos_key_make_key;
static GHashTable *button_key_width = NULL;

/*-------class functions---------*/
static void cdos_key_init(CdosKey *self);
static void cdos_key_class_init(CdosKeyClass *klass);
static void cdos_key_dispose(GObject *object);
static void cdos_key_finalize(GObject *object);

/*-------functions-------------*/
static void cdos_key_constructor(CdosKey *self,CaribouKeyModel *key);

static StWidget* cdos_key_make_key(CdosKey *self);

static gchar* cdos_key_get_unichar(CdosKey *self,CaribouKeyModel *key);

static void cdos_key_get_extended_keys(CdosKey *self);

static void cdos_key_ungrab(CdosKey *self);

static gboolean cdos_key_stage_on_captured(ClutterActor *actor,ClutterEvent *event,CdosKey *self);

static void cdos_key_key_on_notify_show_subkeys(GObject *object,GParamSpec *pspec,CdosKey *self);

static gboolean cdos_key_key_on_pressed(CaribouIKeyboardObject *ikey,CaribouKeyModel *key,CdosKey *self);

static gboolean cdos_key_key_on_released(CaribouIKeyboardObject *ikey,CaribouKeyModel *key,CdosKey *self);

static gboolean cdos_key_button_on_press(ClutterActor *actor,ClutterEvent *event,CaribouKeyModel *key);

static gboolean cdos_key_button_on_release(ClutterActor *actor,ClutterEvent *event,CaribouKeyModel *key);

static gboolean cdos_key_actor_extended_keys_GHRFunc(StWidget *key,StWidget *value,StWidget *user_data);
static gboolean cdos_key_button_key_width_GHRFunc(StWidget *key,gdouble value,StWidget *user_data);
static gboolean cdos_key_key_extended_key_GHRFunc(StWidget *key,CaribouKeyModel *value,StWidget *user_data);
static gboolean cdos_key_pretty_keys_GHRFunc(gchar *key,gchar *value,gchar *user_data);

/**
 * 注册类；
 *
 *
 * */
GType
cdos_key_get_type(void)
{
    static GType cdos_key_type = 0;
    if(G_UNLIKELY(cdos_key_type==0))
    {
        static const GTypeInfo cdos_key_info = {
                sizeof(CdosKeyClass),
                NULL,NULL,
                (GClassInitFunc)cdos_key_class_init,
                NULL,NULL,
                sizeof(CdosKey),
                0,
                (GInstanceInitFunc)cdos_key_init
        };
        //类注册
        cdos_key_type = g_type_register_static(G_TYPE_OBJECT,"CdosKey",&cdos_key_info,0);
        //g_type_add_class_private(cdos_key_type, sizeof(CdosKeyClassPrivate));
        CdosKey_private_offset =
                g_type_add_instance_private(cdos_key_type, sizeof(CdosKeyPrivate));

    }
    return cdos_key_type;
}
/**
 *  获取实例的私有属性结构
 * */
static inline gpointer
cdos_key_get_instance_private(CdosKey *self){
    return (G_STRUCT_MEMBER_P (self, CdosKey_private_offset));
}
//实例初始化；
static void
cdos_key_init(CdosKey *self)
{
    CdosKeyPrivate *priv;
    self->priv=cdos_key_get_instance_private(self);
    priv=self->priv;
    /*--------------实例初始化（构造实例）-------------*/
    priv->signal_ids = g_new0(CdosKeySignalIds,1);
    priv->pretty_keys = g_hash_table_new(g_str_hash,g_str_equal);

    g_hash_table_insert(priv->pretty_keys,"BackSpace","\u232b");
    g_hash_table_insert(priv->pretty_keys,"space","");
    g_hash_table_insert(priv->pretty_keys,"Return","\u23ce");
    g_hash_table_insert(priv->pretty_keys,"Caribou_Prefs","\u2328");
    g_hash_table_insert(priv->pretty_keys,"Caribou_ShiftUp","\u2b06");
    g_hash_table_insert(priv->pretty_keys,"Caribou_ShiftDown","\u2b07");
    g_hash_table_insert(priv->pretty_keys,"Caribou_Emoticons","\u263a");
    g_hash_table_insert(priv->pretty_keys,"Caribou_Symbols","123");
    g_hash_table_insert(priv->pretty_keys,"Caribou_Symbols_More","{#*");
    g_hash_table_insert(priv->pretty_keys,"Caribou_Alpha","Abc");
    g_hash_table_insert(priv->pretty_keys,"Tab","Tab");
    g_hash_table_insert(priv->pretty_keys,"Escape","Esc");
    g_hash_table_insert(priv->pretty_keys,"Control_L","Ctrl");
    g_hash_table_insert(priv->pretty_keys,"Alt_L","Alt");

}
//类结构初始化；

static void
cdos_key_class_init(CdosKeyClass *klass)
{
    //转换为gobject；
    GObjectClass *gobject_class=G_OBJECT_CLASS(klass);
    g_type_class_adjust_private_offset(klass,&CdosKey_private_offset);
    cdos_key_parent_class = g_type_class_peek_parent(klass);
    /*-------------重写父类的虚方法---------------*/
    //gobject_class->constructed=cdos_key_constructed;
    gobject_class->dispose=cdos_key_dispose;
    gobject_class->finalize=cdos_key_finalize;
    if(actor_extended_keys == NULL){
        actor_extended_keys = g_hash_table_new(NULL,NULL);
    }
    if(key_extended_keys == NULL){
        key_extended_keys = g_hash_table_new(NULL,NULL);
    }
    if(button_key_width == NULL){
        button_key_width = g_hash_table_new( g_double_hash,NULL);
    }
}

/*------------类方法定义-----------*/
static void
cdos_key_dispose(GObject *object){
    CdosKey *self = CDOS_KEY(object);
    CdosKeyPrivate *priv = self->priv;

    G_OBJECT_CLASS(cdos_key_parent_class)->dispose(object);
};

static void
cdos_key_finalize(GObject *object){
    CdosKey *self = CDOS_KEY(object);
    CdosKeyPrivate *priv = self->priv;

    if(g_object_is_floating(object)){
        g_warning ("A floating object was finalized. This means that someone called g_object_unref() on an object that had only a floating reference; the initial floating reference is not owned by anyone and must be removed with g_object_ref_sink().");
    }
    G_OBJECT_CLASS(cdos_key_parent_class)->finalize(object);
};

static void
cdos_key_constructor(CdosKey *self,CaribouKeyModel *key){
    CdosKeyPrivate *priv = self->priv;
    priv->key = key;
    priv->actor = cdos_key_make_key(self);

    priv->extended_keys = caribou_key_model_get_extended_keys(priv->key,&priv->extended_keys_length);
    priv->extended_keyboard = NULL;

    gchar *key_name = caribou_key_model_get_name(priv->key);
    if(g_str_equal(key_name,"Control_L") || g_str_equal(key_name,"Alt_L")){
        //what is 'latch'
    }
    priv->signal_ids->key_pressed_id = \
            g_signal_connect((CaribouIKeyboardObject*)priv->key,"key-pressed",G_CALLBACK(cdos_key_key_on_pressed),self);
    priv->signal_ids->key_released_id = \
            g_signal_connect((CaribouIKeyboardObject*)priv->key,"key-released",G_CALLBACK(cdos_key_key_on_released),self);

    if(sizeof(priv->extended_keys) > 0){
        priv->grabbed = FALSE;
        priv->signal_ids->key_notify_show_subkeys_id = \
                g_signal_connect((GObject *)priv->key,"notify::show-subkeys",G_CALLBACK(cdos_key_key_on_notify_show_subkeys),self);

        priv->box_pointer = cdos_box_pointer_new(ST_SIDE_BOTTOM);
        StBin *box_pointer_bin = ST_BIN(priv->box_pointer->bin);
        StWidget *box_pointer_actor = (StWidget *)priv->box_pointer->actor;

        st_bin_set_fill(box_pointer_bin,TRUE,TRUE);
        st_bin_set_alignment(box_pointer_bin,ST_ALIGN_START,ST_ALIGN_MIDDLE);
        st_widget_add_style_class_name(box_pointer_actor,"keyboard-subkeys");
        cdos_key_get_extended_keys(self);

        g_hash_table_insert(actor_extended_keys,priv->actor,priv->extended_keyboard);

        clutter_actor_hide(CLUTTER_ACTOR(box_pointer_actor));
        CdosLayoutManager *layout_manager = cdos_monkey_plugin_get_layout_manager(monkey_plugin_get());
        cdos_layout_manager_add_chrome(layout_manager,CLUTTER_ACTOR(box_pointer_actor), 4);
    }
}

static StWidget*
cdos_key_make_key(CdosKey *self){
    CdosKeyPrivate *priv = self->priv;
    gchar *key_name = caribou_key_model_get_name(priv->key);
    StWidget *button = NULL;
    if(key_name){
        gchar *pretty = g_hash_table_find(priv->pretty_keys,cdos_key_pretty_keys_GHRFunc,key_name);
        gchar *label = NULL;
        if(pretty){
            label = pretty;
        }else{
            label = cdos_key_get_unichar(self,priv->key);
        }
        gchar *new_label = g_markup_escape_text (label,-1);
        StWidget *button = st_button_new();
        gdouble key_width = caribou_key_model_get_width (priv->key);
        g_hash_table_insert(button_key_width,button,&key_width);

        st_button_set_label(ST_BUTTON(button),new_label);
        st_widget_set_style_class_name(button,"keyboard-key");
        g_signal_connect(CLUTTER_ACTOR(button),"button-press-event",G_CALLBACK(cdos_key_button_on_press),priv->key);
        g_signal_connect(CLUTTER_ACTOR(button),"button-release-event",G_CALLBACK(cdos_key_button_on_release),priv->key);
    }
    return button;
}

static gchar*
cdos_key_get_unichar(CdosKey *self,CaribouKeyModel *key){
    CdosKeyPrivate *priv = self->priv;
    guint keyval = caribou_key_model_get_keyval(priv->key);
    guint32 unichar = clutter_keysym_to_unicode (keyval);
    gchar *new_unichar = NULL;
    if(unichar){
        new_unichar = (char *)malloc(6);
        g_unichar_to_utf8 (unichar,new_unichar);
    }else{
        new_unichar = caribou_key_model_get_name(priv->key);
    }
    return new_unichar;
}

static void
cdos_key_get_extended_keys(CdosKey *self){
    CdosKeyPrivate *priv = self->priv;
    priv->extended_keyboard = st_box_layout_new();
    st_widget_set_style_class_name(priv->extended_keyboard,"keyboard-layout");
    st_box_layout_set_vertical(ST_BOX_LAYOUT(priv->extended_keyboard),FALSE);
    guint i = 0;
    for(i;i < priv->extended_keys_length;i++){
        CaribouKeyModel *extended_key = priv->extended_keys[i];
        gchar *label = cdos_key_get_unichar(self,extended_key);
        StWidget *key = st_button_new();
        g_hash_table_insert(key_extended_keys,key,extended_key);
        st_button_set_label(ST_BUTTON(key),label);
        st_widget_set_style_class_name(key,"keyboard-key");
        g_signal_connect(CLUTTER_ACTOR(key),"button-press-event",G_CALLBACK(cdos_key_button_on_press),key);
        g_signal_connect(CLUTTER_ACTOR(key),"button-release-event",G_CALLBACK(cdos_key_button_on_release),key);
        clutter_container_add_actor(CLUTTER_CONTAINER(priv->extended_keyboard),CLUTTER_ACTOR(key));
    }
    clutter_container_add_actor(CLUTTER_CONTAINER(priv->box_pointer->bin),CLUTTER_ACTOR(priv->extended_keyboard));

}

static void
cdos_key_ungrab(CdosKey *self){
    CdosKeyPrivate *priv = self->priv;
    ClutterStage *global_stage = cdos_global_get_stage(global);
    g_signal_handler_disconnect(global,priv->signal_ids->stage_captured_id);
    priv->signal_ids->stage_captured_id = 0;
    priv->grabbed = FALSE;
    cdos_pop_modal(priv->actor,cdos_global_get_current_time(global));
}

static gboolean
cdos_key_stage_on_captured(ClutterActor *actor,ClutterEvent *event,CdosKey *self){
    CdosKeyPrivate *priv = self->priv;
    ClutterActor *source = clutter_event_get_source (event);
    ClutterEventType type = clutter_event_type (event);
    if((type == CLUTTER_BUTTON_PRESS || type == CLUTTER_BUTTON_RELEASE) && clutter_actor_contains (CLUTTER_ACTOR(priv->extended_keyboard),source)){
        CaribouKeyModel *extended_key = cdos_key_get_key_extended_key(ST_WIDGET(source));
        caribou_key_model_press(extended_key);
        caribou_key_model_release(extended_key);
    }
    if(type == CLUTTER_BUTTON_PRESS){
        clutter_actor_hide(CLUTTER_ACTOR(priv->box_pointer->actor));
        cdos_key_ungrab(self);
        return TRUE;
    }
    return FALSE;
}

static void
cdos_key_key_on_notify_show_subkeys(GObject *object,GParamSpec *pspec,CdosKey *self){
    CdosKeyPrivate *priv = self->priv;
    if(caribou_key_model_get_show_subkeys(priv->key)){
        st_button_fake_release(ST_BUTTON(priv->actor));
        clutter_actor_raise_top(CLUTTER_ACTOR(priv->box_pointer->actor));
        cdos_box_pointer_set_position(priv->box_pointer,priv->actor,0.5);
        cdos_box_pointer_show(priv->box_pointer,TRUE);
        st_widget_set_hover(priv->actor,FALSE);
        if(!priv->grabbed){
            MetaDisplay *global_stage = cdos_global_get_stage(global);
            cdos_pop_modal(priv->actor,cdos_global_get_current_time(global));
            priv->signal_ids->stage_captured_id = \
                    g_signal_connect(global_stage,"captured-event",G_CALLBACK(cdos_key_stage_on_captured),self);
            priv->grabbed = TRUE;
        }
        caribou_key_model_release(priv->key);
    }else{
        if(priv->grabbed){
            cdos_key_ungrab(self);
        }
        cdos_box_pointer_hide(priv->box_pointer,TRUE);
    }
}

static gboolean
cdos_key_key_on_pressed(CaribouIKeyboardObject *ikey,CaribouKeyModel *key,CdosKey *self){
    CdosKeyPrivate *priv = self->priv;
    if(priv->actor){
        st_button_set_checked(ST_BUTTON(priv->actor),TRUE);
    }
    return FALSE;
}

static gboolean
cdos_key_key_on_released(CaribouIKeyboardObject *ikey,CaribouKeyModel *key,CdosKey *self){
    CdosKeyPrivate *priv = self->priv;
    if(priv->actor){
        st_button_set_checked(ST_BUTTON(priv->actor),FALSE);
    }
    return FALSE;
}
static gboolean cdos_key_button_on_press(ClutterActor *actor,ClutterEvent *event,CaribouKeyModel *key){
    caribou_key_model_press(key);
    return FALSE;
}

static gboolean cdos_key_button_on_release(ClutterActor *actor,ClutterEvent *event,CaribouKeyModel *key){
    caribou_key_model_release(key);
    return FALSE;
}

static gboolean cdos_key_actor_extended_keys_GHRFunc(StWidget *key,StWidget *value,StWidget *user_data){
    if(key == user_data){
        return  TRUE;
    }
    return FALSE;
}

static gboolean cdos_key_button_key_width_GHRFunc(StWidget *key,gdouble value,StWidget *user_data){
    if(key == user_data){
        return  TRUE;
    }
    return FALSE;
}

static gboolean cdos_key_key_extended_key_GHRFunc(StWidget *key,CaribouKeyModel *value,StWidget *user_data){
    if(key == user_data){
        return  TRUE;
    }
    return FALSE;
}

static gboolean
cdos_key_pretty_keys_GHRFunc(gchar *key,gchar *value,gchar *user_data){
    if(g_str_equal(key,user_data)){
        return  TRUE;
    }
    return FALSE;
}

/*-------------public functions--------------*/
/**
 * cdos_key_new:
 * @key: CaribouKeyMdoule * key
 *
 * 新建一个 #CdosKey 类的实例。
 *
 * Returns: 返回一个 #CdosKey 的实例。
 */
CdosKey*
cdos_key_new(CaribouKeyModel *key)
{
    CdosKey *self;
    self = g_object_new(CDOS_TYPE_KEY, NULL);
    cdos_key_constructor(self,key);
    return self;
}

/**
 * cdos_key_get_actor:
 * @self: A #CdosKey instance
 *
 * 获取self的private actor成员
 *
 * Returns: a #StWidget instance
 */
StWidget*
cdos_key_get_actor(CdosKey *self){
    g_return_val_if_fail(CDOS_IS_KEY(self),NULL);
    CdosKeyPrivate *priv = self->priv;
    return priv->actor;
}

/**
 * cdos_key_get_actor_extended_keys:
 * @actor: StWidget 对象
 *
 * get actor extended keys.
 *
 * Returns: a #StWidget instance
 */
StWidget*  
cdos_key_get_actor_extended_keys(StWidget *actor){
    if(actor_extended_keys == NULL){
        g_warning("Warning:The hash table of actor_extended_keys is NULL!");
    }
    StWidget *v = g_hash_table_find(actor_extended_keys,cdos_key_actor_extended_keys_GHRFunc,actor);
    return v;
}

/**
 * cdos_key_get_button_key_width:
 * @button: StWidget Button 对象
 *
 * get width of button key.
 *
 * Returns: button宽度值
 */
gdouble* 
cdos_key_get_button_key_width(StWidget *button){
    if(actor_extended_keys == NULL){
        g_warning("Warning:The hash table of actor_extended_keys is NULL!");
        return 0;
    }
    gdouble *v = g_hash_table_find(button_key_width,cdos_key_button_key_width_GHRFunc,button);
    if(!v){
        return 0;
    }
    return v;
}

/**
 * cdos_key_get_key_extended_key:
 * @key: StWidget 对象
 *
 * get extended key of key.
 *
 * Returns: a #CaribouKeyModel instance
 */
CaribouKeyModel *
cdos_key_get_key_extended_key(StWidget *key){
    if(actor_extended_keys == NULL){
        g_warning("Warning:The hash table of actor_extended_keys is NULL!");
    }
    CaribouKeyModel *v = g_hash_table_find(key_extended_keys,cdos_key_key_extended_key_GHRFunc,key);
    return v;
}
