#include "cdos-keybinding-manager.h"
#include "cdos-global.h"
#include "cdos-util.h"

/**
 * SECTION: cdos-keybinding-manager
 * @title: CdosKeybindingManager
 * @short_description: 用来管理用户自定义的快捷键
 * @include: cdos-keybinding-manager.h
 *
 * CdosKeybindingManager 用来添加或者移除用户自定义的快捷键。
 */

#define CSD_DBUS_NAME  "org.cdos.SettingsDaemon"
#define CSD_DBUS_PATH  "/org/cdos/SettingsDaemon/KeybindingHandler"

#define CUSTOM_KEYS_PARENT_SCHEMA  "org.cdos.keybindings"
#define CUSTOM_KEYS_BASENAME  "/org/cdos/keybindings/custom-keybindings"
#define CUSTOM_KEYS_SCHEMA  "org.cdos.keybindings.custom-keybinding"

#define CUSTOM_NEW_KEY_SCHEMA "/org/cdos/keybindings/custom-keybindings/%s/"
#define CUSTOM_KEY_LENGTH       60
#define MEDIA_KEYS_SCHEMA  "org.cdos.settings-daemon.plugins.media-keys"

G_DEFINE_TYPE (CdosKeybindingManager, cdos_keybinding_manager, G_TYPE_OBJECT);   //父类类型参数？

//#define parent_class cdos_keybinding_manager_parent_class   //是否需要？


const gchar *json = " \
	{ \
	\"MUTE_KEY\"                : {\"value\": \"3\",  \"key\": \"volume-mute\",             \"kb\": null                    }, \
	\"VOLUME_DOWN_KEY\"         : {\"value\": \"4\",  \"key\": \"volume-down\",             \"kb\": null                    }, \
  	\"VOLUME_UP_KEY\"           : {\"value\": \"5\",  \"key\": \"volume-up\",               \"kb\": null                    }, \
  	\"EJECT_KEY\"               : {\"value\": \"11\", \"key\": \"eject\",                   \"kb\": null                    }, \
  	\"MEDIA_KEY\"               : {\"value\": \"13\", \"key\": \"media\",                   \"kb\": null                    }, \
  	\"SCREENSHOT_KEY\"          : {\"value\": \"19\", \"key\": \"screenshot\",              \"kb\": null                    }, \
  	\"WINDOW_SCREENSHOT_KEY\"   : {\"value\": \"20\", \"key\": \"window-screenshot\",       \"kb\": null                    }, \
  	\"PLAY_KEY\"                : {\"value\": \"27\", \"key\": \"play\",                    \"kb\": null                    }, \
  	\"PAUSE_KEY\"               : {\"value\": \"28\", \"key\": \"pause\",                   \"kb\": null                    }, \
  	\"STOP_KEY\"                : {\"value\": \"29\", \"key\": \"stop\",                    \"kb\": null                    }, \
  	\"PREVIOUS_KEY\"            : {\"value\": \"30\", \"key\": \"previous\",                \"kb\": null                    }, \
  	\"NEXT_KEY\"                : {\"value\": \"31\", \"key\": \"next\",                    \"kb\": null                    }, \
  	\"REWIND_KEY\"              : {\"value\": \"32\", \"key\": null,                      \"kb\": \"XF86AudioRewind\"       }, \
  	\"FORWARD_KEY\"             : {\"value\": \"33\", \"key\": null,                      \"kb\": \"XF86AudioForward\"      }, \
  	\"REPEAT_KEY\"              : {\"value\": \"34\", \"key\": null,                      \"kb\": \"XF86AudioRepeat\"       }, \
  	\"RANDOM_KEY\"              : {\"value\": \"35\", \"key\": null,                      \"kb\": \"XF86AudioRandomPlay\"   } \
	}";

#define MediaKeyTypes json

static void setup_custom_keybindings_callback (MetaDisplay *display,
                                               MetaScreen *screen,
                                               MetaWindow *window,
                                               ClutterKeyEvent *event,
                                               MetaKeyBinding *binding,
                                               gpointer user_data);
                                   
static void on_customs_changed (GSettings *settings, gchar *key, gpointer data);

static void on_media_key_pressed (MetaDisplay *display,
                                  MetaScreen *screen,
                                  MetaWindow *window,
                                  ClutterKeyEvent *event,
                                  MetaKeyBinding *binding,
                                  gpointer user_data);

typedef struct
{
	GDBusProxy *proxy;
	gchar *value;
} ProxyData;

struct _CdosKeybindingManagerPrivate
{
	/*私有*/
	GDBusProxy *proxy;   //DBus接口类型
	GList *add_hot_key_user_data;
};

static void
cdos_keybinding_manager_dispose (GObject *object)
{
	CdosKeybindingManager *self = CDOS_KEYBINDING_MANAGER(object);

	g_clear_object(&self->priv->proxy);
	g_clear_object(&self->kb_schema);
	g_clear_object(&self->csd_kb_settings);

	if(self->bindings){
		json_object_unref(self->bindings);
	}
	G_OBJECT_CLASS (cdos_keybinding_manager_parent_class)->dispose (object);
}

static void
cdos_keybinding_manager_finalize (GObject *object)
{
	CdosKeybindingManager *self = CDOS_KEYBINDING_MANAGER(object);
	CdosKeybindingManagerPrivate *priv = self->priv;

	if(priv->add_hot_key_user_data){
		GList *user_data = priv->add_hot_key_user_data;
		while (user_data != NULL){
			ProxyData *data = user_data->data;

			g_free(data->value);
			g_free(data);

			user_data = user_data->next;
		}
		g_list_free(priv->add_hot_key_user_data);
	}
  	G_OBJECT_CLASS (cdos_keybinding_manager_parent_class)->finalize (object);
}

//类初始化时干什么？
static void
cdos_keybinding_manager_class_init (CdosKeybindingManagerClass *klass)
{
	GObjectClass *object_class;
	
	object_class = G_OBJECT_CLASS (klass);
	
	object_class->finalize = cdos_keybinding_manager_finalize;
	object_class->dispose = cdos_keybinding_manager_dispose;

	g_type_class_add_private (klass, sizeof (CdosKeybindingManagerPrivate));
}

/*
static void
proxy_ready (GObject *source,
             GAsyncResult *result,
             gpointer user_data)
{
	CdosKeybindingHandler *proxy;
	GError *error;
	
	error = NULL;
	
	proxy = cdos_keybinding_handler_proxy_new_for_bus_finish (result, &error);
	(CdosKeybindingHandler *)user_data = proxy;     //能给self->priv->proxy赋值吗？
	
	g_object_unref (proxy);
}
*/

static void
cdos_keybinding_manager_init (CdosKeybindingManager *self)
{
	//g_print ("\n---->in cdos_keybinding_manager_init\n");   ////////调试
	
	GError *error;
	
	error = NULL;
	
	self->priv = CDOS_KEYBINDING_MANAGER_GET_PRIVATE (self);
	
	CdosGlobal *global = cdos_global_get ();
  	MetaScreen *screen = cdos_global_get_screen (global);
  	self->meta_display = meta_screen_get_display (screen);
	
	/*生成DBus代理*/
	/*
	cdos_keybinding_handler_proxy_new_for_bus (G_BUS_TYPE_SESSION,
	                                               G_DBUS_PROXY_FLAGS_NONE,
	                                               "org.cdos.SettingsDaemon",
	                                               "/org/cdos/SettingsDaemon/KeybindingHandler",
	                                               NULL,
	                                               proxy_ready,
	                                               self->priv->proxy);
	*/
	//同步还是异步？
	self->priv->proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
	                                                   G_DBUS_PROXY_FLAGS_NONE,
                                                       NULL,
	                                                   "org.cdos.SettingsDaemon",
	                                                   "/org/cdos/SettingsDaemon/KeybindingHandler",
	                                                   "org.cdos.SettingsDaemon.KeybindingHandler",
	                                                    NULL,
	                                                    &error);
	g_assert (self->priv->proxy != NULL);  //输出错误信息
	
	self->bindings = json_object_new ();
	
	self->kb_schema = g_settings_new (CUSTOM_KEYS_PARENT_SCHEMA);
	cdos_keybinding_manager_setup_custom_keybindings (self);
    //g_print ("after set up custom\n");
	
	self->csd_kb_settings = g_settings_new (MEDIA_KEYS_SCHEMA);
	g_signal_connect (self->csd_kb_settings, 
	                  "changed", 
	                  G_CALLBACK (cdos_keybinding_manager_setup_media_keys), 
	                  self);
	                  
	cdos_keybinding_manager_setup_media_keys (self);
	
	/*调试*/
	/*
	GList *list;
	GList *l;
	
	list = json_object_get_members (self->bindings);
	
	for (l = list; l != NULL; l = l->next)
	{
		JsonNode *node = json_object_get_member (self->bindings, (const gchar *)l->data);
		
		const gchar *value = json_node_get_string (node);
		
		g_print ("---->%s:%s\t\n", (const gchar *)l->data, value);
	}
	*/
}

/**
*cdos_keybinding_manager_new
*
*创建#CdosKeybindingManager对象
*
*Returns: 一个#CdosKeybindingManager对象
*/

CdosKeybindingManager *
cdos_keybinding_manager_new (void)
{
	CdosKeybindingManager *keybinding_manager;
	
	keybinding_manager = g_object_new (CDOS_TYPE_KEYBINDING_MANAGER, NULL);
	
    g_signal_connect (keybinding_manager->kb_schema, 
	                  "changed::custom-list", 
	                  G_CALLBACK (on_customs_changed), 
	                  keybinding_manager);
	
	g_print ("---->cdos_keybinding_manager_new\n");
	
	return keybinding_manager;
}

/**
*get_bindings_value_by_name
*@keybinding_manager: a #CdosKeybindingManager
*@name: name of a key
*
*寻找指定名称的key
*
*Returns: bindings中指定key的值。
*/

static
const gchar *
get_bindings_value_by_name (CdosKeybindingManager *keybinding_manager, gchar *name)
{
	JsonNode *node;
	JsonReader *reader;
    const gchar *value = NULL;
	
	g_return_if_fail (CDOS_IS_KEYBINDING_MANAGER (keybinding_manager));
	node = json_object_get_member (keybinding_manager->bindings, name);
    //g_print ("get value by name\n");
    if (node) {
	    reader = json_reader_new (node);
	    value = json_reader_get_string_value (reader);
    	//g_print ("---->value=%s\n", value);	 //////调试
	    json_reader_end_member (reader);
	    g_object_unref (reader);
    }
	return value;		
}

/**
*set_bindings_value
*@keybinding_manager: a #CdosKeybindingManager
*@name: name of a key
*@binding: value of a key
*
*设置指定名称key的值
*
*Returns: void
*/

static 
void
set_bindings_value (CdosKeybindingManager *keybinding_manager, gchar *name, gchar *binding)
{
	g_return_if_fail (CDOS_IS_KEYBINDING_MANAGER (keybinding_manager));
	
	/*
	gboolean has_member = json_object_has_member (keybinding_manager->bindings, name);
	if (!has_member)
	{
		g_print ("Don't have such name in bindings!\n");
		return;
	}
	*/
	json_object_remove_member (keybinding_manager->bindings, name);

	json_object_set_string_member (keybinding_manager->bindings, name, binding);
} 

/**
 * cdos_keybinding_manager_add_hot_key:
 * @keybinding_manager: a #CdosKeybindingManager
 * @name: name of a key
 * @binding: value of a key
 * @callback: function to run when the keybinding is invoked
 * @user_data: the data to pass to callback
 * @free_data: function to free user_data
 *
 * 添加热键绑定 
 *
 * Returns: 成功绑定热键返回TRUE; 否则，返回FALSE。
 */

gboolean
cdos_keybinding_manager_add_hot_key (CdosKeybindingManager *keybinding_manager,
                                     gchar *name, 
                                     gchar *binding, 
                                     MetaKeyHandlerFunc callback,
                                     gpointer user_data,
                                     GDestroyNotify free_data)
{
	//g_print ("\n---->in cdos_keybinding_manager_add_hot_key\n");   ////////调试
    //g_print ("name is %s\n", name);	
    //g_print ("In add hot key , code  is %s\n", (ProxyData *)user_data->value);	
	g_return_if_fail (CDOS_IS_KEYBINDING_MANAGER (keybinding_manager));
		
	if (get_bindings_value_by_name (keybinding_manager, name))
	{
		meta_display_remove_custom_keybinding (keybinding_manager->meta_display, name);
	}
		
	if (binding == NULL)
	{
		g_print ("Empty keybinding set for %s, ignoring\n", name);
			
		if (get_bindings_value_by_name (keybinding_manager, name))
		{
			json_object_remove_member (keybinding_manager->bindings, name);
		}
		
		meta_display_rebuild_keybindings (keybinding_manager->meta_display);
		
		return TRUE;
	}
	
    //g_print ("%s %d: %s 0x%x\n", __func__, __LINE__, binding, binding);
    //gpointer transfer = NULL;
    if (g_strrstr (name, "settings-daemon")){
        //ProxyData *data = (ProxyData *)user_data;
        
        //g_print ("In add hot key , code  is %s\n", data->value);	
    }else {
    }
	
	gboolean add_key = meta_display_add_custom_keybinding (keybinding_manager->meta_display,
	                                                       name,
	                                                       binding,
	                                                       (MetaKeyHandlerFunc)callback,
	                                                       (gpointer)user_data,
	                                                       free_data);
	
	if (!add_key)
	{
		g_print ("Warning, unable to bind hotkey with name \"%s\". The selected keybinding could already be in use.\n", name);
		
		meta_display_rebuild_keybindings (keybinding_manager->meta_display);
		
		return FALSE;
	}
	else
	{
		set_bindings_value (keybinding_manager, name, binding);
	}
	
	meta_display_rebuild_keybindings (keybinding_manager->meta_display);
	
	return TRUE;
}

/**
 * cdos_keybinding_manager_remove_hot_key:
 * @keybinding_manager: a #CdosKeybindingManager
 * @name: name of the key
 *
 * 移除热键绑定
 *
 */

void
cdos_keybinding_manager_remove_hot_key (CdosKeybindingManager *keybinding_manager, gchar *name)
{
	g_return_if_fail (CDOS_IS_KEYBINDING_MANAGER (keybinding_manager));
	
	meta_display_remove_custom_keybinding (keybinding_manager->meta_display, name);
	
	meta_display_rebuild_keybindings (keybinding_manager->meta_display);
	
	json_object_remove_member (keybinding_manager->bindings, name);
}

static
gchar *
create_custom_path (gchar *str)
{
	gchar *dest = (gchar *)g_malloc0 (CUSTOM_KEY_LENGTH);
	
	g_snprintf (dest, CUSTOM_KEY_LENGTH, CUSTOM_NEW_KEY_SCHEMA, str);
	
	//g_print ("dest=%s\n", dest);   ////////调试
	
	return dest;
}

static GPid
try_spawn (gchar **argv)
{
	GPid child_pid;
	GError *error = NULL;
	gboolean success = g_spawn_async (NULL,
	               argv,
	               NULL,
	               G_SPAWN_SEARCH_PATH | G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL,
	               NULL,
	               NULL,
	               &child_pid,
	               &error);
	if (!success)
	{
		g_print ("Error invoking g_spawn_async!\n");
		return 0;
	}
	if (error)
	{
		if (error->code == G_SPAWN_ERROR_NOENT)
		{
			g_print ("Command not found\n");
			g_error_free (error);
			return 0;
		}
	}
	return child_pid;
}

static GPid
spawn_command_line (gchar *command_line)
{
	gint argc;
	gchar **argv;
	GPid pid;
	
	gboolean success = g_shell_parse_argv (command_line, &argc, &argv, NULL);
	if (!success)
	{
		g_print ("Execution of %s failed!\n", command_line);
		return 0;
	}
	pid = try_spawn (argv);
	g_strfreev(argv);
	return pid;
}

static
void
setup_custom_keybindings_callback (MetaDisplay *display,
                                   MetaScreen *screen,
                                   MetaWindow *window,
                                   ClutterKeyEvent *event,
                                   MetaKeyBinding *binding,
                                   gpointer user_data)
{
	gchar *command = (gchar *) user_data;
	
    g_print ("command is %s\n", command);
	gint32 ret = cdos_util_spawn_command_line (command);
}

/**
 * cdos_keybinding_manager_setup_custom_keybindings:
 * @keybinding_manager: a #CdosKeybindingManager
 *
 * 设置用户自定义的按键绑定
 *
 */


void
cdos_keybinding_manager_setup_custom_keybindings (CdosKeybindingManager *keybinding_manager)
{
	//g_print ("\n---->in cdos_keybinding_manager_setup_custom_keybindings\n");  ////调试
	
	guint i;
	guint length;
	gchar **list;
	gchar *custom_path;
	GSettings *schema;
	gchar *command = NULL;
	gchar *binding = NULL;
	gchar *name;
	
	//g_return_if_fail (CDOS_IS_KEYBINDING_MANAGER (keybinding_manager));
	
	list = g_settings_get_strv (keybinding_manager->kb_schema, "custom-list");
	//g_print ("\n---->list[0]=%s\n", list[0]);     ///////调试
	length = g_strv_length (list);
	//g_print ("---->length=%d\n", length);  ///////调试
	
	//for (i = 0; list[i] != NULL; i++)
	for (i = 0; i < length; i++)
	{
		// g_print ("---->list[%d]=%s\n", i, list[i]);   ///////调试
		
		custom_path = create_custom_path (list[i]);
		
		//g_print ("---->custom_path=%s\n", custom_path);  ///////调试
		
		schema = g_settings_new_with_path (CUSTOM_KEYS_SCHEMA, custom_path);

		command = g_settings_get_string (schema, "command");
		//g_print ("---->command=%s\n", command);  ///////调试
		binding = g_settings_get_string (schema, "binding");
		
		//g_print ("---->command=%s\n", binding);  ///////调试
		name = g_strdup (list[i]);
		
		cdos_keybinding_manager_add_hot_key (keybinding_manager,
		                                     name,
		                                     binding,
											 (MetaKeyHandlerFunc) setup_custom_keybindings_callback,
		                                     command,
		                                     NULL);   //是否需要free command的函数？
		g_free(custom_path);
		custom_path = NULL;
		g_object_unref(schema);
		schema = NULL;
		g_free(name);
		name = NULL;
	}
	
	g_strfreev (list);
}

static
void
on_customs_changed (GSettings *settings, gchar *key, gpointer data)
{
    //g_print ("custom changed\n");
    CdosKeybindingManager *self = (CdosKeybindingManager *)data;
	cdos_keybinding_manager_remove_custom_keybindings (self);
	cdos_keybinding_manager_setup_custom_keybindings (self);
}

/**
 * cdos_keybinding_manager_remove_custom_keybindings:
 * @keybinding_manager: a #CdosKeybindingManager
 *
 * 移除用户自定义的按键绑定
 *
 */

void
cdos_keybinding_manager_remove_custom_keybindings (CdosKeybindingManager *keybinding_manager)
{

	GList *list;
	GList *l;

	g_return_if_fail (CDOS_IS_KEYBINDING_MANAGER (keybinding_manager));
	
	list = json_object_get_members (keybinding_manager->bindings);
	
	for (l = list; l != NULL; l = l->next)
	{
		if (g_strstr_len (l->data, -1, "custom") != NULL)
		{
			cdos_keybinding_manager_remove_hot_key (keybinding_manager, l->data);
		}
	}
	g_list_free(list);
}

static void
on_media_key_pressed (MetaDisplay *display,
                      MetaScreen *screen,
                      MetaWindow *window,
                      ClutterKeyEvent *event,
                      MetaKeyBinding *binding,
                      gpointer user_data)
{  
	GVariant *result;
	//GVariant *out_signature;
	GError *error = NULL;
	
    ProxyData *data = (ProxyData *)user_data;
    gchar *value_str = data->value;
    gint64 value = g_ascii_strtoull (value_str, NULL, 0);
    //g_print ("media key press: %s sha sha sha %d\n", value_str, value);
	//通过代理调用HandleKeybinding方法
	//需要传递整形参数，key 的代码，比如print screen 是20
	result = g_dbus_proxy_call_sync (data->proxy, "HandleKeybinding", g_variant_new ("(u)", value),
                                                    G_DBUS_CALL_FLAGS_NONE, -1,
                                                    NULL, &error);
	if (result == NULL)
	{
		g_print ("Call method HandleKeybinding failed!\n");
	}
}

/**
*parse_json_from_string
*@str: C string in JSON format
*
*解析JSON格式的C语言字符串
*
*Returns: JsonObject对象
*/

static JsonObject *
parse_json_from_string (const gchar *str)
{
	JsonParser *parser;
	JsonNode *root;
	JsonObject *json_object;
	
	parser = json_parser_new ();
	json_parser_load_from_data (parser, str, -1, NULL);
	
	root = json_parser_get_root (parser);

	json_object = json_node_dup_object (root);
	
	g_object_unref (parser);
	
	return json_object;
}

/**
*get_node_value
*@node: a JsonNode
*@member: a member in JsonNode
*
*获取JsonNode中指定名称所对应的值
*
*Returns: member对应的值
*/

static gpointer
get_node_value (JsonNode *node, gchar *member)
{
	JsonReader *reader;
	void *value = NULL;
	
	reader = json_reader_new (node);
	
	if (g_strcmp0 (member, "value_bak") == 0)
	{
		if (json_reader_read_member (reader, member))
		{
			gint32 res = json_reader_get_int_value (reader);
            //g_print ("value is %d\n", res);
			value = &res;
		}
	} 
	else
	{
		if (json_reader_read_member (reader, member))
		{
			gchar *str = (gchar *)json_reader_get_string_value (reader);
			value = str;
		}
	}
	
	g_object_unref (reader);
	
	return value;
}

static
void
proxy_data_free (gpointer data)
{
	g_slice_free (ProxyData, data);
    //g_print ("destroy proxy data\n");
}

/**
 * cdos_keybinding_manager_setup_media_keys:
 * @keybinding_manager: a #CdosKeybindingManager
 *
 * 设置多媒体按键绑定
 *
 */

void
cdos_keybinding_manager_setup_media_keys (CdosKeybindingManager *keybinding_manager)
{
	JsonParser *parser;
	JsonNode *root;
	JsonObject *json_object;
	JsonNode *node;
	GList *list;
	GList *l;
	gchar *keybinding;
	
	//g_print ("\n---->in cdos_keybinding_manager_setup_media_keys\n");  ///////调试
	
	keybinding = NULL;
	
	g_return_if_fail (CDOS_IS_KEYBINDING_MANAGER (keybinding_manager));
	
	parser = json_parser_new ();
	json_parser_load_from_data (parser, MediaKeyTypes, -1, NULL);
	
	root = json_parser_get_root (parser);
	
	json_object = json_node_get_object (root);
	
	//json_object = parse_json_from_string (MediaKeyTypes);
	
	list = json_object_get_members (json_object);

	keybinding_manager->priv = CDOS_KEYBINDING_MANAGER_GET_PRIVATE (keybinding_manager);
	CdosKeybindingManagerPrivate *priv = keybinding_manager->priv;
	for (l = list; l != NULL; l = l->next)
	{
		node = json_object_get_member (json_object, (const gchar *)l->data);
		
		gchar *key_value = (gchar *)get_node_value (node, "key");
		gchar *kb_value = (gchar *)get_node_value (node, "kb");
		gchar *keycode = (gchar *)get_node_value (node, "value");
		//g_print ("\n---->key=%s\t kb=%s\t value=%s\n", key_value, kb_value, keycode);  //////调试
		gchar *name = g_strconcat ("settings-daemon-", (const gchar *)l->data, NULL);
		
		ProxyData *data = NULL;
        data = g_malloc0 (sizeof (ProxyData));

		data->proxy = keybinding_manager->priv->proxy;
		data->value = g_strdup (keycode);
		
		if (key_value)
		{
            //g_print ("before add hot key , keycode %s\n", data->value);
			keybinding = g_settings_get_string (keybinding_manager->csd_kb_settings, key_value);
			cdos_keybinding_manager_add_hot_key (keybinding_manager,
			                                     name,
			                                     keybinding,
			                                     (MetaKeyHandlerFunc) on_media_key_pressed,
			                                     data,
			                                     (GDestroyNotify) proxy_data_free);
			                                     
			//g_print ("\n---->key_value:add_hot_key\n");   //////调试

		}
		else if (kb_value)
		{
			keybinding = g_strdup (kb_value);
			
			cdos_keybinding_manager_add_hot_key (keybinding_manager,
			                                     name,
			                                     keybinding,
			                                     (MetaKeyHandlerFunc) on_media_key_pressed,
			                                     data,
			                                     (GDestroyNotify) proxy_data_free);
			                                     
			//g_print ("\n---->kb_value:add_hot_key\n");   //////调试
			g_free (keybinding);
		}
		else
		{
			g_print ("Invalid media-keys keybinding: %s\n", (const gchar *)l->data);
		}
		g_free(name);
		priv->add_hot_key_user_data = g_list_append(priv->add_hot_key_user_data,data);
		keybinding = NULL;
	}

	g_list_free (list);
	g_object_unref (parser);

}
