//
// Created by zhangtao on 16-7-5.
// source:keyboard.js:KeyboardSource
/**
 * SECTION: cdos-keyboard-source
 * @short_description: 在托盘区创建虚拟键盘快捷操作标识。
 * @title: CdosKeyboardSource
 * @include: cdos-keyboard-source.h
 *
 * #CdosKeyboardSource继承自#CdosSource,当激活虚拟键盘后，将在托盘区创建对应的标识图标，提供相关的快捷操作。
 *
 * #CdosKeyboardSource仅供#CdosKeyboard使用，在其内部实例化，不推荐在外部实例化。
 */
#include "cdos-monkey-plugin.h"
#include "cdos-keyboard-source.h"
#include "cdos-keyboard.h"

typedef struct _CdosKeyboardSourceSignalIds CdosKeyboardSourceSignalIds;
/*-----------------信号枚举-------------------*/
enum {

    LAST_SIGNAL
};
/*-----------------实例私有结构体-------------------*/
struct _CdosKeyboardSourcePrivate{
    CdosKeyboard *keyboard;
    CdosKeyboardSourceSignalIds *signal_ids;
};


/*-----------------信号id-------------------*/
struct _CdosKeyboardSourceSignalIds{
    gulong self_on_open_id;
};
/*-----------------属性枚举-------------------*/
enum {
    PROP_0
};

/*-------global variables-------------*/
static gint CdosKeyboardSource_private_offset = 0;
static gint cdos_keyboard_source_signals[LAST_SIGNAL] = { 0 };
static gpointer cdos_keyboard_source_parent_class = NULL;

/*-------class functions---------*/
static void cdos_keyboard_source_init(CdosKeyboardSource *self);
static void cdos_keyboard_source_class_init(CdosKeyboardSourceClass *klass);
static void cdos_keyboard_source_dispose(GObject *object);
static void cdos_keyboard_source_finalize(GObject *object);

/**
 * 注册类；
 *
 *
 * */
GType
cdos_keyboard_source_get_type(void)
{
    static GType cdos_keyboard_source_type = 0;
    if(G_UNLIKELY(cdos_keyboard_source_type==0))
    {
        static const GTypeInfo cdos_keyboard_source_info = {
                sizeof(CdosKeyboardSourceClass),
                NULL,NULL,
                (GClassInitFunc)cdos_keyboard_source_class_init,
                NULL,NULL,
                sizeof(CdosKeyboardSource),
                0,
                (GInstanceInitFunc)cdos_keyboard_source_init
        };
        //类注册
        cdos_keyboard_source_type = g_type_register_static(CDOS_TYPE_SOURCE,"CdosKeyboardSource",&cdos_keyboard_source_info,0);
        //g_type_add_class_private(cdos_keyboard_source_type, sizeof(CdosKeyboardSourceClassPrivate));
        CdosKeyboardSource_private_offset =
                g_type_add_instance_private(cdos_keyboard_source_type, sizeof(CdosKeyboardSourcePrivate));

    }
    return cdos_keyboard_source_type;
}
/**
 *  获取实例的私有属性结构
 * */
static inline gpointer
cdos_keyboard_source_get_instance_private(CdosKeyboardSource *self){
    return (G_STRUCT_MEMBER_P (self, CdosKeyboardSource_private_offset));
}


/**
 * 实例初始化；
 *
 * */
static void
cdos_keyboard_source_init(CdosKeyboardSource *self)
{
    CdosKeyboardSourcePrivate *priv;
    self->priv=cdos_keyboard_source_get_instance_private(self);
    priv=self->priv;
    /*--------------实例初始化（构造实例）-------------*/
    priv->signal_ids = g_new0(CdosKeyboardSourceSignalIds,1);
}

/**
 *
 * 类结构初始化；
 *
 * */
static void
cdos_keyboard_source_class_init(CdosKeyboardSourceClass *klass)
{
    //转换为gobject；
    GObjectClass *gobject_class=G_OBJECT_CLASS(klass);
    g_type_class_adjust_private_offset(klass,&CdosKeyboardSource_private_offset);
    cdos_keyboard_source_parent_class = g_type_class_peek_parent(klass);

    /*-------------重写父类的虚方法---------------*/
    //gobject_class->constructed=cdos_keyboard_source_constructed;
    gobject_class->dispose=cdos_keyboard_source_dispose;
    gobject_class->finalize=cdos_keyboard_source_finalize;
}

/*------------类方法定义-----------*/
static void cdos_keyboard_source_dispose(GObject *object){
    CdosKeyboardSource *self = CDOS_KEYBOARD_SOURCE(object);
    CdosKeyboardSourcePrivate *priv = self->priv;



    G_OBJECT_CLASS(cdos_keyboard_source_parent_class)->dispose(object);
};

static void cdos_keyboard_source_finalize(GObject *object){
    CdosKeyboardSource *self = CDOS_KEYBOARD_SOURCE(object);
    CdosKeyboardSourcePrivate *priv = self->priv;

    if(g_object_is_floating(object)){
        g_warning ("A floating object was finalized. This means that someone called g_object_unref() on an object that had only a floating reference; the initial floating reference is not owned by anyone and must be removed with g_object_ref_sink().");
    }
    G_OBJECT_CLASS(cdos_keyboard_source_parent_class)->finalize(object);
};


/**
 * cdos_keyboard_source_new:
 * @keyboard:虚拟键盘实例对象
 *
 * 创建一个托管在托盘区的虚拟键盘快捷操作方式。
 *
 * Returns:当前实例对象
 * */
CdosKeyboardSource*
cdos_keyboard_source_new(CdosKeyboard *keyboard)
{
    CdosKeyboardSource *self;
    self = g_object_new(CDOS_TYPE_KEYBOARD_SOURCE, NULL);
    CdosKeyboardSourcePrivate *priv = self->priv;
    priv->keyboard = keyboard;
    cdos_source_set_title (CDOS_SOURCE(self),_("Keyboard"));
    StWidget *icon = cdos_keyboard_source_create_notification_icon(self);

    priv->signal_ids->self_on_open_id = \
            g_signal_connect(CDOS_SOURCE(self),"open",G_CALLBACK(cdos_keyboard_source_open),self);


    return self;
}

/**
 * cdos_keyboard_source_create_notification_icon:
 * @self:当前实例对象
 *
 * 创建快捷操作方式的图标
 *
 * Returns:返回一个#StWidget图标控件
 */
StWidget*
cdos_keyboard_source_create_notification_icon(CdosKeyboardSource *self){
    CdosKeyboardSourcePrivate *priv = self->priv;
    StWidget *icon = st_icon_new();
    st_icon_set_icon_name(ST_ICON(icon),"input-keyboard");
    st_icon_set_icon_type(ST_ICON(icon),ST_ICON_SYMBOLIC);
    st_icon_set_icon_size(ST_ICON(icon),SOURCE_ICON_SIZE);
    return icon;
}
/**
 * cdos_keyboard_source_handle_summary_click:
 * @self:当前实例对象
 *
 * 控制当前点击事件的响应和传递。
 *
 * 改方法是对父类方法的重写。
 *
 * Returns:TRUE-当前事件将不再继续传播，FALSE-当前事件将继续传播。
 */
gboolean
cdos_keyboard_source_handle_summary_click(CdosKeyboardSource *self){
    CdosKeyboardSourcePrivate *priv = self->priv;
    ClutterEvent *event = clutter_get_current_event();
    ClutterEventType type = clutter_event_type(event);
    if(type != CLUTTER_BUTTON_RELEASE){
        return FALSE;
    }
    cdos_keyboard_source_open(CDOS_SOURCE(self));
    return TRUE;
}
/**
 * cdos_keyboard_source_open:
 * @self:当前实例对象
 *
 * 显示虚拟键盘。
 *
 * 改方法是对父类方法的重写。
 */
void
cdos_keyboard_source_open(CdosKeyboardSource *self){
    CdosKeyboardSourcePrivate *priv = self->priv;
    cdos_keyboard_show(priv->keyboard);
}