/**
 * SECTION: cdos-message-tray
 * @short_description: 消息展示与处理.
 * @include: cdos-message-tray.h
 *
 * CdosMessageTrayClass 类用来展示消息,转发或者删除消息.
 */

#define CLUTTER_DISABLE_DEPRECATION_WARNINGS

#include "cdos-message-tray.h"
#include "cdos-global.h"
#include "cdos-notification.h"
#include "cdos-source.h"
#include "cdos-layout-manager.h"
#include "cdos-applet-manager.h"
#include "cdos-monkey-plugin.h"
#include "cdos-panel.h"

#define NOTIFICATION_TIMEOUT 3
#define NOTIFICATION_CRITICAL_TIMEOUT_WITH_APPLET 10
#define NOTIFICATION_ANIMATION_TIME 200
static gint notificationHideAnimCount = 0;
static gint notificationShownAnimCount = 0;
static gint notificationExpandAnimCount = 0;
static gboolean want_send_to_applet = TRUE;

G_DEFINE_TYPE (CdosMessageTray, cdos_message_tray, G_TYPE_OBJECT);

// signal
enum {
	NOTIFIY_APPLET_UPDATE,
	NOTIFIY_APPLET_LAST_SIGNAL
};

static guint cdos_message_tray_signals [NOTIFIY_APPLET_LAST_SIGNAL] = {0};

/* ========== declare ========== */
static gint cdos_message_tray_get_source_index (CdosMessageTray *self, CdosSource *source);
static void cdos_message_tray_lock (CdosMessageTray *self);
static void cdos_message_tray_show_notification (CdosMessageTray *self);
static void cdos_message_tray_update_showing_notification (CdosMessageTray *self);
static void cdos_message_tray_expand_notification (CdosMessageTray *self, gboolean autoExpanding);
static void cdos_message_tray_update_notification_timeout (CdosMessageTray *self, guint timeout);
static void cdos_message_tray_hide_notification (CdosMessageTray *self);

/* callback function */
static void cdos_message_tray_on_status_changed_cb (GObject *obj, guint state, gpointer user_data);
static void cdos_message_tray_unlock_cb (GObject *obj, gpointer user_data);
static void cdos_message_tray_on_button_pressed_cb (GObject *obj, ClutterActor *source, gpointer user_data);
static void cdos_message_tray_set_size_position_cb (GObject *obj, gpointer user_data);
static void cdos_message_tray_escape_tray_cb (GObject *obj, CdosMessageTray *self);
static void cdos_message_tray_on_source_destroy_cb (GObject *obj, guint reason, gpointer user_data);
static void cdos_message_tray_on_notification_destroy_cb (GObject *obj, guint reason, gpointer user_data);
static void cdos_message_tray_on_notification_expanded_cb (GObject *obj, gpointer user_data);
static void cdos_message_tray_update_lock_state_cb (GObject *obj, gpointer user_data);
static void cdos_message_tray_on_notification_enabled_updated_cb (GObject *obj, GParamSpec *pspec, gpointer user_data);

/* Private */
static void cdos_message_tray_update_state (CdosMessageTray *self)
{
	gboolean notificationUrgent = FALSE;
    gboolean notificationsPending = FALSE;
    gboolean notificationExpired = FALSE;
	gboolean notificationExpanded = FALSE;
    gboolean canShowNotification = FALSE;

	CdosNotification *notification = NULL;
    if (self->notificationQueue)
        notification = self->notificationQueue->data;

    //把第一个元素取出来，如果其urgency状态為critical
	notificationUrgent = notification && (notification->urgency == CRITICAL);

	//如果session presence状态不是busy且其urgency状态為critical
	notificationsPending = notification &&  (!self->busy || notificationUrgent);

    notificationExpanded = self->notificationBin == NULL?  FALSE:clutter_actor_get_y (CLUTTER_ACTOR (self->notificationBin)) < 0;

	notificationExpired = (self->notificationTimeoutId == 0 &&
                            !(self->notification && self->notification->urgency == CRITICAL) &&
                            !self->pointerInTray &&
                            !self->locked &&
                            !(self->pointerInKeyboard && notificationExpanded)
                            ) || self->notificationRemoved;

	//canShowNotification 取决于同时满足notificationsPending与notificationsEnabled;
	canShowNotification = notificationsPending && self->notificationsEnabled;

	if (self->notificationState == HIDDEN) {
		if (canShowNotification) {
			cdos_message_tray_show_notification (self);
		} else if (!self->notificationsEnabled) {

			if (notificationsPending) {
                GList *Queue_0 = self->notificationQueue; //this._notificationQueue.shift()
                self->notificationQueue = self->notificationQueue->next;
                self->notificationQueue->prev = NULL;
                self->notification = Queue_0->data;
                g_signal_emit_by_name (self, "notify-applet-update", self->notification);
                g_slice_free(GList, Queue_0);
			}
		}
	} else if (self->notificationState == SHOWN)
		if (notificationExpired)
			cdos_message_tray_hide_notification (self);
}

static gint cdos_message_tray_get_source_index (CdosMessageTray *self, CdosSource *source)
{
    gint index = g_list_index (self->sources, source);
    return index;
}

static void cdos_message_tray_lock (CdosMessageTray *self)
{
    self->locked = TRUE;
}

static void cdos_message_tray_show_notification (CdosMessageTray *self)
{
    gint x, y;
    ClutterModifierType mods;
    gdouble margin;
    MetaRectangle *monitor;
    CdosLayoutManager *layout_manager = cdos_layout_manager_get_default ();
    CdosPanel *panel = cdos_monkey_plugin_get_panel (monkey_plugin_get ());

    // this prevents a race condition with the messagetray wanting
    // to hide a notification before it's done showing it, when updating from applet
    self->notificationTimeoutId = 1;

    GList *Queue_0 = self->notificationQueue;
    self->notification = self->notificationQueue->data;
    self->notificationQueue = self->notificationQueue->next;
    if (self->notificationQueue)
        self->notificationQueue->prev = NULL;
    g_slice_free (GList, Queue_0);

    //如果有parent就让其把自己从被parent的child中移除
    if (self->notification && self->notification->actor_parent_container) {

        cdos_notification_collapseCompleted (self->notification);
        g_object_ref (self->notification->actor);
        clutter_container_remove_actor ((ClutterContainer *)self->notification->actor_parent_container, (ClutterActor *)self->notification->actor);
    }

    self->notificationClickedId = g_signal_connect (self->notification, "done-displaying", G_CALLBACK (cdos_message_tray_escape_tray_cb), self);

    st_bin_set_child (self->notificationBin, CLUTTER_ACTOR (self->notification->actor));

    clutter_actor_set_opacity (CLUTTER_ACTOR(self->notificationBin), 0);

    //取得layout manager中的primary monitor
    monitor = cdos_layout_manager_get_primary_monitor (layout_manager);

    //self->notificationBin.y = monitor.y + (Main.panel.actor.get_height()+5) * 2; // Notifications appear from here (for the animation)
    clutter_actor_set_y (CLUTTER_ACTOR(self->notificationBin),
                         monitor->y + ((clutter_actor_get_height (CLUTTER_ACTOR(panel)) + 5) * 2));

    //let margin = this._notification._table.get_theme_node().get_length('margin-from-right-edge-of-screen');
    margin = st_theme_node_get_length (st_widget_get_theme_node (ST_WIDGET (self->notification->table)),
                                       "margin-from-right-edge-of-screen");

    //self->notificationBin.x = monitor.x + monitor.width - this._notification._table.width - margin;
    clutter_actor_set_x (CLUTTER_ACTOR(self->notificationBin), monitor->x
                                                             + monitor->width
                                                             - clutter_actor_get_width (CLUTTER_ACTOR (self->notification->table))
                                                             - (gfloat) margin);

    clutter_actor_show (CLUTTER_ACTOR (self->notificationBin));

    cdos_message_tray_update_showing_notification (self);

    cdos_global_get_pointer (global, &x, &y, &mods);

    // We save the position of the mouse at the time when we started showing the notification
    // in order to determine if the notification popped up under it. We make that check if
    // the user starts moving the mouse and _onTrayHoverChanged() gets called. We don't
    // expand the notification if it just happened to pop up under the mouse unless the user
    // explicitly mouses away from it and then mouses back in.
    self->showNotificationMouseX = x;
    self->showNotificationMouseY = y;
    // We save the y coordinate of the mouse at the time when we started showing the notification
    // and then we update it in _notifiationTimeout() if the mouse is moving towards the
    // notification. We don't pop down the notification if the mouse is moving towards it.
    self->lastSeenMouseY = y;
}

static void cdos_message_tray_on_status_changed_cb (GObject *obj, guint status, gpointer user_data)
{

	CdosMessageTray *self = CDOS_MESSAGE_TRAY (user_data);
	self->backFromAway = (self->userStatus == IDLE) && (self->userStatus != status);
    self->userStatus = status;

	if (status == BUSY) {

		// remove notification and allow the summary to be closed now
		cdos_message_tray_update_notification_timeout (self, 0);
		self->busy = TRUE;
	} else if (status != IDLE) {

		// We preserve the previous value of this._busy if the status turns to IDLE
		// so that we don't start showing notifications queued during the BUSY state
		// as the screensaver gets activated.
		self->busy = FALSE;
	}

	cdos_message_tray_update_state(self);
}

static void cdos_message_tray_unlock_cb (GObject *obj, gpointer user_data)
{
	CdosMessageTray *self = CDOS_MESSAGE_TRAY (user_data);

	if (!self->locked) {
        return ;
    }

	self->locked = FALSE;
	cdos_message_tray_update_state (self);
}

static void cdos_message_tray_set_size_position_cb (GObject *obj, gpointer user_data)
{
    //let monitor = Main.layoutManager.primaryMonitor;
    //this._notificationBin.x = monitor.width - 500;
    //this._notificationBin.width = monitor.width;
}

static gint notification_compare_func (CdosNotification *notification1, CdosNotification *notification2)
{
    return (notification2->urgency - notification1->urgency);
}

static void cdos_message_tray_on_notify_cb (GObject *obj, CdosNotification *notification, gpointer user_data)
{
    CdosMessageTray *self = CDOS_MESSAGE_TRAY (user_data);


    // If a notification that is being shown is updated, we update
    // how it is shown and extend the time until it auto-hides.
    // If a new notification is updated while it is being hidden,
    // we stop hiding it and show it again.

    if (self->notification == notification) {
        cdos_message_tray_update_showing_notification (self);
    } else if (g_list_index (self->notificationQueue, notification) < 0) {

        g_signal_connect (notification, "destroy", G_CALLBACK (cdos_message_tray_on_notification_destroy_cb), self);

        self->notificationQueue = g_list_append (self->notificationQueue, notification);

        self->notificationQueue = g_list_sort (self->notificationQueue, (GCompareFunc)notification_compare_func);
    }

    cdos_message_tray_update_state (self);
}

static void cdos_message_tray_on_source_destroy_cb (GObject *obj, guint reason, gpointer user_data)
{
    CdosSource *source = CDOS_SOURCE (obj);
    CdosMessageTray *self = CDOS_MESSAGE_TRAY (user_data);

    gint index = cdos_message_tray_get_source_index (self, source);
    if (index == -1) {
        return ;
    }

    self->sources = g_list_remove (self->sources, source);

    gboolean needUpdate = FALSE;

    if (self->notification != NULL && self->notification->source == source) {

        cdos_message_tray_update_notification_timeout (self, 0);
        self->notificationRemoved = TRUE;
        needUpdate = TRUE;
    }

    if (needUpdate)
        cdos_message_tray_update_state (self);
}

static void cdos_message_tray_on_notification_destroy_cb (GObject *obj, guint reason, gpointer user_data)
{
    CdosNotification *notification = CDOS_NOTIFICATION (obj);
    CdosMessageTray *self = CDOS_MESSAGE_TRAY (user_data);

    gboolean tmp = self->notificationState == SHOWN || self->notificationState == SHOWING;

    if (self->notification == notification && tmp) {
        cdos_message_tray_update_notification_timeout (self, 0);

        self->notificationRemoved = TRUE;

        cdos_message_tray_update_state (self);

        return;
    }

    gint index = g_list_index (self->notificationQueue, notification);

    cdos_notification_destroy (notification, NOREASON); //reason unknow

    if (index != -1)
        self->notificationQueue = g_list_remove ( self->notificationQueue , notification );
}

static void cdos_message_tray_escape_tray_cb (GObject *obj, CdosMessageTray *self)
{
    cdos_message_tray_unlock_cb (obj, self);
    self->pointerInTray = FALSE;
    cdos_message_tray_update_notification_timeout (self, 0);
    cdos_message_tray_update_state (self);
}

static void cdos_message_tray_show_notification_completed_cb (ClutterTimeline *tml, gpointer user_data)
{
    CdosMessageTray *self = CDOS_MESSAGE_TRAY (user_data);
    self->notificationState = SHOWN;
    self->notificationTimeoutId = 0;

    if (self->notification->urgency != CRITICAL)
        cdos_message_tray_update_notification_timeout (self, NOTIFICATION_TIMEOUT * 1000);
    else
        cdos_message_tray_update_notification_timeout (self, NOTIFICATION_CRITICAL_TIMEOUT_WITH_APPLET);
    cdos_message_tray_update_state (self);
}

static void cdos_message_tray_update_showing_notification (CdosMessageTray *self)
{
    clutter_actor_remove_transition (CLUTTER_ACTOR (self->notificationBin), "notification-show");

    cdos_message_tray_expand_notification (self, TRUE);
    self->notificationState = SHOWING;
    cdos_transition_to (CLUTTER_ACTOR (self->notificationBin),
                        CLUTTER_EASE_OUT_QUAD,
                        NOTIFICATION_ANIMATION_TIME,
                        0,
                        (TransCallback)cdos_message_tray_show_notification_completed_cb, self,
                        "notification-show",
                        "opacity", TRANS_PROP_UCHAR, 255,
                        NULL);
}

static void cdos_message_tray_on_expand_notification_completed_cb (ClutterTimeline *tml, CdosMessageTray *self)
{
    self->notificationState = SHOWN;

    cdos_message_tray_update_state (self);
}

static void cdos_message_tray_on_notification_expanded_cb (GObject *obj, gpointer user_data)
{
    CdosMessageTray *self = CDOS_MESSAGE_TRAY (user_data);
    gfloat expandedY = clutter_actor_get_height (CLUTTER_ACTOR (self->notification->actor)) -
                        clutter_actor_get_height (CLUTTER_ACTOR (self->notificationBin));

    CdosLayoutManager *layout_manager = cdos_layout_manager_get_default ();
    MetaRectangle *monitor = cdos_layout_manager_get_primary_monitor (layout_manager);

    CdosPanel *panel = cdos_monkey_plugin_get_panel (monkey_plugin_get ());
    gfloat newY = monitor->y + clutter_actor_get_height (CLUTTER_ACTOR (panel)) + 5;



    if (clutter_actor_get_y (CLUTTER_ACTOR (self->notificationBin)) < expandedY) {
        clutter_actor_set_y (CLUTTER_ACTOR (self->notificationBin), expandedY);
    } else if (clutter_actor_get_y (CLUTTER_ACTOR (self->notificationBin)) != expandedY) {
        self->notificationState = SHOWING;
        clutter_actor_remove_transition (CLUTTER_ACTOR (self->notificationBin), "notification-expand");

        cdos_transition_to (CLUTTER_ACTOR (self->notificationBin),
                            CLUTTER_EASE_OUT_QUAD,
                            NOTIFICATION_ANIMATION_TIME,
                            0,
                            (TransCallback)cdos_message_tray_on_expand_notification_completed_cb, self,
                            "notification-expand",
                            "y", TRANS_PROP_FLOAT, newY,
                            NULL);
    }
}

static void cdos_message_tray_expand_notification (CdosMessageTray *self, gboolean autoExpanding)
{
    if (self->notificationExpandedId == 0) {
        self->notificationExpandedId =
            g_signal_connect (self->notification,
                              "expanded",
                              G_CALLBACK (cdos_message_tray_on_notification_expanded_cb),
                              self);
    }

    cdos_notification_expand (self->notification, !autoExpanding);
}

static gboolean cdos_message_tray_notification_timeout_cb (CdosMessageTray *self)
{
    int x, y;
    ClutterModifierType mods;
    cdos_global_get_pointer (global, &x, &y, &mods);

    if (y > self->lastSeenMouseY + 10) {

        self->lastSeenMouseY = y;
        cdos_message_tray_update_notification_timeout (self, 1000);
    } else {

        self->notificationTimeoutId = 0;
        cdos_message_tray_update_state (self);
    }

    return FALSE;
}

static void cdos_message_tray_update_notification_timeout (CdosMessageTray *self, guint timeout)
{
    if (self->notificationTimeoutId) {
        g_source_remove (self->notificationTimeoutId);
        self->notificationTimeoutId = 0;
    }

    if (timeout > 0)
        self->notificationTimeoutId =  g_timeout_add (timeout, (GSourceFunc)cdos_message_tray_notification_timeout_cb, self);

    return ;
}

static void cdos_message_tray_hide_notification_completed_cb (ClutterTimeline *tml, CdosMessageTray *self)
{

    if (self->notificationState == HIDDEN)
        return;
    self->notificationState = HIDDEN;

    g_object_ref(self->notification->actor);
    clutter_actor_hide (CLUTTER_ACTOR (self->notificationBin));
    st_bin_set_child (self->notificationBin, NULL);

    cdos_notification_collapseCompleted (self->notification);
    if(self->notificationClickedId > 0) {
        g_signal_handler_disconnect(self->notification, self->notificationClickedId);
        self->notificationClickedId = 0;
    }
    CdosNotification *notification = self->notification;

    if (want_send_to_applet && !self->notificationRemoved) {
        g_signal_emit_by_name (self, "notify-applet-update", notification);
        //g_object_unref (self->notification->actor);
    } else {
        if (notification->isTransient) {
            cdos_notification_destroy (notification, EXPIRED);
        }
    }

    self->notification = NULL;
    want_send_to_applet = TRUE;
    self->notificationRemoved = FALSE;
    cdos_message_tray_update_state (self);

}

static void
cdos_message_tray_hide_notification_stopped (ClutterTimeline *tml, gboolean is_finished, CdosMessageTray *self)
{
    if (!is_finished)
        cdos_message_tray_hide_notification_completed_cb (tml, self);
}

static void cdos_message_tray_hide_notification (CdosMessageTray *self)
{
    if (self->notificationExpandedId) {
        g_signal_handler_disconnect (self->notification, self->notificationExpandedId);
        self->notificationExpandedId = 0;
    }
    clutter_actor_remove_all_transitions ((ClutterActor *)self->notificationBin);

    if (self->notificationExpandedId) {

        g_signal_handler_disconnect (self->notification, self->notificationExpandedId);
        self->notificationExpandedId = 0;
    }

    CdosLayoutManager *layout_manager = cdos_monkey_plugin_get_layout_manager (monkey_plugin_get ());
    MetaRectangle *monitor = cdos_layout_manager_get_primary_monitor (layout_manager);
    self->notificationState = HIDING;
    cdos_transition_to (CLUTTER_ACTOR (self->notificationBin),
                        CLUTTER_EASE_OUT_QUAD,
                        NOTIFICATION_ANIMATION_TIME,
                        0,
                        (TransCallback)cdos_message_tray_hide_notification_completed_cb, self,
                        "hide-notification",
                        "opacity", TRANS_PROP_UCHAR, 0,
                        // 修复通知消失时,屏幕刷新
                        // "x", TRANS_PROP_FLOAT, (gfloat) (monitor->width),
                        "y", TRANS_PROP_FLOAT, (gfloat) (-10000),
                        "signal::stopped", cdos_message_tray_hide_notification_stopped, self,
                        NULL);
}

static void cdos_message_tray_update_lock_state_cb (GObject *obj, gpointer user_data)
{

    CdosMessageTray *self = CDOS_MESSAGE_TRAY (user_data);

    if (self->locked) {
        cdos_message_tray_unlock_cb (obj, self);
    } else {
        cdos_message_tray_update_state (self);
    }
}

static void cdos_message_tray_on_notification_enabled_updated_cb (GObject *obj, GParamSpec *pspec, gpointer user_data)
{

    g_return_if_fail (NULL != user_data && CDOS_IS_MESSAGE_TRAY (user_data));
    GSettings *settings = G_SETTINGS (obj);
    CdosMessageTray *self = CDOS_MESSAGE_TRAY (user_data);

    self->notificationsEnabled = g_settings_get_boolean (settings, "display-notifications");
}

/* public */
/**
  * cdos_message_tray_contains:
  * @self: A #CdosMessageTray
  * @source: #CdosSource 类型的参数
  *
  * 判断source已经存在于 (#GList)self->sources 里。
  *
  * Returns (boolean): 若存在，返回%TRUE；否则，返回%FALSE。
  */
gboolean cdos_message_tray_contains (CdosMessageTray *self, CdosSource *source)
{
    g_return_val_if_fail(CDOS_IS_MESSAGE_TRAY(self),FALSE);
    gboolean res = cdos_message_tray_get_source_index (self, source) >= 0;

    return res;
}

/**
  * cdos_message_tray_add:
  * @self: A #CdosMessageTray
  * @source: #CdosSource 参数,用来绑定信号.
  *
  * 判断source是否在 #CdosMessageTray 成员变量sources里：如果存在就退出，不存在就将对source绑定信号。
  */
void cdos_message_tray_add (CdosMessageTray *self, CdosSource *source)
{
    g_return_if_fail(CDOS_IS_MESSAGE_TRAY(self));
    g_return_if_fail(CDOS_IS_SOURCE(source));
    if (cdos_message_tray_contains (self, source)) {
        g_warning ("Trying to re-add source %s", source->title);
        return ;
    }

    g_signal_connect (source, "cdos-source-notify", G_CALLBACK (cdos_message_tray_on_notify_cb), self);
    g_signal_connect (source, "destroy", G_CALLBACK (cdos_message_tray_on_source_destroy_cb), self);
}

/**
  * cdos_message_tray_toggle:
  * @self: A #CdosMessageTray
  *
  * 切换显示 #CdosMessageTray 实例
  *
  */
void cdos_message_tray_toggle (CdosMessageTray *self)
{
    g_return_if_fail(CDOS_IS_MESSAGE_TRAY(self));
    self->traySummoned = !self->traySummoned;
    cdos_message_tray_update_state (self);
}

/**
  * cdos_message_tray_hide:
  * @self: A #CdosMessageTray
  *
  * 隐藏 #CdosMessageTray 的实例
  *
  */
void cdos_message_tray_hide (CdosMessageTray *self)
{
    g_return_if_fail(CDOS_IS_MESSAGE_TRAY(self));
    self->traySummoned = FALSE;
    cdos_message_tray_update_state (self);
}

/**
 * cdos_message_tray_want_send_to_applet:
 * @willSend: 是否需要通知发送到applet
 *
 * 自定义是否将同志发送到applet
 *
 */
void cdos_message_tray_want_send_to_applet (gboolean willSend)
{
    want_send_to_applet = willSend;
}

/* ======== Functions END ======= */

/* ========== GObject ========== */
static void
cdos_message_tray_dispose ( GObject *object ) {
    CdosMessageTray * self = CDOS_MESSAGE_TRAY(object);
    CdosLayoutManager *layout_manager = cdos_monkey_plugin_get_layout_manager (monkey_plugin_get ());
    cdos_layout_manager_remove_chrome (layout_manager, CLUTTER_ACTOR(self->notificationBin));
    if(self->notificationClickedId > 0){
        g_signal_handler_disconnect(self->notification,self->notificationClickedId);
        self->notificationClickedId = 0;
    }
    if(self->notificationExpandedId > 0){
        g_signal_handler_disconnect(self->notification,self->notificationExpandedId);
        self->notificationExpandedId = 0;
    }
    g_clear_object(&self->presence);
    g_clear_object(&self->notificationBin);
    g_list_free(self->notificationQueue);
    g_list_free(self->sources);

    G_OBJECT_CLASS(cdos_message_tray_parent_class)->dispose(object);
}
static void
cdos_message_tray_finalize ( GObject *object )
{
    //CdosMessageTray *self = CDOS_MESSAGE_TRAY(object);

    G_OBJECT_CLASS(cdos_message_tray_parent_class)->finalize(object);
}

static void
cdos_message_tray_class_init (CdosMessageTrayClass *klass)
{

	GObjectClass *gobject_class = G_OBJECT_CLASS(klass);

    /**
     * CdosMessageTray::notify-applet-update:
     * @object (GObject): 绑定信号的对象
     * @notification (CdosNotification): 发送的notification
     * @user_data (gpointer): 真对回调函数所传入的参数
     *
     * 当满足notification的条件可以被下发到 #AppletNotifications 时，
     * 可以捕获该信号
     */
	cdos_message_tray_signals [NOTIFIY_APPLET_UPDATE] =
			g_signal_new ("notify-applet-update",
						  G_OBJECT_CLASS_TYPE (klass),
						  G_SIGNAL_RUN_LAST,
						  0,
						  NULL, NULL,
						  g_cclosure_marshal_VOID__OBJECT,
						  G_TYPE_NONE, 1,
						  G_TYPE_OBJECT);

	gobject_class->finalize = cdos_message_tray_finalize;
	gobject_class->dispose = cdos_message_tray_dispose;

}

static void cdos_message_tray_init (CdosMessageTray *self)
{
	CdosGlobal *global = cdos_global_get ();

	GSettings *global_settings = cdos_global_get_settings (global);

	CdosLayoutManager *layout_manager = cdos_layout_manager_get_default ();

	self->presence = cdos_session_presence_new ();

	self->userStatus = AVAILABLE;

	self->busy = FALSE;

	self->backFromAway = FALSE;

	//g_signal_connect (self->presence, "StatusChanged", G_CALLBACK (cdos_message_tray_on_status_changed_cb), self);

    guint status = cdos_session_presence_get_status (self->presence, (GFunc)cdos_message_tray_on_status_changed_cb);
    cdos_message_tray_on_status_changed_cb (G_OBJECT (self->presence), status, self);

	self->notificationBin = ST_BIN (st_bin_new ());
	clutter_actor_hide (CLUTTER_ACTOR (self->notificationBin));

	self->notificationQueue = NULL; //GList
	self->notification = NULL;
	self->notificationClickedId = 0;

	self->pointerBarrier = 0;

	self->trayState = HIDDEN;
	self->locked = FALSE;
	self->traySummoned = FALSE;
	self->useLongerTrayLeftTimeout = FALSE;
	self->trayLeftTimeoutId = 0;
	self->pointerInTray = FALSE;
	self->pointerInKeyboard = FALSE;
	self->notificationState = HIDDEN;
	self->notificationTimeoutId = 0;
	self->notificationExpandedId = 0;
	self->notificationRemoved = FALSE;
	self->reNotifyAfterHideNotification = NULL;

	self->sources = NULL;

	//将notificationBin加入到layoutManager中
	cdos_layout_manager_add_chrome (layout_manager, CLUTTER_ACTOR(self->notificationBin), 4);

	//监听layoutManager的 monitior-change信号
	g_signal_connect (layout_manager, "monitors-changed", G_CALLBACK (cdos_message_tray_set_size_position_cb), self);

	/*
	 * FIXME
	 * 监听global settings的changed::display-notifications 信号
	 * 但是GSettings没有这个属性
	 */
	g_signal_connect (global_settings , "changed::display-notifications", G_CALLBACK (cdos_message_tray_on_notification_enabled_updated_cb), self );

    cdos_message_tray_on_notification_enabled_updated_cb (G_OBJECT(global_settings), NULL, self);
	cdos_message_tray_set_size_position_cb (G_OBJECT (layout_manager), self);

    CdosOverview *overview = cdos_overview_get ();
	g_signal_connect (overview, "showing", G_CALLBACK (cdos_message_tray_update_lock_state_cb), self);
	g_signal_connect (overview, "hiding", G_CALLBACK (cdos_message_tray_update_lock_state_cb), self);
}

/**
  * cdos_message_tray_new:
  *
  * 新建一个 #CdosMessageTray 类的实例。
  *
  * Returns (CdosMessageTray): 返回一个 #CdosMessageTray 类的实例.
  */
CdosMessageTray *cdos_message_tray_new (void)
{
    CdosMessageTray *self = g_object_new (CDOS_TYPE_MESSAGE_TRAY, NULL);

    return self;
}
