/**
 * SECTION: cdos-notification-daemon-source
 * @short_description: 通知守护进程数据结构
 * @include: cdos-notification-daemon-source.h
 * 
 * CdosNotificationDaemonSourceClass 类用来处理通知
 */
#include <gio/gio.h>
#include "cdos-notification-daemon-source.h"

#include "cdos-monkey-plugin.h"

/*全局变量*/
gulong global_handler_id; //use in global.disconnect(id)
CdosGlobal *global;

enum {
    PROP_TITLE_0 = 1,
    PROP_PID,
    PROP_SENDER,
    PROP_TRAY_ICON,
    NUM_PROPERTIES
};

struct _CdosNotificationDaemonSourcePrivate
{
    gchar *title, *sender;
    guint pid;

    CdosTrayIcon *trayIcon;
};

/*注册函数*/
G_DEFINE_TYPE_WITH_PRIVATE (CdosNotificationDaemonSource, cdos_notification_daemon_source, CDOS_TYPE_SOURCE);

#define CDOS_NOTIFICATION_DAEMON_SOURCE_GET_PRIVATE(obj) \
        (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CDOS_TYPE_NOTIFICATION_DAEMON_SOURCE, CdosNotificationDaemonSourcePrivate))

typedef struct _HandlerSummaryClick HandlerSummaryClick;

struct _HandlerSummaryClick {
    /*回调函数数据类型*/
    CdosTrayIcon *trayIcon;
    ClutterEvent *event;
};

/*声明*/
void handler_summary_click_cb (CdosGlobal *c_global, GParamSpec *pspec, HandlerSummaryClick *data);

static void cdos_notification_daemon_source_on_name_vanished (GDBusConnection *connection, const gchar *name,
                                                              CdosNotificationDaemonSource *self);

void cdos_notification_daemon_source_set_app (CdosNotificationDaemonSource *self);

static void cdos_notification_daemon_source_last_notification_removed (CdosNotificationDaemonSource *self);
static void cdos_notification_daemon_source_set_property (GObject *gobject, guint prop_id, const GValue *value, GParamSpec *pspec);
static void cdos_notification_daemon_source_get_property (GObject *gobject, guint prop_id, GValue *value, GParamSpec *pspec);

/* ========== finalize ========== */
static void cdos_notification_daemon_source_finalize (GObject *gobject)
{

    CdosNotificationDaemonSource *self = CDOS_NOTIFICATION_DAEMON_SOURCE(gobject);

    /*释放GObject类型成员变量*/
    g_free (self->app);
    g_free (self->priv->trayIcon);

    /* 调用父类的 finalize() 方法 */
    G_OBJECT_CLASS(cdos_notification_daemon_source_parent_class)->finalize (gobject);
};

/* ========= set property ========= */
static void
cdos_notification_daemon_source_set_property (GObject *gobject,
                                              guint prop_id,
                                              const GValue *value,
                                              GParamSpec *pspec)
{
    CdosNotificationDaemonSource *self = CDOS_NOTIFICATION_DAEMON_SOURCE (gobject);

    switch (prop_id)
    {
        case PROP_TITLE_0:
            cdos_notification_daemon_source_set_title (self, (gchar *)g_value_get_string (value));
            break;
        case PROP_PID:
            cdos_notification_daemon_source_set_pid (self, g_value_get_uint (value));
            break;
        case PROP_SENDER:
            cdos_notification_daemon_source_set_sender (self, (gchar *)g_value_get_string (value));
            break;
        case PROP_TRAY_ICON:
            cdos_notification_daemon_source_set_trayicon (self, g_value_get_object (value));
            break;

        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
            break;
    }
}



static void
cdos_notification_daemon_source_get_property (GObject *gobject,
                                              guint prop_id,
                                              GValue *value,
                                              GParamSpec *pspec)
{
    CdosNotificationDaemonSource *
    new_obj = CDOS_NOTIFICATION_DAEMON_SOURCE (gobject);

    switch (prop_id)
    {
        case PROP_TITLE_0:
            g_value_set_string (value, new_obj->priv->title);
            break;

        case PROP_PID:
            g_value_set_int (value, new_obj->priv->pid);
            break;

        case PROP_SENDER:
            g_value_set_string (value, new_obj->priv->sender);
            break;

        case PROP_TRAY_ICON:
            g_value_set_object (value, new_obj->priv->trayIcon);
            break;

        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
            break;
    }
}

//_init & _init
static void cdos_notification_daemon_source_class_init (CdosNotificationDaemonSourceClass *klass)
{
    g_type_class_add_private (klass, sizeof (CdosNotificationDaemonSourcePrivate));
    /*继承*/
    GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
    gobject_class->finalize = cdos_notification_daemon_source_finalize;

    gobject_class->get_property = cdos_notification_daemon_source_get_property;
    gobject_class->set_property = cdos_notification_daemon_source_set_property;

    CdosSourceClass *cdos_source_class = CDOS_SOURCE_CLASS (klass);
    cdos_source_class->open = cdos_notification_daemon_source_open;
    cdos_source_class->remove_last = cdos_notification_daemon_source_last_notification_removed;
    cdos_source_class->handle_summary_click = cdos_notification_daemon_source_handle_summary_click;

    /**
     * CdosNotificationDaemonSource:pid:
     * @gobject_class: 属性绑定的对象
     *
     * #CdosNotificationDaemonSource 的pid属性
     */
    g_object_class_install_property (gobject_class,
                                     PROP_PID,
                                     g_param_spec_uint ("pid", /*identifier*/
                                                        "PID", /*nickname*/
                                                        "The PID of the Notification Daemon Source",/*discreption*/
                                                        0,/*min*/
                                                        G_MAXUINT,/*max*/
                                                        0,/*default*/
                                                        G_PARAM_READWRITE/*flags*/
                                     ));

    /**
     * CdosNotificationDaemonSource:title:
     * @gobject_class: 属性绑定的对象
     *
     * #CdosNotificationDaemonSource 的title属性
     */
    g_object_class_install_property (gobject_class,
                                     PROP_TITLE_0,
                                     g_param_spec_string ("title",
                                                          "Title",
                                                          "The title of Notification Daemon Source",
                                                          NULL,
                                                          G_PARAM_READWRITE
                                     ));
    /**
     * CdosNotificationDaemonSource:sender:
     * @gobject_class: 属性绑定的对象
     *
     * #CdosNotificationDaemonSource 的sender属性
     */
    g_object_class_install_property (gobject_class,
                                     PROP_SENDER,
                                     g_param_spec_string ("sender",
                                                          "Sender",
                                                          "The sender of Notification Daemon Source",
                                                          NULL,
                                                          G_PARAM_READWRITE
                                     ));
    /**
     * CdosNotificationDaemonSource:trayIcon:
     * @gobject_class: 属性绑定的对象
     *
     * #CdosNotificationDaemonSource 的trayIcon属性
     */    
    g_object_class_install_property (gobject_class,
                                     PROP_TRAY_ICON,
                                     g_param_spec_object ("trayIcon",
                                                          "Tray Icon",
                                                          "The trayIcon of Notification Daemon Source",
                                                          G_TYPE_OBJECT,
                                                          G_PARAM_READWRITE
                                     ));
}

/**
 * cdos_notification_daemon_source_new:
 * @title: 标题
 * @pid: pid
 * @sender: 发送者
 * @trayIcon: 图标
 *
 * 新建一个 #CdosNotificationDaemonSource 类的实例。
 *
 * Returns: 返回一个 #CdosNotificationDaemonSource 的实例。
 */
CdosNotificationDaemonSource *
cdos_notification_daemon_source_new (gchar *title, guint pid, gchar *sender, CdosTrayIcon *trayIcon)
{


    CdosNotificationDaemonSource *self = g_object_new (CDOS_TYPE_NOTIFICATION_DAEMON_SOURCE, NULL);
    gchar *temp_title = g_strdup(title);
    cdos_source_set_title (CDOS_SOURCE (self), temp_title); //父类_init(this, title);
    g_object_set (self, "title", temp_title, "pid", pid, "sender",
                  sender, "trayIcon", trayIcon, NULL);

    /*仍然有必要写出参数列表,实例src/cdos-recorder.c:1571*/
    //cdos_notification_daemon_source_init (self);
    self->initialTitle = temp_title;

    if (sender) {
        /*
         * TODO:
         * dbus-glib implementation of watch_name() doesn’t return an id to be used for
         * unwatch_name() or implement unwatch_name(), however when we move to using GDBus implementation,
         * we should save the id here and call unwatch_name() with it in destroy().
         * Moving to GDBus is the work in progress: https://bugzilla.gnome.org/show_bug.cgi?id=648651
         * and https://bugzilla.gnome.org/show_bug.cgi?id=622921 .
         */

        g_bus_watch_name (G_BUS_TYPE_SESSION,
                          sender,
                          G_BUS_NAME_WATCHER_FLAGS_NONE,
                          NULL,
                          (GBusNameVanishedCallback) cdos_notification_daemon_source_on_name_vanished,
                          self,
                          NULL);

        cdos_notification_daemon_source_set_app (self);
        if (self->app) {
            cdos_source_set_title (CDOS_SOURCE(self), (gchar *) cdos_app_get_name (self->app)); /*const char=>gchar类型*/
        } else {
            self->useNotificationIcon = 1;
        }


        CdosTrayIcon * tray_icon = cdos_notification_daemon_source_get_trayicon (self);
        if (tray_icon) {
            cdos_source_set_summary_icon (CDOS_SOURCE(self), ST_ICON(tray_icon));/*self->trayIcon*/
            self->useNotificationIcon = 0;
        }
    }

    return self;
}

static void cdos_notification_daemon_source_init (CdosNotificationDaemonSource *self)
{
    self->priv = CDOS_NOTIFICATION_DAEMON_SOURCE_GET_PRIVATE (self);
};

/**
 * cdos_notification_daemon_source_set_title:
 * @self: A #CdosNotificationDaemonSource
 * @title: 标题 
 *
 * 设置self的"标题"属性
 *
 */
void cdos_notification_daemon_source_set_title (CdosNotificationDaemonSource *self, gchar *title)
{
    if (self->priv->title) {
        g_free (self->priv->title);
        self->priv->title = NULL;
    }

    self->priv->title = g_strdup ((const gchar *)title);

}

/**
 * cdos_notification_daemon_source_set_sender:
 * @self: A #CdosNotificationDaemonSource
 * @sender: 发送者 
 *
 * 设置self的"sender"属性
 *
 */
void cdos_notification_daemon_source_set_sender (CdosNotificationDaemonSource *self, gchar *sender)
{
    if (self->priv->sender) {
        g_free (self->priv->sender);
        self->priv->sender = NULL;
    }
    self->priv->sender = g_strdup ((const gchar *)sender);
}

/**
 * cdos_notification_daemon_source_set_pid:
 * @self: A #CdosNotificationDaemonSource
 * @pid: pid
 *
 * 设置self的"pid"属性
 *
 */
void cdos_notification_daemon_source_set_pid (CdosNotificationDaemonSource *self, guint pid)
{
    self->priv->pid = pid;
}

/**
 * cdos_notification_daemon_source_set_trayicon:
 * @self: A #CdosNotificationDaemonSource
 * @trayIcon: 设置图标
 *
 * 设置self的"trayIcon"属性
 *
 */
void
cdos_notification_daemon_source_set_trayicon (CdosNotificationDaemonSource *self, CdosTrayIcon *trayIcon)
{
    self->priv->trayIcon = trayIcon;
}

/**
 * cdos_notification_daemon_source_get_title:
 * @self: A #CdosNotificationDaemonSource
 *
 * 获取self的"标题"属性
 *
 * Returns: title.
 */
gchar * cdos_notification_daemon_source_get_title (CdosNotificationDaemonSource *self)
{
    return self->priv->title;
}

/**
 * cdos_notification_daemon_source_get_sender:
 * @self: A #CdosNotificationDaemonSource
 * @sender: 发送者 
 *
 * 读取self的"sender"属性
 *
 * Returns: sender.
 */
gchar * cdos_notification_daemon_source_get_sender (CdosNotificationDaemonSource *self)
{
    return self->priv->sender;
}

/**
 * cdos_notification_daemon_source_set_pid:
 * @self: A #CdosNotificationDaemonSource
 * @pid: pid
 *
 * 获取self的"pid"属性
 *
 * Returns: pid. 
 */
guint  cdos_notification_daemon_source_get_pid (CdosNotificationDaemonSource *self)
{
    return self->priv->pid;
}

/**
 * cdos_notification_daemon_source_get_trayicon:
 * @self: A #CdosNotificationDaemonSource
 *
 * 获取self的"trayIcon"属性
 *
 * Returns: trayIcon.
 */
CdosTrayIcon * cdos_notification_daemon_source_get_trayicon (CdosNotificationDaemonSource *self)
{
    return self->priv->trayIcon;
}

// other functions
void cdos_notification_daemon_source_on_name_vanished (GDBusConnection *connection, const gchar *name,
                                                       CdosNotificationDaemonSource *self)
{
    /*
     * Destroy the notification source when its sender is removed from DBus.
     * Only do so if this.app is set to avoid removing "notify-send" sources, senders
     * of which аre removed from DBus immediately.
     * Sender being removed from DBus would normally result in a tray icon being removed,
     * so allow the code path that handles the tray icon being removed to handle that case.
     */
    CdosTrayIcon *tray_icon = cdos_notification_daemon_source_get_trayicon (self);
    if (!tray_icon && self->app) {
        cdos_notification_daemon_source_destory (self);
    }
}

void cdos_notification_daemon_source_set_app (CdosNotificationDaemonSource *self)
{
    CdosWindowTracker *tracker;
    tracker = cdos_window_tracker_get_default ();
    if (self->app) {
        return;
    }

    guint pid = cdos_notification_daemon_source_get_pid (self);
    self->app = cdos_window_tracker_get_app_from_pid (tracker, pid);

    if (!self->app) {
        return;
    }

    // Only override the icon if we were previously using
    // notification-based icons (ie, not a trayicon) or if it was unset before
    CdosTrayIcon *tray_icon = cdos_notification_daemon_source_get_trayicon (self);
    if (!tray_icon) {
        self->useNotificationIcon = FALSE;
        StWidget *icon = NULL;
        /*GAppInfo is implemented by GDesktopAppInfo*/
        /*GDesktopAppInfo *appInfo;*/
        GAppInfo * info = G_APP_INFO (cdos_app_get_app_info (self->app));
        GIcon * info_icon = g_app_info_get_icon (info);;
        /*此处类型转换:GDesktopAppInfo->GAppInfo*/

        if (info != NULL && g_app_info_get_icon (info) != NULL) {
            icon = st_bin_new ();
            st_icon_set_gicon (ST_ICON (icon), info_icon);
            st_icon_set_icon_type (ST_ICON (icon), ST_ICON_FULLCOLOR);
            st_icon_set_icon_size (ST_ICON (icon), SOURCE_ICON_SIZE);
        }

        if (icon == NULL) {
            st_icon_set_icon_name (ST_ICON (icon), "application-x-executable");
            st_icon_set_icon_type (ST_ICON (icon), ST_ICON_FULLCOLOR);
            st_icon_set_icon_size (ST_ICON (icon), SOURCE_ICON_SIZE);
        }
        /* messageTray.js */
        cdos_source_set_summary_icon (CDOS_SOURCE(self), ST_ICON(tray_icon));
    }
}

/**
 * cdos_notification_daemon_source_open_app:
 * @self: A #CdosNotificationDaemonSource
 *
 * 打开App
 *
 */
void cdos_notification_daemon_source_open_app (CdosNotificationDaemonSource *self)
{
    if (self->app == NULL) {
        return;
    }

    GSList *windows;

    windows = cdos_app_get_windows (self->app);
    if (g_slist_length (windows) > 0) {
        /*cdos-app.c:990*/
        MetaWindow *mostRecentWindow = windows->data;
        cdos_activate_window (mostRecentWindow, 0, 0);
    }
}

/**
 * cdos_notification_daemon_source_open:
 * @self: A #CdosNotificationDaemonSource
 * @notification: 打开notification 
 *
 * 打开APP
 *
 */
void cdos_notification_daemon_source_open (CdosNotificationDaemonSource *self, CdosNotification *notification)
{
    cdos_source_destroy_non_resident_notifications (CDOS_SOURCE(self));
    cdos_notification_daemon_source_open_app (self);
}

/**
 * cdos_notification_daemon_source_destory:
 * @self: A #CdosNotificationDaemonSource
 *
 * 调用父类方法删除self
 *
 */
void cdos_notification_daemon_source_destory (CdosNotificationDaemonSource *self)
{
    cdos_source_destroy (CDOS_SOURCE(self), NOREASON);
}

static void cdos_notification_daemon_source_last_notification_removed (CdosNotificationDaemonSource *self)
{
    CdosTrayIcon *tray_icon = cdos_notification_daemon_source_get_trayicon (self);
    if (!tray_icon) {
        cdos_notification_daemon_source_destory (self);
    }
}

/*c_handler in cdos_notification_daemon_source_handle_summary_click*/
void handler_summary_click_cb (CdosGlobal *c_global, GParamSpec *pspec, HandlerSummaryClick *data)
{
}

/**
 * cdos_notification_daemon_source_handle_summary_click:
 * @self: A #CdosNotificationDaemonSource
 *
 * Left clicks are passed through only where there aren't unacknowledged
 * notifications, so it possible to open them in summary mode; right
 * clicks are always forwarded, as the right click menu is not useful for
 * tray icons
 *
 * Returns: %FALSE 的条件: 
 * （1）#CdosNotificationDaemonSource 获取不到trayIcon
 * （2）点击右键，并且通知多余1
 * 其他返回 %TRUE 
 */
gboolean cdos_notification_daemon_source_handle_summary_click (CdosNotificationDaemonSource *self)
{
    HandlerSummaryClick *params;

    CdosTrayIcon *tray_icon = cdos_notification_daemon_source_get_trayicon (self);
    if (!tray_icon) {
        return FALSE;
    }

    /*clutter-event.h:518*/
    const ClutterEvent *event;
    event = clutter_get_current_event ();
    if (clutter_event_type (event) != CLUTTER_BUTTON_RELEASE) {
        return TRUE;
    }

    /*
     * Left clicks are passed through only where there aren't unacknowledged
     * notifications, so it possible to open them in summary mode; right
     * clicks are always forwarded, as the right click menu is not useful for
     * tray icons
     */

    guint length = g_list_length (CDOS_SOURCE(self)->notifications);

    if (clutter_event_get_button (event) == 1 && length > 0) {
        return FALSE;
    }

    /*
     * We can't just connect to Main.overview's 'hidden' signal,
     * because it's emitted *before* it calls popModal()...
     */
    global = cdos_global_get ();

    CdosOverview *over_view = cdos_overview_get ();
    if (cdos_overview_get_visible (over_view)) {

        params = g_new0(HandlerSummaryClick, 1);
        params->event = (ClutterEvent *)event;
        params->trayIcon = tray_icon;

        global_handler_id = g_signal_connect(global, "notify::stage-input-mode",
                                             G_CALLBACK (handler_summary_click_cb),
                                             params);//id即是返回值,又是传入的参数, 设置为全局变量
        //cdos_overview_hide (over_view);
        clutter_actor_hide ((ClutterActor *)over_view);
    } else {
        //cdos_tray_icon_click (tray_icon, (ClutterEvent *) (event));
    }

    return TRUE;
}

/**
 * cdos_notification_daemon_source_process_notification:
 * @self: A #CdosNotificationDaemonSource
 * @notification: 接收的通知
 * @icon: 图标 
 *
 * 处理通知的进程程序。
 */
void cdos_notification_daemon_source_process_notification (CdosNotificationDaemonSource *self,
                                                           CdosNotification *notification, CdosTrayIcon *icon)
{
    if (!self->app) {
        cdos_notification_daemon_source_set_app (self);
    }
    if (!self->app && icon) {
        /*应该是父类方法*/
        cdos_source_set_summary_icon (CDOS_SOURCE(self), icon);/*self, icon*/
    }

    CdosWindowTracker *tracker = cdos_window_tracker_get_default ();
    CdosApp *focus_app;
    g_object_get (tracker, "focus-app", &focus_app, NULL);

    if (notification->resident && (self->app != NULL) && (focus_app == self->app)) {
        /*原为:notification.resident && this.app && tracker.focus_app == this.app*/
        cdos_source_push_notification (CDOS_SOURCE(self), notification);
    } else {
        cdos_source_notify (CDOS_SOURCE(self), notification);
    }
    g_object_unref (tracker);
    g_free (focus_app);
}
