/**
 * SECTION: cdos-notification-daemon
 * @short_description: 通知守护进程
 * @include: cdos-notification-daemon.h
 * 
 * CdosNotificationDaemonClass 类用来处理消息转发
 */
#include <stdio.h>
#include <string.h>
#include <glib.h>
#include <regex.h>

#include "cdos-notification-daemon-source.h"
#include "cdos-notification-daemon.h"
#include "cdos-generate-from-notification-daemon-iface-xml.h"
#include "cdos-monkey-plugin.h"

/*messageTray.js: IMAGE_SIZE: 125*/
#define IMAGE_SIZE 125

/*================= 注册, 定义 ==================*/
/*注册, 定义*/
G_DEFINE_TYPE(CdosNotificationDaemon, cdos_notification_daemon, G_TYPE_OBJECT)

/*G_CALLBACK user_data*/
/*use in notification.connect-destroy*/
typedef struct _notificationParams notificationParams;
/*use in Notify, 二级回调*/
typedef struct _sourceParams sourceParams;
typedef struct _callMethodParams callMethodParams;
typedef struct _ndataStruct ndataStruct;

static const gchar introspection_xml[] =
        "<node name=\"/org/freedesktop/Notifications\">"
        "<interface name=\"org.freedesktop.Notifications\">"
          "<method name=\"Notify\">"
            "<arg type=\"s\" name=\"app_name\" direction=\"in\" />"
            "<arg type=\"u\" name=\"id\" direction=\"in\" />"
            "<arg type=\"s\" name=\"icon\" direction=\"in\" />"
            "<arg type=\"s\" name=\"summary\" direction=\"in\" />"
            "<arg type=\"s\" name=\"body\" direction=\"in\" />"
            "<arg type=\"as\" name=\"actions\" direction=\"in\" />"
            "<arg type=\"a{sv}\" name=\"hints\" direction=\"in\" />"
            "<arg type=\"i\" name=\"timeout\" direction=\"in\" />"
            "<arg type=\"u\" name=\"return_id\" direction=\"out\" />"
          "</method>"
          "<method name=\"CloseNotification\">"
            "<arg  type=\"u\" direction=\"in\" />"
          "</method>"
          "<method name=\"GetCapabilities\">"
            "<arg  type=\"as\" direction=\"out\" />"
          "</method>"
          "<method name=\"GetServerInformation\">"
            "<arg  type=\"s\" name=\"return_name\"         direction=\"out\" />"
            "<arg  type=\"s\" name=\"return_vendor\"       direction=\"out\" />"
            "<arg  type=\"s\" name=\"return_version\"      direction=\"out\" />"
            "<arg  type=\"s\" name=\"return_spec_version\" direction=\"out\" />"
          "</method>"
          "<signal name=\"NotificationClosed\">"
            "<arg  type=\"uu\" />"
          "</signal>"
          "<signal name=\"ActionInvoked\">"
            "<arg  type=\"us\" />"
          "</signal>"
        "</interface>"
      "</node>";

struct _notificationParams
{
    guint ndata_id;
    CdosNotificationDaemon *self;
};

struct _sourceParams
{
    gchar *sender;
    CdosNotificationDaemon *self;
};

struct _callMethodParams
{
    guint id;
    gchar *sender;
    CdosNotificationDaemon *self;
    GObject *ex;//没用到
};

/*ndata结构*/
struct _ndataStruct
{
    GList *actions; /*data为gchar类型的GList*/
    gchar *app_name;
    gchar *body;
    GHashTable *hints;
    guint id;
    gchar *icon;
    CdosNotification *notification;
    gchar *summary;
    guint timeout;
};

/*私有函数声明*/
static ClutterActor *cdos_notification_daemon_icon_for_notification_data (gchar *icon, GHashTable *hints, int size);

static CdosNotificationDaemonSource *cdos_notification_daemon_lookup_source (CdosNotificationDaemon *self, gchar *title,
                                                                      guint pid, CdosTrayIcon *trayIcon,
                                                                      gboolean trayIconStatus);

static CdosSource *cdos_notification_daemon_get_source (CdosNotificationDaemon *self, gchar *title,
                                                 guint pid, ndataStruct *ndata, gchar *sender,
                                                 CdosTrayIcon *icon);

static void cdos_notification_daemon_notify_for_source (CdosNotificationDaemon *self,
                                                        CdosNotificationDaemonSource *source, ndataStruct *ndata);

static void cdos_notification_daemon_on_focus_app_changed_cb (GObject *gobject, GParamSpec *pspec, gpointer user_data);

static void cdos_notification_daemon_emit_notification_closed (CdosNotificationDaemon *self, guint id, guint reason);

static void cdos_notification_daemon_emit_action_invoked (CdosNotificationDaemon *self, guint id, gchar *action);

static void cdos_notification_daemon_on_tray_icon_added_cb (CdosIconDispatcher *obj, CdosTrayIcon *icon,
                                                     gpointer user_data);

static void cdos_notification_daemon_on_tray_icon_removed_cb (GObject *obj, CdosTrayIcon *icon, gpointer user_data);

static void splice_source_cb (GObject *object, gpointer user_data);

/*自定义函数声明*/
static void generate_new_proxy (CdosNotificationDaemon *self);

static GList *create_new_rewrite_rules (void);

static void get_all_version_hints_image_data (GValue *image_data_struct, guchar *data, gint *width, gint *height,
                                       gint *rowstride, gboolean *has_alpha, gint *bits_per_sample, gint *n_channels,
                                       gsize *length);

static const gchar *get_latest_hints_image_path (GHashTable *hints);

static const gchar *get_old_hints_image_path (GHashTable *hints);

static guint get_hints_urgency (GHashTable *hints);

static gboolean get_hints_transient (GHashTable *hints);

static gboolean get_hints_action_icons (GHashTable *hints);

static gboolean get_hints_resident (GHashTable *hints);

static gint get_sender_to_pid (GHashTable *senderPid, gchar *sender);

static void notification_destroy_cb (GObject *obj, guint reason, gpointer user_data);

static void notification_action_invoked_cb (GObject *obj, gchar *actionId, gpointer user_data);

static void callback_notification_clicked (GObject *obj, notificationParams *params);

static void proxy_sender_cb (GObject *source_obj , GAsyncResult *result, gpointer user_data);
static void source_destroy_cb (GObject *object, guint reason, gpointer user_data);
static void on_notification_dbus_acquried(GObject *obj, const gchar *name, gpointer user_data);
static void handle_method_call(GDBusConnection *connection G_GNUC_UNUSED,
                               const gchar *sender G_GNUC_UNUSED,
                               const gchar *object_path G_GNUC_UNUSED,
                               const gchar *interface_name,
                               const gchar *method_name,
                               GVariant *parameters,
                               GDBusMethodInvocation *invocation,
                               gpointer user_data);

/*外部接口, 待替换*/


/*信号*/
enum NotificationClosedReason {
    DAEMON_EXPIRED = 1,
    DAEMON_DISMISSED,
    DAEMON_APP_CLOSED,
    DAEMON_UNDEFINED
};

enum Notification_Urgency {
    URGENCY_LOW = 0,
    URGENCY_NORMAL,
    URGENCY_CRITICAL
};

/* 全局变量 */
static const GDBusInterfaceVTable interface_vtable = {
    handle_method_call,
    NULL,
    NULL,
};
guint nextNotificationId = 0;

/*================= 函数声明, 布尔值 , 全局变量, 外部接口 END ====================*/

/*======================== const Bus, const rewriteRules ==============================*/

static GQuark
notification_daemon_error_quark ()
{
    static GQuark notification_error_quark = 0;
    if (G_UNLIKELY (notification_error_quark == 0))
        notification_error_quark = g_quark_from_static_string ("notification-daemon-quark");
    return notification_error_quark;
}

static void generate_new_proxy (CdosNotificationDaemon *self)
{
    
    GError *proxy_error = NULL;//g_error_new(notification_daemon_error_quark (), 0, "notification-daemon");
    self->bus_proxy = NULL;
    /*GDBusInterfaceInfo *interfaceInfo = org_freedesktop_dbus_interface_info();*/

    /*新建代理*/
    const gchar *bus_name = "org.freedesktop.DBus";
    const gchar *bus_path = "/org/freedesktop/DBus";
    const gchar *interface_name = "org.freedesktop.DBus";

    /*原型:g_dbus_proxy_new_sync(connection, G_DBUS_PROXY_FLAGS_NONE, interfaceInfo, bus_name,bus_path, cancellable, &error);*/
    /*该函数是生成函数, ,类型转换:OrgFreedesktopDBus->GDBusProxy*/

    self->bus_proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
                                                     G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES,
                                                     NULL,
                                                     bus_name,
                                                     bus_path,
                                                     interface_name,
                                                     NULL, &proxy_error);
    if (self->bus_proxy == NULL) {
        g_warning ("Couldn't get org.freedesktop.DBus proxy: %s", proxy_error->message);
        if (proxy_error)
            g_error_free (proxy_error);
        return;
    }
    
}

/*正则表达式*/
/*正则表达式节点结构体*/
typedef struct _RuleXChatNode RuleXChatNode;

struct _RuleXChatNode {
    const gchar *pattern;
    const gchar *replacement;
};

/*创建正则结构体*/
static GList *create_new_rewrite_rules (void)
{
    GList *rule_xchat_list = NULL;

    RuleXChatNode *rule_xchat_node1, *rule_xchat_node2, *rule_xchat_node3;/*申请空间, 存放pattern-replacement*/
    rule_xchat_node1 = g_new0(RuleXChatNode, 1);
    rule_xchat_node2 = g_new0(RuleXChatNode, 1);
    rule_xchat_node3 = g_new0(RuleXChatNode, 1);

    /*正则表达式节点初始化*/
    rule_xchat_node1->pattern = "/^XChat: Private message from: (\\S*) \\(.*\\)$/";
    rule_xchat_node1->replacement = "<$1>";

    rule_xchat_node2->pattern = "/^XChat: New public message from: (\\\\S*) \\\\((.*)\\\\)$/";
    rule_xchat_node2->replacement = "$2 <$1>";

    rule_xchat_node3->pattern = "/^XChat: Highlighted message from: (\\S*) \\((.*)\\)$/";
    rule_xchat_node3->replacement = "$2 <$1>";

    /*XChat:value->reule_xchar_list*/
    rule_xchat_list = g_list_append (rule_xchat_list, rule_xchat_node1);
    rule_xchat_list = g_list_append (rule_xchat_list, rule_xchat_node2);
    rule_xchat_list = g_list_append (rule_xchat_list, rule_xchat_node3);

    return rule_xchat_list;
}

/*===============================  END ====================================*/

/*=============================== GHashTable 解析函数 ==============================*/
/*
目前已知的hints成员:
 image-path, image-data: > V1.2, latest version
 image_path, image_data: v1.1, old version
 icon_data: early version, only use in image-path not available
 hints_urgency: 0, 1, 2
 hints_transient,
 hints_resident,
 hints_action-icons
*/

/*解析hints:image data, 但是image-data-strcut不同版本,不一样, 版本区别方法类似GValue *image_data_struct = (GValue *)g_hash_table_lookup(hints, "image_data");*/
/*mate-notification-daemon/src/daemon/daemon.c:1454*/
static void get_all_version_hints_image_data (GValue *image_data_struct, guchar *data, gint *width, gint *height,
                                       gint *rowstride, gboolean *has_alpha, gint *bits_per_sample, gint *n_channels,
                                       gsize *length)
{
    //参考mate-notification-daemon源码包, mate-notification-daemon/src/daemon/daemon.c:850

    GArray *tmp_array, *image_struct;/*GValue->GArray*/;
    gsize expected_len;

    /*width, height, rowstride, has_alpha, bits_per_sample, n_channels, temp_array, */
    /*类型转换*/
    image_struct = (GArray *) g_value_get_boxed (image_data_struct);/*GValue->GArray*/

    /*取出image_data结构体的成员*/
    *width = g_array_index (image_struct, gint, 0);/*value: width*/

    /*取出第二个成员,以此类推*/
    *height = g_array_index (image_struct, gint, 1);

    *rowstride = g_array_index (image_struct, gint, 2);

    *has_alpha = g_array_index (image_struct, gboolean, 3);

    *bits_per_sample = g_array_index (image_struct, gint, 4);

    *n_channels = g_array_index (image_struct, gint, 5);

    tmp_array = &g_array_index (image_struct, GArray, 6);

    expected_len = ((*height) - 1) * (*rowstride) + (*width) * (((*n_channels) * (*bits_per_sample) + 7) / 8);

    if (expected_len != tmp_array->len) {
        g_warning ("get all version hints image data expected image data to be not the expect length");
        return;
    }

    *length = expected_len;

    data = (guchar *) g_memdup (tmp_array->data, tmp_array->len);
    /*返回一个从tmp_array->data开始, tmp_array->len bytes长度的字符串拷贝*/
}

/*解析hints:所有版本的image path, 解析hints:image-path, Latest Version*/
static const gchar *get_latest_hints_image_path (GHashTable *hints)
{
    const gchar *image_path_new;

    const GValue *image_path_new_value = (GValue *) g_hash_table_lookup (hints, "image-path");
    image_path_new = g_value_get_string (image_path_new_value);

    return image_path_new;
}

/*解析hints:image_path: V1.1*/
static const gchar *get_old_hints_image_path (GHashTable *hints)
{
    //const GValue *image_path_old_value = (GValue *) g_hash_table_lookup (hints, "image_path");
    const gchar *image_path_old_value = g_hash_table_lookup (hints, "image_path");
    return image_path_old_value;
}

/*解析hints:urgency*/
static guint get_hints_urgency (GHashTable *hints)
{
    gulong urgency = (gulong)(gpointer)g_hash_table_lookup (hints, "urgency");
    return urgency;
}

static gboolean get_hints_transient (GHashTable *hints)
{
    gboolean transient = (gulong)(gpointer)g_hash_table_lookup (hints, "transient");
    return transient;
}

static gboolean get_hints_action_icons (GHashTable *hints)
{
    const GValue *hints_action_icons_value = g_hash_table_lookup (hints, "action-icons");

    gboolean hints_action_icon = g_value_get_boolean (hints_action_icons_value);

    return hints_action_icon;
}

static gboolean get_hints_resident (GHashTable *hints)
{
    gboolean resident = (gulong)(gpointer)g_hash_table_lookup (hints, "resident");

    return resident;
}

static gint get_sender_to_pid (GHashTable *senderPid, gchar *sender)
{
    gint pid;
    const GValue *sender_to_pid_value = g_hash_table_lookup (senderPid, (const gchar *)sender);
    pid = g_value_get_int (sender_to_pid_value);

    return pid;
}

/*=============================== handle GHashTable END ============================*/

/*=============================== CALLBACK FUNCTION ==================================*/
static void splice_source_cb (GObject *object, gpointer user_data)
{
    CdosNotificationDaemonSource *source = CDOS_NOTIFICATION_DAEMON_SOURCE (object);
    CdosNotificationDaemon *self = CDOS_NOTIFICATION_DAEMON (user_data);

    GList *l = g_list_find (self->sources, source);
    if (l) {
        /* if found */
        CdosNotificationDaemonSource *tmp = l->data;
        self->sources = g_list_remove (self->sources, tmp);
        g_object_unref(tmp);
    }
}

static void notification_destroy_cb (GObject *obj, guint reason, gpointer user_data)
{
    //CdosNotification *notification = CDOS_NOTIFICATION (obj);
    notificationParams *params = (notificationParams *)user_data;
    CdosNotificationDaemon *self = params->self;
    guint ndata_id = params->ndata_id;

    ndataStruct *data = g_list_nth_data (self->notifications, ndata_id);/*取出第ndata_id个*/
    self->notifications = g_list_remove (self->notifications, data);

    guint notificationClosedReason = 0;
    switch (reason) {
        case EXPIRED:
            notificationClosedReason = DAEMON_EXPIRED;
            break;
        case DISMISSED:
            notificationClosedReason = DAEMON_DISMISSED;
            break;
        case SOURCE_CLOSED:
            notificationClosedReason = DAEMON_APP_CLOSED;
            break;
    }

    cdos_notification_daemon_emit_notification_closed (self, ndata_id, notificationClosedReason);
    g_free(params);
}

static void notification_action_invoked_cb (GObject *obj, gchar *actionId, gpointer user_data)
{

    notificationParams *params = (notificationParams *)user_data;
    CdosNotificationDaemon *self = params->self;
    guint ndata_id = params->ndata_id;

    cdos_notification_daemon_emit_action_invoked (self, ndata_id, actionId);
    g_free(params);
}

static void callback_notification_clicked (GObject *obj, notificationParams *params)
{
    CdosNotificationDaemon *self = params->self;
    guint ndata_id = params->ndata_id;

    cdos_notification_daemon_emit_action_invoked (self, ndata_id, "default");
    g_free(params);

}

static void
cdos_notification_daemon_on_tray_icon_added_cb (CdosIconDispatcher *obj,
                                                CdosTrayIcon *icon,
                                                gpointer user_data)
{
    //原函数_onTrayIconAdded: function(o, icon),statusIconDispatcher.js中没有emit动作
    //猜测o是object, 假定信号类型是_VOID_CHAR_OBJECT

    CdosNotificationDaemon *self = CDOS_NOTIFICATION_DAEMON (user_data);
    gchar *title = NULL, *wm_class = NULL, *str = NULL;
    guint pid;
    g_object_get (icon, "wm-class", &wm_class, "title", &title, "pid", &pid, NULL);
    if (title)
        str = g_strdup ((const gchar *) title);
    else if (wm_class) {
        str = g_strdup ((const gchar *) wm_class);
    } else {
        str = "Unknown";
    }

    CdosSource *source = cdos_notification_daemon_get_source (self, str, pid, NULL, NULL, icon);
    g_free(str);
    if (!source) {
        g_warning ("source get error.");
    }
}

static void cdos_notification_daemon_on_tray_icon_removed_cb (GObject *obj, CdosTrayIcon *icon, gpointer user_data)
{
    CdosNotificationDaemon *self = CDOS_NOTIFICATION_DAEMON (user_data);
    guint pid;
    g_object_get (icon, "pid", &pid, NULL);

    CdosNotificationDaemonSource *source = cdos_notification_daemon_lookup_source (self, NULL, pid, icon, FALSE);

    if (source) {
        cdos_notification_daemon_source_destory (source);
    }
}

static void cdos_notification_daemon_on_focus_app_changed_cb (GObject *gobject, GParamSpec *pspec, gpointer user_data)
{
    g_return_if_fail(CDOS_IS_NOTIFICATION_DAEMON(user_data));
    CdosNotificationDaemon *self = CDOS_NOTIFICATION_DAEMON(user_data);

    CdosWindowTracker *tracker;
    tracker = cdos_window_tracker_get_default ();
    CdosApp *focus_app;
    g_object_get (tracker, "focus-app", &focus_app, NULL);

    if (focus_app != NULL) {
        return;
    }
    GList *sources = self->sources;
    while(sources != NULL){
        CdosNotificationDaemonSource *source = sources->data;
        if (source->app == focus_app) {
            //FIXME:不需要常驻后台到通知可以被删除。
            //cdos_source_destroy_non_resident_notifications (CDOS_SOURCE(source));
            return;
        }
        sources = sources->next;
    }
}

static void
cdos_notification_daemon_on_focus_app_changed_cb1 (CdosOverview *overview, CdosNotificationDaemon *self)
{
    cdos_notification_daemon_on_focus_app_changed_cb (NULL, NULL, self);
}
/*Notify一级回调*/
//FIXME:猜测可能是dbus的call操作
static void proxy_sender_cb (GObject *source_obj , GAsyncResult *result, gpointer user_data)
{
    callMethodParams *params = (callMethodParams *)user_data;
    CdosNotificationDaemon *self = params->self;
    guint id = params->id;
    gchar *sender = g_strdup (params->sender);
    GError *call_error = NULL; //g_error_new(notification_daemon_error_quark (), 0, "notification-daemon");
    g_free(user_data);
    GVariant *res;
    guint pid;
    GDBusProxy *proxy = G_DBUS_PROXY (source_obj);

    res = g_dbus_proxy_call_finish (proxy, result, &call_error);
    g_variant_get (res, "(u)", &pid);

    if (res == NULL) {
        g_warning ("couldn't logout using proxy_sender_cb: %s",
                   call_error->message);
    } else {
        g_variant_unref (res);
    }
    if (call_error)
        g_error_free (call_error);
    ndataStruct *ndata = g_list_nth_data (self->notifications, id);

    if (!ndata) {
        return;
    }

    gchar *appName = ndata->app_name;
    CdosSource *source = cdos_notification_daemon_get_source (self, appName,
                                                              pid, ndata,
                                                              sender, NULL);
    /* We only store sender-pid entries for persistent sources.*/
    /* Removing the entries once the source is destroyed*/
    /* would result in the entries associated with transient*/
    /* sources removed once the notification is shown anyway.*/
    /* However, keeping these pairs would mean that we would*/
    /* possibly remove an entry associated with a persistent*/
    /* source when a transient source for the same sender is*/
    /* distroyed.*/

    if (source->isTransient) {
        gboolean result = g_hash_table_replace (self->senderToPid, sender, &pid);
        if (result == FALSE) {
            g_hash_table_remove (self->senderToPid, sender);
            g_warning ("The key of sender is not found!");
        }
        sourceParams *user_data = g_new0(sourceParams, 1);
        user_data->self = self;
        user_data->sender = g_strdup (sender);
        g_signal_connect(source, "destroy", G_CALLBACK (source_destroy_cb), user_data);
    }

    cdos_notification_daemon_notify_for_source (self, CDOS_NOTIFICATION_DAEMON_SOURCE (source), ndata);
}

/*Notify二级回调*/
static void source_destroy_cb (GObject *object, guint reason, gpointer user_data)
{
    sourceParams *params = (sourceParams *)user_data;

    CdosNotificationDaemon *self = params->self;

    gchar *sender = params->sender;

    if (!g_hash_table_remove (self->senderToPid, sender)) {
        g_warning ("The key of sender is not found!");
    }
    g_free(params->sender);
    g_free(params);
}

/*========================= CALLBACK FUNCTION END ==============================*/

/*=============================== GObject ====================================*/
static void cdos_notification_daemon_dispose (GObject *gobject)
{
    CdosNotificationDaemon *self = CDOS_NOTIFICATION_DAEMON(gobject);

    if(self->sources){
        GList *sources = self->sources;
        while(sources != NULL){
            if(sources->data){
                g_object_unref(sources->data);
                sources->data = NULL;
            }
            sources = sources->next;
        }
        g_list_free(self->sources);
    }
    if(self->notifications){
        GList *notifications = self->notifications;
        while(notifications != NULL){
            if(notifications->data != NULL){
                ndataStruct *ndata = notifications->data;
                g_hash_table_unref(ndata->hints);
                g_object_unref(ndata->notification);
                g_list_free(ndata->actions);
            }
            notifications = notifications->next;
        }
        g_list_free(self->notifications);
    }

    g_object_unref (self->bus_proxy);
    g_dbus_node_info_unref(self->notification_node_info);

    G_OBJECT_CLASS(cdos_notification_daemon_parent_class)->dispose (gobject);
}
static void cdos_notification_daemon_finalize (GObject *gobject)
{
    //CdosNotificationDaemon *self = CDOS_NOTIFICATION_DAEMON(gobject);

    /*调用父类方法*/
    G_OBJECT_CLASS(cdos_notification_daemon_parent_class)->finalize (gobject);
}

/*_class_init & _init*/
static void cdos_notification_daemon_class_init (CdosNotificationDaemonClass *klass)
{
    /*继承*/
    GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
    gobject_class->finalize = cdos_notification_daemon_finalize;
    gobject_class->dispose = cdos_notification_daemon_dispose;

    /*可能需要创建信号,例如*/
}

static void cdos_notification_daemon_init (CdosNotificationDaemon *self)
{
    
    GDBusConnection *conn;

    //FIXME 是否可行
    /* =============== exportObject ============== */
    GError *conn_error = NULL;
    conn = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &conn_error);
    if (conn == NULL) {
        /*handle connection error*/
        g_warning ("Failed to open connection to bus in notification Daemon:%s", conn_error->message);
        return;
    }

    self->notification_node_info = g_dbus_node_info_new_for_xml(introspection_xml, &conn_error);
    if (NULL == self->notification_node_info) {
        g_error ("Failed to call g_dbus_node_info_new_for_xml: %s!\n", conn_error->message);
    }
    //g_assert(NULL != self->notification_node_info);

    if (0 == g_bus_own_name(G_BUS_TYPE_SESSION,
                            "org.freedesktop.Notifications",
                            G_BUS_NAME_OWNER_FLAGS_REPLACE | G_BUS_NAME_OWNER_FLAGS_ALLOW_REPLACEMENT,
                            (GBusAcquiredCallback )on_notification_dbus_acquried,
                            NULL,
                            NULL,
                            self,
                            NULL)) {
        g_warning ("Failed to call g_bus_own_name_on_connection!\n");
    }

    /* ============== export END ================ */

    self->sources = NULL;
    self->senderToPid = g_hash_table_new (g_str_hash, NULL); //FIXME
    /*第一个参数表示键类型是string, 第二个参数不确定, 待修改*/
    //self->notifications = g_list_alloc();
    self->notifications = NULL;

    MonkeyPlugin *plugin = monkey_plugin_get ();
    CdosIconDispatcher *icon_dispatcher = cdos_monkey_plugin_get_status_icon_dispatcher (plugin);
    g_signal_connect(icon_dispatcher, "message-icon-added", G_CALLBACK (cdos_notification_daemon_on_tray_icon_added_cb),
                     self);
    g_signal_connect(icon_dispatcher, "message-icon-removed",
                     G_CALLBACK (cdos_notification_daemon_on_tray_icon_removed_cb), self);

    CdosWindowTracker *tracker;
    tracker = cdos_window_tracker_get_default ();
    g_signal_connect(tracker, "notify::focus-app",
                     G_CALLBACK (cdos_notification_daemon_on_focus_app_changed_cb), self);
    /*focus-app是CdosWindowTracker类的一个属性,假设发送者为回调函数self*/
    g_signal_connect(cdos_overview_get (), "hidden",
                     G_CALLBACK (cdos_notification_daemon_on_focus_app_changed_cb1), self);
    if (conn_error)
        g_error_free(conn_error);

}

static ClutterActor *cdos_notification_daemon_icon_for_notification_data (gchar *icon, GHashTable *hints, int size)
{
    /*GError*/
    GError *error = NULL;

    StTextureCache *texture_cache;
    texture_cache = st_texture_cache_get_default ();

    /*
     * If an icon is not specified, we use 'image-data' or 'image-path' hint for an icon
     * and don't show a large image. There are currently many applications that use
     * notify_notification_set_icon_from_pixbuf() from libnotify, which in turn sets
     * the 'image-data' hint. These applications don't typically pass in 'app_icon'
     * argument to Notify() and actually expect the pixbuf to be shown as an icon.
     * So the logic here does the right thing for this case. If both an icon and either
     * one of 'image-data' or 'image-path' are specified, we show both an icon and
     * a large image.
     */

    /*used in compare string*/
    gchar *path_head_7_char = NULL, *uri;
    const gchar *path_begin_f = "file://";

    g_utf8_strncpy (path_head_7_char, (const gchar *) icon, 7);

    /*used in create St*/
    ClutterActor *St;

    /*used in hints*/
    int width, height, row_stride, bits_per_sample, n_channels;
    guchar *data = NULL;
    gboolean has_alpha;
    gsize len;

    /*hints["image-data"]*/
    GValue *latest_image_data_struct =
            (GValue *) g_hash_table_lookup (hints, "image-data");

    /*存放从GValue取出的值*/
    //GArray *latest_image_data_struct_data;

    //latest_image_data_struct_data = (GArray *) g_value_get_boxed (latest_image_data_struct);

    const gchar *image_path_new;
    image_path_new = get_latest_hints_image_path (hints);


    if (icon) {
        if (g_utf8_collate ((const gchar *) path_head_7_char, path_begin_f) == 0) {
            St = st_texture_cache_load_uri_async (texture_cache, (const gchar *) icon, size, size);
        } else if (g_utf8_collate (&icon[0], "/") == 0) {
            uri = g_filename_to_uri ((const gchar *) icon, NULL, NULL);
            St = st_texture_cache_load_uri_async (texture_cache, (const gchar *) uri, size, size);
        } else {
            St = st_icon_new ();/*type:ClutterActor*/
            st_icon_set_icon_name (ST_ICON(St), (const gchar *) icon);
            st_icon_set_icon_type (ST_ICON(St), ST_ICON_FULLCOLOR);
            st_icon_set_icon_size (ST_ICON(St), (gint) size);
        }

        return St;

    } else if (latest_image_data_struct) {

        /*通过参数地址进行赋值*/
        //FIXME :第二个参数是否正确?
        get_all_version_hints_image_data (latest_image_data_struct,
                                          data, &width, &height, &row_stride,
                                          &has_alpha, &bits_per_sample, &n_channels, &len);


        /*此时, data, height等等都已经被赋新值*/
        St = st_texture_cache_load_from_raw (texture_cache, (const guchar *) data,
                                             len, has_alpha, width, height,
                                             row_stride, size, &error);

        return St;

    } else if (image_path_new) {

        uri = g_filename_to_uri (image_path_new, NULL, NULL);
        St = st_texture_cache_load_uri_async (texture_cache, uri, size, size);
        return St;

    } else {

        gchar *stockIcon = NULL;
        gint hints_urgency = get_hints_urgency (hints);

        switch (hints_urgency) {
            case URGENCY_LOW :
                break;
            case URGENCY_NORMAL:
                stockIcon = "gtk-dialog-info";
                break;
            case URGENCY_CRITICAL :
                stockIcon = "gtk-dialog-error";
                break;
            default:
                g_warning ("\nhints_urgency not correct!\n");
        }

        St = st_icon_new ();

        st_icon_set_icon_name (ST_ICON(St), (const gchar *) stockIcon);
        st_icon_set_icon_type (ST_ICON(St), ST_ICON_FULLCOLOR);
        st_icon_set_icon_size (ST_ICON(St), (gint) size);

        return St;
    }
}

static CdosNotificationDaemonSource *
cdos_notification_daemon_lookup_source (CdosNotificationDaemon *self, gchar *title,
                                                                      guint pid, CdosTrayIcon *trayIcon,
                                                                      gboolean trayIconStatus)
{
    GList *sources = self->sources;
    while(sources != NULL){
        CdosNotificationDaemonSource *source = sources->data;
        guint source_pid = cdos_notification_daemon_source_get_pid (source);
        CdosTrayIcon *source_tray_icon = cdos_notification_daemon_source_get_trayicon (source);
        if (trayIconStatus == FALSE) {
            if (source_pid == pid &&
                (g_utf8_collate (source->initialTitle, title) == 0 || source_tray_icon == trayIcon)) {
                return source;
            }
        } else {
            if (source_pid == pid && (g_utf8_collate (source->initialTitle, title) == 0 || source_tray_icon)) {
                return source;
            }
        }
        sources = sources->next;
    }
    return NULL;
}

/*
 *Returns the source associated with ndata.notification if it is set.
 *Otherwise, returns the source associated with the title and pid if
 *such source is stored in self._sources and the notification is not
 *transient. If the existing or requested source is associated with
 *a tray icon and passed in pid matches a pid of an existing source,
 *the title match is ignored to enable representing a tray icon and
 *notifications from the same application with a single source.
 *
 *If no existing source is found, a new source is created as long as
 *pid is provided.
 *
 *Either a pid or ndata.notification is needed to retrieve or
 *create a source.
 */

static CdosSource *
cdos_notification_daemon_get_source (CdosNotificationDaemon *self, gchar *title,
                                                 guint pid, ndataStruct *ndata, gchar *sender,
                                                 CdosTrayIcon *icon)
{
    /*
     * FIXME:
     * 这个函数既可能返回父类也可能返回子类, 强转的时候要慎重!
     * 返回父类的情况调用情况是 (notification && notification.ndata)
     */

    if (NULL == ndata) {
        return NULL;
    }
    CdosNotification *notification = ndata->notification;

    if (!pid && !(ndata && notification)) {
        return NULL;
    }

    /*
     * We use notification's source for the notifications we still have
     * around that are getting replaced because we don't keep sources
     * for transient notifications in self._sources, but we still want
     * the notification associated with them to get replaced correctly.
     */
    if (ndata && notification) {

        CdosSource *source = notification->source;
        return source;
    }


    gboolean is_for_transient_notification = (ndata && get_hints_transient (ndata->hints) == TRUE);
    /*
     * We don't want to override a persistent notification
     * with a transient one from the same sender, so we
     * always create a new source object for new transient notifications
     * and never add it to self._sources .
     */

    if (!is_for_transient_notification) {
        CdosNotificationDaemonSource *source = cdos_notification_daemon_lookup_source (self, title, pid, icon, FALSE);
        if (source) {
            /*调用CdosNotificationDaemonSource的父类CdosSource中的方法*/
            cdos_source_set_title (CDOS_SOURCE(source), g_strdup(title));

            return CDOS_SOURCE (source);
        }
    }

    /*创建新实例*/
    CdosNotificationDaemonSource *source = cdos_notification_daemon_source_new (title, pid, sender, icon);
    /*调用CdosNotificationDaemonSource的父类CdosSource中的方法*/
    cdos_source_set_transient (CDOS_SOURCE(source), is_for_transient_notification);

    if (!is_for_transient_notification) {
        self->sources = g_list_append (self->sources, source);
        g_signal_connect(source, "destory", G_CALLBACK (splice_source_cb), self);
    }

    /*Main.messageTray, 待替换*/
    MonkeyPlugin *plugin = monkey_plugin_get ();
    CdosMessageTray *message_tray = cdos_monkey_plugin_get_message_tray (plugin);

    if (message_tray) {
        cdos_message_tray_add (message_tray, CDOS_SOURCE(source));
    }

    return CDOS_SOURCE (source);
}

/**
 * cdos_notification_daemon_notify:
 * @self: A #CdosNotificationDaemon
 * @appName: 应用名
 * @replaceId: 待替换元素的位置
 * @icon: 图标
 * @summary: 消息内容
 * @body: 消息主体
 * @actions: notification自定义的按钮动作
 * @hints: notification DBus相关数据
 * @timeout: 过期时间
 * @out_id: 返回结果
 * @sender: 发送者(C语言中不好获取,所以在此当参数传过来使用).
 *
 * notification的守护进程,负责转发通知.
 *
 * Returns: %TRUE 表示工作正常.
 */
gboolean
cdos_notification_daemon_notify (CdosNotificationDaemon *self, gchar *appName, guint replaceId,
                                 gchar *icon, gchar *summary, gchar *body, GList *actions, GHashTable *hints,
                                 guint timeout, guint* out_id, const gchar *sender)
{
    
    //GList *rewriteRules;
    guint id;
    gchar *rewrite_rules_key = "XChat";
    gboolean result = TRUE;

    if (g_utf8_collate (rewrite_rules_key, appName) == 0) {

        //如果应用名称是 XChat
        // rewriteRules = create_new_rewrite_rules ();

        // while (rewriteRules) {
        //     RuleXChatNode *rule = rewriteRules->data;
        //     GMatchInfo *match_info;
        //     GRegex *regex;
        //     regex = g_regex_new (rule->pattern, 0, 0, NULL);

        //     g_regex_match (regex, summary, 0, &match_info);

        //     while (g_match_info_matches (match_info)) {
        //         summary = g_regex_replace (regex, (const gchar *)summary,
        //                                    -1, 0, rule->replacement, 0, NULL);
        //     }

        //     rewriteRules = rewriteRules->next;
        // }
    }

    /* 说明:misc/params.js
     * hints = Params.parse(hints, { urgency: Urgency.NORMAL }, true);
     * 第一个参数为假,则置空;
     * 第二个参数的key判断是否在hints里,如果不在就拷贝进去
     * 如果在就什么都不做
     */

    if (hints) {
        if (!g_hash_table_contains (hints, "urgency")) {
            guchar u = URGENCY_NORMAL;
            result = g_hash_table_replace (hints, "urgency", GSIZE_TO_POINTER(u));
        }

        /*
         * Be compatible with the various hints for image data and image path
         * 'image-data' and 'image-path' are the latest name of these hints, introduced in 1.2
         */

        const gchar *hints_image_path_old = NULL;
        const gchar *hints_image_path_new = NULL;
        if (g_hash_table_contains (hints, "image_path")) {
            hints_image_path_old = get_old_hints_image_path (hints);
        }

        if (g_hash_table_contains (hints, "image-path")) {
            hints_image_path_new = get_latest_hints_image_path (hints);
        }

        if (!g_hash_table_contains (hints, "image-path") && g_hash_table_contains (hints, "image_path")) {
            result = g_hash_table_replace (hints, "image-path", (gchar *)hints_image_path_old);
        }
        //FIXME
        if (!g_hash_table_contains (hints, "image-data")) {
            if (g_hash_table_contains (hints, "image_data")) {
                /*Version 1.1 of the spec*/
                GValue *hints_image_data_struct_old = (GValue *) g_hash_table_lookup (hints, "image_data");
                //GArray *hints_image_data_old = (GArray *) g_value_get_boxed (hints_image_data_struct_old);
                //result = g_hash_table_replace (hints, "image-data", hints_image_data_old->data);
                result = g_hash_table_replace (hints, "image-data", hints_image_data_struct_old);
            } else if (g_hash_table_contains (hints, "icon_data") && !hints_image_path_new) {
                /* early versions of the spec; 'icon_data' should only be used if 'image-path' is not available */
                GValue *hints_icon_data_struct_early = (GValue *) g_hash_table_lookup (hints, "icon_data");
                //GArray *hints_image_data_early = (GArray *) g_hash_table_lookup (hints, hints_icon_data_struct_early);
                //result = g_hash_table_replace (hints, "image-data", hints_image_data_early->data);
                result = g_hash_table_replace (hints, "image-data", hints_icon_data_struct_early);
            }
        }
    }
    
    if (!result) {
        /*handle error*/
        g_warning ("%s:%d occur error.",  __FUNCTION__, __LINE__);
    }

    ndataStruct *ndata = g_new0(ndataStruct, 1);
    ndata->app_name = appName;
    ndata->icon = icon;
    ndata->summary = summary;
    ndata->body = body;
    ndata->actions = actions;
    ndata->hints = hints;
    ndata->timeout = timeout;

    ndataStruct *replace_id_data = g_list_nth_data (self->notifications, replaceId);
    if (replaceId != 0 && replace_id_data) {
        id = replaceId;
        ndata->id = id;
        ndata->notification = replace_id_data->notification;
    } else {
        replaceId = 0;
        id = nextNotificationId;
        nextNotificationId++;
        ndata->id = id;
    }

    //CdosSource* source = CDOS_SOURCE(cdos_notification_daemon_source_new(ndata->summary, getpid(), ndata->app_name, NULL));
    CdosSource *source = cdos_notification_daemon_get_source (self, appName, (guint)getpid(), ndata, (gchar *)sender, NULL);

    if (NULL != ndata->notification) {
        //g_object_unref(ndata->notification);
    }

    Params* params = g_new0(Params, 1);
    if (NULL != ndata->icon) {
        params->icon = ST_ICON(st_icon_new());
        st_icon_set_icon_name (ST_ICON(params->icon), g_strdup (ndata->icon));
        st_icon_set_icon_type (ST_ICON(params->icon), ST_ICON_FULLCOLOR);
        st_icon_set_icon_size (ST_ICON(params->icon), 36);
    }
    params->bannerMarkup = TRUE;
    ndata->notification = cdos_notification_new(source, ndata->summary, ndata->body, params);
    cdos_notification_set_transient(ndata->notification, get_hints_transient (hints));
    cdos_notification_set_urgency(ndata->notification, get_hints_urgency(hints));

    CdosMessageTray* messageTray = cdos_monkey_plugin_get_message_tray(monkey_plugin_get()); //cdos_message_tray_new();
    cdos_message_tray_add(messageTray, source);

    //g_hash_table_destroy (hints);
    /*下面两句替换self._notifications[id]=ndata;*/
    /*定位*/
    GList *number_id_element = g_list_nth (self->notifications, id);
    
    /*直接更新GList的Element的data?*/
    if (NULL != number_id_element) {
        number_id_element->data = ndata;
    } else {
        self->notifications = g_list_insert(self->notifications, ndata, id);
    }
    g_free(params);
    /*DBus.getCurrentMessageContext()不清楚怎么生成的*/
    //FIXME
//    GDBusMessage *message = g_dbus_message_new_signal ("/org/freedesktop/Notification", "org.freedesktop.Notification", "destroy");
//
//    gchar *sender = (gchar *) g_dbus_message_get_sender (message); /*const gchar -> gchar*/

//    gint pid = get_sender_to_pid (self->senderToPid, sender);
    
//    CdosSource *source = cdos_notification_daemon_get_source (self,
//                                                 appName, (guint) pid,
//                                                 ndata, sender, NULL);
    CdosNotification *ndata_notification = ndata->notification;

    if (source) {
        cdos_source_notify(source, ndata_notification);
        *out_id = id;
        
        return TRUE;
    }

    if (replaceId) {
        /*
         * There's already a pending call to GetConnectionUnixProcessID,
         * which will see the new notification data when it finishes,
         * so we don't have to do anything.
         */
        *out_id = id;
        
        return TRUE;
    }

    /******************************************************/
    /*this._busProxy.GetConnectionUnixProcessIDRemote不会写*/
    //FIXME: 猜测此处可能是call_method
    /*func:source_destroy_cb*/
    /******************************************************/

    callMethodParams *call_methodparams = g_new0(callMethodParams, 1);
    call_methodparams->self = self;
    call_methodparams->id = id;
    call_methodparams->sender = sender;
    g_dbus_proxy_call (self->bus_proxy,
                       "GetConnectionUnixProcessID",
                       g_variant_new ("(s)", sender),
                       G_DBUS_CALL_FLAGS_NONE,
                       G_MAXINT,
                       NULL,
                       (GAsyncReadyCallback) proxy_sender_cb,
                       call_methodparams);
    *out_id = id;
    return TRUE;
}

static void cdos_notification_daemon_notify_for_source (CdosNotificationDaemon *self,
                                                 CdosNotificationDaemonSource *source, ndataStruct *ndata)
{
    ClutterActor *iconActor = cdos_notification_daemon_icon_for_notification_data (ndata->icon,
                                                                                   ndata->hints,
                                                                                   SOURCE_ICON_SIZE);
    guchar *data = NULL;
    gint width, height, rowstride, bits_per_sample, n_channels;
    gsize length;
    gboolean has_alpha;
    GError *error = NULL;

    CdosNotification *notification = ndata->notification;
    if (notification == NULL) {
        Params *new_params = g_new0(Params, 1); /*是否销毁?*/
        new_params->icon = ST_ICON (iconActor);
        new_params->bannerMarkup = TRUE;

        notification = cdos_notification_new (CDOS_SOURCE(source), ndata->summary,
                                              ndata->body, new_params);

        ndata->notification = notification;

        notificationParams *destroyParams = g_new0(notificationParams, 1);
        destroyParams->ndata_id = ndata->id;
        destroyParams->self = self;

        notificationParams *action_invoked_params = g_memdup(destroyParams, 1);

        g_signal_connect(notification, "destroy", G_CALLBACK (notification_destroy_cb), destroyParams);
        g_signal_connect(notification, "action-invoked", G_CALLBACK (notification_action_invoked_cb), action_invoked_params);
        g_free(new_params);
    } else {
        Params *update_params = g_new0(Params, 1);;
        update_params->icon = ST_ICON(iconActor);
        update_params->bannerMarkup = TRUE;
        update_params->clear = TRUE;

        cdos_notification_update (notification, ndata->summary, ndata->body, update_params);
        g_object_unref(update_params->icon);
        g_free(update_params);
    }

    /*We only display a large image if an icon is also specified.*/


    GValue *new_image_data_struct = (GValue *) g_hash_table_lookup (ndata->hints, "image-data");
    const gchar *image_path_new = get_latest_hints_image_path (ndata->hints);

    if (ndata->icon && (new_image_data_struct || image_path_new)) {
        ClutterActor *image = NULL;
        if (new_image_data_struct != NULL) {
            get_all_version_hints_image_data (new_image_data_struct, data, &width, &height, &rowstride, &has_alpha,
                                              &bits_per_sample, &n_channels, &length);

            image = st_texture_cache_load_from_raw (st_texture_cache_get_default (), data, length, has_alpha, width,
                                                    height, rowstride, IMAGE_SIZE, &error);
        } else if (image_path_new != NULL) {
            image = st_texture_cache_load_uri_async (st_texture_cache_get_default (),
                                                     g_filename_to_uri (image_path_new, NULL, NULL), IMAGE_SIZE,
                                                     IMAGE_SIZE);
        }
        cdos_notification_set_image (notification, image);

    } else {
        cdos_notification_unset_image (notification);
    }

    if (ndata->actions) {
        /*GList:actions不为空*/
        gboolean actions_icon_value = get_hints_action_icons (ndata->hints);
        cdos_notification_set_use_action_icons (notification, (actions_icon_value == TRUE));

        GList *actions = ndata->actions;
        while(actions != NULL) {
            gchar * actions_i = actions->data;
            gchar * actions_i_add_1 = actions->next->data;
            if (g_utf8_collate(actions_i, "default") == 0) {
                notificationParams *clicked_params = g_new0(notificationParams, 1);
                clicked_params->self = self;
                clicked_params->ndata_id = ndata->id;
                g_signal_connect(notification, "clicked", G_CALLBACK(callback_notification_clicked), clicked_params);
            } else {
                cdos_notification_add_button(notification, actions_i, actions_i_add_1);
            }
            if(actions->next != NULL){
                actions = actions->next->next;
            }else{
                actions = NULL;
            }
        }
    }
    gint hints_urgency = get_hints_urgency (ndata->hints);

    switch (hints_urgency) {
        case URGENCY_LOW:
            cdos_notification_set_urgency (notification, LOW);
            break;
        case URGENCY_NORMAL:
            cdos_notification_set_urgency (notification, NORMAL);
            break;
        case URGENCY_CRITICAL:
            cdos_notification_set_urgency (notification, CRITICAL);
            break;
    }

    gboolean hints_resident = get_hints_resident (ndata->hints);
    cdos_notification_set_resident (notification, (hints_resident == TRUE));

    gboolean hints_transient = get_hints_transient (ndata->hints);
    cdos_notification_set_transient (notification, (hints_transient == TRUE));

    ClutterActor *sourceIconActor = source->useNotificationIcon ? cdos_notification_daemon_icon_for_notification_data (
            ndata->icon, ndata->hints, SOURCE_ICON_SIZE) : NULL;
    cdos_notification_daemon_source_process_notification (source, notification, CDOS_TRAY_ICON(sourceIconActor));

    g_error_free(error);
}

/**
 * cdos_notification_daemon_close_notification:
 * @self: A #CdosNotificationDaemon
 * @reason: 关闭原因 
 *
 * 关闭守护进程，并说明原因。
 *
 * Returns: 正常调用，返回 %TURE
 */
gboolean cdos_notification_daemon_close_notification (CdosNotificationDaemon *self, guint id)
{
    ndataStruct *ndata = g_list_nth_data (self->notifications, id);
    if (ndata) {
        if (ndata->notification) {
            cdos_notification_destroy (ndata->notification, SOURCE_CLOSED);
        }
        self->notifications = g_list_remove (self->notifications, ndata);
        g_hash_table_unref(ndata->hints);
    }
    return TRUE;
}

/**
 * cdos_notification_daemon_get_capabilities:
 * @self: A #CdosNotificationDaemon
 * @vBuilder: 能被dbus读取的数据结构 
 *
 * 被dbus使用到，获取某些属性。
 *
 * Returns: %TRUE.
 */
gboolean cdos_notification_daemon_get_capabilities(CdosNotificationDaemon* self, GVariantBuilder* vBuilder)
{
    
//    static gchar* _caps[7];
//    _caps[0] = g_strdup("action");
//    _caps[1] = g_strdup("action-icons");
//    _caps[2] = g_strdup("body");
//    _caps[3] = g_strdup("body-markup");
//    _caps[4] = g_strdup("icon-static");
//    _caps[5] = g_strdup("persistence");
//    _caps[6] = NULL;
//    *caps = _caps;
    g_variant_builder_add(vBuilder, "s", "action");
    g_variant_builder_add(vBuilder, "s", "action-icons");
    g_variant_builder_add(vBuilder, "s", "body");
    g_variant_builder_add(vBuilder, "s", "body-markup");
    g_variant_builder_add(vBuilder, "s", "icon-static");
    g_variant_builder_add(vBuilder, "s", "persistence");
    return TRUE;
}

/**
 * cdos_notification_daemon_get_server_information:
 * @self: A #CdosNotificationDaemon
 * @out_name: 桌面名称
 * @out_vendor: GNOME
 * @out_version: 1.0 
 * @out_sepc_ver: 0.1
 *
 * 获取系统服务信息
 *
 * Returns: 正常输出，返回%TRUE
 */
gboolean cdos_notification_daemon_get_server_information (CdosNotificationDaemon *self, gchar **out_name, gchar **out_vendor, gchar **out_version, gchar **out_spec_ver)
{
    /*参数一和参数三固定*/
    
    *out_name = "cdos";
    *out_vendor = "GNOME";
    *out_version = "1.0";
    *out_spec_ver = "0.1";
    return TRUE;
}


static void cdos_notification_daemon_emit_notification_closed (CdosNotificationDaemon *self, guint id, guint reason)
{
    GError *error = NULL;
    GDBusConnection *connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &error);
    if (connection == NULL) {
        g_warning ("Failed to open connection to bus in notification Daemon:%s", error->message);
        g_object_unref (connection);
        if (error)
            g_error_free(error);
        return;
    }

    g_dbus_connection_emit_signal (connection, NULL, "/org/freedesktop/Notifications",
                                   "org.freedesktop.Notifications", "NotificationClosed",
                                   g_variant_new ("(uu)", id, reason), NULL);
    g_clear_error (&error);
}

static void cdos_notification_daemon_emit_action_invoked (CdosNotificationDaemon *self, guint id, gchar *action)
{
    GError *error = NULL; //g_error_new(notification_daemon_error_quark (), 0, "notification-daemon");

    GDBusConnection *connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &error);
    if (connection == NULL) {
        g_warning ("Failed to open connection to bus in notification Daemon:%s", error->message);
        g_object_unref (connection);
        if (error)
            g_error_free(error);
        return;
    }

    if (!g_dbus_connection_emit_signal (connection,
                                        NULL,
                                        "/org/freedesktop/Notifications",
                                        "org.freedesktop.Notifications",
                                        "ActionInvoked",
                                        g_variant_new ("(us)", id, action),
                                        &error)) {
        /*return TURE unless error, handle error*/
        g_warning ("g_dbus_connection_emit_signal wrong");
    }
    g_error_free(error);
}

static void on_notification_dbus_acquried(GObject *obj, const gchar *name, gpointer user_data) {
    
    GDBusConnection *connection = G_DBUS_CONNECTION (obj);
    GError *conn_error = NULL;//g_error_new(notification_daemon_error_quark (), 0, "notification-daemon");
    //GDBusInterfaceSkeleton *interface = G_DBUS_INTERFACE_SKELETON (org_freedesktop_notifications_skeleton_new());
    CdosNotificationDaemon* self = CDOS_NOTIFICATION_DAEMON(user_data);

    if (!g_dbus_connection_register_object(connection,
                                           "/org/freedesktop/Notifications",
                                           self->notification_node_info->interfaces[0],
                                           &interface_vtable,
                                           self,
                                           NULL,
                                           &conn_error)) {
        /*handle export error */
        g_warning ("g_dbus_connection_register_object error: %s", conn_error->message);
    }

    //gboolean res = g_dbus_interface_skeleton_export (interface, connection, "org/freedesktop/Notifications", &conn_error);
    generate_new_proxy (self);
    g_clear_error(&conn_error);
    
}

static void handle_method_call(GDBusConnection *connection G_GNUC_UNUSED,
                               const gchar *sender,
                               const gchar *object_path G_GNUC_UNUSED,
                               const gchar *interface_name,
                               const gchar *method_name,
                               GVariant *parameters,
                               GDBusMethodInvocation *invocation,
                               gpointer user_data) {
    g_print("%s -----> enter, sender(%s) obj_path(%s) method(%s), interface_name(%s)!\n", __func__, sender, object_path, method_name, interface_name);
    if (0 == g_strcmp0(method_name, "GetServerInformation")) {
       gchar* out_name, *out_vendor, *out_version, *out_spec_ver;
       if (cdos_notification_daemon_get_server_information(CDOS_NOTIFICATION_DAEMON(user_data),
                                                           &out_name,
                                                           &out_vendor,
                                                           &out_version,
                                                           &out_spec_ver)) {
          g_print("%s -----> name(%s), vendor(%s), version(%s), spec_ver(%s)!\n", __func__, out_name, out_vendor, out_version, out_spec_ver);
          g_dbus_method_invocation_return_value(invocation, g_variant_new("(ssss)", out_name, out_vendor, out_version, out_spec_ver));
           //g_free (out_name);
           //g_free (out_vendor);
           //g_free (out_version);
           //g_free (out_spec_ver);
       }

    } else if (0 == g_strcmp0(method_name, "GetCapabilities")) {
        GVariantBuilder * caps_builder = g_variant_builder_new(G_VARIANT_TYPE("as"));
        if (cdos_notification_daemon_get_capabilities(CDOS_NOTIFICATION_DAEMON(user_data), caps_builder)) {
           g_dbus_method_invocation_return_value(invocation, g_variant_new("(as)", caps_builder));
        }
        g_variant_builder_unref (caps_builder);
    } else if (0 == g_strcmp0(method_name, "CloseNotification")) {
        guint sound_id = 0;
        g_variant_get(parameters, "(u)", sound_id);
        cdos_notification_daemon_close_notification (CDOS_NOTIFICATION_DAEMON(user_data), sound_id);
        g_dbus_method_invocation_return_value(invocation, NULL);
    } else if (0 == g_strcmp0(method_name, "Notify")) {
        gchar *appName;
        guint replaceId;
        gchar *icon;
        gchar *summary;
        gchar *body;
        GVariantIter *actions; // 为"as"类型的GVariant
        GVariantIter *hints; // 为"a{sv}"类型的GVariant
        guint timeout;
        guint out_id = 0;
        g_variant_get(parameters, "(&susssasa{sv}i)", &appName, &replaceId, &icon, &summary, &body, &actions, &hints, &timeout);
        g_print("%s -----> appName(%s), replaceId(%u), icon(%s), summary(%s), body(%s), timeout(%u)!\n",
                __func__, appName, replaceId, icon, summary, body, timeout);

        GHashTable* hints_hashtbl = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, NULL);
        GList* actionsList = NULL;
        gchar* key = NULL;
        GVariant* value;

        while(g_variant_iter_loop (actions, "s", &key)) {
            actionsList = g_list_append(actionsList, key);
            cdos_debug ("%s", key);
        }


        while (g_variant_iter_loop(hints, "{sv}", &key, &value)) {
            const gchar *value_type = g_variant_get_type_string (value);
            gulong val = 0;
            switch (*value_type)
            {
#define VAL_TYPE_CASE(format,type) case format:{\
    type v;\
    g_variant_get(value, value_type, &v);\
    val = (gulong)v;\
    g_print("key(%s), value type(%c), value: 0x%lx\n", key, *value_type, val); \
    break;}
                VAL_TYPE_CASE('y',guchar)
                VAL_TYPE_CASE('b',gboolean)
                VAL_TYPE_CASE('n',gint16)
                VAL_TYPE_CASE('q',guint16)
                VAL_TYPE_CASE('i',gint32)
                VAL_TYPE_CASE('u',guint32)
                VAL_TYPE_CASE('x',gint64)
                VAL_TYPE_CASE('t',guint64)
                VAL_TYPE_CASE('d',gdouble)
#undef VAL_TYPE_CASE
#define VAL_TYPE_CASE(format) case format:{\
    gchar *v;\
    g_variant_get(value, value_type, &v);\
    val = (gulong)v;\
    g_print("key(%s), value type(%c), value: %s\n", key, *value_type, v); \
    break;}
                VAL_TYPE_CASE('s')
                VAL_TYPE_CASE('o')
                VAL_TYPE_CASE('g')
#undef VAL_TYPE_CASE
            }
            g_hash_table_insert (hints_hashtbl, g_strdup(key), (gpointer)val);
        }

        if (cdos_notification_daemon_notify(CDOS_NOTIFICATION_DAEMON(user_data),
                                            appName, replaceId, icon,
                                            summary, body, actionsList,
                                            hints_hashtbl, timeout,
                                            &out_id, sender)) {
            g_dbus_method_invocation_return_value(invocation, g_variant_new("(u)", out_id));
        }
    }
}

/*====================== GObject END =========================*/
