#ifndef __CDOS_NOTIFICATION_H__
#define __CDOS_NOTIFICATION_H__

#include "cdos-url-highlighter.h"
#include <clutter/clutter.h>
#include <cdos-generic-container.h>
#include "st.h"

GType cdos_notification_get_type (void);

#define CDOS_TYPE_NOTIFICATION        (cdos_notification_get_type ())
#define CDOS_NOTIFICATION(obj)              (G_TYPE_CHECK_INSTANCE_CAST ((obj), CDOS_TYPE_NOTIFICATION, CdosNotification))
#define CDOS_IS_NOTIFICATION(obj)           (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CDOS_TYPE_NOTIFICATION))
#define CDOS_NOTIFICATION_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), CDOS_TYPE_NOTIFICATION, CdosNotificationClass))
#define CDOS_IS_NOTIFICATION_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), CDOS_TYPE_NOTIFICATION))
#define CDOS_NOTIFICATION_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), CDOS_TYPE_NOTIFICATION, CdosNotificationClass))

typedef struct _CdosNotification        CdosNotification;
typedef struct _CdosNotificationClass          CdosNotificationClass;

typedef struct _Params    Params;
//因為 CdosSource和CdosNotification结构体中互相依赖，因此在此作前置宣告解决头文件互相依赖问题
typedef struct _CdosSource    CdosSource;

struct _CdosNotificationClass
{
    GObjectClass parent_class;

};


struct _CdosNotification
{
    GObject parent_instance;

    //调用时传入的参数
    CdosSource *source;
    gchar *title;
    gchar *banner;
    Params *params;

    guint urgency;
    gboolean resident;
    gboolean isTransient;
    gboolean expanded;
    gboolean destroyed;
    gboolean useActionIcons;
    gboolean customContent;
    gchar *bannerBodyText;
    gboolean bannerBodyMarkup;
    gboolean titleFitsInBannerMode;
    guint titleDirection;
    gdouble spacing;

    StBin *imageBin;
    GDateTime* timestamp;
    gboolean inNotificationBin;

    StWidget *actor;
    CdosNotification *actor_delegate; //js中有动态建立的delegate和parent_container成员
    StWidget *actor_parent_container; //同上

    StWidget *table;
    StFocusManager *buttonFocusManager;

    ClutterActor *bannerBox;

    StLabel *timeLabel;

     CdosUrlHighlighter *titleUrlHighlighter;
    StLabel *titleLabel;

    CdosUrlHighlighter *bannerUrlHighlighter;
    StLabel *bannerLabel;

    StIcon *icon;
    StWidget *scrollArea;
    StBoxLayout *contentArea;
    StBoxLayout *actionArea;   //类型不确定
    StBoxLayout *buttonBox;

    guint destroyedReason;
};

/**
 * MessageTrayState:
 * @HIDDEN: 表示已经隐藏
 * @SHOWING: 动画过程中,正在显示
 * @SHOWN: 已经显示
 * @HIDING: 动画过程中,正在隐藏
 * 
 * CdosMessageTray 的状态枚举变量
 */
typedef enum _MessageTrayState{
    HIDDEN = 0,
    SHOWING,
    SHOWN,
    HIDING
} MessageTrayState;

/**
 * NotificationDestroyedReason: 
 * @NOREASON: 对应js里的undefine
 * @EXPIRED: 表示使用者还没有与通知交互
 * @DISMISSED: 表示通知被使用者关闭
 * @SOURCE_CLOSED: 表示被其他相关的source要求关闭
 * 
 * CdosNotification 关闭原因的枚举变量
 */
typedef enum _NotificationDestroyedReason{
    NOREASON = 0,
    EXPIRED,
    DISMISSED,
    SOURCE_CLOSED
} NotificationDestroyedReason;

/**
 * Urgency:
 * @LOW: 紧急程度低
 * @NORMAL: 紧急程度一般
 * @HIGH: 紧急程度高
 * @CRITICAL: 崩溃
 *
 * 表示从notification daemon接收到通知的紧急程度的枚举变量
 */
typedef enum _Urgency{
    LOW = 0,
    NORMAL,
    //HIGH,
    CRITICAL
} Urgency;


/**
 * #Params
 * @customContent: 自定义内容
 * @body: 消息体
 * @icon: 图标
 * @titleMarkup: 标题是否允许标记
 * @bannerMarkup: 消息体是可以标记
 * @bodyMarkup: C版本里与bannerMarkup相同
 * @clear: 是否可清除,默认 %FALSE
 *
 */
struct _Params {
    gboolean customContent;
    gchar *body;   
    StIcon *icon;
    gboolean titleMarkup;
    gboolean bannerMarkup;
    gboolean bodyMarkup;
    gboolean clear;
};

typedef struct _CStTableChildProps CStTableChildProps;

/* CdosNotification 公有函数 */

void cdos_notification_update ( CdosNotification *self, const gchar *title, const gchar *banner, const Params *params );

void cdos_notification_set_icon_visible ( CdosNotification *self, gboolean visible );

void cdos_notification_add_actor (CdosNotification *self, ClutterActor *actor, const char *style );

StLabel *cdos_notification_add_body ( CdosNotification *self, const gchar *text, gboolean markup, const gchar *style);

void cdos_notification_scroll_to ( CdosNotification *self , StSide side );

void cdos_notification_set_action_area ( CdosNotification *self , StBoxLayout *actor, CStTableChildProps *props);

void cdos_notification_set_image ( CdosNotification *self , ClutterActor *image );

void cdos_notification_unset_image ( CdosNotification *self );

void cdos_notification_add_button ( CdosNotification *self, gchar *id, gchar *label );

void cdos_notification_set_urgency ( CdosNotification *self, guint urgency );

void cdos_notification_set_resident ( CdosNotification *self, gboolean resident );

void cdos_notification_set_transient ( CdosNotification *self, gboolean isTransient );

void cdos_notification_set_use_action_icons ( CdosNotification *self, gboolean useIcons );

void cdos_notification_expand ( CdosNotification *self , gboolean animate );

void cdos_notification_collapseCompleted ( CdosNotification *self );

void cdos_notification_destroy ( CdosNotification *self, guint reason );

CdosNotification *cdos_notification_new ( CdosSource *source, gchar *title, gchar *banner, Params *params );

#endif
