#define CLUTTER_DISABLE_DEPRECATION_WARNINGS

#include "cdos-overview-thumbnails-box.h"
#include "cdos-overview-workspace-thumbnail.h"
#include "cdos-monkey-plugin.h"
#include "cdos-window-manager.h"
#include "cdos-layout-manager.h"
#include "cdos-window-overlay.h"
#include "cdos-base-util.h"
#include "cdos-workspace.h"

#include "cdos-global.h"

#include <meta/window.h>
#include <meta/workspace.h>

#include <math.h>


/**
 * SECTION:cdos-overview-thumbnails-box
 * @Title: CdosOverviewThumbnailsBox
 * @Short_description: 存放工作区缩略图的容器
 * 
 * CdosOverviewThumbnailsBox 用于工作区的展示方式，目前工作区默认按横排显示，
 * 通过修改 org.cdos.workspace-overview-view-as-grid 的值，可以网格显示工作区缩略图
 * 成员 thumnbnails 保存系统中所有的 CdosOverviewWorkspaceThumbnail 对象，默认有四个工作区，也就是四个工作区缩略图
 */
typedef struct {

    gpointer param1;
    gpointer param2;

} Params_overview;

enum {
    ALLOCATED,
    LAST_SIGNAL
};

static gint cdos_overview_thumbnails_box_signals [LAST_SIGNAL] = { 0 };

enum {
    PROP_0,
    PROP_SCALE
};

// The maximum size of a thumbnail is 1/8 the width and height of the screen
#define MAX_THUMBNAIL_SCALE 0.9

#define POINTER_LEAVE_MILLISECONDS_GRACE 500
#define POINTER_ENTER_MILLISECONDS_GRACE 150

#define RESCALE_ANIMATION_TIME 200

enum {    //DND.DragMotionResult
    DRAG_MOTION_RESULT_NO_DROP,
    DRAG_MOTION_RESULT_COPY_DROP,
    DRAG_MOTION_RESULT_MOVE_DROP,
    DRAG_MOTION_RESULT_CONTINUE
};

struct _CdosOverviewThumbnailsBox {
    CdosGenericContainer parent;

    GPtrArray *thumbnails;        //类型為CdosOverviewWorkspaceThumbnail
    ClutterActorBox box;
    MetaRectangle porthole;
    CdosOverviewWorkspaceThumbnail *lastActiveWorkspace;

    CdosOverviewWorkspaceThumbnail *lastHovered;
    
    gint kbThumbnailIndex;
    guint allocId;

    guint switchWorkspaceNotifyId;
    guint workspaceAddedId;
    guint workspaceRemovedId;
};

struct _CdosOverviewThumbnailsBoxClass {
    CdosGenericContainerClass parent_class;
};

G_DEFINE_TYPE  ( CdosOverviewThumbnailsBox, cdos_overview_thumbnails_box, CDOS_TYPE_GENERIC_CONTAINER);

static void
getPreferredWidth (CdosOverviewThumbnailsBox *self, gfloat *forHeight, CdosGenericContainerAllocation *alloc)
{
    if (self->thumbnails == NULL)
        return;

    gdouble spacing;
    spacing = st_theme_node_get_length (st_widget_get_theme_node (ST_WIDGET (self)), "spacing" );

    guint nWorkspaces;
    MetaScreen *screen = cdos_global_get_screen (cdos_global_get());
    nWorkspaces = meta_screen_get_n_workspaces (screen);
    gfloat totalSpacing;
    totalSpacing = (nWorkspaces - 1) * spacing;

    CdosLayoutManager *layout_manager = cdos_layout_manager_get_default ();
    MetaRectangle *rect;
    rect = cdos_layout_manager_get_primary_monitor (layout_manager);

    gfloat width;
    width = rect->width * thumbnail_box_scale;

    alloc->min_size = totalSpacing;
    alloc->natural_size = width * nWorkspaces + totalSpacing;
}

static void
getPreferredHeight (CdosOverviewThumbnailsBox *self, gfloat *forWidth, CdosGenericContainerAllocation *alloc)
{
    if (self->thumbnails == NULL)
        return;

    gdouble spacing = st_theme_node_get_length (st_widget_get_theme_node (ST_WIDGET (self)), "spacing" );

    MetaScreen *screen = cdos_global_get_screen (global);
    guint nWorkspaces;
    nWorkspaces = meta_screen_get_n_workspaces (screen);
    gfloat totalSpacing;
    totalSpacing = (nWorkspaces - 1) * spacing;

    CdosLayoutManager *layout_manager = cdos_layout_manager_get_default ();
    MetaRectangle *rect = cdos_layout_manager_get_primary_monitor (layout_manager);

    gfloat avail;
    avail = *forWidth - totalSpacing;

    gfloat scale;
    scale = (avail / nWorkspaces) / rect->width;
    scale = min (scale, thumbnail_box_scale);

    gfloat height = rect->height * scale;

    alloc->min_size = height;
    alloc->natural_size = height;
}

static void
allocate (CdosOverviewThumbnailsBox *self, ClutterActorBox *box, ClutterAllocationFlags flags)
{
    if (self->thumbnails == NULL) // not visible
        return;

    CdosLayoutManager *layout = cdos_layout_manager_get_default ();
    MetaRectangle *primary = cdos_layout_manager_get_primary_monitor (layout);

    self->box = *box;

    gdouble spacing = st_theme_node_get_length (st_widget_get_theme_node (ST_WIDGET (self)), "spacing" );
    spacing = max (spacing, 5.0);

    guint nWorkspaces = self->thumbnails->len;

    guint nColumns = nWorkspaces;

    gfloat totalSpacingX = (nColumns - 1) * spacing;
    gfloat availX = (self->box.x2 - self->box.x1) - totalSpacingX - spacing * 2 ;
    gfloat availY = (self->box.y2 - self->box.y1) - 2 * spacing;
    //gfloat scaleY = availY / (box->y2 - box->y1);
    //gfloat thumbnailHeight = box->y2 - box->y1 - 2 * spacing; //availY * thumbnail_box_scale;

    gfloat scale = 1.0 / nWorkspaces;
    if (thumbnail_box_scale > scale)
        thumbnail_box_scale = scale;

    gfloat thumbnailWidth  = availX * thumbnail_box_scale;

    gfloat x = 0.f;//spacing + (availX - nColumns * thumbnailHeight) / 2.0;
    gfloat y = (box->y2 - box->y1 - availY) / 2.0;
    gfloat neededX = (thumbnailWidth * nColumns) + totalSpacingX + (spacing * 2);
    gfloat extraSpaceX = (self->box.x2 - self->box.x1) - neededX;
    x = spacing + extraSpaceX / 2.0;
    for (int i = 0; i < nWorkspaces; i++)
    {
        CdosOverviewWorkspaceThumbnail *thumbnail = self->thumbnails->pdata[i];

        ClutterActorBox childBox;
        childBox.x1 = x;
        childBox.x2 = x + primary->width;
        childBox.y1 = y;
        childBox.y2 = y + primary->height;

        clutter_actor_set_scale (CLUTTER_ACTOR (thumbnail), thumbnail_box_scale, thumbnail_box_scale);

        clutter_actor_allocate (CLUTTER_ACTOR (thumbnail), &childBox, flags);
        //y = y + spacing + thumbnailHeight;
        x = x + spacing + thumbnailWidth;
    }

    g_signal_emit_by_name (self, "allocated");
}


//尚未完成 source类型未知
static guint background_handleDragOver (CdosOverviewThumbnailsBox *self, void *source, ClutterActor *actor, gint x, gint y, guint time) {
    /*
       MetaWindow *metaWindow = source->metaWindow;

       if (metaWindow != NULL && !meta_window_is_on_all_workspaces (metaWindow)) 
       return DRAG_MOTION_RESULT_MOVE_DROP;
       else
       return DRAG_MOTION_RESULT_CONTINUE;
*/
    return 0;
}

//尚未完成 source类型未知
static gboolean background_acceptDrop (CdosOverviewThumbnailsBox *self, void *source, ClutterActor *actor, gint x, gint y, guint time) {
/*
    MetaWindow *metaWindow = source->metaWindow;
        
    //这句不懂
    if (background_handleDragOver (self, source, actor, x, y, time) == DRAG_MOTION_RESULT_MOVE_DROP) {
                CdosDraggable *draggable = source._draggable;

                ClutterActor *parent = clutter_actor_get_parent (actor);
        clutter_actor_remove_child (parent, actor);

                draggable._dragOrigParent.add_actor(actor);
        clutter_actor_add_child ();

                clutter_actor_set_opacity (actor, draggable._dragOrigOpacity);

                meta_window_stick (metaWindow);

                return TRUE;
    }
*/    return FALSE;
}


static void onScrollEvent (GObject *gobject, ClutterEvent *event, gpointer user_data) {

        switch (clutter_event_get_scroll_direction (event)) {

            case CLUTTER_SCROLL_UP:
                cdos_wm_action_move_workspace_up ();
                break;

            case CLUTTER_SCROLL_DOWN:
                cdos_wm_action_move_workspace_up ();
                break;
            default:
                break;
        }
}


static void
activateSelectedWorkspace (CdosOverviewThumbnailsBox *self)
{
    CdosOverviewWorkspaceThumbnail *thumbnail;
    thumbnail = self->thumbnails->pdata[self->kbThumbnailIndex];
    cdos_overview_workspace_thumbnail_activate_workspace (thumbnail);
}

static gboolean
selectNextWorkspace (CdosOverviewThumbnailsBox *self, guint symbol)
{
    gint prev_index = self->kbThumbnailIndex;

    MetaScreen *screen = cdos_global_get_screen (global);
    guint n_columns = meta_screen_get_n_workspaces (screen);

    gint next_index = 0;
    switch (symbol)
    {
        case CLUTTER_KEY_Right:
        case CLUTTER_KEY_Down:
            next_index = prev_index + 1;
            if (next_index == n_columns)
                next_index = 0;
            break;
        case CLUTTER_KEY_Left:
        case CLUTTER_KEY_Up:
            next_index = prev_index - 1;
            if (next_index < 0)
                next_index = n_columns - 1;
            break;
        default:
            next_index = -1;
            break;
    }

    cdos_debug ("next index  %d, prev index %d\n", next_index, prev_index);
    if (next_index >= 0) {
        self->kbThumbnailIndex = next_index;
        return TRUE;
    }
    else 
    {
        gint index = symbol - 48 - 1; // convert '1' to index 0, etc

        if (index >= 0 && index < 10) {
            // OK
        }
        else 
        {
            index = symbol - CLUTTER_KP_1; // convert Num-pad '1' to index 0, etc

            if (index < 0 || index > 9) {
                return FALSE; // not handled
            }
        }

        if (index > n_columns - 1) {
            return TRUE; // handled, but out of range
        }

        self->kbThumbnailIndex = index;
        activateSelectedWorkspace (self);

        show_workspace_OSD (screen);

        return TRUE; // handled
    }
}

static void
activeWorkspaceChanged (CdosBaseWM *cdos_base_wm,
                        gint from,
                        gint to,
                        gint direction,
                        CdosOverviewThumbnailsBox *self)
{
    MetaScreen *screen = cdos_global_get_screen (cdos_global_get());

    MetaWorkspace *activeWorkspace = meta_screen_get_active_workspace (screen);
    MetaWorkspace *lastActiveWorkspace = meta_screen_get_workspace_by_index (screen, from);

    for (int i = 0; i < self->thumbnails->len; i++)
    {
        CdosOverviewWorkspaceThumbnail *th = self->thumbnails->pdata[i];
        if (th->metaWorkspace == activeWorkspace)
            th->isActive = TRUE;

        else if (th->metaWorkspace == lastActiveWorkspace)
        {
            th->isActive = FALSE;
            cdos_overview_workspace_thumbnail_shade (th, TRUE);
        }
        gint m = max (to, from);
        if (i >= m)
            break;
        i++;
    }

    self->kbThumbnailIndex = to;
} 

//判断是否為内部事件   
static gboolean
isInternalEvent (CdosOverviewWorkspaceThumbnail *thumbnail, ClutterActor *actor, ClutterEvent *event) {

    ClutterActor *relatedActor = clutter_event_get_related (event);

    return actor == relatedActor || clutter_actor_contains (CLUTTER_ACTOR (thumbnail), relatedActor);
}

static void onThumbnailDragOver (CdosOverviewWorkspaceThumbnail *thumbnail, CdosOverviewThumbnailsBox *self) {

    self->lastHovered = thumbnail;
}

static gboolean
onThumbnailMotionEvent (ClutterActor *actor,
                        ClutterEvent *event,
                        CdosOverviewThumbnailsBox *self)
{
    CdosOverviewWorkspaceThumbnail *thumbnail = CDOS_OVERVIEW_WORKSPACE_THUMBNAIL (actor);

    if (!cdos_pointer_tracker_has_moved (thumbnail->pointerTracker))
        return FALSE;

    if (!thumbnail->hovering) {
        thumbnail->hovering = TRUE;
        self->lastHovered = thumbnail;
        cdos_overview_workspace_thumbnail_highlight (thumbnail);
        for (int i = 0; i < self->thumbnails->len; i++)
        {
            CdosOverviewWorkspaceThumbnail *th = self->thumbnails->pdata[i];
            if (th == thumbnail)
            {
                self->kbThumbnailIndex = i;
                break;
            }
        }
    }
    return TRUE;
}

static gboolean
onThumbnailLeaveEvent (ClutterActor *actor,
                       ClutterEvent *event,
                       CdosOverviewThumbnailsBox *self)
{
    CdosOverviewWorkspaceThumbnail *thumbnail = CDOS_OVERVIEW_WORKSPACE_THUMBNAIL (actor);

    if (!cdos_pointer_tracker_has_moved (thumbnail->pointerTracker))
         return FALSE;

    if (thumbnail->hovering && !isInternalEvent (thumbnail, actor, event)) {
        thumbnail->hovering = FALSE;
        if (thumbnail->isActive == FALSE)
            cdos_overview_workspace_thumbnail_shade (thumbnail, TRUE);
    }
    return FALSE;
}

static void
addThumbnails (CdosOverviewThumbnailsBox *self, gint start, gint count)
{
    MetaScreen *global_screen = cdos_global_get_screen (cdos_global_get());

    gint k;
    for (k = start; k < start + count; k++)
    {
        MetaWorkspace *metaWorkspace = meta_screen_get_workspace_by_index (global_screen, k);
        CdosOverviewWorkspaceThumbnail *thumbnail = cdos_overview_workspace_thumbnail_new (metaWorkspace);

        g_ptr_array_add (self->thumbnails, thumbnail);

        if (metaWorkspace == meta_screen_get_active_workspace (global_screen)) {
            self->lastActiveWorkspace = thumbnail;
            //cdos_overview_workspace_thumbnail_set_active (thumbnail, TRUE);
        }

        clutter_container_add_actor ((ClutterContainer *)self, (ClutterActor *)thumbnail);

        //        g_signal_connect (thumbnail, "drag-over", G_CALLBACK (onThumbnailDragOver), self);

        // We want to ignore spurious events caused by animations
        // (when the contents are moving and not the pointer).
        //thumbnail->pointerTracker = cdos_pointer_tracker_new ();

        g_signal_connect (thumbnail, "motion-event", G_CALLBACK (onThumbnailMotionEvent), self);
        g_signal_connect (thumbnail, "leave-event", G_CALLBACK (onThumbnailLeaveEvent), self);
    }
}

static void
onWorkspaceAdded (CdosBaseWM *cdos_base_wm, gint screen, gint index, CdosOverviewThumbnailsBox *self)
{
    addThumbnails (self, index, 1);
}

static void
onWorkspaceRemoved (CdosBaseWM *wm, gint index, CdosOverviewThumbnailsBox *self)
{
    clutter_actor_destroy (self->thumbnails->pdata[index]);
    g_ptr_array_remove_index (self->thumbnails, index);
}

static void
thumbnails_box_disconnect_signals (CdosOverviewThumbnailsBox *self)
{
    CdosBaseWM *cdos_base_wm = cdos_global_get_window_manager (cdos_global_get ());

    if(self->switchWorkspaceNotifyId){
        g_signal_handler_disconnect (cdos_base_wm, self->switchWorkspaceNotifyId);
        self->switchWorkspaceNotifyId = 0;
    }

    MetaScreen *screen = cdos_global_get_screen (cdos_global_get ());
    if(self->workspaceAddedId){
        g_signal_handler_disconnect (screen, self->workspaceAddedId);
        self->workspaceAddedId = 0;
    }

    if(self->workspaceRemovedId){
        g_signal_handler_disconnect (screen, self->workspaceRemovedId);
        self->workspaceRemovedId = 0;
    }
}

static void
cdos_overview_thumbnails_box_dispose (GObject *object)
{
    CdosOverviewThumbnailsBox *self = CDOS_OVERVIEW_THUMBNAILS_BOX (object);
    thumbnails_box_disconnect_signals (self);

    if(self->thumbnails){
        g_ptr_array_foreach (self->thumbnails, (GFunc)clutter_actor_destroy, NULL);
        g_ptr_array_free (self->thumbnails, TRUE);
        self->thumbnails = NULL;
    }

    G_OBJECT_CLASS (cdos_overview_thumbnails_box_parent_class)->dispose (object);
}

static void
cdos_overview_thumbnails_box_finalize (GObject *object)
{
    G_OBJECT_CLASS (cdos_overview_thumbnails_box_parent_class)->finalize (object);
}

static void
cdos_overview_thumbnails_box_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec  *pspec)
{
    //CdosOverviewThumbnailsBox *self = CDOS_OVERVIEW_THUMBNAILS_BOX (object);
    switch (prop_id)
    {
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
            break;
    }
}

static void
cdos_overview_thumbnails_box_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec  *pspec)
{
    //CdosOverviewThumbnailsBox *self = CDOS_OVERVIEW_THUMBNAILS_BOX (object);
    switch (prop_id)
    {
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
            break;
    }
}

static void
cdos_overview_thumbnails_box_allocate (ClutterActor           *self,
                                  const ClutterActorBox  *box,
                                  ClutterAllocationFlags  flags)
{
  StThemeNode *theme_node;
  ClutterActorBox content_box;

  CLUTTER_ACTOR_CLASS (cdos_overview_thumbnails_box_parent_class)->allocate (self, box, flags);

  theme_node = st_widget_get_theme_node (ST_WIDGET (self));
  st_theme_node_get_content_box (theme_node, box, &content_box);

  allocate ((CdosOverviewThumbnailsBox *)self, &content_box, flags);
}

static void
cdos_overview_thumbnails_box_get_preferred_width (ClutterActor *actor,
                                             gfloat        for_height,
                                             gfloat       *min_width_p,
                                             gfloat       *natural_width_p)
{
  CdosGenericContainerAllocation *alloc = g_slice_new0 (CdosGenericContainerAllocation);
  StThemeNode *theme_node = st_widget_get_theme_node (ST_WIDGET (actor));

  st_theme_node_adjust_for_height (theme_node, &for_height);

  /*
  g_signal_emit (G_OBJECT (actor), cdos_generic_container_signals[GET_PREFERRED_WIDTH], 0,
                 for_height, alloc);
  */
  getPreferredWidth ((CdosOverviewThumbnailsBox *)actor, &for_height, alloc);
  if (min_width_p)
    *min_width_p = alloc->min_size;
  if (natural_width_p)
    *natural_width_p = alloc->natural_size;
  g_slice_free (CdosGenericContainerAllocation, alloc);

  st_theme_node_adjust_preferred_width (theme_node, min_width_p, natural_width_p);
}

static void
cdos_overview_thumbnails_box_get_preferred_height (ClutterActor *actor,
                                              gfloat        for_width,
                                              gfloat       *min_height_p,
                                              gfloat       *natural_height_p)
{
  CdosGenericContainerAllocation *alloc = g_slice_new0 (CdosGenericContainerAllocation);
  StThemeNode *theme_node = st_widget_get_theme_node (ST_WIDGET (actor));

  st_theme_node_adjust_for_width (theme_node, &for_width);

  /*
  g_signal_emit (G_OBJECT (actor), cdos_generic_container_signals[GET_PREFERRED_HEIGHT], 0,
                 for_width, alloc);
  */
  getPreferredHeight ((CdosOverviewThumbnailsBox *)actor, &for_width, alloc);

  if (min_height_p)
    *min_height_p = alloc->min_size;
  if (natural_height_p)
    *natural_height_p = alloc->natural_size;
  g_slice_free (CdosGenericContainerAllocation, alloc);

  st_theme_node_adjust_preferred_height (theme_node, min_height_p, natural_height_p);
}

static void
cdos_overview_thumbnails_box_show (ClutterActor *actor)
{
    CdosOverviewThumbnailsBox *self = (gpointer)actor;
    CdosBaseWM *cdos_base_wm = cdos_global_get_window_manager (cdos_global_get ());
    MetaScreen *global_screen = cdos_global_get_screen (cdos_global_get());
    self->switchWorkspaceNotifyId = g_signal_connect (cdos_base_wm, "switch-workspace", G_CALLBACK (activeWorkspaceChanged), self);
    self->workspaceAddedId = g_signal_connect (global_screen, "workspace-added", G_CALLBACK (onWorkspaceAdded), self);
    self->workspaceRemovedId = g_signal_connect (global_screen, "workspace-removed", G_CALLBACK (onWorkspaceRemoved), self);

    CLUTTER_ACTOR_CLASS (cdos_overview_thumbnails_box_parent_class)->show (actor);
    addThumbnails (self, 0, meta_screen_get_n_workspaces (global_screen));

    //ClutterStage *global_stage = cdos_global_get_stage (cdos_global_get ());
    //clutter_stage_set_key_focus (global_stage, (ClutterActor *)self);
}

/*
static void
cdos_overview_thumbnails_box_hide (ClutterActor *actor)
{
    CdosOverviewThumbnailsBox *self = (gpointer)actor;
    g_return_if_fail(CDOS_IS_OVERVIEW_THUMBNAILS_BOX(self));
    thumbnails_box_disconnect_signals (self);
    GList *iter;
    for (iter = self->thumbnails; iter; iter = iter->next)
    {
        CdosOverviewWorkspaceThumbnail *thumbnail = iter->data;
        //cdos_overview_workspace_thumbnail_destroy (thumbnail_index);
        clutter_actor_destroy ((ClutterActor *)thumbnail);
    }
    g_list_free(self->thumbnails);
    self->thumbnails = NULL;
    CLUTTER_ACTOR_CLASS (cdos_overview_thumbnails_box_parent_class)->hide (actor);
}
*/

static gboolean
cdos_overview_thumbnails_box_enter (ClutterActor *actor,
                                    ClutterCrossingEvent *event)
{
    ClutterActor *stage = clutter_actor_get_stage (actor);
    clutter_stage_set_key_focus ((ClutterStage *)stage, actor);
    return TRUE;
}

static gboolean
cdos_overview_thumbnails_box_key_press_event (ClutterActor *actor, ClutterKeyEvent *key_event)
{
    CdosOverviewThumbnailsBox *self = CDOS_OVERVIEW_THUMBNAILS_BOX (actor);
    ClutterEvent *event = (ClutterEvent *)key_event;
    ClutterModifierType modifiers = cdos_base_util_get_event_state (event);

    ClutterModifierType ctrlAltMask = CLUTTER_CONTROL_MASK | CLUTTER_MOD1_MASK;

    guint symbol = clutter_event_get_key_symbol (event);

    if (symbol == CLUTTER_KEY_Return || symbol == CLUTTER_KEY_space || symbol == CLUTTER_KP_Enter)
    {
        activateSelectedWorkspace (self);
        return TRUE;
    }

    if (modifiers & ctrlAltMask)
        return FALSE;

    return selectNextWorkspace (self, symbol);
}

static void
cdos_overview_thumbnails_box_class_init (CdosOverviewThumbnailsBoxClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

    gobject_class->finalize = cdos_overview_thumbnails_box_finalize;
    gobject_class->dispose = cdos_overview_thumbnails_box_dispose;

    gobject_class->set_property = cdos_overview_thumbnails_box_set_property;
    gobject_class->get_property = cdos_overview_thumbnails_box_get_property;

    ClutterActorClass *parent_class = CLUTTER_ACTOR_CLASS (klass);

    parent_class->get_preferred_width = cdos_overview_thumbnails_box_get_preferred_width;
    parent_class->get_preferred_height = cdos_overview_thumbnails_box_get_preferred_height;
    parent_class->allocate = cdos_overview_thumbnails_box_allocate;
    parent_class->show = cdos_overview_thumbnails_box_show;
    //parent_class->hide = cdos_overview_thumbnails_box_hide;
    parent_class->enter_event = cdos_overview_thumbnails_box_enter;
    parent_class->key_press_event = cdos_overview_thumbnails_box_key_press_event;
    //parent_class->leave_event = cdos_overview_thumbnails_box_leave;

    cdos_overview_thumbnails_box_signals [ALLOCATED] =
        g_signal_new ("allocated",
                G_TYPE_FROM_CLASS (klass),
                G_SIGNAL_RUN_LAST,
                0,
                NULL, NULL,
                g_cclosure_marshal_VOID__VOID,
                G_TYPE_NONE, 0 );

}

static void
cdos_overview_thumbnails_box_init (CdosOverviewThumbnailsBox *self)
{
    MetaScreen *screen = cdos_global_get_screen (cdos_global_get());

    self->thumbnails = g_ptr_array_new ();

    // The "porthole" is the portion of the screen that we show in the workspaces
    self->porthole.x = 0;
    self->porthole.y = 0;
    meta_screen_get_size(screen, &self->porthole.width, &self->porthole.height);

    self->kbThumbnailIndex = meta_screen_get_active_workspace_index (screen);
}

static void
cdos_overview_thumbnails_box_post_init (CdosOverviewThumbnailsBox *self)
{
    st_widget_add_style_class_name (ST_WIDGET (self), "workspace-thumbnails");    
    clutter_actor_set_reactive ((ClutterActor *)self, TRUE);
    clutter_actor_set_request_mode ((ClutterActor *)self, CLUTTER_REQUEST_HEIGHT_FOR_WIDTH /* 横向排列工作区缩略图 */);

    g_signal_connect (self, "scroll-event", G_CALLBACK (onScrollEvent), self);
}

CdosOverviewThumbnailsBox *
cdos_overview_thumbnails_box_new ()
{
    CdosOverviewThumbnailsBox *instance = g_object_new (CDOS_TYPE_OVERVIEW_THUMBNAILS_BOX, NULL);

    cdos_overview_thumbnails_box_post_init (instance);

    return instance;
}

