#include "cdos-layout-manager.h"
#include "cdos-start.h"
#include "cdos-panel-item-tooltip.h"

G_DEFINE_TYPE ( CdosPanelItemTooltip, cdos_panel_item_tooltip, CDOS_TYPE_TOOLTIP);

static void 
cdos_panel_item_tooltip_finalize (GObject *object) {
	CdosPanelItemTooltip *self = CDOS_PANEL_ITEM_TOOLTIP(object);
	G_OBJECT_CLASS(cdos_panel_item_tooltip_parent_class)->finalize(object);
}

static void 
panel_item_tooltip_show (CdosTooltip *tooltip)
{
    if (menu_stack_depth > 0)
        return;
	if (tooltip->preventShow)
		return;
    if (st_label_get_text ((StLabel *)tooltip) == NULL ||
            g_strcmp0 (st_label_get_text ((StLabel *)tooltip), "") == 0)
        return;

    clutter_actor_remove_all_transitions ((ClutterActor *)tooltip);

    CdosPanelItemTooltip *self = CDOS_PANEL_ITEM_TOOLTIP (tooltip);

	ClutterActorBox box;;
	clutter_actor_get_allocation_box (CLUTTER_ACTOR (tooltip), &box);

    gfloat tooltip_height = box.y2 - box.y1;
    gfloat tooltip_width  = box.x2 - box.x1;

	CdosLayoutManager *layout_manager = cdos_layout_manager_get_default ();
	//调用layout manager中的 findMonitorForActor, 尚没有此接口
	MetaRectangle *monitor; // = Main.layoutManager.findMonitorForActor(this._item);
    gfloat tooltip_top;

    ClutterActorBox item_box;
	clutter_actor_get_allocation_box (tooltip->item, &item_box);

    /* FIXME 当 applets 在 group-applet 中时，tooltip 位置不正确 */
    if (self->orientation == ST_SIDE_BOTTOM)
    {
        // FIXME layout_manager 还没有 bottom_monitor
        //monitor = cdos_layout_manager_get_bottom_monitor (layout_manager);
        monitor = cdos_layout_manager_get_primary_monitor(layout_manager);
        tooltip_top = monitor->y + monitor->height - tooltip_height -
            item_box.y2 + item_box.y1;

    } else {
        monitor = cdos_layout_manager_get_primary_monitor(layout_manager);
        tooltip_top = monitor->y + item_box.y2;
    }

    gint tooltip_left = tooltip->mousePositionX - tooltip_width / 2;
    if (tooltip_left < monitor->x)
        tooltip_left = monitor->x;
    if (tooltip_left + tooltip_width > monitor->x + monitor->width)
        tooltip_left = monitor->x + monitor->width - tooltip_width;

    clutter_actor_set_position ((ClutterActor *)tooltip, tooltip_left, tooltip_top);

	clutter_actor_show (CLUTTER_ACTOR (tooltip));
    st_container_raise ((ClutterContainer *)ui_group, (ClutterActor *)tooltip, NULL);
	tooltip->visible = TRUE;
}

static void
cdos_panel_item_tooltip_class_init (CdosPanelItemTooltipClass *klass)
{
	GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
	gobject_class->finalize = cdos_panel_item_tooltip_finalize;

    CdosTooltipClass *parent_class = CDOS_TOOLTIP_CLASS (klass);
    parent_class->show = panel_item_tooltip_show;
}



static void
cdos_panel_item_tooltip_init (CdosPanelItemTooltip *self) {
}

//公有函数定义

CdosPanelItemTooltip* 
cdos_panel_item_tooltip_new (ClutterActor *item, 
                             gchar *init_title,
                             StSide orientation)
{
	CdosPanelItemTooltip *tooltip;
	tooltip = g_object_new (CDOS_TYPE_PANEL_ITEM_TOOLTIP, 
            "item", item, "text", init_title, "name", "Tooltip", NULL);
    st_container_add_actor ((ClutterContainer *)ui_group, (ClutterActor *)tooltip);
    clutter_actor_hide ((ClutterActor *)tooltip);
    tooltip->orientation = orientation;
	return tooltip;
}

