#include "cdos-panel.h"
#include "cdos-window-manager.h"
#include <girepository.h>
#include <clutter/clutter.h>
#include "config.h"
#include "cdos.h"

/**
 * SECTION:cdos-panel
 * @title: CdosPanel
 * @short_description: 任务栏
 *
 * 任务栏分为三部分：左侧，中间，右侧。左侧放menu、scale、文件夹、终端，中间目前还没用到，右侧放grouped-applets、可移动磁盘、网络、声音、systray、showdesktop
 */
#define AUTOHIDE_ANIMATION_TIME  200
// 添加右键菜单
//
typedef struct _PanelLauncherStruct
{
    CdosPopupMenu *menu;
    gchar *keyword;
} PanelLauncherStruct;

typedef struct _SettingsLauncher
{
    CdosPopupBaseMenuItem *laucher;
    StTable *table;
    StLabel *label;
    StIcon *icon ;
    PanelLauncherStruct *transfer;
    gulong transfer_id;
} SettingsLauncher;

G_DEFINE_TYPE (CdosPanel, cdos_panel, ST_TYPE_CONTAINER);

// temple
gfloat xpos = 10.0;

/* 下面这个宏可以用来访问新建对象的私有结构*/
#define CDOS_PANEL_GET_PRIVATE(obj) \
	(G_TYPE_INSTANCE_GET_PRIVATE ((obj), CDOS_TYPE_PANEL, CdosPanelPrivate))

struct _CdosPanelPrivate
{
//  StButton *button;
  StBoxLayout *leftBox;
  StBoxLayout *centerBox;
  StBoxLayout *rightBox;
  gboolean    bottomPosition :1;
  gboolean    _hideable :1;
  gint        hideDelay;
  gint        showDelay;
  guint       hideTimer;
  guint       showTimer;
  gchar       *text;
  gboolean    hidden;
  gboolean    disabled;
  gboolean    isMouseOverPanel;
  gboolean    is_primary :1;
  CdosPopupMenu *context_menu;
  CdosPopupMenuManager *menu_manager;
  GList *settings_launchers;
};

enum {
  PROP_NUM = 0,
  PROP_TEXT ,
  PROP_LEFT,
  PROP_CENTER,
  PROP_RIGHT,
  PROP_POSITION,
  PROP_HIDEABLE,
  PROP_PRIMARY,
  NUM_PROPERTIES
};

static GParamSpec *pspecs[NUM_PROPERTIES] = { NULL, };
/* 列出类的所有信号;
 * LAST_SIGNAL 只是用来说明信号数组的结束 
 */
enum {
  ALLOCATE,
  LAST_SIGNAL
};

/* 信号数组 */
static guint cdos_panel_signals[LAST_SIGNAL] = { 0, };

static void cdos_panel_set_primary (CdosPanel *self, gboolean is_primary);

/**
 * cdos_panel_get_leftbox: 
 * @panel: #CdosPanel类的一个对象
 *
 * 获取左侧的box
 *
 * Returns:左侧的box
 */
StBoxLayout *
cdos_panel_get_leftbox (CdosPanel *panel)
{
    g_return_val_if_fail (CDOS_IS_PANEL (panel), NULL);
    return panel->priv->leftBox;
}


/**
 * cdos_panel_get_centerbox:
 * @panel: #CdosPanel类的一个对象
 *
 * 获取中间的box
 *
 * Returns:中间的box
 */
StBoxLayout *
cdos_panel_get_centerbox (CdosPanel *panel){
    g_return_val_if_fail (CDOS_IS_PANEL (panel), NULL);
    return panel->priv->centerBox;
}


/**
 * cdos_panel_get_rightbox:
 * @panel: #CdosPanel类的一个对象
 *
 * 获取右侧的box
 *
 * Returns:右侧的box
 */
StBoxLayout *
cdos_panel_get_rightbox (CdosPanel *panel){
    g_return_val_if_fail (CDOS_IS_PANEL (panel), NULL);
    return panel->priv->rightBox;
}

static void
cdos_panel_set_leftbox (CdosPanel    *panel,
            			StBoxLayout *button)
{
	CdosPanelPrivate *priv;
    priv = panel->priv;
	priv->leftBox = button;
}

static void
cdos_panel_set_centerbox (CdosPanel    *panel,
            			  StBoxLayout *button)
{
	CdosPanelPrivate *priv;
    priv = panel->priv;
	priv->centerBox = button;
}

static void
cdos_panel_set_rightbox (CdosPanel    *panel,
			             StBoxLayout *button)
{
	CdosPanelPrivate *priv;
    priv = panel->priv;
	priv->rightBox = button;
}

static void
cdos_panel_set_text (CdosPanel *self, const gchar *text)
{
  CdosPanelPrivate *priv;

  g_return_if_fail (CDOS_IS_PANEL (self));

  priv = self->priv;
    if(priv->text){
        g_free (priv->text);
    }
  if (text)
    priv->text = (gchar *)text;
  else
    priv->text = "";
}

static void
cdos_panel_dispose (GObject *gobject)
{
    CdosPanelPrivate *priv = CDOS_PANEL (gobject)->priv;

    if(priv->settings_launchers){
        GList *launchers = priv->settings_launchers;
        while(launchers != NULL){
            SettingsLauncher *launcher = launchers->data;
            if(launcher->transfer_id > 0){
                g_signal_handler_disconnect(launcher->laucher,launcher->transfer_id);
                launcher->transfer_id = 0;
                g_free(launcher->transfer);
            }
            g_object_unref(launcher->label);
            g_object_unref(launcher->icon);
            g_object_unref(launcher->table);
            g_object_unref(launcher->laucher);
            launchers ->data = NULL;
            launchers = launchers->next;
        }
        g_list_free(launchers);
    }

    g_clear_object(&priv->context_menu);
    g_clear_object(&priv->menu_manager);

    g_object_unref (priv->leftBox);
    g_object_unref (priv->centerBox);
    g_object_unref (priv->rightBox);
    /* 调用父类的 finalize() 方法 */
    G_OBJECT_CLASS (cdos_panel_parent_class)->dispose (gobject);
}

static void
cdos_panel_finalize (GObject *gobject)
{
  CdosPanelPrivate *priv = CDOS_PANEL (gobject)->priv;


    if(priv->text){
        g_free(priv->text);
    }

  /* 调用父类的 finalize() 方法 */
  G_OBJECT_CLASS (cdos_panel_parent_class)->finalize (gobject);
}

static void
cdos_panel_set_property (GObject      *gobject,
                        guint         prop_id,
                        const GValue *value,
                        GParamSpec   *pspec)
{
  //g_print ("%s\n", __func__);
  CdosPanel *obj = CDOS_PANEL (gobject);

  switch (prop_id)
    {
    case PROP_TEXT:
      cdos_panel_set_text (obj, g_value_get_string (value)); // 公共接口
      break;
    case PROP_LEFT:
      //printf("set prop left\n");
      cdos_panel_set_leftbox (obj, g_value_get_object(value));
      break;
    case PROP_CENTER:
      cdos_panel_set_centerbox (obj, g_value_get_object(value));
      break;
    case PROP_RIGHT:
      cdos_panel_set_rightbox (obj, g_value_get_object(value));
      break;
    case PROP_POSITION:
      //g_print ("set bottom position\n");
      cdos_panel_set_bottom_position(obj, g_value_get_boolean(value));
      break;
    case PROP_HIDEABLE:
      cdos_panel_set_bottom_position(obj, g_value_get_boolean(value));
      break;
    case PROP_PRIMARY:
      //g_print ("set is primary\n");
      cdos_panel_set_primary (obj, g_value_get_boolean (value));
      break;

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
      break;
    }
} 

static void
cdos_panel_get_property (GObject    *gobject,
                        guint       prop_id,
                        GValue     *value,
                        GParamSpec *pspec)
{
  //g_print ("%s\n", __func__);
  CdosPanelPrivate *priv = CDOS_PANEL (gobject)->priv;
  //CdosPanel *newobj = CDOS_PANEL (gobject);

  switch (prop_id)
    {
    case PROP_TEXT:
      g_value_set_string (value, priv->text);
      break;
    case PROP_LEFT:
      //printf("get prop left\n");
      g_value_set_object(value, priv->leftBox);
      break;
    case PROP_CENTER:
      g_value_set_object(value, priv->centerBox);
      break;
    case PROP_RIGHT:
      g_value_set_object(value, priv->rightBox);
      break;
    case PROP_POSITION:
      g_value_set_boolean(value, priv->bottomPosition);
      break;
    case PROP_HIDEABLE:
      g_value_set_boolean(value, priv->_hideable);
      break;
    case PROP_PRIMARY:
      g_value_set_boolean (value, priv->is_primary);

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
      break;
    }
}

static void
cdos_panel_get_preferred_height (ClutterActor *self,
                                gfloat for_width,
                                gfloat *min_height_p,
                                gfloat *natural_height_p)
{
  //CdosPanelPrivate *priv = CDOS_PANEL (self)->priv;
  StThemeNode *theme_node = st_widget_get_theme_node ((StWidget *)self);
  st_theme_node_adjust_for_width (theme_node, &for_width);

  /*
  clutter_actor_get_preferred_height (priv->leftBox,
                                      for_width,
                                      min_height_p,
                                      natural_height_p);
  */

  if (min_height_p)
      *min_height_p = -1;
  if (natural_height_p)
      *natural_height_p = -1;
  st_theme_node_adjust_preferred_height (theme_node, min_height_p, natural_height_p);
}

static void
cdos_panel_get_preferred_width (ClutterActor *self,
                               gfloat for_height,
                               gfloat *min_width_p,
                               gfloat *natural_width_p)
{
  //CdosPanelPrivate *priv = CDOS_PANEL (self)->priv;
  //MetaRectangle *monitor = NULL;

  StThemeNode *node = st_widget_get_theme_node ((StWidget *)self);
  st_theme_node_adjust_for_height (node, &for_height);

  /*
  gfloat min_width_pl,min_width_pc, min_width_pr = 0;
  gfloat natural_width_pl, natural_width_pc, natural_width_pr = 0;
  clutter_actor_get_preferred_width (priv->leftBox,
                                     for_width,
                                     &min_width_pl,
                                     &natural_width_pl);
  clutter_actor_get_preferred_width (priv->centerBox,
                                     for_width,
                                     &min_width_pc,
                                     &natural_width_pc);
  clutter_actor_get_preferred_width (priv->rightBox,
                                     for_width,
                                     &min_width_pr,
                                     &natural_width_pr);

  *min_width_p = min_width_pl+ min_width_pc + min_width_pr;
  *natural_width_p = natural_width_pl + natural_width_pc+ natural_width_pr;

  monitor = cdos_layout_manager_get_primary_monitor (cdos_monkey_plugin_get_layout_manager (monkey_plugin_get ()));
  if (*min_width_p < monitor->width && *natural_width_p < monitor->width){
    *natural_width_p = monitor->width;
   }
  */
  *min_width_p = -1;
  *natural_width_p = 1920;
  st_theme_node_adjust_preferred_width (node, min_width_p, natural_width_p);
}

static void
cdos_panel_allocate (ClutterActor           *self,
                                  const ClutterActorBox  *box,
                                  ClutterAllocationFlags  flags)
{
    gfloat allocWidth, allocHeight;

    allocWidth = box->x2 - box->x1;
    allocHeight = box->y2 - box->y1;

    // allocWidth = 1366;
    // allocHeight = 50;

    StThemeNode *theme_node;
    ClutterActorBox content_box = {0,};

    CdosPanelPrivate *priv = CDOS_PANEL(self)->priv;
    CLUTTER_ACTOR_CLASS (cdos_panel_parent_class)->allocate (self, box, flags);

    theme_node = st_widget_get_theme_node (ST_WIDGET (self));
    //children = st_container_get_children_list (ST_CONTAINER (actor));
    //    if (children == NULL)
    //	    return;

    st_theme_node_get_content_box (theme_node, box, &content_box);

    allocHeight = content_box.y2 - content_box.y1;

//    clutter_actor_allocate(priv->leftBox, &content_box, flags);
  //  printf("en %d %d\n", allocWidth, allocHeight);
    gfloat leftWidthf, centerWidthf, rightWidthf = 0;
    clutter_actor_get_preferred_width ((ClutterActor *)priv->leftBox, -1.0,
                                     NULL,
                                     &leftWidthf);
    clutter_actor_get_preferred_width ((ClutterActor *)priv->centerBox, -1.0,
                                     NULL,
                                     &centerWidthf);
    clutter_actor_get_preferred_width ((ClutterActor *)priv->rightBox, -1.0,
                                     NULL,
                                     &rightWidthf);
//    *leftWidth = MAX (*leftWidth, 256);
//    *rightWidth = MAX (*rightWidth, 256);

//    printf("attentin: %f %f %f\n", *leftWidthf, *centerWidthf, *rightWidthf);
    gfloat spaceNeeded = 0.f;
    spaceNeeded = leftWidthf + rightWidthf;
    if(spaceNeeded < allocWidth){
	    centerWidthf = allocWidth - spaceNeeded;
    }

    content_box.x1 = 0;
    content_box.x2 = leftWidthf;
    content_box.y1 = 0;
    content_box.y2 = allocHeight;
    clutter_actor_allocate ((ClutterActor *)priv->leftBox, &content_box, flags);
//    g_signal_emit (G_OBJECT (self), cdos_panel_signals[ALLOCATE], 0,
//                 &content_box, flags);
    content_box.x1 = leftWidthf;
    content_box.x2 = allocWidth -rightWidthf;
    content_box.y1 = 0.f;
    content_box.y2 = allocHeight;
    clutter_actor_allocate ((ClutterActor *)priv->centerBox, &content_box, flags);

    content_box.x1 = allocWidth - rightWidthf;
    content_box.x2 = allocWidth;
    content_box.y1 = 0.f;
    content_box.y2 = allocHeight;
    clutter_actor_allocate ((ClutterActor *)priv->rightBox, &content_box, flags);
}

static void
cdos_panel_paint (ClutterActor  *actor)
{
  //CdosPanel *self = (CdosPanel*) actor;
  //GList *iter, *children;

  CLUTTER_ACTOR_CLASS (cdos_panel_parent_class)->paint (actor);

/*  children = st_container_get_children_list (ST_CONTAINER (actor));
  for (iter = children; iter; iter = iter->next)
    {
      ClutterActor *child = iter->data;

      clutter_actor_paint (child);
      printf("paint panel\n");
    }*/
  CdosPanelPrivate *priv = CDOS_PANEL(actor)->priv;
  clutter_actor_paint ((ClutterActor *)priv->leftBox);
  clutter_actor_paint ((ClutterActor *)priv->centerBox);
  clutter_actor_paint ((ClutterActor *)priv->rightBox);
}

/**
 * cdos_panel_on_button_press:
 * @actor: #ClutterActor
 * @event: #ClutterEvent
 * @data: user data
 *
 * 点击出现右键菜单
 *
 * Returns: TRUE,出现右键菜单 
 */
gboolean
cdos_panel_on_button_press (ClutterActor *actor,
		 ClutterEvent *event,
		 gpointer data){
    CdosPanel *self = (CdosPanel *)actor;
    CdosPanelPrivate *priv = self->priv;
    guint event_type;


    event_type = clutter_event_get_button (event);
    if (event_type == 1){
        if (priv->context_menu && CDOS_POPUP_MENU_BASE (priv->context_menu)->is_open){
            cdos_popup_menu_base_toggle ((CdosPopupMenuBase *)priv->context_menu);
        }
    }else if (event_type == 3){
        gfloat x, y;
        ClutterStage *stage;
        
        stage = cdos_global_get_stage (cdos_global_get ());
        clutter_event_get_coords (event, &x, &y);
        ClutterActor *target = clutter_stage_get_actor_at_pos (stage, CLUTTER_PICK_ALL, x, y);
        ClutterActor *parent = clutter_actor_get_parent ((ClutterActor *)target);

        if (parent == actor){
            cdos_popup_menu_base_toggle (CDOS_POPUP_MENU_BASE (priv->context_menu));
        }
        if (!CDOS_POPUP_MENU_BASE (priv->context_menu)->is_open){
            return FALSE;
        }

        CdosBoxPointer *box_pointer = cdos_popup_menu_get_box_pointer (priv->context_menu);
        x -=  cdos_box_pointer_get_arrow_origin (box_pointer);
  
        ClutterActorBox  allocation_box;
        clutter_actor_get_allocation_box ((ClutterActor *)box_pointer->actor, &allocation_box);
        gfloat myWidth = allocation_box.x2 - allocation_box.x1;

        CdosLayoutManager *layout_manager = cdos_layout_manager_get_default ();
        MetaRectangle *monitor = cdos_layout_manager_get_primary_monitor (layout_manager);
    
        if (x + myWidth - monitor->x > monitor->width){
            x = monitor->width + monitor->x - myWidth;
        }

        if (x < monitor->x){
            x = monitor->x;
        }
        
        xpos = round (x);
        cdos_box_pointer_set_x_position (box_pointer, round (x));
        cdos_box_pointer_shift_actor (box_pointer);
    }
	return TRUE;
}

static void
panel_layout_window (ClutterEvent *event, guint mode){
    //CdosWindowTracker *tracker = cdos_window_tracker_get_default();
   gboolean maximized_h = FALSE;
   gboolean maximized_v = FALSE;
   GList *windowList = NULL;

    GList *window_actors = cdos_global_get_window_actors (cdos_global_get ());
    GList *wa = window_actors;
    while(wa != NULL){
        MetaWindow *mwin = meta_window_actor_get_meta_window (wa->data);
        windowList = g_list_append (windowList, mwin);
        wa = wa->next;
    }

    GList *window_filter = NULL;
    MetaScreen *screen = cdos_global_get_screen (cdos_global_get ());
    while (windowList != NULL){
        GList *next = windowList->next;
        MetaWindow *window = windowList->data;
        MetaWorkspace *workspace = meta_screen_get_active_workspace (screen);
        if (meta_window_get_workspace (window) == workspace){
            if (cdos_window_tracker_is_window_interesting (window)){
                window_filter = g_list_prepend (window_filter, window);
            }
        }
        windowList = next;
    }

    CdosLayoutManager *data = cdos_layout_manager_get_default ();
    MetaRectangle *monitor = cdos_layout_manager_get_primary_monitor (data);
    guint maxWidth = round (monitor->width);
    guint ystart = monitor->y;


    CdosPanel *panel = cdos_monkey_plugin_get_panel ( monkey_plugin_get ( ) );
    guint maxHeight = round (monitor->height - clutter_actor_get_height ((ClutterActor *)panel)); // get_height

    GList *layoutWindows = NULL;
    guint maxWindows = 2;
    guint windowNums = 0;

    //guint length = g_list_length (window_filter);
    GList *wf = window_filter;
    while ( wf != NULL ){
        MetaWindow *meta_win = wf->data;
        gboolean minimized = FALSE;
        g_object_get(meta_win,"minimized",&minimized,NULL);
       
        if (!minimized ){
           layoutWindows = g_list_append (layoutWindows, meta_win);
           windowNums ++;
        }
        if (windowNums == maxWindows){
            break;
        }

        wf = wf->next;
    }
        if (windowNums == 0){
            return;
        }else if (windowNums == 1){
            MetaWindow *to_be_shown = g_list_nth_data (layoutWindows, 0);
            meta_window_move_resize_frame (to_be_shown, 1, monitor->x, ystart, maxWidth, maxHeight);
        }else {
            if ( mode == 1){
                MetaWindow *to_be_shown1 = g_list_nth_data (layoutWindows, 0);
                MetaWindow *to_be_shown2 = g_list_nth_data (layoutWindows, 1);
	        g_object_get(to_be_shown1, "maximized-horizontally",&maximized_h,NULL);
        	g_object_get(to_be_shown1, "maximized-vertically",&maximized_v,NULL);
	
		if (maximized_h && maximized_v){
			meta_window_unmaximize (to_be_shown1, META_MAXIMIZE_HORIZONTAL);
			meta_window_unmaximize (to_be_shown1, META_MAXIMIZE_VERTICAL);
		}
		maximized_h = FALSE;
		maximized_v = FALSE;
	        g_object_get(to_be_shown2, "maximized-horizontally",&maximized_h,NULL);
        	g_object_get(to_be_shown2, "maximized-vertically",&maximized_v,NULL);
		if (maximized_h && maximized_v){
			meta_window_unmaximize (to_be_shown2, META_MAXIMIZE_HORIZONTAL);
			meta_window_unmaximize (to_be_shown2, META_MAXIMIZE_VERTICAL);
		}

                meta_window_move_resize_frame (to_be_shown1, 1, monitor->x, monitor->y, maxWidth / 2, maxHeight);
                meta_window_move_resize_frame (to_be_shown2, 1, maxWidth / 2, monitor->y, maxWidth / 2, maxHeight);
		meta_window_raise (to_be_shown1);
                meta_window_raise (to_be_shown2);
            }else if (mode == 2){
                MetaWindow *to_be_shown1 = g_list_nth_data (layoutWindows, 0);
                MetaWindow *to_be_shown2 = g_list_nth_data (layoutWindows, 1);
	        g_object_get(to_be_shown1, "maximized-horizontally",&maximized_h,NULL);
        	g_object_get(to_be_shown1, "maximized-vertically",&maximized_v,NULL);
	
		if (maximized_h && maximized_v){
			meta_window_unmaximize (to_be_shown1, META_MAXIMIZE_HORIZONTAL);
			meta_window_unmaximize (to_be_shown1, META_MAXIMIZE_VERTICAL);
		}
		maximized_h = FALSE;
		maximized_v = FALSE;
	        g_object_get(to_be_shown2, "maximized-horizontally",&maximized_h,NULL);
        	g_object_get(to_be_shown2, "maximized-vertically",&maximized_v,NULL);
		if (maximized_h && maximized_v){
			meta_window_unmaximize (to_be_shown2, META_MAXIMIZE_HORIZONTAL);
			meta_window_unmaximize (to_be_shown2, META_MAXIMIZE_VERTICAL);
		}

		/*MetaRectangle rect1, rect2;
		meta_window_get_outer_rect (to_be_shown1, &rect1);
		meta_window_get_outer_rect (to_be_shown2, &rect2);
		 这里打算添加恢复的
		g_print ("to be shown one :%s , x %d y %d width %d height %d ... and two: %s, x %d y %d width %d height %d\n", 
				meta_window_get_title (to_be_shown1), rect1.x, rect1.y, rect1.width, rect1.height, 
				meta_window_get_title (to_be_shown2), rect2.x, rect2.y, rect2.width, rect2.height);*/
                meta_window_move_resize_frame (to_be_shown1, 1, monitor->x, ystart, maxWidth, maxHeight / 2);
                meta_window_move_resize_frame (to_be_shown2, 1, monitor->x, ystart + maxHeight / 2, maxWidth, maxHeight / 2);
		meta_window_raise (to_be_shown1);
		meta_window_raise (to_be_shown2);
            }else if (mode == 3){
            
            }
        }        
}

static void
panel_settings_launcher_activate (CdosPopupBaseMenuItem *actor, 
                                  ClutterEvent *event, 
                                  gboolean keep_menu,
                                  gpointer data){
    // get keyword
    PanelLauncherStruct *transfer = (PanelLauncherStruct *)data;
    if (g_strcmp0 (transfer->keyword, "show-desktop") == 0){
        MetaScreen *screen = cdos_global_get_screen (cdos_global_get ());
        meta_screen_show_desktop (screen, cdos_global_get_current_time (cdos_global_get ()));
    }else if (g_strcmp0 (transfer->keyword, "gnome-system-monitor") == 0){
        cdos_util_spawn_command_line ("gnome-system-monitor");
    }else if (g_strcmp0 (transfer->keyword, "stack-windows") == 0){
        panel_layout_window (event, 2);
    }else if (g_strcmp0 (transfer->keyword, "abreast-windows") == 0){
        panel_layout_window (event, 1);
    }else if (g_strcmp0 (transfer->keyword, "overlapping-windows")){
        panel_layout_window (event, 3);
    }
    cdos_popup_menu_base_toggle (CDOS_POPUP_MENU_BASE (transfer->menu));
}


static SettingsLauncher *
panel_new_settings_launcher (const gchar *label_text,
                             const gchar *keyword,
                             const gchar *icon_name,
                             CdosPopupMenu *menu){
    SettingsLauncher *launcher = g_new0(SettingsLauncher,1);
    CdosPopupBaseMenuItem *settings_launcher = cdos_popup_base_menu_item_new();
    launcher->laucher = settings_launcher;
    CdosGenericContainer *launcher_actor = cdos_popup_base_menu_item_get_actor(settings_launcher);
    st_widget_add_style_class_name(ST_WIDGET(launcher_actor),"popup-contextmenu-item");
    StTable *table = g_object_new (ST_TYPE_TABLE, "reactive", TRUE, "homogeneous", FALSE, NULL);
    launcher->table = table;
    //StButton *button = st_button_new_with_label (label_text);
    StWidget *label = st_label_new (label_text);
    st_widget_set_style_class_name(label,"popup-menu-item-label");
    launcher->label = (StLabel *)label;
    StIcon *icon = g_object_new (ST_TYPE_ICON, "icon-name", icon_name,
                                                "icon-size",16,
                                                "icon-type",ST_ICON_SYMBOLIC,
                                                "style-class", "popup-menu-icon",
                                                NULL);
    launcher->icon = icon;
    clutter_container_add_actor (CLUTTER_CONTAINER (table), (ClutterActor *)icon);
    StTableChild *child = (StTableChild *)clutter_container_get_child_meta (CLUTTER_CONTAINER (table), (ClutterActor *)icon);
    child->row = 0;
    child->col = 0;
    g_object_set (child, "row", 0, "col", 0, NULL);
    child->col_span = 1;
    child->x_expand = FALSE;
    child->x_align = ST_ALIGN_START;
    clutter_actor_queue_relayout ((ClutterActor *)icon);

    clutter_container_add_actor (CLUTTER_CONTAINER (table), (ClutterActor *)label);
    StTableChild *child2 = (StTableChild *)clutter_container_get_child_meta (CLUTTER_CONTAINER (table), (ClutterActor *)label);
    //child2->row = 0;
    //child2->col = 1;
    g_object_set (child2, "row", 0, "col", 1, NULL);
    child2->col_span = 1;
    child2->x_align = ST_ALIGN_START;
    clutter_actor_queue_relayout ((ClutterActor *)label);
    st_widget_set_style ((StWidget *)table, "padding:0;");
    ActorChild actor_child;
    //cdos_popup_base_menu_item_get_init_params (table_params);
    actor_child.expand = 1;
    actor_child.span = 1;
    actor_child.align = ST_ALIGN_START;
    cdos_popup_base_menu_item_add_actor (settings_launcher, (ClutterActor *)table, &actor_child);
    /*
    StBoxLayout *table = st_box_layout_new ();
    StIcon *icon = g_object_new (ST_TYPE_ICON, "icon-name", icon_name,
                                                "icon-type", ST_ICON_SYMBOLIC,
                                                "style-class", "popup-menu-icon",
                                                NULL);

    clutter_container_add_actor (table, icon);

    clutter_container_add_actor (table, button);
    clutter_actor_set_margin_left (CLUTTER_ACTOR (button), 15.0);

    PopupBaseMenuItemInit *table_params = g_malloc0 (sizeof (PopupBaseMenuItemInit));
    cdos_popup_base_menu_item_get_init_params (table_params);
    cdos_popup_base_menu_item_add_actor (settings_launcher, table, table_params);
    */

    PanelLauncherStruct *transfer = g_malloc0 (sizeof(PanelLauncherStruct));
    transfer->menu = menu;
    transfer->keyword = (gchar *)keyword;
    launcher->transfer = transfer;
    launcher->transfer_id = g_signal_connect (settings_launcher, "activate", G_CALLBACK (panel_settings_launcher_activate), transfer);
    return launcher;
}


static CdosPopupMenu *
panel_get_context_menu (CdosPanel *self, guint orientation){

    ClutterActor *source_actor = CLUTTER_ACTOR(self);
    GList *launchers = self->priv->settings_launchers;
    CdosPopupMenu *menu = cdos_popup_menu_new_with_params (source_actor, 0.0, ST_SIDE_BOTTOM);
    ClutterActor *ui_group = monkey_plugin_get_ui_group (monkey_plugin_get ());

    ClutterActor *menu_actor = (ClutterActor *)menu->actor;
    st_container_add_actor (CLUTTER_CONTAINER (ui_group), menu_actor);
    //clutter_actor_set_width (menu_actor, 250);
    st_widget_add_style_class_name(ST_WIDGET(menu_actor),"popup-contextmenu");

// 显示桌面
    SettingsLauncher *show_desktop_item = panel_new_settings_launcher (_("Show desktop"),
                                                            "show-desktop", "show-desktop", menu);
    cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE (menu), show_desktop_item->laucher, -1);
    launchers = g_list_append(launchers,show_desktop_item);

    CdosPopupSeparatorMenuItem  *s1 = cdos_popup_separator_menu_item_new();
    cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE (menu),CDOS_POPUP_BASE_MENU_ITEM(s1), -1);

// 系统
    SettingsLauncher *menu_settings_item = panel_new_settings_launcher (_("Open the system monitor"),
                                                            "gnome-system-monitor", "system-monitor", menu);
    cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE (menu), menu_settings_item->laucher, -1);
    launchers = g_list_append(launchers,menu_settings_item);

    CdosPopupSeparatorMenuItem  *s2 = cdos_popup_separator_menu_item_new();
    cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE (menu),CDOS_POPUP_BASE_MENU_ITEM(s2), -1);
// 系统设置
    SettingsLauncher *stack_windows_item = panel_new_settings_launcher (_("Stack windows"),
                                                            "stack-windows", "stack-windows", menu);
    cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE (menu), stack_windows_item->laucher, -1);
    launchers = g_list_append(launchers,stack_windows_item);

    //CdosPopupSeparatorMenuItem  *s3 = cdos_popup_separator_menu_item_new();
    //cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE (menu),CDOS_POPUP_BASE_MENU_ITEM(s3), -1);

    SettingsLauncher *abreast_windows_item = panel_new_settings_launcher (_("Abreast windows"),
                                                            "abreast-windows", "abreast-windows", menu);
    cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE (menu), abreast_windows_item->laucher, -1);
    launchers = g_list_append(launchers,abreast_windows_item);
/*
    CdosPopupSeparatorMenuItem  *s4 = cdos_popup_separator_menu_item_new();
    cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE (menu),CDOS_POPUP_BASE_MENU_ITElaucherM(s4), -1);

    SettingsLauncher *overlapping_windows_item = panel_new_settings_launcher (_("Overlapping windows"),
                                                            "overlapping-windows", "show-desktop", menu);
    cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE (menu), overlapping_windows_item->laucher, -1);
     launchers = g_list_append(launchers,overlapping_windows_item);

    ClutterActor *show_desktop_actor = (ClutterActor *)cdos_popup_base_menu_item_get_actor ((CdosPopupBaseMenuItem *)show_desktop_item->laucher);
   // g_signal_connect (show_desktop_actor, "button-release-event", G_CALLBACK (panel_settings_launcher_activate), "show-desktop");
    */
    return menu;
}

/* class init: 类的继承关系，将子类自己的方法绑定到父类, 
 * 定义属性和信号
 */
static void
cdos_panel_class_init (CdosPanelClass *klass)
{
  //g_print ("%s\n", __func__);
  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);// 自己的父类型是什么？
  ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);

  gobject_class->finalize = cdos_panel_finalize;
  gobject_class->dispose = cdos_panel_dispose;
  gobject_class->set_property = cdos_panel_set_property;
  gobject_class->get_property = cdos_panel_get_property;

  actor_class->get_preferred_height = cdos_panel_get_preferred_height;
  actor_class->get_preferred_width = cdos_panel_get_preferred_width;
  actor_class->allocate = cdos_panel_allocate;
  actor_class->paint = cdos_panel_paint;

  /*
  cdos_panel_signals[ALLOCATE] =
    g_signal_new ("allocate",
                  G_TYPE_FROM_CLASS (klass),
                  G_SIGNAL_RUN_LAST,
                  0,
                  NULL, NULL,
                  gi_cclosure_marshal_generic,
                  G_TYPE_NONE, 2, G_TYPE_FLOAT, CLUTTER_TYPE_ALLOCATION_FLAGS);
  */

  pspecs[PROP_TEXT] = g_param_spec_string ("text",
                               "Text",
                               "Text of the object",
                               NULL,
                               G_PARAM_READWRITE);

  pspecs[PROP_LEFT] = g_param_spec_object ("left",
				"Left Box",
				"Left Box of Panel",
				ST_TYPE_BOX_LAYOUT,
				G_PARAM_READWRITE);
  pspecs[PROP_CENTER] = g_param_spec_object ("center",
				"Center Box",
				"Center Box of Panel",
				ST_TYPE_BOX_LAYOUT,
				G_PARAM_READWRITE);
  pspecs[PROP_RIGHT] = g_param_spec_object ("right",
				"Right Box",
				"Right Box of Panel",
				ST_TYPE_BOX_LAYOUT,
				G_PARAM_READWRITE);

  pspecs[PROP_POSITION] = g_param_spec_boolean("bottom-position",
				"Bottom  position",
				"The panel is bottom or top",
				TRUE,
				G_PARAM_READWRITE);
  pspecs[PROP_HIDEABLE] = g_param_spec_boolean("hideable",
				"hide panel",
				"The panel can hide or not",
				FALSE,
				G_PARAM_READWRITE);

  pspecs[PROP_PRIMARY] = g_param_spec_boolean ("is-primary",
				"is primary",
				"the panel is primary or not",
				TRUE,
				G_PARAM_READWRITE);


  g_object_class_install_properties (gobject_class, NUM_PROPERTIES, pspecs);//新建书香并安装
 
  g_type_class_add_private (klass, sizeof (CdosPanelPrivate));
  
}

static void container_allocate (CdosGenericContainer *container,
                    ClutterActorBox *box,
                    ClutterAllocationFlags flags,
                    gpointer data){
    CdosBoxPointer *self = (CdosBoxPointer *)data;
    cdos_box_pointer_set_x_position (self, xpos);
    cdos_box_pointer_shift_actor (self);
}

static void
cdos_panel_init (CdosPanel *self)
{
  CdosPanelPrivate *priv;
  //g_print("%s\n", __func__);
  priv = self->priv = CDOS_PANEL_GET_PRIVATE (self);

  // StButton *button = st_button_new_with_label("CDOS"); 
  // StButton *button2 = st_button_new_with_label("Center");
  // StButton *button3 = st_button_new_with_label("Right");
  StBoxLayout *left_box = (StBoxLayout *)st_box_layout_new();
  StBoxLayout *center_box = (StBoxLayout *)st_box_layout_new();
  StBoxLayout *right_box = (StBoxLayout *)st_box_layout_new();

  clutter_actor_set_name(CLUTTER_ACTOR(left_box), "panelLeft");
  clutter_actor_set_name(CLUTTER_ACTOR(center_box), "panelCenter");
  clutter_actor_set_name(CLUTTER_ACTOR(right_box), "panelRight");

  cdos_panel_set_leftbox (self, left_box);
  cdos_panel_set_centerbox (self, center_box);
  cdos_panel_set_rightbox (self, right_box);

  priv->bottomPosition = TRUE;
  priv->_hideable = TRUE;
  // clutter_container_add_actor(CLUTTER_CONTAINER(left_box), CLUTTER_ACTOR(button));
  // clutter_container_add_actor(CLUTTER_CONTAINER(center_box), CLUTTER_ACTOR(button2));
  // clutter_container_add_actor(CLUTTER_CONTAINER(right_box), CLUTTER_ACTOR(button3));
  clutter_container_add_actor(CLUTTER_CONTAINER(self), CLUTTER_ACTOR(left_box));
  clutter_container_add_actor(CLUTTER_CONTAINER(self), CLUTTER_ACTOR(center_box));
  clutter_container_add_actor(CLUTTER_CONTAINER(self), CLUTTER_ACTOR(right_box));
  //g_print ("%s finished\n", __func__);

  // st_button_set_label(self->leftBox, "button");
  // g_signal_connect (CLUTTER_ACTOR(button), "button-press-event", G_CALLBACK (on_button_press), NULL);
  // g_signal_connect (CLUTTER_ACTOR(button2), "button-press-event", G_CALLBACK (on_button_press), NULL);
  // g_signal_connect (CLUTTER_ACTOR(button3), "button-press-event", G_CALLBACK (on_button_press), NULL);
}

static void 
cdos_panel_on_panel_hide_delay_changed(CdosPanel *self)
{
    CdosPanelPrivate *priv;
   
    g_return_if_fail(CDOS_IS_PANEL(self));

    priv = self->priv;

    CdosGlobal *global = cdos_global_get();
    GSettings *settings = cdos_global_get_settings (global);

    priv->hideDelay = g_settings_get_int(settings,"panel-hide-delay");

}

static void 
cdos_panel_on_panel_show_delay_changed(CdosPanel *self)
{
    CdosPanelPrivate *priv;
   
    g_return_if_fail(CDOS_IS_PANEL(self));

    priv = self->priv;

    CdosGlobal *global = cdos_global_get();
    GSettings *settings = cdos_global_get_settings (global);

    priv->showDelay = g_settings_get_int(settings,"panel-show-delay");

}

static gboolean
cdos_panel_hide_panel(CdosPanel *self,gboolean force)
{
    CdosPanelPrivate *priv;
    priv = self->priv;

    if((priv->_hideable && !force) || priv->isMouseOverPanel)
        return FALSE;

    CdosGlobal *global = cdos_global_get();
    MetaScreen *screen = cdos_global_get_screen(global);
    CdosLayoutManager *data = cdos_layout_manager_get_default ();

    cdos_layout_manager_on_window_restacked(screen,data);

    //gfloat height = clutter_actor_get_height(self);
    gfloat height = 40;
    guint animationTime = AUTOHIDE_ANIMATION_TIME;

    const MetaRectangle *monitor = cdos_layout_manager_get_primary_monitor(data);

    float y = priv->bottomPosition ? (monitor->y + monitor->height - 1) : (monitor->y - height + 1);

    ClutterActor *actor_parent = clutter_actor_get_parent((ClutterActor *)self);
    cdos_transition_simple(actor_parent,
                           CLUTTER_EASE_OUT_QUAD,
                           animationTime,
                           "y",y,
                           // "opacity",TRANS_PROP_UCHAR,255,
                           NULL);
    priv->hidden = TRUE;
    g_print ("hide panel: %s %d\n", __func__, __LINE__);
    return FALSE;
}


static gboolean
cdos_panel_show_panel(CdosPanel *self)
{
    CdosPanelPrivate *priv;
    priv = self->priv;

    if(priv->disabled)
        return FALSE;
    
    if(!priv->hidden)
        return FALSE;

    CdosGlobal *global = cdos_global_get();
    MetaScreen *screen = cdos_global_get_screen(global);
    CdosLayoutManager *data = cdos_layout_manager_get_default ();

    cdos_layout_manager_on_window_restacked(screen,data);

    //gfloat height = clutter_actor_get_height(self);
    gfloat height = 40;
    gfloat animationTime = AUTOHIDE_ANIMATION_TIME;

    const MetaRectangle *monitor = cdos_layout_manager_get_primary_monitor(data);

    float y = priv->bottomPosition ? (monitor->y + monitor->height - height) : (monitor->y);
    
    ClutterActor *actor_parent = clutter_actor_get_parent((ClutterActor *)self);
    cdos_transition_simple (actor_parent,
                            CLUTTER_EASE_OUT_QUAD,
                            animationTime + 50,
                            "y", y,
                            NULL);

    g_print ("show panel: %s %d\n", __func__, __LINE__);
    priv->hidden = FALSE;
    return FALSE;
}

static void 
cdos_panel_clear_timers(CdosPanel *self)
{
    CdosPanelPrivate *priv;
    priv = self->priv;

    if(priv->showTimer)
        g_source_remove(priv->showTimer);
    
    if(priv->hideTimer)
        g_source_remove(priv->hideTimer);

}


static void cdos_panel_enter_panel(CdosPanel *self)
{
    CdosPanelPrivate *priv;
    priv = self->priv;

    priv->isMouseOverPanel = TRUE;

    cdos_panel_clear_timers(self);

    if(priv->showDelay > 0)
        priv->showTimer = g_timeout_add(priv->showDelay,(GSourceFunc)cdos_panel_show_panel,self);
    else
        cdos_panel_show_panel(self);
}

static void
cdos_panel_leave_panel(CdosPanel *self)
{
    CdosPanelPrivate *priv;
    priv = self->priv;
    priv->isMouseOverPanel =FALSE;
    cdos_panel_clear_timers(self);

    if(priv->hideDelay > 0 && ! priv->disabled)
    {
        priv->hideTimer = g_timeout_add(priv->hideDelay,(GSourceFunc)cdos_panel_hide_panel,self);
    }else {
        cdos_panel_hide_panel(self,TRUE);
    }

}


/**
 * cdos_panel_enable:
 * @self: #CdosPanel 类的对象
 *
 * 显示面板
 */
void 
cdos_panel_enable(CdosPanel  *self)
{
    CdosPanelPrivate *priv;
    g_return_if_fail(CDOS_IS_PANEL(self));

    priv = self->priv;
    
    priv ->disabled = FALSE;
    
    cdos_panel_show_panel(self);
}


/**
 * cdos_panel_disable:
 * @self: #CdosPanel 类的对象
 *
 * 隐藏面板
 */
void 
cdos_panel_disable(CdosPanel *self)
{
    CdosPanelPrivate *priv;
    g_return_if_fail(CDOS_IS_PANEL(self));

    priv = self->priv;

    priv ->disabled = TRUE;
    
    cdos_panel_leave_panel(self);

}

/**
 * cdos_panel_set_bottom_position:
 * @self: #CdosPanel的一个类
 * @bottom: 是否把panel放到底部
 *
 * 设置panel的位置，是否把面板放到底部
 */
void
cdos_panel_set_bottom_position (CdosPanel    *self,
                    gboolean bottom)
{
  CdosPanelPrivate *priv;

  g_return_if_fail (CDOS_IS_PANEL (self));

  // printf("[TEST]: bottom is %d\n", bottom);
  priv = self->priv;

  priv->bottomPosition = bottom;
}

/**
 * cdos_panel_set_hideable:
 * @self: #CdosPanel类的一个对象
 * @hideable: 是否隐藏panel的boolean值
 *
 * 设置隐藏panel的boolean值
 *
 */
void
cdos_panel_set_hideable (CdosPanel    *self,
                    gboolean hideable)
{
  CdosPanelPrivate *priv;

  g_return_if_fail (CDOS_IS_PANEL (self));

  priv = self->priv;

  priv->_hideable = hideable;
}


static void cdos_panel_set_primary (CdosPanel    *self,
			gboolean is_primary)
{
  CdosPanelPrivate *priv;
  priv = self->priv;
  priv->is_primary = is_primary;
}

static const gchar *
cdos_panel_get_text (CdosPanel *self)
{
  return self->priv->text;
}


/**
 * cdos_panel_get_bottom_position:
 * @self: #CdosPanel的一个类
 *
 * 获取是否把面板放到底部的boolean值
 *
 * Returns: 获取是否把panel放到底部的boolean值 
 */
gboolean
cdos_panel_get_bottom_position (CdosPanel *self)
{
  g_return_val_if_fail (CDOS_IS_PANEL (self), FALSE);

  return self->priv->bottomPosition;
}

static gboolean
cdos_panel_isHideable (CdosPanel *self)
{
  return self->priv->_hideable;
}

/**
 * cdos_panel_new:
 * @bottom_position: 是否把panel放到底部的boolean值
 * @is_primary: TRUE
 *
 * 新建一个面板
 *
 * Returns: panel
 */
CdosPanel *
cdos_panel_new (gboolean bottom_position, gboolean is_primary)
{
  CdosPanel *panel;

  panel = g_object_new (CDOS_TYPE_PANEL, 
			"bottom-position", bottom_position,
			"is-primary", is_primary,
			NULL); 
  CdosPanelPrivate *priv = CDOS_PANEL_GET_PRIVATE (panel);

  priv->context_menu = panel_get_context_menu (panel, ST_SIDE_BOTTOM);
  priv->menu_manager = cdos_popup_menu_manager_new (CLUTTER_ACTOR(panel));
  cdos_popup_menu_manager_add_menu (priv->menu_manager, CDOS_POPUP_MENU_BASE (priv->context_menu), -1);

  CdosBoxPointer *boxp = cdos_popup_menu_get_box_pointer (priv->context_menu);
  CdosGenericContainer *container = cdos_box_pointer_get_container (boxp);
  g_signal_connect (container, "allocate", G_CALLBACK (container_allocate), boxp);
//  g_signal_connect (CLUTTER_ACTOR(panel), "button-press-event", G_CALLBACK (on_button_press), panel);

  return panel;
}

void cdos_panel_set_height(CdosPanel *cdos_panel,gfloat height){
    g_return_if_fail(CDOS_IS_PANEL(cdos_panel));
    if(height >= 0.f){
        clutter_actor_set_height (CLUTTER_ACTOR(cdos_panel), height);
    }
}

gfloat cdos_panel_get_height(CdosPanel *cdos_panel){
    g_return_val_if_fail(CDOS_IS_PANEL(cdos_panel), 0);
    return clutter_actor_get_height (CLUTTER_ACTOR(cdos_panel));
}
