#include "cdos-popup-alternating-menu-item.h"
#include <clutter/clutter.h>
#include "cdos-global.h"
#include "cdos-generic-container.h"
#include <glib.h>
#include "st.h"

/**
 * SECTION: cdos-popup-alternating-menu-item.c
 * @Title: CdosPopupAlternatingMenuItem
 * @short_description: 继承自CdosPopupBaseMenuItem, 提供了一个可选择的text
 * 
 *
 * CdosPopupAlternatingMenuItem 提供一个可选择的text文本。
 */

static void 
cdos_popup_alternating_menu_item_init(CdosPopupAlternatingMenuItem *item);

static void 
cdos_popup_alternating_menu_item_class_init(CdosPopupAlternatingMenuItemClass *klass);

static void 
cdos_popup_alternating_menu_item_finalize(GObject *gobject);

static void
cdos_popup_alternating_menu_item_dispose(GObject *gobject);

static void 
cdos_popup_alternating_menu_item_on_mapped(CdosPopupAlternatingMenuItem *item);

static void 
cdos_popup_alternating_menu_item_set_status(CdosPopupAlternatingMenuItem *item, 
                                            gboolean        state);

static void 
cdos_popup_alternating_menu_item_update_state_from_modifiers(CdosPopupAlternatingMenuItem *item);

static gboolean
cdos_popup_alternating_menu_item_on_captured_event(ClutterActor                 *actor,
                                                   ClutterEvent                 *event,
                                                   CdosPopupAlternatingMenuItem *item);

static void 
cdos_popup_alternating_menu_item_update_label(CdosPopupAlternatingMenuItem *item);

static gboolean 
cdos_popup_alternating_menu_item_can_alternate(CdosPopupAlternatingMenuItem *item);

#define CDOS_POPUP_ALTERNATING_MENU_ITEM_GET_PRIVATE(obj) 	\
	(G_TYPE_INSTANCE_GET_PRIVATE((obj), CDOS_TYPE_POPUP_ALTERNATING_MENU_ITEM, CdosPopupAlternatingMenuItemPrivate))

enum {
	DEFAULT,
	ALTERNATIVE,
};

G_DEFINE_TYPE (CdosPopupAlternatingMenuItem, cdos_popup_alternating_menu_item, CDOS_TYPE_POPUP_BASE_MENU_ITEM);

struct _CdosPopupAlternatingMenuItemPrivate
{
	gchar 		*text;
	gchar 		*alternateText;
	StLabel         *label;	
	gboolean 	 state;
	gint 		 capturedEventId;
};

static void 
cdos_popup_alternating_menu_item_init(CdosPopupAlternatingMenuItem *item)
{
	CdosGenericContainer *actor = NULL;
	item->priv = CDOS_POPUP_ALTERNATING_MENU_ITEM_GET_PRIVATE(item);
	
	item->priv->text = NULL;
	item->priv->alternateText = NULL;
	item->priv->label =ST_LABEL(st_label_new(" "));
	item->priv->state = DEFAULT;
	item->priv->capturedEventId = 0;

	ActorChild 	params;
	cdos_popup_base_menu_item_get_actor_child(&params);

	g_object_get(CDOS_POPUP_BASE_MENU_ITEM(item),  "actor", &actor, NULL); 

	st_widget_add_style_class_name(ST_WIDGET(actor), "popup-alternating-menu-item");

	cdos_popup_base_menu_item_add_actor(CDOS_POPUP_BASE_MENU_ITEM(actor),
		CLUTTER_ACTOR(item->priv->label), 
		&params);

	g_signal_connect((ClutterActor)actor, "notify::mapped",
		G_CALLBACK(cdos_popup_alternating_menu_item_on_mapped), item);
}

/**
 * cdos_popup_alternating_menu_item_set_text:
 * @item: A #CdosPopupAlternatingMenuItem
 * @text:  CdosPopupAlternatingMenuItem显示的text
 * @alternateText: 可选text
 *
 * 设置item的text和可选text
 *
 */
void 
cdos_popup_alternating_menu_item_set_text(CdosPopupAlternatingMenuItem  *item,
					const gchar *text, 
					const gchar *alternateText)
{
	CdosPopupAlternatingMenuItemPrivate *priv;
	priv = item->priv;

	if (priv->text) {
		g_free(priv->text);
	}
	priv->text = g_strdup(text);
	
	if (priv->alternateText) {
		g_free(priv->alternateText);
	}
	priv->alternateText = g_strdup(alternateText);

	st_label_set_text(priv->label, priv->text);
}

static void 
cdos_popup_alternating_menu_item_class_init(CdosPopupAlternatingMenuItemClass *klass)
{
	GObjectClass *gobject_class = G_OBJECT_CLASS(klass);

	g_type_class_add_private(klass, sizeof(CdosPopupAlternatingMenuItemPrivate));

	gobject_class->finalize = cdos_popup_alternating_menu_item_finalize;
	gobject_class->dispose = cdos_popup_alternating_menu_item_dispose;
}

static void 
cdos_popup_alternating_menu_item_finalize(GObject *gobject)
{
	CdosPopupAlternatingMenuItemPrivate *priv = CDOS_POPUP_ALTERNATING_MENU_ITEM(gobject)->priv;
	if (priv->text) {
		g_free(priv->text);
	}
	if (priv->alternateText) {
		g_free(priv->alternateText);
	}
	G_OBJECT_CLASS(cdos_popup_alternating_menu_item_parent_class)->finalize(gobject);
}
static void
cdos_popup_alternating_menu_item_dispose(GObject *gobject){
	CdosPopupAlternatingMenuItemPrivate *priv = CDOS_POPUP_ALTERNATING_MENU_ITEM(gobject)->priv;
	g_clear_object(&priv->label);

	G_OBJECT_CLASS(cdos_popup_alternating_menu_item_parent_class)->dispose(gobject);
}
static void 
cdos_popup_alternating_menu_item_on_mapped(ClutterActor *actor,GParamSpec *pspec,CdosPopupAlternatingMenuItem *item)
{

	CdosPopupAlternatingMenuItemPrivate  *priv;
	CdosGlobal *global = cdos_global_get();
	ClutterStage  *stage = cdos_global_get_stage(global);
	priv = item->priv;
	g_return_if_fail(CDOS_IS_POPUP_ALTERNATING_MENU_ITEM(item));
	gboolean mapped = CLUTTER_ACTOR_IS_MAPPED(CLUTTER_ACTOR(actor));
	if (mapped) {
		priv->capturedEventId = g_signal_connect(stage, "captured-event",
						G_CALLBACK(cdos_popup_alternating_menu_item_on_captured_event), item);

		cdos_popup_alternating_menu_item_update_state_from_modifiers(item);

	} else {
		if (priv->capturedEventId != 0) {
			g_signal_handler_disconnect(item, priv->capturedEventId);
			priv->capturedEventId = 0;
		}
	}
}

static void 
cdos_popup_alternating_menu_item_set_status(CdosPopupAlternatingMenuItem *item, 
				            gboolean 	state)
{
	g_return_if_fail(CDOS_IS_POPUP_ALTERNATING_MENU_ITEM(item));
	CdosPopupAlternatingMenuItemPrivate *priv = item->priv;
	if (priv->state != state) {
		if (state == ALTERNATIVE && !cdos_popup_alternating_menu_item_can_alternate(item))
			return;
		priv->state = state;
		cdos_popup_alternating_menu_item_update_label(item);
	}
}

static void 
cdos_popup_alternating_menu_item_update_state_from_modifiers(CdosPopupAlternatingMenuItem *item)
{
	CdosGlobal   		*global;
	int 			x;
	int 			y;
	ClutterModifierType     *mods;
	gboolean 		state;	

	g_return_if_fail(CDOS_IS_POPUP_ALTERNATING_MENU_ITEM(item));

	global = cdos_global_get();
	cdos_global_get_pointer(global, &x, &y, mods);

	if ((*mods & CLUTTER_MOD1_MASK) == 0) {
		state = DEFAULT;
	} else {
		state = ALTERNATIVE;
	}

	cdos_popup_alternating_menu_item_set_status(item, state);

}

static gboolean
cdos_popup_alternating_menu_item_on_captured_event(ClutterActor *actor,
					ClutterEvent *event,
					CdosPopupAlternatingMenuItem *item)
{

	ClutterEventType type = clutter_event_type(event);
	if (type != CLUTTER_KEY_PRESS && type != CLUTTER_KEY_RELEASE) 
		return FALSE;

	guint key =  clutter_event_get_key_symbol(event);

	if (key == CLUTTER_KEY_Alt_L || key == CLUTTER_KEY_Alt_R)
		cdos_popup_alternating_menu_item_update_state_from_modifiers(item);

	return FALSE;
}

static void 
cdos_popup_alternating_menu_item_update_label(CdosPopupAlternatingMenuItem *item)
{

	CdosGenericContainer *actor = NULL;

	CdosPopupAlternatingMenuItemPrivate *priv = item->priv;
	g_object_get(CDOS_POPUP_BASE_MENU_ITEM(item),  "actor", &actor, NULL); 

	if (priv->state == ALTERNATIVE) {
		st_widget_add_style_class_name(ST_WIDGET(actor), "alternating");
		st_label_set_text(priv->label, priv->alternateText);
	} else {
		st_widget_remove_style_class_name(ST_WIDGET(actor), "alternating");
		st_label_set_text(priv->label, priv->text);
	}
}

static gboolean 
cdos_popup_alternating_menu_item_can_alternate(CdosPopupAlternatingMenuItem *item)
{
	g_return_if_fail(CDOS_IS_POPUP_ALTERNATING_MENU_ITEM(item));

	CdosPopupAlternatingMenuItemPrivate *priv= item->priv;
	if ( (priv->state == DEFAULT) && (!priv->alternateText)) {
		return FALSE;
	}

	return TRUE;
}

/**
 * cdos_popup_alternating_menu_item_new:
 * @text: 显示的text 
 * @alternateText: 可选的text
 * @params: 初始化的参数
 * 
 * 新建一个 #CdosPopupAlternatingMenuItem 类的实例。
 *
 * Returns: 返回一个 #CdosPopupAlternatingMenuItem 的实例。
 */
CdosPopupAlternatingMenuItem *
cdos_popup_alternating_menu_item_new(const gchar *text, 
			const gchar *alternateText,
			PopupBaseMenuItemInit *params)
{
	CdosPopupAlternatingMenuItem  *gobject;
	 gobject = g_object_new(CDOS_TYPE_POPUP_ALTERNATING_MENU_ITEM, NULL);

	 cdos_popup_base_menu_item_set_params(gobject, params);
	 
	 cdos_popup_alternating_menu_item_set_text(gobject, text, alternateText);
}

/**
 * cdos_popup_alternating_menu_item_update_text:
 * @item: A #CdosPopupAlternatingMenuItem
 * @text:  CdosPopupAlternatingMenuItem显示的text
 * @alternateText: 可选text
 *
 * 更新item的text和可选text
 *
 */
void 
cdos_popup_alternating_menu_item_update_text(CdosPopupAlternatingMenuItem *item,
				 gchar *text, 
				 gchar *alternateText)
{
	g_return_if_fail(CDOS_IS_POPUP_ALTERNATING_MENU_ITEM(item));

	CdosPopupAlternatingMenuItemPrivate *priv = item->priv;

	if (priv->text) {
		g_free(priv->text);
	}

	priv->text = g_strdup(text);

	if (priv->alternateText) {
		g_free(priv->alternateText);
	}

	priv->alternateText = g_strdup(alternateText);

	if (!cdos_popup_alternating_menu_item_can_alternate(item))    
		cdos_popup_alternating_menu_item_set_status(item, FALSE);	

	cdos_popup_alternating_menu_item_update_label(item);	
}

