#include "cdos-popup-base-menu-item.h"
#include <math.h>
#include <glib.h>
#include <clutter/clutter.h>
#include "cdos-start.h"

/**
 * SECTION: cdos-popup-base-menu-item.c
 * @Title: CdosPopupBaseMenuItem
 * @short_description: CdosPopupBaseMenuItem的作用是为menu提供一个弹出菜单里的选项，
 * 这个控件必须放在Menu里。这个控件为我们提供了一个便于加入menu的item。
 *
 * CdosPopupBaseMenuItem 提供一个可以和menu配套使用的控件。其其实就是由box构成。
 */

static void 
cdos_popup_base_menu_item_finalize(GObject *gobject);
static void
cdos_popup_base_menu_item_dispose(GObject *gobject);
static void  
cdos_popup_base_menu_item_init(CdosPopupBaseMenuItem  *item) ;

static void
cdos_popup_base_menu_item_set_property(GObject      *gobject,
                                       guint         prop_id,
                                       const GValue *value,    
                                       GParamSpec   *pspec);

static void 
cdos_popup_base_menu_item_get_property(GObject    *gobject,
                                       guint       prop_id,
                                       GValue     *value,
                                       GParamSpec *pspec);

static void 
cdos_popup_base_menu_item_class_init(CdosPopupBaseMenuItemClass *klass);

static void 
cdos_popup_base_menu_item_on_style_changed(CdosGenericContainer  *actor, 
                                           CdosPopupBaseMenuItem *item);

static gboolean 
cdos_popup_base_menu_item_on_button_release_event(CdosGenericContainer  *actor, 
                                                  ClutterEvent          *event,
                                                  CdosPopupBaseMenuItem *item);

static gboolean
cdos_popup_base_menu_item_on_key_press_event(ClutterActor          *actor,
                                             ClutterEvent          *event, 
                                             CdosPopupBaseMenuItem *item)
{
    CdosPopupBaseMenuItemClass *klass = CDOS_POPUP_BASE_MENU_ITEM_GET_CLASS (item);
    return klass->key_press_event (actor, event, item);
}

static void 
cdos_popup_base_menu_item_on_key_focus_in(ClutterActor          *actor, 
                                          CdosPopupBaseMenuItem *item);

static void 
cdos_popup_base_menu_item_on_key_focus_out(ClutterActor          *actor,
                                           CdosPopupBaseMenuItem *item);

static void 
cdos_popup_base_menu_item_on_hover_changed(ClutterActor          *actor,
		                           GParamSpec            *spec,	 
                                           CdosPopupBaseMenuItem *item);

static void 
cdos_popup_base_menu_item_call_back(ClutterActor          *actor,
                                    CdosPopupBaseMenuItem *item);

static void 
cdos_popup_base_menu_item_remove_child(CdosPopupBaseMenuItem *item,
				       ClutterActor          *child);

static void 
cdos_popup_base_menu_item_on_repaint_dot(StDrawingArea         *area,
                                         CdosPopupBaseMenuItem *item);

static void 
cdos_popup_base_menu_item_get_perferred_width(ClutterActor *actor,
                                        gfloat          for_height,
                                        CdosGenericContainerAllocation *alloc,
                                        CdosPopupBaseMenuItem     *item);

static void 
cdos_popup_base_menu_item_get_preferred_height(ClutterActor *actor,
                                               gfloat for_width,
                                               CdosGenericContainerAllocation *alloc,
                                               CdosPopupBaseMenuItem *item);

static void 
cdos_popup_base_menu_item_allocate(ClutterActor *actor,
                            const ClutterActorBox *box,
                             ClutterAllocationFlags flags,
                             CdosPopupBaseMenuItem *item);


struct  _CdosPopupBaseMenuItemPrivate
{
		CdosGenericContainer 	 	*actor;
		GArray 			         	*children;
		StDrawingArea 		      	*dot;
		gint                         spacing;
		gboolean 	        		 active;
		gboolean 	          		 activatable;
		gboolean 	          		 sensitive;
		gboolean 	          		 focusOnHover;
		gint 		          		 margin_left_label;
		gint 		          		 margin_left_label_icon;
		gulong  					 button_release_event_id;
		gulong 						 key_press_event_id;
		gulong 						 notify_hover_id;
		gulong 						 key_focus_in_id;
		gulong   					 key_focus_out_id;
		gboolean 					 keep_menu;
        gulong                  actor_destroy_id;
};

#define CDOS_POPUP_BASE_MENU_ITEM_GET_PRIVATE(obj) 	(G_TYPE_INSTANCE_GET_PRIVATE((obj), CDOS_TYPE_POPUP_BASE_MENU_ITEM, CdosPopupBaseMenuItemPrivate))

G_DEFINE_TYPE(CdosPopupBaseMenuItem, cdos_popup_base_menu_item, G_TYPE_OBJECT);

enum
{
	PROP_0,

	PROP_ACTOR,
	PROP_DOT,
	PROP_SPACING,
	PROP_ACTIVE,
	PROP_ACTIVETABLE,
	PROP_SENSITIVE,
	PROP_FOCUSONHOVER,
	PROP_MARGIN_LEFT_LABEL,
	PROP_MARGIN_LEFT_LABEL_ICON
};


enum
{
	ACTIVATE,
	ACTIVE_CHANGED,
	SENSITIVE_CHANGED,
  	DESTROY,
	VALUE_CHANGED,
	DRAG_ENG,
    LAST_SIGNAL
};
static guint cdos_popup_base_menu_item_signals[LAST_SIGNAL] =  { 0, };

static void
cdos_popup_base_menu_item_dispose(GObject *gobject)
{
	CdosPopupBaseMenuItem *self = CDOS_POPUP_BASE_MENU_ITEM(gobject);
	CdosPopupBaseMenuItemPrivate *priv = self->priv;

    if (priv->actor_destroy_id)
    {
        g_signal_handler_disconnect (priv->actor, priv->actor_destroy_id);
        priv->actor_destroy_id = 0;
    }
    clutter_actor_destroy ((ClutterActor *)priv->actor);

	G_OBJECT_CLASS( cdos_popup_base_menu_item_parent_class )->dispose( gobject );
}

static void 
cdos_popup_base_menu_item_finalize(GObject *gobject)
{
	CdosPopupBaseMenuItemPrivate *priv = CDOS_POPUP_BASE_MENU_ITEM(gobject)->priv;
	if (priv->children) {
        g_array_unref(priv->children);
	}

    G_OBJECT_CLASS (cdos_popup_base_menu_item_parent_class )->finalize( gobject );
}

static void 
cdos_popup_base_menu_item_on_actor_destroy (ClutterActor *actor,
                                            CdosPopupBaseMenuItem *self)
{
    g_object_unref (self);
}

static void  
cdos_popup_base_menu_item_init(CdosPopupBaseMenuItem  *item) 
{
	CdosPopupBaseMenuItemPrivate *priv; 
	priv = item->priv =  CDOS_POPUP_BASE_MENU_ITEM_GET_PRIVATE(item);

	priv->actor = g_object_new(CDOS_TYPE_GENERIC_CONTAINER,
				"style-class", "popup-menu-item",
				"reactive", TRUE,
				"track-hover", TRUE,
				"can-focus", TRUE,
				NULL
				);
   	item->actor = (ClutterActor *)priv->actor;
    g_object_set_data ((GObject *)item->actor, "_delegate", item);
	g_signal_connect(priv->actor, "get-preferred-width",
		G_CALLBACK(cdos_popup_base_menu_item_get_perferred_width), item);

	g_signal_connect(priv->actor, "get-preferred-height",
		 G_CALLBACK(cdos_popup_base_menu_item_get_preferred_height), item);

	g_signal_connect(priv->actor, "allocate",
		G_CALLBACK(cdos_popup_base_menu_item_allocate), item);

	g_signal_connect(priv->actor, "style-changed",
		G_CALLBACK(cdos_popup_base_menu_item_on_style_changed), item);

	priv->children = g_array_new(TRUE, TRUE, sizeof(ActorChild));
	priv->dot = NULL;
	priv->spacing = 0;
	priv->active = FALSE;
	priv->sensitive = TRUE;	
	priv->activatable = TRUE;
	priv->focusOnHover = TRUE;
	priv->keep_menu = FALSE;
	

	priv->margin_left_label = 10;
	priv->margin_left_label_icon = 10;

	//cdos_popup_base_menu_item_set_sensitive(item, TRUE);
   	gboolean sensitive = TRUE;
	priv->sensitive = sensitive;
	clutter_actor_set_reactive((ClutterActor *)priv->actor, sensitive);
	st_widget_set_can_focus((StWidget *)priv->actor, sensitive);
   	st_widget_remove_style_pseudo_class((StWidget *)priv->actor, "insensitive");

	priv->button_release_event_id = g_signal_connect(priv->actor, 
			"button-release-event",
			G_CALLBACK(cdos_popup_base_menu_item_on_button_release_event),
			item);

	priv->key_press_event_id = g_signal_connect(priv->actor,
			"key-press-event",
			G_CALLBACK(cdos_popup_base_menu_item_on_key_press_event),
			item);	

	priv->notify_hover_id = g_signal_connect(priv->actor, 
			"notify::hover",
			G_CALLBACK(cdos_popup_base_menu_item_on_hover_changed), 
			item);
    
    priv->actor_destroy_id = g_signal_connect (priv->actor,
                                "destroy",
                                G_CALLBACK(cdos_popup_base_menu_item_on_actor_destroy),
                                item);
// linjj 注释开始
// 这两个信号先注释掉，以解决以下问题：
//     右键点击菜单中的favorite按钮，然后点击右键菜单以外的地方，favorite按钮
//     一直处于高亮状态。
	priv->key_focus_in_id = g_signal_connect(priv->actor,
			"key-focus-in",
			G_CALLBACK(cdos_popup_base_menu_item_on_key_focus_in),
			item);

	priv->key_focus_out_id = g_signal_connect(priv->actor,
			"key-focus-out",
			G_CALLBACK(cdos_popup_base_menu_item_on_key_focus_out),
			item);
// linjj 注释结束
}

static void
cdos_popup_base_menu_item_set_property(GObject 	*gobject,
				guint 	prop_id,
				const GValue *value,	
				GParamSpec *pspec)
{
	CdosPopupBaseMenuItem *item = CDOS_POPUP_BASE_MENU_ITEM(gobject);

	switch(prop_id) 
	{
		case PROP_ACTOR:
		cdos_popup_base_menu_item_set_actor(item,
					g_value_get_object(value));
			break;
		case PROP_DOT:
		cdos_popup_base_menu_item_set_dot(item,
					g_value_get_object(value));
			break;
		case  PROP_SPACING:
		cdos_popup_base_menu_item_set_spacing(item,
					g_value_get_int(value));
			break;
		case PROP_ACTIVE:
		cdos_popup_base_menu_item_set_active(item,
					g_value_get_boolean(value));
			break;
		case PROP_ACTIVETABLE:
		cdos_popup_base_menu_item_set_activatable(item,
					g_value_get_boolean(value));
			break;
		case PROP_SENSITIVE:
		cdos_popup_base_menu_item_set_sensitive(item,
					g_value_get_boolean(value));
			break;
		case PROP_FOCUSONHOVER:
		cdos_popup_base_menu_item_set_focusOnHover(item,
					g_value_get_boolean(value));
			break;
		case PROP_MARGIN_LEFT_LABEL:
		cdos_popup_base_menu_item_set_margin_left_label(item,
					g_value_get_int(value));
			break;
		case PROP_MARGIN_LEFT_LABEL_ICON:
		cdos_popup_base_menu_item_set_margin_left_label_icon(item, 
					g_value_get_int(value));
			break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);								
	}
}

static void 
cdos_popup_base_menu_item_get_property(GObject    *gobject,
				guint 	prop_id,
				GValue 	 *value,
				GParamSpec   *pspec)
{
	CdosPopupBaseMenuItemPrivate *priv = CDOS_POPUP_BASE_MENU_ITEM(gobject)->priv;

	switch(prop_id) 
	{
		case PROP_ACTOR:
			g_value_set_object(value, priv->actor);
			break;
		case PROP_DOT:
			g_value_set_object(value, priv->dot);
			break;
		case  PROP_SPACING:
			g_value_set_int(value, priv->spacing);
			break;
		case PROP_ACTIVE:
			g_value_set_boolean(value, priv->active);
			break;
		case PROP_ACTIVETABLE:
			g_value_set_boolean(value, priv->activatable);
			break;
		case PROP_SENSITIVE:
			g_value_set_boolean(value, priv->sensitive);
			break;
		case PROP_FOCUSONHOVER:
			g_value_set_boolean(value, priv->focusOnHover);
			break;
		case PROP_MARGIN_LEFT_LABEL:
			g_value_set_int(value, priv->margin_left_label);
			break;
		case PROP_MARGIN_LEFT_LABEL_ICON:
			g_value_set_int(value, priv->margin_left_label_icon);
			break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
	}
}

static gboolean
cdos_popup_base_menu_item_on_key_press_event_real (ClutterActor *actor,
					ClutterEvent *event, 
					CdosPopupBaseMenuItem *item)
{ 
	guint 	symbol = clutter_event_get_key_symbol(event);
	if (symbol == CLUTTER_KEY_space || symbol == CLUTTER_KEY_Return) {
		CdosPopupBaseMenuItemPrivate *priv = item->priv;
		cdos_popup_base_menu_item_activate(item, event, priv->keep_menu);
		return TRUE;
	}
	return FALSE;
} 

static void 
cdos_popup_base_menu_item_class_init(CdosPopupBaseMenuItemClass *klass)
{
	GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
	GParamSpec *pspec;

	g_type_class_add_private(klass, sizeof(CdosPopupBaseMenuItemPrivate));

	gobject_class->set_property = cdos_popup_base_menu_item_set_property;
	gobject_class->get_property = cdos_popup_base_menu_item_get_property;
	gobject_class->finalize = cdos_popup_base_menu_item_finalize;
	gobject_class->dispose = cdos_popup_base_menu_item_dispose;

    klass->key_press_event = cdos_popup_base_menu_item_on_key_press_event_real;

	pspec = g_param_spec_object("actor",
				"Actor",
				"The conatiner of the item",
				CLUTTER_TYPE_ACTOR,
				G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class, PROP_ACTOR, pspec);

	pspec = g_param_spec_object("dot",
				"Dot",
				"The drawing area of item",
				CLUTTER_TYPE_ACTOR,
				G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class, PROP_DOT, pspec);

	pspec =g_param_spec_int("spacing", 
			"Spacing",
			"The spacing of item",
			-1, G_MAXINT, -1,
			G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class, PROP_SPACING, pspec);

	pspec = g_param_spec_boolean("active",
				"Active",
				"The active of item",
				TRUE,
				G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class, PROP_ACTIVE, pspec);

	pspec = g_param_spec_boolean("activatable",
				"Activatable",
				"The activatable of item",
				TRUE,
				G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class, PROP_ACTIVETABLE, pspec);

	pspec = g_param_spec_boolean("sensitive",
				"sensitive",
				"The sensitive of item",
				TRUE,
				G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class, PROP_SENSITIVE, pspec);

	pspec = g_param_spec_boolean("focusOnHover",
				"FocusOnHover",
				"Weather the item onhover",
				TRUE,
				G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class, PROP_FOCUSONHOVER, pspec);

	pspec = g_param_spec_int("margin_left_label",
			"Margin_left_label",
			"The margin of item label",
			-1, G_MAXINT, -1,
			G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class, PROP_MARGIN_LEFT_LABEL, pspec);

	pspec = g_param_spec_int("margin_left_label_icon",
			"Margin_left_label_icon",
			"The margin of item label icon",
			-1, G_MAXINT, -1,
			G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class, PROP_MARGIN_LEFT_LABEL_ICON, pspec);

//signals
	cdos_popup_base_menu_item_signals[ACTIVATE] = g_signal_new (
			"activate",
			G_TYPE_FROM_CLASS(klass),
			G_SIGNAL_RUN_LAST,
			G_STRUCT_OFFSET(CdosPopupBaseMenuItemClass, activate),
			NULL,
			NULL, 
			_st_marshal_VOID__POINTER_BOOLEAN,
			G_TYPE_NONE,
		   	2,
			G_TYPE_POINTER,
			G_TYPE_BOOLEAN);

	cdos_popup_base_menu_item_signals[ACTIVE_CHANGED] = g_signal_new (
			"active-changed",
			G_TYPE_FROM_CLASS(klass),
			G_SIGNAL_RUN_LAST,
			G_STRUCT_OFFSET(CdosPopupBaseMenuItemClass, active_changed),
			NULL,
			NULL,
			g_cclosure_marshal_VOID__BOOLEAN,
			G_TYPE_NONE,
		   	1,
			G_TYPE_BOOLEAN);

	cdos_popup_base_menu_item_signals[SENSITIVE_CHANGED] = g_signal_new (
			"sensitive-changed",
			G_TYPE_FROM_CLASS(klass),
			G_SIGNAL_RUN_LAST,
			G_STRUCT_OFFSET(CdosPopupBaseMenuItemClass, sensitive_changed),
			NULL,
			NULL, 
			g_cclosure_marshal_VOID__BOOLEAN,
			G_TYPE_NONE,
			1,
			G_TYPE_BOOLEAN);

	cdos_popup_base_menu_item_signals[DESTROY] = g_signal_new (
			"destroy",
			G_TYPE_FROM_CLASS(klass),
			G_SIGNAL_RUN_LAST,
			G_STRUCT_OFFSET(CdosPopupBaseMenuItemClass, destroy),
			NULL,
			NULL,
			g_cclosure_marshal_VOID__VOID,
			G_TYPE_NONE,
			0);

	cdos_popup_base_menu_item_signals[VALUE_CHANGED] = g_signal_new (
			"value-changed",
			G_TYPE_FROM_CLASS(klass),
			G_SIGNAL_RUN_LAST,
			G_STRUCT_OFFSET(CdosPopupBaseMenuItemClass, value_changed),
			NULL,
			NULL,
			g_cclosure_marshal_VOID__FLOAT,
			G_TYPE_NONE,
			1,
			G_TYPE_FLOAT);

	cdos_popup_base_menu_item_signals[DRAG_ENG] = g_signal_new (
			"drag-end",
			G_TYPE_FROM_CLASS(klass),
			G_SIGNAL_RUN_LAST,
			G_STRUCT_OFFSET(CdosPopupBaseMenuItemClass, drag_end),
			NULL,
			NULL,
			g_cclosure_marshal_VOID__OBJECT,
			G_TYPE_NONE,
			0);

}

/**
 * cdos_popup_base_menu_item_new：
 * 
 * 新建一个 #CdosPopupBaseMenuItem 类的实例。
 *
 * Returns value: 返回一个 #CdosPopupBaseMenuItem 的实例。
 */
CdosPopupBaseMenuItem *
cdos_popup_base_menu_item_new(void)
{
	return g_object_new(CDOS_TYPE_POPUP_BASE_MENU_ITEM, NULL);
}

/**
 * cdos_popup_base_menu_item_new_with_params:
 * @params: 初始化类需要的基本信息 
 * 
 * 新建一个 #CdosPopupBaseMenuItem 类的实例。
 *
 * Return value: 返回一个 #CdosPopupBaseMenuItem 的实例。
 */
CdosPopupBaseMenuItem *
cdos_popup_base_menu_item_new_with_params(PopupBaseMenuItemInit *paramso)
{
	CdosPopupBaseMenuItem *item;
	item = g_object_new(CDOS_TYPE_POPUP_BASE_MENU_ITEM, NULL);
    PopupBaseMenuItemInit *params = g_slice_copy (sizeof (PopupBaseMenuItemInit), paramso);
	cdos_popup_base_menu_item_set_params(item, params);
    g_slice_free (PopupBaseMenuItemInit, params);
	return item;

}

/**
 * cdos_popup_base_menu_item_set_params:
 * @item: 需要设置类的属性
 * @params: 设置类的基本信息 
 * 
 */
void 
cdos_popup_base_menu_item_set_params(CdosPopupBaseMenuItem *item, 
				PopupBaseMenuItemInit *params)
{
	CdosPopupBaseMenuItemPrivate 		*priv;
	g_return_if_fail(CDOS_IS_POPUP_BASE_MENU_ITEM(item));

	if (NULL == params)
        return;

	priv = item->priv;

	if ((params->reactive == TRUE) && 
		(params->activate == TRUE) && 
		(params->hover == TRUE)  &&
        (params->sensitive == TRUE) &&
        (params->style_class == NULL) &&
        (params->focusOnHover == TRUE))
    {
        return;
    }

	priv->activatable = params->reactive && params->activate;
	priv->focusOnHover = params->focusOnHover;
	priv->margin_left_label = params->margin_left_label;
	priv->margin_left_label_icon = params->margin_left_label_icon;

	cdos_popup_base_menu_item_set_sensitive(item, (priv->activatable && params->sensitive));

	if (!params->style_class)
		st_widget_remove_style_class_name(ST_WIDGET(priv->actor), "popup-menu-item");
	else
		st_widget_set_style_class_name(ST_WIDGET(priv->actor), params->style_class);
	
	if (!priv->activatable) { 

		if (priv->button_release_event_id) {
			g_signal_handler_disconnect(priv->actor, priv->button_release_event_id);
			priv->button_release_event_id = 0L;
		}  
		
		if (priv->key_press_event_id) {
			g_signal_handler_disconnect(priv->actor, priv->key_press_event_id);
			priv->key_press_event_id = 0L;
		}		
	}

	if (!(params->reactive && params->hover)) 
		if (priv->notify_hover_id) {
            g_signal_handler_disconnect(priv->actor, priv->notify_hover_id);			
            priv->notify_hover_id = 0L;
	    }
	
	if (!params->reactive) {
		if (priv->key_focus_out_id) {
			g_signal_handler_disconnect(priv->actor, priv->key_focus_out_id);
			priv->key_focus_out_id = 0L;
		}
		
		if (priv->key_focus_in_id) {
			g_signal_handler_disconnect(priv->actor, priv->key_focus_in_id);
			priv->key_focus_in_id = 0L;
		}
	}
}

static void 
cdos_popup_base_menu_item_on_style_changed(CdosGenericContainer *actor, 
					CdosPopupBaseMenuItem *item)
{
	g_return_if_fail(CDOS_IS_POPUP_BASE_MENU_ITEM(item));
	g_return_if_fail(CDOS_IS_GENERIC_CONTAINER(actor));
	StThemeNode 					*themeNode;
	gdouble						length;
	gint 						spacing;

	themeNode = st_widget_get_theme_node(ST_WIDGET(actor));

	length = st_theme_node_get_length(themeNode, "spacing");

	spacing=(gint)(length+0.5)>(gint)length ? (gint)length+1:(gint)length; 

	item->priv->spacing = spacing;
}

static gboolean 
cdos_popup_base_menu_item_on_button_release_event(CdosGenericContainer *actor, 
						ClutterEvent *event,
						CdosPopupBaseMenuItem *item)
{
	CdosPopupBaseMenuItemPrivate *priv = item->priv;
	cdos_popup_base_menu_item_activate(item, event, priv->keep_menu);
	return FALSE;
}

static void 
cdos_popup_base_menu_item_on_key_focus_in(ClutterActor *actor,
					CdosPopupBaseMenuItem *item)
{
	cdos_popup_base_menu_item_set_active(item, TRUE);
}

static void 
cdos_popup_base_menu_item_on_key_focus_out(ClutterActor *actor,
					CdosPopupBaseMenuItem *item)
{
	cdos_popup_base_menu_item_set_active(item, FALSE);
}

static void 
cdos_popup_base_menu_item_on_hover_changed(ClutterActor *actor,
					GParamSpec *spec,	 
					CdosPopupBaseMenuItem *item)
{
	gboolean hover = st_widget_get_hover(ST_WIDGET(actor));
	cdos_popup_base_menu_item_set_active(item, hover);
}

/**
 * cdos_popup_base_menu_item_activate:
 * @item: A #CdosPopupBaseMenuItem
 * @event: 事件:
 * @keepMenu: ？？ 
 *
 * 使item处于activate状态
 *
 */
void 
cdos_popup_base_menu_item_activate(CdosPopupBaseMenuItem *item,
				   ClutterEvent          *event,
				   gboolean               keepMenu)
{
	g_signal_emit(item, cdos_popup_base_menu_item_signals[ACTIVATE], 0, event, keepMenu);
}

/**
 * cdos_popup_base_menu_item_set_active:
 * @self: A #CdosPopupBaseMenuItem
 * @active: 设置activate状态 
 *
 * 设置self的activate状态
 *
 */
void   
cdos_popup_base_menu_item_set_active(CdosPopupBaseMenuItem *item,
				gboolean active)
{
	CdosPopupBaseMenuItemPrivate *priv;
	g_return_if_fail(CDOS_IS_POPUP_BASE_MENU_ITEM(item));
	priv = item->priv;
	gboolean activeChanged = (active != priv->active);
	if(activeChanged) {
		priv->active = active;
		if(active){
			st_widget_set_style_pseudo_class(ST_WIDGET (priv->actor),  "active");
			if (priv->focusOnHover) 
				clutter_actor_grab_key_focus(CLUTTER_ACTOR (priv->actor));
        }
        else
            st_widget_remove_style_pseudo_class(ST_WIDGET (priv->actor), "active");
        g_signal_emit_by_name(item, "active-changed", active);
    }
}

/**
 * cdos_popup_base_menu_item_set_sensitive:
 * @item: A #CdosPopupBaseMenuItem
 * @sensitive: 是否是可以点击状态 
 *
 * 设置self的是否是可以点击
 */
void 
cdos_popup_base_menu_item_set_sensitive(CdosPopupBaseMenuItem *item,
				gboolean 	sensitive)
{	
	CdosPopupBaseMenuItemPrivate *priv;
	g_return_if_fail(CDOS_IS_POPUP_BASE_MENU_ITEM(item));

	priv = CDOS_POPUP_BASE_MENU_ITEM_GET_PRIVATE(item);
	if (!priv->activatable)
		return;

	if (priv->sensitive == sensitive) 
		return;

	priv->sensitive = sensitive;
	clutter_actor_set_reactive((ClutterActor *)priv->actor, sensitive);
	st_widget_set_can_focus((StWidget *)priv->actor, sensitive);
	if (sensitive) 
		st_widget_remove_style_pseudo_class((StWidget *)priv->actor, "insensitive");
	else 	
		st_widget_add_style_pseudo_class((StWidget *)priv->actor, "insensitive");

	g_signal_emit_by_name(item, "sensitive-changed", sensitive);
}

/**
 * cdos_popup_base_menu_item_destroy:
 * @item: A #CdosPopupBaseMenuItem
 *
 * 销毁item
 */
void 
cdos_popup_base_menu_item_destroy(CdosPopupBaseMenuItem *item)
{
	CdosPopupBaseMenuItemPrivate 		*priv;
	g_return_if_fail(CDOS_IS_POPUP_BASE_MENU_ITEM(item));
	priv = item->priv;
	g_signal_emit(item, cdos_popup_base_menu_item_signals[DESTROY], 0);
	clutter_actor_destroy((ClutterActor *)priv->actor);
}

/**
 * cdos_popup_base_menu_item_add_actor:
 * @self: A #CdosPopupBaseMenuItem
 * @child: 需要添加的actor
 * @params: 添加的布局信息 
 *
 * 往CdosPopupBaseMenuItem添加一个＃ClutterActor
 *
 */
void 
cdos_popup_base_menu_item_add_actor(CdosPopupBaseMenuItem *item, 
				    ClutterActor          *child, 
				    ActorChild            *params)
{
	CdosPopupBaseMenuItemPrivate *priv;
	g_return_if_fail(CDOS_IS_POPUP_BASE_MENU_ITEM(item));
	g_return_if_fail(CLUTTER_IS_ACTOR(child));
	priv = item->priv;
	ActorChild ac;
	ac.actor = child;
	if (params)
	{
		ac.span = params->span;
		ac.expand = params->expand;
		ac.align = (params->align < ST_ALIGN_CENTER_SPECIAL) ? params->align : ST_ALIGN_START;
	}else {
		ac.span = 0;
		ac.expand = FALSE;
		ac.align = ST_ALIGN_START;
	}
	
	priv->children = g_array_append_val(priv->children, ac);

//	g_signal_connect (child, "destroy",
//		G_CALLBACK(cdos_popup_base_menu_item_call_back), item);
	clutter_container_add_actor((ClutterContainer *)item->priv->actor, child);
}

static void 
cdos_popup_base_menu_item_call_back(ClutterActor          *actor,
				    CdosPopupBaseMenuItem *item)
{
	cdos_popup_base_menu_item_remove_child(item, actor);
}

static void 
cdos_popup_base_menu_item_remove_child(CdosPopupBaseMenuItem *item, 
				       ClutterActor          *child)
{
	CdosPopupBaseMenuItemPrivate *priv = item->priv;
	for (gint i = 0; i < priv->children->len; i++) {
		ActorChild c = g_array_index(priv->children, ActorChild, i);
		if (c.actor == child) {
			priv->children = g_array_remove_index(priv->children, i);
			return;
		}
	}
}

/**
 * cdos_popup_base_menu_item_remove_actor:
 * @self: A #CdosPopupBaseMenuItem
 * @child: 需要添加的actor
 *
 * 删除CdosPopupBaseMenuItem子控件＃ClutterActor
 *
 */
void 
cdos_popup_base_menu_item_remove_actor(CdosPopupBaseMenuItem *item,
				       ClutterActor          *child)
{	
	//CdosPopupBaseMenuItemPrivate *priv = item->priv;
	g_return_if_fail(CDOS_IS_POPUP_BASE_MENU_ITEM(item));
	g_return_if_fail(CLUTTER_IS_ACTOR(child));
	st_container_remove_actor ((ClutterContainer *)item->priv->actor, child);
	cdos_popup_base_menu_item_remove_child(item, child);
}

/**
 * cdos_popup_base_menu_item_set_show_dot:
 * @item: A #CdosPopupBaseMenuItem
 * @show: 是否显示 
 *
 * 是否显示item中的dot
 *
 */
void 
cdos_popup_base_menu_item_set_show_dot(CdosPopupBaseMenuItem *item,
				       gboolean               show)
{	
	CdosPopupBaseMenuItemPrivate *priv = item->priv;

	if (show) {
		if (priv->dot)
			return;

		priv->dot = g_object_new(ST_TYPE_DRAWING_AREA, 
				"style-class", "popup-menu-item-dot",
				NULL);
		g_signal_connect(priv->dot, "repaint",
			G_CALLBACK(cdos_popup_base_menu_item_on_repaint_dot), item);
		st_container_add_actor((ClutterContainer *)priv->actor, (ClutterActor *)priv->dot);
	} else {
		if (!priv->dot)
			return;
		clutter_actor_destroy(CLUTTER_ACTOR(priv->dot));
		priv->dot = NULL;		
	}
}

static void 
cdos_popup_base_menu_item_on_repaint_dot(StDrawingArea         *area,
					 CdosPopupBaseMenuItem *item)
{
	cairo_t 	*cr;
	guint		width;
	guint		height;
	StThemeNode  	*themeNode;
	ClutterColor 	color;

	cr = st_drawing_area_get_context(area);
	st_drawing_area_get_surface_size(area, &width, &height);
	themeNode = st_widget_get_theme_node((StWidget *)area);
	st_theme_node_get_foreground_color(themeNode, &color);
	cairo_set_source_rgba(cr, 
			color.red / 255,
			color.green / 255,
			color.blue / 255,
			color.alpha / 255);
	cairo_arc(cr, width / 2, height / 2, width / 2, 0, 2 * G_PI);

	cairo_fill(cr);
}

static void 
cdos_popup_base_menu_item_get_perferred_width(ClutterActor *actor,
					gfloat for_height,
					CdosGenericContainerAllocation *alloc,
					CdosPopupBaseMenuItem 	*item)
{
	CdosPopupBaseMenuItemPrivate 			 *priv;
	gfloat 						 width;
	gint 						 i;
	gfloat 						 min;
	gfloat 						 natural;
	ActorChild 				   	 *child;

	priv = item->priv;
    width = 0;
    for (i = 0; i < (priv->children->len); i++) {
        child = &g_array_index(priv->children, ActorChild, i);

        //if (i > 0)
        //    width += priv->spacing;
        gfloat tmp_height = -1;
        clutter_actor_get_preferred_width(child->actor, tmp_height, &min, &natural);
        width += natural;
    }
    width += (i - 1) * priv->spacing;
	alloc->min_size = alloc->natural_size = width;
}

static void 
cdos_popup_base_menu_item_get_preferred_height(ClutterActor *actor,
					gfloat 	for_width,
					CdosGenericContainerAllocation *alloc,
					CdosPopupBaseMenuItem *item)
{
	CdosPopupBaseMenuItemPrivate 			*priv;
	gfloat 						 height;
	gfloat 						 x;
	gfloat 						 minWidth;
	gfloat 						 childWidth;
	gint 						 i;
	gint 						 j;
	ActorChild 					 *child;
	//gfloat 						 min;
	gfloat 						 natural;

	priv = item->priv;
	gfloat 	tmp = -1;

	height = 0;
	x = 0;

	for (i = 0; i < priv->children->len; i++) {
		child = &g_array_index(priv->children, ActorChild, i);
		if(child && child->actor != NULL){
            if (child->span == -1)
                childWidth = for_width -x;
            else
                clutter_actor_get_preferred_width(child->actor,
                        (gfloat)tmp,
                        &minWidth,
                        &childWidth);

            x += childWidth;

			clutter_actor_get_preferred_height(child->actor, childWidth, &minWidth, &natural);
			if (natural > height)
				height = natural;
		}
	}

	alloc->min_size = alloc->natural_size = height;
}

static void 
cdos_popup_base_menu_item_allocate(ClutterActor *actor,
			     const ClutterActorBox *box,
			     ClutterAllocationFlags flags,
			     CdosPopupBaseMenuItem *item)
{
	gfloat 				height;
	StTextDirection 	 	direction;
	CdosPopupBaseMenuItemPrivate  	*priv;
	ClutterActorBox	 		*dotBox;
	gfloat				dotwidth; 	
	gfloat				x = 0;
	gint 				i;
	gfloat 				minWidth;
	gfloat 				naturalWidth;
	gfloat 				availWidth = 0;
	gfloat 				extraWidth = 0;
	gfloat 				minHeight;
	gfloat 				naturalHeight;

	height = box->y2 - box->y1;
	priv = item->priv;	
	direction = st_widget_get_direction(ST_WIDGET(priv->actor));

	if (priv->dot) {
		dotBox = clutter_actor_box_new(-1, -1, -1, -1);
		dotwidth = roundf(box->x1 / 2 + 2);

		if (direction == ST_TEXT_DIRECTION_LTR) {
			dotBox->x1 = roundf(box->x1 + 5);
			dotBox->x2 = dotBox->x1 + dotwidth;
		} else {
			dotBox->x2 = box->x2 + 3 *roundf(box->x1 / 4);
			dotBox->x1 = dotBox->x2 - dotwidth;
		}
		dotBox->y1 = roundf(box->y1 + (height - dotwidth) / 2);
		dotBox->y2 = dotBox->y1 + dotwidth;

		clutter_actor_allocate(CLUTTER_ACTOR(priv->dot), dotBox, flags);
		clutter_actor_box_free(dotBox);
	}

    if(x == 0){
        x = direction == ST_TEXT_DIRECTION_LTR ? box->x1 : box->x2;
    }

	for (i = 0; i < priv->children->len; i++) {
		ActorChild *child = &g_array_index(priv->children, ActorChild, i);
		ClutterActorBox *childBox = clutter_actor_box_alloc();
		gfloat for_height = -1;
		clutter_actor_get_preferred_width(child->actor, for_height,
					&minWidth,
                    &naturalWidth);
        if (child->span == -1) {
            if (direction == ST_TEXT_DIRECTION_LTR)
                availWidth = box->x2 - x;
            else
                availWidth = x - box->x1;
        } else {
            availWidth = naturalWidth;
        }
        extraWidth = availWidth;
        if(extraWidth == 0){
            extraWidth = availWidth == 0 ? naturalWidth : availWidth;
        }

        if (direction == ST_TEXT_DIRECTION_LTR)
            extraWidth = box->x2 - x - naturalWidth;
        else
            extraWidth = x - box->x1 - naturalWidth;

        if (direction == ST_TEXT_DIRECTION_LTR) {
			if (child->expand) {
				childBox->x1 = x;
				childBox->x2 = x + availWidth;
			} 
			else if (child->align == ST_ALIGN_MIDDLE) {
				childBox->x1 = x + roundf(extraWidth / 2);
				childBox->x2 = childBox->x1 + naturalWidth;
			}
			else if (child->align == ST_ALIGN_END) {

				childBox->x2 = box->x2 ;
				childBox->x1= childBox->x2 - naturalWidth;
			} 
			else {
				childBox->x1 = x;
				childBox->x2 = x + naturalWidth;
			}
		} else {
			if (child->expand) {
				childBox->x1 = x -availWidth;
				childBox->x2 = x;
			}
			else if (child->align == ST_ALIGN_MIDDLE) {
				childBox->x1 = x - roundf(extraWidth / 2);
				childBox->x2 = childBox->x1 + naturalWidth;
			}
			else if (child->align == ST_ALIGN_END) {
				childBox->x1 = x - availWidth;
				childBox->x2 = childBox->x1 + naturalWidth;
			}
			else {
				childBox->x2 = x;
				childBox->x1 = x - naturalWidth;
			}
		}
        
		gfloat tmp = childBox->x2 - childBox->x1 ;
		clutter_actor_get_preferred_height(child->actor, 
						tmp,
						&minHeight,
						&naturalHeight);

		childBox->y1 = roundf(box->y1 + (height - naturalHeight) / 2);
		childBox->y2 = childBox->y1 + naturalHeight;

		clutter_actor_allocate(child->actor, childBox, flags);

		if (direction == ST_TEXT_DIRECTION_LTR) 
			x += (availWidth + priv->spacing);
		else 
			x -= (availWidth + priv->spacing);

        clutter_actor_box_free(childBox);
	}
}

/**
 * cdos_popup_base_menu_item_get_actor:
 * @item: A #CdosPopupBaseMenuItem
 *
 * 获取item的容器
 *
 * Return value: A #CdosGenericContainer
 */
CdosGenericContainer *
cdos_popup_base_menu_item_get_actor(CdosPopupBaseMenuItem *item)
{
	g_return_val_if_fail(CDOS_IS_POPUP_BASE_MENU_ITEM(item), NULL);

	CdosPopupBaseMenuItemPrivate *priv = item->priv;
	return priv->actor;
}

/**
 * cdos_popup_base_menu_item_set_actor:
 * @item: A #CdosPopupBaseMenuItem
 * @actor: 新的box 
 *
 * 设置item控件的actor
 *
 */
void
cdos_popup_base_menu_item_set_actor(CdosPopupBaseMenuItem *item,
				CdosGenericContainer *actor)
{
	g_return_if_fail(CDOS_IS_POPUP_BASE_MENU_ITEM(item));
	if (NULL == actor)
		return;

	CdosPopupBaseMenuItemPrivate *priv = item->priv;

    if (priv->actor)
    {
        if (priv->actor_destroy_id)
        {
            g_signal_handler_disconnect (priv->actor, priv->actor_destroy_id);
            priv->actor_destroy_id = 0;
        }
        clutter_actor_destroy ((ClutterActor *)priv->actor);
    }

	priv->actor = actor;
    item->actor = (ClutterActor *)actor;
	//g_object_ref(priv->actor);
    priv->actor_destroy_id = g_signal_connect (actor, "destroy", 
                                    G_CALLBACK (cdos_popup_base_menu_item_on_actor_destroy),
                                    item);
}

/**
 * cdos_popup_base_menu_item_get_dot:
 * @item: A #CdosPopupBaseMenuItem
 *
 * 获取item的dot成员
 *
 * Returns: #StDrawingArea
 */
StDrawingArea *
cdos_popup_base_menu_item_get_dot(CdosPopupBaseMenuItem *item)
{
	g_return_val_if_fail(CDOS_IS_POPUP_BASE_MENU_ITEM(item), NULL);

	CdosPopupBaseMenuItemPrivate *priv = item->priv;

	return priv->dot;
}

/**
 * cdos_popup_base_menu_item_set_dot:
 * @item: A #CdosPopupBaseMenuItem
 * @dot: #StDrawingArea 描绘的区域 
 *
 * 设置self的dot成员
 *
 */
void
cdos_popup_base_menu_item_set_dot(CdosPopupBaseMenuItem *item,
				  StDrawingArea         *dot)
{
	g_return_if_fail(CDOS_IS_POPUP_BASE_MENU_ITEM(item));

	if (NULL == dot) 
		return;

	CdosPopupBaseMenuItemPrivate *priv = item->priv;
	if (priv->dot) 
	{
	    clutter_actor_destroy ((ClutterActor *)priv->dot);
	}
	priv->dot = dot;
	g_object_ref(priv->dot);
}

/**
 * cdos_popup_base_menu_item_get_spacing:
 * @item: A #CdosPopupBaseMenuItem
 *
 * 获取item控件的spacing值
 *
 * Return value：a #gint
 */
gint 
cdos_popup_base_menu_item_get_spacing(CdosPopupBaseMenuItem *item)
{
	g_return_val_if_fail(CDOS_IS_POPUP_BASE_MENU_ITEM(item), 0);

	CdosPopupBaseMenuItemPrivate *priv = item->priv;

	return priv->spacing;
}

/**
 * cdos_popup_base_menu_item_set_spacing:
 * @item: A #CdosPopupBaseMenuItem
 * @spacing: spacing的值
 *
 * 设置Item的Spacing的值
 *
 */
void
cdos_popup_base_menu_item_set_spacing(CdosPopupBaseMenuItem *item,
				gint spacing)
{
	g_return_if_fail(CDOS_IS_POPUP_BASE_MENU_ITEM(item));
	if (spacing == -1) 
		return;

	CdosPopupBaseMenuItemPrivate *priv = item->priv;
	priv->spacing = spacing;

	g_object_notify(G_OBJECT(item), "spacing");
}

/**
 * cdos_popup_base_menu_item_get_active:
 * @item: A #CdosPopupBaseMenuItem
 *
 * 获取item的activate属性
 *
 * Return value： A #gboolean
 */
gboolean
cdos_popup_base_menu_item_get_active(CdosPopupBaseMenuItem *item)
{
	g_return_val_if_fail(CDOS_IS_POPUP_BASE_MENU_ITEM(item), FALSE);

	return item->priv->active;
}

/**
 * cdos_popup_base_menu_item_get_activatable:
 * @item: A #CdosPopupBaseMenuItem
 *
 * 获取item的activatable属性
 *
 * Return value： A #gboolean
 */
gboolean 
cdos_popup_base_menu_item_get_activatable(CdosPopupBaseMenuItem *item)
{
	g_return_val_if_fail(CDOS_IS_POPUP_BASE_MENU_ITEM(item), FALSE);

	return item->priv->activatable;
}

/**
 * cdos_popup_base_menu_item_set_activatable:
 * @item: A #CdosPopupBaseMenuItem
 * @activatable: 是否activatable 
 *
 * 设置的activatable
 *
 */
void
cdos_popup_base_menu_item_set_activatable(CdosPopupBaseMenuItem *item,
				gboolean  	activatable)
{
	g_return_if_fail(CDOS_IS_POPUP_BASE_MENU_ITEM(item));

	item->priv->activatable = activatable;
}

/**
 * cdos_popup_base_menu_item_get_sensitive:
 * @item: A #CdosPopupBaseMenuItem
 *
 * 获取item的sensitive属性
 *
 * Return value： A #gboolean
 */
gboolean 
cdos_popup_base_menu_item_get_sensitive(CdosPopupBaseMenuItem *item)
{
	g_return_val_if_fail(CDOS_IS_POPUP_BASE_MENU_ITEM(item), FALSE);

	return item->priv->sensitive;
}

/**
 * cdos_popup_base_menu_item_get_focusOnHover:
 * @item: A #CdosPopupBaseMenuItem
 *
 * 获取item的focusOnHover属性
 *
 * Return value： A #gboolean
 */
gboolean
cdos_popup_base_menu_item_get_focusOnHover(CdosPopupBaseMenuItem *item)
{
	g_return_val_if_fail(CDOS_IS_POPUP_BASE_MENU_ITEM(item), FALSE);

	return item->priv->focusOnHover;
}

/**
 * cdos_popup_base_menu_item_set_focusOnHover:
 * @item: A #CdosPopupBaseMenuItem
 * @focusOnHover: hover 是是否可以接受焦点
 *
 * 设置item的focusOnHover
 *
 */
void
cdos_popup_base_menu_item_set_focusOnHover(CdosPopupBaseMenuItem *item,
				gboolean focusOnHover)
{
	g_return_if_fail(CDOS_IS_POPUP_BASE_MENU_ITEM(item));

	item->priv->focusOnHover = focusOnHover;
}

/**
 * cdos_popup_base_menu_item_get_margin_left_label:
 * @item: A #CdosPopupBaseMenuItem
 *
 * 获取item的margin_left_label属性
 *
 * Return value： A #gint
 */
gint 
cdos_popup_base_menu_item_get_margin_left_label(CdosPopupBaseMenuItem *item)
{
	if (!CDOS_IS_POPUP_BASE_MENU_ITEM(item))
        return 0;

	return item->priv->margin_left_label;
}

/**
 * cdos_popup_base_menu_item_set_margin_left_label:
 * @item: A #CdosPopupBaseMenuItem
 * @margin_left_label: 距离左边label的距离
 *
 * 设置item的margin_left_label
 *
 */
void
cdos_popup_base_menu_item_set_margin_left_label(CdosPopupBaseMenuItem *item,
					gint 	margin_left_label)
{
	g_return_if_fail(CDOS_IS_POPUP_BASE_MENU_ITEM(item));

	item->priv->margin_left_label = margin_left_label;
}

/**
 * cdos_popup_base_menu_item_get_margin_left_label_icon:
 * @item: A #CdosPopupBaseMenuItem
 *
 * 获取item的margin_left_label_icon
 *
 * Return value: A #margin_left_label_icon的大小
 */
gint 
cdos_popup_base_menu_item_get_margin_left_label_icon(CdosPopupBaseMenuItem *item)
{
	if (!CDOS_IS_POPUP_BASE_MENU_ITEM(item))
        return 0;

	return item->priv->margin_left_label_icon;
}

/**
 * cdos_popup_base_menu_item_set_margin_left_label_icon:
 * @item: A #CdosPopupBaseMenuItem
 * @margin_left_label_icon: 距离左边icon的距离
 *
 * 设置item的margin_left_label_icon
 *
 */
void
cdos_popup_base_menu_item_set_margin_left_label_icon(CdosPopupBaseMenuItem *item, 
					gint margin_left_label_icon)
{
	g_return_if_fail(CDOS_IS_POPUP_BASE_MENU_ITEM(item));

	item->priv->margin_left_label_icon = margin_left_label_icon;
}

/**
 * cdos_popup_base_menu_item_get_init_params:
 * @params: A #PopupBaseMenuItemInit
 *
 * 当我们创建一个PopupBaseMenuItem的控件时需要的默认参数，可以更改
 *
 * Return value： A#PopupBaseMenuItemInit
 */
PopupBaseMenuItemInit *
cdos_popup_base_menu_item_get_init_params(PopupBaseMenuItemInit *params)
{
	params->reactive = TRUE;
	params->activate = TRUE;
	params->hover = TRUE;
	params->sensitive = TRUE;
	params->style_class = NULL;
	params->focusOnHover = TRUE;
	params->margin_left_label = 10;
	params->margin_left_label_icon = 10;

	return params;
}

/**
 * cdos_popup_base_menu_item_get_actor_child:
 * @child: A #ActorChild 
 *
 * 当我们添加一个控件需要的必备参数
 *
 * Return value: a #ActorChild
 */
ActorChild *
cdos_popup_base_menu_item_get_actor_child(ActorChild *child)
{
	if (NULL == child)
		return NULL;
	
	child->span = 1;
	child->expand = FALSE;
	child->align = ST_ALIGN_START;

	return child;
}

/**
 * cdos_popup_base_menu_item_hide:
 * @item: A #CdosPopupBaseMenuItem
 *
 * 将item控件隐藏起来
 *
 */
void 
cdos_popup_base_menu_item_hide(CdosPopupBaseMenuItem *item)
{
	if (item == NULL)
		return;

	clutter_actor_hide((ClutterActor *)cdos_popup_base_menu_item_get_actor(item));
}

/**
 * cdos_popup_base_menu_item_show:
 * @item: A #CdosPopupBaseMenuItem
 *
 * 显示item控件
 *
 */
void 
cdos_popup_base_menu_item_show(CdosPopupBaseMenuItem *item)
{
	if (item == NULL)
		return;

	clutter_actor_show((ClutterActor *)cdos_popup_base_menu_item_get_actor(item));
}

/**
 * cdos_popup_base_menu_item_set_keep_menu:
 * @item: A #CdosPopupBaseMenuItem
 * @keep_menu: #gboolean  
 *
 * 设置item的keep_menu属性
 *
 */
void 
cdos_popup_base_menu_item_set_keep_menu(CdosPopupBaseMenuItem *item,
					gboolean               keep_menu)
{
	g_return_if_fail(CDOS_IS_POPUP_BASE_MENU_ITEM(item));
	CdosPopupBaseMenuItemPrivate *priv = item->priv;
	priv->keep_menu = keep_menu;
}

/**
 * cdos_popup_base_menu_item_get_keep_menu:
 * @item: A #CdosPopupBaseMenuItem
 *
 * 获取self的keep_menu成员
 *
 * Returns: ＃gboolean
 */
gboolean 
cdos_popup_base_menu_item_get_keep_menu(CdosPopupBaseMenuItem *item)
{
	g_return_val_if_fail(CDOS_IS_POPUP_BASE_MENU_ITEM(item),FALSE);
	CdosPopupBaseMenuItemPrivate *priv = item->priv;
	return priv->keep_menu;
}

void
cdos_popup_base_menu_item_set_actor_align(CdosPopupBaseMenuItem *item,ClutterActor *actor,StAlign align){
	g_return_if_fail(CDOS_IS_POPUP_BASE_MENU_ITEM(item));
	g_return_if_fail(CLUTTER_ACTOR(actor));
    CdosPopupBaseMenuItemPrivate *priv = item->priv;
	GArray *children = priv->children;
	guint i;
	for (i = 0; i < (children->len); i++) {
		ActorChild *child = &g_array_index(children, ActorChild, i);
		if(child->actor == actor){
            child->align = align;
            break ;
		}
	}
}
