#ifndef __CDOS_POPUP_BASE_MENU_ITEM_H__
#define __CDOS_POPUP_BASE_MENU_ITEM_H__

#include "st.h"
#include <glib.h>
#include "cdos-generic-container.h"
#include "st-marshal.h"


G_BEGIN_DECLS

#define CDOS_TYPE_POPUP_BASE_MENU_ITEM  \
	(cdos_popup_base_menu_item_get_type())

#define  CDOS_POPUP_BASE_MENU_ITEM(obj) \
	(G_TYPE_CHECK_INSTANCE_CAST((obj), CDOS_TYPE_POPUP_BASE_MENU_ITEM, CdosPopupBaseMenuItem))

#define CDOS_IS_POPUP_BASE_MENU_ITEM(obj) \
	(G_TYPE_CHECK_INSTANCE_TYPE((obj), CDOS_TYPE_POPUP_BASE_MENU_ITEM))

#define CDOS_POPUP_BASE_MENU_ITEM_CLASS(klass) \
	(G_TYPE_CHECK_CLASS_CAST((klass), CDOS_TYPE_POPUP_BASE_MENU_ITEM, CdosPopupBaseMenuItemClass))

#define CDOS_IS_POPUP_BASE_MENU_ITEM_CLASS(klass) \
	(G_TYPE_CHECK_CLASS_TYPE((klass), CDOS_TYPE_POPUP_BASE_MENU_ITEM))

#define CDOS_POPUP_BASE_MENU_ITEM_GET_CLASS(obj) \
	(G_TYPE_INSTANCE_GET_CLASS((obj), CDOS_TYPE_POPUP_BASE_MENU_ITEM, CdosPopupBaseMenuItemClass))	


typedef struct _CdosPopupBaseMenuItem 		CdosPopupBaseMenuItem;
typedef struct _CdosPopupBaseMenuItemPrivate  	CdosPopupBaseMenuItemPrivate;
typedef struct _CdosPopupBaseMenuItemClass 	CdosPopupBaseMenuItemClass;


struct  _CdosPopupBaseMenuItem
{
	GObject parent_instance;
        ClutterActor *actor;
	CdosPopupBaseMenuItemPrivate *priv;
};

struct _CdosPopupBaseMenuItemClass
{
	GObjectClass parent_class;
	/*signals*/
	void (*activate) (CdosPopupBaseMenuItem * menuitem, gpointer event, gboolean keepMenu);
	void (*active_changed) (CdosPopupBaseMenuItem * menuitem, gboolean active);
	void (*sensitive_changed) (CdosPopupBaseMenuItem * menuitem, gboolean sensitive);
	void (*destroy) (CdosPopupBaseMenuItem * menuitem);
	void (*value_changed) (CdosPopupBaseMenuItem *menuItem, gfloat value);
	void (*drag_end) (CdosPopupBaseMenuItem *menuitem);
    gboolean (*key_press_event) (ClutterActor *actor,
					             ClutterEvent *event, 
                                 CdosPopupBaseMenuItem *item);
};


typedef struct  
{
	gboolean 	reactive ;
	gboolean 	activate;
	gboolean 	hover;
	gboolean 	sensitive;
	gchar* 	   	style_class;
	gboolean 	focusOnHover;
	gint 	    	margin_left_label;
	gint 	  	margin_left_label_icon;
}PopupBaseMenuItemInit;

typedef struct 
{
	gint 		span;
	gboolean	expand;
	StAlign         align;
	ClutterActor 	*actor;
}ActorChild;


GType  cdos_popup_base_menu_item_get_type(void) G_GNUC_CONST;

CdosPopupBaseMenuItem   *cdos_popup_base_menu_item_new(void);

CdosPopupBaseMenuItem  *cdos_popup_base_menu_item_new_with_params(PopupBaseMenuItemInit *params);

void  cdos_popup_base_menu_item_set_params(CdosPopupBaseMenuItem *item, PopupBaseMenuItemInit *params);

void  cdos_popup_base_menu_item_activate(CdosPopupBaseMenuItem *item, ClutterEvent *event, gboolean keepMenu);


void  cdos_popup_base_menu_item_set_active(CdosPopupBaseMenuItem *item, gboolean active);

void  cdos_popup_base_menu_item_set_sensitive(CdosPopupBaseMenuItem *item, gboolean sensitive);

void cdos_popup_base_menu_item_destroy(CdosPopupBaseMenuItem *item);

void cdos_popup_base_menu_item_add_actor(CdosPopupBaseMenuItem *item, ClutterActor *child, ActorChild *params);

void cdos_popup_base_menu_item_remove_actor(CdosPopupBaseMenuItem *item, ClutterActor *child);				

void cdos_popup_base_menu_item_set_show_dot(CdosPopupBaseMenuItem *item, gboolean  show);


CdosGenericContainer *cdos_popup_base_menu_item_get_actor(CdosPopupBaseMenuItem *item);	


void cdos_popup_base_menu_item_set_actor(CdosPopupBaseMenuItem *item, CdosGenericContainer *actor);	

StDrawingArea * cdos_popup_base_menu_item_get_dot(CdosPopupBaseMenuItem *item);

void cdos_popup_base_menu_item_set_dot(CdosPopupBaseMenuItem *item, StDrawingArea *dot);					 

gint cdos_popup_base_menu_item_get_spacing(CdosPopupBaseMenuItem *item);

void cdos_popup_base_menu_item_set_spacing(CdosPopupBaseMenuItem *item, gint spacing);

gboolean cdos_popup_base_menu_item_get_active(CdosPopupBaseMenuItem *item);	

void cdos_popup_base_menu_item_set_active(CdosPopupBaseMenuItem *item, gboolean active);

gboolean  cdos_popup_base_menu_item_get_activatable(CdosPopupBaseMenuItem *item);

void cdos_popup_base_menu_item_set_activatable(CdosPopupBaseMenuItem *item, gboolean  activatable);	

gboolean  cdos_popup_base_menu_item_get_sensitive(CdosPopupBaseMenuItem *item);

gboolean cdos_popup_base_menu_item_get_focusOnHover(CdosPopupBaseMenuItem *item);

void cdos_popup_base_menu_item_set_focusOnHover(CdosPopupBaseMenuItem *item, gboolean focusOnHover);		

gint  cdos_popup_base_menu_item_get_margin_left_label(CdosPopupBaseMenuItem *item);	

void cdos_popup_base_menu_item_set_margin_left_label(CdosPopupBaseMenuItem *item, gint margin_left_label);

gint cdos_popup_base_menu_item_get_margin_left_label_icon(CdosPopupBaseMenuItem *item);

void cdos_popup_base_menu_item_set_margin_left_label_icon(CdosPopupBaseMenuItem *item, gint margin_left_label_icon);

PopupBaseMenuItemInit  *cdos_popup_base_menu_item_get_init_params(PopupBaseMenuItemInit *params);

ActorChild *cdos_popup_base_menu_item_get_actor_child(ActorChild *child);


void cdos_popup_base_menu_item_hide(CdosPopupBaseMenuItem *item);

void cdos_popup_base_menu_item_show(CdosPopupBaseMenuItem *item);

void cdos_popup_base_menu_item_set_keep_menu(CdosPopupBaseMenuItem *item,gboolean keep_menu);

gboolean cdos_popup_base_menu_item_get_keep_menu(CdosPopupBaseMenuItem *item);

void cdos_popup_base_menu_item_set_actor_align(CdosPopupBaseMenuItem *item,ClutterActor *actor, StAlign align);

G_END_DECLS

#endif /*__CDOS_POPUP_BASE_MENU_ITEM_H__*/
