#include <math.h>
#include "cdos-popup-combo-menu.h"
#include "cdos-popup-menu-section.h"
#include "cdos-global.h"

/**
 * SECTION: cdos-popup-combo-menu.c
 * @Title: CdosPopupComboMenu
 * @short_description: 继承自CdosPopupMenuBase，CdosPopupComboMenu是一个组合菜单，相比其他
 *　菜单，你可以显示你处于activate的menu是哪一个。 
 *
 * CdosPopupComboMenu 提供了一个组合菜单。
 */

G_DEFINE_TYPE(CdosPopupComboMenu, cdos_popup_combo_menu, CDOS_TYPE_POPUP_MENU_BASE);

#define POPUP_COMBO_MENU_GET_PRIVATE(obj)(G_TYPE_INSTANCE_GET_PRIVATE ((obj), CDOS_TYPE_POPUP_COMBO_MENU, CdosPopupComboMenuPrivate))

struct _CdosPopupComboMenuPrivate
{
  gchar  *text;
  gint    active_item_pos;
};

enum
{
  PROP_0,
  PROP_TEXT
};

enum
{
  CLICKED,
  LAST_SIGNAL
};

//static guint cdos_popup_combo_menu_signals[LAST_SIGNAL] = { 0, };

static void
cdos_popup_combo_menu_finalize (GObject * gobject)
{
  CdosPopupComboMenuPrivate * priv = CDOS_POPUP_COMBO_MENU(gobject)->priv;

  g_free(priv->text);

  G_OBJECT_CLASS (cdos_popup_combo_menu_parent_class)->finalize (gobject);
}

static void
cdos_popup_combo_menu_dispose (GObject * gobject)
{
  //CdosPopupComboMenuPrivate * priv = CDOS_POPUP_COMBO_MENU(gobject)->priv;

  G_OBJECT_CLASS (cdos_popup_combo_menu_parent_class)->dispose (gobject);
}

static void
cdos_popup_combo_menu_set_property (GObject * gobject, guint prop_id,
				  const GValue * value, GParamSpec * pspec)
{
  CdosPopupComboMenu * newobj = CDOS_POPUP_COMBO_MENU (gobject);

  switch (prop_id)
    {
    case PROP_TEXT:
      cdos_popup_combo_menu_set_text (newobj, g_value_get_string (value));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID(gobject, prop_id, pspec);
      break;
    }
}

static void
cdos_popup_combo_menu_get_property (GObject * gobject, guint prop_id,
				  GValue * value, GParamSpec * pspec)
{

  CdosPopupComboMenuPrivate * priv = CDOS_POPUP_COMBO_MENU(gobject)->priv;
  switch (prop_id)
    {
    case PROP_TEXT:
      g_value_set_string (value, priv->text);
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID(gobject, prop_id, pspec);
      break;
    }
}

static ClutterActor *
cdos_popup_combo_menu_get_actor(CdosPopupMenuBase *self)
{
  CdosPopupComboMenu *combo_menu = CDOS_POPUP_COMBO_MENU(self);
  return CLUTTER_ACTOR(combo_menu->actor);
}

static void
cdos_popup_combo_menu_class_init(CdosPopupComboMenuClass * klass)
{
  CdosPopupMenuBaseClass *parent_class = CDOS_POPUP_MENU_BASE_CLASS(klass);
  GObjectClass * gobject_class = G_OBJECT_CLASS(klass);
  GParamSpec * pspec;

  parent_class->open = cdos_popup_combo_menu_open;
  parent_class->close = cdos_popup_combo_menu_close;
  parent_class->get_actor = cdos_popup_combo_menu_get_actor;
  gobject_class->finalize = cdos_popup_combo_menu_finalize;
  gobject_class->dispose = cdos_popup_combo_menu_dispose;
  gobject_class->set_property = cdos_popup_combo_menu_set_property;
  gobject_class->get_property = cdos_popup_combo_menu_get_property;

  g_type_class_add_private (klass, sizeof(CdosPopupComboMenuPrivate));

  pspec = g_param_spec_string ("text",
			       "Text",
			       "Text of the object",
			       NULL,
			       G_PARAM_READWRITE);
  g_object_class_install_property (gobject_class, PROP_TEXT, pspec);
}

static gboolean
cdos_popup_combo_menu_on_key_press_event (ClutterActor       *actor,
				                                  ClutterEvent    *event,
				                                  CdosPopupComboMenu *self)
{
  if(clutter_event_get_key_symbol(event) == CLUTTER_Escape)
  {
      cdos_popup_combo_menu_open((CdosPopupMenuBase *)self,TRUE);
      return TRUE;
  }

  return FALSE;
}

static void
cdos_popup_combo_menu_on_key_focus_in (ClutterActor       *actor,
				                               CdosPopupComboMenu *self)
{
  CdosPopupComboMenuPrivate *priv = self->priv;
  GList *items;
  gpointer active_item;

  items = cdos_popup_menu_base_get_menu_items(CDOS_POPUP_MENU_BASE(self));
  active_item = g_list_nth(items, priv->active_item_pos)->data;

  if(CDOS_IS_POPUP_MENU_SECTION(active_item))
    clutter_actor_grab_key_focus(CLUTTER_ACTOR(CDOS_POPUP_MENU_SECTION(active_item)->actor));

  if(CDOS_IS_POPUP_BASE_MENU_ITEM(active_item))
    clutter_actor_grab_key_focus(menu_item_get_actor(active_item));
}

static void
cdos_popup_combo_menu_init (CdosPopupComboMenu * self)
{
  CdosPopupComboMenuPrivate * priv;
  priv = self->priv = POPUP_COMBO_MENU_GET_PRIVATE(self);
  self->actor = cdos_popup_menu_base_get_box(CDOS_POPUP_MENU_BASE(self));
  g_object_set_data(G_OBJECT(self->actor),"_delegate", self);
  g_signal_connect(self->actor,"key-press-event",G_CALLBACK(cdos_popup_combo_menu_on_key_press_event),self);
  g_signal_connect(self->actor, "key-focus-in", G_CALLBACK(cdos_popup_combo_menu_on_key_focus_in), self);
  priv->active_item_pos = -1;
  //global.focus_manager.add_group(this.actor);
}

/**
 * cdos_popup_combo_menu_set_text:
 * @self: A #CdosPopupComboMenu
 * @text:  CdosPopupComboMenu的text属性
 *
 * 设置self的text属性
 *
 */
void
cdos_popup_combo_menu_set_text (CdosPopupComboMenu * self, const gchar * text)
{
  CdosPopupComboMenuPrivate * priv;
  g_return_if_fail(CDOS_POPUP_COMBO_MENU (self));

  priv = self->priv;

  g_free (priv->text);
  if (text)
    priv->text = g_strdup (text);
  else
    priv->text = "";
}

/**
 * cdos_popup_combo_menu_get_text:
 * @self: A #CdosPopupComboMenu
 *
 * 获取self的text属性
 *
 * Returns: text
 */
const gchar *
cdos_popup_combo_menu_get_text (CdosPopupComboMenu * self)
{

  g_return_val_if_fail(CDOS_POPUP_COMBO_MENU (self), NULL);

  return self->priv->text;
}

/**
 * cdos_popup_combo_menu_new:
 * @source_actor: source actor 
 * 
 * 新建一个 #CdosPopupComboMenu 类的实例。
 *
 * Returns: 返回一个 #CdosPopupComboMenu 的实例。
 */
CdosPopupComboMenu *
cdos_popup_combo_menu_new (ClutterActor *source_actor)
{
  return g_object_new (CDOS_TYPE_POPUP_COMBO_MENU,
		                  "source_actor", source_actor,
		                  "style-class", "popup-combo-menu",
		                   NULL);
}

/**
 * cdos_popup_combo_menu_open:
 * @obj: A #CdosPopupMenuBase 
 *
 * 打开menu
 *
 */
void
cdos_popup_combo_menu_open (CdosPopupMenuBase *obj,gboolean animate)
{
  g_return_if_fail(CDOS_IS_POPUP_MENU_BASE(obj));
  GList *items;
  CdosPopupBaseMenuItem *active_item;
  CdosPopupComboMenuPrivate *priv = CDOS_POPUP_COMBO_MENU(obj)->priv;
  gfloat source_x, source_y;
  CdosGlobal *global = cdos_global_get();//
  ClutterStage *stage = cdos_global_get_stage (global);
  //ClutterActor *actor;

  if(obj->is_open)
    return;
  obj->is_open = TRUE;
  items = cdos_popup_menu_base_get_menu_items(obj);
  active_item = g_list_nth_data(items, priv->active_item_pos);
  clutter_actor_get_transformed_position(CLUTTER_ACTOR(CDOS_POPUP_COMBO_MENU(obj)->actor), &source_x, &source_y);
  clutter_actor_set_position(CLUTTER_ACTOR(CLUTTER_ACTOR(CDOS_POPUP_COMBO_MENU(obj)->actor)),roundf(source_x),roundf(source_y - clutter_actor_get_y(menu_item_get_actor(active_item))));
  clutter_actor_raise_top(CLUTTER_ACTOR(CDOS_POPUP_COMBO_MENU(obj)->actor));
  //clutter_actor_set_opacity(CDOS_POPUP_COMBO_MENU(obj)->actor);
  clutter_actor_show(CLUTTER_ACTOR(CDOS_POPUP_COMBO_MENU(obj)->actor));
  //  Tweener.addTween(this.actor,
  //                   { opacity: 255,
  //                     transition: 'linear',
  //                     time: BoxPointer.POPUP_ANIMATION_TIME });
  CDOS_POPUP_COMBO_MENU(obj)->saved_focus_actor = clutter_stage_get_key_focus(stage);
  clutter_stage_set_key_focus(stage,CLUTTER_ACTOR(CDOS_POPUP_COMBO_MENU(obj)->actor));
  g_signal_emit_by_name(CDOS_POPUP_COMBO_MENU(obj),"open-state-changed", TRUE);
}

/**
 * cdos_popup_combo_menu_close:
 * @obj: A #CdosPopupMenuBase 
 *
 * 关闭menu
 *
 */
void
cdos_popup_combo_menu_close (CdosPopupMenuBase *obj,gboolean animate)
{
  g_return_if_fail(CDOS_IS_POPUP_MENU_BASE(obj));
  CdosGlobal *global = cdos_global_get();//
  ClutterStage *stage = cdos_global_get_stage (global);

  if(!obj->is_open)
    return;
  obj->is_open = FALSE;
  //  Tweener.addTween(this.actor,
  //                   { opacity: 0,
  //                     transition: 'linear',
  //                     time: BoxPointer.POPUP_ANIMATION_TIME,
  //                     onComplete: Lang.bind(this,
  //                         function() {
  //                             this.actor.hide();
      //                         })
     //                   });
  clutter_stage_set_key_focus(stage,CDOS_POPUP_COMBO_MENU(obj)->saved_focus_actor);
  g_signal_emit_by_name(CDOS_POPUP_COMBO_MENU(obj),"open-state-changed", TRUE);
}

/**
 * cdos_popup_combo_menu_set_active_item:
 * @self: A #CdosPopupComboMenu
 * @position: 位置 
 *
 * 设置self的活动item
 *
 */
void
cdos_popup_combo_menu_set_active_item (CdosPopupComboMenu *self, gint position)
{
  g_return_if_fail(CDOS_IS_POPUP_COMBO_MENU(self));
  CdosPopupComboMenuPrivate *priv = self->priv;
  priv->active_item_pos = position;
}

/**
 * cdos_popup_combo_menu_set_item_visible:
 * @self: A #CdosPopupComboMenu
 * @position: 位置 
 * @visible: 是否可见
 *
 * 设置menu某个位置的item是否可见
 *
 */
void
cdos_popup_combo_menu_set_item_visible (CdosPopupComboMenu *self, 
                                        gint                position,
				                                gboolean            visible)
{
  g_return_if_fail(CDOS_IS_POPUP_COMBO_MENU(self));
  GList   *items;
  gpointer active_item;
  CdosPopupComboMenuPrivate *priv = self->priv;

  if(!visible && position == priv->active_item_pos){
      g_print("Trying to hide the active menu item.");
      return;
  }
  items = cdos_popup_menu_base_get_menu_items(CDOS_POPUP_MENU_BASE(self));
  active_item = g_list_nth(items, position)->data;
  g_object_set(CLUTTER_ACTOR(active_item),"visible", visible, NULL);
}

/**
 * cdos_popup_combo_menu_get_item_visible:
 * @self: A #CdosPopupComboMenu
 * @position: 位置
 *
 * 获取某个位置的可见item项
 *
 * Returns: $gpointer
 */
gpointer
cdos_popup_combo_menu_get_item_visible (CdosPopupComboMenu *self, gint position)
{
  //return this._getMenuItems()[position].actor.visible;
  g_return_val_if_fail(CDOS_IS_POPUP_COMBO_MENU(self),NULL);
  GList *items;
  gpointer active_item;
  items = cdos_popup_menu_base_get_menu_items(CDOS_POPUP_MENU_BASE(self));
  active_item = g_list_nth(items, position)->data;
  return active_item;
}
