#include "cdos-popup-image-menu-item.h"

/**
 * SECTION: cdos-popup-image-menu-item.c
 * @Title: CdosPopupImageMenuItem
 * @short_description: 继承自CdosPopupBaseMenuItem，这个控件为我们做了一定的封装，如果你
 *　 希望在你的item里不仅仅显示文字，那么你可以使用这个控件。这个控件为我们添加了显示图片的功能。
 *
 * CdosPopupImageMenuItem 提供了一个可以显示图片的menuItem。
 */

static void 
cdos_popup_image_menu_item_finalize(GObject *gobject);

static void
cdos_popup_image_menu_item_dispose(GObject *gobject);

static void
cdos_popup_image_menu_item_init(CdosPopupImageMenuItem *item);

static void 
cdos_popup_image_menu_item_class_init(CdosPopupImageMenuItemClass *klass);

G_DEFINE_TYPE(CdosPopupImageMenuItem, cdos_popup_image_menu_item, CDOS_TYPE_POPUP_BASE_MENU_ITEM);

struct  _CdosPopupImageMenuItemPrivate
{
	StLabel  *label;
	StIcon   *icon;
};

static void
cdos_popup_image_menu_item_dispose(GObject *gobject){

	CdosPopupImageMenuItemPrivate *priv;
	priv = CDOS_POPUP_IMAGE_MENU_ITEM(gobject)->priv;

	g_clear_object(&priv->icon);
	g_clear_object(&priv->label);

	G_OBJECT_CLASS(cdos_popup_image_menu_item_parent_class)->dispose(gobject);
}

static void 
cdos_popup_image_menu_item_finalize(GObject *gobject)
{
	CdosPopupImageMenuItemPrivate *priv;
	priv = CDOS_POPUP_IMAGE_MENU_ITEM(gobject)->priv;

	G_OBJECT_CLASS(cdos_popup_image_menu_item_parent_class)->finalize(gobject);
}

static void
cdos_popup_image_menu_item_init(CdosPopupImageMenuItem *item)
{
	item->priv = G_TYPE_INSTANCE_GET_PRIVATE ((item), CDOS_TYPE_POPUP_IMAGE_MENU_ITEM, CdosPopupImageMenuItemPrivate);
	item->priv->label = (StLabel *)st_label_new(NULL);
	st_widget_set_style_class_name(item->priv->label,"popup-menu-item-label");
	item->priv->icon = (StIcon *)g_object_new(ST_TYPE_ICON,
				"style-class", "popup-menu-item-icon",
				 NULL);
	cdos_popup_base_menu_item_add_actor(CDOS_POPUP_BASE_MENU_ITEM(item), CLUTTER_ACTOR(item->priv->icon), NULL);
	cdos_popup_base_menu_item_add_actor(CDOS_POPUP_BASE_MENU_ITEM(item), CLUTTER_ACTOR(item->priv->label), NULL);

}

static void 
cdos_popup_image_menu_item_class_init(CdosPopupImageMenuItemClass *klass)
{
	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

	g_type_class_add_private(klass, sizeof(CdosPopupImageMenuItemPrivate));
	gobject_class->finalize = cdos_popup_image_menu_item_finalize;
	gobject_class->dispose = cdos_popup_image_menu_item_dispose;
}

/**
 * cdos_popup_image_menu_item_new:
 * 
 * 新建一个 #CdosPopupImageMenuItem 类的实例。
 *
 * Returns: 返回一个 #CdosPopupImageMenuItem 的实例。
 */
CdosPopupImageMenuItem *
cdos_popup_image_menu_item_new(void)
{
	return g_object_new(CDOS_TYPE_POPUP_IMAGE_MENU_ITEM, NULL);
}

/**
 * cdos_popup_image_menu_item_new_icon:
 * @text: 显示的text 
 * @iconName: 显示图片的iconName
 * @params: 初始化基类所需要的基本数据
 * 
 * 新建一个 #CdosPopupImageMenuItem 类的实例。
 *
 * Returns: 返回一个 #CdosPopupImageMenuItem 的实例。
 */
CdosPopupImageMenuItem *
cdos_popup_image_menu_item_new_with_params( const gchar *icon_name,const gchar *text)
{
	CdosPopupImageMenuItem 		*gobject;
	gobject = g_object_new(CDOS_TYPE_POPUP_IMAGE_MENU_ITEM, NULL);
	cdos_popup_image_menu_item_set_label(gobject, text);
	cdos_popup_image_menu_item_set_icon_by_gicon(gobject, icon_name);
	return gobject;
}

/**
 * cdos_popup_image_menu_item_get_label:
 * @item: A #CdosPopupImageMenuItem
 *
 * 获取item的label属性
 *
 * Returns: 获取CdosPopupImageMenuItem的lable
 */
StLabel *
cdos_popup_image_menu_item_get_label(CdosPopupImageMenuItem *item)
{
	CdosPopupImageMenuItemPrivate *priv;
	g_return_val_if_fail(CDOS_POPUP_IMAGE_MENU_ITEM(item),NULL);
	priv = item->priv;
	return priv->label;
}

/**
 * cdos_popup_image_menu_item_get_icon:
 * @item: A #CdosPopupImageMenuItem
 *
 * 获取item的icon属性
 *
 * Returns: 获取CdosPopupImageMenuItem的icon
 */
StIcon *
cdos_popup_image_menu_item_get_icon(CdosPopupImageMenuItem *item)
{
	CdosPopupImageMenuItemPrivate *priv;
	g_return_val_if_fail(CDOS_POPUP_IMAGE_MENU_ITEM(item),NULL);
	priv = item->priv;
	return priv->icon;
}

/**
 * cdos_popup_image_menu_item_set_icon:
 * @item: A #CdosPopupImageMenuItem
 * @icon_name: item的icon名称 
 *
 * 设置item的icon
 *
 */
void 
cdos_popup_image_menu_item_set_icon(CdosPopupImageMenuItem *item, const gchar *icon_name,StIconType icon_type)
{
	g_return_if_fail(CDOS_IS_POPUP_IMAGE_MENU_ITEM(item));
	CdosPopupImageMenuItemPrivate *priv = item->priv;
	st_icon_set_icon_name(priv->icon, icon_name);
	st_icon_set_icon_type(priv->icon,icon_type);
}

/**
 * cdos_popup_image_menu_item_set_label:
 * @item: A #CdosPopupImageMenuItem
 * @icon_name: item的label 
 *
 * 设置item的label
 *
 */
void 
cdos_popup_image_menu_item_set_label(CdosPopupImageMenuItem *item,
				     const gchar            *text)
{
	CdosPopupImageMenuItemPrivate 		       *priv;
	g_return_if_fail(CDOS_IS_POPUP_IMAGE_MENU_ITEM(item));
	priv = item->priv;
	if (priv->label) {
		st_label_set_text(priv->label, text);
	}
}
void
cdos_popup_image_menu_item_set_icon_by_gicon(CdosPopupImageMenuItem *item, const gchar *icon_name){
	g_return_if_fail(CDOS_IS_POPUP_IMAGE_MENU_ITEM(item));
	if(icon_name == NULL){
		return;
	}
	CdosPopupImageMenuItemPrivate *priv = item->priv;
	GError *error = NULL;
	GIcon *gicon = g_icon_new_for_string (icon_name, &error);
	if(error){
		g_warning("\n%s:can't get the gicon!\n",__func__);
		g_error_free(error);
		return ;
	}
	if (gicon){
		st_icon_set_gicon (priv->icon, gicon);
		g_object_unref (gicon);
	}
}
