#include "cdos-popup-menu-base.h"
#include "cdos-popup-sub-menu.h"
#include "cdos-popup-menu.h"
#include "cdos-popup-menu-section.h"
#include "cdos-popup-submenu-menu-item.h"
#include <string.h>
#include "cdos-debug.h"
#include "cdos-start.h"
#include "cdos-util.h"

/**
 * SECTION: cdos-popup-menu-base.c
 * @Title: CdosPopupMenuBase
 * @short_description: 菜单控件的一个抽象基类，我们不可以直接创建一个这样的对象。
 *　 
 *
 * CdosPopupImageMenuItem 实现菜单控件的基本功能
 */

gpointer
_obj_get_delegate (GObject *obj)
{
  return g_object_get_data (obj, "_delegate");
}

ClutterActor *
_menu_item_get_actor (GObject *obj)
{
  ClutterActor *actor;
  g_object_get(obj,
                "actor", &actor,
                NULL);
  return actor;
}

G_DEFINE_TYPE(CdosPopupMenuBase, cdos_popup_menu_base, G_TYPE_OBJECT);

#define POPUP_MENU_BASE_GET_PRIVATE( obj ) \
  (G_TYPE_INSTANCE_GET_PRIVATE( (obj), CDOS_TYPE_POPUP_MENU_BASE, CdosPopupMenuBasePrivate ) )
struct _CdosPopupMenuBasePrivate
{
    StWidget *source_actor;
    gchar *style_class;
    StBoxLayout *box;
    CdosPopupBaseMenuItem *active_menu_item;
    GList *child_menus;
};


enum
{
  PROP_0,

  PROP_SOURCE_ACTOR,
  PROP_STYLE_CLASS,
  PROP_BOX
};

enum
{
    ACTIVATE,
	ACTIVE_CHANGED,
  CHILD_MENU_ADDED,
  CHILD_MENU_REMOVED,
  MENU_ITEM_ADD,
  OPEN_STATE_CHANGED,
  DESTROY,

  LAST_SIGNAL
};

static guint cdos_popup_menu_base_signals[LAST_SIGNAL] = { 0, };

static void
cdos_popup_menu_base_set_box (CdosPopupMenuBase *self, StBoxLayout *box);


static void
cdos_popup_menu_base_dispose (GObject * gobject)
{
  CdosPopupMenuBase *self = CDOS_POPUP_MENU_BASE( gobject );
  CdosPopupMenuBasePrivate * priv = self->priv;

  g_signal_emit (self, cdos_popup_menu_base_signals[DESTROY], 0);
  if(priv->child_menus != NULL){
    GList *children_menus = priv->child_menus;
    while(children_menus != NULL){
      if(children_menus->data){
        cdos_popup_menu_base_remove_child_menu(self,children_menus->data);
      }
      children_menus = children_menus->next;
    }
    g_list_free (priv->child_menus);
  }
  //cdos_popup_menu_base_remove_all (self);
  //g_clear_object(&priv->box);
  G_OBJECT_CLASS(cdos_popup_menu_base_parent_class)->dispose (gobject);
}

static void
cdos_popup_menu_base_finalize (GObject * gobject)
{
  CdosPopupMenuBasePrivate * priv = CDOS_POPUP_MENU_BASE( gobject )->priv;
  g_free(priv->style_class);
  G_OBJECT_CLASS(cdos_popup_menu_base_parent_class)->finalize (gobject);
}

static void
cdos_popup_menu_base_set_property (GObject      *gobject, 
                                   guint         prop_id,
				                           const GValue *value, 
                                   GParamSpec   *pspec)
{
  CdosPopupMenuBase * obj = CDOS_POPUP_MENU_BASE(gobject);

  switch (prop_id)
    {
    case PROP_SOURCE_ACTOR:
      cdos_popup_menu_base_set_source_actor (obj, g_value_get_object (value));
      break;
    case PROP_STYLE_CLASS:
      cdos_popup_menu_base_set_style_class (obj, g_value_get_string (value));
      break;
    case PROP_BOX:
      cdos_popup_menu_base_set_box (obj, g_value_get_object (value));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID(gobject, prop_id, pspec);
      break;
    }
}

static void
cdos_popup_menu_base_get_property (GObject    *gobject, 
                                   guint       prop_id,
				                           GValue     *value, 
                                   GParamSpec *pspec)
{
  CdosPopupMenuBasePrivate * priv = CDOS_POPUP_MENU_BASE( gobject )->priv;

  switch (prop_id)
    {
    case PROP_SOURCE_ACTOR:
      g_value_set_object (value, priv->source_actor);
      break;
    case PROP_STYLE_CLASS:
      g_value_set_string (value, priv->style_class);
      break;
    case PROP_BOX:
      g_value_set_object (value, priv->box);
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID(gobject, prop_id, pspec);
      break;
    }
}

static ClutterActor *
cdos_popup_menu_base_get_actor(CdosPopupMenuBase *self)
{
  return NULL;
}
static void
cdos_popup_menu_base_open(CdosPopupMenuBase *obj, gboolean animate)
{
  /*do nothing*/
}
static void
cdos_popup_menu_base_close(CdosPopupMenuBase *obj, gboolean animate)
{
  /*do nothing*/
}

static void
cdos_popup_menu_base_class_init (CdosPopupMenuBaseClass * klass)
{
  GObjectClass * gobject_class = G_OBJECT_CLASS(klass);
  GParamSpec * pspec;

  gobject_class->finalize = cdos_popup_menu_base_finalize;
  gobject_class->dispose = cdos_popup_menu_base_dispose;
  gobject_class->set_property = cdos_popup_menu_base_set_property;
  gobject_class->get_property = cdos_popup_menu_base_get_property;

  klass->get_actor = cdos_popup_menu_base_get_actor;
  klass->open = cdos_popup_menu_base_open;
  klass->close = cdos_popup_menu_base_close;

  g_type_class_add_private (klass, sizeof(CdosPopupMenuBasePrivate));

  pspec = g_param_spec_string ("style-class",
			       "StyleClass",
			       "StyleClass of the object",
				   	   	   	   0,
			       G_PARAM_READWRITE);
  g_object_class_install_property (gobject_class, PROP_STYLE_CLASS, pspec);

  pspec = g_param_spec_object ("source_actor",
			       "SourceActor",
			       "SourceActor of the object",
				   	 ST_TYPE_WIDGET,
			       G_PARAM_READWRITE);
  g_object_class_install_property (gobject_class, PROP_SOURCE_ACTOR, pspec);

  pspec = g_param_spec_object ("box",
			       "Box",
			       "Box of the object",
				      ST_TYPE_BOX_LAYOUT,
			       G_PARAM_READWRITE);
  g_object_class_install_property (gobject_class, PROP_BOX, pspec);

	cdos_popup_menu_base_signals[ACTIVATE] = g_signal_new (
			"activate",
            CDOS_TYPE_POPUP_MENU_BASE,
			G_SIGNAL_RUN_LAST,
            0,
			NULL,
			NULL, 
			_st_marshal_VOID__POINTER_BOOLEAN,
			G_TYPE_NONE,
		   	2,
			G_TYPE_POINTER,
			G_TYPE_BOOLEAN);

  cdos_popup_menu_base_signals[ACTIVE_CHANGED] = g_signal_new (
      "active-changed", G_TYPE_FROM_CLASS(klass), G_SIGNAL_RUN_LAST,
      G_STRUCT_OFFSET(CdosPopupMenuBaseClass, active_changed),
      NULL,
      NULL, g_cclosure_marshal_VOID__OBJECT,
      G_TYPE_NONE,
      1,
	  G_TYPE_OBJECT);

  cdos_popup_menu_base_signals[CHILD_MENU_ADDED] = g_signal_new (
      "child-menu-added", G_TYPE_FROM_CLASS(klass), G_SIGNAL_RUN_LAST,
      G_STRUCT_OFFSET(CdosPopupMenuBaseClass, child_menu_added),
      NULL,
      NULL, g_cclosure_marshal_VOID__OBJECT,
      G_TYPE_NONE,
	           1,
      G_TYPE_OBJECT);

  cdos_popup_menu_base_signals[CHILD_MENU_REMOVED] = g_signal_new (
      "child-menu-removed", G_TYPE_FROM_CLASS(klass), G_SIGNAL_RUN_LAST,
      G_STRUCT_OFFSET(CdosPopupMenuBaseClass, child_menu_removed),
      NULL,
      NULL, g_cclosure_marshal_VOID__VOID,
      G_TYPE_NONE,
      0);

  cdos_popup_menu_base_signals[DESTROY] = g_signal_new (
      "destroy", G_TYPE_FROM_CLASS(klass), G_SIGNAL_RUN_LAST,
      G_STRUCT_OFFSET(CdosPopupMenuBaseClass, destroy),
      NULL,
      NULL, g_cclosure_marshal_VOID__VOID,
      G_TYPE_NONE,
      0);

  cdos_popup_menu_base_signals[MENU_ITEM_ADD] = g_signal_new (
      "menu-item-add", G_TYPE_FROM_CLASS(klass), G_SIGNAL_RUN_LAST,
      G_STRUCT_OFFSET(CdosPopupMenuBaseClass, menu_item_add),
      NULL,
      NULL, g_cclosure_marshal_VOID__VOID,
      G_TYPE_NONE,
      0);

  cdos_popup_menu_base_signals[OPEN_STATE_CHANGED] = g_signal_new (
      "open-state-changed", G_TYPE_FROM_CLASS(klass), G_SIGNAL_RUN_LAST,
      G_STRUCT_OFFSET(CdosPopupMenuBaseClass, open_state_changed),
      NULL,
      NULL, g_cclosure_marshal_VOID__BOOLEAN,
      G_TYPE_NONE,
	  	  	   1,
      G_TYPE_BOOLEAN);
}

static void
foreach_queue_relayout (ClutterActor *item)
{
  clutter_actor_queue_relayout (item);
}

static void
cdos_popup_menu_base_menu_queue_relayout (ClutterActor *actor,
                                          CdosPopupMenuBase  *self)
{
  CdosPopupMenuBasePrivate *priv = self->priv;

  GList *children = clutter_actor_get_children (CLUTTER_ACTOR(priv->box));
  g_list_foreach(children, (GFunc)foreach_queue_relayout, NULL);
  g_list_free(children);
}

static void _style_class_changed_cb(CdosPopupMenuBase *self,
                                    GParamSpec        *spec) 
{

	CdosPopupMenuBasePrivate * priv;
	priv = self->priv;

	if (!priv->style_class)
		st_widget_set_style_class_name(ST_WIDGET(priv->box), priv->style_class);

}

static void
cdos_popup_menu_base_init (CdosPopupMenuBase * self)
{
  CdosPopupMenuBasePrivate * priv;
  StBoxLayout *box;

  priv = self->priv = POPUP_MENU_BASE_GET_PRIVATE(self);

  box = ST_BOX_LAYOUT(st_box_layout_new());
  priv->box = box;
  self->box = box;
  st_box_layout_set_vertical (priv->box, TRUE);
  st_widget_set_style (ST_WIDGET(priv->box), "overflow:hidden;");

  g_signal_connect(priv->box, "queue-relayout",
		   G_CALLBACK(cdos_popup_menu_base_menu_queue_relayout), self);

  priv->style_class = NULL;
  priv->active_menu_item = NULL;
  self->length = 0;
  self->is_open = FALSE;
  self->block_source_events = FALSE;
  self->pass_events = FALSE;
  priv->child_menus = NULL;

  g_signal_connect(self, "notify::style-class", G_CALLBACK(_style_class_changed_cb),NULL);

}

/**
 * cdos_popup_menu_base_set_source_actor:
 * @self: A #CdosPopupMenuBase
 * @actor: 设置的source_actor 
 *
 * 设置self的source_actor
 *
 */
void
cdos_popup_menu_base_set_source_actor (CdosPopupMenuBase *self, StWidget *actor)
{
  CdosPopupMenuBasePrivate *priv;
  g_return_if_fail(CDOS_IS_POPUP_MENU_BASE(self));
  priv = self->priv;

  //clutter_actor_destroy((ClutterActor *)priv->source_actor);
  if (actor)
    priv->source_actor = actor;
  else
    priv->source_actor = NULL;
}

/**
 * cdos_popup_menu_base_get_source_actor:
 * @self: A #CdosPopupMenuBase
 *
 * 获取self的source_actor
 *
 * Returns: ＃StWidget
 */
StWidget *
cdos_popup_menu_base_get_source_actor (CdosPopupMenuBase *self)
{
  g_return_val_if_fail(CDOS_IS_POPUP_MENU_BASE(self), NULL);
  return self->priv->source_actor;
}

/**
 * cdos_popup_menu_base_set_style_class:
 * @self: A #CdosPopupMenuBase
 *
 * 设置self的style_class（样式,默认配置在css文件中）
 *
 */
void
cdos_popup_menu_base_set_style_class (CdosPopupMenuBase *self,
                                      const gchar       *style_class)
{
  CdosPopupMenuBasePrivate * priv;
  g_return_if_fail(CDOS_IS_POPUP_MENU_BASE( self ));

  priv = self->priv;
  if(priv->style_class){
	g_free (priv->style_class);
  }
  if (style_class)
    priv->style_class = g_strdup (style_class);
  else
    priv->style_class = NULL;
}

/**
 * cdos_popup_menu_base_get_style_class:
 * @self: A #CdosPopupMenuBase
 *
 * 获取self的style_class（样式,默认配置在css文件中）
 *
 * Returns: CdosPopupMenuBase的样式＃gchar * 
 */
const gchar *
cdos_popup_menu_base_get_style_class (CdosPopupMenuBase * self)
{
  g_return_val_if_fail(CDOS_IS_POPUP_MENU_BASE (self), NULL);
  return (self->priv->style_class);
}

/**
 * cdos_popup_menu_base_set_box:
 * @self: A #CdosPopupMenuBase
 * @box: 需要设置的box 
 *
 * 设置self的box
 *
 */
static void
cdos_popup_menu_base_set_box (CdosPopupMenuBase *self, StBoxLayout *box)
{
  CdosPopupMenuBasePrivate *priv;
  g_return_if_fail(CDOS_IS_POPUP_MENU_BASE(self));
  priv = self->priv;

  if(priv->box){
    g_object_unref(priv->box);
  }
  if (box)
  {
    priv->box = box;
    self->box = box;
  }
  else
  {
    priv->box = NULL;
    self->box = NULL;
  }
}

/**
 * cdos_popup_menu_base_get_box:
 * @self: A #CdosPopupMenuBase
 *
 * 获取self的box成员
 *
 * Returns: #StBoxLayout
 */
StBoxLayout *
cdos_popup_menu_base_get_box(CdosPopupMenuBase *self)
{
  g_return_val_if_fail(CDOS_IS_POPUP_MENU_BASE(self), NULL);
  return self->priv->box;
}

/**
 * cdos_popup_menu_base_new:
 * 
 * 新建一个 #CdosPopupMenuBase 类的实例。
 *
 * Returns: 返回一个 #CdosPopupMenuBase 的实例。
 */
CdosPopupMenuBase *
cdos_popup_menu_base_new (void)
{
  CdosPopupMenuBase *self;
  self = g_object_new ( CDOS_TYPE_POPUP_MENU_BASE, NULL);
  return self;
}

static void
sub_menu_activate_callback (CdosPopupMenuBase *menu,
                            ClutterEvent      *event,
                            gboolean           keep_menu,
                            CdosPopupMenuBase *self)
{
  CdosPopupMenuBaseClass *obj_class = CDOS_POPUP_MENU_BASE_GET_CLASS(self);
  g_signal_emit (self, cdos_popup_menu_base_signals[ACTIVATE], 0, event, keep_menu);

  if (!keep_menu)
    obj_class->close(self, TRUE);
}

static void
sub_menu_active_change_callback (CdosPopupSubMenu  *menu,
                                   CdosPopupBaseMenuItem *menu_item,
                                   CdosPopupMenuBase *self)
{
  CdosPopupMenuBasePrivate *priv = self->priv;

  if (priv->active_menu_item != menu_item)
    {
      if (priv->active_menu_item)
    	      cdos_popup_base_menu_item_set_active (priv->active_menu_item, FALSE);
      priv->active_menu_item = menu_item;
      g_signal_emit_by_name (self, "active-changed", menu_item);
    }
}

static void
cdos_popup_menu_base_connect_sub_menu_signals (CdosPopupMenuBase     *self,
                                               CdosPopupBaseMenuItem *object,
                                               CdosPopupSubMenu *menu)
{
  gulong sub_menu_activate_id, sub_menu_activate_change_id;
  sub_menu_activate_id = g_signal_connect(
      menu, "activate", G_CALLBACK(sub_menu_activate_callback), self);

  sub_menu_activate_change_id = g_signal_connect(
                        menu, "active-changed",
                        G_CALLBACK(sub_menu_active_change_callback),
                        self);

  g_object_set_data(G_OBJECT(object),"sub_menu_activate_id", GSIZE_TO_POINTER (sub_menu_activate_id));
  g_object_set_data(G_OBJECT(object), "sub_menu_activate_change_id", GSIZE_TO_POINTER (sub_menu_activate_change_id));
}

static void
menu_item_active_changed_callback (CdosPopupBaseMenuItem          *menu_item,
                                   gboolean           active,
                                   CdosPopupMenuBase *self)
{
  CdosPopupMenuBasePrivate *priv = self->priv;

  if (active && priv->active_menu_item != menu_item)
    {
      if (priv->active_menu_item)
    	      cdos_popup_base_menu_item_set_active (priv->active_menu_item, FALSE);
      priv->active_menu_item = menu_item;
      g_signal_emit_by_name (self, "active-changed", menu_item);
    }
  else if (!active && priv->active_menu_item == menu_item)
    {
      priv->active_menu_item = NULL;
      g_signal_emit_by_name (self, "active-changed", NULL);
    }
}

static void
menu_item_sensitive_change_callback (CdosPopupBaseMenuItem *menu_item,
                                     gboolean sensitive,
                                     CdosPopupMenuBase *self)
{
  CdosPopupMenuBaseClass *obj_class =  CDOS_POPUP_MENU_BASE_GET_CLASS(self);
  CdosPopupMenuBasePrivate *priv = self->priv;
  CdosGlobal *global = cdos_global_get();
  ClutterStage *stage = cdos_global_get_stage (global);
  ClutterActor *self_actor = obj_class->get_actor(self);
  if(self_actor == NULL){
      g_print("%s:self_actor is not exist!",__func__);
      return ;
  }
  if (!sensitive && priv->active_menu_item == menu_item) {
      if (st_widget_navigate_focus (ST_WIDGET(self_actor), CLUTTER_ACTOR(menu_item_get_actor(menu_item)),
				    GTK_DIR_TAB_FORWARD,
				    TRUE))
	      clutter_actor_grab_key_focus (CLUTTER_ACTOR(self_actor));
  } else if (sensitive && priv->active_menu_item == NULL) {
      if (clutter_stage_get_key_focus (stage) == self_actor)
	    clutter_actor_grab_key_focus (CLUTTER_ACTOR(menu_item_get_actor(menu_item)));
  }
}

static void
menu_item_destroy_callback (CdosPopupBaseMenuItem *menu_item, CdosPopupMenuBase *self)
{
  CdosPopupMenuBasePrivate *priv = self->priv;
  //原JS判断 menu_item->menu
  if (CDOS_IS_POPUP_SUBMENU_MENU_ITEM(menu_item)) {
      gulong sub_menu_activate_id,
             sub_menu_activate_change_id,
             closing_id;
      sub_menu_activate_id = GPOINTER_TO_SIZE (g_object_get_data(G_OBJECT(menu_item), "sub_menu_activate_id"));
      sub_menu_activate_change_id = GPOINTER_TO_SIZE (g_object_get_data(G_OBJECT(menu_item), "sub_menu_activate_change_id"));
      closing_id = GPOINTER_TO_SIZE (g_object_get_data(G_OBJECT(menu_item), "closing_id"));
      CdosPopupSubMenu *submenu;
      submenu = cdos_popup_submenu_menu_item_get_menu ((CdosPopupSubmenuMenuItem *)menu_item);
      if(sub_menu_activate_id){
          g_signal_handler_disconnect (submenu, sub_menu_activate_id);
      }
      if(sub_menu_activate_change_id){
          g_signal_handler_disconnect (submenu, sub_menu_activate_change_id);
      }
      if(closing_id){
          g_signal_handler_disconnect (self, closing_id);
      }
  }
  //gulong item_activate = GPOINTER_TO_SIZE (g_object_get_data ((GObject *)menu_item, "activate_id"));
  //g_signal_handler_disconnect (menu_item, item_activate);
  if (menu_item == priv->active_menu_item)
      priv->active_menu_item = NULL;
}

static void
menu_item_activate_callback (CdosPopupBaseMenuItem *menu_item,
                             ClutterEvent *event,
                             gboolean keep_menu,
                             CdosPopupMenuBase *self)
{ 
  g_debug ("ref count: %d %d %p", 
             ((GObject *)menu_item)->ref_count,
             ((GObject *)self)->ref_count,
             menu_item);
  if (!CDOS_IS_POPUP_MENU_BASE (self))
  {
      g_warn_if_reached ();
      return;
  }

  CdosPopupMenuBaseClass *obj_class = CDOS_POPUP_MENU_BASE_GET_CLASS(self);
  g_signal_emit (self, cdos_popup_menu_base_signals[ACTIVATE], 0, event, keep_menu);

  if(!keep_menu){
      obj_class->close(self,TRUE);
  }
}

static void
cdos_popup_menu_base_connect_item_signals (CdosPopupMenuBase     *self,
					                       CdosPopupBaseMenuItem *menu_item)
{
  g_signal_connect(menu_item, 
                "active-changed",
                G_CALLBACK(menu_item_active_changed_callback), 
                self);

  g_signal_connect(menu_item, 
                  "sensitive-changed",
                  G_CALLBACK(menu_item_sensitive_change_callback), 
                  self);

  g_signal_connect(menu_item,
                   "destroy", 
                   G_CALLBACK(menu_item_destroy_callback),
                   self);

  //gulong item_activate;
  g_signal_connect(menu_item,"activate",G_CALLBACK(menu_item_activate_callback),self);
  //g_object_set_data ((GObject *)menu_item, "activate_id", GSIZE_TO_POINTER (item_activate));
}

static void
cdos_popup_menu_base_update_separator_visibility (CdosPopupMenuBase * self,
						CdosPopupBaseMenuItem * menu_item)
{
  CdosPopupMenuBasePrivate *priv = self->priv;
  GList *children = st_container_get_children_list ((StContainer *)priv->box);
  ClutterActor *actor = (ClutterActor *)cdos_popup_base_menu_item_get_actor (menu_item);
  GList *index = g_list_find (children, actor);
  GList *child_before_index, *child_after_index;

  if (index) {
      child_before_index = index->prev;

      while (child_before_index
              && !clutter_actor_is_visible((ClutterActor *)child_before_index->data))
      {
        child_before_index = child_before_index->prev;
      }
      if (child_before_index == NULL ||
              CDOS_IS_POPUP_SEPARATOR_MENU_ITEM (obj_get_delegate (child_before_index->data)))
      {
          clutter_actor_hide (actor);
          return;
      }
      child_after_index = index->next;
      while (child_after_index
              && !clutter_actor_is_visible((ClutterActor *)child_after_index->data))
          child_after_index = child_after_index->next;
      if (child_after_index == NULL ||
              CDOS_IS_POPUP_SEPARATOR_MENU_ITEM (obj_get_delegate (child_after_index->data)))
      {
          clutter_actor_hide (actor);
          return;
      }
      clutter_actor_show (actor);
  }
}

/**
 * cdos_popup_menu_base_get_menu_items:
 * @self: A #CdosPopupMenuBase
 *
 * 获取self的所有子控件
 *
 * Returns: ＃GList
 */
GList *
cdos_popup_menu_base_get_menu_items (CdosPopupMenuBase * self)
{
  g_return_val_if_fail(CDOS_IS_POPUP_MENU_BASE(self), NULL);
  CdosPopupMenuBasePrivate *priv = self->priv;
  GList *children = st_container_get_children_list ((StContainer *)priv->box);
  //GList *l = NULL;
  //GList *delegates = NULL;
  ClutterActor *child = NULL;

  GList *result = NULL;
  for (; children; children = children->next)
  {
      child = children->data;
      gpointer delegate = obj_get_delegate(child);
      if (delegate)
          if (CDOS_IS_POPUP_BASE_MENU_ITEM (delegate) || CDOS_IS_POPUP_MENU_SECTION (delegate))
              result = g_list_prepend (result, delegate);
  }
  result = g_list_reverse (result);
  return result;
}

/**
 * cdos_popup_menu_base_add_action:
 * @self: A #CdosPopupMenuBase
 * @title: 添加的menuitem显示的text 
　* @callback: 点击时回调的函数
　* @data: 传递给回调函数的数据
 *
 * 为CdosPopupMenuBase添加一个可以点击的item，我们可以指定显示item的信息，还有
 *　就是点击之后响应的动作
 *
 * Returns: $CdosPopupMenuItem
 */
CdosPopupMenuItem *
cdos_popup_menu_base_add_action (CdosPopupMenuBase *self, 
                                 const gchar *title,
                                 GCallback callback,
                                 gpointer user_data)
{
  g_return_val_if_fail(CDOS_IS_POPUP_MENU_BASE(self), NULL);
  CdosPopupMenuItem * menu_item;
  // js中为 PopupMenuItem;
  menu_item = cdos_popup_menu_item_new();
  cdos_popup_menu_item_set_label(menu_item, title);  
  g_signal_connect((CdosPopupBaseMenuItem *)menu_item, "activate", G_CALLBACK(callback),user_data);
  cdos_popup_menu_base_add_menu_item(self, (CdosPopupBaseMenuItem *)menu_item, -1);
  return menu_item;
}

static void
add_commandline_action_callback (CdosPopupBaseMenuItem *item, 
                                 ClutterEvent *e,
                                 gboolean k,
                                 const gchar *cmd)
{
  gint *argcp = 0;
  gchar ***argvp = NULL;
  GError **error = NULL;
  g_shell_parse_argv (cmd, argcp, argvp, error);
  //g_print("\n cmd = %s \n",cmd);
  cdos_util_spawn_command_line (cmd);
}

/**
 * cdos_popup_menu_base_add_settings_action:
 * @self: A #CdosPopupMenuBase
 * @title: 添加的menuitem显示的text 
　* @module:　设置gsetting对应的域名
 *
 * 当我们希望为CdosPopupMenuBase中添加一个item,并且这个item点击的目的
 *　是对一gsetting的值进行修改，这个函数就能满足你的需求
 *
 * Returns: $CdosPopupMenuItem
 */
CdosPopupMenuItem *
cdos_popup_menu_base_add_settings_action (CdosPopupMenuBase *self,
                                          const gchar *title,
                                          const gchar *module)
{
  g_return_val_if_fail(CDOS_IS_POPUP_MENU_BASE(self), NULL);
  const gchar *command_line = "cdos-settings ";
  gchar *res = g_strconcat (command_line, module, NULL);
  CdosPopupMenuItem *item = cdos_popup_menu_base_add_action (self, title, G_CALLBACK(add_commandline_action_callback), res);
  return item;
}

/**
 * cdos_popup_menu_base_add_settings_action:
 * @self: A #CdosPopupMenuBase
 * @title: 添加的menuitem显示的text 
　* @cmd:　你期望执行的命令
 *
 * 当我们希望为CdosPopupMenuBase中添加一个item,并且这个item点击的目的
 *　是执行一个命令时，这个函数就能满足你的需求
 *
 * Returns: $CdosPopupMenuItem
 */
CdosPopupMenuItem *
cdos_popup_menu_base_add_commandline_action (CdosPopupMenuBase *self, 
                                             const gchar *title,
                                             gchar *cmd)
{
  g_return_val_if_fail(CDOS_IS_POPUP_MENU_BASE(self), NULL);
  gchar *command_line = cmd;
  return cdos_popup_menu_base_add_action (self, title, G_CALLBACK(add_commandline_action_callback), command_line);
}

/**
 * cdos_popup_menu_base_is_child_menu:
 * @self: A #CdosPopupMenuBase
 * @menu: 需要判断是否是子控件的CdosPopupMenuBase
 *
 * 判断一个CdosPopupMenuBase是否是另一个CdosPopupMenuBase的子控件
 *
 * Returns: gboolean 　True 是子控件，False 则不是
 */
gboolean
cdos_popup_menu_base_is_child_menu (CdosPopupMenuBase *self, CdosPopupMenuBase *menu)
{
  g_return_val_if_fail(CDOS_IS_POPUP_MENU_BASE(self), FALSE);
  CdosPopupMenuBasePrivate *priv = self->priv;
  return (g_list_index (priv->child_menus, menu) != -1);
}

/**
 * cdos_popup_menu_base_remove_child_menu:
 * @self: A #CdosPopupMenuBase
 * @menu: 需要添加到父控件CdosPopupMenuBase中的menu
 *
 * 将一个CdosPopupMenuBase添加到另一个CdosPopupMenuBase中
 *
 */
void
cdos_popup_menu_base_add_child_menu (CdosPopupMenuBase *self, CdosPopupMenuBase *menu)
{
  g_return_if_fail(CDOS_IS_POPUP_MENU_BASE(self));
  g_return_if_fail(CDOS_IS_POPUP_MENU_BASE(menu));

  CdosPopupMenuBasePrivate *priv = self->priv;
  if (cdos_popup_menu_base_is_child_menu (self, menu))
      return;
  priv->child_menus = g_list_append (priv->child_menus, g_object_ref(menu));

  g_signal_emit (self, cdos_popup_menu_base_signals[CHILD_MENU_ADDED], 0, menu);
}

/**
 * cdos_popup_menu_base_remove_child_menu:
 * @self: A #CdosPopupMenuBase
 * @menu: 需要从 父控件CdosPopupMenuBase中移除的menu
 *
 * 将一个CdosPopupMenuBase控件从其父控件CdosPopupMenuBase中移除
 *
 */
void
cdos_popup_menu_base_remove_child_menu (CdosPopupMenuBase *self, CdosPopupMenuBase *menu)
{
  g_return_if_fail(CDOS_IS_POPUP_MENU_BASE(self));
  g_return_if_fail(CDOS_IS_POPUP_MENU_BASE(menu));

  CdosPopupMenuBasePrivate *priv = self->priv;
  priv->child_menus = g_list_remove (priv->child_menus, menu);
  g_signal_emit (self, cdos_popup_menu_base_signals[CHILD_MENU_REMOVED], 0, menu);
  g_object_unref(menu);
}

/*
 * g_signal_connect callbacks .
 *
 */
static void
menu_section_destroy_callback (CdosPopupBaseMenuItem *menu_item, CdosPopupMenuBase *self)
{
  gulong *sub_menu_activate_id, *sub_menu_activate_change_id;
  sub_menu_activate_id = g_object_get_data(G_OBJECT(menu_item), "sub_menu_activate_id");
  sub_menu_activate_change_id = g_object_get_data(G_OBJECT(menu_item), "sub_menu_activate_change_id");
  if(sub_menu_activate_id)
    g_signal_handler_disconnect (menu_item, *sub_menu_activate_id);
  if(sub_menu_activate_change_id)
    g_signal_handler_disconnect (menu_item, *sub_menu_activate_change_id);

  self->length--;
}

static void
sub_menu_menu_item_open_state_changed_callback (CdosPopupMenuBase *self,
						                                    gboolean open)
{
  CdosPopupMenuBaseClass *obj_class = CDOS_POPUP_MENU_BASE_GET_CLASS(self);
  if (open)
    obj_class->close(self, FALSE);
}

static void
separator_menu_item_open_state_changed_callback (CdosPopupMenuBase     *self,
                                                 gboolean               open,
                                                 CdosPopupBaseMenuItem *menu_item)
{
  cdos_popup_menu_base_update_separator_visibility (self, menu_item);
}

/**
 * cdos_popup_menu_base_add_menu_item:
 * @self: A #CdosPopupMenuBase
 * @menu_item: 要添加到menu中的item #CdosPopupBaseMenuItem
 * @position: 添加的位置，-1为默认不指定
 *
 * 将CdosPopupBaseMenuItem添加到CdosPopupMenuBase中，并且可以指定添加的位置
 *
 */
void
cdos_popup_menu_base_add_menu_item (CdosPopupMenuBase     *self,
                                    CdosPopupBaseMenuItem *menu_item,
                                    gint position)
{
  g_return_if_fail(CDOS_IS_POPUP_MENU_BASE(self));
  g_return_if_fail(CDOS_IS_POPUP_BASE_MENU_ITEM(menu_item));

  CdosPopupMenuBasePrivate *priv = self->priv;
  CdosPopupBaseMenuItem *before_item = NULL;
  GList *items;
  guint items_length;
  gulong closing_id;
  CdosPopupSubMenu *menu_item_menu;
  ClutterActor *actor;

  if (position == -1)
  {
    //this.box.add(menuItem.actor);
    // js中.add方法暂时用clutter_container_add
    actor = (ClutterActor *)cdos_popup_base_menu_item_get_actor(menu_item);
    st_container_add_actor((ClutterContainer *)priv->box, actor);

    //clutter_actor_show(actor);
  }
  else
  {
    //cdos_popup_menu_base_get_menu_items 返回glist
    // 其中包括两种类型，CDOS_POPUP_MENU_SECTION 和 CDOS_IS_POPUP_BASE_MENU_ITEM
    //items = cdos_popup_menu_base_get_menu_items(self);
	if(CDOS_IS_POPUP_BASE_MENU_ITEM(menu_item))
    {
        actor = CLUTTER_ACTOR(cdos_popup_base_menu_item_get_actor(menu_item));
	    st_box_layout_insert_actor((StBoxLayout *)priv->box, actor, position);
     }
    /*
    items_length = g_list_length(items);
    if (position < items_length)
    {
        before_item = (CdosPopupBaseMenuItem *)g_list_nth_data(items, position);
        st_box_layout_insert_before (ST_BOX_LAYOUT(priv->box),
                CLUTTER_ACTOR(menu_item_get_actor(menu_item)),
                CLUTTER_ACTOR(menu_item_get_actor(before_item)));
    }
    else
        st_container_add_actor((ClutterContainer *)priv->box, CLUTTER_ACTOR(menu_item_get_actor(menu_item)));
    */
  }
  g_signal_emit (self, cdos_popup_menu_base_signals[MENU_ITEM_ADD], 0);

  if (CDOS_IS_POPUP_MENU_SECTION (menu_item))
  {
      // FIXME
    //cdos_popup_menu_base_connect_sub_menu_signals (self, menu_item, (CdosPopupBaseMenuItem *)menu_item);
    g_signal_connect(menu_item, "destroy",
	       G_CALLBACK(menu_section_destroy_callback), self);

  }
  else if (CDOS_IS_POPUP_SUBMENU_MENU_ITEM(menu_item))
  {
    //g_object_get(menu_item, "menu", &menu_item_menu, NULL);
    CdosPopupSubMenu *menu = cdos_popup_submenu_menu_item_get_menu ((CdosPopupSubmenuMenuItem *)menu_item);
    ClutterActor *menu_actor = CLUTTER_ACTOR(CDOS_POPUP_SUB_MENU(menu)->actor);
    if (before_item == NULL)
      st_container_add_actor((ClutterContainer *)priv->box, menu_actor);
    else
      st_box_layout_insert_before (ST_BOX_LAYOUT(priv->box),
				     CLUTTER_ACTOR(menu_actor),
				     CLUTTER_ACTOR(cdos_popup_base_menu_item_get_actor(before_item)));
    cdos_popup_menu_base_connect_sub_menu_signals (self, menu_item, menu);
    cdos_popup_menu_base_connect_item_signals (self, menu_item);

    closing_id = g_signal_connect(self, "open-state-changed", G_CALLBACK(sub_menu_menu_item_open_state_changed_callback), NULL);

    g_object_set_data(G_OBJECT(menu_item), "closing-id", GSIZE_TO_POINTER (closing_id));
  }
  else if (CDOS_IS_POPUP_SEPARATOR_MENU_ITEM (menu_item))
  {
    cdos_popup_menu_base_connect_item_signals (self, menu_item);
    // updateSeparatorVisibility needs to get called any time the
    // separator's adjacent siblings change visibility or position.
    // open-state-changed isn't exactly that, but doing it in more
    // precise ways would require a lot more bookkeeping.
    g_signal_connect(self, "open-state-changed",
  		  G_CALLBACK(separator_menu_item_open_state_changed_callback), menu_item);
  }
  else if (CDOS_IS_POPUP_BASE_MENU_ITEM (menu_item))
  {
    cdos_popup_menu_base_connect_item_signals (self, menu_item);
  }
  else
  {
    //throw TypeError("Invalid argument to PopupMenuBase.addMenuItem()");
  }
  self->length++;
}

/**
 * cdos_popup_menu_base_add_actor:
 * @self: A #CdosPopupMenuBase
 * @actor: 需要添加的ClutterActor 
 *
 * 将一个ClutterActor添加到CdosPopupMenuBase对象的box中
 *
 */
void
cdos_popup_menu_base_add_actor (CdosPopupMenuBase *self, ClutterActor *actor)
{
    g_return_if_fail(CDOS_IS_POPUP_MENU_BASE(self));
    g_return_if_fail(CLUTTER_IS_ACTOR(actor));
    //wj:st_container_add_actor ((ClutterContainer *)self->box, actor);
    st_container_add_actor ((ClutterContainer *)self->priv->box, actor);
}

static void 
remove_all_foreach(ClutterActor *data, StBoxLayout *user_data)
{
    ClutterActor *child = data;
    ClutterActor *parent = clutter_actor_get_parent(child);
    if(parent == CLUTTER_ACTOR(user_data)) {
        //st_container_remove_actor((ClutterContainer *) user_data, child);
        clutter_actor_destroy (child);
    }
}

/**
 * cdos_popup_menu_base_remove_all:
 * @self: A #CdosPopupMenuBase
 *
 * 移除CdosPopupMenuBase中所有的控件
 *
 */
void
cdos_popup_menu_base_remove_all (CdosPopupMenuBase *self)
{
    g_return_if_fail(NULL != self && CDOS_IS_POPUP_MENU_BASE(self));
    CdosPopupMenuBasePrivate *priv = self->priv;
    GList* children = st_container_get_children_list(ST_CONTAINER(priv->box));
    g_list_foreach(children, (GFunc)remove_all_foreach, priv->box);
}

/**
 * cdos_popup_menu_base_toggle:
 * @self: A #CdosPopupMenuBase 
 *
 * 改变一个CdosPopupMenuBase的打开和关闭状态
 * 如果CdosPopupMenuBase是打开的，函数则会将CdosPopupMenuBase关闭
 *　反之亦然
 */
void
cdos_popup_menu_base_toggle (CdosPopupMenuBase *self)
{
  CdosPopupMenuBasePrivate *priv = self->priv;
  CdosPopupMenuBaseClass *obj_class = CDOS_POPUP_MENU_BASE_GET_CLASS(self);
  if (self->is_open)
    obj_class->close(self, TRUE);
  else
    obj_class->open(self, TRUE);
}

/**
 * cdos_popup_menu_base_toggle:
 * @self: A #CdosPopupMenuBase 
 * @animate: ??
 * @onComplete:  ??
 *
 * 改变一个CdosPopupMenuBase的打开和关闭状态
 * 如果CdosPopupMenuBase是打开的，函数则会将CdosPopupMenuBase关闭
 *　反之亦然
 */
void
cdos_popup_menu_base_toggle_with_options (CdosPopupMenuBase *self, 
                                          gboolean animate,
                                          void *onComplete)
{
  CdosPopupMenuBasePrivate *priv = self->priv;
  CdosPopupMenuBaseClass *obj_class = CDOS_POPUP_MENU_BASE_GET_CLASS(self);
  if (self->is_open)
    obj_class->close (self, animate);
  else
    obj_class->open (self, animate);
}

/**
 * cdos_popup_menu_base_destroy:
 * @self: A #CdosPopupMenuBase
 *
 * 销毁CdosPopupMenuBase
 *
 */
void
cdos_popup_menu_base_destroy (CdosPopupMenuBase *self)
{
  CdosPopupMenuBaseClass *obj_class =  CDOS_POPUP_MENU_BASE_GET_CLASS(self);
  cdos_popup_menu_base_remove_all (self);
  ClutterActor *actor = obj_class->get_actor(self);
  if(actor){
      clutter_actor_destroy (actor);
  }
  g_signal_emit (self, cdos_popup_menu_base_signals[DESTROY], 0);
}


void
cdos_popup_menu_base_remove_menu_item (CdosPopupMenuBase *self,CdosPopupBaseMenuItem *item){
  g_return_if_fail(CDOS_IS_POPUP_MENU_BASE(self));
  g_return_if_fail(CDOS_IS_POPUP_BASE_MENU_ITEM(item));

  CdosPopupMenuBasePrivate *priv = self->priv;

  g_object_set_data(G_OBJECT(item), "closing-id", NULL);
  g_object_unref (item);
  ClutterActor *actor = (ClutterActor *)cdos_popup_base_menu_item_get_actor(item);
  st_container_remove_actor((ClutterContainer *)priv->box, actor);

  self->length--;
}
