#include "cdos-popup-menu-item.h"
#include "st.h"

/**
 * SECTION: cdos-popup-menu-item.c
 * @Title: CdosPopupMenuItem
 * @short_description: CdosPopupMenuItem继承自CdosPopupBaseMenuItem, 此控件为我们默认往item
 *　 里面添加了一个text。
 *
 * CdosPopupMenuItem 为我们提供了一个默认添加label的item。
 */

#define CDOS_POPUP_MENU_ITEM_GET_PRIVATE(obj)    \
	(G_TYPE_INSTANCE_GET_PRIVATE((obj), CDOS_TYPE_POPUP_MENU_ITEM, CdosPopupMenuItemPrivate))

static void 
cdos_popup_menu_item_class_init(CdosPopupMenuItemClass *klass);

static void 
cdos_popup_menu_item_init(CdosPopupMenuItem *item);

static void 
cdos_popup_menu_item_finalize(GObject *gobject);

static void
cdos_popup_menu_item_dispose(GObject *gobject);


G_DEFINE_TYPE(CdosPopupMenuItem, cdos_popup_menu_item, CDOS_TYPE_POPUP_BASE_MENU_ITEM);

struct _CdosPopupMenuItemPrivate
{
	StAlign align;
	StLabel  *label;
};

static void 
cdos_popup_menu_item_class_init(CdosPopupMenuItemClass *klass)
{
	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

	g_type_class_add_private(klass, sizeof(CdosPopupMenuItemPrivate));

	gobject_class->finalize = cdos_popup_menu_item_finalize;
	gobject_class->dispose = cdos_popup_menu_item_dispose;
}

static void 
cdos_popup_menu_item_init(CdosPopupMenuItem *item)
{
	CdosPopupMenuItemPrivate *priv;
	priv = item->priv = CDOS_POPUP_MENU_ITEM_GET_PRIVATE(item);
	priv->label = ST_LABEL(st_label_new(""));
	ActorChild actor;
	//actor.align = ST_ALIGN_MIDDLE;
	//actor.expand = TRUE;
	actor.span = -1;
	cdos_popup_base_menu_item_get_actor_child(&actor);
	cdos_popup_base_menu_item_add_actor((CdosPopupBaseMenuItem *)item, (ClutterActor *)priv->label, &actor);
	//gfloat margin = cdos_popup_base_menu_item_get_margin_left_label((CdosPopupBaseMenuItem *)item);
}

/**
 * cdos_popup_menu_item_new:
 * 
 * 新建一个 #CdosPopupMenuItem 类的实例。
 *
 * Returns: 返回一个 #CdosPopupMenuItem 的实例。
 */
CdosPopupMenuItem *
cdos_popup_menu_item_new(void)
{
	return g_object_new(CDOS_TYPE_POPUP_MENU_ITEM, NULL);
}

/**
 * cdos_popup_menu_item_new_with_params:
 * @text: text 
 * @params: 初始化默认的值
 * 
 * 新建一个 #CdosPopupMenuItem 类的实例。
 *
 * Returns: 返回一个 #CdosPopupMenuItem 的实例。
 */
CdosPopupMenuItem *
cdos_popup_menu_item_new_with_params(const gchar *text, 
				     PopupBaseMenuItemInit *params)
{
	CdosPopupMenuItem *gobject;

	gobject = g_object_new(CDOS_TYPE_POPUP_MENU_ITEM, NULL);

	cdos_popup_base_menu_item_set_params((CdosPopupBaseMenuItem *)gobject, params);

	cdos_popup_menu_item_set_label(gobject, text);

	return gobject;
}
static void
cdos_popup_menu_item_dispose(GObject *gobject)
{
	CdosPopupMenuItemPrivate  *priv = CDOS_POPUP_MENU_ITEM(gobject)->priv;
	G_OBJECT_CLASS(cdos_popup_menu_item_parent_class)->dispose(gobject);
}
static void 
cdos_popup_menu_item_finalize(GObject *gobject)
{

	G_OBJECT_CLASS(cdos_popup_menu_item_parent_class)->finalize(gobject);
}

/**
 * cdos_popup_menu_item_set_label:
 * @item: A #CdosPopupMenuItem
 * @label: 需要显示的text 
 *
 * 设置item的label显示的文本
 *
 */
void 
cdos_popup_menu_item_set_label(CdosPopupMenuItem *item, const gchar *label)
{
	CdosPopupMenuItemPrivate *priv;
	g_return_if_fail(CDOS_IS_POPUP_MENU_ITEM(item));

	priv = item->priv;

	if (priv->label ){
		st_label_set_text(priv->label, label);
	}
}

/**
 * cdos_popup_menu_item_get_label:
 * @item: A #CdosPopupMenuItem
 *
 * 获取item的label
 *
 * Returns: A #StLabel
 */
StLabel *
cdos_popup_menu_item_get_label(CdosPopupMenuItem *item)
{
	CdosPopupMenuItemPrivate *priv;
	g_return_val_if_fail(CDOS_IS_POPUP_MENU_ITEM(item), NULL);

	priv = item->priv;
	return priv->label;
}

/**
 * cdos_popup_menu_item_set_label_align:
 * @item: A #Cdos{type}
 * @align: ＃ClutterActorAlign　 
 * @margin: margind的大小 
 *
 * 设置item的label中文本的对其方式
 *
 */
void 
cdos_popup_menu_item_set_label_align(CdosPopupMenuItem *item, StAlign align, guint margin)
{
	CdosPopupMenuItemPrivate *priv;
	g_return_if_fail(CDOS_IS_POPUP_MENU_ITEM(item));
	priv = item->priv;
	priv->align = align;
	CdosGenericContainer *actor = cdos_popup_base_menu_item_get_actor((CdosPopupBaseMenuItem *)item);
	if(align == ST_ALIGN_END){
		clutter_actor_set_margin_right(CLUTTER_ACTOR(priv->label), margin);
	}else if(align == ST_ALIGN_START){
		clutter_actor_set_margin_left(CLUTTER_ACTOR(priv->label), margin);
	}
	cdos_popup_base_menu_item_set_actor_align(CDOS_POPUP_BASE_MENU_ITEM(item),CLUTTER_ACTOR(priv->label),align);
}
