/*
 * cdos-popup-menu-manager.c
 *
 *  Created on: 2016年4月1日
 *      Author: wj
 */

#include "cdos-popup-menu-manager.h"
#include "cdos-debug.h"
#include "cdos-global.h"
#include "cdos-start.h"
#include "cdos_desktop_dbus.h"

static gboolean xunli_dbus_inited = FALSE;
static CdosPopupMenuManager *current_active_manager = NULL;

/**
 * SECTION: cdos-popup-menu-manager.c
 * @Title: CdosPopupMenuManager
 * @short_description: 为menu提供一个管理器
 *　 
 *
 * CdosPopupMenuManager 管理menu 
 */

G_DEFINE_TYPE( CdosPopupMenuManager, cdos_popup_menu_manager, G_TYPE_OBJECT);

#define POPUP_MENU_MANAGER_GET_PRIVATE( obj ) (G_TYPE_INSTANCE_GET_PRIVATE( (obj), CDOS_TYPE_POPUP_MENU_MANAGER, CdosPopupMenuManagerPrivate ))

struct _CdosPopupMenuManagerPrivate
{
  ClutterActor       *owner;
  gulong              event_capture_id;
  gulong              enter_event_id;
  gulong              leave_event_id;
  gulong              key_focus_notify_id;
  CdosPopupMenuBase  *active_menu;
  GList              *menus;
  GList              *menu_stack;
  //CdosStageInputMode  pre_grab_input_mode; 
  gboolean            grabbed_from_keynav;
  gboolean            did_pop;

  gboolean (*on_event_capture)(ClutterActor *actor, ClutterEvent *event, gpointer user_data);
};

typedef struct
{
  CdosPopupMenuBase *menu;
  gulong             open_state_change_id;
  gulong             child_menu_added_id;
  gulong             child_menu_removed_id;
  gulong             destroy_id;
  gulong             enter_id;
  gulong             focus_in_id;
}MenuData;

typedef struct
{
	CdosPopupMenuBase    *menu;
	CdosPopupMenuManager *self;
}UserData;

enum
{
  PROP_0,
  PROP_OWNER
};

static void
cdos_popup_menu_manager_finalize( GObject * gobject )
{
    CdosPopupMenuManager *self = CDOS_POPUP_MENU_MANAGER( gobject );
    CdosPopupMenuManagerPrivate * priv = self->priv;
    g_list_free(priv->menu_stack);
    g_list_free(priv->menus);

    G_OBJECT_CLASS(cdos_popup_menu_manager_parent_class)->finalize(gobject);
}

static void
cdos_popup_menu_manager_dispose( GObject * gobject )
{
    CdosPopupMenuManager *self = CDOS_POPUP_MENU_MANAGER( gobject );
    CdosPopupMenuManagerPrivate * priv = self->priv;
    CdosGlobal *global = cdos_global_get();
    ClutterStage *stage = cdos_global_get_stage (global);
    if(priv->enter_event_id > 0){
        g_signal_handler_disconnect(stage,priv->enter_event_id);
        priv->enter_event_id = 0;
    }
    if(priv->event_capture_id > 0){
        g_signal_handler_disconnect(stage,priv->event_capture_id);
        priv->event_capture_id = 0;
    }
    if(priv->leave_event_id > 0){
        g_signal_handler_disconnect(stage,priv->leave_event_id);
        priv->leave_event_id = 0;
    }
    if(priv->key_focus_notify_id > 0){
        g_signal_handler_disconnect(stage,priv->key_focus_notify_id);
        priv->key_focus_notify_id = 0;
    }

    GList *menus = priv->menus;
    if(menus){
        while (menus != NULL){
            MenuData *menudata = menus->data;
            if(menudata){
                cdos_popup_menu_manager_remove_menu(self,menudata->menu);
            }
            menus = menus->next;
        }
      }
    g_object_unref(priv->owner);
    G_OBJECT_CLASS(cdos_popup_menu_manager_parent_class)->dispose(gobject);
}

static void
cdos_popup_menu_manager_set_property (GObject      *gobject, 
                                      guint         prop_id,
                                      const GValue *value,
                                      GParamSpec   *pspec)
{
  CdosPopupMenuManager *obj = CDOS_POPUP_MENU_MANAGER (gobject);

  switch (prop_id)
  {
    case PROP_OWNER:
      cdos_popup_menu_manager_set_owner(obj, g_value_get_object(value));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID(gobject, prop_id, pspec);
      break;
  }
}

static void
cdos_popup_menu_manager_get_property (GObject    *gobject, 
                                      guint       prop_id,
				                              GValue     *value, 
                                      GParamSpec *pspec)
{
  CdosPopupMenuManagerPrivate * priv = CDOS_POPUP_MENU_MANAGER (gobject)->priv;
  switch (prop_id)
  {
    case PROP_OWNER:
      g_value_set_object(value, priv->owner);
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID(gobject, prop_id, pspec);
      break;
  }
}

static gboolean
cdos_popup_menu_manager_active_menu_contains(CdosPopupMenuManager *self,
					                                   ClutterActor         *actor)
{
  CdosPopupMenuManagerPrivate * priv = self->priv;
  CdosPopupMenuBaseClass *menu_class;
  gboolean ret = FALSE;

  if (priv->active_menu != NULL)
  {
      menu_class = CDOS_POPUP_MENU_BASE_GET_CLASS(priv->active_menu);
      ret = (clutter_actor_contains(menu_class->get_actor(priv->active_menu),actor));
      ClutterActor *source_actor;
      source_actor = (ClutterActor *)cdos_popup_menu_base_get_source_actor (priv->active_menu);
      ret |= clutter_actor_contains (source_actor, actor);
  }

  return ret;
}

static gboolean
cdos_popup_menu_manager_event_is_on_active_menu(CdosPopupMenuManager *self,
						                                    ClutterEvent         *event)
{
  //ClutterActor *source;
  return cdos_popup_menu_manager_active_menu_contains(self, clutter_event_get_source(event));
}

static gboolean
cdos_popup_menu_manager_should_block_event(CdosPopupMenuManager *self,
					                                 ClutterEvent         *event)
{
  CdosPopupMenuManagerPrivate * priv = self->priv;
  ClutterActor *src = clutter_event_get_source (event);
  CdosPopupMenuBaseClass *menu_class;
  GList * l;
  CdosPopupMenuBase *menu;
  ClutterActor *source_actor;

  if (priv->active_menu)
  {
      menu_class = CDOS_POPUP_MENU_BASE_GET_CLASS(priv->active_menu);
      if (clutter_actor_contains (menu_class->get_actor (priv->active_menu), src))
    	  return FALSE;
  };

  for (l = priv->menus; l->next != NULL; l = l->next)
    {
      menu = ((MenuData *) l->data)->menu;
      source_actor = CLUTTER_ACTOR(cdos_popup_menu_base_get_source_actor(menu));
      if (source_actor && !menu->block_source_events
	  && clutter_actor_contains (source_actor, src))
	    return FALSE;
    }

  return TRUE;
}

static void
cdos_popup_menu_manager_close_menu(CdosPopupMenuManager *self)
{
  CdosPopupMenuManagerPrivate * priv = self->priv;
  CdosPopupMenuBaseClass *menu_class;

  if(priv->active_menu)
    {
      menu_class = CDOS_POPUP_MENU_BASE_GET_CLASS(priv->active_menu);
      menu_class->close(priv->active_menu, TRUE);
    }
}

static gboolean
cdos_popup_menu_manager_on_event_capture(ClutterActor *actor,
					                               ClutterEvent *event,
					                               gpointer      user_data)
{
  CdosPopupMenuManager *self = CDOS_POPUP_MENU_MANAGER(user_data);
  CdosPopupMenuManagerPrivate * priv = self->priv;
  gboolean active_menu_contains;
  ClutterEventType event_type;

  if(!self->grabbed)
      return FALSE;

/*  if (this._owner.menuEventFilter &&
      this._owner.menuEventFilter(event))
      return true;
*/

    if(priv->active_menu != NULL && priv->active_menu->pass_events)
        return FALSE;

    if(priv->did_pop)
    {
        priv->did_pop = FALSE;
        return TRUE;
    }

    active_menu_contains = cdos_popup_menu_manager_event_is_on_active_menu(self, event);
    event_type = clutter_event_type(event);

    if (event_type == CLUTTER_BUTTON_RELEASE)
    {
        if (active_menu_contains)
        {
            return FALSE;
        }
        else
        {
            //cdos_popup_menu_manager_close_menu (self);
            return TRUE;
        }
    }
    else if (event_type == CLUTTER_BUTTON_PRESS && !active_menu_contains)
    {
        cdos_popup_menu_manager_close_menu (self);
        return TRUE;
    }
    else if (!cdos_popup_menu_manager_should_block_event (self, event))
    {
        return FALSE;
    }

    return FALSE;
}

static gint
cdos_popup_menu_manager_find_menu(CdosPopupMenuManager *self, 
                                  CdosPopupMenuBase    *item)
{
    CdosPopupMenuManagerPrivate * priv = self->priv;
    GList * l;
    MenuData *menudata;
    gint index = -1;
    for (l = priv->menus; l != NULL; l = l->next) {
        index++ ;
        menudata = l->data;
        if (menudata !=NULL && item == (CdosPopupMenuBase *)menudata->menu)
            return index;
    }
    return -1;
}

static void
cdos_popup_menu_manager_on_key_focus_changed(ClutterStage *stage,
                                             GParamSpec *pspec,
                                             CdosPopupMenuManager *self)
{
  CdosPopupMenuManagerPrivate * priv = self->priv;
  ClutterActor *focus;
  ClutterActor *delegate;
  CdosPopupMenuBase *delegate_menu;
  //最后一个条件是根据DND.isDragging进行判断。
  if (!self->grabbed || priv->active_menu == NULL)
    return;

  focus = clutter_stage_get_key_focus (stage);
  if (focus)
    {
      if (cdos_popup_menu_manager_active_menu_contains (self, focus))
	      return;
      if (g_list_length (priv->menu_stack) > 0)
	      return;
      delegate = obj_get_delegate (focus);
      if (delegate)
	      g_object_get (G_OBJECT(delegate), "menu", &delegate_menu, NULL);
      if (delegate && delegate_menu
	      && cdos_popup_menu_manager_find_menu (self, delegate_menu) != -1)
	      return;
    }
  cdos_popup_menu_manager_close_menu (self);
}

static void
cdos_popup_menu_manager_grab(CdosPopupMenuManager *self)
{
  CdosPopupMenuManagerPrivate * priv = self->priv;
  CdosGlobal *global = cdos_global_get();
  ClutterStage *stage = cdos_global_get_stage (global);
  //CdosPopupMenuManagerClass *klass = CDOS_POPUP_MENU_MANAGER_GET_CLASS(self);
// Main 部分功能。
//  if (!Main.pushModal (this._owner.actor))
//    {
//      return;
//    }
  if (!cdos_push_modal (self->priv->owner, 0, 0))
      return;

  priv->event_capture_id = g_signal_connect(stage,
					    "captured-event",
					    G_CALLBACK(priv->on_event_capture),
					    self);

  priv->enter_event_id = g_signal_connect(stage,
					    "enter-event",
					    G_CALLBACK(priv->on_event_capture),
					    self);

  priv->leave_event_id = g_signal_connect(stage,
					    "leave-event",
					    G_CALLBACK(priv->on_event_capture),
					    self);
  /*
  priv->key_focus_notify_id = g_signal_connect(stage,
					    "notify::key-focus",
					    G_CALLBACK(cdos_popup_menu_manager_on_key_focus_changed),
					    self);
  */

  self->grabbed = TRUE;
}

static void
cdos_popup_menu_manager_ungrab(CdosPopupMenuManager *self)
{
  CdosPopupMenuManagerPrivate * priv = self->priv;
  CdosGlobal *global = cdos_global_get();
  ClutterStage *stage = cdos_global_get_stage (global);

  if(!self->grabbed)
    return;

  g_signal_handler_disconnect(stage, priv->event_capture_id);
  priv->event_capture_id = 0;

  g_signal_handler_disconnect(stage, priv->enter_event_id);
  priv->enter_event_id = 0;

  g_signal_handler_disconnect(stage, priv->leave_event_id);
  priv->leave_event_id = 0;

  if (priv->key_focus_notify_id)
  {
    g_signal_handler_disconnect(stage, priv->key_focus_notify_id);
    priv->key_focus_notify_id = 0;
  }

  self->grabbed = FALSE;

  cdos_pop_modal (self->priv->owner, 0);
}

static void
cdos_popup_menu_manager_on_menu_open_state(CdosPopupMenuBase *menu,
					                       gboolean           open,
                                           CdosPopupMenuManager *self)
{
    CdosPopupMenuManagerPrivate * priv = self->priv;
    CdosPopupMenuBaseClass *menu_class = CDOS_POPUP_MENU_BASE_GET_CLASS(menu);
    CdosGlobal *global = cdos_global_get();
    ClutterStage *stage = cdos_global_get_stage (global);
    ClutterActor *focus = NULL;
    gboolean had_focus;
    //CdosStageInputMode *stage_input_mode;
    ClutterActor *source_actor;

    source_actor = CLUTTER_ACTOR(cdos_popup_menu_base_get_source_actor(menu));
    if (open)
    {
        if (priv->active_menu
                && cdos_popup_menu_base_is_child_menu(priv->active_menu, menu))
		{
			priv->menu_stack = g_list_append(priv->menu_stack, priv->active_menu);
			clutter_actor_grab_key_focus(menu_class->get_actor(menu));
		}
		priv->active_menu = menu;
        current_active_manager = self;
	} else {
		if (g_list_length(priv->menu_stack))
		{
			priv->active_menu = (CdosPopupMenuBase *)g_list_last(priv->menu_stack)->data;
			priv->menu_stack = g_list_remove(priv->menu_stack, g_list_last(priv->menu_stack)->data);
			if (source_actor)
			{
				clutter_actor_grab_key_focus(source_actor);
				priv->did_pop = TRUE;
			}
		}
	}

    // Check what the focus was before calling pushModal/popModal
    focus = clutter_stage_get_key_focus(stage);
    had_focus = focus && cdos_popup_menu_manager_active_menu_contains(self, focus);
    if (open)
    {
        if (!self->grabbed)
		{
			//priv->pre_grab_input_mode = cdos_global_get_stage_input_mode(global);
			priv->grabbed_from_keynav = had_focus;
			cdos_popup_menu_manager_grab(self);

			if (had_focus)
				clutter_actor_grab_key_focus(focus);
			else{
                ClutterActor *actor_grab = menu_class->get_actor(menu);
				clutter_actor_grab_key_focus(actor_grab);
            }
		}
    }
  else if (priv->active_menu == menu)
    {
	  //g_print("file: %s, line :%d,  open %d , self->grabbed %d\n", __FILE__, __LINE__, open, self->grabbed);
		if (self->grabbed)
			cdos_popup_menu_manager_ungrab(self);

		priv->active_menu = NULL;
		if (priv->grabbed_from_keynav)
		{
		/*	if (priv->pre_grab_input_mode
					== CDOS_STAGE_INPUT_MODE_FOCUSED)
			{
				cdos_global_set_stage_input_mode(global,
						CDOS_STAGE_INPUT_MODE_FOCUSED);
			}*/
			if (had_focus && source_actor)
			{
				clutter_actor_grab_key_focus(source_actor);
			} else if (focus)
			{
				clutter_actor_grab_key_focus(focus);
			}
		}
    }
}

static void
cdos_popup_menu_manager_on_child_menu_added(CdosPopupMenuBase *menu,
					                                  CdosPopupMenuBase *child_menu,
					                                  gpointer           data)
{
  CdosPopupMenuManager *self = data;
  cdos_popup_menu_manager_add_menu(self, child_menu, -1);
}

static void
cdos_popup_menu_manager_on_child_menu_removed(CdosPopupMenuBase *menu,
					                                    CdosPopupMenuBase *child_menu,
					                                    gpointer           data)
{
  CdosPopupMenuManager *self = data;
  cdos_popup_menu_manager_remove_menu(self, child_menu);
}

static void
cdos_popup_menu_manager_change_menu(CdosPopupMenuManager *self,
				                            CdosPopupMenuBase    *new_menu)
{
  CdosPopupMenuManagerPrivate * priv = self->priv;
  CdosPopupMenuBase *old_menu;
  GList *l;
  CdosPopupMenuBaseClass *menu_class;
  menu_class = CDOS_POPUP_MENU_BASE_GET_CLASS(new_menu);
  if (priv->active_menu)
  {
    old_menu = priv->active_menu;
    priv->active_menu = NULL;
    for (l = priv->menu_stack; l->next != NULL; l = l->next)
    {
      menu_class->close ((CdosPopupMenuBase *) l->data, FALSE);
    }
    menu_class->close (old_menu, FALSE);
    menu_class->open (new_menu, FALSE);
  }
  else
  {
    menu_class->open (new_menu, FALSE);
  }
}

static gboolean
cdos_popup_menu_manager_on_menu_source_enter(CdosPopupMenuManager *self,
		                                         CdosPopupMenuBase    *menu)
{
  CdosPopupMenuManagerPrivate * priv = self->priv;

  if(!self->grabbed || menu == priv->active_menu)
    return FALSE;

  if(priv->active_menu && cdos_popup_menu_base_is_child_menu(priv->active_menu, menu))
    return FALSE;

  if(g_list_find(priv->menu_stack, menu)!= NULL)
    return FALSE;

  if(g_list_length(priv->menu_stack) > 0 && cdos_popup_menu_base_is_child_menu(g_list_nth_data(priv->menu_stack, 0), menu))
    return FALSE;

	cdos_popup_menu_manager_change_menu(self, menu);

  return FALSE;
}

static gboolean
source_actor_on_enter_event_callback(ClutterActor *actor,
                                     ClutterEvent *event,
                                     gpointer      user_data)
{
	UserData *data = user_data;
	return cdos_popup_menu_manager_on_menu_source_enter(data->self, data->menu);
}

static void
source_actor_on_key_focus_in_callback(ClutterActor *actor,
                                      gpointer      user_data)
{
	UserData *data = user_data;
	cdos_popup_menu_manager_on_menu_source_enter(data->self, data->menu);
}

static void
cdos_popup_menu_manager_on_menu_destroy(CdosPopupMenuBase *menu,
					                              gpointer           data)
{
  CdosPopupMenuManager *self = data;
  cdos_popup_menu_manager_remove_menu(self, menu);
}

static void
cdos_popup_menu_manager_class_init( CdosPopupMenuManagerClass * klass )
{
    GObjectClass * gobject_class = G_OBJECT_CLASS( klass );
    GParamSpec * pspec;

    gobject_class->finalize = cdos_popup_menu_manager_finalize;
    gobject_class->dispose = cdos_popup_menu_manager_dispose;
    gobject_class->set_property = cdos_popup_menu_manager_set_property;
    gobject_class->get_property = cdos_popup_menu_manager_get_property;

    g_type_class_add_private( klass, sizeof(CdosPopupMenuManagerPrivate) );

    pspec = g_param_spec_object ("owner",
  			                         "Owner",
  			                         "Owner of the object",
  				                       CLUTTER_TYPE_ACTOR,
  			                         G_PARAM_READWRITE);

    g_object_class_install_property (gobject_class, PROP_OWNER, pspec);
}

static void
on_handle_exit_none (Xunli *skeleton,
			GDBusMethodInvocation *invocation,
			guint test,
			gpointer data)
{
	GVariant *value;
	CdosPopupMenuManager *self = current_active_manager;
    if (!self)
        goto finish;

    if (self->grabbed)
    {
        ClutterActor *owner = self->priv->owner;
        if (owner != NULL) {

            g_debug ("test screensaver to desktop, owner! %d", __LINE__);

            cdos_pop_modal (self->priv->owner, 0);
        }
        cdos_popup_menu_manager_close_menu (self);
    }
	g_debug ("test screensaver after close menu, grab? %d", self->grabbed);
finish:
	xunli_complete_exit_none (skeleton, invocation);
}


static void
on_name_acquired (GDBusConnection *connection,
			const gchar *name,
			gpointer data)
{
	CdosPopupMenuManager	*self = (CdosPopupMenuManager *)data;

	Xunli *skeleton = NULL;
    skeleton = xunli_skeleton_new ();

	g_signal_connect (skeleton,
			   "handle-exit-none",
			   G_CALLBACK (on_handle_exit_none),
			   self);

	g_dbus_interface_skeleton_export (G_DBUS_INTERFACE_SKELETON (skeleton),
					   connection,
					   "/org/Cdos/Xunli",
					   NULL);

}

static void 
cdos_dbus_init (CdosPopupMenuManager *self){
  // 这个 dbus 提供给 cdos-screensaver 使用，在某些情况下，锁屏无法 grab keyboard 和 mouse
  // 某些情况如下：
  // Alt-F2， popupmenu 等
  // 也就是说，锁屏的时候，需要主动发消息给 cdos desktop，让它退出这几种模式

    g_bus_own_name (G_BUS_TYPE_SESSION,
		    "org.Cdos.Xunli",
		    G_BUS_NAME_OWNER_FLAGS_NONE,
		    NULL,
		    on_name_acquired,
		    NULL,
		    self,
		    NULL);

}

static void
cdos_popup_menu_manager_init( CdosPopupMenuManager * self )
{
    CdosPopupMenuManagerPrivate * priv;
    priv = self->priv = POPUP_MENU_MANAGER_GET_PRIVATE(self);

    self->grabbed = FALSE;
    priv->event_capture_id = 0;
    priv->enter_event_id = 0;
    priv->leave_event_id = 0;
    priv->key_focus_notify_id = 0;
    priv->active_menu = NULL;
    priv->menus = NULL;
    priv->menu_stack = NULL;
    //priv->pre_grab_input_mode = CDOS_STAGE_INPUT_MODE_NORMAL;
    priv->grabbed_from_keynav = FALSE;

    priv->on_event_capture = cdos_popup_menu_manager_on_event_capture;

}

/**
 * cdos_popup_menu_manager_new:
 * @owner: menuManager的拥有者
 * 
 * 新建一个 #CdosPopupMenuManager类的实例。
 *
 * Returns: 返回一个 #CdosPopupMenuManager的实例。
 */

CdosPopupMenuManager *
cdos_popup_menu_manager_new(ClutterActor *owner)
{
    CdosPopupMenuManager *object;
    object = g_object_new(CDOS_TYPE_POPUP_MENU_MANAGER, "owner", owner, NULL);
    if (!xunli_dbus_inited)
    {
        cdos_dbus_init (object);
        xunli_dbus_inited = TRUE;
    }
    return object;
}

/**
 * cdos_popup_menu_manager_get_owner:
 * @self: A #CdosPopupMenuManager
 *
 * 获取self的owner成员
 *
 * Returns: ＃ClutterActor　
 */
ClutterActor *
cdos_popup_menu_manager_get_owner(CdosPopupMenuManager *self)
{
    g_return_val_if_fail(CDOS_IS_POPUP_MENU_MANAGER(self), NULL);
    return self->priv->owner;
}

/**
 * cdos_popup_menu_manager_set_owner:
 * @self: A #CdosPopupMenuManager
 * @owner:　CdosPopupMenuManager的拥有对象  
 *
 * 设置self的owner属性
 *
 */
void
cdos_popup_menu_manager_set_owner(CdosPopupMenuManager *self, ClutterActor *owner)
{
  g_return_if_fail(CDOS_IS_POPUP_MENU_MANAGER(self));

  CdosPopupMenuManagerPrivate *priv;
  priv = self->priv;
  if (priv->owner)
  {
    g_object_unref(priv->owner);
  }

  if(owner)
  {
    g_object_ref (owner);
    priv->owner = owner;
  } 
  else
  {
    priv->owner = NULL;
  }
}

/**
 * cdos_popup_menu_manager_add_menu:
 * @self: A #CdosPopupMenuManager
 * @menu: 要添加的menu
 * @position: 添加menu的位置，-1为缺省设置 
 *
 * 添加menu到CdosPopupMenuManager中
 *
 */
void
cdos_popup_menu_manager_add_menu(CdosPopupMenuManager *self,
				                         CdosPopupMenuBase    *menu,
				                         gint                  position)
{
    g_return_if_fail(CDOS_IS_POPUP_MENU_MANAGER(self));
    g_return_if_fail(CDOS_IS_POPUP_MENU_BASE(menu));
  CdosPopupMenuManagerPrivate * priv = self->priv;
  ClutterActor *source;
  MenuData *menudata = g_malloc(sizeof(MenuData));
  UserData *user_data = g_malloc(sizeof(UserData));

  user_data->menu = menu;
  user_data->self = self;

  menudata->menu = menu;//g_object_ref(menu);
  menudata->open_state_change_id = g_signal_connect(menu,
						    "open-state-changed",
						    G_CALLBACK(cdos_popup_menu_manager_on_menu_open_state),
						    self);

  menudata->child_menu_added_id = g_signal_connect(menu,
  						  "child-menu-added",
  						  G_CALLBACK(cdos_popup_menu_manager_on_child_menu_added),
  						  self);

  menudata->child_menu_removed_id = g_signal_connect(menu,
    						"child-menu-removed",
    						 G_CALLBACK(cdos_popup_menu_manager_on_child_menu_removed),
    						 self);

  menudata->destroy_id = g_signal_connect(menu,
      					"destroy",
      					G_CALLBACK(cdos_popup_menu_manager_on_menu_destroy),
      					self);

  menudata->enter_id = 0;
  menudata->focus_in_id = 0;

  g_object_get(G_OBJECT(menu),"source_actor", &source, NULL);
  g_object_set_data(G_OBJECT(menu),"source_user_data", user_data);

  if (NULL != source)
    {
      menudata->enter_id = g_signal_connect(source,
					            "enter-event",
					             G_CALLBACK(source_actor_on_enter_event_callback),
						           user_data);

      menudata->focus_in_id = g_signal_connect(source,
					             "key-focus-in",
					             G_CALLBACK(source_actor_on_key_focus_in_callback),
						           user_data);
    }

  if (position < 0)
    {
      priv->menus = g_list_append (priv->menus, menudata);
    }
  else
    {
        MenuData *menudata = g_list_nth_data(priv->menus, position);
        if(menudata){
            cdos_popup_menu_manager_remove_menu(self,menudata->menu);
        }
      //priv->menus = g_list_remove(priv->menus, menudata);
      priv->menus = g_list_insert(priv->menus, menudata, position);
    }
}

/**
 * cdos_popup_menu_manager_remove_menu:
 * @self: A #CdosPopupMenuManager
 * @menu: 要移除的menu 
 *
 * 移除CdosPopupMenuManager中的某个menu
 *
 */
void
cdos_popup_menu_manager_remove_menu (CdosPopupMenuManager *self,
				                             CdosPopupMenuBase    *menu)
{
    g_return_if_fail(CDOS_IS_POPUP_MENU_MANAGER(self));
    g_return_if_fail(CDOS_IS_POPUP_MENU_BASE(menu));

  CdosPopupMenuManagerPrivate * priv = self->priv;
  MenuData *menudata;
  UserData *user_data = NULL;
  ClutterActor *source;

  if (menu == priv->active_menu)
      cdos_popup_menu_manager_close_menu(self);

  gint position = cdos_popup_menu_manager_find_menu (self, menu);

  if (position == -1)
    return;

  menudata = g_list_nth_data(priv->menus, (guint)position);
  g_signal_handler_disconnect(menu, menudata->open_state_change_id);
  g_signal_handler_disconnect(menu, menudata->child_menu_added_id);
  g_signal_handler_disconnect(menu, menudata->child_menu_removed_id);
  g_signal_handler_disconnect(menu, menudata->destroy_id);

  if (menudata->enter_id > 0)
    {
      g_object_get (G_OBJECT(menu), "source_actor", &source, NULL);
      g_signal_handler_disconnect (source, menudata->enter_id);
        menudata->enter_id = 0L;
    }
  if (menudata->enter_id > 0)
    {
      g_object_get (G_OBJECT(menu), "source_actor", &source, NULL);
      g_signal_handler_disconnect (source, menudata->focus_in_id);
        menudata->focus_in_id = 0L;
    }

  user_data = g_object_get_data (G_OBJECT(menu), "source_user_data");

  if (NULL != user_data) {
    g_free(user_data);
    g_object_set_data (G_OBJECT(menu), "source_user_data", NULL);
  }
  g_object_unref(menudata->menu);
  priv->menus = g_list_remove(priv->menus, menudata);
  g_free(menudata);
}

/**
 * cdos_popup_menu_manager_set_event_capture:
 * @manager: A #CdosPopupMenuManager
 * @fun: 回调函数
 *
 * 设置self的on_event_capture。捕捉到事件的回调函数
 *
 */
void
cdos_popup_menu_manager_set_event_capture(CdosPopupMenuManager *manager, 
      gboolean (*fun )(ClutterActor *actor, ClutterEvent *event, gpointer user_data))
{
    g_return_if_fail(CDOS_IS_POPUP_MENU_MANAGER(manager));
	  CdosPopupMenuManagerPrivate * priv = manager->priv;

	  if(fun)
		  priv->on_event_capture = fun;
	  else
		  priv->on_event_capture = cdos_popup_menu_manager_on_event_capture;
}

