#include "cdos-popup-menu.h"
#include "cdos-popup-base-menu-item.h"
#include "cdos-start.h"
#include <clutter/clutter.h>

G_DEFINE_TYPE( CdosPopupMenu, cdos_popup_menu, CDOS_TYPE_POPUP_MENU_BASE);


#define POPUP_MENU_GET_PRIVATE( obj )(G_TYPE_INSTANCE_GET_PRIVATE( (obj), CDOS_TYPE_POPUP_MENU, CdosPopupMenuPrivate ) )

/**
 * SECTION: cdos-popup-menu.c
 * @Title: CdosPopupMenu
 * @short_description: 继承自CdosPopupMenuBase, 可以创建一个menu实例。
 *　 你可以往menu里添加menuitem.
 *
 * CdosPopupMenu 实现一个菜单功能
 */
struct _CdosPopupMenuPrivate
{
  gfloat               arrow_alignment;
  gint                 arrow_side;
  CdosBoxPointer       *box_pointer;
  CdosGenericContainer *box_wrapper;
};

enum
{
  PROP_0,

  PROP_ARROW_ALIGNMENT,
  PROP_ARROW_SIDE
};

static void
cdos_popup_menu_dispose( GObject * gobject )
{
    CdosPopupMenu *self = CDOS_POPUP_MENU(gobject);
    CdosPopupMenuPrivate * priv = self->priv;

    //g_clear_object(&priv->box_wrapper);
    g_clear_object(&priv->box_pointer);

    G_OBJECT_CLASS(cdos_popup_menu_parent_class )->dispose( gobject );
}


static void
cdos_popup_menu_finalize( GObject * gobject )
{
    //CdosPopupMenuPrivate * priv = CDOS_POPUP_MENU( gobject )->priv;

    G_OBJECT_CLASS(cdos_popup_menu_parent_class )->finalize( gobject );
}

static void
cdos_popup_menu_set_property(GObject      *gobject,
                             guint         prop_id,
                             const GValue *value,
                             GParamSpec   *pspec)
{
    CdosPopupMenu * obj = CDOS_POPUP_MENU( gobject );
    switch ( prop_id )
    {
    case PROP_ARROW_ALIGNMENT:
          cdos_popup_menu_set_arrow_alignment(obj, g_value_get_float(value));
          break;
    case PROP_ARROW_SIDE:
          cdos_popup_menu_set_arrow_side(obj, g_value_get_int(value));
          break;
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID( gobject, prop_id, pspec );
        break;
    }
}

static void
cdos_popup_menu_get_property(GObject    *gobject,
                             guint       prop_id,
                             GValue     *value,
                             GParamSpec *pspec )
{
    CdosPopupMenuPrivate * priv = CDOS_POPUP_MENU( gobject )->priv;
    switch ( prop_id )
    {
    case PROP_ARROW_ALIGNMENT:
          g_value_set_float(value, priv->arrow_alignment);
         break;
    case PROP_ARROW_SIDE:
          g_value_set_int(value, priv->arrow_side);
         break;
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID( gobject, prop_id, pspec );
        break;
    }
}

static ClutterActor*
cdos_popup_menu_get_actor(CdosPopupMenuBase *obj)
{
    CdosPopupMenu *self = CDOS_POPUP_MENU(obj);
  return CLUTTER_ACTOR(self->actor);
}

static void
cdos_popup_menu_class_init( CdosPopupMenuClass * klass )
{
    CdosPopupMenuBaseClass *parent_class = CDOS_POPUP_MENU_BASE_CLASS(klass);
    GObjectClass * gobject_class = G_OBJECT_CLASS( klass );
    GParamSpec * pspec;
    
    parent_class->open = cdos_popup_menu_open;
    parent_class->close = cdos_popup_menu_close;
    parent_class->get_actor = cdos_popup_menu_get_actor;
    gobject_class->finalize = cdos_popup_menu_finalize;
    gobject_class->dispose = cdos_popup_menu_dispose;
    gobject_class->set_property = cdos_popup_menu_set_property;
    gobject_class->get_property = cdos_popup_menu_get_property;

    g_type_class_add_private( klass, sizeof(CdosPopupMenuPrivate) );

    pspec = g_param_spec_float("arrow_alignment",
            "ArrowAlignemnt",
            "ArrowAlignemnt of the object",
            0, 100, 0, G_PARAM_READWRITE);
    g_object_class_install_property(gobject_class, PROP_ARROW_ALIGNMENT, pspec);

    pspec = g_param_spec_int("arrow_side",
            "ArrowSide",
            "ArrowSide of the object",
            0, 100, 2, G_PARAM_READWRITE);
    g_object_class_install_property(gobject_class, PROP_ARROW_SIDE, pspec);
}

static gboolean
cdos_popup_menu_on_key_press_event(ClutterActor    *actor,
                                   ClutterEvent *event,
                                   CdosPopupMenu   *self)
{
      if(clutter_event_get_key_symbol(event) == CLUTTER_KEY_Escape){
          cdos_popup_menu_close((CdosPopupMenuBase *)self, TRUE);
          return TRUE;
      }
      return FALSE;
}

static void
cdos_popup_menu_box_get_preferred_width(ClutterActor * actor,
                                        gfloat for_height,
                                        CdosGenericContainerAllocation *alloc,
                                        gpointer data)
{
  CdosPopupMenu *self = (CdosPopupMenu *)data;
  StBoxLayout * _box = cdos_popup_menu_base_get_box(CDOS_POPUP_MENU_BASE (self));
  gfloat min_size, natural_size;

  clutter_actor_get_preferred_width(CLUTTER_ACTOR(_box), for_height, &min_size, &natural_size);
  alloc->min_size = min_size;
  alloc->natural_size = natural_size;
}

static void
cdos_popup_menu_box_get_preferred_height(ClutterActor *actor,
                                         gfloat        for_width,
                                         CdosGenericContainerAllocation *alloc,
			  	  	                           gpointer      data) 
{
		 CdosPopupMenu *self = (CdosPopupMenu *)data;
		 StBoxLayout * _box = cdos_popup_menu_base_get_box(CDOS_POPUP_MENU_BASE (self));
    gfloat min_size, natural_size;

    clutter_actor_get_preferred_height(CLUTTER_ACTOR(_box), for_width, &min_size, &natural_size);
    alloc->min_size = min_size;
    alloc->natural_size = natural_size;
}

static void
cdos_popup_menu_box_allocate(ClutterActor  *actor,
                             ClutterActorBox  *box,
                             ClutterAllocationFlags flags,
                             gpointer data)
{
			CdosPopupMenu *self = (CdosPopupMenu *)data;
			//CdosPopupMenuPrivate * priv = self->priv;
			StBoxLayout * _box = cdos_popup_menu_base_get_box(CDOS_POPUP_MENU_BASE (self));
			clutter_actor_allocate(CLUTTER_ACTOR(_box), box, flags);
}

static void
cdos_popup_menu_init( CdosPopupMenu * self )
{

    self->priv = POPUP_MENU_GET_PRIVATE(self);
}

/**
 * cdos_popup_menu_get_source_actor:
 * @self: A #CdosPopupMenu
 *
 * 获取self的source_actor属性
 *
 * Returns: source_actor #ClutterActor
 */
ClutterActor *
cdos_popup_menu_get_source_actor(CdosPopupMenu *self)
{
    g_return_val_if_fail(CDOS_IS_POPUP_MENU(self), NULL);
    StWidget *source_actor = cdos_popup_menu_base_get_source_actor(CDOS_POPUP_MENU_BASE(self));
  return (ClutterActor *)source_actor;
}

/**
 * cdos_popup_menu_set_source_actor:
 * @self: A #CdosPopupMenu
 * @actor:  #ClutterActor
 *
 * 设置self的source_actor
 *
 */
void
cdos_popup_menu_set_source_actor(CdosPopupMenu *self, ClutterActor *actor)
{
    g_return_if_fail(CDOS_IS_POPUP_MENU(self));
  cdos_popup_menu_base_set_source_actor(CDOS_POPUP_MENU_BASE(self), (StWidget *)actor);
}

/**
 * cdos_popup_menu_get_arrow_alignment:
 * @self: A #CdosPopupMenu
 *
 * 获取self的arrow_alignment属性
 *
 * Returns: ＃gint
 */
gint
cdos_popup_menu_get_arrow_alignment(CdosPopupMenu *self)
{
    g_return_val_if_fail(CDOS_IS_POPUP_MENU(self), 0);
    return self->priv->arrow_alignment; 
}

/**
 * cdos_popup_menu_set_arrow_alignment:
 * @self: A #CdosPopupMenu
 * @alignment: alignment 
 *
 * 设置self的arrow_alignment
 *
 */
void 
cdos_popup_menu_set_arrow_alignment(CdosPopupMenu *self, gint alignment)
{
  CdosPopupMenuPrivate *priv;
  g_return_if_fail(CDOS_IS_POPUP_MENU(self));
  priv = self->priv;
  if(alignment)
    priv->arrow_alignment = alignment;
  else
    priv->arrow_alignment = 0;
}

/**
 * cdos_popup_menu_get_arrow_side:
 * @self: A #CdosPopupMenu
 *
 * 获取self的arrow_side成员
 *
 * Returns: ＃gint
 */
gint
cdos_popup_menu_get_arrow_side(CdosPopupMenu *self)
{
    g_return_val_if_fail(CDOS_IS_POPUP_MENU(self), 0);
    return self->priv->arrow_side;
}

/**
 * cdos_popup_menu_set_arrow_side:
 * @self: A #CdosPopupMenu
 * @side: 方向 
 *
 * 设置self的arrow_side的值
 *
 */
void 
cdos_popup_menu_set_arrow_side(CdosPopupMenu *self, gint side)
{
  CdosPopupMenuPrivate *priv;
  g_return_if_fail(CDOS_IS_POPUP_MENU(self));
  priv = self->priv;
  if(side)
    priv->arrow_side = side;
  else
    priv->arrow_side = 0;

    cdos_box_pointer_set_arrow_side(priv->box_pointer, side);
}

/**
 * cdos_popup_menu_new:
 * 
 * 新建一个 #CdosPopupMenu 类的实例。
 *
 * Returns: 返回一个 #CdosPopupMenu 的实例。
 */
CdosPopupMenu *
cdos_popup_menu_new( void )
{
    CdosPopupMenu *obj = NULL;
    obj = g_object_new( CDOS_TYPE_POPUP_MENU, NULL );

    return obj;
}

/**
 * cdos_popup_menu_new:
 * @source_actor:  #ClutterActor
 * @arrow_alignment: #gfloat
 * @arrow_side:      #gint 
 * 
 * 新建一个 #CdosPopupMenu 类的实例。
 *
 * Returns: 返回一个 #CdosPopupMenu 的实例。
 */
CdosPopupMenu *
cdos_popup_menu_new_with_params(ClutterActor *source_actor, 
                                gfloat        arrow_alignment,
                                gint          arrow_side)
{
	CdosPopupMenu *menu = cdos_popup_menu_new();
	cdos_popup_menu_constructor(menu, source_actor, arrow_alignment, arrow_side);
	return menu;
}

/**
 * cdos_popup_menu_constructor:
 * @menu A #CdosPopupMenu
 * @source_actor:  #ClutterActor
 * @arrow_alignment: #gfloat
 * @arrow_side:      #gint 
 * 
 * 对#CdosPopupMenu 实例进行属性设置。
 *
 */
void
cdos_popup_menu_constructor(CdosPopupMenu *menu,
						                ClutterActor  *source_actor,
						                gfloat         arrow_alignment,
						                gint           arrow_side)
{
    g_return_if_fail(CDOS_IS_POPUP_MENU(menu));
	CdosPopupMenuPrivate *priv;
	CdosBoxPointer *box_pointer;
	StBoxLayout * box;
	CdosGlobal *global = cdos_global_get();
	StFocusManager *focus_manager = cdos_global_get_focus_manager(global);

	priv = menu->priv;
	priv->arrow_side = arrow_side;
    priv->box_pointer = box_pointer = cdos_box_pointer_new(priv->arrow_side);

	g_object_set(menu,
              "source_actor", source_actor,
              "arrow_alignment", arrow_alignment,
              "arrow_side", arrow_side,
			  "style-class", "popup-menu-content",
			  NULL );

	//cdos_box_pointer_set_arrow_side(box_pointer, priv->arrow_side);
	//g_object_set(box_pointer->bin, "x-fill", TRUE, "y-fill", TRUE, "x-align", ST_ALIGN_START, NULL);

	menu->actor = box_pointer->actor;
	//this.actor._delegate = this;
	g_object_set_data(G_OBJECT(menu->actor), "_delegate", menu);

	st_widget_set_style_class_name(ST_WIDGET(menu->actor), "popup-menu-boxpointer");
	g_signal_connect(menu->actor, "key-press-event", G_CALLBACK(cdos_popup_menu_on_key_press_event), menu);

	priv->box_wrapper = g_object_new(CDOS_TYPE_GENERIC_CONTAINER, NULL);

	g_signal_connect(priv->box_wrapper, "get-preferred-width", G_CALLBACK(cdos_popup_menu_box_get_preferred_width), menu);
	g_signal_connect(priv->box_wrapper, "get-preferred-height", G_CALLBACK(cdos_popup_menu_box_get_preferred_height), menu);
	g_signal_connect(priv->box_wrapper, "allocate", G_CALLBACK(cdos_popup_menu_box_allocate), menu);

	st_bin_set_child(priv->box_pointer->bin, (ClutterActor *)priv->box_wrapper);

	clutter_actor_hide(CLUTTER_ACTOR(priv->box_pointer->border));
	clutter_actor_hide(CLUTTER_ACTOR(menu->actor));

	box = cdos_popup_menu_base_get_box(CDOS_POPUP_MENU_BASE(menu));
	clutter_container_add_actor((ClutterContainer *)priv->box_wrapper, (ClutterActor *)box);

	st_widget_add_style_class_name(ST_WIDGET(menu->actor), "popup-menu");
	st_focus_manager_add_group(focus_manager, ST_WIDGET(menu->actor));
	clutter_actor_set_reactive(CLUTTER_ACTOR(menu->actor), TRUE);
}

/**
 * cdos_popup_menu_set_arrow_origin:
 * @self: A #CdosPopupMenu
 * @origin:　？？
 *
 * ？？
 *
 */
void 
cdos_popup_menu_set_arrow_origin(CdosPopupMenu *self, guint origin)
{
    g_return_if_fail(CDOS_IS_POPUP_MENU(self));
  CdosPopupMenuPrivate *priv = self->priv;
  cdos_box_pointer_set_arrow_origin(priv->box_pointer, origin);
}

/**
 * cdos_popup_menu_set_source_alignment:
 * @self: A #CdosPopupMenu
 * @alignment:　对其方式
 *
 * 设置menu的对其方式
 *
 */
void 
cdos_popup_menu_set_source_alignment(CdosPopupMenu *self, guint alignment)
{
    g_return_if_fail(CDOS_IS_POPUP_MENU(self));
    CdosPopupMenuPrivate *priv = self->priv;
    cdos_box_pointer_set_source_alignment(priv->box_pointer, alignment);
}


/**
 * cdos_popup_menu_open:
 * @obj: A #CdosPopupMenuBase
 * @animate: 是否显示boxpointer中的actor
 *
 * 将menu显示出来
 *
 */
void 
cdos_popup_menu_open(CdosPopupMenuBase *obj, gboolean animate)
{
    g_return_if_fail(CDOS_IS_POPUP_MENU_BASE(obj));
  CdosPopupMenuPrivate *priv = CDOS_POPUP_MENU(obj)->priv;
  if(!obj->is_open){
      CdosLayoutManager *layout_manager = cdos_layout_manager_get_default ();
      cdos_layout_manager_add_chrome (layout_manager, CLUTTER_ACTOR(CDOS_POPUP_MENU(obj)->actor), 0x4);
      cdos_popup_menu_set_max_height(CDOS_POPUP_MENU(obj));
      obj->is_open = TRUE;
      menu_stack_depth += 1;
     // hover menu 挂掉的bug 在此规避 
      gboolean ret = cdos_box_pointer_set_position(priv->box_pointer,
              (ClutterActor *)cdos_popup_menu_base_get_source_actor(obj), priv->arrow_alignment);

      if (ret){
      	cdos_box_pointer_show(priv->box_pointer, animate);
        ClutterActor *parent = clutter_actor_get_parent (CLUTTER_ACTOR(CDOS_POPUP_MENU(obj)->actor));
//      	clutter_actor_raise_top(CLUTTER_ACTOR(CDOS_POPUP_MENU(obj)->actor));
        clutter_actor_set_child_above_sibling (parent, CLUTTER_ACTOR(CDOS_POPUP_MENU(obj)->actor), NULL);
     
      	g_signal_emit_by_name(obj, "open-state-changed", TRUE);
      }
  }
}

/**
 * cdos_popup_menu_open_to_x_y:
 * @obj: A #CdosPopupMenuBase
 * @animate: 是否显示boxpointer中的actor
　* @x: 新位置的x坐标
 * @y: 新位置的y坐标 
 *
 * 将一个弹出的menu显示在你指定的位置
 *
 */
void 
cdos_popup_menu_open_to_x_y(CdosPopupMenuBase *obj,
                            gboolean animate,
                            gfloat x,
                            gfloat y)
{
    g_return_if_fail(CDOS_IS_POPUP_MENU_BASE(obj));
    CdosPopupMenuPrivate *priv = CDOS_POPUP_MENU(obj)->priv;
    if(!obj->is_open){
        cdos_popup_menu_set_max_height(CDOS_POPUP_MENU(obj));
        cdos_layout_manager_add_chrome (cdos_layout_manager_get_default (),
	                                    CLUTTER_ACTOR(CDOS_POPUP_MENU(obj)->actor), 4);
        obj->is_open = TRUE;
        menu_stack_depth += 1;

	      cdos_box_pointer_set_position_x_y(priv->box_pointer, (ClutterActor *)cdos_popup_menu_base_get_source_actor(obj), priv->arrow_alignment, x, y);
	      cdos_box_pointer_show(priv->box_pointer, animate);

//	      clutter_actor_raise_top(CLUTTER_ACTOR(CDOS_POPUP_MENU(obj)->actor));
          ClutterActor *parent = clutter_actor_get_parent (CLUTTER_ACTOR(CDOS_POPUP_MENU(obj)->actor));
          clutter_actor_set_child_above_sibling (parent, CLUTTER_ACTOR(CDOS_POPUP_MENU(obj)->actor), NULL);

	      g_signal_emit_by_name(obj, "open-state-changed", TRUE);
	  }
}

/**
 * cdos_popup_menu_close:
 * @obj: A #CdosPopupMenuBase
 * @animate: ？？ 
 *
 * 将popupmenu隐藏起来
 *
 */
void 
cdos_popup_menu_close(CdosPopupMenuBase *obj, gboolean animate)
{
    g_return_if_fail(CDOS_IS_POPUP_MENU_BASE(obj));
    CdosPopupMenuPrivate *priv = CDOS_POPUP_MENU(obj)->priv;
    if(obj->is_open){
      obj->is_open = FALSE;
      menu_stack_depth -= 1;
      // Main.panel._hidePanel();
      // if (Main.panel2 != null)
      // Main.panel2._hidePanel();
      if(obj->active_menu_item)
        cdos_popup_base_menu_item_set_active(obj->active_menu_item, FALSE);

      cdos_box_pointer_hide(priv->box_pointer, FALSE);

      CdosLayoutManager *layout_manager = cdos_layout_manager_get_default ();
      cdos_layout_manager_remove_chrome (layout_manager,  CLUTTER_ACTOR(CDOS_POPUP_MENU(obj)->actor));
      g_signal_emit_by_name(obj, "open-state-changed", FALSE);
    }
}

/**
 * cdos_popup_menu_set_max_height:
 * @self: A #CdosPopupMenu
 *
 * 设置self的最大的高度
 *
 */
void 
cdos_popup_menu_set_max_height(CdosPopupMenu *self)
{
    StWidget *source_actor;
    source_actor = cdos_popup_menu_base_get_source_actor ((CdosPopupMenuBase *)self);

    CdosLayoutManager *layout = cdos_layout_manager_get_default ();
    MetaRectangle *monitor;
    monitor = cdos_layout_manager_find_monitor_for_actor (layout, (ClutterActor *)source_actor);
    gint max_height = monitor->height - 100;
    gint max_width = monitor->width;

    StThemeNode *theme_node = st_widget_get_theme_node ((StWidget *)self->actor);
    int top    = st_theme_node_get_border_width (theme_node, ST_SIDE_TOP);
    int bottom = st_theme_node_get_border_width (theme_node, ST_SIDE_BOTTOM);
    int left   = st_theme_node_get_border_width (theme_node, ST_SIDE_LEFT);
    int right  = st_theme_node_get_border_width (theme_node, ST_SIDE_RIGHT);

    max_height -= top + bottom;
    max_width -= left + right;

    gchar style[255] = {0};
    snprintf(style, sizeof(style) - 1, "max-height:%dpx;max-width:%dpx;",
             max_height,
             max_width);
    st_widget_set_style ((StWidget *)self->actor, style);
}

/**
 * cdos_popup_menu_get_box_pointer:
 * @menu: A #CdosPopupMenu
 *
 * 获取self的box_pointer成员
 *
 * Returns: #CdosBoxPointer 
 */
CdosBoxPointer *
cdos_popup_menu_get_box_pointer (CdosPopupMenu *menu)
{
    CdosPopupMenuPrivate *priv = CDOS_POPUP_MENU(menu)->priv;
    // panel 右键菜单需要定位弹出菜单的位置
    return priv->box_pointer;
}

