#include "cdos-popup-separator-menu-item.h"
#include "st.h"
#include <cairo.h>

/**
 * SECTION: cdos-popup-separator-menu-item.c
 * @Title: CdosPopupSeparatorMenuItem
 * @short_description: 继承自CdosPopupBaseMenuItem, 这个控件里是一根线，你可以选择线的方向
 *　　还有就是线是根据css里配置信息，使用cario描绘的。当在一个menu中添加多个item时，而且item是不同的
 *　　类型，就可以使用这个控件。
 * 
 * CdosPopupSeparatorMenuItem 为我们提供了一个有一条线的item
 */

static void 
cdos_popup_separator_menu_item_on_repaint(StDrawingArea *area, CdosPopupSeparatorMenuItem *item);

static void 
cdos_popup_separator_menu_item_finalize(GObject *gobject);
static void
cdos_popup_separator_menu_item_dispose(GObject *gobject);

static void
cdos_popup_separator_menu_item_class_init(CdosPopupSeparatorMenuItemClass *klass);

static void
cdos_popup_separator_menu_item_init(CdosPopupSeparatorMenuItem *item);

#define CDOS_POPUP_SEPARATOR_MENU_ITEM_GET_PRIVATE(obj)    \
	(G_TYPE_INSTANCE_GET_PRIVATE((obj), CDOS_TYPE_POPUP_SEPARATOR_MENU_ITEM, CdosPopupSeparatorMenuItemPrivate))

G_DEFINE_TYPE(CdosPopupSeparatorMenuItem, cdos_popup_separator_menu_item, CDOS_TYPE_POPUP_BASE_MENU_ITEM);

struct _CdosPopupSeparatorMenuItemPrivate
{
	StDrawingArea    *drawingArea;
};
static void cdos_popup_separator_menu_item_add_drawing_area(CdosPopupSeparatorMenuItem *item){
	CdosPopupSeparatorMenuItemPrivate  *priv = item->priv;
	ActorChild 	child;
	child.span = -1;
	child.expand = TRUE;
	cdos_popup_base_menu_item_add_actor(CDOS_POPUP_BASE_MENU_ITEM(item), CLUTTER_ACTOR(priv->drawingArea), &child);
}

static void
cdos_popup_separator_menu_item_init(CdosPopupSeparatorMenuItem *item)
{
	item->priv = CDOS_POPUP_SEPARATOR_MENU_ITEM_GET_PRIVATE(item);

	item->priv->drawingArea = g_object_new(ST_TYPE_DRAWING_AREA, 
					"style-class",
					"popup-separator-menu-item", 
					NULL);

	CdosGenericContainer *actor = NULL;
	cdos_popup_separator_menu_item_add_drawing_area(item);

	g_object_get(CDOS_POPUP_BASE_MENU_ITEM(item),  "actor", &actor, NULL);
	g_signal_connect(item->priv->drawingArea, "repaint",
			G_CALLBACK(cdos_popup_separator_menu_item_on_repaint), item);
}

static void
cdos_popup_separator_menu_item_class_init(CdosPopupSeparatorMenuItemClass *klass)
{
	GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
	g_type_class_add_private(klass, sizeof(CdosPopupSeparatorMenuItemPrivate));

	//gobject_class->finalize = cdos_popup_separator_menu_item_finalize;
	//gobject_class->dispose = cdos_popup_separator_menu_item_dispose;
}

/*
static void
cdos_popup_separator_menu_item_dispose(GObject *gobject)
{
	CdosPopupSeparatorMenuItemPrivate  	*priv;
	priv  = CDOS_POPUP_SEPARATOR_MENU_ITEM(gobject)->priv;
	G_OBJECT_CLASS(cdos_popup_separator_menu_item_parent_class)->dispose (gobject);
}

static void 
cdos_popup_separator_menu_item_finalize(GObject *gobject)
{
	//CdosPopupSeparatorMenuItemPrivate  	*priv;
	//priv  = CDOS_POPUP_SEPARATOR_MENU_ITEM(gobject)->priv;

	G_OBJECT_CLASS(cdos_popup_separator_menu_item_parent_class)->finalize (gobject);
}
*/

/**
 * cdos_popup_separator_menu_item_new:
 * 
 * 新建一个 #CdosPopupSeparatorMenuItem 类的实例。
 *
 * Returns: 返回一个 #CdosPopupSeparatorMenuItem 的实例。
 */
CdosPopupSeparatorMenuItem *
cdos_popup_separator_menu_item_new(void)
{
	CdosPopupSeparatorMenuItem     *gobject;
	PopupBaseMenuItemInit  params;

	gobject = g_object_new(CDOS_TYPE_POPUP_SEPARATOR_MENU_ITEM, NULL);
	cdos_popup_base_menu_item_get_init_params(&params);
	params.reactive = FALSE;
	cdos_popup_base_menu_item_set_params(CDOS_POPUP_BASE_MENU_ITEM(gobject), &params);
    ClutterActor *actor = ((CdosPopupBaseMenuItem *)gobject)->actor;
    clutter_actor_set_reactive (actor, FALSE);
    st_widget_set_can_focus ((StWidget *)actor, FALSE);

	return gobject;
}

static void 
cdos_popup_separator_menu_item_on_repaint(StDrawingArea *area, CdosPopupSeparatorMenuItem *item)
{
	cairo_t 			*cr;
	StThemeNode  		 	*themeNode;
	guint 				width;
	guint 				height;
	gdouble				margin;
	gdouble 			gradientHeight;
	gdouble 			gradientWidth;
	gdouble 			gradientOffset;
	ClutterColor			startColor;
	ClutterColor			endColor;
	cairo_pattern_t 		*pattern;


	cr = st_drawing_area_get_context(area);

	themeNode = st_widget_get_theme_node(ST_WIDGET(area));	

	st_drawing_area_get_surface_size(area, &width, &height);

	margin = st_theme_node_get_length (themeNode, "-margin-horizontal");

	gradientHeight = st_theme_node_get_length(themeNode, "-gradient-height");

	st_theme_node_get_color(themeNode, "-gradient-start", &startColor);

	st_theme_node_get_color(themeNode, "-gradient-end", &endColor);

	gradientWidth = (width - margin * 2.0);

	gradientOffset = (height - gradientHeight) / 2.0;

	pattern = cairo_pattern_create_linear(margin, gradientOffset, width - margin, gradientOffset + gradientHeight);

	cairo_pattern_add_color_stop_rgba(pattern, 0,
                    			startColor.red / 255.0,
			 		startColor.green / 255.0, 
			 		startColor.blue / 255.0,
			 		startColor.alpha / 255.0);


	cairo_pattern_add_color_stop_rgba(pattern, 
					0.5,
                    			endColor.red / 255.0,
			 		endColor.green / 255.0, 
			 		endColor.blue / 255.0,
			 		endColor.alpha / 255.0);

	cairo_pattern_add_color_stop_rgba(pattern, 
					1,
                    			startColor.red / 255.0,
			 		startColor.green / 255.0, 
			 		startColor.blue / 255.0,
			 		startColor.alpha / 255.0);
	cairo_set_source(cr, pattern);

	cairo_rectangle(cr, margin, gradientOffset, gradientWidth, gradientHeight);

	cairo_fill(cr);
}

/**
 * cdos_popup_separator_menu_item_get_drawing_area:
 * @item: A #CdosPopupSeparatorMenuItem
 *
 * 获取item的drawingArea属性
 *
 * Returns: StDrawingArea 控件绘图区域
 */
StDrawingArea *
cdos_popup_separator_menu_item_get_drawing_area (CdosPopupSeparatorMenuItem *item)
{
	g_return_val_if_fail(CDOS_IS_POPUP_SEPARATOR_MENU_ITEM(item),NULL);
	CdosPopupSeparatorMenuItemPrivate  *priv;
	
	priv = item->priv;
	
	return priv->drawingArea;
}

void cdos_popup_separator_menu_item_hide(CdosPopupSeparatorMenuItem *item){
	g_return_if_fail(CDOS_IS_POPUP_SEPARATOR_MENU_ITEM(item));
	CdosPopupSeparatorMenuItemPrivate  *priv = item->priv;
	ClutterActor *drawing_area =  CLUTTER_ACTOR(priv->drawingArea);
	clutter_actor_hide(drawing_area);
	//CdosGenericContainer *actor = cdos_popup_base_menu_item_get_actor(item);
	//st_widget_set_style(actor,"background:#F00;");
	cdos_popup_base_menu_item_hide((CdosPopupBaseMenuItem *)item);
}

void cdos_popup_separator_menu_item_show(CdosPopupSeparatorMenuItem *item){
	g_return_if_fail(CDOS_IS_POPUP_SEPARATOR_MENU_ITEM(item));
	CdosPopupSeparatorMenuItemPrivate  *priv = item->priv;
	ClutterActor *drawing_area =  CLUTTER_ACTOR(priv->drawingArea);
	//CdosGenericContainer *actor = cdos_popup_base_menu_item_get_actor(item);
	//st_widget_set_style(actor,"background:#00f;");
	clutter_actor_show(drawing_area);
	cdos_popup_base_menu_item_show((CdosPopupBaseMenuItem *)item);
}
