#include "cdos-popup-slider-menu-item.h"
#include <clutter/clutter.h>
#include <cairo.h>
/**
 * SECTION: cdos-popup-slider-menu-item.c
 * @Title: CdosPopupSliderMenuItem
 * @short_description: 一个具备滑块，可以移动的item
 *　 
 *
 * CdosPopupSliderMenuItem 一个具备滑块，可以移动的item
 */

#define SLIDER_SCROLL_STEP  0.05  /* Slider scrolling step in %*/

#define CDOS_POPUP_SLIDER_MENU_ITEM_GET_PRIVATE(obj)    \
	(G_TYPE_INSTANCE_GET_PRIVATE((obj), CDOS_TYPE_POPUP_SLIDER_MENU_ITEM, CdosPopupSliderMenuItemPrivate))


G_DEFINE_TYPE(CdosPopupSliderMenuItem, cdos_popup_slider_menu_item, ST_TYPE_WIDGET);

static void 
cdos_popup_slider_menu_item_init(CdosPopupSliderMenuItem *item);

static void 
cdos_popup_slider_menu_item_class_init(CdosPopupSliderMenuItemClass *klass);

static void 
cdos_popup_slider_menu_item_finalize(GObject *gobject);

static void
cdos_popup_slider_menu_item_dispose(GObject *gobject);

static void 
cdos_popup_slider_menu_item_start_dragging(ClutterActor            *actor,
                                           ClutterEvent            *event,
                                           CdosPopupSliderMenuItem *item);

static gboolean 
cdos_popup_slider_menu_item_end_dragging(ClutterActor            *actor,
                                         ClutterEvent            *event,
                                         CdosPopupSliderMenuItem *item);

static void 
cdos_popup_slider_menu_item_on_scroll_event(ClutterActor            *actor, 
                                            ClutterEvent            *event,
                                            CdosPopupSliderMenuItem *item);

static gboolean 
cdos_popup_slider_menu_item_motion_event(ClutterActor            *actor, 
                                         ClutterEvent            *event,
                                         CdosPopupSliderMenuItem *item);

static void 
cdos_popup_slider_menu_item_move_handle(CdosPopupSliderMenuItem *item, 
                                        gfloat                   absX, 
                                        gfloat                   absY);

static gboolean 
cdos_popup_slider_menu_item_ok_key_press_event(ClutterActor            *actor, 
                                               ClutterEvent            *event,
                                               CdosPopupSliderMenuItem *item);

static void 
cdos_popup_slider_menu_item_slider_repaint(CdosPopupSliderMenuItem *item);


struct  _CdosPopupSliderMenuItemPrivate
{
	gfloat 		value;
	gfloat		max_value;
	gfloat 		min_value;
	gfloat		step;

	CdosPopupSliderMenuItemOrientation orientation;

	gulong 		release_id;
	gulong 		motion_id;
	gulong 		repaint_id;
	gulong 		press_event_id;
	gulong 		scroll_event_id;
	gboolean 	dragging;

	StWidget	*box;
	StWidget 	*slider_container;
	StWidget	*value_flag;
	StWidget 	*slider_button;

	StDrawingArea   *slider;
};

/**
 * cdos_popup_slider_menu_item_new:
 * 
 * 新建一个 #CdosPopupSliderMenuItem 类的实例。
 *
 * Returns: 返回一个 #CdosPopupSliderMenuItem 的实例。
 */
CdosPopupSliderMenuItem * 
cdos_popup_slider_menu_item_new(void)
{
	g_object_new(CDOS_TYPE_POPUP_SLIDER_MENU_ITEM, NULL);
}

/**
 * cdos_popup_slider_menu_item_new_with_value:
 * @value: slider宽度的比例，cairo 描绘矩形
 * 
 * 新建一个 #CdosPopupSliderMenuItem 类的实例。
 *
 * Returns: 返回一个 #CdosPopupSliderMenuItem 的实例。
 */
CdosPopupSliderMenuItem *
cdos_popup_slider_menu_item_new_with_value(gfloat value,gfloat max_value,gfloat min_value,gfloat step)
{
	
	CdosPopupSliderMenuItem    *gobject;
	gobject = g_object_new(CDOS_TYPE_POPUP_SLIDER_MENU_ITEM, NULL);
	
	PopupBaseMenuItemInit       params;
	cdos_popup_base_menu_item_get_init_params(&params);

	params.activate = FALSE;

	cdos_popup_base_menu_item_set_params(CDOS_POPUP_BASE_MENU_ITEM(gobject), &params);
	
	cdos_popup_slider_menu_item_set_value(gobject, value);

	return gobject;
}

static void 
cdos_popup_slider_menu_item_init(CdosPopupSliderMenuItem *item)
{
	CdosGenericContainer 	        *actor;
	CdosPopupSliderMenuItemPrivate  *priv;
	ActorChild 			 child;	
		
	cdos_popup_base_menu_item_get_actor_child(&child);

	priv = item->priv = CDOS_POPUP_SLIDER_MENU_ITEM_GET_PRIVATE(item);
	g_object_get(item, "actor", &actor, NULL);

	g_signal_connect(actor, "key-press-event", 
		G_CALLBACK(cdos_popup_slider_menu_item_ok_key_press_event), item);

	
	priv->value = 1.0;
	priv->slider = g_object_new(ST_TYPE_DRAWING_AREA,
				"style-class",
				"ui-popup-slider-menu-item",
				"reactive",
				TRUE,
				NULL);

	child.span = -1;
	child.expand = TRUE;
	cdos_popup_base_menu_item_add_actor(CDOS_POPUP_BASE_MENU_ITEM(item), CLUTTER_ACTOR(priv->slider), &child);

	priv->repaint_id = g_signal_connect(priv->slider, "repaint",
		G_CALLBACK(cdos_popup_slider_menu_item_slider_repaint), item);

	priv->press_event_id = g_signal_connect(actor, "button-press-event",
		G_CALLBACK(cdos_popup_slider_menu_item_start_dragging), item);

	priv->scroll_event_id = g_signal_connect(actor, "scroll-event",
		G_CALLBACK(cdos_popup_slider_menu_item_on_scroll_event), item);

	priv->release_id = 0;
	priv->motion_id = 0;
	priv->dragging = FALSE;
}

static void
cdos_popup_slider_menu_item_class_init(CdosPopupSliderMenuItemClass *klass)
{
	GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
	GParamSpec *pspec;

	g_type_class_add_private(klass, sizeof(CdosPopupSliderMenuItemPrivate));

	gobject_class->finalize = cdos_popup_slider_menu_item_finalize;
	gobject_class->dispose = cdos_popup_slider_menu_item_dispose;
}

static void
cdos_popup_slider_menu_item_dispose(GObject *gobject)
{
	CdosPopupSliderMenuItemPrivate 	*priv;
	priv = CDOS_POPUP_SLIDER_MENU_ITEM(gobject)->priv;
	CdosGenericContainer	*actor = cdos_popup_base_menu_item_get_actor(CDOS_POPUP_BASE_MENU_ITEM(gobject));
	if(priv->release_id > 0){
		g_signal_handler_disconnect(priv->slider, priv->release_id);
		priv->release_id = 0L;
	}
	if(priv->motion_id > 0){
		g_signal_handler_disconnect(priv->slider, priv->motion_id);
		priv->motion_id = 0L;
	}
	if(priv->repaint_id > 0){
		g_signal_handler_disconnect(priv->slider, priv->repaint_id);
		priv->repaint_id = 0L;
	}
	if(priv->press_event_id > 0){
		g_signal_handler_disconnect(actor, priv->press_event_id);
		priv->press_event_id = 0L;
	}
	if(priv->scroll_event_id > 0){
		g_signal_handler_disconnect(actor, priv->scroll_event_id);
		priv->scroll_event_id = 0L;
	}
	g_clear_object(&priv->slider);

	G_OBJECT_CLASS(cdos_popup_slider_menu_item_parent_class)->dispose(gobject);
}

static void 
cdos_popup_slider_menu_item_finalize(GObject *gobject)
{
	CdosPopupSliderMenuItemPrivate 	*priv;
	priv = CDOS_POPUP_SLIDER_MENU_ITEM(gobject)->priv;

	G_OBJECT_CLASS(cdos_popup_slider_menu_item_parent_class)->finalize(gobject);
}	

/**
 * cdos_popup_slider_menu_item_set_value:
 * @item: A #CdosPopupSliderMenuItem
 * @value: sliderwidth*value 
 *
 * 设置self的value值
 *
 */
void 
cdos_popup_slider_menu_item_set_value(CdosPopupSliderMenuItem *item, gfloat value)
{
	CdosPopupSliderMenuItemPrivate *priv;
	g_return_if_fail(CDOS_IS_POPUP_SLIDER_MENU_ITEM(item));

	priv->value = MAX(MIN(value, 1), 0);

	st_drawing_area_queue_repaint(priv->slider);
}

static void 
cdos_popup_slider_menu_item_start_dragging(ClutterActor            *actor,
					   ClutterEvent            *event,
					   CdosPopupSliderMenuItem *item)
{
	CdosPopupSliderMenuItemPrivate *priv;
	gfloat absX;
	gfloat absY;
	g_return_if_fail(CDOS_IS_POPUP_SLIDER_MENU_ITEM(item));

	priv = item->priv;
	if (priv->dragging)     // dont't allow two drags at the same time 
		return;

	priv->dragging = TRUE;

	clutter_grab_pointer(CLUTTER_ACTOR(priv->slider));
	priv->release_id = g_signal_connect(priv->slider, "button-release-event",
		G_CALLBACK(cdos_popup_slider_menu_item_end_dragging), item);

	priv->motion_id = g_signal_connect(priv->slider, "motion-event",
		G_CALLBACK(cdos_popup_slider_menu_item_motion_event), item);

	clutter_event_get_coords(event, &absX, &absY);
	cdos_popup_slider_menu_item_move_handle(item, absX, absY);	

}

static gboolean 
cdos_popup_slider_menu_item_end_dragging(ClutterActor            *actor,
				         ClutterEvent            *event,
				         CdosPopupSliderMenuItem *item)
{
	CdosPopupSliderMenuItemPrivate *priv;
	priv = item->priv;

	if (priv->dragging){
		g_signal_handler_disconnect(priv->slider, priv->release_id);
		priv->release_id = 0L;
		g_signal_handler_disconnect(priv->slider, priv->motion_id);
		priv->motion_id = 0L;
		clutter_ungrab_pointer();
		priv->dragging = FALSE;

		g_signal_emit_by_name(item, "drag-end");
	}	
	return TRUE;
}

static void 
cdos_popup_slider_menu_item_on_scroll_event(ClutterActor            *actor, 
					    ClutterEvent            *event,
					    CdosPopupSliderMenuItem *item)
{
	ClutterScrollDirection direction;

	direction = clutter_event_get_scroll_direction(event);

	if (direction == CLUTTER_SCROLL_DOWN){
		item->priv->value = MAX(0, item->priv->value - SLIDER_SCROLL_STEP);
	}else if (direction == CLUTTER_SCROLL_UP){
		item->priv->value = MIN(1, item->priv->value + SLIDER_SCROLL_STEP);
	}

	st_drawing_area_queue_repaint(item->priv->slider);
	g_signal_emit_by_name(item, "value-changed", item->priv->value);
}

static gboolean 
cdos_popup_slider_menu_item_motion_event(ClutterActor            *actor, 
				         ClutterEvent            *event,
				         CdosPopupSliderMenuItem *item)
{
	gfloat				absX;
	gfloat				absY;

	clutter_event_get_coords(event, &absX, &absY);
	cdos_popup_slider_menu_item_move_handle(item, absX, absY);
	return TRUE;	
}

static void 
cdos_popup_slider_menu_item_move_handle(CdosPopupSliderMenuItem *item, 
				        gfloat                   absX, 
				        gfloat                   absY)
{
	gfloat 			           width;
	gfloat 		  	           handleRadius;
	gfloat                             relX;
	gfloat                             relY;
	gfloat                             siderX;
	gfloat                             siderY;
	gfloat		  	           newValue;
	CdosPopupSliderMenuItemPrivate    *priv;
	StThemeNode 		          *themeNode;

	priv = item->priv;
	clutter_actor_get_transformed_position(CLUTTER_ACTOR(priv->slider), &siderX, &siderY);
	relX = absX - siderX;
	relY = absY - siderY;

	width = clutter_actor_get_width(CLUTTER_ACTOR(priv->slider));
	themeNode = st_widget_get_theme_node(ST_WIDGET(priv->slider));
	handleRadius = st_theme_node_get_length(themeNode, "-slider-handler-radius");

	 if (relX < handleRadius)
	 	newValue = 0;
	 else if (relX > width -handleRadius)
	 	newValue = 1;
	 else
	 	newValue = (relX - handleRadius) / (width - 2 * handleRadius);

	 priv->value = newValue;
	 st_drawing_area_queue_repaint(priv->slider);	 
	 g_signal_emit_by_name(item, "value-changed", priv->value);

}

static gboolean 
cdos_popup_slider_menu_item_ok_key_press_event(ClutterActor            *actor, 
					       ClutterEvent            *event,
					       CdosPopupSliderMenuItem *item)
{
	guint				   key;
	guint				   delta;
	CdosPopupSliderMenuItemPrivate    *priv;
	gfloat 				   value;

	priv = item->priv;

	key = clutter_event_get_key_symbol(event);
	if (key == CLUTTER_KEY_Right || key == CLUTTER_KEY_Left){
		 delta = key == CLUTTER_KEY_Right ? 1.0 : -0.1;
		 if (priv -> value + delta < 1)
		 	value = value + delta;
		 else 
		 	value = 1;
		 if (value > 0) 
		 	priv->value = value;
		 else 
		 	priv->value = 0;
		 st_drawing_area_queue_repaint(priv->slider);
		 g_signal_emit_by_name(item, "value-changed", priv->value);
		 g_signal_emit_by_name(item, "drag-end");
	} 
	return FALSE;
}

/**
 * cdos_popup_slider_menu_item_get_value:
 * @item: A #CdosPopupSliderMenuItem
 *
 * 获取item的value属性
 *
 * Returns: #gfloat　value的大小
 */
gfloat 
cdos_popup_slider_menu_item_get_value(CdosPopupSliderMenuItem *item)
{
	CdosPopupSliderMenuItemPrivate *priv;
	g_return_if_fail(CDOS_IS_POPUP_SLIDER_MENU_ITEM(item));

	priv = item->priv;
	return priv->value;
}

static void 
cdos_popup_slider_menu_item_slider_repaint(CdosPopupSliderMenuItem *item)
{

	CdosPopupSliderMenuItemPrivate 		       *priv;
	cairo_t 				       *cr;
	StThemeNode 				       *themeNode;
	guint 						width;
	guint 						height;
	gfloat					    	handleRadius;
	gfloat						sliderWidth;
	gfloat						sliderHeight;
	gfloat						sliderBorderWidth;
	gfloat						handleY;
	gfloat						handleX;

	ClutterColor				       *sliderBorderColor;
	ClutterColor				       *sliderColor;
	ClutterColor				       *sliderActiveBorderColor;
	ClutterColor				       *sliderActiveColor;
	ClutterColor 				       *color;

	priv = item->priv;

	cr = st_drawing_area_get_context(priv->slider);
	themeNode = st_widget_get_theme_node(ST_WIDGET(priv->slider));
	st_drawing_area_get_surface_size(priv->slider, &width, &height);

	handleRadius = st_theme_node_get_length(themeNode, "-slider-handler-radius");

	sliderWidth = width - 2 * handleRadius;
	sliderHeight = st_theme_node_get_length(themeNode, "-slider-height");

	sliderBorderWidth = st_theme_node_get_length(themeNode, "-slider-border-width");

	st_theme_node_get_color(themeNode, "-slider-border-color", sliderBorderColor);

	st_theme_node_get_color(themeNode, "-slider-background-color", sliderColor);

	st_theme_node_get_color(themeNode, "-slider-active-border-color", sliderActiveBorderColor);

	st_theme_node_get_color(themeNode, "-slider-active-background-color", sliderActiveColor);

	cairo_set_source_rgba(cr, 
		sliderActiveColor->red / 255,
		sliderActiveColor->green / 255,
		sliderActiveColor->blue / 255,
		sliderActiveColor->alpha / 255);

	cairo_rectangle(cr, 
		handleRadius,
		(height - sliderHeight) / 2,
		sliderWidth * (priv->value),
		sliderHeight);

	cairo_fill_preserve(cr);

	cairo_set_source_rgba(cr,
		sliderActiveBorderColor->red / 255,
		sliderActiveBorderColor->green / 255,
		sliderActiveBorderColor->blue / 255,
		sliderActiveBorderColor->alpha / 255);

	cairo_set_line_width(cr, sliderBorderWidth);
	cairo_stroke(cr);
	handleY = height / 2;
	handleX = handleRadius + (width - 2 * handleRadius ) * priv->value;

	st_theme_node_get_foreground_color(themeNode, color);
	cairo_set_source_rgba(cr,
		color->red / 255,
		color->green / 255,
		color->blue / 255,
		color->alpha / 255);
	cairo_arc(cr,
		handleX,
		handleY,
		handleRadius,
		0,
		2*G_PI);
	cairo_fill(cr);
}

void
cdos_popup_slider_menu_item_set_max_value(CdosPopupSliderMenuItem *item, gfloat value){

	g_return_if_fail(CDOS_IS_POPUP_SLIDER_MENU_ITEM(item));
	CdosPopupSliderMenuItemPrivate *priv = item->priv;
	priv->max_value = value;

}

gfloat
cdos_popup_slider_menu_item_get_max_value(CdosPopupSliderMenuItem *item){
	g_return_if_fail(CDOS_IS_POPUP_SLIDER_MENU_ITEM(item));
	CdosPopupSliderMenuItemPrivate *priv = item->priv;

	return priv->max_value;
}

void
cdos_popup_slider_menu_item_set_min_value(CdosPopupSliderMenuItem *item, gfloat value){
	g_return_if_fail(CDOS_IS_POPUP_SLIDER_MENU_ITEM(item));
	CdosPopupSliderMenuItemPrivate *priv = item->priv;
	priv->min_value = value;

}

gfloat
cdos_popup_slider_menu_item_get_min_value(CdosPopupSliderMenuItem *item){
	g_return_if_fail(CDOS_IS_POPUP_SLIDER_MENU_ITEM(item));
	CdosPopupSliderMenuItemPrivate *priv = item->priv;
	return priv->min_value;
}

void
cdos_popup_slider_menu_item_set_step(CdosPopupSliderMenuItem *item, gfloat step){
	g_return_if_fail(CDOS_IS_POPUP_SLIDER_MENU_ITEM(item));
	CdosPopupSliderMenuItemPrivate *priv = item->priv;
	priv->step = step;
}

gfloat
cdos_popup_slider_menu_item_get_step(CdosPopupSliderMenuItem *item){
	g_return_if_fail(CDOS_IS_POPUP_SLIDER_MENU_ITEM(item));
	CdosPopupSliderMenuItemPrivate *priv = item->priv;
	return priv->step;
}

void
cdos_popup_slider_menu_item_set_orientation(CdosPopupSliderMenuItem *item,CdosPopupSliderMenuItemOrientation orientation){
	g_return_if_fail(CDOS_IS_POPUP_SLIDER_MENU_ITEM(item));
	CdosPopupSliderMenuItemPrivate *priv = item->priv;
	priv->orientation = orientation;
}

CdosPopupSliderMenuItemOrientation
cdos_popup_slider_menu_item_get_orientation(CdosPopupSliderMenuItem *item){
	g_return_if_fail(CDOS_IS_POPUP_SLIDER_MENU_ITEM(item));
	CdosPopupSliderMenuItemPrivate *priv = item->priv;
	return priv->orientation;
}
