#include "cdos-popup-sub-menu.h"
#include "cdos-popup-menu.h"

/**
 * SECTION: cdos-popup-sub-menu.c
 * @Title: CdosPopupSubMenu
 * @short_description: 提供一个子菜单控件。
 *　 
 *
 * CdosPopupSubMenu 提供一个子菜单的控件 
 */

G_DEFINE_TYPE (CdosPopupSubMenu, cdos_popup_sub_menu, CDOS_TYPE_POPUP_MENU_BASE);

#define POPUP_SUB_MENU_GET_PRIVATE(obj)(G_TYPE_INSTANCE_GET_PRIVATE ((obj), CDOS_TYPE_POPUP_SUB_MENU, CdosPopupSubMenuPrivate))

struct _CdosPopupSubMenuPrivate
{
    ClutterActor *source_actor;
    ClutterActor *arrow;
    guint rotation_center_z_gravity;
    guint rotation_angle_z;
};

enum
{
    PROP_0
};

static void
cdos_popup_sub_menu_finalize (GObject * gobject)
{
    G_OBJECT_CLASS (cdos_popup_sub_menu_parent_class)->finalize (gobject);
}

static void
cdos_popup_sub_menu_dispose (GObject * gobject)
{
    CdosPopupSubMenu *self = CDOS_POPUP_SUB_MENU(gobject);
    CdosPopupSubMenuPrivate * priv = self->priv;

    g_object_unref(priv->arrow);
    if (self->actor)
        clutter_actor_destroy ((ClutterActor *)self->actor);
    self->actor = NULL;

    G_OBJECT_CLASS (cdos_popup_sub_menu_parent_class)->dispose (gobject);
}

static void
cdos_popup_sub_menu_set_property (GObject * gobject,
                                guint prop_id,
                                const GValue * value,
                                GParamSpec * pspec)
{
    //CdosPopupSubMenu * newobj = CDOS_POPUP_SUB_MENU (gobject);

    switch (prop_id)
    {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
        break;
    }
}

static void
cdos_popup_sub_menu_get_property (GObject * gobject,
                                guint prop_id,
                                GValue * value,
                                GParamSpec * pspec)
{

    //CdosPopupSubMenuPrivate * priv = CDOS_POPUP_SUB_MENU (gobject)->priv;
    switch (prop_id)
    {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
        break;
    }
}

static ClutterActor*
cdos_popup_sub_menu_get_actor(CdosPopupMenuBase *obj)
{
    CdosPopupSubMenu *self = CDOS_POPUP_SUB_MENU(obj);
    return CLUTTER_ACTOR(self->actor);
}

static void
cdos_popup_sub_menu_class_init (CdosPopupSubMenuClass * klass)
{
    CdosPopupMenuBaseClass *parent_class = CDOS_POPUP_MENU_BASE_CLASS(klass);
    GObjectClass * gobject_class = G_OBJECT_CLASS (klass);
    parent_class->open = cdos_popup_sub_menu_open;
    parent_class->close = cdos_popup_sub_menu_close;
    parent_class->get_actor=cdos_popup_sub_menu_get_actor;
    gobject_class->finalize = cdos_popup_sub_menu_finalize;
    gobject_class->dispose = cdos_popup_sub_menu_dispose;
    gobject_class->set_property = cdos_popup_sub_menu_set_property;
    gobject_class->get_property = cdos_popup_sub_menu_get_property;

    g_type_class_add_private (klass, sizeof (CdosPopupSubMenuPrivate));

}

static void
cdos_popup_sub_menu_init (CdosPopupSubMenu * self)
{

    //CdosPopupSubMenuPrivate * priv;
    self->priv = POPUP_SUB_MENU_GET_PRIVATE (self);
}


static void 
_vscroll_start_callback(CdosPopupSubMenu *self)
{
      CdosPopupMenuBase *top_menu = cdos_popup_sub_menu_get_top_menu(self);
      if (top_menu)
		  CDOS_POPUP_MENU_BASE(top_menu)->pass_events = TRUE;
}

static void 
_vscroll_stop_callback(CdosPopupSubMenu *self)
{
	   CdosPopupMenuBase *top_menu = cdos_popup_sub_menu_get_top_menu(self);
      if (top_menu)
	      CDOS_POPUP_MENU_BASE(top_menu)->pass_events = FALSE;
}

static gboolean
cdos_popup_sub_menu_on_key_press_event(ClutterActor *actor, ClutterEvent *event, CdosPopupSubMenu *self)
{

    if(CDOS_POPUP_MENU_BASE(self)->is_open && clutter_event_get_key_symbol(event) == CLUTTER_KEY_Left){
      cdos_popup_sub_menu_close(CDOS_POPUP_MENU_BASE(self), TRUE);
      CdosPopupBaseMenuItem *item = _obj_get_delegate ((GObject *)self->priv->source_actor);
      cdos_popup_base_menu_item_set_active (item, TRUE);
      return TRUE;
    }
    return FALSE;
}

/**
 * cdos_popup_sub_menu_new:
 * @source_actor: source_actor
 * @source_arrow: source_arrow
 * 
 * 新建一个 #CdosPopupSubMenu 类的实例。
 *
 * Returns: 返回一个 #CdosPopupSubMenu 的实例。
 */
CdosPopupSubMenu *
cdos_popup_sub_menu_new (ClutterActor *source_actor, ClutterActor *source_arrow)
{
    CdosPopupSubMenu *self;
    CdosPopupSubMenuPrivate *priv;
    self = g_object_new (CDOS_TYPE_POPUP_SUB_MENU, NULL);
    priv = self->priv = POPUP_SUB_MENU_GET_PRIVATE (self);

    //g_object_set(CDOS_POPUP_MENU_BASE(self), "source_actor", source_actor, NULL);
    priv->source_actor = source_actor;
    priv->arrow = NULL;
    if(source_arrow){
        priv->arrow = g_object_ref(source_arrow);
      //priv->rotation_center_z_gravity = CLUTTER_GRAVITY_CENTER;
      clutter_actor_set_pivot_point (priv->arrow, 0.5f, 0.5f);
    }
    //with long content, we make it scrollable - the scrollbar will only take
    //effect if a CSS max-height is set on the top menu.
    self->actor = g_object_new (ST_TYPE_SCROLL_VIEW, 
                                "style-class", "popup-sub-menu",
                                "hscrollbar-policy", GTK_POLICY_NEVER,
                                "vscrollbar-policy", GTK_POLICY_NEVER, NULL);
    g_object_set_data ((GObject *)self->actor, "_delegate", self);
    // StScrollbar plays dirty tricks with events, calling
    // clutter_set_motion_events_enabled (FALSE) during the scroll; this
    // confuses our event tracking, so we just turn it off during the
    // scroll.
    ClutterActor *vscroll = st_scroll_view_get_vscroll_bar(self->actor);
    g_signal_connect(vscroll, "scroll-start", G_CALLBACK(_vscroll_start_callback), self);
    g_signal_connect(vscroll, "scroll-stop", G_CALLBACK(_vscroll_stop_callback), self);
    clutter_container_add_actor ((ClutterContainer *)self->actor, CLUTTER_ACTOR(cdos_popup_menu_base_get_box(CDOS_POPUP_MENU_BASE(self))));
    // this.actor._delegate = this;
    clutter_actor_set_clip_to_allocation(CLUTTER_ACTOR(self->actor), TRUE);
    g_signal_connect(self->actor, "key-press-event", G_CALLBACK(cdos_popup_sub_menu_on_key_press_event), self);
    clutter_actor_hide(CLUTTER_ACTOR(self->actor));

    return self;
}

/**
 * cdos_popup_sub_menu_get_top_menu:
 * @self: A #CdosPopupSubMenu
 *
 * 获取CdosPopupMenuBase的顶端menu
 *
 * Returns: #CdosPopupMenuBase
 */
CdosPopupMenuBase * 
cdos_popup_sub_menu_get_top_menu(CdosPopupSubMenu *self)
{
    g_return_val_if_fail(CDOS_IS_POPUP_SUB_MENU(self),NULL);
    ClutterActor *actor = clutter_actor_get_parent(CLUTTER_ACTOR(self->actor));
    while(actor != NULL ){
        //if (actor._delegate && actor._delegate instanceof PopupMenu)
        //    return actor._delegate;
        if(obj_get_delegate(actor) && CDOS_IS_POPUP_MENU(obj_get_delegate(actor)))
          return obj_get_delegate(actor);
        actor = clutter_actor_get_parent(CLUTTER_ACTOR(actor));
    }
    return NULL;
}

/**
 * cdos_popup_sub_menu_needs_scrollbar:
 * @self: A #CdosPopupSubMenu
 *
 * 获取CdosPopupMenuBase是否需要scrollbar
 *
 * Returns: #gboolean 是否需要scrollbar 1需要　0不需要
 */
static gboolean
cdos_popup_sub_menu_needs_scrollbar(CdosPopupSubMenu *self)
{
    CdosPopupMenuBase *top_menu = cdos_popup_sub_menu_get_top_menu(self);
    if (!top_menu)
        return 0;

    gfloat top_min_height, top_natural_height;
    ClutterActor *actor = CDOS_POPUP_MENU_BASE_GET_CLASS(top_menu)->get_actor(top_menu);
    clutter_actor_get_preferred_height(actor, -1, &top_min_height, &top_natural_height);
    StThemeNode *top_theme_node = st_widget_get_theme_node(ST_WIDGET(actor));
    int top_max_height = st_theme_node_get_max_height(top_theme_node);
    return top_max_height >= 0 && top_natural_height >= top_max_height;
}

/**
 * cdos_popup_sub_menu_open:
 * @obj: A #CdosPopupMenuBase
 * @animate: 是否显示 
 *
 *　打开menu
 *
 */
void
cdos_popup_sub_menu_open(CdosPopupMenuBase *obj, gboolean animate)
{
    CdosPopupSubMenu *self = CDOS_POPUP_SUB_MENU(obj);
    g_return_if_fail(CDOS_IS_POPUP_SUB_MENU(self));
    if(obj->is_open)
      return;
    obj->is_open = TRUE;
    clutter_actor_show(CLUTTER_ACTOR(self->actor));

    gboolean needs_scrollbar = cdos_popup_sub_menu_needs_scrollbar (self);
    GtkPolicyType policy = needs_scrollbar ? GTK_POLICY_AUTOMATIC : GTK_POLICY_NEVER;
    st_scroll_view_set_policy (self->actor, GTK_POLICY_NEVER, policy);

    if (animate && needs_scrollbar)
        animate = TRUE;
    gdouble rotation_angle = 90;
    if (st_widget_get_direction ((StWidget *)self->actor) == ST_TEXT_DIRECTION_RTL)
        rotation_angle = 270.0;
    if (self->priv->arrow)
    {
        clutter_actor_set_rotation_angle (self->priv->arrow,
                                          CLUTTER_Z_AXIS,
                                          rotation_angle);
    }
    g_signal_emit_by_name(CDOS_POPUP_SUB_MENU(obj), "open-state-changed", TRUE);
}

/**
 * cdos_popup_sub_menu_close:
 * @obj: A #CdosPopupMenuBase
 * @animate: ?? 
 *
 *　关闭menu
 *
 */
void
cdos_popup_sub_menu_close(CdosPopupMenuBase *obj, gboolean animate)
{
    CdosPopupSubMenu *self = CDOS_POPUP_SUB_MENU(obj);
    g_return_if_fail(CDOS_IS_POPUP_SUB_MENU(self));
    if (!obj->is_open)
        return;
    obj->is_open = FALSE;
    if (obj->active_menu_item)
        cdos_popup_base_menu_item_set_active(obj->active_menu_item, FALSE);
    gboolean needs_scrollbar = cdos_popup_sub_menu_needs_scrollbar (self);
    //GtkPolicyType policy = needs_scrollbar ? GTK_POLICY_AUTOMATIC : GTK_POLICY_NEVER;

    if (animate && needs_scrollbar)
        animate = TRUE;
    gdouble rotation_angle = 90.0;
    if (st_widget_get_direction ((StWidget *)self->actor) == ST_TEXT_DIRECTION_RTL)
        rotation_angle = 270.0;
    if (self->priv->arrow)
    {
        clutter_actor_set_rotation_angle (self->priv->arrow,
                                          CLUTTER_Z_AXIS,
                                          0.0);
    }
    clutter_actor_hide ((ClutterActor *)self->actor);
    g_signal_emit_by_name(CDOS_POPUP_SUB_MENU(obj), "open-state-changed", FALSE);
}
