 #include "cdos-popup-submenu-menu-item.h"
#include "cdos-popup-sub-menu.h"
#include <gtk/gtk.h>
#include "cdos-start.h"

/**
 * SECTION: cdos-popup-submenu-menu-item.c
 * @Title: CdosPopupSubmenuMenuItem
 * @short_description:　具备多个子菜单项的item 
 *　
 *
 * CdosPopupSubmenuMenuItem  具备多个子菜单项的item
 */

#define CDOS_POPUP_SUBMENU_MENU_ITEM_GET_PRIVATE(obj)	\
    (G_TYPE_INSTANCE_GET_PRIVATE((obj), CDOS_TYPE_POPUP_SUBMENU_MENU_ITEM, CdosPopupSubmenuMenuItemPrivate))

G_DEFINE_TYPE(CdosPopupSubmenuMenuItem, cdos_popup_submenu_menu_item, CDOS_TYPE_POPUP_BASE_MENU_ITEM);

enum {
    PROP_0,
    PROP_LABEL,
    PROP_HIDE_EXPANDER
};

struct  _CdosPopupSubmenuMenuItemPrivate
{
	StTable 		*table;
	StIcon 			*triangle;
	gboolean        hide_expander;
	StLabel 		*label;
	gfloat 			 margin_left_label_icon;
};

static void
cdos_popup_submenu_menu_item_finalize(GObject *gobject);
static void
cdos_popup_submenu_menu_item_dispose(GObject *gobject);

static void 
cdos_popup_submenu_menu_item_init(CdosPopupSubmenuMenuItem *item);

static void 
cdos_popup_submenu_menu_item_class_init(CdosPopupSubmenuMenuItemClass *klass);

static void
cdos_popup_submenu_menu_item_set_text(CdosPopupSubmenuMenuItem *item, 
                                      const gchar *text,
                                      gboolean hide_expander);

static void 
cdos_popup_submenu_menu_item_open_state_changed(CdosPopupMenuBase        *menu,
						gboolean                  open,
                                                CdosPopupSubmenuMenuItem *item) 
{
    g_return_if_fail (CDOS_IS_POPUP_SUBMENU_MENU_ITEM (item));
    CdosPopupBaseMenuItem *self = (void *)item;
    StWidget *actor = (StWidget *)self->actor;
	if (open)
		st_widget_add_style_pseudo_class (actor, "open");
	else
		st_widget_remove_style_pseudo_class (actor, "open");
}

static gboolean
cdos_popup_submenu_menu_item_on_key_press_event(ClutterActor          *actor,
                                                ClutterEvent          *event,
                                                CdosPopupBaseMenuItem *item);

static void 
cdos_popup_submenu_menu_item_on_button_release_event(CdosPopupSubmenuMenuItem *item)
{
        cdos_popup_menu_base_toggle((CdosPopupMenuBase *)item->menu);
}

static void
cdos_popup_submenu_menu_item_dispose(GObject *gobject){

	CdosPopupSubmenuMenuItem *self = CDOS_POPUP_SUBMENU_MENU_ITEM(gobject);
	CdosPopupSubmenuMenuItemPrivate 		*priv = self->priv;

	g_clear_object(&self->menu);

	G_OBJECT_CLASS(cdos_popup_submenu_menu_item_parent_class)->dispose (gobject);
}

static void
cdos_popup_submenu_menu_item_finalize(GObject *gobject)
{
	//CdosPopupSubmenuMenuItem *self = CDOS_POPUP_SUBMENU_MENU_ITEM(gobject);
	//CdosPopupSubmenuMenuItemPrivate 		*priv = self->priv;

	G_OBJECT_CLASS(cdos_popup_submenu_menu_item_parent_class)->finalize (gobject);
}

static void 
cdos_popup_submenu_menu_item_init(CdosPopupSubmenuMenuItem *item)
{
	CdosPopupSubmenuMenuItemPrivate  *priv;
	CdosGenericContainer 		 *actor;

	priv = item->priv = CDOS_POPUP_SUBMENU_MENU_ITEM_GET_PRIVATE(item);

	actor = cdos_popup_base_menu_item_get_actor((CdosPopupBaseMenuItem *)item);

	st_widget_add_style_class_name(ST_WIDGET(actor), "cdos-popup-submenu-menu-item");

	priv->table = g_object_new(ST_TYPE_TABLE,
			"reactive", TRUE,
			"homogeneous", FALSE,
			NULL);

//	clutter_container_add_actor(priv->table, priv->label);

    	priv->label = (StLabel *)st_label_new(NULL);
		st_widget_set_style_class_name(priv->label,"popup-menu-item-label");

    	st_container_add_actor((ClutterContainer *)priv->table, (ClutterActor *)priv->label);

    	ActorChild child;
    	cdos_popup_base_menu_item_get_actor_child (&child);
    	child.expand = 1;
    	child.span = 1;
    	child.align = ST_ALIGN_START;
    	cdos_popup_base_menu_item_add_actor(CDOS_POPUP_BASE_MENU_ITEM(item), (ClutterActor *)priv->table, &child);
}

static void 
cdos_popup_submenu_menu_item_activate(CdosPopupBaseMenuItem *item, 
				      gpointer               event,
                                      gboolean               keep_menu)
{
   	CdosPopupSubmenuMenuItem *self = CDOS_POPUP_SUBMENU_MENU_ITEM (item);
   	//CdosPopupSubmenuMenuItemPrivate *priv = self->priv;
	//cdos_popup_sub_menu_open((CdosPopupMenuBase *)priv->menu, TRUE);
    	cdos_popup_menu_base_toggle((CdosPopupMenuBase *)self->menu);    
}

static void 
set_hide_expander (CdosPopupSubmenuMenuItem *item, gboolean hide_expander)
{
    	CdosPopupSubmenuMenuItemPrivate *priv = item->priv;
    	StTableChild *child = NULL;

	if (!hide_expander) {
		priv->triangle = g_object_new(ST_TYPE_ICON,
					"icon-name", "media-playback-start",
					"icon-type", ST_ICON_SYMBOLIC,
					"style-class", "popup-menu-icon",
					NULL);

		st_container_add_actor((ClutterContainer *)priv->table, (ClutterActor *)priv->triangle);

		child = (StTableChild *)clutter_container_get_child_meta(CLUTTER_CONTAINER(priv->table), (ClutterActor *)priv->triangle);

		child->row = 0;
		child->col = 0;
		g_object_set (child, "row", 0, "col", 0, NULL);
		child->col_span = 1;
		child->x_expand = FALSE;
		child->x_align = ST_ALIGN_START;
		clutter_actor_queue_relayout ((ClutterActor *)priv->triangle);

		clutter_actor_set_margin_left(CLUTTER_ACTOR(priv->label), priv->margin_left_label_icon);

		child = (StTableChild *)clutter_container_get_child_meta(CLUTTER_CONTAINER(priv->table), (ClutterActor *)priv->label);
		
		//child->row = 0;
		//child->col = 1;
		g_object_set (child, "row", 0, "col", 1, NULL);
		child->col_span = 1;
		child->x_align = ST_ALIGN_START;
        
		clutter_actor_queue_relayout ((ClutterActor *)priv->label);
	} else {
		if (priv->triangle)
        {
			st_container_remove_actor ((ClutterContainer *)priv->table, (ClutterActor *)priv->triangle);
           	g_object_unref(priv->triangle);
			priv->triangle = NULL;
        }

		child = (StTableChild *)clutter_container_get_child_meta((ClutterContainer *)priv->table, (ClutterActor *)priv->label);
		child->row = 0;
		child->col = 0;
		g_object_set (child, "row", 0, "col", 0, NULL);
		child->col_span = 1;
		child->x_align = ST_ALIGN_START;
		clutter_actor_queue_relayout ((ClutterActor *)priv->label);
	}
}

static void 
set_property (GObject *gobject,
              guint prop_id,
              const GValue *value,
              GParamSpec *pspec)
{
    CdosPopupSubmenuMenuItem *item = CDOS_POPUP_SUBMENU_MENU_ITEM (gobject);
    CdosPopupSubmenuMenuItemPrivate *priv = item->priv;
    switch (prop_id)
    {
        case PROP_LABEL:
          {
            const char *text = g_value_get_string (value);
            st_label_set_text (priv->label, text);
            break;
          }
        case PROP_HIDE_EXPANDER:
            priv->hide_expander = g_value_get_boolean (value);
            set_hide_expander (item, priv->hide_expander);
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
            break;
    }
}

static void 
get_property (GObject *gobject,
              guint prop_id,
              GValue *value,
              GParamSpec *pspec)
{
    CdosPopupSubmenuMenuItem *item = CDOS_POPUP_SUBMENU_MENU_ITEM (gobject);
    CdosPopupSubmenuMenuItemPrivate *priv = item->priv;
    switch (prop_id)
    {
        case PROP_LABEL:
            g_value_set_string (value, st_label_get_text (priv->label));
            break;
        case PROP_HIDE_EXPANDER:
            g_value_set_boolean (value, priv->hide_expander);
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
            break;
    }
}

static void 
cdos_popup_submenu_menu_item_class_init(CdosPopupSubmenuMenuItemClass *klass)
{
	GParamSpec 	*pspec;
	GObjectClass 	*gobject_class = G_OBJECT_CLASS(klass);
	gobject_class->set_property = set_property;
	gobject_class->get_property = get_property;

	g_type_class_add_private(klass, sizeof(CdosPopupSubmenuMenuItemPrivate));

	gobject_class->finalize = cdos_popup_submenu_menu_item_finalize;
	gobject_class->dispose = cdos_popup_submenu_menu_item_dispose;

    CdosPopupBaseMenuItemClass *base_class = CDOS_POPUP_BASE_MENU_ITEM_CLASS (klass);
    base_class->activate = cdos_popup_submenu_menu_item_activate;
    base_class->key_press_event = cdos_popup_submenu_menu_item_on_key_press_event;
    
    pspec = g_param_spec_string ("label",
                                 "menu label",
                                 "menu lable text",
                                 NULL,
                                 G_PARAM_READWRITE);
    g_object_class_install_property (gobject_class, PROP_LABEL, pspec);

    pspec = g_param_spec_boolean ("hide-expander",
                                  "Hide Expander",
                                  "Whether show triangle",
                                  FALSE,
                                  G_PARAM_READWRITE);
    g_object_class_install_property (gobject_class, PROP_HIDE_EXPANDER, pspec);

}

/**
 * cdos_popup_submenu_menu_item_new:
 * @text: 显示的text  
 * @hide_expander: ＃gboolean
 * 
 * 新建一个 #CdosPopupSubmenuMenuItem 类的实例。
 *
 * Returns: 返回一个 #CdosPopupSubmenuMenuItem 的实例。
 */
CdosPopupSubmenuMenuItem *
cdos_popup_submenu_menu_item_new(const gchar *text,
		 	 	 gboolean hide_expander)
{
	CdosPopupSubmenuMenuItem *object;
	object = CDOS_POPUP_SUBMENU_MENU_ITEM(g_object_new(CDOS_TYPE_POPUP_SUBMENU_MENU_ITEM, 
               			 	"label", text,
                			"hide-expander", hide_expander,
                			NULL));

	PopupBaseMenuItemInit       params;

	cdos_popup_base_menu_item_get_init_params(&params);

	cdos_popup_base_menu_item_set_params((CdosPopupBaseMenuItem *)object, &params);

	//cdos_popup_submenu_menu_item_set_text(object, text, hide_expander);

    	CdosPopupSubmenuMenuItemPrivate *priv = object->priv;
	CdosGenericContainer *actor = cdos_popup_base_menu_item_get_actor((CdosPopupBaseMenuItem *)object);
	object->menu = cdos_popup_sub_menu_new((ClutterActor *)actor, (ClutterActor *)priv->triangle);   

	g_signal_connect(object->menu, "open-state-changed",
		G_CALLBACK(cdos_popup_submenu_menu_item_open_state_changed), 
		object);

	return object;
}

static void
cdos_popup_submenu_menu_item_set_text(CdosPopupSubmenuMenuItem *item, 
				      const  gchar *text,
				      gboolean 	hide_expander) 
{

	CdosPopupSubmenuMenuItemPrivate      *priv;
	StTableChild 			     *child;
	//CdosGenericContainer 		     *actor;
	priv = item->priv;

	if (!hide_expander) {
		priv->triangle = g_object_new(ST_TYPE_ICON,
					"icon-name", "media-playback-start",
					"icon-type", ST_ICON_SYMBOLIC,
					"style-class", "popup-menu-icon",
					NULL);

		st_container_add_actor((ClutterContainer *)priv->table, (ClutterActor *)priv->triangle);

		child = (StTableChild *)clutter_container_get_child_meta(CLUTTER_CONTAINER(priv->table), (ClutterActor *)priv->triangle);

		child->row = 0;
		child->col = 0;
		g_object_set (child, "row", 0, "col", 0, NULL);
		child->col_span = 1;
		child->x_expand = FALSE;
		child->x_align = ST_ALIGN_START;
		clutter_actor_queue_relayout ((ClutterActor *)priv->triangle);

		priv->label = (StLabel *)st_label_new(text);
		st_container_add_actor((ClutterContainer *)priv->table, (ClutterActor *)priv->label);

		clutter_actor_set_margin_left(CLUTTER_ACTOR(priv->label), priv->margin_left_label_icon);

		child = (StTableChild *)clutter_container_get_child_meta(CLUTTER_CONTAINER(priv->table), (ClutterActor *)priv->label);
		
		//child->row = 0;
		//child->col = 1;
		g_object_set (child, "row", 0, "col", 1, NULL);
		child->col_span = 1;
		child->x_align = ST_ALIGN_START;
        
		clutter_actor_queue_relayout ((ClutterActor *)priv->label);
	} else {
		priv->label = (StLabel *)st_label_new(text);

		st_container_add_actor((ClutterContainer *)priv->table, (ClutterActor *)priv->label);

		clutter_actor_set_margin_left((ClutterActor *)priv->label, (priv->margin_left_label_icon));

		child = (StTableChild *)clutter_container_get_child_meta((ClutterContainer *)priv->table, (ClutterActor *)priv->label);
		
		child->row = 0;
		child->col = 0;
		g_object_set (child, "row", 0, "col", 0, NULL);
		child->col_span = 1;
		child->x_align = ST_ALIGN_START;
		clutter_actor_queue_relayout ((ClutterActor *)priv->label);
	}
}

static gboolean
cdos_popup_submenu_menu_item_on_key_press_event(ClutterActor *actor,
                    					        ClutterEvent *event,
                                                CdosPopupBaseMenuItem *item)
{
    CdosPopupSubmenuMenuItem *self = CDOS_POPUP_SUBMENU_MENU_ITEM (item);
	guint 		symbol;

	symbol = clutter_event_get_key_symbol(event);
	if(self->menu == NULL){
		return FALSE;
	}
	if (symbol == CLUTTER_KEY_Right) {
		cdos_popup_sub_menu_open(CDOS_POPUP_MENU_BASE(self->menu), TRUE);
		st_widget_navigate_focus(ST_WIDGET(self->menu->actor), NULL, GTK_DIR_DOWN, FALSE);
		return TRUE;
	} 
	else if (symbol == CLUTTER_KEY_Left && CDOS_POPUP_MENU_BASE(self->menu)->is_open) {
		cdos_popup_sub_menu_close(CDOS_POPUP_MENU_BASE(self->menu), FALSE);
		return TRUE;
	}
    return CDOS_POPUP_BASE_MENU_ITEM_CLASS (cdos_popup_submenu_menu_item_parent_class)->
        key_press_event (actor, event, item);
}

/**
 * cdos_popup_submenu_menu_item_get_menu:
 * @item: A #CdosPopupSubmenuMenuItem
 *
 * 获取item的menu成员
 *
 * Returns: #CdosPopupSubMenu 
 */
CdosPopupSubMenu *
cdos_popup_submenu_menu_item_get_menu(CdosPopupSubmenuMenuItem *item)
{
	g_return_val_if_fail(CDOS_IS_POPUP_SUBMENU_MENU_ITEM(item),NULL);
    return item->menu;
}
