#ifndef __CDOS_PROVIDER_H__
#define __CDOS_PROVIDER_H__

#include "st.h"
#include "cdos-extension-type.h"
#include "cdos-extension.h"

G_BEGIN_DECLS

#define CDOS_TYPE_PROVIDER  (cdos_provider_get_type())

#define CDOS_PROVIDER(obj)  \
        (G_TYPE_CHECK_INSTANCE_CAST((obj), CDOS_TYPE_PROVIDER, CdosProvider))

#define CDOS_IS_PROVIDER(obj) \
        (G_TYPE_CHECK_INSTANCE_TYPE((obj), CDOS_TYPE_PROVIDER))

#define CDOS_PROVIDER_CLASS(klass) \
        (G_TYPE_CHECK_CLASS_CAST((klass), CDOS_TYPE_PROVIDER, CdosProviderClass))

#define CDOS_IS_PROVIDER_CLASS(klass) \
        (G_TYPE_CHECK_CLASS_TYPE((klass), CDOS_TYPE_PROVIDER ))

#define CDOS_PROVIDER_GET_CLASS(obj) \
        (G_TYPE_INSTANCE_GET_CLASS((obj), CDOS_TYPE_PROVIDER, CdosProviderClass ))

typedef struct _CdosProvider CdosProvider;
typedef struct _CdosProviderPrivate CdosProviderPrivate;
typedef struct _CdosProviderClass CdosProviderClass;


typedef void (*func)();

typedef enum 
{
        IN =1,
        OUT,
        BIDIRECTIONAL
}BINDINGDIRECTION;


struct _CdosProvider
{
    GObject parent_instance;
        
    CdosProviderPrivate *priv;
};

struct _CdosProviderClass
{
    GObjectClass parent_class;
};

GType cdos_provider_get_type(void) G_GNUC_CONST;

CdosProvider *
cdos_provider_new_with_params (GObject *xlet, gchar *uuid, gchar *instancedId, CdosExtensionType* type, gchar *string);

CdosProvider *
cdos_provider_new(void);

void 
cdos_provider_set_params(CdosProvider *provider,
                         GObject  *xlet, 
                         gchar  *uuid, 
                         gchar  *instancedId, 
                         CdosExtensionType* type, 
                         gchar  *string);

gboolean
cdos_provider_bind_property(CdosProvider *provider,
                            BINDINGDIRECTION sync_type,
                            gchar *key_name,
                            gchar *applet_var,
                            func applet_callback,
                            gpointer user_data);

gboolean
cdos_provider_unbind_property(CdosProvider *provider, const gchar *key_name);

gchar *
cdos_provider_get_value(CdosProvider *provider, 
                        const gchar *key_name);

void 
cdos_provider_set_value(CdosProvider *provider, 
                        const gchar *key_name, 
                        const gchar *value);

void
cdos_provider_value_changed_notify(CdosProvider *provider, 
                                   const gchar *key, 
                                   const gchar* oldval, 
                                   const gchar *newval);

void
cdos_provider_setting_file_changed_notify(CdosProvider *provider);

gchar* cdos_provider_get_string(CdosProvider *provider,
                                gchar *group_name,
                                gchar *key);

gboolean cdos_provider_get_boolean(CdosProvider *provider,
                                   gchar *group_name,
                                   gchar *key);


gint cdos_provider_get_int(CdosProvider *provider,
                           gchar *group_name,
                           gchar *key);

gchar** cdos_provider_get_array(CdosProvider *provider,
                                gchar *group_name,
                                gchar *key,
                                gsize length);

void cdos_provider_set_string(CdosProvider *provider,
                              gchar *group_name,
                              gchar *key,
                              gchar *value);

void cdos_provider_set_boolean(CdosProvider *provider,
                               gchar *group_name,
                               gchar *key,
                               gboolean value);

void cdos_provider_set_int(CdosProvider *provider,
                           gchar *group_name,
                           gchar *key,
                           gint  value);

void cdos_provider_set_array(CdosProvider *provider,
                             gchar *group_name,
                             gchar *key,
                             GList *value);

G_END_DECLS

#endif /*__CDOS_PROVIDER_H__*/
