#include "cdos-setting.h"
#include "cdos-provider.h"

struct _CdosSettingPrivate
{
        BINDINGDIRECTION       sync_type;
        GHashTable                   *obj;
        gchar                             *key_name;
        CdosSettingObj           *settings_obj;
        gchar                             *applet_var;
        func                               cb;
        guint                             settings_obj_connection_id;
        gpointer                       user_data;
};


enum
{
    PROP_0,

    PROP_APPLET_VAR,
};

#define CDOS_SETTING_GET_PRIVATE(obj)    \
        (G_TYPE_INSTANCE_GET_PRIVATE((obj), CDOS_TYPE_SETTING, CdosSettingPrivate))

G_DEFINE_TYPE(CdosSetting, cdos_setting, G_TYPE_OBJECT);

static void
cdos_setting_file_changed(CdosSetting *setting);

static void
cdos_setting_monitor_applet_var(CdosSetting *setting, gboolean state);

static  void
cdos_setting_class_init(CdosSettingClass *klass);

static void 
cdos_setting_init(CdosSetting  *setting);


static void
cdos_setting_dispose(GObject *gobject);

static void
cdos_setting_finalize(GObject *gobject);

static void 
cdos_setting_get_property(GObject    *gobject,
                guint   prop_id,
                GValue   *value,
                GParamSpec   *pspec);

static void
cdos_setting_set_property(GObject  *gobject,
                guint   prop_id,
                const GValue *value,    
                GParamSpec *pspec);
static void
cdos_setting_set_applet_var(CdosSetting *setting, gchar *applet_var);

static void
cdos_setting_dispose(GObject *gobject)
{
    CdosSettingPrivate *priv = CDOS_SETTING(gobject)->priv;

    if (priv->obj){
        g_hash_table_unref(priv->obj);
    }
    if (priv->settings_obj) {
        g_object_unref(priv->settings_obj);
    }
    G_OBJECT_CLASS(cdos_setting_parent_class)->dispose(gobject);
}

static void
cdos_setting_finalize(GObject *gobject)
{
    CdosSettingPrivate *priv = CDOS_SETTING(gobject)->priv;
    if (priv->key_name) {
        g_free(priv->key_name);
    }
    if (priv->applet_var) {
        g_free(priv->applet_var);
    }
    G_OBJECT_CLASS(cdos_setting_parent_class)->finalize(gobject);
}

static void 
cdos_setting_init(CdosSetting  *setting)
{
        CdosSettingPrivate  *priv;
        priv = setting->priv = CDOS_SETTING_GET_PRIVATE(setting);

        priv->key_name = NULL;
        priv->settings_obj = NULL;
        priv->sync_type = 0;
        priv->applet_var = NULL;
        priv->obj = NULL;
        priv->cb = NULL;
        priv->settings_obj_connection_id = 0;
        priv->user_data = NULL;
}

static  void
cdos_setting_class_init(CdosSettingClass *klass)
{
        GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
        GParamSpec *pspec;

        g_type_class_add_private(klass, sizeof(CdosSettingPrivate));

        gobject_class->dispose = cdos_setting_dispose;
        gobject_class->finalize = cdos_setting_finalize;
        gobject_class->set_property = cdos_setting_set_property;
        gobject_class->get_property = cdos_setting_get_property;

        pspec = g_param_spec_string("applet_var",
                                            "Applet_var",
                                            "the string of applet var",
                                            NULL,
                                            G_PARAM_READWRITE);

        g_object_class_install_property(gobject_class, PROP_APPLET_VAR, pspec);

}

static void
cdos_setting_set_property(GObject  *gobject,
                guint   prop_id,
                const GValue *value,    
                GParamSpec *pspec)
{
    CdosSetting *setting = CDOS_SETTING(gobject);

    switch(prop_id) 
    {
        case PROP_APPLET_VAR:
            cdos_setting_set_applet_var(setting, g_value_get_object(value));
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
    }
}

static void 
cdos_setting_get_property(GObject    *gobject,
                guint   prop_id,
                GValue   *value,
                GParamSpec   *pspec)
{
    CdosSettingPrivate *priv = CDOS_SETTING(gobject)->priv;
    switch(prop_id) 
    {
        case PROP_APPLET_VAR:
            g_value_set_string(value, priv->applet_var);
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
    }
}

static void
cdos_setting_set_applet_var(CdosSetting *setting, gchar *applet_var)
{
    g_return_if_fail(CDOS_IS_SETTING(setting));
    if (setting->priv->applet_var) {
        g_free(setting->priv->applet_var);
    }
    setting->priv->applet_var = g_strdup(applet_var);
    g_object_notify(G_OBJECT(setting), "applet_var");
}

gchar *
cdos_setting_get_applet_var(CdosSetting *setting)
{
    g_return_val_if_fail(CDOS_IS_SETTING(setting),NULL);
    return setting->priv->applet_var;
}

/**
 * cdos_setting_new_with_params
 * @sync_type: 同步类型。三种类型：IN、OUT、BIDIRECTIONAL
 * @obj: 一个#GObject对象
 * @key_name: 键
 * @setting_obj: 一个#CdosSettingObj对象
 * @applet_var: 应用的名称
 * @applet_callback: 应用的回调函数
 * @user_data: 传递给回调函数的参数
 *
 * 根据传入的参数创建#CdosSetting对象，并设置相应属性
 *
 * Returns: 一个#CdosSetting对象
 */

CdosSetting *
cdos_setting_new_with_params(BINDINGDIRECTION sync_type,
                                                GHashTable  *obj,
                                                const gchar *key_name,
                                                CdosSettingObj *setting_obj,
                                                gchar *applet_var,
                                                func     applet_callback,
                                                gpointer  user_data)
{

    g_return_val_if_fail(CDOS_IS_SETTING_OBJ(setting_obj),NULL);

    CdosSetting *gobject;
    gobject = g_object_new(CDOS_TYPE_SETTING, NULL);
    cdos_setting_set_params(  CDOS_SETTING(gobject),
                                sync_type,
                                obj,
                                key_name,
                                setting_obj,
                                applet_var,
                                applet_callback,
                                user_data);
    return gobject;
}

CdosSetting *
cdos_setting_set_params( CdosSetting *setting,    
                                        BINDINGDIRECTION sync_type,
                                        GHashTable  *obj,
                                        const gchar *key_name,
                                        CdosSettingObj *setting_obj,
                                        gchar *applet_var,
                                        func     applet_callback,
                                        gpointer  user_data)
{
    g_return_val_if_fail(CDOS_IS_SETTING(setting),NULL);
    g_return_val_if_fail(CDOS_IS_SETTING_OBJ(setting_obj),NULL);

    CdosSettingPrivate *priv;
    priv = setting->priv;

    priv->key_name = g_strdup(key_name);
    priv->settings_obj = g_object_ref(setting_obj);

    priv->sync_type = sync_type;
    g_hash_table_insert(obj, applet_var,  cdos_setting_obj_get_data(priv->settings_obj, priv->key_name, "value"));
    priv->obj = g_hash_table_ref(obj);
    priv->cb = applet_callback;

    if (user_data) {
        priv->user_data = user_data;
    }

    if (priv->sync_type != OUT) {
        priv->settings_obj_connection_id = g_signal_connect(priv->settings_obj, "setting-file-changed", G_CALLBACK(cdos_setting_file_changed), setting);
    }
    cdos_setting_monitor_applet_var(setting, TRUE);
    return  setting;
}

static void
cdos_setting_monitor_applet_var(CdosSetting *setting, gboolean state)
{
    CdosSettingPrivate *priv = setting->priv;

    if (priv->sync_type < OUT)
            return;
    // pass
}


static void
cdos_setting_file_changed(CdosSetting *setting)
{
    if (setting->priv->sync_type != OUT) {
        gchar *value = cdos_setting_get_value(setting);
        cdos_setting_set_applet_var_and_cb(setting, value);
    }
}

/**
 * cdos_setting_get_data
 * @setting: 一个#CdosSetting对象
 * @value: 需要获取的键
 *
 * 获取value所对应的字符串，该方法调用cdos_setting_obj_get_data。
 *
 * Returns: value所对应的字符串;如果没有找到，则返回NULL。
 */

gchar *
cdos_setting_get_data(CdosSetting *setting,
                                        const gchar *value)
{
    g_return_val_if_fail(CDOS_IS_SETTING(setting),NULL);
    return cdos_setting_obj_get_data(setting->priv->settings_obj, setting->priv->key_name, value);
}

/**
 * cdos_setting_get_value
 * @setting: 一个#CdosSetting对象
 *
 * 获取键"value"所对应的字符串，该方法调用cdos_setting_obj_get_data。
 *
 * Returns: 键"value"所对应的字符串;如果没有找到，则返回NULL。
 */

gchar *
cdos_setting_get_value(CdosSetting *setting)
{
    g_return_val_if_fail(CDOS_IS_SETTING(setting),NULL);
        
    return cdos_setting_obj_get_data(setting->priv->settings_obj, setting->priv->key_name, "value");
}

/**
 * cdos_setting_set_val
 * @setting: 一个#CdosSetting对象
 * @value: 一个新的字符串
 *
 * 设置键"value"所对应的字符串，该方法调用cdos_setting_obj_set_value。
 *
 */

void
cdos_setting_set_val(CdosSetting *setting, const gchar *value)
{
        g_return_if_fail(CDOS_IS_SETTING(setting));

        cdos_setting_obj_set_value(setting->priv->settings_obj, setting->priv->key_name, value);
}

/**
 * cdos_setting_set_applet_var_and_cb
 * @setting: 一个#CdosSetting对象
 * @new_val: 一个新的应用名称
 *
 * 设置应用名称和回调函数。
 *
 */

void
cdos_setting_set_applet_var_and_cb(CdosSetting *setting, gchar *new_val)
{
    g_return_if_fail(CDOS_IS_SETTING(setting));
    CdosSettingPrivate *priv = setting->priv;

    gchar *val = g_hash_table_lookup(priv->obj, priv->applet_var);
    if (g_strcmp0(new_val, val)) {
        cdos_setting_monitor_applet_var(setting, FALSE);
        g_hash_table_insert(priv->obj, priv->applet_var, new_val);
        if (priv->user_data) {
                priv->cb(priv->user_data);
        } else {
                priv->cb(NULL);
        }
        cdos_setting_monitor_applet_var(setting, TRUE);
    }
}



