#include "glib.h"
#include "cdos-settings-manager.h"

#define CDOS_SETTINGS_MANAGER_GET_PRIVATE(obj)  \
        (G_TYPE_INSTANCE_GET_PRIVATE((obj), CDOS_TYPE_SETTINGS_MANAGER, CdosSettingsManagerPrivate))
 
G_DEFINE_TYPE(CdosSettingsManager, cdos_settings_manager, CDOS_TYPE_SETTINGS_MANAGER)

struct _CdosSettingsManagerPrivate
{
        GList *list;
};

static CdosSettingsManager *settingManager = NULL;

static void
cdos_settings_manager_finalize(GObject *gobject);

static void
cdos_settings_manager_dispose(GObject *gobject);

static void
cdos_settings_manager_class_init(CdosSettingsManagerClass *klass);

static void
cdos_settings_manager_init(CdosSettingsManager *manager);

static void
cdos_settings_manager_finalize(GObject *gobject)
{
    CdosSettingsManagerPrivate *priv = CDOS_SETTINGS_MANAGER(gobject)->priv;

    g_list_free(priv->list);

    G_OBJECT_CLASS(cdos_settings_manager_parent_class)->finalize(gobject);
}

static void
cdos_settings_manager_dispose(GObject *gobject)
{
    CdosSettingsManagerPrivate *priv = CDOS_SETTINGS_MANAGER(gobject)->priv;
    GList *list = priv->list;

    if(list){
        while (list) {
            Uuids *struct_u = (Uuids *)list->data;
            if (struct_u != NULL) {
                g_object_unref(struct_u->obj);
                g_free(struct_u->uuid);
                g_free(struct_u->instance_id);
                g_free(struct_u);
            }
            list->data = NULL;

            list = list->next;
        }
    }
    G_OBJECT_CLASS(cdos_settings_manager_parent_class)->dispose(gobject);
}

static void
cdos_settings_manager_class_init(CdosSettingsManagerClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS(klass);

    g_type_class_add_private(klass, sizeof(CdosSettingsManagerPrivate));
    gobject_class->finalize = cdos_settings_manager_finalize;
    gobject_class->dispose = cdos_settings_manager_dispose;

}

static void
cdos_settings_manager_init(CdosSettingsManager *manager)
{
        manager->priv = CDOS_SETTINGS_MANAGER_GET_PRIVATE(manager);

        manager->priv->list = NULL;
}

/**
 * cdos_settings_manager_register:
 * @manager: 一个#CdosSettingsManager对象
 * @m_uuid: 通用唯一识别码
 * @m_instance_id: 应用实例ID
 * @m_obj: 一个#GObject对象
 *
 * 注册应用程序
 *
 */

void
cdos_settings_manager_register(CdosSettingsManager *manager,
                                                        const gchar *m_uuid, 
                                                        const gchar *m_instance_id, 
                                                        GObject *m_obj)
{
    g_return_if_fail(CDOS_IS_SETTINGS_MANAGER(manager));
    if (NULL == m_uuid|| NULL == m_instance_id || NULL == m_obj)
        return;

    CdosSettingsManagerPrivate  *priv = manager->priv;
    GList *list = priv->list;
    Uuids  *struct_u;

    while(list) {
        struct_u = (Uuids*)list->data;
        if ((!g_strcmp0(m_uuid, struct_u->uuid)) && (!g_strcmp0(m_instance_id, struct_u->instance_id))){
            if (struct_u->obj)
                g_object_unref(struct_u->obj);
            g_object_ref(m_obj);
            struct_u->obj = m_obj;
            return;
        }
        list = list->next;
    }
    Uuids  *uuids = (Uuids*)g_new0(Uuids,1);
    uuids->uuid = g_strdup(m_uuid);
    uuids->instance_id = g_strdup(m_instance_id);
    uuids->obj = g_object_ref(m_obj);

    priv->list =g_list_append(priv->list, uuids);
}

/**
 * cdos_settings_manager_register:
 * @manager: 一个#CdosSettingsManager对象
 * @uuid: 通用唯一识别码
 * @instance_id: 应用实例ID
 *
 * 注销应用程序
 *
 */

void 
cdos_settings_manager_unregister(CdosSettingsManager *manager,
                                                        const gchar *uuid, 
                                                        const gchar *instance_id) {
    g_return_if_fail(CDOS_IS_SETTINGS_MANAGER(manager));
    if (NULL == uuid || NULL == instance_id)
        return;
    CdosSettingsManagerPrivate *priv = manager->priv;
    GList *list = priv->list;
    Uuids *struct_u = NULL;
    Uuids *target = NULL;

    while (list) {
        struct_u = (Uuids *) list->data;
        if ((!g_strcmp0(uuid, struct_u->uuid)) && (!g_strcmp0(instance_id, struct_u->instance_id))) {
            target = struct_u;
            g_object_unref(struct_u->obj);
            g_free(struct_u->uuid);
            g_free(struct_u->instance_id);
            break ;
        }
        list = list->next;
    }
    priv->list = g_list_remove(priv->list,target);
    g_free(target);

}

/**
 * cdos_settings_manager_get_default:
 *
 * 获取#CdosSettingsManager对象
 *
 * Returns: #GObject对象
 */

CdosSettingsManager *
cdos_settings_manager_get_default(void)
{
    if (settingManager == NULL) {
        g_print("\n%s:settingManager is NULL!",__func__);
    }
    return settingManager;
}

/**
 * cdos_settings_manager_new:
 *
 * 创建#CdosSettingsManager对象
 *
 * Returns: 一个#CdosSettingsManager对象
 */

CdosSettingsManager *
cdos_settings_manager_new (gboolean is_default)
{
    CdosSettingsManager *manager;
    manager = g_object_new(CDOS_TYPE_SETTINGS_MANAGER, NULL);
    if(is_default == TRUE){
        settingManager = manager;
    }
    return manager;
}

/*
 * cdos_settings_manager_set_default:
 * @manager:实例对象
 *
 * 设置默认的SettingsManager。
 **/
void
cdos_settings_manager_set_default(CdosSettingsManager *manager){
    g_return_if_fail(CDOS_IS_SETTINGS_MANAGER(manager));
    settingManager = manager;
}
