#include <girepository.h>
#include <clutter/clutter.h>
#include <gio/gio.h>
#include "cdos-icon-applet.h"
#include "cdos-showdesktop.h"
#include "st.h"
#include "cdos-debug.h"

/**
 * SECTION:cdos-showdesktop
 * @title: CdosShowdesktop
 * @short_description: 显示桌面
 *
 * CdosShowdesktop 在桌面上有窗口显示时,点击显示桌面,桌面就可以把窗口隐藏,只显示桌面
 */
G_DEFINE_TYPE(CdosShowdesktop, cdos_showdesktop, CDOS_TYPE_ICONAPPLET);

#define CDOS_SHOWDESKTOP_GET_PRIVATE(obj) \
        (G_TYPE_INSTANCE_GET_PRIVATE((obj), CDOS_TYPE_SHOWDESKTOP, CdosShowdesktopPrivate))

struct _CdosShowdesktopPrivate
{
     StBin           *applet_icon_box;
//     StBoxLayout     *actor;
     gboolean   show;
}; 

static void
cdos_showdesktop_finalize(GObject *object)
{
    //CdosShowdesktopPrivate *priv = CDOS_SHOWDESKTOP(object)->priv;


    G_OBJECT_CLASS(cdos_showdesktop_parent_class)->finalize(object);
}

static void
cdos_showdesktop_dispose(GObject *object)
{
    CdosShowdesktopPrivate *priv = CDOS_SHOWDESKTOP(object)->priv;
    g_object_unref(priv->applet_icon_box);
    G_OBJECT_CLASS(cdos_showdesktop_parent_class)->dispose(object);
}

static gboolean 
cdos_showdesktop_on_applet_clicked(ClutterActor *actor, ClutterEvent *event, CdosApplet *self)
{

    CdosShowdesktop *me = CDOS_SHOWDESKTOP (self);

    CdosGlobal *global = cdos_global_get ();
    MetaScreen *screen = cdos_global_get_screen (global);
    guint32 time = cdos_global_get_current_time (global);
    gboolean is_show_desktop = meta_screen_get_showing_desktop(screen);
    if (!is_show_desktop){
        meta_screen_show_desktop (screen, time);
        me->priv->show = TRUE;
    } else {
        meta_screen_unshow_desktop (screen);
        me->priv->show = FALSE;
    }

    return FALSE;
}

static void 
on_showdesktop_button_press_cb (StBoxLayout *actor,  ClutterEvent *event, gpointer data){
    
    CdosShowdesktop *obj = (CdosShowdesktop *)data;
    CDOS_APPLET_CLASS(G_OBJECT_GET_CLASS(obj))->on_applet_clicked((CdosApplet *)obj);
}


static void 
cdos_showdesktop_class_init( CdosShowdesktopClass *kclass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (kclass);
    CdosAppletClass *parent_class = CDOS_APPLET_CLASS (kclass);

    object_class->finalize = cdos_showdesktop_finalize;
    object_class->dispose = cdos_showdesktop_dispose;

    //parent_class->on_applet_clicked = cdos_showdesktop_on_applet_clicked;

    g_type_class_add_private (kclass, sizeof (CdosShowdesktopPrivate));

}

static void
cdos_showdesktop_init(CdosShowdesktop *self)
{
    //CdosShowdesktopPrivate *priv;
    self->priv = CDOS_SHOWDESKTOP_GET_PRIVATE(self);
    self->priv->show = FALSE;
    cdos_applet_set_popup_menu_active((CdosApplet *)self,FALSE);
}


static CdosShowdesktop *
cdos_showdesktop_new(guint orientation, guint panel_height)
{
    CdosShowdesktop *showdesktop;
    showdesktop = g_object_new(CDOS_TYPE_SHOWDESKTOP, "orientation", orientation, NULL);  //创建我们自定义的类对象

    CdosShowdesktopPrivate *priv = showdesktop->priv;

    st_widget_remove_style_class_name (ST_WIDGET(CDOS_APPLET(showdesktop)->actor),"applet-box");
    st_widget_set_style_class_name (ST_WIDGET(CDOS_APPLET(showdesktop)->actor),"show-desktop");

    priv->applet_icon_box = cdos_iconapplet_get_applet_icon_box (CDOS_ICONAPPLET (showdesktop));

    //clutter_actor_set_size (CLUTTER_ACTOR (priv->applet_icon_box), 18, panel_height);
//    cdos_iconapplet_set_icon_name (CDOS_ICONAPPLET (showdesktop), "show-desktop");
    cdos_applet_set_applet_tooltip((CdosShowdesktop *)showdesktop, _("show-desktop"));
    st_widget_set_style_class_name ((StWidget *)priv->applet_icon_box,"show-desktop-box");
    clutter_actor_set_reactive(CLUTTER_ACTOR(priv->applet_icon_box),TRUE);
    st_widget_set_track_hover((StWidget *)priv->applet_icon_box,TRUE);

    g_signal_connect (CDOS_APPLET(showdesktop)->actor, "button-press-event",
                    G_CALLBACK(cdos_showdesktop_on_applet_clicked),
                    CDOS_APPLET (showdesktop));
    //set_applet_tooltip(cdos,"Show desktop");

    return showdesktop;
}   

/**
 * cdos_showdesktop_main:
 * @meta: a #MetaData
 * @orientation: 桌面显示的方向
 * @panel_height: panel的高度
 *
 * 新建一个showdesktop,并实现显示桌面的功能
 *
 * Returns: showdesktop
 */
CdosApplet * 
cdos_showdesktop_main(MetaData *meta,guint orientation, guint panel_height)
{
    static CdosShowdesktop *showdesktop = NULL;

    if(showdesktop == NULL){
        showdesktop = cdos_showdesktop_new (orientation, panel_height);
    }
    return CDOS_APPLET (showdesktop);
}
