/**
 * SECTION: cdos-source
 * @short_description: 创建,处理 CdosSource
 * @include: cdos-source.h
 * 
 * CdostypeClass 类用来创建,处理 #CdosSource
 */
#include "cdos-source.h"
#include "cdos-notification.h"


G_DEFINE_TYPE (CdosSource, cdos_source, G_TYPE_OBJECT);


/* signal */
enum {
    TITLE_CHANGED,       //title-changed
    NOTIFICATION_ADDED,      //notification-added
    CDOS_SOURCE_NOTIFY,         //cdos-source-notify
    DESTROY,        //destroy
    OPEN,            //open
    LAST_SIGNAL
};


static guint cdos_source_signals[LAST_SIGNAL] = {0};

//declare
static void lastNotificationRemoved (CdosSource *self);

static void getPreferredWidth (ClutterActor *actor, gfloat for_height, CdosGenericContainerAllocation *alloc,
                               gpointer user_data) {

    CdosSource *self = CDOS_SOURCE (user_data);

    gfloat min, nat;

    clutter_actor_get_preferred_width (CLUTTER_ACTOR (self->iconBin),
                                       for_height,
                                       &min,
                                       &nat);

    alloc->min_size = min;
    alloc->natural_size = nat;
}


static void getPreferredHeight (ClutterActor *actor, gfloat for_width, CdosGenericContainerAllocation *alloc,
                                gpointer user_data) {

    CdosSource *self = CDOS_SOURCE (user_data);

    gfloat min, nat;

    clutter_actor_get_preferred_width (CLUTTER_ACTOR (self->iconBin),
                                       for_width,
                                       &min,
                                       &nat);

    alloc->min_size = min;
    alloc->natural_size = nat;
}


static void allocate (ClutterActor *actor, ClutterActorBox *box, ClutterAllocationFlags flags, gpointer user_data) {

    CdosSource *self = CDOS_SOURCE (user_data);

    ClutterActorBox *childBox;
    gfloat minWidth, minHeight, naturalWidth, naturalHeight;
    ClutterTextDirection direction;

    // the iconBin should fill our entire box
    //this._iconBin.allocate(box, flags);
    clutter_actor_allocate (CLUTTER_ACTOR (self->iconBin), box, flags);

    //let childBox = new Clutter.ActorBox();
    childBox = clutter_actor_box_alloc ();

    //let [minWidth, minHeight, naturalWidth, naturalHeight] = this._counterBin.get_preferred_size();
    clutter_actor_get_preferred_size (CLUTTER_ACTOR (self->counterBin),
                                      &minWidth,
                                      &minHeight,
                                      &naturalWidth,
                                      &naturalHeight);


    //let direction = this.actor.get_direction();
    direction = clutter_actor_get_text_direction (actor);

    //if (direction == St.TextDirection.LTR) {
    if (direction == ST_TEXT_DIRECTION_LTR) {
        // allocate on the right in LTR
        childBox->x1 = box->x2 - naturalWidth;
        childBox->x2 = box->x2;
    }
    else {
        // allocate on the left in RTL
        childBox->x1 = 0;
        childBox->x2 = naturalWidth;
    }

    childBox->y1 = box->y2 - naturalHeight;
    childBox->y2 = box->y2;

    //this._counterBin.allocate(childBox, flags);
    clutter_actor_allocate (CLUTTER_ACTOR (self->counterBin), box, flags);

}


static void onDestroy (GObject *gobject, gpointer user_data) {

    CdosSource *self = CDOS_SOURCE (user_data);

    self->actorDestroyed = TRUE;
}


static void
cdos_source_finalize (GObject *obj)
{
    CdosSource *self = CDOS_SOURCE (obj);

    g_free (self->actor);
    g_free (self->counterBin);
    g_free (self->counterLabel);
    g_free (self->iconBin);
    g_list_free (self->notifications);

    G_OBJECT_CLASS(cdos_source_parent_class)->finalize(obj);
}

static void
cdos_source_class_init (CdosSourceClass *klass) {

    GObjectClass *gobject_class = G_OBJECT_CLASS(klass);

    /* singal */
    /**
     * CdosSource::title-changed:
     * @object: 绑定信号的对象
     * @user_data: 真对回调函数所传入的参数
     *
     * 该信号对#CdosSource 的标题变化进行监听
     */
     cdos_source_signals[TITLE_CHANGED] =
     g_signal_new ("title-changed",
                   G_OBJECT_CLASS_TYPE (klass),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (CdosSourceClass, title_changed),
                   NULL, NULL,
                   g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);

    /**
     * CdosSource::notification-added:
     * @object: 绑定信号的对象
     * @user_data: 真对回调函数所传入的参数
     *
     * 该信号对#CdosSource 的添加notification进行监听
     */
     cdos_source_signals[NOTIFICATION_ADDED] =
     g_signal_new ("notification-added",
                   G_OBJECT_CLASS_TYPE (klass),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (CdosSourceClass, notification_added),
                   NULL, NULL,
                   g_cclosure_marshal_VOID__OBJECT,
                   G_TYPE_NONE, 1,
                   CDOS_TYPE_NOTIFICATION);

    /**
     * CdosSource::cdos-source-notify:
     * @object: 绑定信号的对象
     * @user_data: 真对回调函数所传入的参数
     *
     * 该信号对#CdosSource 的发送通知进行监听
     */
     cdos_source_signals[CDOS_SOURCE_NOTIFY] =
     g_signal_new ("cdos-source-notify",
                   G_OBJECT_CLASS_TYPE (klass),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (CdosSourceClass, cdos_source_notify_signal),
                   NULL, NULL,
                   g_cclosure_marshal_VOID__OBJECT,
                   G_TYPE_NONE, 1,
                   CDOS_TYPE_NOTIFICATION);

    /**
     * CdosSource::destroy:
     * @object: 绑定信号的对象
     * @user_data: 真对回调函数所传入的参数
     *
     * 该信号对#CdosSource 的销毁通知进行监听
     */
     cdos_source_signals[DESTROY] =
     g_signal_new ("destroy",
                   G_OBJECT_CLASS_TYPE (klass),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (CdosSourceClass, destroy),
                   NULL, NULL,
                   g_cclosure_marshal_VOID__UINT,
                   G_TYPE_NONE, 1,
                   G_TYPE_UINT);

    /**
     * CdosSource::notification-open:
     * @object: 绑定信号的对象
     * @user_data: 真对回调函数所传入的参数
     *
     * 该信号对#CdosSource 的打开通知进行监听
     */
     cdos_source_signals[OPEN] =
     g_signal_new ("notification-open",
                   G_OBJECT_CLASS_TYPE (klass),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (CdosSourceClass, notification_open),
                   NULL, NULL,
                   g_cclosure_marshal_VOID__OBJECT,
                   G_TYPE_NONE, 1,
                   G_TYPE_OBJECT);

    /* override */
     klass->handle_summary_click = cdos_source_handle_summary_click;
     klass->set_summary_icon = cdos_source_set_summary_icon;
     klass->open = cdos_source_open;
     klass->remove_last = lastNotificationRemoved;

     gobject_class->finalize = cdos_source_finalize;
 }

static void
cdos_source_init (CdosSource *self) {

    //this.actor = new Cdos.GenericContainer();
    //self->actor = g_object_new (CDOS_TYPE_GENERIC_CONTAINER, NULL);
    self->actor = ST_BOX_LAYOUT (st_box_layout_new ());
    //g_signal_connect (self->actor, "get-preferred-width", G_CALLBACK (getPreferredWidth), self);
    //g_signal_connect (self->actor, "get-preferred-height", G_CALLBACK (getPreferredHeight), self);
    //g_signal_connect (self->actor, "allocate", G_CALLBACK (allocate), self);
    g_signal_connect (self->actor, "destroy", G_CALLBACK (onDestroy), self);

    self->actorDestroyed = FALSE;

    self->counterLabel = ST_LABEL(st_label_new ("default"));

    self->counterBin = ST_BIN(st_bin_new ());
    st_widget_add_style_class_name (ST_WIDGET (self->counterBin), "summary-source-counter");
    st_bin_set_child (self->counterBin, CLUTTER_ACTOR (self->counterLabel));

    clutter_actor_hide (CLUTTER_ACTOR (self->counterBin));

    self->iconBin = ST_BIN (st_bin_new ());
    st_bin_set_fill (self->iconBin, TRUE, TRUE);

    //clutter_actor_add_child (CLUTTER_ACTOR (self->actor), CLUTTER_ACTOR (self->iconBin));
    //clutter_actor_add_child (CLUTTER_ACTOR (self->actor), CLUTTER_ACTOR (self->counterBin));
    st_box_layout_insert_actor (self->actor, CLUTTER_ACTOR (self->iconBin), -1);
    st_box_layout_insert_actor (self->actor, CLUTTER_ACTOR (self->counterBin), -1);

    self->isTransient = FALSE;
    self->isChat = FALSE;

    self->notifications = NULL;
}

static void setCount (CdosSource *self, guint count, gboolean visible) {
    gchar *str = NULL;

    if (self->actorDestroyed)
        return;

    g_object_set (CLUTTER_ACTOR (self->counterBin), "visible", visible, NULL);

    str = g_strdup_printf("%d", count);
    st_label_set_text (self->counterLabel, str);
}


static void updateCount (CdosSource *self) {

    guint count = g_list_length (self->notifications);

    setCount (self, count, (count > 1));

}


static void notificationDestroy (CdosNotification *notification, guint reason, gpointer user_data) {

    CdosSource *self = CDOS_SOURCE (user_data);

    gint index = g_list_index (self->notifications, notification);

    if (index < 0) {
        return ;
    }

    self->notifications = g_list_remove (self->notifications, notification);

    if (g_list_length (self->notifications) == 0) {
        lastNotificationRemoved (self);
    }

    updateCount (self);
}


/*公有函数定义*/

/**
 * cdos_source_set_transient:
 * @self: A #CdosSource
 * @isTransient: 设置self是否是暂时的 
 *
 * 设置 #CdosSource 实例是否时暂时的.
 *
 */
void cdos_source_set_transient (CdosSource *self, gboolean isTransient) {

    self->isTransient = isTransient;
}

/**
 * cdos_source_set_title:
 * @self: A #CdosSource
 * @newTitle: 设置新标题 
 *
 * 设置 #CdosSource 实例的新标题
 *
 */
void cdos_source_set_title (CdosSource *self, gchar *newTitle) {
    self->title = newTitle;
    g_signal_emit_by_name (self, "title-changed");
}

/**
 * cdos_source_create_notification:
 * @self: A #CdosSource
 *
 * Called to create a new icon actor (of size this.ICON_SIZE).
 * Must be overridden by the subclass if you do not pass icons
 * explicitly to the cdos_notification_new () constructor.
 *
 * js实现里是为了处理抛出的异常的, C中不使用.
 */
void cdos_source_create_notification (CdosSource *self) {

    //throw new Error('no implementation of createNotificationIcon in ' + this);
}

/**
 * cdos_source_get_summary_icon:
 * @self: A #CdosSource
 *
 * Unlike cdos_source_create_notification, this always returns the same actor;
 * there is only one summary icon actor for a Source.
 * 
 * Returns: 返回self的成员变量actor
 */
StBoxLayout *cdos_source_get_summary_icon (CdosSource *self) {
    return self->actor;
}

/**
 * cdos_source_push_notification:
 * @self: A #CdosSource
 * @notificatin: 待加入链表的节点 
 *
 * 判断notification是否在self->notifications里,若不存在就加入链表;
 * 为notification绑定两个信号: clicked和destroy.
 *
 */
void cdos_source_push_notification (CdosSource *self, CdosNotification *notification) {
    if (g_list_index (self->notifications, notification) < 0) {
        self->notifications = g_list_append (self->notifications, notification);
        g_signal_emit_by_name (self, "notification-added", notification);
    }

    g_signal_connect (notification, "clicked", G_CALLBACK (cdos_source_open), self);
    g_signal_connect (notification, "destroy", G_CALLBACK (notificationDestroy), self);

    updateCount (self);
}

/**
 * cdos_source_notify:
 * @self: A #CdosSource
 * @notification: 待发送的data 
 *
 * 发送notification
 *
 */
void cdos_source_notify (CdosSource *self, CdosNotification *notification) {
    cdos_source_push_notification (self, notification);
    g_signal_emit_by_name (self, "cdos-source-notify", notification);
}

/**
 * cdos_source_destroy:
 * @self: A #CdosSource
 * @reason: 删除的原因 
 *
 * 发送 #CdosSource 的 "destroy" 信号
 *
 */
void cdos_source_destroy (CdosSource *self, guint reason) {
    g_signal_emit_by_name (self, "destroy", reason);
}

/**
 * cdos_source_handle_summary_click:
 *
 * A subclass can redefine this to "steal" clicks from the
 * summaryitem; Use Clutter.get_current_event() to get the
 * details, return true to prevent the default handling from
 * ocurring.可以被子类重写
 *
 * Returns: %FALSE,不阻止默认的处理.
 */
gboolean cdos_source_handle_summary_click (void) {
    return FALSE;
}

/**
 * cdos_source_open:
 * @obj: A #GObject
 * @user_data: 传送的数据 
 *
 * 默认什么都不做,但是子类可以重写
 *
 */ 
void cdos_source_open (GObject *obj,  gpointer user_data) {
   return ;    
}

/**
 * cdos_source_destroy_non_resident_notifications:
 * @self: A #CdosSource
 *
 * 销毁resident属性為false的notification
 */
void cdos_source_destroy_non_resident_notifications (CdosSource *self) {

    guint i = g_list_length (self->notifications);
    gint j = i - 1;

    /*
     *   for (let i = this.notifications.length - 1; i >= 0; i--)
     *       if (!this.notifications[i].resident)
     *           this.notifications[i].destroy();
     */
    for (; j >= 0; j--) {
        CdosNotification *notification = g_list_nth_data (self->notifications, j);
        if (!notification->resident) {
            cdos_notification_destroy (notification, NOREASON); //reason
        }
    }

    updateCount (self);
}

/**
 * cdos_source_new:
 * @title: 设置实例的标题 
 * 
 * 新建一个 #CdosSource 类的实例。
 *
 * Returns: 返回一个 #CdosSource 的实例。
 */
CdosSource *cdos_source_new (gchar *title) {

    CdosSource *instance = g_object_new (CDOS_TYPE_SOURCE, NULL);

    instance->title = g_strdup (title);

    return instance;
}

/**
 * cdos_source_set_summary_icon:
 * @self: A #CdosSource
 * @icon: 要使用的图标 
 *
 * 设置Source的图标
 *
 */
void cdos_source_set_summary_icon (CdosSource *self, StIcon *icon) {
    ClutterActor *child = st_bin_get_child (self->iconBin);

    if (child) {
        clutter_actor_destroy (child);
    }
    st_bin_set_child (self->iconBin, CLUTTER_ACTOR (icon));
}

static void lastNotificationRemoved (CdosSource *self) {
    guint reason = 0;
    cdos_source_destroy (self, reason);
}