#ifndef __CDOS_SOURCE_H__
#define __CDOS_SOURCE_H__

#include "st.h"
#include "cdos-notification.h"
#include "cdos-generic-container.h"




GType cdos_source_get_type (void);

#define CDOS_TYPE_SOURCE		(cdos_source_get_type ())
#define CDOS_SOURCE(obj)              (G_TYPE_CHECK_INSTANCE_CAST ((obj), CDOS_TYPE_SOURCE, CdosSource))
#define CDOS_IS_SOURCE(obj)           (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CDOS_TYPE_SOURCE))
#define CDOS_SOURCE_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), CDOS_TYPE_SOURCE, CdosSourceClass))
#define CDOS_IS_SOURCE_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), CDOS_TYPE_SOURCE))
#define CDOS_SOURCE_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), CDOS_TYPE_SOURCE, CdosSourceClass))

typedef struct _CdosSource		CdosSource;
typedef struct _CdosSourceClass      	CdosSourceClass;


#define SOURCE_ICON_SIZE 24

struct _CdosSourceClass
{
    GObjectClass parent_class;

    //signal
    void (*title_changed) (CdosSource *self, gchar *title);
    void (*notification_added) (CdosSource *self, CdosNotification *notification);
    void (*cdos_source_notify_signal) (CdosSource *self, CdosNotification *notification);
    void (*destroy) (CdosSource *self, guint reason);
    void (*notification_open) (CdosSource *self, CdosSource source);

    //function
    gboolean (*handle_summary_click)(void);
    void (*set_summary_icon)(CdosSource *self, StIcon *icon);
    void (*open)(GObject *obj, gpointer user_data);
    void (*remove_last)(CdosSource *self);
};


struct _CdosSource
{
    GObject parent_instance;

    //传入参数
    gchar *title;

    //成员变量
    StBoxLayout *actor;
 
    gboolean isTransient;
    gboolean isChat;

    GList *notifications; //指向CdosNotification

    /*<private>*/
    StLabel *counterLabel;
    StBin *counterBin;
    StBin *iconBin;

    gboolean actorDestroyed;
};

/* Source 公有函数 */

void cdos_source_set_transient ( CdosSource *self, gboolean isTransient );

void cdos_source_set_title ( CdosSource *self, gchar *newTitle );

void cdos_source_create_notification ( CdosSource *self );

StBoxLayout *cdos_source_get_summary_icon ( CdosSource *self );

void cdos_source_push_notification ( CdosSource *self, CdosNotification *notification );

void cdos_source_notify ( CdosSource *self, CdosNotification *notification );

void cdos_source_destroy ( CdosSource *self, guint reason );

gboolean cdos_source_handle_summary_click (void);

void cdos_source_open ( GObject *obj, gpointer user_data );

void cdos_source_destroy_non_resident_notifications ( CdosSource *self );

CdosSource *cdos_source_new ( gchar *title );

/*从私有改為公有方法*/
void cdos_source_set_summary_icon ( CdosSource *self, StIcon *icon );

#endif
