#define CLUTTER_DISABLE_DEPRECATION_WARNINGS

#include "cdos-debug.h"
#include "cdos-start.h"
#include "cdos-window-manager.h"
#include "cdos-confirm-dialog.h"
#include "cdos-layout-manager.h"
#include "cdos-monkey-plugin.h"
#include "st.h"
#include <gbcode.h>
#include "cdos-applet.h"
#include "cdos-text-applet.h"
#include "cdos-text-icon-applet.h"
#include "cdos.h"
#include "cdos-global.h"
#include "cdos-test.h"
#include "cdos-test-dnd.h"
#include "cdos-keybinding-manager.h"
#include "cdos-run-dialog.h"
#include "cdos-background.h"
#include "applets/applets.h"
#include "cdos-notification-daemon-source.h"
#include "cdos-notification-daemon.h"
#include "cdos-system-notification-source.h"
#include "cdos-export-overview.h"
#include "cdos-applet-manager.h"
#include "cdos-auto-mount-manager.h"

#include <string.h>
#include <X11/extensions/shape.h>
#include <X11/extensions/Xfixes.h>
#include <meta/keybindings.h>
#include <meta/meta-background-group.h>
#include <meta/main.h>


extern int debug_call;
extern int debug_mode;


CdosGlobal *global = NULL;
MetaScreen *screen;
ClutterActor *ui_group = NULL;
ClutterActor *cdos_panel = NULL;
gchar *desktop_layout = NULL;
guint menu_stack_depth = 0;

static gboolean software_rendering = FALSE;
static CdosWindowTracker *tracker = NULL;
static ClutterStage *stage = NULL;
static unsigned int modal_count = 0;
static GList *modal_actor_focus_stack = NULL;
static gchar *css_style_sheet = NULL;
static const gchar *default_css_style_sheet = "/usr/share/themes/NFS-Classic/cdos/cdos.css";
static CdosActionMode action_mode = CDOS_ACTION_MODE_NORMAL;
static guint screen_width, screen_h;
static gfloat panel_height;
// 工作区补充 liting
static gboolean dynamicWorkspaces = FALSE;
static GList *workspaces = NULL;
static guint checkWorkspacesId;
static guint nWorks = 0;
#define        WORKSPACE_NAME_LENGTH    32
#define     WORKSPACE_NUM           4

static gchar **workspace_names = NULL;

void cdos_notify_error(const char *msg, const char *details)
{
    g_warning ("global ERROR: %s\n%s\n", msg, details);
    CdosSource *source = cdos_source_new (NULL);
    Params *params = g_new0(Params, 1);
    CdosMessageTray* messageTray = cdos_monkey_plugin_get_message_tray(monkey_plugin_get()); //cdos_message_tray_new();
    cdos_message_tray_add(messageTray, source);
    CdosNotification* notification = cdos_notification_new(source, msg, details, params);
    cdos_notification_set_transient(notification, TRUE);
    cdos_notification_set_urgency(notification, NORMAL);
    cdos_source_notify(source, notification);
    g_free (params);
}

static void uiGroup_allocate(gpointer   self,
                             gpointer   box,
                             guint      flags)
{
    GList *iter, *children;

    children = st_container_get_children_list (ST_CONTAINER (self));
    for (iter = children; iter; iter = iter->next)
    {
        ClutterActor *child = iter->data;
        clutter_actor_allocate_preferred_size (child, (ClutterAllocationFlags)flags);
    }
    return;
}

static void uiGroup_get_preferred_width(gpointer    self,
                                        gfloat      for_width,
                                        gpointer    allocation)
{
    CdosGenericContainerAllocation *alloc = allocation;
    gfloat width = clutter_actor_get_width ((ClutterActor *)stage);
    alloc->min_size = width;
    alloc->natural_size = width;
    return;
}

static void uiGroup_get_preferred_height(gpointer self,
                                         gfloat   for_height,
                                         gpointer allocation)
{
    CdosGenericContainerAllocation *alloc = allocation;
    gfloat height = clutter_actor_get_height ((ClutterActor *)stage);
    alloc->min_size = height;
    alloc->natural_size = height;
    return;
}
void 
cdos_reparent_actor(ClutterActor *actor, ClutterActor *parent)
{
    g_return_if_fail(CLUTTER_IS_ACTOR(actor));
    g_return_if_fail(CLUTTER_IS_ACTOR(parent));
    ClutterActor *old_parent = clutter_actor_get_parent (actor);
    if(old_parent != NULL)
    {
        g_object_ref (actor);
        if (ST_IS_CONTAINER (old_parent) || CLUTTER_IS_GROUP (old_parent) || ST_IS_BIN (old_parent))
            clutter_container_remove_actor ((ClutterContainer *)old_parent, actor);
        else 
            clutter_actor_remove_child (old_parent, actor);
    }
    if(parent) {
        if (ST_IS_CONTAINER (parent) || CLUTTER_IS_GROUP (parent) || ST_IS_BIN (parent))
            clutter_container_add_actor ((ClutterContainer *)parent, actor);
        else
            clutter_actor_add_child (parent, actor);
    }
}

void
st_container_remove_actor (ClutterContainer *container,
                           ClutterActor *actor)
{
    g_return_if_fail(CLUTTER_IS_CONTAINER(container));
    g_return_if_fail(CLUTTER_IS_ACTOR(actor));
    /*
    ((struct ClutterContainerIface *)g_type_interface_peek(
        (void *)g_type_check_instance_cast(
            (struct GTypeInstance *)parent, 
            st_container_get_type())->g_class, clutter_container_get_type()))->add();
    */
    ClutterActor *parent;
    parent = clutter_actor_get_parent (actor);
    if(parent != (ClutterActor *)container)
    {
        g_warning ("Attempting to remove actor of type '%s: %p' from "
                "group of class '%s: %p', but the container is not "
                "the actor's parent.",
                g_type_name (G_OBJECT_TYPE (actor)), actor,
                g_type_name (G_OBJECT_TYPE (container)), container);
        return;
    }
    clutter_container_destroy_child_meta (container, actor);
    (G_TYPE_INSTANCE_GET_INTERFACE (ST_CONTAINER (container),
                CLUTTER_TYPE_CONTAINER, ClutterContainerIface))->remove (container, actor);
}

void
st_container_add_actor(ClutterContainer *container, ClutterActor *actor)
{
    //g_assert(container != NULL && actor != NULL);
    g_return_if_fail(CLUTTER_IS_CONTAINER(container));
    g_return_if_fail(CLUTTER_IS_ACTOR(actor));
    ClutterActor *old_parent = clutter_actor_get_parent (actor);
    if (CLUTTER_CONTAINER(old_parent) == container)
        return;

    if(G_UNLIKELY (old_parent != NULL))
    {
        g_warning ("Attempting to add actor of type '%s: %p' to a "
                "container of type '%s: %p', but the actor has "
                "already a parent of type '%s: %p'.",
                g_type_name (G_OBJECT_TYPE (actor)), actor,
                g_type_name (G_OBJECT_TYPE (container)), container,
                g_type_name (G_OBJECT_TYPE (old_parent)), old_parent);
        g_object_ref (actor);

        if(ST_IS_CONTAINER (old_parent) || ST_IS_BIN (old_parent) || CLUTTER_IS_GROUP (old_parent))
            clutter_container_remove_actor ((ClutterContainer *)old_parent, actor);
        else if(CLUTTER_IS_ACTOR (old_parent))
            clutter_actor_remove_child (old_parent, actor);
        else {
            g_object_unref (actor);
            g_warning ("%s: I dont't kown how to add actor: %p to StContainer: %p.", __func__, actor, container);
            return;
        }
    }
 
    clutter_container_create_child_meta (container, actor);
    (G_TYPE_INSTANCE_GET_INTERFACE (ST_CONTAINER (container), 
                CLUTTER_TYPE_CONTAINER, ClutterContainerIface))->add (container, actor);
}

typedef struct _ModalRecord {
    ClutterActor *actor;
    ClutterActor *prev_focus;
    gulong actor_destroy_id;
    gulong prev_focus_destroy_id;
    CdosActionMode action_mode;
}ModalRecord;

static gint _find_modal (ClutterActor *actor){
    gint index = 0;
    GList *focus_stack = NULL;
    focus_stack = modal_actor_focus_stack;
    ModalRecord *record = NULL;

    while (focus_stack != NULL){
        GList * next = focus_stack->next;
        record = focus_stack->data;
        if (record && record->actor == actor){
            return index;
        }
        index ++;
        focus_stack = next;
    }
    return -1;
}

static void
on_focus_destroy (ClutterActor *actor, ModalRecord *record)
{
    guint index = _find_modal (actor);

    if (index > 0){
        record->prev_focus = NULL;
        g_signal_handler_disconnect (record->prev_focus, record->prev_focus_destroy_id);
        record->prev_focus_destroy_id = 0;
    }
}

static void
_cdos_pop_modal (ClutterActor *actor)
{
    guint focus_index = _find_modal (actor);
    if (focus_index >= 0)
        cdos_pop_modal (actor, 0);
}

gint cdos_get_modal_count (void)
{
    return modal_count;
}

CdosActionMode cdos_get_action_mode (){
    return action_mode;
}

gboolean
cdos_push_modal (ClutterActor *actor, guint timestamp, MetaModalOptions options)
{
    g_return_val_if_fail(CLUTTER_IS_ACTOR(actor),FALSE);

    MetaScreen *screen = cdos_global_get_screen (global);
    if(!timestamp)
        timestamp = cdos_global_get_current_time (global);
    if(modal_count == 0)
    {
        if(!cdos_global_begin_modal (global, timestamp, options))
        {
            g_critical (" %s: %p\n", __func__, actor);
            return FALSE;
        }
        meta_disable_unredirect_for_screen (screen);
    } 
    ClutterStage *stage = cdos_global_get_stage (global);

    modal_count += 1;
    gulong actor_destroy_id = g_signal_connect (actor, "destroy",
            G_CALLBACK (_cdos_pop_modal), NULL);

    ModalRecord *record = g_new0 (ModalRecord, 1);
    record->actor = actor;
    record->prev_focus = clutter_stage_get_key_focus (stage);
    record->actor_destroy_id = actor_destroy_id;
    record->action_mode = CDOS_ACTION_MODE_NONE;

    if(record->prev_focus != NULL)
    {
        record->prev_focus_destroy_id = g_signal_connect (record->prev_focus, "destroy",
                G_CALLBACK (on_focus_destroy), record);
    }
    modal_actor_focus_stack = g_list_append (modal_actor_focus_stack, record);
    action_mode = CDOS_ACTION_MODE_NONE;
    clutter_stage_set_key_focus (stage, actor);
    return TRUE;
}

void
cdos_pop_modal (ClutterActor *actor, guint timestamp)
{
    g_return_if_fail(CLUTTER_IS_ACTOR(actor));
    gint focus_index = 0;

    if(!timestamp)
        timestamp = cdos_global_get_current_time (global);
    focus_index = _find_modal (actor);

    ClutterStage *stage = cdos_global_get_stage (global);
    if(focus_index < 0)
    {
        clutter_stage_set_key_focus (stage, NULL);
        cdos_global_end_modal (global, timestamp);
    	action_mode = CDOS_ACTION_MODE_NORMAL;
        g_critical (": %s %p\n", __func__, actor);
        return;
    }
    modal_count -= 1;
    ModalRecord *record = g_list_nth_data (modal_actor_focus_stack, focus_index);
    g_signal_handler_disconnect (record->actor, record->actor_destroy_id);

    guint focus_length = g_list_length ( modal_actor_focus_stack);
    if(focus_index == focus_length - 1)
    {
        if(record->prev_focus != NULL)
            g_signal_handler_disconnect (record->prev_focus, record->prev_focus_destroy_id);
        action_mode = record->action_mode;
        clutter_stage_set_key_focus (stage, record->prev_focus);
    }else {
        GList *last_item = g_list_last (modal_actor_focus_stack);
        if (last_item){
            ModalRecord *t = last_item->data;
            if(t->prev_focus)
                g_signal_handler_disconnect (t->prev_focus, t->prev_focus_destroy_id);
        }
        guint i = 0;
        for (i= focus_length - 1; i > focus_index; i--)
        {
        ModalRecord *m = g_list_nth_data (modal_actor_focus_stack, i);
        ModalRecord *n = g_list_nth_data (modal_actor_focus_stack, i-1);
        m->prev_focus = n->prev_focus;
        m->prev_focus_destroy_id = n->prev_focus_destroy_id;
        m->action_mode = n->action_mode;
        }
    }
    // delete
    modal_actor_focus_stack = g_list_remove (modal_actor_focus_stack, record); // FIXME, only remove first one
    if(modal_count > 0)
        return;
    cdos_global_end_modal (global, timestamp);

    CdosLayoutManager *layout_manager = cdos_layout_manager_get_default ();
    cdos_layout_manager_update_regions (layout_manager);
    MetaScreen *screen = cdos_global_get_screen (global);
    meta_enable_unredirect_for_screen (screen);
    action_mode = CDOS_ACTION_MODE_NORMAL;
}

const gchar *
cdos_get_theme_style_sheet (void)
{
    return css_style_sheet;
}

void
cdos_set_theme_style_sheet (const gchar *css)
{
    if (css == NULL)
        return;
    g_free (css_style_sheet);
    css_style_sheet = g_strdup (css);
}

void
cdos_load_theme (void)
{
    StThemeContext *context = st_theme_context_get_for_stage (
            cdos_global_get_stage (global));
    StTheme *theme = st_theme_new (NULL, NULL, NULL);
    gboolean loaded = false;
    GError *err = NULL;
    if(css_style_sheet)
        loaded = st_theme_load_stylesheet (theme, css_style_sheet, &err);
    if(!loaded)
    {
        if(err != NULL){
            g_error_free(err);
        }
        st_theme_load_stylesheet (theme, default_css_style_sheet, &err);
        if(css_style_sheet)
            g_warning ("there was some problem parsing the theme: %s. Fallback to the default theme.", css_style_sheet);
    }
    st_theme_context_set_theme (context, theme);
    if(err != NULL){
        g_error_free(err);
    }
}

gboolean 
cdos_is_software_rendering (void)
{
    return software_rendering;
}

gboolean
cdos_is_interesting (MetaWindow *window)
{
    const char *title = meta_window_get_title (window);
    if (title != NULL)
        if (strcmp(title, "JavaEmbeddedFrame") == 0)
            return false;

    if (cdos_window_tracker_is_window_interesting (window))
        return true;
    
    if (cdos_window_tracker_get_window_app (tracker, window))
        return false;

    MetaWindowType type = meta_window_get_window_type (window);
    return (type == META_WINDOW_DIALOG) || (type == META_WINDOW_MODAL_DIALOG);
}

static gboolean
win_activate (MetaWindow *window, void *data)
{
   guint32 time = cdos_global_get_current_time (global);
    
    meta_window_activate (window, time);
    return true;
}
static int
win_foreach_transient (MetaWindow *win)
{
    meta_window_foreach_transient (win, win_activate, 0);
    return 0;
}
void
cdos_activate_window (MetaWindow *window, guint32 time, int workspace_num)
{
    g_return_if_fail (META_IS_WINDOW (window));
    MetaScreen *scr = cdos_global_get_screen (global);
    int active_wks_num = meta_screen_get_active_workspace_index (scr);
    int window_wks_num;
    if (workspace_num >= 0)
        window_wks_num = workspace_num;
    else
    {
        MetaWorkspace *wks = meta_window_get_workspace (window);
        window_wks_num = meta_workspace_index (wks);
    }
    if (!time)
        time = cdos_global_get_current_time (global);

    if (window_wks_num != active_wks_num)
    {
        MetaWorkspace *workspace = meta_screen_get_workspace_by_index (scr, window_wks_num);
        meta_workspace_activate_with_focus (workspace, window, time);
    }
    else
    {
        meta_window_activate (window, time);
        g_idle_add ((GSourceFunc)win_foreach_transient, window);
    }

    CdosOverview *overview = cdos_overview_get ();
    clutter_actor_hide ((ClutterActor *)overview);
}

GList *
cdos_get_tab_list (MetaWorkspace *workspace_opt, MetaScreen *screen_opt)
{
    //g_return_val_if_fail(META_IS_WORKSPACE(workspace_opt),NULL);
    //g_return_val_if_fail(META_IS_SCREEN(screen_opt),NULL);
    MetaScreen *screen = screen_opt ? screen_opt : cdos_global_get_screen (global);
    MetaDisplay *display = meta_screen_get_display (screen);
    MetaWorkspace *workspace = workspace_opt ? workspace_opt : meta_screen_get_active_workspace (screen);
    GList *windows = NULL;
    GList *all_windows = meta_display_get_tab_list (display, META_TAB_LIST_NORMAL_ALL, workspace);
    GList *iter = NULL;
    for (iter = all_windows; iter; iter = iter->next)
    {
        MetaWindow *w = iter->data;
        if (cdos_is_interesting (w))
        {
            if (g_list_find (windows, iter->data) == NULL)
            {
                windows = g_list_append (windows, iter->data);
            }
        }
    }
    return windows;
}

// add by xunli
gboolean window_actor_displayed_on_workspace (MetaWindowActor *win, gint workspaceIndex){

    g_return_val_if_fail(META_IS_WINDOW_ACTOR(win),FALSE);
    MetaScreen *scr = cdos_global_get_screen (global);
    if (meta_screen_get_active_workspace_index (scr) == workspaceIndex){
        return TRUE;
    }
    MetaWindow *mwin = meta_window_actor_get_meta_window (win);
    return mwin && meta_window_is_on_all_workspaces (mwin) ; //FIXME
}

GList *
get_window_actors_for_workspace (MetaWorkspace *workspace)
{
    CdosGlobal * global = cdos_global_get ();

    GList *window_actors = NULL;
    GList *windowfilter = NULL;

    window_actors = cdos_global_get_window_actors (global);

    while (window_actors != NULL){
        GList *next = window_actors->next;
        MetaWindowActor *win = window_actors->data;
        MetaWindow *mwin = meta_window_actor_get_meta_window (win);
        
        //if (window_actor_displayed_on_workspace(win, workspaceIndex)){
        if (meta_window_located_on_workspace(mwin, workspace)){
            windowfilter = g_list_prepend (windowfilter, win);
        }
        window_actors = next;
    }
    windowfilter = g_list_reverse (windowfilter);
    return windowfilter;
}
// end

// 不是测试代码，用于panel 的右键点击
static gboolean
press_callback_for_panel_right (ClutterActor *actor, ClutterEvent *event, gpointer user_data){
    cdos_panel_on_button_press (cdos_panel, event, NULL);
    return FALSE;
}

// add by liting for workspace :(
typedef struct _WrapWorkspace
{
    MetaWorkspace *workspace;
    MetaWindow *lastRemovedWindow;
    guint windowAddedId;
    guint windowRemovedId;
} WrapWorkspace;

static gboolean cdos_start_check_workspaces (void){
    if (!dynamicWorkspaces){
        return FALSE;
    }
    guint len = g_list_length (workspaces);
    guint i = 0;
    gboolean *emptyWorkspaces = {FALSE};
    emptyWorkspaces = g_malloc0 (sizeof(gboolean) * len);
    GList *ws = workspaces;
    while(ws != NULL){
        WrapWorkspace *wworkspace = ws->data;
        MetaWindow *lastRemoved = wworkspace->lastRemovedWindow;
        MetaWindowType type = meta_window_get_window_type (lastRemoved);
        if (lastRemoved && (type == META_WINDOW_SPLASHSCREEN || type == META_WINDOW_DIALOG
                            ||type == META_WINDOW_MODAL_DIALOG)){
            emptyWorkspaces [i] = FALSE;
        }else {
            emptyWorkspaces [i] = TRUE;
        }
        ws = ws->next;
    }

    GList *win_actors = cdos_global_get_window_actors (global);
    while (win_actors != NULL){
        MetaWindowActor *win = win_actors->data;

        MetaWindow *mwin = meta_window_actor_get_meta_window (win);
        if (meta_window_is_on_all_workspaces (mwin)){
            continue;
        }
        guint workspaceIndex = meta_screen_get_active_workspace_index (cdos_global_get_screen (global));
        emptyWorkspaces[workspaceIndex] =  FALSE;
        win_actors = win_actors->next;
    }


    return FALSE;
    // FIXME 写了一半，撒手不管先，先 staticWorkspaces
}

/*
 *  该函数问题比较多，需要重构。
 */
static void criticalNotify(gchar* msg, gchar* details, ClutterActor* icon) {
    //CdosSource *source = cdos_source_new (msg);
    CdosSource *source = cdos_system_notification_source_new ();
    CdosMessageTray* messageTray = cdos_monkey_plugin_get_message_tray(monkey_plugin_get()); //cdos_message_tray_new();
    cdos_message_tray_add(messageTray, source);
    Params* params = g_new0(Params, 1);
    params->icon = ST_ICON(icon);
    CdosNotification* notification = cdos_notification_new(source, msg, details, params);
    cdos_notification_set_transient(notification, TRUE);
    cdos_notification_set_urgency(notification, CRITICAL);
    cdos_source_notify(source, notification);
//    g_free(params);
}

static void notifyCinnamon2d(void) {
    ClutterActor* icon = st_icon_new();
    st_icon_set_icon_name(ST_ICON(icon), "display");
    st_icon_set_icon_type(ST_ICON(icon), ST_ICON_FULLCOLOR);
    st_icon_set_icon_size(ST_ICON(icon), 36);

    gchar *buffer = g_strconcat (_("Cdos is currently running without video hardware acceleration and, as a result, you may observe much higher than normal CPU usage.\n\n"),
                                 _("There could be a problem with your drivers or some other issue.  For the best experience, it is recommended that you only use this mode for"),
                                 _(" troubleshooting purposes."),
                                 NULL);
    //criticalNotify(_("Running in software rendering mode"), buffer, icon);
    g_free (buffer);
}

static gboolean
queue_check_workspaces (void){
    if (!dynamicWorkspaces){
        return FALSE;
    }
    if (checkWorkspacesId == 0){
        checkWorkspacesId = meta_later_add (META_LATER_BEFORE_REDRAW,
                                            (GSourceFunc) cdos_start_check_workspaces,
                                            NULL,NULL);
    }
    return TRUE;
}

typedef struct _TransferWorkspaceWindow{
    WrapWorkspace *wworkspace;
    MetaWindow *mwin;
} TransferWorkspaceWindow;

static void onTimeoutRemoved (TransferWorkspaceWindow *transfer) {

    WrapWorkspace *wworkspace = transfer->wworkspace;

    if (wworkspace->lastRemovedWindow == transfer->mwin){
        wworkspace->lastRemovedWindow = NULL;
        queue_check_workspaces();
    }
    g_free (transfer);
}

static void
on_window_removed (MetaWorkspace *workspace, MetaWindow *mwin, gpointer data)
{
    WrapWorkspace *wworkspace = (WrapWorkspace *)data;
    wworkspace->lastRemovedWindow = mwin;
    queue_check_workspaces ();
    
    TransferWorkspaceWindow *transfer = g_malloc0 (sizeof (TransferWorkspaceWindow));
    transfer->wworkspace = wworkspace;
    transfer->mwin = mwin;
// timeout add
    g_timeout_add (1000, (GSourceFunc) onTimeoutRemoved, transfer);
}
static gboolean
cdos_start_n_workspaces_changed (void){
    if (!dynamicWorkspaces){
        return FALSE;
    }
    guint oldNumWorkspaces = g_list_length (workspaces);
    guint newNumWorkspaces = meta_screen_get_n_workspaces (screen);

    if (oldNumWorkspaces == newNumWorkspaces){
        return FALSE;
    }
    //GList *lostWorkspaces = NULL;
    guint windowAddedId = 0;
    guint windowRemovedId = 0; // 这俩会挪到封装结构体中
    if (newNumWorkspaces > oldNumWorkspaces){
        guint wnum = 0;
        for (wnum = oldNumWorkspaces; wnum < newNumWorkspaces; wnum++){
// 这里获取的 workspace 并没有 removed window ，需要封装后再加入链表
            workspaces = g_list_append (workspaces, meta_screen_get_workspace_by_index (screen, wnum));
            MetaWorkspace *workspace = g_list_nth_data (workspaces, wnum);
            windowAddedId = g_signal_connect (workspace, "window-added", 
                                                G_CALLBACK (queue_check_workspaces), NULL);

            windowRemovedId = g_signal_connect (workspace, "window-removed", 
                                                G_CALLBACK (on_window_removed), NULL);
        }
    } else {
        guint removedIndex = 0;
        guint removedNum = oldNumWorkspaces - newNumWorkspaces;
        guint w = 0;
        for (w=0; w< oldNumWorkspaces; w++){
            MetaWorkspace *workspace = meta_screen_get_workspace_by_index (screen, w);

            if (g_list_nth_data (workspaces, w) != workspace){
                removedIndex = w;
                break;
            }
        }
        for (w = 0; w < removedNum; w++){
            MetaWorkspace *remove = g_list_nth_data (workspaces, removedIndex + w);
            workspaces = g_list_remove (workspaces, remove);
            //lostWorkspaces = g_list_append (lostWorkspaces, remove);
            g_signal_handler_disconnect (remove, windowAddedId);
            g_signal_handler_disconnect (remove, windowRemovedId);

        }
    }
    queue_check_workspaces ();
    return FALSE;
}

static gboolean
cdos_start_static_workspaced (void){

    gint dif = nWorks - meta_screen_get_n_workspaces (screen);

    if (dif > 0) {
        guint i = 0;
        for (i=0; i< dif; i++){
            meta_screen_append_new_workspace (screen, FALSE, 
                                                cdos_global_get_current_time (global));
        }
    }else {
        gint i = 0;
        if (nWorks == 0){
            return FALSE;
        }
        for (i=0; i > dif; i--){
            guint removeWorkspaceIndex = meta_screen_get_n_workspaces (screen) - 1;
            MetaWorkspace *removeW = meta_screen_get_workspace_by_index (screen, removeWorkspaceIndex);
            //lastRemoved 并没有
            meta_screen_remove_workspace (screen, removeW, 
                                                    cdos_global_get_current_time(global)); 
            
        }
    }
    return TRUE;
}

static gchar * make_default_workspace_name (int index)
{
    return g_strdup_printf("%s %d", _("Workspace"), index + 1);
}

static void cdos_trim_workspace_names ( int index){
    guint len = g_strv_length (workspace_names);
    gint i = 0;
     
    guint new_len = len;

    for (i = len -1; i >=0 && (i >= nWorks || !strlen (workspace_names[i])); i--)
    {
        g_free (workspace_names[i]);
        new_len--;
    }
    workspace_names = g_renew (gchar *, workspace_names, new_len +1 );
    workspace_names[new_len] = NULL;
    
    cdos_debug ("len is %d, new_len: %d\n", len, new_len);
}

static gchar*
c_strdup (const char *str)
{
    if (str == NULL)
        return NULL;
    if (strlen (str) > 0)
        return g_strdup (str);
    char *new_str = g_malloc0 (1);
    return new_str;
}

// fill 本是为了填充空位， overrides 值为 "" 时，显示默认名
static void cdos_fill_workspace_names (int index){
    guint len = g_strv_length (workspace_names);
    cdos_debug ("len in fill : %d, index: %d\n", len, index);

    if (index >= len)
    {
        if (workspace_names == NULL)
            workspace_names = g_new0 (gchar *, index + 2);
        else{
           // gchar **temp = NULL;
            //temp = g_renew (gchar *, workspace_names, index + 2);
            //g_strfreev(workspace_names);
            //workspace_names = temp;
            workspace_names = g_renew(gchar *,workspace_names, index + 2);
        }
        workspace_names[index + 1] = NULL;
        gint i = 0;
        for (i = index; i >= len && i >= 0; --i){
            cdos_debug ("%d %d %d", i, index, len);
            workspace_names[i] = c_strdup ("");
        }
    }
}

void cdos_set_workspace_name (guint index, gchar *name){
    // trim name
    gchar *ws_name = cdos_get_workspace_name (index);
    if ( g_strcmp0 (name, ws_name) != 0){
        //g_free (ws_name);
        cdos_fill_workspace_names (index);

        gchar *default_name = make_default_workspace_name (index);
        if ( g_strcmp0 (name, default_name) != 0) {
            workspace_names[index] = g_strdup (name);
        }else 
            workspace_names[index] = c_strdup ("");
        //g_free (default_name);

        cdos_trim_workspace_names (index);
        GSettings *settings = cdos_global_get_settings (global);

        cdos_debug ("%d %d", index, g_strv_length (workspace_names));

        g_settings_set_strv (settings, "workspace-name-overrides", (const char *const *)workspace_names);
    }
}

/**
 *
 *
 */
gchar * cdos_get_workspace_name (guint index){
    guint len = g_strv_length (workspace_names);
    gchar *wsName = index < len ? workspace_names[index] : NULL;

    return (wsName && strlen(wsName) > 0) ? g_strdup (wsName) : make_default_workspace_name (index);    
}

gboolean
cdos_start_add_workspace (void){
    if (dynamicWorkspaces || (nWorks >= 12)){
        if (nWorks >= 12){
            // comfirm
            //
        }
        return FALSE;
    }
    nWorks++;
    GSettings *settings = cdos_global_get_settings (global);
    g_settings_set_int (settings, "number-workspaces", nWorks);
    cdos_start_static_workspaced ();
    return TRUE;
}

static void
cdos_start_window_entered_monitor (MetaScreen *screen, guint monitorIndex,MetaWindow *mwin){
     
}

static void
cdos_start_window_left_monitor (MetaScreen *screen, guint monitorIndex, MetaWindow *mwin){

}

static void
cdos_start_window_restacked (MetaScreen *screen, gpointer data){
    cdos_global_sync_pointer (global);
    
}
// end

static void
cdos_start_init_recorder (void)
{
  //TODO
  cdos_debug ();
}

static void 
cdos_init_user_session_dialog_callback (MetaDisplay *display,
                MetaScreen *screen,
                MetaWindow *window,
                XEvent *event,
                MetaKeyBinding *binding,
                gpointer user_data)
{
    CdosRunDialog *runDialog =  cdos_run_dialog_get_default();
    cdos_run_dialog_open(runDialog);
}

static void
cdos_init_user_session_menu_callback (MetaDisplay *display,
                MetaScreen *screen,
                MetaWindow *window,
                XEvent *event,
                MetaKeyBinding *binding,
                gpointer user_data)
{
  cdos_debug ();
}

static void
cdos_start_init_user_session (void)
{
  cdos_start_init_recorder ();
  MetaScreen *global_screen = cdos_global_get_screen (global);
  meta_screen_override_workspace_layout (global_screen, META_SCREEN_TOPLEFT, FALSE, 1, -1);
  CdosSystrayManager *systray_manager = cdos_systray_manager_new ();
  cdos_monkey_plugin_set_systray_manager (monkey_plugin_get (), systray_manager); 

  //TODO: ExtensionSystem.init();

  meta_keybindings_set_custom_handler ("panel-run-dialog", (MetaKeyHandlerFunc)cdos_init_user_session_dialog_callback, NULL, NULL);
  meta_keybindings_set_custom_handler ("panel-main-menu", (MetaKeyHandlerFunc)cdos_init_user_session_menu_callback, NULL, NULL);
  cdos_debug();
}

static void cdos_ui_group_show_done(ClutterTimeline *tml, CdosLayoutManager *layoutManager){
    cdos_layout_manager_update_regions (layoutManager);
}

static void
hide_panel_in_fullscreen (MetaScreen *screen, gpointer data)
{
    gint monitor = cdos_layout_manager_get_index ();
    gint current_monitor = meta_screen_get_current_monitor (screen);
    gint primary_monitor = meta_screen_get_primary_monitor (screen);

    //g_print ("layout monitor: %d, current monitor: %d, primary monitor: %d\n", 
    //		    			monitor, current_monitor, primary_monitor);
    if (meta_screen_get_monitor_in_fullscreen (screen, monitor))
    {
	if (primary_monitor == current_monitor)
	{
	    cdos_start_disable_panels ();
	}
    }else {
    	cdos_start_enable_panels ();
    } 
}

int cdos_plugin_start(CdosGlobal *main_plugin_global)
{
    MonkeyPlugin *plugin;
    CdosPanel *panel = NULL, *panel2 = NULL;

    bindtextdomain (GETTEXT_PACKAGE, "/usr/share/cdos-de/locale");
    textdomain ("cdos-desktop");

    if(g_getenv ("CINNAMON_SOFTWARE_RENDERING"))
    {
        g_print ("Cinnamon Software Rendering mode enabled\n");
        software_rendering = true;
    }

    plugin = monkey_plugin_get ();
    monkey_plugin_set_applet_side (plugin, ST_SIDE_BOTTOM);

    if (main_plugin_global)
        global = main_plugin_global;
    else
        global = cdos_global_get ();
//FIXME    cdos_global_set_error_notify_func (global, grab_notify_error);

    g_desktop_app_info_set_desktop_env ("X-Cdos-desktop");

    screen = cdos_global_get_screen (global);
    meta_screen_get_size (screen, &screen_width, &screen_h);
    tracker = cdos_window_tracker_get_default ();
    cdos_app_system_get_default ();
  
    GSettings *settings = cdos_global_get_settings (global);
    monkey_plugin_set_gsettings (plugin, settings);

    desktop_layout = g_settings_get_string (settings, "desktop-layout");
    monkey_plugin_set_desktop_layout (plugin, desktop_layout);

    if (!g_strcmp0 (desktop_layout, LAYOUT_FLIPPED)) {
        monkey_plugin_set_applet_side (plugin, ST_SIDE_TOP);
    } else if (g_strcmp0 (desktop_layout, LAYOUT_CLASSIC)) {
        monkey_plugin_set_applet_side (plugin, ST_SIDE_TOP);
    }

    gtk_icon_theme_append_search_path (gtk_icon_theme_get_default(), "/usr/share/cdos-de/icons");

    stage = cdos_global_get_stage (global);

    CdosThemeManager *tm = g_object_new (CDOS_TYPE_THEME_MANAGER, NULL);
    monkey_plugin_set_theme_manager (plugin, tm);

    CdosSoundManager *sm G_GNUC_UNUSED 
        = cdos_sound_manager_new ();

    //stage 各层
    CdosGenericContainer *uiGroup = g_object_new (CDOS_TYPE_GENERIC_CONTAINER, NULL);
    ui_group = (ClutterActor *)uiGroup;
    st_set_ui_root ((ClutterStage *)stage, (ClutterContainer *)uiGroup);
    monkey_plugin_set_ui_group (plugin, ui_group);
    g_signal_connect (uiGroup, "allocate", G_CALLBACK (uiGroup_allocate), NULL);
    g_signal_connect (uiGroup, "get-preferred-width", G_CALLBACK (uiGroup_get_preferred_width), NULL);
    g_signal_connect (uiGroup, "get-preferred-height", G_CALLBACK (uiGroup_get_preferred_height), NULL);

    //最底层 window-group， 文件管理器与窗口都位于这一层
    ClutterActor *window_group = NULL;
    window_group = cdos_global_get_window_group (global);
    g_object_ref (window_group);
    clutter_actor_remove_child ((ClutterActor *)stage, window_group);
    clutter_container_add_actor ((ClutterContainer *)uiGroup, window_group);
    clutter_actor_add_child ((ClutterActor *)stage, (ClutterActor *)uiGroup);

    ClutterActor *background_group = meta_background_group_new ();
    clutter_actor_add_child (window_group, background_group);
    clutter_actor_lower_bottom (background_group);

    clutter_container_raise_child ((ClutterContainer *)stage, (ClutterActor *)uiGroup, NULL);
    // 根据 screen 获取 display， 根据 display 获取 focus window
    // 监听 focus window 的position change 信号
/*    MetaDisplay *display = meta_screen_get_display (screen);
    g_signal_connect (display, "notify::focus-window",
                                G_CALLBACK (focus_window_changed), NULL);*/

    // panelBox ==> panel 
    CdosLayoutManager *layout_manager = cdos_layout_manager_new ();
    cdos_monkey_plugin_set_layout_manager (plugin, layout_manager);
    StBoxLayout *panelBox = cdos_layout_manager_get_panelbox (layout_manager);
    st_box_layout_set_vertical (panelBox, TRUE);
    clutter_actor_set_reactive ((ClutterActor *)panelBox, TRUE);

    panel = cdos_panel_new (TRUE, TRUE);
    st_widget_add_style_class_name ((StWidget *)panel, "panel-bottom");
    cdos_monkey_plugin_set_panel (plugin, panel);

    cdos_panel = (ClutterActor *)panel;
    clutter_actor_set_reactive (cdos_panel, TRUE);
    clutter_container_add_actor ((ClutterContainer *)panelBox, (ClutterActor *)cdos_panel);

    cdos_panel_set_bottom_position (panel, TRUE);
    cdos_panel_set_hideable (panel, TRUE);

    g_signal_connect (screen, "in-fullscreen-changed", G_CALLBACK (hide_panel_in_fullscreen), layout_manager);
    StBoxLayout *left_box = cdos_panel_get_leftbox (panel);
    StBoxLayout *center_box = cdos_panel_get_centerbox (panel);
    StBoxLayout *right_box = cdos_panel_get_rightbox (panel);
    g_signal_connect ((ClutterActor *)panelBox, "button-release-event", G_CALLBACK (press_callback_for_panel_right), NULL);
    clutter_actor_set_name ((ClutterActor *)left_box, "panelLeft");
    clutter_actor_set_name ((ClutterActor *)center_box, "panelCenter");
    clutter_actor_set_name ((ClutterActor *)right_box, "panelRight");

    //右键菜单，包括输入法菜单等位于 top window group
    ClutterActor *top_window_group = NULL;
    top_window_group = cdos_global_get_top_window_group (global);
    clutter_actor_set_child_above_sibling ((ClutterActor *)stage, top_window_group, NULL);

    ClutterActor *feedback_group = NULL;
    feedback_group = meta_get_feedback_group_for_screen (screen);
    g_object_ref (feedback_group);
    clutter_container_remove_actor ((ClutterContainer *)stage, feedback_group);
    clutter_container_add_actor ((ClutterContainer *)ui_group, feedback_group);

// 0214
    CdosSystrayManager *systray = cdos_systray_manager_new ();
    cdos_monkey_plugin_set_systray_manager (plugin, systray);
    CdosIconDispatcher *status_icon_dispatcher = cdos_icon_dispatcher_new ();
    cdos_monkey_plugin_set_status_icon_dispatcher (plugin, status_icon_dispatcher);
    CdosOverview *overview = cdos_overview_new();

    CdosMessageTray *message_tray = cdos_message_tray_new ();
    cdos_monkey_plugin_set_message_tray (plugin, message_tray);

    CdosNotificationDaemon* notification_daemon = g_object_new(CDOS_TYPE_NOTIFICATION_DAEMON, NULL);
    cdos_monkey_plugin_set_notification_daemon(plugin, notification_daemon);
// 0214
    /* leit: get panel themeNode height */
    StThemeNode *panel_node = st_widget_get_theme_node ((StWidget *)panelBox);
    panel_height = st_theme_node_get_length (panel_node, "height");
    if (panel_height > 0.f)
        cdos_panel_set_height(panel,panel_height);
    else
        cdos_panel_set_height(panel,DEFAULT_PANEL_HEIGHT);

    cdos_layout_manager_update_boxes (layout_manager);
    cdos_layout_manager_update_regions (layout_manager);

    cdos_applet_manager_init ();
    cdos_start_init_user_session ();

    CdosBaseWM *cdos_base_wm = NULL;
    cdos_base_wm = cdos_global_get_window_manager (global);
    monkey_plugin_set_cdos_base_wm (plugin, cdos_base_wm);

    StFocusManager *focus_manager = NULL;
    focus_manager = cdos_global_get_focus_manager (global);
    monkey_plugin_set_focus_manager (plugin, focus_manager);

    CdosWM *wm = cdos_wm_new (cdos_base_wm);
    //zhangdj
    CdosKeybindingManager *keybinding_manager = cdos_keybinding_manager_new ();
    CdosAutoMountManager *automount = cdos_auto_mount_manager_new ();

    // 工作区 
    nWorks =  g_settings_get_int (settings, "number-workspaces");
    dynamicWorkspaces = FALSE;

    if (!dynamicWorkspaces){
        cdos_start_static_workspaced ();
    }

    workspace_names = g_settings_get_strv (settings, "workspace-name-overrides");

    g_signal_connect (screen, "notify::n-workspaces", 
                            G_CALLBACK (cdos_start_n_workspaces_changed), NULL);
    g_signal_connect (screen, "window-entered-monitor",
                            G_CALLBACK (cdos_start_window_entered_monitor), NULL);
    g_signal_connect (screen, "window-left-monitor",
                            G_CALLBACK (cdos_start_window_left_monitor), NULL);
    g_signal_connect (screen, "restacked", G_CALLBACK (cdos_start_window_restacked), NULL);

    cdos_start_n_workspaces_changed (); 
     
    cdos_background_manager_new (background_group, 0, 0);

    CdosKeyboard *keyboard = cdos_keyboard_new();
    cdos_monkey_plugin_set_keyboard(keyboard);
    cdos_keyboard_start(keyboard);

    clutter_actor_show ((ClutterActor *)stage);
    cdos_transition_to (ui_group,
                        CLUTTER_EASE_OUT_QUAD, 250, 0, 
                        (TransCallback)cdos_ui_group_show_done, layout_manager,
                        "show-uigroup",
                        "scale-x", TRANS_PROP_DOUBLE, 1.0,
                        "scale-y", TRANS_PROP_DOUBLE, 1.0,
                        "opacity", TRANS_PROP_UCHAR, 255,
                                                NULL);
    meta_register_with_session ();
    cdos_util_spawn_command_line ("xset dpms 0 0 0");
    return 0;
}

void cdos_start_enable_panels(void){
    MonkeyPlugin *plugin;
    CdosPanel *panel, *panel2;
    plugin = monkey_plugin_get ();

    panel = cdos_monkey_plugin_get_panel(plugin);
    panel2 = cdos_monkey_plugin_get_panel2(plugin);

    if(panel){
        //notice:该函数暂无，待解封
        //cdos_panel_enable(panel);
        cdos_panel_enable(panel);
    }
    if(panel2){
        //notice:该函数暂无，待解封
        //cdos_panel_enable(panel2);

    }
};

void cdos_start_disable_panels(void){
    MonkeyPlugin *plugin;
    CdosPanel *panel, *panel2;
    plugin = monkey_plugin_get ();

    panel = cdos_monkey_plugin_get_panel(plugin);
    panel2 = cdos_monkey_plugin_get_panel2(plugin);
    if(panel){
        //notice:该函数暂无，待解封
        //cdos_panel_disable(panel);
        cdos_panel_disable(panel);
    }
    if(panel2){
        //notice:该函数暂无，待解封
        //cdos_panel_disable(panel2);
    }
};

//end
