#ifndef __CDOS_START_H__
#define __CDOS_START_H__

#include <stdlib.h>
//#include <libintl.h>
#include <stdio.h>
#include <stdint.h>

#include <meta/util.h>
#include <meta/display.h>
#include <meta/workspace.h>
#include <meta/screen.h>

#include "cdos-global.h"
#include "cdos-wm.h"
#include "cdos-generic-container.h"
#include "cdos-window-tracker.h"
#include "cdos-overview.h"
#include "cdos-monkey-plugin.h"

extern CdosGlobal *global;
extern ClutterActor *ui_group;
extern ClutterActor *cdos_panel;
extern gchar *desktop_layout;
extern guint menu_stack_depth;

#define DEFAULT_BACKGROUND_COLOR clutter_color_new(0x22, 0x66, 0xbb, 0xff)
#define LAYOUT_TRADITIONAL "traditional"
#define LAYOUT_FLIPPED "flipped"
#define LAYOUT_CLASSIC "classic"

//#define CIN_LOG_FOLDER "/.cdos/"

void cdos_reparent_actor(ClutterActor *actor, ClutterActor *parent);

void st_container_add_actor (ClutterContainer *container, ClutterActor *actor);
void st_container_remove_actor (ClutterContainer *container, ClutterActor *actor);

int cdos_plugin_start(CdosGlobal *cdos_plugin_global);

void cdos_load_theme (void);
const gchar *cdos_get_theme_style_sheet (void);
void cdos_set_theme_style_sheet (const gchar *css);

void cdos_windows_restacked (void);
void cdos_window_entered_monitor (MetaScreen *screen,
                             int monitor_index,
                             MetaWindow *meta_win);
void cdos_window_left_monitor    (MetaScreen *screen,
                             int monitor_index,
                             MetaWindow *meta_win);

gint cdos_get_modal_count (void);

gboolean cdos_push_modal (ClutterActor *actor, guint timestamp, MetaModalOptions options);

void cdos_pop_modal (ClutterActor *actor, guint timestamp);

gboolean cdos_is_software_rendering (void);

gboolean cdos_is_interesting (MetaWindow *window);

GList *get_window_actors_for_workspace (MetaWorkspace *workspace);

void cdos_activate_window (MetaWindow *window, guint32 time, int workspace_num);

GList *cdos_get_tab_list (MetaWorkspace *workspace_opt, MetaScreen *screen_opt);

gchar * cdos_get_workspace_name (guint index);
void cdos_set_workspace_name (guint index, gchar *name);
CdosActionMode cdos_get_action_mode ();

gboolean cdos_start_add_workspace (void);

gboolean window_actor_displayed_on_workspace (MetaWindowActor *win, gint workspaceIndex);
//add by zhangtao 20160512
void cdos_start_enable_panels(void);
void cdos_start_disable_panels(void);

/* clutter 动画 api 封装 */
typedef void (*TransCallback) (ClutterTimeline *tml, void *data);
enum TransPropType {
    TRANS_PROP_CHAR,
    TRANS_PROP_UCHAR,
    TRANS_PROP_INT,
    TRANS_PROP_UINT,
    TRANS_PROP_FLOAT,
    TRANS_PROP_DOUBLE
};
ClutterTransition *cdos_transition_to (ClutterActor *actor, 
                                        ClutterAnimationMode mode,
                                        guint duration,
                                        guint repeat_count,
                                        TransCallback callback,
                                        void *data,
                                        const char *name,
                                        const char *first_property_name,
                                        ...) G_GNUC_NULL_TERMINATED;

ClutterTransition *cdos_transition_simple (ClutterActor *actor,
                                           ClutterAnimationMode mode,
                                           guint duration,
                                           const char *first_property_name,
                                           ...) G_GNUC_NULL_TERMINATED;

void cdos_notify_error (const char *msg, const char *details);

#endif // __CDOS_STRAT_H__
