/**
 * SECTION: cdos-summary-item
 * @short_description: 构建通知条目
 * @include: cdos-summary-item.h
 * 
 * CdosSummaryItemClass 类用构建条目和处理某些事件.
 */
#include "cdos-summary-item.h"
#include "cdos-notification.h"
#include "cdos-source.h"
#include "cdos-popup-menu-item.h"
#include "cdos-popup-base-menu-item.h"

#include "st.h"
#include <clutter/clutter.h>

#include "cdos-global.h"
#include "cdos-generic-container.h"


G_DEFINE_TYPE ( CdosSummaryItem, cdos_summary_item, G_TYPE_OBJECT );

typedef struct {
    CdosNotification *notification;
    gulong notificationExpandedId;
    gulong notificationDoneDisplayingId;
    gulong notificationDestroyedId;
} StackNotification;

/* signal */
enum
{
        CONTENT_UPDATE,       //content-updated
        DONE_DISPLAY_CONTENT,      //done-displaying-content
        LAST_SIGNAL
};


static guint cdos_summary_item_signals [LAST_SIGNAL] = { 0 };


static void contentUpdated ( CdosNotification *notification, gpointer user_data ) {

        CdosSummaryItem *self = CDOS_SUMMARY_ITEM( user_data );

	g_signal_emit_by_name ( self, "content-updated");

}


static void notificationDoneDisplaying ( GObject *gobject, gpointer user_data ) {

        CdosSummaryItem *self = CDOS_SUMMARY_ITEM( user_data );

        g_signal_emit_by_name ( self, "done-displaying-content");

}


static void notificationDestroyed ( CdosNotification *notification, gpointer user_data ) {

	CdosSummaryItem *self = CDOS_SUMMARY_ITEM( user_data );

	guint i;
	
	GList *list;

    for ( i = 0 ; i < g_list_length ( self->stackedNotifications ) ; i++ ) {

		StackNotification *stackedNotification = g_list_nth_data ( self->stackedNotifications , i );		

		if ( stackedNotification->notification == notification ) {

			g_signal_handler_disconnect (notification, stackedNotification->notificationExpandedId);
			g_signal_handler_disconnect (notification, stackedNotification->notificationDoneDisplayingId);
			g_signal_handler_disconnect (notification, stackedNotification->notificationDestroyedId);

			//this._stackedNotifications.splice(i, 1);
			self->stackedNotifications = g_list_remove (self->stackedNotifications, notification);

			contentUpdated (notification, self);
			break;
		}
    }

	list = clutter_actor_get_children ( CLUTTER_ACTOR( self->notificationStack ) );

    if ( g_list_length ( list ) > 0 ) {
		
		CdosNotification *notif;

		notif = g_list_nth_data ( list , 0 );

		cdos_notification_set_icon_visible ( notif->actor_delegate, TRUE ); 
	}

}


static void appendNotificationToStack ( CdosSummaryItem *self, CdosNotification *notification ) {

        StackNotification *stackedNotification = g_new0 ( StackNotification, 1 );

        stackedNotification->notification = notification;

        stackedNotification->notificationExpandedId = 
			g_signal_connect ( notification , "expanded", G_CALLBACK ( contentUpdated ), self );

        stackedNotification->notificationDoneDisplayingId = 
			g_signal_connect ( notification , "done-displaying", G_CALLBACK ( notificationDoneDisplaying ), self );

        stackedNotification->notificationDestroyedId = 
			g_signal_connect ( notification , "destroy", G_CALLBACK ( notificationDestroyed ), self );

        //self->stackedNotifications.push(stackedNotification);
	self->stackedNotifications = g_list_append ( self->stackedNotifications, stackedNotification );

        //if (this.notificationStack.get_children().length > 0) 
	if ( g_list_length ( clutter_actor_get_children ( CLUTTER_ACTOR( self->notificationStack ) ) ) > 0 ) {

		cdos_notification_set_icon_visible ( notification, FALSE );
	}

        //this.notificationStack.add(notification.actor);
	clutter_actor_add_child ( CLUTTER_ACTOR ( self->notificationStack ), CLUTTER_ACTOR ( notification->actor ) );

	cdos_notification_expand ( notification , FALSE );

}


static void notificationAddedToSource ( GObject *gobject, gpointer user_data ) {
	
	CdosSummaryItem *self = CDOS_SUMMARY_ITEM( user_data );	
	
	CdosNotification *notification = CDOS_NOTIFICATION ( gobject );

        if ( CLUTTER_ACTOR_IS_MAPPED ( CLUTTER_ACTOR ( self->notificationStack ) ) ) {
 
           	appendNotificationToStack ( self, notification );
	}
}


static void onTitleChanged ( GObject *gobject, gpointer user_data ) {

	CdosSummaryItem *self = CDOS_SUMMARY_ITEM( user_data );

	CdosSource *source = CDOS_SOURCE ( gobject );

	st_label_set_text ( self->sourceTitle, source->title );

}


static void onChanged ( GObject *gobject, gpointer user_data ) {

	CdosSummaryItem *self = CDOS_SUMMARY_ITEM( user_data );

	StAdjustment *adjustment =  ST_ADJUSTMENT ( gobject );
	
	gdouble currentValue, value, lower, upper, page_size;


	st_adjustment_get_values ( adjustment, &value, &lower, &upper, NULL, NULL, &page_size );

	currentValue = value + page_size;

	if ( currentValue == self->oldMaxScrollAdjustment )
		cdos_summary_item_scroll_to ( self, ST_SIDE_BOTTOM );
        	
	self->oldMaxScrollAdjustment = upper;

}


static void onActivateOpen ( GObject *gobject, gpointer user_data ) {

        CdosSummaryItem *self = CDOS_SUMMARY_ITEM( user_data );

	CdosNotification *notification = CDOS_NOTIFICATION ( gobject );

	//可以被复写的open方法
        cdos_source_open ( G_OBJECT (self->source), user_data );

	g_signal_emit_by_name ( self, "done-displaying-content");

}


static void onActivateDestroy ( GObject *gobject, gpointer user_data ) {

        CdosSummaryItem *self = CDOS_SUMMARY_ITEM( user_data );

	cdos_source_destroy ( self->source, 0 ); //reason

 	g_signal_emit_by_name ( self, "done-displaying-content");

}

static void
cdos_summary_item_finalize (GObject *obj)
{
	CdosSummaryItem *self = CDOS_SUMMARY_ITEM (obj);

    G_OBJECT_CLASS(cdos_summary_item_parent_class)->finalize(obj);
}

static void 
cdos_summary_item_class_init ( CdosSummaryItemClass *klass ) {

    GObjectClass *gobject_class = G_OBJECT_CLASS(klass);

    /**
     * CdosSummaryItem::content-update:
     * @object: 绑定信号的对象
     * @user_data: 真对回调函数所传入的参数
     *
     * 该信号对#CdosSummaryItem 的内容更新进行监听
     */
    cdos_summary_item_signals [CONTENT_UPDATE] =
        g_signal_new ("content-updated",
                G_TYPE_FROM_CLASS (klass),
                G_SIGNAL_RUN_LAST,
                0,
                NULL, NULL,
                g_cclosure_marshal_VOID__VOID,
                G_TYPE_NONE, 0 );

    /**
     * CdosSummaryItem::done-displaying-content:
     * @object: 绑定信号的对象
     * @user_data: 真对回调函数所传入的参数
     *
     * 该信号对#CdosSummaryItem 的展示内容进行监听
     */
    cdos_summary_item_signals [DONE_DISPLAY_CONTENT] =
        g_signal_new ("done-displaying-content",
                G_TYPE_FROM_CLASS (klass),
                G_SIGNAL_RUN_LAST,
                0,
                NULL, NULL,
                g_cclosure_marshal_VOID__VOID,
                G_TYPE_NONE, 0 );


    gobject_class->finalize = cdos_summary_item_finalize;
}

static void
cdos_summary_item_init (CdosSummaryItem *self) {

	StScrollBar *vscroll;
	StAdjustment *adjustment;
	CdosPopupMenuItem *item;   
	StFocusManager *focusManager;	
	CdosGlobal *global = cdos_global_get ();

	self->actor = ST_BUTTON (st_button_new ( ));

	st_widget_set_style_class_name (ST_WIDGET (self->actor), "summary-source-button");
	st_bin_set_fill (ST_BIN (self->actor), FALSE, TRUE);
	clutter_actor_set_reactive (CLUTTER_ACTOR (self->actor), TRUE);
	st_button_set_button_mask (self->actor, ST_BUTTON_ONE | ST_BUTTON_TWO | ST_BUTTON_THREE);
	st_widget_set_track_hover (ST_WIDGET (self->actor), TRUE );


	self->sourceBox = ST_BOX_LAYOUT ( st_box_layout_new () );

	st_widget_set_style_class_name ( ST_WIDGET( self->sourceBox ), "summary-source" );

	self->sourceIcon = ST_BIN ( cdos_source_get_summary_icon ( self->source ) );

	
	self->sourceTitleBin = ST_BIN (st_bin_new ());
    g_object_set (self->sourceTitleBin, "y-fill", TRUE, "y-align", ST_ALIGN_MIDDLE, NULL);
	clutter_actor_set_clip_to_allocation ( CLUTTER_ACTOR ( self->sourceTitleBin ), TRUE );

	self->sourceTitle = ST_LABEL ( st_label_new ( self->source->title ) );

	st_widget_set_style_class_name ( ST_WIDGET( self->sourceTitle ), "source-title" );

	clutter_text_set_ellipsize ( CLUTTER_TEXT( self->sourceTitle ) , PANGO_ELLIPSIZE_NONE );

	st_bin_set_child ( self->sourceTitleBin , CLUTTER_ACTOR (self->sourceTitle));

	clutter_actor_set_width (CLUTTER_ACTOR (self->sourceTitleBin), 0);

    g_object_set (self->sourceIcon, "y-fill", TRUE, NULL);
    st_box_layout_insert_actor (self->sourceBox, CLUTTER_ACTOR (self->sourceIcon), -1);

    g_object_set (self->sourceTitleBin, "y-fill", FALSE, NULL);
    g_object_set (CLUTTER_ACTOR (self->sourceTitleBin), "x-expand", TRUE, "y-expand", NULL);
    st_box_layout_insert_actor (self->sourceBox, CLUTTER_ACTOR (self->sourceTitleBin), -1);

    st_bin_set_child (ST_BIN (self->actor), CLUTTER_ACTOR (self->sourceBox));

	/*
        this.notificationStackView = new St.ScrollView({ name: source.isChat ? '' : 'summary-notification-stack-scrollview',
                                                         vscrollbar_policy: source.isChat ? Gtk.PolicyType.NEVER : Gtk.PolicyType.AUTOMATIC,
                                                         hscrollbar_policy: Gtk.PolicyType.NEVER,
                                                         style_class: 'vfade' });
	*/	
	self->notificationStackView = ST_SCROLL_VIEW ( st_scroll_view_new () );
	clutter_actor_set_name ( CLUTTER_ACTOR( self->notificationStackView ), self->source->isChat ? NULL : "summary-notification-stack-scrollview" );
	st_scroll_view_set_policy ( self->notificationStackView, self->source->isChat ? GTK_POLICY_NEVER : GTK_POLICY_AUTOMATIC,
					GTK_POLICY_NEVER );
	st_widget_set_style_class_name ( ST_WIDGET( self->notificationStackView ), "vfade" );	


	self->notificationStack = ST_BOX_LAYOUT ( st_box_layout_new () );
	clutter_actor_set_name ( CLUTTER_ACTOR( self->notificationStack ), "summary-notification-stack" );
	st_box_layout_set_vertical ( self->notificationStack, TRUE );

	clutter_container_add_actor ( CLUTTER_CONTAINER ( self->notificationStackView ), CLUTTER_ACTOR ( self->notificationStack ) );

	self->stackedNotifications = NULL;

	self->oldMaxScrollAdjustment = 0;

    vscroll = ST_SCROLL_BAR ( st_scroll_view_get_vscroll_bar ( self->notificationStackView ) );

	adjustment = st_scroll_bar_get_adjustment (vscroll);

	g_signal_connect ( adjustment , "changed", G_CALLBACK ( onChanged ), self );

	self->rightClickMenu = ST_BOX_LAYOUT ( st_box_layout_new () );

    clutter_actor_set_name ( CLUTTER_ACTOR( self->rightClickMenu ), "summary-right-click-menu" );
    st_box_layout_set_vertical ( self->rightClickMenu, TRUE );

	item = cdos_popup_menu_item_new_with_params ("Open", cdos_popup_base_menu_item_new ());

	g_signal_connect ( item , "activate", G_CALLBACK ( onActivateOpen ), self );

    st_box_layout_insert_actor (self->rightClickMenu, CLUTTER_ACTOR (cdos_popup_base_menu_item_get_actor (item)), -1);

	item = cdos_popup_menu_item_new_with_params ("Remove",  cdos_popup_base_menu_item_new ());

	g_signal_connect ( item , "activate", G_CALLBACK ( onActivateDestroy ), self );

    st_box_layout_insert_actor (self->rightClickMenu, CLUTTER_ACTOR (cdos_popup_base_menu_item_get_actor (item)), -1);

	focusManager = st_focus_manager_get_for_stage ( cdos_global_get_stage (global) );

	st_focus_manager_add_group ( focusManager , ST_WIDGET( self->rightClickMenu ) );
}



/*公有函数定义*/

/**
 * cdos_summary_item_scroll_to:
 * @self: A #CdosSummaryItem
 * @side: 滑动的方向 
 *
 * 滚动条滑动展示信息
 *
 */
void cdos_summary_item_scroll_to ( CdosSummaryItem *self, guint side ) {

	StScrollBar *vscroll = ST_SCROLL_BAR ( st_scroll_view_get_vscroll_bar ( self->notificationStackView ) );

	StAdjustment *adjustment = st_scroll_bar_get_adjustment (vscroll);

        gdouble value, lower, upper, page_size;


        st_adjustment_get_values ( adjustment, &value, &lower, &upper, NULL, NULL, &page_size );

        if ( side ==  ST_SIDE_TOP )
                st_adjustment_set_value ( adjustment , lower );

        else if ( side ==  ST_SIDE_BOTTOM )
                st_adjustment_set_value ( adjustment , upper );

}

/**
 * cdos_summary_item_new:
 * @source: A #CdosSource  
 * 
 * 新建一个 #CdosSource 类的实例。
 *
 * Returns: 返回一个 #CdosSource 的实例。
 */
CdosSummaryItem *cdos_summary_item_new ( CdosSource *source ) {

    CdosSummaryItem *instance = g_object_new ( CDOS_TYPE_SUMMARY_ITEM, NULL );

    instance->source = source;

    g_signal_connect (instance->source, "notification-added", G_CALLBACK (notificationAddedToSource), instance);

    g_signal_connect (instance->source , "title-changed", G_CALLBACK (onTitleChanged), instance );

    return instance;
}










