/**
 * SECTION: cdos-system-notification-source
 * @short_description: 系统消息资源
 * @include: cdos-system-notification-source.h
 * 
 * CdosSystemNotificationSourceClass 类用来新建 #CdosSource,
 * 用于构建 #CdosMessageTray .
 */
#include "cdos-system-notification-source.h"
#include "cdos.h"

G_DEFINE_TYPE ( CdosSystemNotificationSource, cdos_system_notification_source, CDOS_TYPE_SOURCE);

static void
cdos_system_notification_source_finalize (GObject *obj)
{
	CdosSystemNotificationSource *self = CDOS_SYSTEM_NOTIFICATION_SOURCE (obj);

    G_OBJECT_CLASS(cdos_system_notification_source_parent_class)->finalize(obj);
}

static void 
cdos_system_notification_source_class_init ( CdosSystemNotificationSourceClass *klass ) {
	CdosSourceClass *cdos_source_class = CDOS_SOURCE (klass);
	cdos_source_class->open = cdos_system_notification_source_open;

	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
	gobject_class->finalize = cdos_system_notification_source_finalize;
}



static void
cdos_system_notification_source_init ( CdosSystemNotificationSource *self ) {
	StIcon *icon = NULL;
	cdos_source_set_title (CDOS_SOURCE (self), _("System Information"));
	icon = cdos_system_notification_source_create_notification_icon ();
	cdos_source_set_summary_icon (CDOS_SOURCE (self), icon );
}



/*公有函数定义*/

/**
 * cdos_system_notification_source_create_notification_icon:
 *
 * 获取图标为”dialog-information“的 #StIcon
 * 
 * Returns (StIcon): 返回新建的icon。
 */
StIcon *cdos_system_notification_source_create_notification_icon (void) {

	StIcon *icon = ST_ICON( st_icon_new ( ) );
	
	st_icon_set_icon_name ( icon, "dialog-information" );
	st_icon_set_icon_type ( icon, ST_ICON_SYMBOLIC );
	st_icon_set_icon_size ( icon, SOURCE_ICON_SIZE );
		
	return icon;
}

/**
 * cdos_system_notification_source_open:
 * @self: A #CdosSystemNotificationSource
 * 
 * 调用父类 #CdosSource 的方法，删除self
 *
 */
void cdos_system_notification_source_open ( CdosSystemNotificationSource *self ) {

	cdos_source_destroy (CDOS_SOURCE (self), 0 ); //reason
}

/**
 * cdos_system_notification_source_new:
 *
 * 新建一个 #CdosSystemNotificationSource 类的实例,以父类 #CdosSource 类型返回。
 *
 * Returns (CdosSource): 返回一个 #CdosSystemNotificationSource 类的实例。
 */
CdosSource* cdos_system_notification_source_new (void)
{
	CdosSystemNotificationSource *self = g_object_new (CDOS_TYPE_SYSTEM_NOTIFICATION_SOURCE, NULL);
	return CDOS_SOURCE (self);
}