#include "cdos-test-button.h"

G_DEFINE_TYPE (UiButton, ui_button, CLUTTER_TYPE_ACTOR);

#define UI_BUTTON_GET_PRIVATE(obj) \
  (G_TYPE_INSTANCE_GET_PRIVATE ((obj), UI_TYPE_BUTTON, UiButtonPrivate))

struct _UiButtonPrivate
{
  ClutterActor  *child;
  ClutterActor  *label;
  ClutterAction *click_action;
  ClutterAction *drag_action;
  gchar         *text;
};

enum {
  PROP_0,
  PROP_TEXT
};

enum {
  CLICKED,
  DRAG_BEGIN,
  DRAG_END,
  DRAG_MOTION,
  LAST_SIGNAL
};

static guint ui_button_signals[LAST_SIGNAL] = { 0, };

static void
ui_button_finalize (GObject *gobject)
{
  UiButtonPrivate *priv = UI_BUTTON (gobject)->priv;

  g_free (priv->text);

  G_OBJECT_CLASS (ui_button_parent_class)->finalize (gobject);
}

static void
ui_button_set_property (GObject      *gobject,
                        guint         prop_id,
                        const GValue *value,
                        GParamSpec   *pspec)
{
  UiButton *button = UI_BUTTON (gobject);

  switch (prop_id)
    {
    case PROP_TEXT:
      ui_button_set_text (button, g_value_get_string (value));
      break;

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
      break;
    }
}

static void
ui_button_get_property (GObject    *gobject,
                        guint       prop_id,
                        GValue     *value,
                        GParamSpec *pspec)
{
  UiButtonPrivate *priv = UI_BUTTON (gobject)->priv;

  switch (prop_id)
    {
    case PROP_TEXT:
      g_value_set_string (value, priv->text);
      break;

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
      break;
    }
}

static void
ui_button_destroy (ClutterActor *self)
{
  UiButtonPrivate *priv = UI_BUTTON (self)->priv;

  if (priv->child)
    {
      clutter_actor_destroy (priv->child);
      priv->child = NULL;
    }

  if (CLUTTER_ACTOR_CLASS (ui_button_parent_class)->destroy)
    CLUTTER_ACTOR_CLASS (ui_button_parent_class)->destroy (self);
}

static void
ui_button_get_preferred_height (ClutterActor *self,
                                gfloat for_width,
                                gfloat *min_height_p,
                                gfloat *natural_height_p)
{
  UiButtonPrivate *priv = UI_BUTTON (self)->priv;

  clutter_actor_get_preferred_height (priv->child,
                                      for_width,
                                      min_height_p,
                                      natural_height_p);

  *min_height_p += 20.0;
  *natural_height_p += 20.0;
}

static void
ui_button_get_preferred_width (ClutterActor *self,
                               gfloat for_height,
                               gfloat *min_width_p,
                               gfloat *natural_width_p)
{
  UiButtonPrivate *priv = UI_BUTTON (self)->priv;

  clutter_actor_get_preferred_width (priv->child,
                                     for_height,
                                     min_width_p,
                                     natural_width_p);

  *min_width_p += 20.0;
  *natural_width_p += 20.0;
}

static void
ui_button_allocate (ClutterActor          *actor,
                    const ClutterActorBox *box,
                    ClutterAllocationFlags flags)
{
  UiButtonPrivate *priv = UI_BUTTON (actor)->priv;
  ClutterActorBox child_box = { 0, };

  CLUTTER_ACTOR_CLASS (ui_button_parent_class)->allocate (actor, box, flags);

  child_box.x1 = 0.0;
  child_box.y1 = 0.0;
  child_box.x2 = clutter_actor_box_get_width (box);
  child_box.y2 = clutter_actor_box_get_height (box);

  clutter_actor_allocate (priv->child, &child_box, flags);
}

static void
ui_button_paint (ClutterActor *actor)
{
  UiButtonPrivate *priv = UI_BUTTON (actor)->priv;

  clutter_actor_paint (priv->child);
}

static void
ui_button_clicked (ClutterClickAction *action,
                   ClutterActor       *actor,
                   gpointer            user_data)
{
  g_signal_emit (actor, ui_button_signals[CLICKED], 0);
}

static void
ui_button_drag_begin (ClutterDragAction *action,
                   ClutterActor       *actor,
                   gpointer            user_data)
{
  // g_print ("%s\n", __func__);
  g_signal_emit (actor, ui_button_signals[DRAG_BEGIN], 0);
  clutter_drag_action_set_drag_handle (action, actor);
}

static void
ui_button_drag_end (ClutterDragAction *action,
                    ClutterActor *actor,
                    gpointer user_data)
{
  // g_print ("%s\n", __func__);
  ClutterActor *drag_handle;

  drag_handle = clutter_drag_action_get_drag_handle (action);
  g_signal_emit (drag_handle, ui_button_signals[DRAG_END], 0);
}

static void 
ui_button_drag_motion (ClutterDragAction *action,
                       ClutterActor *actor,
                       gfloat delta_x,
                       gfloat delta_y,
                       gpointer user_data)
{
  // g_print ("%s\n", __func__);
  g_signal_emit (actor, ui_button_signals[DRAG_MOTION], 0);
}

static void
ui_button_class_init (UiButtonClass *klass)
{
  ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
  GParamSpec *pspec;

  gobject_class->finalize = ui_button_finalize;
  gobject_class->set_property = ui_button_set_property;
  gobject_class->get_property = ui_button_get_property;

  actor_class->destroy = ui_button_destroy;
  actor_class->get_preferred_height = ui_button_get_preferred_height;
  actor_class->get_preferred_width = ui_button_get_preferred_width;
  actor_class->allocate = ui_button_allocate;
  actor_class->paint = ui_button_paint;

  g_type_class_add_private (klass, sizeof (UiButtonPrivate));

  pspec = g_param_spec_string ("text",
                               "Text",
                               "Text of the button",
                               NULL,
                               G_PARAM_READWRITE);
  g_object_class_install_property (gobject_class, PROP_TEXT, pspec);

  ui_button_signals[CLICKED] =
    g_signal_new ("clicked",
                  G_TYPE_FROM_CLASS (klass),
                  G_SIGNAL_RUN_LAST,
                  G_STRUCT_OFFSET (UiButtonClass, clicked),
                  NULL,
                  NULL,
                  g_cclosure_marshal_VOID__VOID,
                  G_TYPE_NONE,
                  0);

  ui_button_signals[DRAG_BEGIN] =
    g_signal_new ("drag-begin",
                  G_TYPE_FROM_CLASS (klass),
                  G_SIGNAL_RUN_LAST,
                  G_STRUCT_OFFSET (UiButtonClass, drag),
                  NULL,
                  NULL,
                  g_cclosure_marshal_VOID__VOID,
                  G_TYPE_NONE,
                  0);

  ui_button_signals[DRAG_END] =
    g_signal_new ("drag-end",
                  G_TYPE_FROM_CLASS (klass),
                  G_SIGNAL_RUN_LAST,
                  G_STRUCT_OFFSET (UiButtonClass, drag),
                  NULL,
                  NULL,
                  g_cclosure_marshal_VOID__VOID,
                  G_TYPE_NONE,
                  0);

  ui_button_signals[DRAG_MOTION] = 
    g_signal_new ("drag-motion",
                  G_TYPE_FROM_CLASS (klass),
                  G_SIGNAL_RUN_LAST,
                  G_STRUCT_OFFSET (UiButtonClass, drag),
                  NULL,
                  NULL,
                  g_cclosure_marshal_VOID__VOID,
                  G_TYPE_NONE,
                  0);
}

static void
ui_button_init (UiButton *self)
{
  UiButtonPrivate *priv;
  ClutterLayoutManager *layout;

  priv = self->priv = UI_BUTTON_GET_PRIVATE (self);

  clutter_actor_set_reactive (CLUTTER_ACTOR (self), TRUE);

  layout = clutter_bin_layout_new (CLUTTER_BIN_ALIGNMENT_CENTER,
                                   CLUTTER_BIN_ALIGNMENT_CENTER);

  priv->child = clutter_actor_new ();
  clutter_actor_set_layout_manager (priv->child, layout);

  clutter_actor_add_child (CLUTTER_ACTOR (self), priv->child);

  priv->label = g_object_new (CLUTTER_TYPE_TEXT,
                              "line-alignment", PANGO_ALIGN_CENTER,
                              "ellipsize", PANGO_ELLIPSIZE_END,
                              NULL);

  clutter_actor_add_child (priv->child, priv->label);

  priv->click_action = clutter_click_action_new ();
  clutter_actor_add_action (CLUTTER_ACTOR (self), priv->click_action);

  g_signal_connect (priv->click_action,
                    "clicked",
                    G_CALLBACK (ui_button_clicked),
                    NULL);

  priv->drag_action = clutter_drag_action_new ();
  clutter_actor_add_action (CLUTTER_ACTOR (self), priv->drag_action);
  clutter_actor_set_reactive (CLUTTER_ACTOR (self), TRUE);
    
  g_signal_connect (priv->drag_action,
                    "drag-begin",
                    G_CALLBACK (ui_button_drag_begin),
                    NULL);
  g_signal_connect (priv->drag_action,
                    "drag-end",
                    G_CALLBACK (ui_button_drag_end),
                    NULL);
  g_signal_connect (priv->drag_action,
                    "drag-motion",
                    G_CALLBACK (ui_button_drag_motion),
                    NULL);
}

void
ui_button_set_text (UiButton    *self,
                    const gchar *text)
{
  UiButtonPrivate *priv;

  g_return_if_fail (UI_IS_BUTTON (self));

  priv = self->priv;

  g_free (priv->text);

  if (text)
    priv->text = g_strdup (text);
  else
    priv->text = g_strdup ("");

  clutter_text_set_text (CLUTTER_TEXT (priv->label), priv->text);
}

void
ui_button_set_background_color (UiButton           *self,
                                const ClutterColor *color)
{
  g_return_if_fail (UI_IS_BUTTON (self));

  clutter_actor_set_background_color (self->priv->child, color);
}

void
ui_button_set_text_color (UiButton           *self,
                          const ClutterColor *color)
{
  g_return_if_fail (UI_IS_BUTTON (self));

  clutter_text_set_color (CLUTTER_TEXT (self->priv->label), color);
}

const gchar *
ui_button_get_text (UiButton *self)
{
  g_return_val_if_fail (UI_IS_BUTTON (self), NULL);

  return self->priv->text;
}

ClutterActor *
ui_button_new (void)
{
  return g_object_new (UI_TYPE_BUTTON, NULL);
}
