#include "cdos-test-dnd.h"
#include "cdos.h"
#include "cdos-start.h"

typedef struct _DNDTest {
    ClutterActor *actor;
}DNDTest;

static ClutterActor *
get_drag_actor (DNDTest *self)
{
    cdos_debug ("self: %p, actor: %p\n", self, self->actor);
    return clutter_clone_new (self->actor);
}

static ClutterActor *
get_drag_actor_source (DNDTest *self)
{
    cdos_debug ("%p\n", self);
    return self->actor;
}

DragMotionResult
handle_drag_over (DNDTest      *self,
                  void         *source,/* target delegate */
                  ClutterActor *actor,
                  float         x,
                  float         y,
                  guint32       time)
{
    /* 显示不同的光标样式
    DRAG_MOTION_NO_DROP,
    DRAG_MOTION_COPY_DROP, 
    DRAG_MOTION_MOVE_DROP,
    DRAG_MOTION_CONTINUE
    */
    cdos_debug ("%p %p\n", source, actor);
    return DRAG_MOTION_COPY_DROP;
}


static gboolean
accept_drop (DNDTest      *self,
             void         *source, /* target delegate */
             ClutterActor *actor,
             float         x,
             float         y,
             guint32       time)
{
    cdos_debug ("self:%p, self.actor: %p, source:%p\n", self, self->actor, source);
    /* cdos-2.2.13 panel-launcher applet.js 的 acceptDrop() 有 bug。
     * 如果返回值 false, 不要在这个函数里 destroy actor.
     * 即使返回 true, 也不需要 destroy actor.
     */
    //clutter_actor_destroy (actor);
    StContainer *container = (StContainer *)clutter_actor_get_parent (self->actor);
    ClutterActor *src = cdos_draggable_get_delegate_actor (source);
    ClutterActor *p2 = clutter_actor_get_parent (src);
    if ((void *)container == p2)
        /* 交换 A B 的 位置，
         * 或者用 st_container_move_child()移动 A 到 B 前或后面.
         */
        st_container_swap_child (container, self->actor, src);
    else 
        cdos_debug (" parent of %p and %p is different.\n", self->actor, src);
    return true;
}

static void 
on_drag_begin (gpointer drag, guint32 time)
{
    cdos_debug ("%d %p\n", time, drag);
    cdos_sound_manager_play ("map");
}
static void 
on_drag_end (gpointer drag, guint32 time, guint snap)
{
    cdos_debug ("%d %d %p\n", time, snap, drag);
    cdos_sound_manager_play ("logout");
}

static void 
on_drag_cancelled(gpointer drag, guint32 time)
{
    cdos_debug ("%d %p\n", time, drag);
}

static DNDTest * 
cdos_draggable_test (const char *name)
{
    DNDTest *self = g_new0 (DNDTest, 1);

    ClutterActor *icon = st_icon_new ();
    StWidget *box = st_box_layout_new ();
    st_container_add_actor((ClutterContainer *)box, icon);

    self->actor = (ClutterActor *)box;

    st_icon_set_icon_name ((StIcon *)icon, name);
    st_icon_set_icon_type ((StIcon *)icon, ST_ICON_FULLCOLOR);
    st_icon_set_icon_size ((StIcon *)icon, 32);
    ClutterActor *panel = PLUGIN_PANEL;
    ClutterActor *right = (ClutterActor *)cdos_panel_get_rightbox ((CdosPanel *)panel);

    st_container_add_actor ((ClutterContainer *)right, (ClutterActor *)box);

    clutter_actor_set_reactive ((ClutterActor *)box, 1);
    CdosDraggable *drag = cdos_make_draggable ((ClutterActor *)box, NULL, 3, 0, 0, 50);

    cdos_draggable_set_delegate ((ClutterActor *)box, self);
    cdos_draggable_set_func ((ClutterActor *)box, get_drag_actor, DND_GET_DRAG_ACTOR);
    cdos_draggable_set_func ((ClutterActor *)box, get_drag_actor_source, DND_GET_DRAG_ACTOR_SRC);
    cdos_draggable_set_func ((ClutterActor *)box, handle_drag_over, DND_HANDLE_DRAG_OVER);
    cdos_draggable_set_func ((ClutterActor *)box, accept_drop, DND_ACCEPT_DROP);

    g_signal_connect (drag, "drag-begin", G_CALLBACK(on_drag_begin), NULL);
    g_signal_connect (drag, "drag-end", G_CALLBACK(on_drag_end), NULL);
    g_signal_connect (drag, "drag-cancelled", G_CALLBACK(on_drag_cancelled), NULL);
    return self;
}

void cdos_test_dnd ()
{
    DNDTest *test = cdos_draggable_test ("inkscape");
    DNDTest *test2 G_GNUC_UNUSED = cdos_draggable_test ("gvim");
    void *delegate = cdos_draggable_get_delegate (test->actor);
    void *get_drag_actor = cdos_draggable_get_func (test->actor, DND_GET_DRAG_ACTOR);
    void *get_drag_actor_source G_GNUC_UNUSED = 
        cdos_draggable_get_func(test->actor, DND_GET_DRAG_ACTOR_SRC);

    AcceptDrop accept_drop = cdos_draggable_get_func (test->actor, DND_ACCEPT_DROP);

    HandleDragOver handle = cdos_draggable_get_func (test->actor, DND_HANDLE_DRAG_OVER);
    cdos_debug ("actor: %p\ndelegate: %p\n"
                    "get_drag_actor: %p\n"
                    "handle_drag_over: %p\n"
                    "accept_drop: %p\n",
                    test->actor, delegate, get_drag_actor, handle, accept_drop);
}
