#include <stdlib.h>
#include "cdos-test-button.h"
#include "cdos-start.h"
#include "cdos-monkey-plugin.h"
#include "cdos-applet-manager.h"

/* colors */
static const ClutterColor stage_color = { 0x33, 0x33, 0x55, 0xff };
static const ClutterColor white_color = { 0xff, 0xff, 0xff, 0xff };
static const ClutterColor pink_color = { 0xff, 0x88, 0x88, 0x99 };

/* click handler */
static void
test_button_clicked (UiButton *button,
         gpointer  data)
{
  const gchar *current_text;

  g_debug ("Clicked");

  current_text = ui_button_get_text (button);

  printf ("[%s] current_text: %s\n", __func__, current_text);

  if (g_strcmp0 (current_text, "hello") == 0)
    ui_button_set_text (button, "world");
  else
    ui_button_set_text (button, "hello");
}

/* drag handler */
static void
test_button_drag_begin (UiButton *button)
{
  printf ("[%s] This is a test info, drag begin...\n", __func__);
}

static void
test_button_drag_motion (UiButton *button)
{
  printf ("[%s] This is a test info, drag motion...\n", __func__);
}

static void
test_button_drag_end (UiButton *button)
{
  printf("[%s] This is a test info, drag end...\n", __func__);
}

ClutterActor *
test_button_new_public (void)
{
  ClutterActor *button = ui_button_new ();
  ui_button_set_text (UI_BUTTON (button), "hello");

  g_signal_connect (button, "clicked", G_CALLBACK (test_button_clicked), NULL);
  g_signal_connect (button, "drag-begin", G_CALLBACK (test_button_drag_begin), NULL);
  g_signal_connect (button, "drag-motion", G_CALLBACK (test_button_drag_motion), NULL);
  g_signal_connect (button, "drag-end", G_CALLBACK (test_button_drag_end), NULL);

  return button;
}

static void
test_widget_button (ClutterActor *stage)
{
  ClutterActor *button;
  ClutterConstraint *align_x_constraint;
  ClutterConstraint *align_y_constraint;

  button = ui_button_new ();
  ui_button_set_text (UI_BUTTON (button), "hello");

  ui_button_set_text_color (UI_BUTTON (button), &white_color);
  ui_button_set_background_color (UI_BUTTON (button), &pink_color);
    
  g_signal_connect (button, "clicked", G_CALLBACK (test_button_clicked), NULL);
  g_signal_connect (button, "drag-begin", G_CALLBACK (test_button_drag_begin), NULL);
  g_signal_connect (button, "drag-motion", G_CALLBACK (test_button_drag_motion), NULL);
  g_signal_connect (button, "drag-end", G_CALLBACK (test_button_drag_end), NULL);

  align_x_constraint = clutter_align_constraint_new (stage,
                                                     CLUTTER_ALIGN_X_AXIS,
                                                     0.95);

  align_y_constraint = clutter_align_constraint_new (stage,
                                                     CLUTTER_ALIGN_Y_AXIS,
                                                     0.95);

  clutter_actor_add_constraint (button, align_x_constraint);
  clutter_actor_add_constraint (button, align_y_constraint);

  clutter_actor_add_child (stage, button);
}

static gboolean
test_set_stage_input_region_test (void)
{
    MonkeyPlugin *plugin;
    MetaRectangle rect;
    GSList *slist = NULL;
    
    plugin = monkey_plugin_get ();

    if (NULL == plugin)
        return FALSE;

    // MetaScreen *screen = cdos_global_get_screen (cdos_global_get ());
    // meta_screen_get_monitor_geometry (screen, 0, &rect);

    rect.x = monkey_plugin_get_screen_width (plugin) - 500;
    rect.y = monkey_plugin_get_screen_height (plugin) -300;
    rect.width = 500;
    rect.height = 300;
    slist = g_slist_append (slist, &rect);
    cdos_global_set_stage_input_region (cdos_global_get (), slist);

    return TRUE;
}

void 
ui_test_func (ClutterActor *stage)
{
    g_print ("%s, %s [test]\n", __FILE__, __func__);
    if (NULL == stage)
        return;

    /* set stage input region */
    if (!test_set_stage_input_region_test ())
        return;

    test_widget_button (stage);

    // cdos_applet_manager_init ();
}
