#include "cdos-text-applet.h"


G_DEFINE_TYPE (CdosTextApplet, cdos_text_applet, CDOS_TYPE_APPLET);

static void
cdos_text_applet_on_applet_added_to_panel (CdosApplet *applet, gboolean applets_loaded)
{
  //g_print ("%s\n", __func__);
}

static void
cdos_text_applet_dispose (GObject *gobject)
{
    CdosTextApplet *self = CDOS_TEXT_APPLET(gobject);
    StBoxLayout *applet_actor = CDOS_APPLET (self)->actor;
    clutter_container_remove_actor (CLUTTER_CONTAINER(applet_actor),CLUTTER_ACTOR(self->applet_label));
    g_clear_object(&self->applet_label);

    G_OBJECT_CLASS(cdos_text_applet_parent_class)->dispose(gobject);
}

static void
cdos_text_applet_finalize (GObject *gobject)
{

    G_OBJECT_CLASS(cdos_text_applet_parent_class)->finalize(gobject);
}

static void
cdos_text_applet_init (CdosTextApplet *self)
{
    //g_print ("%s\n", __func__);
    StLabel *applet_label = g_object_new (ST_TYPE_LABEL,
                "reactive", TRUE,
                "track-hover", TRUE,
                "style-class", "applet-label",
                NULL);
    self->applet_label = applet_label;
    ClutterActor *clutter_text = st_label_get_clutter_text (applet_label);
    clutter_text_set_ellipsize (CLUTTER_TEXT(clutter_text), PANGO_ELLIPSIZE_NONE);
    StBoxLayout *applet_actor = CDOS_APPLET (self)->actor;
    clutter_container_add_actor (CLUTTER_CONTAINER(applet_actor), CLUTTER_ACTOR(applet_label));
    StBoxLayoutChild *child_meta = (StBoxLayoutChild *)clutter_container_get_child_meta (
            CLUTTER_CONTAINER (applet_actor), (ClutterActor *)applet_label);
    child_meta->y_align = ST_ALIGN_MIDDLE;
    child_meta->y_fill = FALSE;
}

static void
cdos_text_applet_class_init (CdosTextAppletClass *klass)
{
  //g_print ("%s\n", __func__);
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  object_class->finalize = cdos_text_applet_finalize;
  object_class->dispose = cdos_text_applet_dispose;

  CdosAppletClass *parent_class = CDOS_APPLET_CLASS (klass);
  parent_class->on_applet_added_to_panel = cdos_text_applet_on_applet_added_to_panel;

}


/**
 * cdos_text_applet_set_applet_label:
 * @text_applet: A #CdosTextApplet instance
 * @text: A string 
 *
 * 设置text_applet的label
 *
 */
void
cdos_text_applet_set_applet_label (CdosTextApplet *text_applet, gchar *text)
{
  g_return_if_fail (CDOS_IS_TEXT_APPLET (text_applet));
  st_label_set_text (text_applet->applet_label, text);
}

/**
 * cdos_text_applet_new:
 * @orientation: orientation
 * @panel_height: panel height
 *
 * 新建一个 #CdosTextApplet 类的实例。
 *
 * Returns: 返回一个 #CdosTextApplet 的实例。
 */
CdosTextApplet *
cdos_text_applet_new (guint orientation, gfloat panel_height, gchar *instance_id)
{
  //g_print ("%s\n", __func__);

  CdosTextApplet *text_applet = g_object_new (CDOS_TYPE_TEXT_APPLET,
				"orientation", orientation,
				"panel-height", panel_height,
                                "instance-id", instance_id,
				NULL);
 
  /* //g_print ("[%s,test] panel-height: %f, instance_id: %s\n", __func__, 
		cdos_applet_get_panel_height (CDOS_APPLET (text_applet)),
		cdos_applet_get_instance_id (CDOS_APPLET (text_applet)));
   */
  return text_applet;
}
