#include "cdos-layout-manager.h"
#include "cdos-monkey-plugin.h"
#include "cdos-tooltip.h"

/**
 * SECTION:cdos-tooltip
 * @title: CdosTooltip
 * @short_description: 用于显示应用程序的描述信息
 * 
 * CdosTooltip 当鼠标hover到应用程序时，弹出含有该应用描述信息的小框，这个小框就是tooltip。
 */

struct _CdosTooltipPrivate {
	gboolean prevent_show;
};

enum {
    PROP_0,
    PROP_ITEM
};

G_DEFINE_TYPE ( CdosTooltip, cdos_tooltip, ST_TYPE_LABEL);


#define CDOS_TOOLTIP_GET_PRIVATE( obj ) \
  (G_TYPE_INSTANCE_GET_PRIVATE( (obj), CDOS_TYPE_TOOLTIP, CdosTooltipPrivate ))
static gboolean cdos_tooltip_item_on_enter (ClutterActor *gobject, ClutterEvent *event, gpointer user_data) {
	CdosTooltip *self = CDOS_TOOLTIP (user_data);

	//Tweener.addTween(this, {time: 0.3, onComplete: Lang.bind(this, this._onTimerComplete)});
	clutter_event_get_coords (event, &self->mousePositionX , &self->mousePositionY );
	if(st_label_get_text((StLabel *)self)){
		cdos_tooltip_show(self);
	}
	return FALSE;
}

static gboolean cdos_tooltip_item_on_leave (ClutterActor *gobject, ClutterEvent *event, gpointer user_data) {
	CdosTooltip *self = CDOS_TOOLTIP (user_data);
	cdos_tooltip_hide (self);
	return FALSE;
}

static gboolean cdos_tooltip_item_on_motion (ClutterActor *gobject, ClutterEvent *event, gpointer user_data) {
	CdosTooltip *self = CDOS_TOOLTIP (user_data);
	//Tweener.removeTweens(this);
	if (!self->visible){
		if(st_label_get_text((StLabel *)self)){
			cdos_tooltip_show(self);
		}
		//Tweener.addTween(this, {time: 0.3, onComplete: Lang.bind(this, this._onTimerComplete)});
		clutter_event_get_coords (event, &self->mousePositionX , &self->mousePositionY );
	}
	return FALSE;
}

static gboolean cdos_tooltip_item_on_button_release (ClutterActor *gobject, ClutterEvent *event, gpointer user_data) {

	CdosTooltip *self = CDOS_TOOLTIP (user_data);
	cdos_tooltip_hide (self);
	return FALSE;
}

static gboolean idle_cb (gpointer user_data) {
	CdosTooltip *self = CDOS_TOOLTIP (user_data);
	cdos_tooltip_hide (self);
	return FALSE;
}

static void cdos_tooltip_item_on_allocation_changed (ClutterActor *gobject,ClutterActorBox *box, ClutterAllocationFlags flags,gpointer user_data) {

	CdosTooltip *self = CDOS_TOOLTIP (user_data);
	// An allocation change could mean that the actor has moved,
	// so hide, but wait until after the allocation cycle.
	g_idle_add ((GSourceFunc) idle_cb, self);
}

static gboolean cdos_tooltip_item_on_button_press (ClutterActor *gobject,ClutterEvent *event,gpointer user_data) {

	//CdosTooltip *self = CDOS_TOOLTIP (user_data);
	return FALSE;
}

static void cdos_tooltip_item_on_destroy (ClutterActor *gobject, gpointer user_data) {

	CdosTooltip *self = CDOS_TOOLTIP (user_data);
	//clutter_actor_destroy ((ClutterActor *)self);
	g_object_run_dispose(G_OBJECT(self));
}
//固定的函数

static void cdos_tooltip_finalize (GObject *object) {
	//CdosTooltip *self = CDOS_TOOLTIP(object);
	//CdosTooltipPrivate *priv = self->priv;
	G_OBJECT_CLASS(cdos_tooltip_parent_class)->finalize(object);
}

static void cdos_tooltip_dispose (GObject *object) {
	CdosTooltip *self = CDOS_TOOLTIP(object);
	//CdosTooltipPrivate *priv = self->priv;
	cdos_tooltip_hide(self);

	G_OBJECT_CLASS(cdos_tooltip_parent_class)->dispose(object);
}

static void 
set_item (CdosTooltip *self, ClutterActor *item)
{
    self->item = item;
	g_signal_connect (item, "enter-event", G_CALLBACK (cdos_tooltip_item_on_enter), self);
	g_signal_connect (item, "leave-event", G_CALLBACK (cdos_tooltip_item_on_leave), self);
	g_signal_connect (item, "motion-event", G_CALLBACK (cdos_tooltip_item_on_motion), self);
	g_signal_connect (item, "button-press-event", G_CALLBACK (cdos_tooltip_item_on_button_press), self);
	g_signal_connect (item, "button-release-event", G_CALLBACK (cdos_tooltip_item_on_button_release), self);
	g_signal_connect (item, "destroy", G_CALLBACK (cdos_tooltip_item_on_destroy), self);
	g_signal_connect (item, "allocation-changed", G_CALLBACK (cdos_tooltip_item_on_allocation_changed), self);
}

static void
cdos_tooltip_set_property(GObject *object, guint prop_id, const GValue *value, GParamSpec  *pspec)
{
    CdosTooltip *tooltip = CDOS_TOOLTIP (object);
    switch (prop_id)
    {
        case PROP_ITEM:
          {
            ClutterActor *item = g_value_get_object (value);
            set_item (tooltip, item);
            break;
          }
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
            break;
    }
}

static void
cdos_tooltip_get_property(GObject *object, guint prop_id, GValue *value, GParamSpec  *pspec)
{
    CdosTooltip *tooltip = CDOS_TOOLTIP (object);
    switch (prop_id)
    {
        case PROP_ITEM:
            g_value_set_object (value, tooltip->item);
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
            break;
    }
}

static void 
cdos_tooltip_show_real (CdosTooltip *self)
{
	if (self->preventShow)
		return;
	//Tweener.removeTweens(this);
        MonkeyPlugin *plugin = monkey_plugin_get ();

	ClutterActorBox *box = g_new0(ClutterActorBox,1);
	clutter_actor_get_allocation_box (CLUTTER_ACTOR (self), box);
	gfloat tooltipWidth = box->x2 - box->x1;
	gfloat tooltipHeight = box->y2 - box->y1;

	CdosLayoutManager *layout_manager = cdos_monkey_plugin_get_layout_manager (plugin);
	//调用layout manager中的 findMonitorForActor, 尚没有此接口
	MetaRectangle *monitor; // = Main.layoutManager.findMonitorForActor(this._item);
	monitor = cdos_layout_manager_get_primary_monitor(layout_manager);
        CdosPanel *panel = cdos_monkey_plugin_get_panel (plugin);
    	guint panel_height = cdos_panel_get_height(panel);

	self->mousePositionY = monitor->height - panel_height - tooltipHeight;

	gfloat tooltipTop = self->mousePositionY;  //[1];
	gfloat tooltipLeft = self->mousePositionX - tooltipWidth / 2; //[0];

	if (tooltipLeft < 0){
		tooltipLeft = 0;
	}

	if (tooltipLeft + tooltipWidth > (float)monitor->x + (float)monitor->width){
		tooltipLeft = (float)(monitor->x+monitor->width)-tooltipWidth;
	}
	clutter_actor_set_position (CLUTTER_ACTOR (self), (float)tooltipLeft, (float)tooltipTop);
	clutter_actor_show (CLUTTER_ACTOR (self));
	//clutter_actor_raise_top (CLUTTER_ACTOR(self));
    st_container_raise ((ClutterContainer *)ui_group, (ClutterActor *)self, NULL);
	g_free(box);
	self->visible = TRUE;
}

static void
cdos_tooltip_class_init (CdosTooltipClass *klass)
{
	GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
	g_type_class_add_private (klass, sizeof(CdosTooltipPrivate));
	gobject_class->finalize = cdos_tooltip_finalize;
	gobject_class->dispose = cdos_tooltip_dispose;
	gobject_class->set_property = cdos_tooltip_set_property;
	gobject_class->get_property = cdos_tooltip_get_property;
    klass->show = cdos_tooltip_show_real;
    GParamSpec *spec = g_param_spec_object ("item",
                                            "Item",
                                            "Tooltip's source item",
                                            CLUTTER_TYPE_ACTOR,
                                            G_PARAM_READWRITE);
    g_object_class_install_property (gobject_class, PROP_ITEM, spec);
}



static void
cdos_tooltip_init (CdosTooltip *self) {
	//CdosTooltipPrivate *priv;
	//priv = self->priv = CDOS_TOOLTIP_GET_PRIVATE (self);

	//clutter_actor_set_name(CLUTTER_ACTOR(self),"Tooltip");
	//this._tooltip.show_on_set_parent = false;
	//st_container_add_actor (CLUTTER_CONTAINER(ui_group), CLUTTER_ACTOR (self));
	self->showTimer = NULL;
	self->visible = FALSE;
	self->preventShow = FALSE;
}

//公有函数定义

/**
 * cdos_tooltip_new
 * @item: 一个#ClutterActor对象
 * @init_title: 需要显示的字符串
 *
 * 创建#CdosTooltip对象
 *
 * Returns: #CdosTooltip对象
 */

CdosTooltip* cdos_tooltip_new (ClutterActor *item, gchar *init_title) {
	g_return_val_if_fail(CLUTTER_IS_ACTOR(item),NULL);
	g_return_val_if_fail(init_title != NULL,NULL);
	CdosTooltip *instance;
	instance = g_object_new (CDOS_TYPE_TOOLTIP, 
            "item", item, "text", init_title, "name", "Tooltip", NULL);
	//cdos_tooltip_post_init (instance, item, init_title);
    st_container_add_actor ((ClutterContainer *)ui_group, (ClutterActor *)instance);
    clutter_actor_hide ((ClutterActor *)instance);

	return instance;
}

/**
 * cdos_tooltip_show
 * @self: 一个#CdosTooltip对象
 *
 * 显示#CdosTooltip
 *
 */

void cdos_tooltip_show (CdosTooltip *self) {
	g_return_if_fail(CDOS_IS_TOOLTIP(self));
    ((CdosTooltipClass *)G_OBJECT_GET_CLASS (self))->show (self);
}

/**
 * cdos_tooltip_hide
 * @self: 一个#CdosTooltip对象
 *
 * 隐藏#CdosTooltip
 *
 */

void cdos_tooltip_hide (CdosTooltip *self) {
	g_return_if_fail(CDOS_IS_TOOLTIP(self));
	//Tweener.removeTweens(this);
	clutter_actor_hide (CLUTTER_ACTOR (self));
	self->visible = FALSE;
}

/**
 * cdos_tooltip_set_prevent_show
 * @self: 一个#CdosTooltip对象
 * @prevent_show: TRUE或FALSE。当prevent_show的值为TRUE时，将阻止#CdosTooltip显示
 *
 * 是否显示#CdosTooltip
 *
 */

void cdos_tooltip_set_prevent_show (CdosTooltip *self, gboolean prevent_show) {
	g_return_if_fail(CDOS_IS_TOOLTIP(self));
	self->preventShow = prevent_show;
}

/**
 * cdos_tooltip_set_position
 * @self: 一个#CdosTooltip对象
 * @px: x轴坐标
 * @py: y轴坐标
 *
 * 设置#CdosTooltip显示的位置
 *
 */

void cdos_tooltip_set_position (CdosTooltip *self,gfloat px,gfloat py){
	g_return_if_fail(CDOS_IS_TOOLTIP(self));
	self->mousePositionX = px;
	self->mousePositionY = py;
}



