//
// Created by zhangtao on 16-3-17.
/**
 * SECTION: cdos-util
 * @short_description: 提供了部分辅助功能函数。
 * @title: CdosUtil
 * @include: cdos-util.h
 *
 * #CdosUtil提供了一组辅助功能函数，涉及url,命令行，进程操作等。
 */
#include "cdos.h"
#include "cdos-util.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "cdos-start.h"

struct _MatchedUrl{
    gint start_pos;
    gint end_pos;
    gchar *url;
};

static GRegex* cdos_util_get_url_regexp();
static gchar* cdos_util_get_url_pattern();

static void cdos_util_handle_spawn_error(gchar *command,gchar *error);


/*----------------------------------------------*/
static gchar* cdos_util_get_url_pattern()
{

    static gchar *pattern = NULL;

    if(pattern){
        return pattern;
    }

    //gchar *argv[3];
    //argv[0] = g_strdup("pkill");
    //argv[1] = g_strdup("-f");
    //argv[2] = g_strconcat (temp1, process_name, temp2);

    gchar *balanced_parens = g_strdup("\\((?:[^\\s()<>]+|(?:\\(?:[^\\s()<>]+\\)))*\\)");
    gchar *leading_junk = g_strdup("[\\s`(\\[{\"\\'<\u00AB\u201C\u2018]");
    gchar *not_trailing_junk = g_strdup("[^\\s`!()\\[\\]{};:\"\\'.,<>?\u00AB\u00BB\u201C\u201D\u2018\u2019]");

    gchar *reg1 = g_strdup("(^|");
    gchar *reg2 = g_strdup(")((?:[a-z][\\w-]+://|www\\d{0,3}[.]|[a-z0-9.\\-]+[.][a-z]{2,4}/)(?:[^\\s()<>]+|");
    gchar *reg3 = g_strdup(")+(?:");
    gchar *reg4 = g_strdup("|");
    gchar *reg5 = g_strdup("))");

    pattern = g_strconcat (reg1, leading_junk, reg2,balanced_parens,reg3,balanced_parens,reg4,not_trailing_junk,reg5 ,NULL);

    g_free (balanced_parens);
    g_free (leading_junk);
    g_free (reg1);
    g_free (reg2);
    g_free (reg3);
    g_free (reg4);
    g_free (reg5);
    /*
    gint pattern_length = strlen(reg1) + strlen(reg2) + strlen(reg3) + strlen(reg4) + strlen(reg5) + strlen(balanced_parens) + strlen(leading_junk) + strlen(not_trailing_junk);

    pattern = (gchar *)malloc(pattern_length + 1);
    if(!pattern){
        printf("Error: the url pattern failed !");
        exit(EXIT_FAILURE);
    }
    strcpy(pattern,reg1);
    strcat(pattern,leading_junk);
    strcat(pattern,reg2);
    strcat(pattern,balanced_parens);
    strcat(pattern,reg3);
    strcat(pattern,balanced_parens);
    strcat(pattern,reg4);
    strcat(pattern,not_trailing_junk);
    strcat(pattern,reg5);
    */
    return pattern;
}
static GRegex* cdos_util_get_url_regexp()
{
    gchar *pattern = cdos_util_get_url_pattern();

    GRegex *regex = g_regex_new (pattern, 0, 0, NULL);
    return regex;
}

static void cdos_util_handle_spawn_error(gchar *command,gchar *error){

};

/**
 * cdos_util_find_urls:
 * @urls:需要查找的字符串
 *
 * 在给定的字符串中，查找包含有url格式的子字符串。
 *
 * Returns:返回全部url的链表。
 */
GList* cdos_util_find_urls(gchar *urls){

    GRegex *regex ;
    GList *res = NULL;
    GMatchInfo *match_info;

    regex = cdos_util_get_url_regexp();

    g_regex_match (regex,urls,0,&match_info);
    //g_print("\n ======================================== \n");
    while (g_match_info_matches(match_info)){
        MatchedUrl *matchedUrl;
        gint start_pos,end_pos;
        gchar *url = g_match_info_fetch (match_info, 0);
        if (url){
            matchedUrl = g_new(MatchedUrl,1);
            matchedUrl->url = url;
            //g_print("\n %s:url=%s; \n",__func__,matchedUrl->url);
            if(g_match_info_fetch_pos(match_info,0,&start_pos,&end_pos)){

                //g_print("\n %s:start_pos=%d; \n",__func__,start_pos);
                //g_print("\n %s:end_pos=%d; \n",__func__,end_pos);

                matchedUrl->start_pos = start_pos;
                matchedUrl->end_pos = end_pos;
            }
            res = g_list_append(res,matchedUrl);
        }




        g_match_info_next (match_info, NULL);
    }

    g_match_info_free (match_info);
    g_regex_unref (regex);

    return res;
}

/**
 * cdos_util_spawn:
 * @arg: 需要执行的命令字符串数组
 *
 * 在后台执行命令行操作命令，如果执行异常，则自行处理。
 *
 * Returns:返回对应的进程PID
 */
gint cdos_util_spawn(const gchar **arg){
    gint pid;
    pid = cdos_util_try_spawn((char **)arg);
    return pid;
}

/**
 * cdos_util_spawn_command_line:
 * @command_line: 需要执行的命令字符串
 *
 * 在后台执行命令行操作命令，如果执行异常，则自行处理。
 *
 * Returns:返回对应的进程PID
 */
gint cdos_util_spawn_command_line(const gchar *command_line){

/*    gint pid;

    pid = cdos_util_try_spawn_command_line(command_line);

    return pid; */
    GPid pid;
    GError *error = NULL;
    gint argcp;
    gchar **argvp;

    g_shell_parse_argv (command_line, &argcp, &argvp, &error);
    if( error ){
        g_warning("Warnings:%s-->%s",__func__,error->message);
        g_error_free(error);
        return 0;
    }
    
    g_spawn_async ( NULL, argvp, NULL,  G_SPAWN_SEARCH_PATH | 
                        G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL, 
                        NULL, NULL, &pid, &error);
    if(error){
        g_warning("Warnings:%s-->%s",__func__,error->message);
        g_error_free(error);
        return 0;
    }
    printf ("pid id %d\n", pid);
    g_strfreev (argvp);
    return pid;


}

/**
 * cdos_util_try_spawn:
 * @argv: 需要执行的命令字符串数组
 *
 * 在后台执行命令行操作命令，如果执行失败则抛出错误异常。
 *
 * Returns:执行成功则返回对应的进程PID，否则返回0
 */
gint cdos_util_try_spawn(gchar **argv)
{

    gint pid;
    GError *error = NULL;
    gboolean result;

    result = g_spawn_async(NULL, argv,NULL,(G_SPAWN_SEARCH_PATH | G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL),NULL,NULL,&pid,&error);
    if(error){
        g_warning("Warnings:%s-->%s",__func__,error->message);
        g_error_free(error);
        return 0;
    } else {
        return pid;
    }

}
/**
 * cdos_util_try_spawn_command_line:
 * @command_line: 需要执行的命令字符串指针地址
 *
 * 在后台执行命令行操作命令，如果执行失败则抛出错误异常。
 *
 * Returns:执行成功则返回对应的进程PID，否则返回0
 */
gint cdos_util_try_spawn_command_line(gchar *command_line){

    gint pid = 0;
    gboolean result;
    GError *error;
    gchar **argvp;
    gint argcp;

    result = g_shell_parse_argv(command_line,&argcp,&argvp,&error);
    if(error){
        g_warning("Warnings:%s-->%s",__func__,error->message);
        g_error_free(error);
        return 0;
    } else {
        pid = cdos_util_try_spawn(argvp);
        if(pid)
            return pid;
        //cdos_util_handle_spawn_error(command_line,error->message);
        return 0;
    }
}

/**
 * cdos_util_killall:
 * @process_name: 进程名称
 *
 * 关闭给定名称的全部相关进程。
 */
void cdos_util_killall(gchar *process_name)
{
    if (!process_name)
        return;

    gchar *temp1 = "pkill -f '^([^ ]*/)?";
    gchar *temp2 = "($| )'";
    gchar *cmd = g_new0 (gchar, strlen(temp1) + strlen(process_name) + strlen(temp2) + 1);

    gboolean result;
    GError *error = NULL;
    gint pid;

    /*
    gchar *argv[4] = {NULL};
    argv[0] = g_strdup("pkill");
    argv[1] = g_strdup("-f");
    argv[2] = g_strconcat (temp1, process_name, temp2, NULL);

    result = g_spawn_sync(NULL, argv,NULL,G_SPAWN_SEARCH_PATH,NULL,NULL,&pid,&error);
    */
    sprintf(cmd, "%s%s%s", temp1, process_name, temp2);
    gint status;
    result = g_spawn_command_line_sync (cmd, NULL, NULL, &status, &error);

    if(error){
        g_warning("Failed to kill process!");
        g_error_free(error);
    }
    /*
    g_free (argv[0]);
    g_free (argv[1]);
    g_free (argv[2]);
    */
    g_free (cmd);
}

static const gchar *ignored_words[] = {
    "Semiconductor",
    "Components",
    "Corporation",
    "Communications",
    "Company",
    "Corp.",
    "Corp",
    "Co.",
    "Inc.",
    "Inc",
    "Incorporated",
    "Ltd.",
    "Limited.",
    "Intel",
    "chipset",
    "adapter",
    "[hex]",
    "NDIS",
    "Module"
};
static const gchar *ignored_phrases[] = {
    "Multiprotocol MAC/baseband processor",
    "Wireless LAN Controller",
    "Wireless LAN Adapter",
    "Wireless Adapter",
    "Network Connection",
    "Wireless Cardbus Adapter",
    "Wireless CardBus Adapter",
    "54 Mbps Wireless PC Card",
    "Wireless PC Card",
    "Wireless PC",
    "PC Card with XJACK(r) Antenna",
    "Wireless cardbus",
    "Wireless LAN PC Card",
    "Technology Group Ltd.",
    "Communication S.p.A.",
    "Business Mobile Networks BV",
    "Mobile Broadband Minicard Composite Device",
    "Mobile Communications AB",
    "(PC-Suite Mode)"
};

char* cdos_util_fixup_pci_description(const gchar *desc){
    char *tmp = g_strdup (desc);
    int len = strlen (tmp);
    int i;
    for (i = 0; i < len; i++)
        if (tmp[i] == '_' || tmp[i] == ',')
            tmp[i] = ' ';
    for (i = 0; i < G_N_ELEMENTS (ignored_phrases); i++)
    {
        char *ptr = NULL;
        if ((ptr = strstr (tmp, ignored_phrases[i])) != NULL)
        {
            int len = strlen (ignored_phrases[i]);
            int j;
            for (j = 0; j < len; j++)
                ptr[j] = ' ';
        }
    }
    for (i = 0; i < G_N_ELEMENTS (ignored_words); i++)
    {
        char *ptr = NULL;
        if ((ptr = strstr (tmp, ignored_words[i])) != NULL)
        {
            int len = strlen (ignored_words[i]);
            int j;
            for (j = 0; j < len; j++)
                ptr[j] = ' ';
        }
    }
    char *res = g_new0 (char, len + 1);
    int j = 0;
    for(i = 0; tmp[i] == ' '; i++);
    for(; i < len - 1; i++)
    {
        if (tmp[i] == ' ' && tmp[i+1] == ' ')
            continue;
        res[j] = tmp[i];
        j++;
    }
    if (tmp[i] != ' ')
        res[j] = tmp[i];
    g_free (tmp);
    return res;
}

/**
 * cdos_util_get_url_start_pos:
 * @url:当前匹配到的url数据信息对象
 *
 * 从匹配到的url数据信息中获取该url的起始位置索引。
 *
 * Returns:起始位置索引值。
 */
gint cdos_util_get_url_start_pos(MatchedUrl *url){
    if(url == NULL){
        return -1;
    }
    return url->start_pos;
}

/**
 * cdos_util_get_url_end_pos:
 * @url:当前匹配到的url数据信息对象
 *
 * 从匹配到的url数据信息中获取该url的终止位置索引。
 *
 * Returns:终止位置索引值。
 */
gint cdos_util_get_url_end_pos(MatchedUrl *url){
    if(url == NULL){
        return -1;
    }
    return url->end_pos;
}

/**
 * cdos_util_get_url:
 * @url:当前匹配到的url数据信息对象
 *
 * 从匹配到的url数据信息中获取url字符串。
 *
 * Returns:url字符串。
 */
gchar* cdos_util_get_url(MatchedUrl *url){
    if(url == NULL){
        return NULL;
    }
    return url->url;
}
