

#include "config.h"
#include "cdos-popup-separator-menu-item.h"
#include "cdos-window-context-menu.h"
#include "cdos-layout-manager.h"
#include "cdos-monkey-plugin.h"

#include "st.h"

#include <glib/gi18n-lib.h>


/**
 * SECTION:cdos-window-context-menu
 * @Title: CdosWindowContextMenu
 * @Short_description: 右键菜单
 * 
 * 缩略图模式下，右键某个克隆窗体，弹出的菜单，目测包含的功能有：移动到上一个或者下一个工作区、关闭这个窗口等，
 * 不过 cdos 桌面还没有开始做右键菜单的功能，所以空空
 */
G_DEFINE_TYPE ( CdosWindowContextMenu, cdos_window_context_menu, G_TYPE_OBJECT );



static gboolean onSourceKeyPress ( GObject *gobject, ClutterEvent *event, gpointer user_data ) {

	CdosWindowContextMenu *self = CDOS_WINDOW_CONTEXT_MENU (user_data);
	CdosPopupMenuBase *menu = CDOS_POPUP_MENU_BASE (self);
	CdosPopupMenuBaseClass *obj = CDOS_POPUP_MENU_BASE_CLASS (menu);

	guint symbol = clutter_event_get_key_symbol (event);

	if ( symbol == CLUTTER_KEY_space || symbol == CLUTTER_KEY_Return ) {
				
		//等同于this.menu.toggle();
            	cdos_popup_menu_base_toggle (menu);

		return TRUE;
	}
	else if ( symbol == CLUTTER_KEY_Escape && menu->is_open ) {

		//等同于this.menu.close();
		obj->close (menu, FALSE);
		
		return TRUE;
	}
	else if ( symbol == CLUTTER_KEY_Down ) {
		
		if (!menu->is_open)
                	cdos_popup_menu_base_toggle (menu);

		st_widget_navigate_focus ( ST_WIDGET (gobject) ,self->actor, GTK_DIR_DOWN, FALSE );

		return TRUE;
	}
	else 
	{
		return FALSE;
	}
}


static void onToggled ( GObject *gobject, gboolean opening, gpointer user_data ) {

	CdosWindowContextMenu *self = CDOS_WINDOW_CONTEXT_MENU (user_data);
	MetaWorkspace *mWorkspace;
	MetaWorkspace *neighbor_L, *neighbor_R;

        if (!opening) {
            	//this.onClose();
		self->onClose ();
            	//this.destroy();
		clutter_actor_destroy (CLUTTER_ACTOR(self));
            	return;
        }

	if ( meta_window_is_on_all_workspaces (self->metaWindow) ) 
		cdos_popup_menu_item_set_label (self->itemOnAllWorkspaces, _("Only on this workspace"));
	else
		cdos_popup_menu_item_set_label (self->itemOnAllWorkspaces, _("Visible on all workspaces"));


	mWorkspace = meta_window_get_workspace (self->metaWindow);
	neighbor_L = meta_workspace_get_neighbor ( mWorkspace, META_MOTION_LEFT );

	if ( mWorkspace != neighbor_L ) 
        	clutter_actor_show ( CLUTTER_ACTOR(self->itemMoveToLeftWorkspace) );
        else
            	clutter_actor_hide ( CLUTTER_ACTOR(self->itemMoveToLeftWorkspace) );

        neighbor_R = meta_workspace_get_neighbor ( mWorkspace, META_MOTION_RIGHT );

        if ( mWorkspace != neighbor_R )
                clutter_actor_show ( CLUTTER_ACTOR(self->itemMoveToRightWorkspace));
        else
                clutter_actor_hide ( CLUTTER_ACTOR(self->itemMoveToRightWorkspace));

	if (meta_window_get_maximized (self->metaWindow)) //水平為0, 垂直為1
		cdos_popup_menu_item_set_label (self->itemMaximizeWindow, _("Unmaximize"));
        else
		cdos_popup_menu_item_set_label (self->itemMaximizeWindow, _("Maximize"));
        
}


static void onCloseWindowActivate ( GObject *gobject, ClutterEvent *event, gpointer user_data ) {

	CdosWindowContextMenu *self = CDOS_WINDOW_CONTEXT_MENU (user_data);

	meta_window_delete (self->metaWindow, cdos_global_get_current_time (cdos_global_get() ));
}


static void onMinimizeWindowActivate ( GObject *gobject, ClutterEvent *event, gpointer user_data ) {

	CdosWindowContextMenu *self = CDOS_WINDOW_CONTEXT_MENU (user_data);
	
	gboolean isMinimized;

	g_object_get (self->metaWindow, "minimized", &isMinimized, NULL);
	
        if (isMinimized)
		meta_window_unminimize (self->metaWindow);
        else 
		meta_window_minimize (self->metaWindow);
}


static void onMaximizeWindowActivate ( GObject *gobject, ClutterEvent *event, gpointer user_data ) {

	CdosWindowContextMenu *self = CDOS_WINDOW_CONTEXT_MENU (user_data);

	if (meta_window_get_maximized (self->metaWindow))
		meta_window_unmaximize (self->metaWindow, META_MAXIMIZE_HORIZONTAL | META_MAXIMIZE_VERTICAL);
	else
		meta_window_maximize (self->metaWindow, META_MAXIMIZE_HORIZONTAL | META_MAXIMIZE_VERTICAL);
}


static void onMoveToLeftWorkspace ( GObject *gobject, ClutterEvent *event, gpointer user_data ) {

	CdosWindowContextMenu *self = CDOS_WINDOW_CONTEXT_MENU (user_data);

}


static void onMoveToRightWorkspace ( GObject *gobject, ClutterEvent *event, gpointer user_data ) {

	CdosWindowContextMenu *self = CDOS_WINDOW_CONTEXT_MENU (user_data);

	MetaWorkspace *mWorkspace = meta_window_get_workspace (self->metaWindow);
        MetaWorkspace *neighbor_L = meta_workspace_get_neighbor ( mWorkspace, META_MOTION_LEFT );

	if (neighbor_L)
		meta_window_change_workspace ( self->metaWindow, neighbor_L );
}


static void toggleOnAllWorkspaces ( GObject *gobject, gpointer user_data ) {

	CdosWindowContextMenu *self = CDOS_WINDOW_CONTEXT_MENU (user_data);
	
	MetaWorkspace *mWorkspace = meta_window_get_workspace (self->metaWindow);
        MetaWorkspace *neighbor_R = meta_workspace_get_neighbor ( mWorkspace, META_MOTION_RIGHT );

        if (neighbor_R)
                meta_window_change_workspace (self->metaWindow, neighbor_R );
}


static void onMoveWindowToNewWorkspace ( GObject *gobject, gpointer user_data ) {

	CdosWindowContextMenu *self = CDOS_WINDOW_CONTEXT_MENU (user_data);

	//Main.moveWindowToNewWorkspace(metaWindow, true);
	//此部份还没写好
}


static void onAddMenuItem_func ( gpointer data, gpointer user_data ) {

	CdosWindowContextMenu *self = CDOS_WINDOW_CONTEXT_MENU (user_data);

	cdos_popup_menu_base_add_menu_item ( CDOS_POPUP_MENU_BASE(self), CDOS_POPUP_BASE_MENU_ITEM(data), -1 );
}


static void onItemChangeMonitor ( GObject *gobject, gpointer user_data ) {

	CdosWindowContextMenu *self = CDOS_WINDOW_CONTEXT_MENU (user_data);
	
	meta_window_move_to_monitor (self->metaWindow, self->index);
}


static void iter_func ( gpointer data, gpointer user_data ) {

	CdosWindowContextMenu *self = CDOS_WINDOW_CONTEXT_MENU (user_data);

	if ( self->index != meta_window_get_monitor (self->metaWindow) ) {

		gchar *string;
		CdosPopupMenuItem *itemChangeMonitor = cdos_popup_menu_item_new ();

		self->index++;

		string = g_strdup_printf ( _("Move to monitor %d"), self->index );

		cdos_popup_menu_item_set_label ( itemChangeMonitor, string );

		g_signal_connect ( itemChangeMonitor, "activate", G_CALLBACK ( onItemChangeMonitor ), self );

		self->monitorItems = g_list_append ( self->monitorItems, itemChangeMonitor );
	}
}


static void 
cdos_window_context_menu_class_init ( CdosWindowContextMenuClass *klass ) {

        GObjectClass *gobject_class = G_OBJECT_CLASS(klass);

	CdosPopupComboMenuClass *combo_class = CDOS_POPUP_COMBO_MENU_CLASS(klass);

}



static void
cdos_window_context_menu_init ( CdosWindowContextMenu *self ) {



}


static void
cdos_window_context_menu_post_init ( CdosWindowContextMenu *self, ClutterActor *actor, MetaWindow *metaWindow, void (*onClose) ) {

	ClutterActor *uiGroup = monkey_plugin_get_ui_group ( monkey_plugin_get () );
	StSide orientation;
	//GList *monitorItems;
	GList *items = NULL;
	CdosPopupMenuItem *itemMoveToNewWorkspace;
	CdosLayoutManager *layout_manager = cdos_monkey_plugin_get_layout_manager ( monkey_plugin_get ( ) );
	gboolean isMinimized;


        //PopupMenu.PopupComboMenu.prototype._init.call(this, actor);
	//进行popupComboMenu继承动作

	self->name = g_strdup ("scale-window-context-menu");

	clutter_actor_add_child (uiGroup, self->actor);

	clutter_actor_hide (self->actor);
        
        orientation = ST_SIDE_TOP;

        self->onClose = onClose;

	g_signal_connect ( actor, "key-press-event", G_CALLBACK ( onSourceKeyPress ), self );
	g_signal_connect ( self, "open-state-changed", G_CALLBACK ( onToggled ), self );

        self->metaWindow = metaWindow;

        self->itemCloseWindow = cdos_popup_menu_item_new ();
	cdos_popup_menu_item_set_label (self->itemCloseWindow, _("Close"));
	g_signal_connect ( self->itemCloseWindow, "activate", G_CALLBACK ( onCloseWindowActivate ), self );

        g_object_get (self->metaWindow, "minimized", &isMinimized, NULL);
        //if (metaWindow.minimized) {
	if (isMinimized) {
            	self->itemMinimizeWindow = cdos_popup_menu_item_new ();
		cdos_popup_menu_item_set_label (self->itemMinimizeWindow, _("Restore"));
	}
        else
	{
		self->itemMinimizeWindow = cdos_popup_menu_item_new ();
                cdos_popup_menu_item_set_label (self->itemMinimizeWindow, _("Minimize"));
	}

	g_signal_connect ( self->itemMinimizeWindow, "activate", G_CALLBACK ( onMinimizeWindowActivate ), self );

	self->itemMaximizeWindow = cdos_popup_menu_item_new ();
        cdos_popup_menu_item_set_label (self->itemMaximizeWindow, _("Maximize"));
	g_signal_connect ( self->itemMaximizeWindow, "activate", G_CALLBACK ( onMaximizeWindowActivate ), self );

        self->itemMoveToLeftWorkspace = cdos_popup_menu_item_new ();
        cdos_popup_menu_item_set_label (self->itemMoveToLeftWorkspace, _("Move to left workspace"));
        g_signal_connect ( self->itemMoveToLeftWorkspace, "activate", G_CALLBACK ( onMoveToLeftWorkspace ), self );

        self->itemMoveToRightWorkspace = cdos_popup_menu_item_new ();
        cdos_popup_menu_item_set_label (self->itemMoveToRightWorkspace, _("Move to right workspace"));
        g_signal_connect ( self->itemMoveToRightWorkspace, "activate", G_CALLBACK ( onMoveToRightWorkspace ), self );

        self->itemOnAllWorkspaces = cdos_popup_menu_item_new ();
        cdos_popup_menu_item_set_label (self->itemOnAllWorkspaces, _("Visible on all workspaces"));
        g_signal_connect ( self->itemOnAllWorkspaces, "activate", G_CALLBACK ( toggleOnAllWorkspaces ), self );

        itemMoveToNewWorkspace = cdos_popup_menu_item_new ();
        cdos_popup_menu_item_set_label (itemMoveToNewWorkspace, _("Move to a new workspace"));
        g_signal_connect ( itemMoveToNewWorkspace, "activate", G_CALLBACK ( onMoveWindowToNewWorkspace ), self );

	GPtrArray *monitors = cdos_layout_manager_get_monitors ( layout_manager );

	if (monitors && monitors->len > 1 ) {

		g_ptr_array_foreach ( monitors, (GFunc) iter_func, self );
		
		self->index = 0;
		self->monitorItems = g_list_append ( self->monitorItems, cdos_popup_separator_menu_item_new() );
		self->index = 0;
	} 

	items = g_list_append ( items, itemMoveToNewWorkspace );
	items = g_list_append ( items, self->itemOnAllWorkspaces );
	items = g_list_append ( items, self->itemMoveToLeftWorkspace );
	items = g_list_append ( items, self->itemMoveToRightWorkspace );
	items = g_list_append ( items, (gpointer)cdos_popup_separator_menu_item_new() );
	items = g_list_append ( items, self->itemMinimizeWindow );
	items = g_list_append ( items, self->itemMaximizeWindow );
	items = g_list_append ( items, self->itemCloseWindow );

	items = g_list_concat ( self->monitorItems, items );

	if (orientation != ST_SIDE_BOTTOM) {

		items = g_list_reverse (items);
		g_list_foreach ( items, (GFunc) onAddMenuItem_func, self );
	}
	
	cdos_popup_combo_menu_set_active_item ( CDOS_POPUP_COMBO_MENU(self), 0 );
}

/**
 * cdos_window_context_menu_new:
 * @actor: 待定
 * @metaWindow: 待定
 * @onClose： 待定
 *
 * 待定
 */
CdosWindowContextMenu *cdos_window_context_menu_new ( ClutterActor *actor, MetaWindow *metaWindow, void (*onClose) ) {

        CdosWindowContextMenu *instance = g_object_new ( CDOS_TYPE_WINDOW_CONTEXT_MENU, NULL );

        cdos_window_context_menu_post_init ( instance, actor, metaWindow, onClose );

        return instance;
}



