
#ifndef __CDOS_WINDOW_OVERLAY_H__
#define __CDOS_WINDOW_OVERLAY_H__

#include <glib-object.h>
#include <clutter/clutter.h>
#include <meta/workspace.h>
#include "st.h"

GType	cdos_window_overlay_get_type (void);


#define CDOS_TYPE_WINDOW_OVERLAY (cdos_window_overlay_get_type())
#define CDOS_WINDOW_OVERLAY(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj),CDOS_TYPE_WINDOW_OVERLAY,CdosWindowOverlay))
#define CDOS_IS_WINDOW_OVERLAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj),CDOS_TYPE_WINDOW_OVERLAY))
#define CDOS_WINDOW_OVERLAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass),CDOS_TYPE_WINDOW_OVERLAY,CdosWindowOverlayClass))
#define CDOS_IS_WINDOW_OVERLAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((obj),CDOS_TYPE_WINDOW_OVERLAY))
#define CDOS_WINDOW_OVERLAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS((obj),CDOS_TYPE_WINDOW_OVERLAY,CdosWindowOverlayClass))


typedef struct _CdosWindowOverlayClass CdosWindowOverlayClass;
typedef struct _CdosWindowOverlay CdosWindowOverlay;

typedef struct _CdosPointerTracker CdosPointerTracker;

struct _CdosPointerTracker {
	MetaCursorTracker *pointer;

	gint lastPointerX;
	gint lastPointerY;
};

CdosPointerTracker *cdos_pointer_tracker_new ();

gboolean cdos_pointer_tracker_has_moved (CdosPointerTracker *self);


typedef struct _CdosWindowClone	CdosWindowClone;

struct _CdosWindowOverlayClass {
	GObjectClass parent_class;
};

struct _CdosWindowOverlay {

	GObject parent_instance;

	CdosWindowClone *windowClone;
	ClutterActor *parentActor;
    gboolean hidden;
    gboolean hovering;
    gboolean isSelected;

	ClutterActor *icon;
   	StBin *applicationIconBox; 
	StButton *closeButton;
	StWidget *title; //StLabel
	gint updateCaptionId;
	CdosPointerTracker *pointerTracker;

	gint idleToggleCloseId;
	gint attentionId;
	gint urgentId;
	void (*disconnectAttentionSignals)(CdosWindowOverlay *self);
	void (*disconnectWindowAdded)(CdosWindowOverlay *self);
	void (*updateIconCaptionWidth)(CdosWindowOverlay *self,
                				   gfloat cloneX,
                                   gfloat cloneY,
                                   gfloat cloneWidth,
                                   gfloat cloneHeight,
                                   gfloat maxWidth);


	gdouble closeButton_overlap;
	gfloat title_spacing;
	gint windowAddedId;
};

CdosWindowOverlay *cdos_window_overlay_new ( CdosWindowClone *windowClone, ClutterActor *parentActor );

void cdos_window_overlay_set_selected ( CdosWindowOverlay *self, gboolean selected );

void cdos_window_overlay_hide (CdosWindowClone *clone, CdosWindowOverlay *self );

void cdos_window_overlay_show (CdosWindowClone *clone, CdosWindowOverlay *self );

void cdos_window_overlay_fadein ( CdosWindowOverlay *self );

gfloat cdos_window_overlay_chrome_width ( CdosWindowOverlay *self );

void cdos_window_overlay_chrome_heights ( CdosWindowOverlay *self, gfloat *h_overlap , gfloat *h_space );

void cdos_window_overlay_update_positions ( CdosWindowOverlay *self,
                                                gfloat cloneX,
                                                gfloat cloneY,
                                                gfloat cloneWidth,
                                                gfloat cloneHeight,
                                                gfloat maxWidth );

void cdos_window_overlay_show_close_button ( CdosWindowOverlay *self );

void cdos_window_overlay_hide_close_button ( CdosWindowOverlay *self );

#endif //__CDOS_WINDOW_OVERLAY_H__
