
#include <clutter/clutter.h>
#include <math.h>

#include "cdos-window-utils.h"
#include <meta/meta-window-actor.h>
#include <meta/display.h>

#include "cdos.h"


#define META_LIST_DEFAULT     0
/**
 * SECTION:cdos-window-utils
 * @Title: cdos-window-utils
 * @Short_description: 给每个应用程序窗口创建缩略窗口
 * 
 * 这不是一个 GObject 对象，只是用于 CdosOverview 的一个接口函数，为每个应用程序窗口创建克隆体。
 */
typedef struct {

	ClutterActor *t;
	gfloat x;
	gfloat y;
	gfloat w;
	gfloat h;

} TextureObj;


// Creates scaled clones of metaWindow and its transients,
// keeping their relative size to each other.
// When scaled, all clones are made to fit in width and height
// if neither width nor height is given, windows are not scaled
/**
 * cdos_create_window_clone:
 * @metaWindow: #MetaWindow 对象，就是实际打开的应用程序窗口
 * @width： 0.0
 * @height: 0.0
 * @withTransients: 是否包含临时窗口
 * @withPositions: FALSE
 *
 * 创建窗口克隆，一个窗口为什么要返回一个链表？因为窗口可能会有临时窗口。具体的处理在 CdosWindowClone 中。
 */
/**
 * ClutterFixedLayout
 **/
GList *
cdos_create_window_clone (MetaWindow *metaWindow, gfloat width, gfloat height, gboolean withTransients, gboolean withPositions )
{
  	GList *clones = NULL;
  	GList *textures = NULL;
	MetaWindowActor *metaWindowActor;
	ClutterActor *texture;

	gfloat windowWidth, windowHeight;
	gfloat x, y;

	gfloat scale;
    gfloat scaleWidth;
    gfloat scaleHeight;
	gint i;

    //if (meta_window_get_transient_for (metaWindow))
    //    return NULL;
	
	metaWindowActor = META_WINDOW_ACTOR( meta_window_get_compositor_private ( metaWindow ) );

  	if ( !metaWindowActor ) {
        return NULL;
  	}

	texture = meta_window_actor_get_texture ( metaWindowActor );

	clutter_actor_get_size ( CLUTTER_ACTOR(metaWindowActor), &windowWidth, &windowHeight );

	clutter_actor_get_position ( CLUTTER_ACTOR(metaWindowActor), &x, &y );	

	return clones;
}

