#include "cdos-workspace.h"
#include "cdos-window-tracker.h"
#include "cdos-window-overlay.h"
#include "cdos-window-clone.h"
#include "cdos.h"
#include "cdos-start.h"
#include "cdos-marshal.h"
#include "cdos-window-manager.h"
#include "cdos-export-overview.h"
#include <math.h>

/**
 * SECTION:cdos-workspace
 * @Title: CdosWorkspace
 * @Short_description: 工作区
 * 
 * CdosWorkspace 就是单个工作区，不同于工作区缩略图，这个对象与 CdosWorkspaceMonitor 一起
 * 管理应用窗口缩略图。
 */
G_DEFINE_TYPE (CdosWorkspace, cdos_workspace, CLUTTER_TYPE_ACTOR);

/* 下面这个宏可以用来访问新建对象的私有结构*/
#define CDOS_WORKSPACE_GET_PRIVATE(obj) \
    (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CDOS_TYPE_WORKSPACE, CdosWorkspacePrivate))

#define DEFAULT_SLOT_FRACTION 0.825
#define WINDOWOVERLAY_ICON_SIZE 32
#define ANIMATION_TIME  250
enum
{
  FOCUS_REFRESH_REQUIRED,
  SELECTION_CHANGED,
  LAST_SIGNAL
};

#define N_POSITIONS 6
static ClutterVertex positions[] = {
    {0.5, 0.525, 0.875},
    {0.25, 0.525, 0.48}, {0.75, 0.525, 0.48},
    {0.25, 0.275, 0.48}, {0.75, 0.275, 0.48}, {0.5, 0.75, 0.48},
    /* 4 */
    {0.25, 0.275, 0.47}, {0.75, 0.275, 0.47},
    {0.25, 0.75, 0.47},  {0.75, 0.75,  0.47},
    /* 5 */
    {0.165, 0.25, 0.32}, {0.495, 0.25, 0.32}, {0.825, 0.25, 0.32},
    {0.25, 0.75, 0.32},  {0.75, 0.75, 0.32},
    /* 6 */
    {0.165, 0.25, 0.32}, {0.495, 0.25, 0.32}, {0.825, 0.25, 0.32},
    {0.165, 0.75, 0.32}, {0.495, 0.75, 0.32}, {0.825, 0.75, 0.32}
};

static guint cdos_workspace_signals [LAST_SIGNAL] = { 0 };
static void select_next_non_empty_monitor(CdosWorkspace *workspace, guint start, guint increment);
static void on_focus_refresh_required (CdosWorkspace *workspace, gpointer data);
static void cdos_workspace_select_active_clone (CdosWorkspace *workspace,
                                                CdosWindowClone *clone, 
                                                CdosWorkspaceMonitor *cwm);
static void
cdos_workspace_monitor_select_clone (CdosWorkspaceMonitor *cwm, CdosWindowClone *clone);

static void
cdos_workspace_monitor_select_index (CdosWorkspaceMonitor *cwm, guint index);

static CdosWindowClone*
cdos_workspace_monitor_add_window_clone (CdosWorkspaceMonitor *obj, MetaWindowActor *win);

static void
cdos_workspace_monitor_show_window_overlay (CdosWorkspaceMonitor *cwm,
                                            CdosWindowClone *clone, gboolean fade);

static int sort_by_stable_sequence (void *a, void *b, GList *windows);

static void cdos_workspace_monitor_position_windows (CdosWorkspaceMonitor *cwm, guint flag);

static void cdos_workspace_monitor_do_window_added (CdosWorkspaceMonitor *obj, MetaWindow *mwindow);

struct _CdosWorkspacePrivate
{
    GList *monitors;//成员为 CdosWorkspaceMonitor
    CdosWindowClone *activeClone;
    guint currentMonitorIndex;
    gulong restack_id;
};

static gboolean
is_my_window (CdosWorkspaceMonitor *cwm, MetaWindowActor *win)
{
    MetaWindow *mwin = meta_window_actor_get_meta_window (win);
    if (meta_window_get_transient_for (mwin))
        return FALSE;

    return (meta_window_located_on_workspace (mwin, cwm->metaWorkspace) &&
            (!mwin || meta_window_get_monitor (mwin) == cwm->monitorIndex));
    /*
    return (window_actor_displayed_on_workspace (win, meta_workspace_index (cwm->metaWorkspace))
            && (!mwin || meta_window_get_monitor (mwin) == cwm->monitorIndex));
    */
}

static gboolean
cdos_workspace_monitor_is_overview_win (MetaWindowActor *win){
      CdosWindowTracker *tracker;
      MetaWindow *mwindow = NULL;

      tracker = cdos_window_tracker_get_default ();
      mwindow = meta_window_actor_get_meta_window (win);

      return cdos_is_interesting (mwindow);
}

static gint
cdos_workspace_monitor_lookup_index (CdosWorkspaceMonitor *obj, MetaWindow *mwin){
    guint index = 0;
    GList *windows = obj->windows;
    while ( windows != NULL ){
        GList *next = windows->next;
        CdosWindowClone *clone = windows->data;

        if (clone->metaWindow == mwin){
            return index;
        }
        index ++;
        windows = next;
    }
    return -1;
}

static gboolean 
delayed_window_repositioning (CdosWorkspaceMonitor *self)
{
    /* FIXME */

    cdos_workspace_monitor_position_windows (self, WINDOW_POSITION_ANIMATE);
    g_signal_emit_by_name (self->myWorkspace, "focus-refresh-required");
    return FALSE;
}
static void
cdos_workspace_monitor_do_remove_window (CdosWorkspaceMonitor *obj, MetaWindow *mwin){
    MetaWindowActor *win = (MetaWindowActor *)meta_window_get_compositor_private (mwin);
    GList *iter;
    for (iter = obj->windows; iter; iter = iter->next)
    {
        CdosWindowClone *clone = iter->data;
        if (clone->metaWindow == mwin)
            break;
    }
    if(iter == NULL){
        /*
        for (iter = obj->windows; iter; iter = iter->next)
        {
            CdosWindowClone *clone = iter->data;
            MetaWindow *w = clone->metaWindow;
            Window W = meta_window_get_xwindow (w);
        }
        */
        return;
    }

    if (win && is_my_window (obj, win)){
        return;
    }

    CdosWindowClone *clone = iter->data;
    obj->windows = g_list_remove (obj->windows, clone);
    if (win)
    {
        /*
        ClutterPoint xy;
        ClutterSize size;
        clutter_actor_get_transformed_position ((ClutterActor *)clone, &xy.x, &xy.y);
        clutter_actor_get_transformed_size ((ClutterActor *)clone, &size.width, &size.height);
        // FIXME
        */
    }
    gint len = g_list_length (obj->windows);
    if (obj->kbWindowIndex >= len)
        obj->kbWindowIndex = len - 1;

    if (clone == obj->myWorkspace->priv->activeClone)
        cdos_workspace_monitor_select_index (obj, obj->kbWindowIndex);

    clutter_actor_destroy ((ClutterActor *)clone);

    /* FIXME
    int x, y;
    ClutterModifierType mask;
    cdos_global_get_pointer (global, &x, &y, &mask);
    obj->cursorX = x;
    obj->cursorY = Y;
    */

    meta_later_add (META_LATER_BEFORE_REDRAW,
                    (GSourceFunc) delayed_window_repositioning,
                    obj,
                    NULL);
}

static void
add_window_actor (CdosWorkspaceMonitor *obj, MetaWindowActor *actor, MetaWindow *window)
{
    if (cdos_workspace_monitor_lookup_index (obj, window) != -1)
        return;

    if (!is_my_window (obj, actor) ||
            !cdos_workspace_monitor_is_overview_win (actor)){
        return;        
    }
    CdosWindowClone *clone = cdos_workspace_monitor_add_window_clone (obj, actor);

    clutter_actor_set_position ((ClutterActor *)clone, obj->x, obj->y);
    ClutterActor *stage = NULL;
    stage = clutter_actor_get_stage (obj->actor);
    if (stage){
       cdos_workspace_monitor_position_windows (obj, WINDOW_POSITION_ANIMATE); 
       g_signal_emit_by_name (obj->myWorkspace, "focus-refresh-required");
    }
}

typedef struct _WindowAddId
{
    CdosWorkspaceMonitor *self;
    MetaWindow *window;
    guint id;
}WindowAddId;

static WindowAddId *
find_id (GPtrArray *arr, MetaWindow *window)
{
    guint i = 0;
    for (; i < arr->len; i++)
    {
        if (window == ((WindowAddId *)arr->pdata[i])->window)
            return arr->pdata[i];
    }
    return NULL;
}

static gboolean
add_window_idle (WindowAddId *id)
{
    CdosWorkspaceMonitor *obj = id->self;
    MetaWindow *window = id->window;

    MetaWindowActor *actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));
    MetaWorkspace *my_workspace = meta_window_get_workspace (window);

    if (actor && (my_workspace == obj->metaWorkspace))
    {
        add_window_actor (obj, actor, window);
        g_source_remove (id->id);
        id->id = 0;
        g_ptr_array_remove (obj->add_window_ids, id);
        return FALSE;
    }

    return TRUE;
}

static void
cdos_workspace_monitor_do_window_added (CdosWorkspaceMonitor *obj, MetaWindow *window)
{
    if (obj->leavingOverview)
        return;

    MetaWindowActor *actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));

    if (!actor){
        WindowAddId *id;
        id = find_id (obj->add_window_ids, window);
        if (id)
            return;
        id = g_new0 (WindowAddId, 1);
        g_ptr_array_add (obj->add_window_ids, id);
        id->window = window;
        id->self = obj;
        id->id = g_idle_add ((GSourceFunc) add_window_idle, id);

        return;
    }
    add_window_actor (obj, actor, window);
}

static void
clone_on_workspace_changed (CdosWindowClone *clone, guint oldws, gpointer data){
    CdosWorkspaceMonitor *cwm = (CdosWorkspaceMonitor *)data;

    cdos_workspace_monitor_do_remove_window (cwm, clone->metaWindow);
}

// 随信号传递的 CdosWorkspaceMonitor 不完整结构，无法使用。
// 换成 clone 的 myContainer
static void 
cdos_workspace_monitor_on_clone_selected ( CdosWindowClone *clone,
                                           gpointer data ){
    CdosWorkspaceMonitor *cwm = clone->myContainer;
   // GList *clones = cwm->windows;
    cdos_workspace_monitor_select_clone (cwm, clone);

}

static void 
cdos_workspace_monitor_on_clone_activated (CdosWindowClone *clone,
                                           gpointer data)
{
    //CdosWorkspaceMonitor *cwm = (CdosWorkspaceMonitor *) data;
    CdosWorkspaceMonitor *cwm = clone->myContainer;
    guint32 time = cdos_global_get_current_time(cdos_global_get ());
    guint wsIndex = -1;
    if (cwm->metaWorkspace){
        wsIndex = meta_workspace_index (cwm->metaWorkspace);
        cdos_activate_window (clone->metaWindow, time, wsIndex);
    }
}

static void
on_show_overlay_close (CdosWindowOverlay *overlay, CdosWorkspaceMonitor *cwm)
{
    GList *iter = NULL;
    for (iter = cwm->windows; iter; iter = iter->next){
        CdosWindowClone *clone = iter->data;
        CdosWindowOverlay *overlay_iter = clone->overlay;
        if (overlay_iter == overlay ){
            continue;
        }
        cdos_window_overlay_hide_close_button (overlay_iter);
    }
}

static void 
cdos_workspace_monitor_on_clone_closed (CdosWindowClone *clone,
                                            gpointer data)
{
    guint32 time = cdos_global_get_current_time(cdos_global_get ());

    meta_window_delete (clone->metaWindow, time);
}

static void  G_GNUC_UNUSED
on_clone_size_changed (CdosWindowClone *clone, CdosWorkspaceMonitor *cwm)
{
    cdos_workspace_monitor_position_windows (cwm, 0);
}

static CdosWindowClone *
cdos_workspace_monitor_add_window_clone (CdosWorkspaceMonitor *cwm, MetaWindowActor *win){
    CdosWindowClone *clone = NULL;
    clone = cdos_window_clone_new (win, cwm);
    CdosWindowOverlay *overlay = cdos_window_overlay_new (clone, cwm->windowOverlaysGroup);
    g_signal_connect (clone, "workspace-changed", G_CALLBACK(clone_on_workspace_changed), cwm);
    g_signal_connect (clone, "selected", G_CALLBACK(cdos_workspace_monitor_on_clone_selected), cwm);
    g_signal_connect (clone, "activated", G_CALLBACK(cdos_workspace_monitor_on_clone_activated), cwm);
    g_signal_connect (clone, "closed", G_CALLBACK(cdos_workspace_monitor_on_clone_closed), cwm);

    //g_signal_connect (clone, "size-changed", G_CALLBACK(on_clone_size_changed), cwm);
    clutter_actor_add_child (cwm->actor, (ClutterActor *)clone);
    g_signal_connect (overlay, "show-close-button", G_CALLBACK (on_show_overlay_close), cwm);
    cwm->windows = g_list_append (cwm->windows, clone);
    clone->overlay = overlay;
    return clone;
}

static void
on_window_added (MetaWorkspace *workspace, 
                 MetaWindow *window,
                 CdosWorkspaceMonitor *self)
{
    //cdos_debug ("%s %p", g_type_name (G_OBJECT_TYPE (window)), window);
    cdos_workspace_monitor_do_window_added (self, window);
}

static void
on_window_removed (MetaWorkspace *workspace, 
                   MetaWindow *window,
                   CdosWorkspaceMonitor *self)
{
// 这个回调函数发生在： overview 中关闭窗口
    cdos_workspace_monitor_do_remove_window (self, window);
}

static void
on_window_enter_monitor (MetaScreen *screen,
                         guint windowMonitorNumber,
                         MetaWindow *window,
                         gpointer data) {

}

static void
on_window_left_monitor (MetaScreen *screen,
                         guint windowMonitorNumber,
                         MetaWindow *window,
                         gpointer data){

}

static void
cdos_workspace_monitor_on_destroy (ClutterActor *actor, gpointer data){
    CdosWorkspaceMonitor *cwm = (CdosWorkspaceMonitor *)data;

    MetaScreen *screen = cdos_global_get_screen (global);
    if (cwm->overviewHiddenId){
        //Main.overview.disconnect(this._overviewHiddenId);
        CdosOverview *overview = cdos_overview_get ();
        g_signal_handler_disconnect (overview, cwm->overviewHiddenId);
        cwm->overviewHiddenId = 0;
    }

    GList *windows = cwm->windows;
    while ( windows != NULL ){
        GList *next = windows->next;
        CdosWindowClone *clone = windows->data;
        windows = next;
    }

    // FIXME stickyCallbackId
    if (cwm->metaWorkspace){
        g_signal_handler_disconnect (cwm->metaWorkspace, cwm->windowAddedId);
        g_signal_handler_disconnect (cwm->metaWorkspace, cwm->windowRemovedId);
    }
    g_signal_handler_disconnect (screen, cwm->windowEnteredMonitorId);
    g_signal_handler_disconnect (screen, cwm->windowLeftMonitorId);

    GList *windows_o = cwm->windows;
    while (windows_o != NULL){
        GList *next = windows_o->next;
        CdosWindowClone *clone = windows_o->data;
        clutter_actor_destroy ((ClutterActor *)clone);
        windows_o = next;
    }
    g_list_free (cwm->windows);
    cwm->windows = NULL;
}

static guint 
calc_new_index (guint itemCount, guint curRow, guint numCols, guint numRows, guint curCol, gint rowDelta){
    guint newIndex = (curRow + rowDelta) * numCols + curCol;
    if (rowDelta >= 0){
        return newIndex < itemCount ? newIndex : (curCol < numCols -1 ? curCol +1 : 0);
    }else {
        guint numFullRows = floor (itemCount /numCols);
        guint numIOILR = itemCount % numCols;

        return newIndex >= 0 ? newIndex : (curCol == 0 ? (numFullRows *numCols) -1 :(numIOILR && curCol > numIOILR ? (((numFullRows - 1) * numCols) + curCol -1): (((numRows -1) *numCols ) +curCol -1)));
    }
}

/**
 * grid_navigator_next_index:
 * @itemCount: 总数
 * @numCols: 列数
 * @currentIndex: 索引值
 * @symbol: 上下左右键
 *
 * 在网格模式中计算键盘按上下左右键切换到窗口或者工作区缩略图的索引值
 *
 * Return value: 索引值
 */
guint
grid_navigator_next_index (guint itemCount,
                           guint numCols,
                           guint currentIndex,
                           guint symbol)
{
    gint result = -1;
    if (itemCount > 3 && (symbol == CLUTTER_KEY_Down || symbol == CLUTTER_KEY_Up))
    {
        guint numRows = ceil (itemCount / numCols);
        guint curRow = currentIndex / numCols;
        guint curCol = currentIndex % numCols;
        if (symbol == CLUTTER_KEY_Down){
            result = calc_new_index (itemCount, curRow, numCols, numRows, curCol, 1);
        }

        if (symbol == CLUTTER_KEY_Up){
            result = calc_new_index (itemCount, curRow, numCols, numRows, curCol, -1);
        }
    }else if (symbol == CLUTTER_KEY_Left || symbol == CLUTTER_KEY_Up){
        result = (currentIndex < 1 ? itemCount : currentIndex) -1;
    }else if (symbol == CLUTTER_KEY_Right || symbol == CLUTTER_KEY_Down){
        result = (currentIndex + 1) % itemCount;
    }else if (symbol == CLUTTER_KEY_Home){
        result = 0;
    }else if (symbol == CLUTTER_KEY_End){
        result = itemCount -1;
    }
    return result;
}

static gboolean
cdos_workspace_monitor_select_another_window (CdosWorkspaceMonitor *cwm, guint key_val){
    guint numWindows = g_list_length (cwm->windows);
    if (numWindows == 0){
        return FALSE;
    }
    guint currentIndex = cwm->kbWindowIndex;
    guint numCols = ceil (sqrt (numWindows));
    guint nextIndex = grid_navigator_next_index (numWindows, numCols, currentIndex, key_val);
    if (nextIndex < 0){
        return FALSE;
    }
    cdos_workspace_monitor_select_index (cwm, nextIndex);
    return TRUE;
}

static gboolean
initial_position_idle (CdosWorkspaceMonitor *cwm){
    if (cwm->kbWindowIndex == 0){
        cdos_workspace_monitor_select_another_window (cwm, CLUTTER_KEY_Home);
    }
    return FALSE;
}

/**
 * cdos_workspace_monitor_on_initial_position_windows:
 * @cwm: #CdosWorkspaceMonitor 对象
 *
 * 初始化时调用，设置默认选中的
 *
 *
 */
void
cdos_workspace_monitor_on_initial_position_windows (CdosWorkspaceMonitor *cwm){
   cdos_workspace_monitor_select_another_window (cwm, CLUTTER_KEY_Home);
   cwm->initial_position_idle_id = g_idle_add ((GSourceFunc) initial_position_idle, cwm);
}

static CdosWorkspaceMonitor *
cdos_workspace_monitor_new ( MetaWorkspace *metaWorkspace, guint monitorIndex, 
                            CdosWorkspace *workspace, gboolean hasFocus)
{
    CdosWorkspaceMonitor *cwm = NULL;
    MetaScreen *screen = cdos_global_get_screen (global);

    cwm = g_malloc0 (sizeof (CdosWorkspaceMonitor));
    cwm->metaWorkspace = metaWorkspace;
    cwm->myWorkspace = workspace;
    cwm->add_window_ids = g_ptr_array_new_with_free_func ((GDestroyNotify)g_free);

    cwm->monitorIndex = monitorIndex;
    cwm->windowOverlaysGroup = clutter_actor_new ();
    clutter_actor_set_size (cwm->windowOverlaysGroup, 0, 0);
    cwm->actor = clutter_actor_new ();
    //clutter_actor_set_size (cwm->actor, 1920, 1080);

    cwm->dropRect = clutter_actor_new ();
    clutter_actor_add_child (cwm->actor, cwm->dropRect);
    clutter_actor_add_child (cwm->actor, cwm->windowOverlaysGroup);
    g_signal_connect (cwm->actor, "destroy", G_CALLBACK (cdos_workspace_monitor_on_destroy), cwm);

    GList *window_actors = cdos_global_get_window_actors (global);
    GList *windows_filter = NULL;
    while (window_actors != NULL){
        MetaWindowActor *win = window_actors->data;
        if(is_my_window (cwm, win))
        //if (meta_window_located_on_workspace (meta_window_actor_get_meta_window(win), metaWorkspace))
            windows_filter = g_list_append (windows_filter, win);
        GList *next = window_actors->next;
        window_actors = next;
    }
    g_list_free (window_actors);
// 这里添加 windowClone，所谓的 windowClone 就是实际窗口的克隆，一般是缩小版的克隆
// 比如 overview 中展示的那些小窗口就是一个个 windowClone
    cwm->windows = NULL;
    GList *iter;
    for (iter = windows_filter; iter != NULL; iter = iter->next)
    {
        MetaWindowActor *win = iter->data;
        if(cdos_workspace_monitor_is_overview_win (win)){
            cdos_workspace_monitor_add_window_clone (cwm, win);
        }
    }
    g_list_free (windows_filter);

    if (cwm->metaWorkspace != NULL){
        cwm->windowAddedId = g_signal_connect (metaWorkspace, "window-added", 
                                        G_CALLBACK (on_window_added), cwm);
        cwm->windowRemovedId = g_signal_connect (metaWorkspace, "window-removed", 
                                        G_CALLBACK (on_window_removed), cwm);
        cwm->windowEnteredMonitorId = g_signal_connect (screen, "window-entered-monitor",
                                        G_CALLBACK (on_window_enter_monitor), cwm);
        cwm->windowLeftMonitorId = g_signal_connect (screen, "window-left-monitor",
                                        G_CALLBACK (on_window_left_monitor), cwm);
    }
    cwm->leavingOverview = FALSE;
    cwm->stackedOnce = FALSE;
    cwm->initial_position_idle_id = 0;
    if (hasFocus){
        cwm->onInitialPositionWindows = TRUE;//对应的函数只执行一次
    }

    return cwm;
}

typedef struct _SlotGeometry
{
    gfloat x;
    gfloat y;
    gfloat width;
    gfloat height;
}SlotGeometry;

typedef struct _AnimateCompleteData
{
    CdosWorkspaceMonitor *cwm;
    CdosWindowClone *clone;
}AnimateCompleteData;

// 计算出每个 window clone 位置的相对系数，比如 0.75,0.25 之类的数值
static ClutterVertex*
compute_all_window_slots (CdosWorkspaceMonitor *cwm, guint window_count)
{
    if (window_count == 0 ){
        return NULL;
    }
    ClutterVertex *slots = g_new0 (ClutterVertex, window_count);

    guint gridW = ceil (sqrt (window_count));
    guint gridH = ceil ((float)window_count / gridW);

    gfloat fraction = DEFAULT_SLOT_FRACTION * ((gfloat) ( 1.0 / gridW ));
    cwm->slotWidth = floor ( fraction * cwm->width);
    guint i;
    for (i = 0; i < window_count; i++ ){
        if (window_count <= N_POSITIONS)
        {
            gint idx = window_count * (window_count - 1) / 2 + i;
            slots[i] = positions [idx];
        }else {
            slots[i].z = fraction;
            slots[i].x = 0.5 / gridW + (gfloat)(i % gridW) / gridW;
            slots[i].y = 0.5 / gridH + floor (i / gridW) / gridH;
        }
    }
    return slots;
}

static gboolean
cdos_workspace_monitor_is_empty (CdosWorkspaceMonitor *cwm){
    return (cwm->windows == NULL);
}

static SlotGeometry *
cdos_workspace_monitor_get_slot_geometry (CdosWorkspaceMonitor *cwm, ClutterVertex *slot)
{
    gfloat width;
    gfloat height;
    gfloat x;
    gfloat y;

    width = (gfloat) ((cwm->width - cwm->margin * 2) * slot->z);
    height = (gfloat) ((cwm->height - cwm->margin * 2) * slot->z);

    x = (gfloat) (cwm->x + cwm->margin + slot->x * (cwm->width - cwm->margin * 2) - width / 2 );
    y = (gfloat) (cwm->y + cwm->margin + slot->y * (cwm->height - cwm->margin * 2) - height / 2 );

    SlotGeometry *sg = g_malloc0 (sizeof (SlotGeometry));
    sg->x =x;
    sg->y = y;
    sg->width = width;
    sg->height = height ;
    return sg;
}

static void
cdos_workspace_monitor_compute_window_layout (CdosWorkspaceMonitor *cwm,
                                              MetaWindow *mwin, 
                                              ClutterVertex *slot,
                                              ClutterVertex *res)
{
   SlotGeometry *sg = cdos_workspace_monitor_get_slot_geometry (cwm, slot);
   MetaRectangle rect;
   meta_window_get_frame_rect (mwin, &rect);
   gfloat buttonOuterHeight = 0.0, captionIconHeight = 0.0;
   gfloat buttonOuterWidth = 0.0;

   if (cwm->windows)
   {
       CdosWindowClone *clone = cwm->windows->data;
       CdosWindowOverlay *overlay = clone->overlay;
       //[buttonOuterHeight, captionIconHeight] = this._windows[0].overlay.chromeHeights();
       cdos_window_overlay_chrome_heights (overlay, &buttonOuterHeight, &captionIconHeight);
       buttonOuterWidth = cdos_window_overlay_chrome_width (overlay);
   }
   gfloat scale = MIN ((gfloat)((sg->width - buttonOuterWidth) / rect.width),
                  (gfloat)((sg->height - buttonOuterHeight - captionIconHeight) / rect.height));
   scale = MIN ( scale, 1.0);
   gfloat x, y;
   x = floor(sg->x + (sg->width - scale * rect.width) / 2.0);
   y = floor(sg->y + ((sg->height - (scale * rect.height)) / 2.0) - captionIconHeight);

   res->x = x;
   res->y = y;
   res->z = scale;

   g_free (sg);
}

static void
on_clone_transitions_completed (ClutterTimeline *tml,  gpointer data)
{
    AnimateCompleteData *acdata = (AnimateCompleteData *)data;
    cdos_workspace_monitor_show_window_overlay (acdata->cwm, acdata->clone, TRUE);
    g_free (acdata);
}

static void
cdos_workspace_monitor_position_windows (CdosWorkspaceMonitor *cwm, guint flag)
{
    GList *clones = cwm->windows;
    guint initialPositioning = flag & WINDOW_POSITION_INITIAL;
    guint animate = flag & WINDOW_POSITION_ANIMATE;
    //._computeAllWindowSlots
    ClutterVertex *slots = compute_all_window_slots (cwm, g_list_length (clones));

    MetaScreen *screen = cdos_global_get_screen (global);
    MetaWorkspace *currentWorkspace = meta_screen_get_active_workspace (screen);
    gboolean isOnCurrentWorkspace = (cwm->metaWorkspace == NULL) || 
                                    (cwm->metaWorkspace == currentWorkspace);
    
    GList *tmp = g_list_copy (clones);
    tmp = g_list_sort_with_data (tmp, (GCompareDataFunc) sort_by_stable_sequence, clones);
    GList *iter = NULL;
    guint i = 0;
    for (iter = tmp; iter; iter = iter->next, i++){
        ClutterVertex *slot = &slots[i];
        CdosWindowClone *clone = iter->data;

        MetaWindow *mwin = clone->metaWindow;
        CdosWindowOverlay *overlay = clone->overlay;

        ClutterVertex res;
        cdos_workspace_monitor_compute_window_layout (cwm, mwin, slot, &res);
        if (overlay){
            cdos_window_overlay_hide (clone, overlay );
        }
        if (animate && isOnCurrentWorkspace){
            guint8 target_opacity = clutter_actor_get_opacity ((ClutterActor *)clone);
            gboolean animate_a = FALSE;
            if (!meta_window_showing_on_its_workspace (mwin)){
                if (initialPositioning){
                    clutter_actor_set_opacity ((ClutterActor *)clone, 0);
                    clutter_actor_set_scale ((ClutterActor *)clone, 0.f, 0.f);
                    clutter_actor_set_position ((ClutterActor *)clone, res.x, res.y);
                }
                target_opacity = 255;
                animate_a = TRUE;
            }
            ClutterTransition *trans = NULL;
            trans = cdos_transition_simple ((ClutterActor *)clone,
                                            CLUTTER_EASE_OUT_QUAD,
                                            ANIMATION_TIME,
                                            "x", res.x,
                                            "y", res.y,
                                            "scale-x", res.z,
                                            "scale-y", res.z,
                                            NULL);
            if (animate_a)
            {
                clutter_actor_set_opacity ((ClutterActor *)clone, target_opacity);
            }

            AnimateCompleteData *data = g_malloc0 (sizeof(AnimateCompleteData));
            data->cwm = cwm;
            data->clone = clone;
            g_signal_connect (trans, "completed", G_CALLBACK (on_clone_transitions_completed), data);
        }else {
            clutter_actor_set_position ((ClutterActor *)clone, res.x, res.y);
            clutter_actor_set_scale ((ClutterActor *)clone, res.z, res.z);
            cdos_workspace_monitor_show_window_overlay (cwm, clone, isOnCurrentWorkspace);
        }
    }
    if (cwm->onInitialPositionWindows ){
       cdos_workspace_monitor_on_initial_position_windows (cwm);
       g_source_remove (cwm->initial_position_idle_id);
       cwm->initial_position_idle_id = 0;
       cwm->onInitialPositionWindows = FALSE;
    }
    g_free (slots);
    g_list_free (tmp);
}

static void
cdos_workspace_monitor_show_window_overlay (CdosWorkspaceMonitor *cwm,
                                            CdosWindowClone *clone,
                                            gboolean fade)
{
    if (cwm->slotWidth){
        gfloat clonex, cloney;
        gfloat width, height;
        gdouble scale_x, scale_y;
        clutter_actor_get_position ((ClutterActor *)clone, &clonex, &cloney);
        clutter_actor_get_size ((ClutterActor *)clone, &width, &height);
        clutter_actor_get_scale ((ClutterActor *)clone, &scale_x, &scale_y);
        width = scale_x * width;
        height = scale_y * height;

        CdosWindowOverlay *overlay = clone->overlay;
        cdos_window_overlay_update_positions (overlay, clonex, cloney,
                                                        width, height,
                                                        cwm->slotWidth);
        if (fade){
            cdos_window_overlay_fadein (overlay);     
        }else{
            cdos_window_overlay_show (clone, overlay);
        }
    }
}

static void
cdos_workspace_monitor_show_all_overlays (CdosWorkspaceMonitor *cwm){
    MetaScreen *screen = cdos_global_get_screen (global);
    MetaWorkspace *currentWorkspace = meta_screen_get_active_workspace (screen);

    GList *iter;
    for (iter = cwm->windows; iter; iter = iter->next){
        CdosWindowClone *clone = iter->data;
        gboolean fade = (cwm->metaWorkspace == NULL) || ((cwm->metaWorkspace == currentWorkspace));
        cdos_workspace_monitor_show_window_overlay (cwm, clone, fade);
    }
}

// workspace 中调用这个接口
static void 
cdos_workspace_monitor_show_windows_overlays (CdosWorkspaceMonitor *cwm){
    if (cwm->leavingOverview){
        return;
    }
    clutter_actor_show (cwm->windowOverlaysGroup);
    cdos_workspace_monitor_show_all_overlays (cwm);
}

static void
cdos_workspace_monitor_hide_windows_overlays (CdosWorkspaceMonitor *cwm){
    clutter_actor_hide (cwm->windowOverlaysGroup);
}
static gboolean
set_workspace_monitor_later_func (CdosWorkspaceMonitor *cwm)
{
    cdos_workspace_monitor_position_windows (cwm, WINDOW_POSITION_INITIAL);
    return FALSE;
}

static int
sort_by_stable_sequence (void *a, void *b, GList *windows){
    CdosWindowClone *clone_a = (CdosWindowClone *)a;
    CdosWindowClone *clone_b = (CdosWindowClone *)b;

    gboolean minimize_a = FALSE;
    g_object_get (clone_a->metaWindow, "minimized", &minimize_a, NULL);
    gboolean minimize_b = FALSE;
    g_object_get (clone_b->metaWindow, "minimized", &minimize_b, NULL);

    gint minimizedDiff = minimize_a - minimize_b;
    if (minimizedDiff)
        return minimizedDiff;

    gint index_a = -1;
    gint index_b = -1;

    GList *iter;
    gint i = 0;
    for (iter = windows; iter; iter = iter->next)
    {
        if (clone_a->realWindow == iter->data)
            index_a = i;
        else if (clone_b->realWindow == iter->data)
            index_b = i;
        i++;
        if (index_a >= 0 && index_b >= 0)
            break;
    }

    //js: return minimizedDiff || (index_a - index_b );
    /*
     * C 里 (-1||*) 返回值为1, js里返回 -1.
     */
    return index_a - index_b;
}

static void 
cdos_workspace_monitor_sync_stacking (CdosWorkspaceMonitor *cwm, GList* windows){
    GList *clones = cwm->windows;
    cwm->stackedOnce = TRUE;

    GList *tmp;
    tmp = g_list_copy (clones);
    tmp = g_list_sort_with_data (tmp, (GCompareDataFunc) sort_by_stable_sequence, windows);

    GList *reverse = g_list_reverse (tmp);
    ClutterActor *below = cwm->dropRect;
    GList *iter;
    for (iter = reverse; iter; iter = iter->next){
        CdosWindowClone *clone = iter->data;
        cdos_window_clone_set_stack_above (clone, below);
        below = (ClutterActor *)clone;
    }
    g_list_free (reverse);
}

static void
cdos_workspace_monitor_set_geometry (CdosWorkspaceMonitor *cwm,
                                     MetaRectangle *rect, guint margin){

    cwm->x = rect->x;
    cwm->y = rect->y;
    cwm->width = rect->width;
    cwm->height = rect->height;
    cwm->margin = margin;

    meta_later_add (META_LATER_BEFORE_REDRAW,
                    (GSourceFunc) set_workspace_monitor_later_func,
                    cwm,
                    NULL);

}

static void cdos_workspace_monitor_zoom_to_overview (CdosWorkspaceMonitor *cwm){
    CdosOverview *overview = cdos_overview_get ();
    gboolean animationInProgress = overview->animation_in_progress;
    if (animationInProgress){
       cdos_workspace_monitor_position_windows (cwm, WINDOW_POSITION_ANIMATE |
                                                        WINDOW_POSITION_INITIAL);
    }else{
       cdos_workspace_monitor_position_windows (cwm, WINDOW_POSITION_INITIAL );
    }
}

static void
on_overview_hidden (CdosOverview *self, gpointer data){
    CdosWorkspaceMonitor *cwm = (CdosWorkspaceMonitor *)data;
    
    cwm->leavingOverview = FALSE;
}

static void
cdos_workspace_monitor_zoom_from_overview (CdosWorkspaceMonitor *cwm){
    //xunli
    MetaScreen *screen = cdos_global_get_screen (global);
    MetaWorkspace *currentWorkspace = meta_screen_get_active_workspace (screen);

    cwm->leavingOverview = TRUE;
    cdos_workspace_monitor_hide_windows_overlays (cwm);

    CdosOverview *overview = cdos_overview_get ();
    cwm->overviewHiddenId = g_signal_connect (overview, "hidden", G_CALLBACK (on_overview_hidden), cwm);

    if (cwm->metaWorkspace != NULL && cwm->metaWorkspace != currentWorkspace)
        return;

    ClutterAnimationMode trans_in = CLUTTER_EASE_IN_QUAD;
    ClutterAnimationMode trans_out = CLUTTER_EASE_OUT_ELASTIC;
    guint8 opacity = 255;
    GList *clones = cwm->windows;
    GList *tmp = g_list_copy (clones);
    tmp = g_list_sort_with_data (tmp, (GCompareDataFunc) sort_by_stable_sequence, clones);
    GList *iter = NULL;
    for (iter = tmp; iter; iter = iter->next){
        CdosWindowClone *clone = iter->data;
        MetaWindow *mwin = clone->metaWindow;
        ClutterActor *actor = (ClutterActor *)clone;
        if (meta_window_showing_on_its_workspace (mwin)){
            clutter_actor_save_easing_state (actor);
            clutter_actor_set_easing_mode (actor, trans_in);
            clutter_actor_set_easing_duration (actor, ANIMATION_TIME);
            clutter_actor_set_position (actor, clone->origX, clone->origY);
            clutter_actor_set_scale (actor, 1.0, 1.0);
            clutter_actor_set_opacity (actor, opacity);
            clutter_actor_restore_easing_state (actor);
        }else {
            clutter_actor_save_easing_state (actor);
            clutter_actor_set_pivot_point (actor, 0.5, 0.5);
            clutter_actor_set_easing_mode (actor, trans_out);
            clutter_actor_set_easing_duration (actor, ANIMATION_TIME);
            clutter_actor_set_scale (actor, 0, 0);
            clutter_actor_set_opacity (actor, 0);
            clutter_actor_restore_easing_state (actor);
        }
    }
    g_list_free (tmp);
}

static void
cdos_workspace_monitor_select_index (CdosWorkspaceMonitor *cwm, guint index){
    cwm->kbWindowIndex = index;

    CdosWindowClone *activeClone = NULL;
    if (index > -1 && index < g_list_length (cwm->windows)){
        activeClone = g_list_nth_data (cwm->windows, cwm->kbWindowIndex);
    }
    cdos_workspace_select_active_clone (cwm->myWorkspace, activeClone, cwm);
}

static void
cdos_workspace_monitor_select_clone (CdosWorkspaceMonitor *cwm, CdosWindowClone *clone){
        //guint index = g_list_index (temptest, clone);
        guint index = 0;
        
        index = g_list_index (cwm->windows, clone);
        cdos_workspace_monitor_select_index (cwm, index);
}

static void
cdos_workspace_monitor_show_active_selection (CdosWorkspaceMonitor *cwm){
    cdos_workspace_monitor_select_index (cwm, cwm->kbWindowIndex);
}

static void
cdos_workspace_monitor_destroy (CdosWorkspaceMonitor *cwm){
    clutter_actor_destroy (cwm->actor);
    if (cwm->add_window_ids)
    {
        guint i;
        for (i = 0; i < cwm->add_window_ids->len; i++)
        {
            WindowAddId *id = cwm->add_window_ids->pdata[i];
            g_source_remove (id->id);
        }
    }
    g_ptr_array_free (cwm->add_window_ids, 1);
    cwm->add_window_ids = NULL;
    g_free (cwm);
}

/**
 * cdos_workspace_monitor_activate_selected_window:
 * @cwm: #CdosWorkspaceMonitor 对象
 *
 * 激活选中的窗口
 */
gboolean
cdos_workspace_monitor_activate_selected_window (CdosWorkspaceMonitor *cwm){
    if (cwm->kbWindowIndex > -1 && cwm->kbWindowIndex < g_list_length (cwm->windows)){
        CdosWindowClone *clone = g_list_nth_data (cwm->windows, cwm->kbWindowIndex);
        cdos_workspace_monitor_on_clone_activated (clone, (gpointer *)cwm);
        return TRUE;
    }
    return FALSE;
}

static void
cdos_workspace_dispose (GObject *gobject)
{
    CdosWorkspace *self = CDOS_WORKSPACE (gobject);
    CdosWorkspacePrivate *priv = CDOS_WORKSPACE_GET_PRIVATE (self);
    GList *monitors = priv->monitors;

    if (priv->restack_id)
    {
        MetaScreen *scr = cdos_global_get_screen (global);
        g_signal_handler_disconnect (scr, priv->restack_id);
        priv->restack_id = 0L;
    }

    while (monitors != NULL){
        GList *next = monitors->next;
        CdosWorkspaceMonitor *cwm = monitors->data;
        cdos_workspace_monitor_destroy (cwm);
        monitors = next;
    }
    if (priv->monitors)
    {
        g_list_free (priv->monitors);
        priv->monitors = NULL;
    }

    G_OBJECT_CLASS (cdos_workspace_parent_class)->dispose (gobject);
}

static void
cdos_workspace_class_init (CdosWorkspaceClass *klass){
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

    gobject_class->dispose = cdos_workspace_dispose;

    g_type_class_add_private (klass, sizeof (CdosWorkspacePrivate));
    
    cdos_workspace_signals[FOCUS_REFRESH_REQUIRED] =
            g_signal_new ("focus-refresh-required",
            G_TYPE_FROM_CLASS (klass),
            G_SIGNAL_RUN_LAST,
            0,
            NULL, NULL,
            _cdos_marshal_VOID__INT_INT,
            G_TYPE_NONE, 2,
            G_TYPE_INT,
            G_TYPE_INT);

    cdos_workspace_signals[SELECTION_CHANGED] =
            g_signal_new ("selection-changed",
            G_TYPE_FROM_CLASS (klass),
            G_SIGNAL_RUN_LAST,
            0,
            NULL, NULL,
            g_cclosure_marshal_VOID__VOID,
            G_TYPE_NONE, 0);
}

static void
cdos_workspace_init (CdosWorkspace *self)
{
     CdosWorkspacePrivate *priv;
     priv = self->priv = CDOS_WORKSPACE_GET_PRIVATE (self);

     priv->monitors = NULL;
     priv->activeClone = NULL;
}

static guint
find_next_non_empty_monitor (CdosWorkspace *self, guint start, guint increment){
    CdosWorkspacePrivate *priv = CDOS_WORKSPACE_GET_PRIVATE (self);
    guint pos = start;
    guint len = g_list_length (priv->monitors);

    guint index = 0;
    for (; index < len; ++index){
        pos = ( len + pos +increment ) % len;
        CdosWorkspaceMonitor *monitor =  g_list_nth_data (priv->monitors, pos);
        if (!cdos_workspace_monitor_is_empty (monitor)){
            return pos;
        }
    }
    return priv->currentMonitorIndex || 0;
}

static void
cdos_workspace_select_monitor (CdosWorkspace *self, guint index){
    CdosWorkspacePrivate *priv = CDOS_WORKSPACE_GET_PRIVATE (self);

    priv->currentMonitorIndex = index;
    CdosWorkspaceMonitor *cwm = NULL;
    cwm = g_list_nth_data (priv->monitors, priv->currentMonitorIndex);
    cdos_workspace_monitor_show_active_selection (cwm);
}

static void
on_focus_refresh_required (CdosWorkspace *workspace, gpointer data){
    CdosWorkspacePrivate *priv = workspace->priv;

    select_next_non_empty_monitor (workspace, priv->currentMonitorIndex -1, 1);
}

static void 
select_next_non_empty_monitor(CdosWorkspace *workspace, 
                                          guint start, guint increment){
    guint pos = find_next_non_empty_monitor (workspace, start, increment);

}

/**
 * cdos_workspace_show_windows_overlays:
 * @self: #CdosWorkspace 对象
 *
 * 显示窗口缩略图的小图标和标题，位于缩略图下方
 *
 */
void
cdos_workspace_show_windows_overlays (CdosWorkspace *self){
    CdosWorkspacePrivate *priv = CDOS_WORKSPACE_GET_PRIVATE (self);
    GList *monitors = priv->monitors;
    while (monitors != NULL){
        GList *next = monitors->next;
        CdosWorkspaceMonitor *cwm = monitors->data;
        cdos_workspace_monitor_show_windows_overlays (cwm);
        monitors = next;
    }
}

/**
 * cdos_workspace_hide_windows_overlays:
 * @self: #CdosWorkspace 对象
 *
 * 隐藏窗口缩略图的小图标和标题，位于缩略图下方
 *
 */
void
cdos_workspace_hide_windows_overlays (CdosWorkspace *self){
    CdosWorkspacePrivate *priv = CDOS_WORKSPACE_GET_PRIVATE (self);
    GList *monitors = priv->monitors;
    while (monitors != NULL){
        GList *next = monitors->next;
        CdosWorkspaceMonitor *cwm = monitors->data;
        cdos_workspace_monitor_hide_windows_overlays (cwm);
        monitors = next;
    }

}

/**
 * cdos_workspace_zoom_to_overview:
 * @self: #CdosWorkspace 对象
 *
 * 进入窗口缩略图模式
 *
 */
void
cdos_workspace_zoom_to_overview (CdosWorkspace *self){
    CdosWorkspacePrivate *priv = CDOS_WORKSPACE_GET_PRIVATE (self);
    GList *monitors = priv->monitors;
    GList *iter;
    for (iter = monitors; iter; iter = iter->next)
    {
        CdosWorkspaceMonitor *cwm = iter->data;
        cdos_workspace_monitor_zoom_to_overview (cwm);
    }
    g_signal_emit_by_name (self, "focus-refresh-required");
}

/**
 * cdos_workspace_zoom_from_overview:
 * @self: #CdosWorkspace 对象
 *
 * 从窗口缩略图模式退出，回到正常桌面
 *
 */
void
cdos_workspace_zoom_from_overview (CdosWorkspace *self){
    CdosWorkspacePrivate *priv = CDOS_WORKSPACE_GET_PRIVATE (self);
    GList *monitors = priv->monitors;
    while (monitors != NULL){
        GList *next = monitors->next;
        CdosWorkspaceMonitor *cwm = monitors->data;
        cdos_workspace_monitor_zoom_from_overview (cwm);
        monitors = next;
    }

}

/**
 * cdos_workspace_position_windows:
 * @self: #CdosWorkspace 对象
 * @flag: 是否第一次设置
 *
 * 获取当前工作区的所有窗口的克隆列表，根据屏幕宽高、窗口个数计算出每个克隆窗口的起始坐标和缩放比例，
 * 并设置显示的位置
 */
void 
cdos_workspace_position_windows (CdosWorkspace *self , guint flag){
    CdosWorkspacePrivate *priv = CDOS_WORKSPACE_GET_PRIVATE (self);
    GList *monitors = priv->monitors;

    while (monitors != NULL){
        GList *next = monitors->next;
        CdosWorkspaceMonitor *cwm = monitors->data;
        if (cwm->myWorkspace == self){ 
            cdos_workspace_monitor_position_windows (cwm, flag);
            break;
        }
        monitors = next;
    } 

}

static void 
cdos_workspace_sync_stacking (MetaScreen *screen, CdosWorkspace *self)
{
    CdosWorkspacePrivate *priv = self->priv;

    GList *monitors = priv->monitors;
    GList *windows = cdos_global_get_window_actors (global);
    while (monitors != NULL)
    {
        GList *next = monitors->next;
        CdosWorkspaceMonitor *cwm = monitors->data;
        cdos_workspace_monitor_sync_stacking (cwm, windows);
        monitors = next;
    }
}

/**
 * cdos_workspace_is_empty:
 * @self: #CdosWorkspace 对象
 *
 * 判断一个工作区是否是空的，也就是说没有打开的应用程序窗口
 *
 */
gboolean
cdos_workspace_is_empty (CdosWorkspace *self){
    gboolean hasWindows = FALSE;
    CdosWorkspacePrivate *priv = CDOS_WORKSPACE_GET_PRIVATE (self);
    GList *monitors = priv->monitors;
    while (monitors != NULL){
        GList *next = monitors->next;
        CdosWorkspaceMonitor *cwm = monitors->data;
        hasWindows = hasWindows || cdos_workspace_monitor_is_empty (cwm);
        monitors = next;
    }
    return hasWindows;

}

/**
 * cdos_workspace_on_key_press:
 * @self: #CdosWorkspace 对象
 * @actor: 可空
 * @event: 键盘事件
 *
 * 对键盘事件做出响应，空格、回车键激活选中的窗口，并退出缩略图模式回到正常桌面
 */
gboolean cdos_workspace_on_key_press (CdosWorkspace *self, ClutterActor *actor,
                                                        ClutterEvent *event){
    ClutterModifierType modifiers = clutter_event_get_state (event);
    guint symbol = clutter_event_get_key_symbol (event);
    CdosWorkspacePrivate *priv = self->priv;

    ClutterModifierType ctrlAltMask = CLUTTER_CONTROL_MASK | CLUTTER_MOD1_MASK;

    if ((symbol == CLUTTER_KEY_ISO_Left_Tab || symbol == CLUTTER_KEY_Tab) &&
                                            !(modifiers & ctrlAltMask)){
        guint increment = ( symbol == CLUTTER_KEY_ISO_Left_Tab ? -1 :1 );
        select_next_non_empty_monitor (self, priv->currentMonitorIndex, increment);
        return TRUE;
    }
    CdosWorkspaceMonitor *currentMonitor = g_list_nth_data (priv->monitors, 
                                                            priv->currentMonitorIndex);
    if ((symbol == CLUTTER_KEY_m) || 
        (symbol == CLUTTER_KEY_M) || 
        (symbol == CLUTTER_KEY_space &&
        modifiers & CLUTTER_CONTROL_MASK))
    {
                return TRUE;
    }
    if ((symbol == CLUTTER_KEY_Return) ||
        (symbol == CLUTTER_KEY_space) || 
        (symbol == CLUTTER_KEY_KP_Enter))
    {
        if (cdos_workspace_monitor_activate_selected_window (currentMonitor)){
            return TRUE;
        }
        //FIXME
        // overview.hide
        CdosOverview *overview = cdos_overview_get ();
        //cdos_overview_hide (overview);
        clutter_actor_hide ((ClutterActor *)overview);
        return TRUE;
    }

    if (modifiers & ctrlAltMask){
        return FALSE;
    }
    return cdos_workspace_monitor_select_another_window (currentMonitor, symbol);
}

static void 
cdos_workspace_select_active_clone (CdosWorkspace *workspace,
                                    CdosWindowClone *clone, 
                                    CdosWorkspaceMonitor *cwm){
    CdosWorkspacePrivate *priv = CDOS_WORKSPACE_GET_PRIVATE (workspace);

    CdosWindowClone *current = priv->activeClone;
    if (clone){
        priv->currentMonitorIndex = cwm->monitorIndex;
    }
    priv->activeClone = clone;
    if (current != priv->activeClone ){
        if (current != NULL){
          CdosWindowOverlay *overlay = current->overlay;
          cdos_window_overlay_set_selected ( overlay, FALSE);
        }
        if (priv->activeClone){
          CdosWindowOverlay *overlay = priv->activeClone->overlay;
          cdos_window_overlay_set_selected ( overlay, TRUE);
        }
        g_signal_emit_by_name (cwm, "selection-changed");
    }
}

/**
 * cdos_workspace_new:
 * @metaWorkspace: #MetaWorkspace 对象
 * @view: #CdosWorkspacesView 对象
 *
 * 新建 #CdosWorkspace 对象
 */
CdosWorkspace *
cdos_workspace_new (MetaWorkspace *metaWorkspace, CdosWorkspacesView *view)
{
    CdosWorkspace *self = g_object_new (CDOS_TYPE_WORKSPACE, NULL);
    CdosWorkspacePrivate *priv = self->priv;
    self->metaWorkspace = metaWorkspace;

    //clutter_actor_set_size ((ClutterActor *)self, 0, 0);
    CdosLayoutManager *layout = cdos_layout_manager_get_default ();
    priv->currentMonitorIndex = cdos_layout_manager_get_index ();

    GPtrArray *monitors = cdos_layout_manager_get_monitors (layout);

    guint index = 0;
    while (monitors != NULL && index < monitors->len){
        gboolean hasFocus = FALSE;
        if( index == priv->currentMonitorIndex ){
            hasFocus = TRUE;
        }

        CdosWorkspaceMonitor *cwm = NULL;
        cwm = cdos_workspace_monitor_new ( metaWorkspace, index, self, hasFocus);
        MetaRectangle *rect = monitors->pdata[index];

        /* ClutterGeometry */
        cdos_workspace_monitor_set_geometry (cwm, rect, 0);
        priv->monitors = g_list_append (priv->monitors, cwm);
        clutter_actor_add_child ((ClutterActor *)self, cwm->actor);
        index++;
     }
     g_signal_connect (self, "focus-refresh-required", 
                     G_CALLBACK(on_focus_refresh_required), self);
     MetaScreen *screen = cdos_global_get_screen (global);
     priv->restack_id = g_signal_connect (screen, "restacked", G_CALLBACK(cdos_workspace_sync_stacking), self);

     return self;
   
}
